@@ -1,23 +1,23 @@
 /* AS path management routines.
-   Copyright (C) 1996, 97, 98, 99 Kunihiro Ishiguro
-   Copyright (C) 2005 Sun Microsystems, Inc.
-
-This file is part of GNU Zebra.
-
-GNU Zebra is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
-
-GNU Zebra is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with GNU Zebra; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+ * Copyright (C) 1996, 97, 98, 99 Kunihiro Ishiguro
+ * Copyright (C) 2005 Sun Microsystems, Inc.
+ *
+ * This file is part of GNU Zebra.
+ *
+ * GNU Zebra is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ *
+ * GNU Zebra is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -80,8 +80,8 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * NOT the internal representation!
  */
 struct assegment_header {
-	u_char type;
-	u_char length;
+	uint8_t type;
+	uint8_t length;
 };
 
 /* Hash for aspath.  This is the top level structure of AS path. */
@@ -109,7 +109,7 @@ const char *aspath_segment_type_str[] = {"as-invalid", "as-set", "as-sequence",
  * the caller should immediately assign data to the segment, as the segment
  * otherwise is not generally valid
  */
-static struct assegment *assegment_new(u_char type, u_short length)
+static struct assegment *assegment_new(uint8_t type, unsigned short length)
 {
 	struct assegment *new;
 
@@ -345,7 +345,7 @@ void aspath_unintern(struct aspath **aspath)
 /* Return the start or end delimiters for a particular Segment type */
 #define AS_SEG_START 0
 #define AS_SEG_END 1
-static char aspath_delimiter_char(u_char type, u_char which)
+static char aspath_delimiter_char(uint8_t type, uint8_t which)
 {
 	int i;
 	struct {
@@ -498,7 +498,8 @@ static void aspath_make_str_count(struct aspath *as, bool make_json)
 	if (!as->segments) {
 		if (make_json) {
 			json_object_string_add(as->json, "string", "Local");
-			json_object_object_add(as->json, "segments", jaspath_segments);
+			json_object_object_add(as->json, "segments",
+					       jaspath_segments);
 			json_object_int_add(as->json, "length", 0);
 		}
 		as->str = XMALLOC(MTYPE_AS_STR, 1);
@@ -575,8 +576,9 @@ static void aspath_make_str_count(struct aspath *as, bool make_json)
 		/* write out the ASNs, with their seperators, bar the last one*/
 		for (i = 0; i < seg->length; i++) {
 			if (make_json)
-				json_object_array_add(jseg_list,
-						      json_object_new_int(seg->as[i]));
+				json_object_array_add(
+					jseg_list,
+					json_object_new_int(seg->as[i]));
 
 			len += snprintf(str_buf + len, str_size - len, "%u",
 					seg->as[i]);
@@ -588,8 +590,9 @@ static void aspath_make_str_count(struct aspath *as, bool make_json)
 
 		if (make_json) {
 			jseg = json_object_new_object();
-			json_object_string_add(jseg, "type",
-					       aspath_segment_type_str[seg->type]);
+			json_object_string_add(
+				jseg, "type",
+				aspath_segment_type_str[seg->type]);
 			json_object_object_add(jseg, "list", jseg_list);
 			json_object_array_add(jaspath_segments, jseg);
 		}
@@ -861,7 +864,7 @@ static void assegment_data_put(struct stream *s, as_t *as, int num,
 		}
 }
 
-static size_t assegment_header_put(struct stream *s, u_char type, int length)
+static size_t assegment_header_put(struct stream *s, uint8_t type, int length)
 {
 	size_t lenp;
 	assert(length <= AS_SEGMENT_MAX);
@@ -901,10 +904,11 @@ size_t aspath_put(struct stream *s, struct aspath *as, int use32bit)
 			while ((seg->length - written) > AS_SEGMENT_MAX) {
 				assegment_header_put(s, seg->type,
 						     AS_SEGMENT_MAX);
-				assegment_data_put(s, seg->as, AS_SEGMENT_MAX,
+				assegment_data_put(s, (seg->as + written), AS_SEGMENT_MAX,
 						   use32bit);
 				written += AS_SEGMENT_MAX;
-				bytes += ASSEGMENT_SIZE(AS_SEGMENT_MAX, use32bit);
+				bytes += ASSEGMENT_SIZE(AS_SEGMENT_MAX,
+							use32bit);
 			}
 
 			/* write the final segment, probably is also the first
@@ -958,7 +962,7 @@ size_t aspath_put(struct stream *s, struct aspath *as, int use32bit)
  * We have no way to manage the storage, so we use a static stream
  * wrapper around aspath_put.
  */
-u_char *aspath_snmp_pathseg(struct aspath *as, size_t *varlen)
+uint8_t *aspath_snmp_pathseg(struct aspath *as, size_t *varlen)
 {
 #define SNMP_PATHSEG_MAX 1024
 
@@ -1537,7 +1541,7 @@ struct aspath *aspath_filter_exclude(struct aspath *source,
 
 /* Add specified AS to the leftmost of aspath. */
 static struct aspath *aspath_add_asns(struct aspath *aspath, as_t asno,
-				      u_char type, unsigned num)
+				      uint8_t type, unsigned num)
 {
 	struct assegment *assegment = aspath->segments;
 	unsigned i;
@@ -1685,7 +1689,7 @@ struct aspath *aspath_reconcile_as4(struct aspath *aspath,
 						" across 2/4 ASN boundary somewhere, broken..");
 				hops = seg->length;
 			}
-			/* fallthru */
+		/* fallthru */
 		case AS_SEQUENCE:
 			cpasns = MIN(seg->length, hops);
 			hops -= seg->length;
@@ -1868,7 +1872,7 @@ enum as_token {
 
 /* Return next token and point for string parse. */
 static const char *aspath_gettoken(const char *buf, enum as_token *token,
-				   u_long *asno)
+				   unsigned long *asno)
 {
 	const char *p = buf;
 
@@ -1926,14 +1930,15 @@ static const char *aspath_gettoken(const char *buf, enum as_token *token,
 
 	/* There is no match then return unknown token. */
 	*token = as_token_unknown;
-	return p++;
+	p++;
+	return p;
 }
 
 struct aspath *aspath_str2aspath(const char *str)
 {
 	enum as_token token = as_token_unknown;
-	u_short as_type;
-	u_long asno = 0;
+	unsigned short as_type;
+	unsigned long asno = 0;
 	struct aspath *aspath;
 	int needtype;
 
@@ -2031,7 +2036,8 @@ int aspath_cmp(const void *arg1, const void *arg2)
 /* AS path hash initialize. */
 void aspath_init(void)
 {
-	ashash = hash_create_size(32768, aspath_key_make, aspath_cmp);
+	ashash = hash_create_size(32768, aspath_key_make, aspath_cmp,
+				  "BGP AS Path");
 }
 
 void aspath_finish(void)
@@ -2071,7 +2077,7 @@ static void aspath_show_all_iterator(struct hash_backet *backet,
 	as = (struct aspath *)backet->data;
 
 	vty_out(vty, "[%p:%u] (%ld) ", (void *)backet, backet->key, as->refcnt);
-	vty_out(vty, "%s%s", as->str, VTY_NEWLINE);
+	vty_out(vty, "%s\n", as->str);
 }
 
 /* Print all aspath and hash information.  This function is used from
