@@ -1,26 +1,27 @@
 /* BGP attributes.
-   Copyright (C) 1996, 97, 98 Kunihiro Ishiguro
-
-This file is part of GNU Zebra.
-
-GNU Zebra is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
-
-GNU Zebra is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with GNU Zebra; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+ * Copyright (C) 1996, 97, 98 Kunihiro Ishiguro
+ *
+ * This file is part of GNU Zebra.
+ *
+ * GNU Zebra is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ *
+ * GNU Zebra is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #ifndef _QUAGGA_BGP_ATTR_H
 #define _QUAGGA_BGP_ATTR_H
 
+#include "mpls.h"
 #include "bgp_attr_evpn.h"
 
 /* Simple bit mapping. */
@@ -56,13 +57,24 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #define BGP_ATTR_NHLEN_VPNV6_GLOBAL       8+IPV6_MAX_BYTELEN
 #define BGP_ATTR_NHLEN_VPNV6_GLOBAL_AND_LL ((8+IPV6_MAX_BYTELEN) * 2)
 
+/* Prefix SID types */
+#define BGP_PREFIX_SID_LABEL_INDEX     1
+#define BGP_PREFIX_SID_IPV6            2
+#define BGP_PREFIX_SID_ORIGINATOR_SRGB 3
+
+#define BGP_PREFIX_SID_LABEL_INDEX_LENGTH      7
+#define BGP_PREFIX_SID_IPV6_LENGTH            19
+#define BGP_PREFIX_SID_ORIGINATOR_SRGB_LENGTH  6
+
+/* PMSI tunnel types (RFC 6514) */
+
 struct bgp_attr_encap_subtlv {
 	struct bgp_attr_encap_subtlv *next; /* for chaining */
 	/* Reference count of this attribute. */
 	unsigned long refcnt;
 	uint16_t type;
 	uint16_t length;
-	uint8_t value[1]; /* will be extended */
+	uint8_t value[0]; /* will be extended */
 };
 
 #if ENABLE_BGP_VNC
@@ -86,15 +98,55 @@ struct overlay_index {
 	union gw_addr gw_ip;
 };
 
-/* Additional/uncommon BGP attributes.
- * lazily allocated as and when a struct attr
- * requires it.
- */
-struct attr_extra {
+enum pta_type {
+	PMSI_TNLTYPE_NO_INFO = 0,
+	PMSI_TNLTYPE_RSVP_TE_P2MP,
+	PMSI_TNLTYPE_MLDP_P2MP,
+	PMSI_TNLTYPE_PIM_SSM,
+	PMSI_TNLTYPE_PIM_SM,
+	PMSI_TNLTYPE_PIM_BIDIR,
+	PMSI_TNLTYPE_INGR_REPL,
+	PMSI_TNLTYPE_MLDP_MP2MP,
+	PMSI_TNLTYPE_MAX = PMSI_TNLTYPE_MLDP_MP2MP
+};
+
+/* BGP core attribute structure. */
+struct attr {
+	/* AS Path structure */
+	struct aspath *aspath;
+
+	/* Community structure */
+	struct community *community;
+
+	/* Reference count of this attribute. */
+	unsigned long refcnt;
+
+	/* Flag of attribute is set or not. */
+	uint64_t flag;
+
+	/* Apart from in6_addr, the remaining static attributes */
+	struct in_addr nexthop;
+	uint32_t med;
+	uint32_t local_pref;
+	ifindex_t nh_ifindex;
+
+	/* Path origin attribute */
+	uint8_t origin;
+
+	/* PMSI tunnel type (RFC 6514). */
+	enum pta_type pmsi_tnl_type;
+
+	/* has the route-map changed any attribute?
+	   Used on the peer outbound side. */
+	uint32_t rmap_change_flags;
+
 	/* Multi-Protocol Nexthop, AFI IPv6 */
 	struct in6_addr mp_nexthop_global;
 	struct in6_addr mp_nexthop_local;
 
+	/* ifIndex corresponding to mp_nexthop_local. */
+	ifindex_t nh_lla_ifindex;
+
 	/* Extended Communities attribute. */
 	struct ecommunity *ecommunity;
 
@@ -116,20 +168,32 @@ struct attr_extra {
 	struct in_addr originator_id;
 
 	/* Local weight, not actually an attribute */
-	u_int32_t weight;
+	uint32_t weight;
 
 	/* Aggregator ASN */
 	as_t aggregator_as;
 
 	/* MP Nexthop length */
-	u_char mp_nexthop_len;
+	uint8_t mp_nexthop_len;
 
 	/* MP Nexthop preference */
-	u_char mp_nexthop_prefer_global;
+	uint8_t mp_nexthop_prefer_global;
+
+	/* Static MAC for EVPN */
+	uint8_t sticky;
+
+	/* Flag for default gateway extended community in EVPN */
+	uint8_t default_gw;
 
 	/* route tag */
 	route_tag_t tag;
 
+	/* Label index */
+	uint32_t label_index;
+
+	/* MPLS label */
+	mpls_label_t label;
+
 	uint16_t encap_tunneltype;		     /* grr */
 	struct bgp_attr_encap_subtlv *encap_subtlvs; /* rfc5512 */
 
@@ -138,37 +202,12 @@ struct attr_extra {
 #endif
 	/* EVPN */
 	struct overlay_index evpn_overlay;
-};
 
-/* BGP core attribute structure. */
-struct attr {
-	/* AS Path structure */
-	struct aspath *aspath;
+	/* EVPN MAC Mobility sequence number, if any. */
+	uint32_t mm_seqnum;
 
-	/* Community structure */
-	struct community *community;
-
-	/* Lazily allocated pointer to extra attributes */
-	struct attr_extra *extra;
-
-	/* Reference count of this attribute. */
-	unsigned long refcnt;
-
-	/* Flag of attribute is set or not. */
-	uint64_t flag;
-
-	/* Apart from in6_addr, the remaining static attributes */
-	struct in_addr nexthop;
-	u_int32_t med;
-	u_int32_t local_pref;
-	ifindex_t nh_ifindex;
-
-	/* Path origin attribute */
-	u_char origin;
-
-	/* has the route-map changed any attribute?
-	   Used on the peer outbound side. */
-	u_int32_t rmap_change_flags;
+	/* EVPN local router-mac */
+	struct ethaddr rmac;
 };
 
 /* rmap_change_flags definition */
@@ -191,14 +230,17 @@ struct cluster_list {
 struct transit {
 	unsigned long refcnt;
 	int length;
-	u_char *val;
+	uint8_t *val;
 };
 
-#define ATTR_FLAG_BIT(X)  (1ULL << ((X) - 1))
+/* "(void) 0" will generate a compiler error.  this is a safety check to
+ * ensure we're not using a value that exceeds the bit size of attr->flag. */
+#define ATTR_FLAG_BIT(X)                                                       \
+	__builtin_choose_expr((X) >= 1 && (X) <= 64, 1ULL << ((X)-1), (void)0)
 
 #define BGP_CLUSTER_LIST_LENGTH(attr)                                          \
 	(((attr)->flag & ATTR_FLAG_BIT(BGP_ATTR_CLUSTER_LIST))                 \
-		 ? (attr)->extra->cluster->length                              \
+		 ? (attr)->cluster->length                                     \
 		 : 0)
 
 typedef enum {
@@ -209,6 +251,7 @@ typedef enum {
 	/* only used internally, send notify + convert to BGP_ATTR_PARSE_ERROR
 	   */
 	BGP_ATTR_PARSE_ERROR_NOTIFYPLS = -3,
+	BGP_ATTR_PARSE_EOR = -4,
 } bgp_attr_parse_ret_t;
 
 struct bpacket_attr_vec_arr;
@@ -219,27 +262,23 @@ extern void bgp_attr_finish(void);
 extern bgp_attr_parse_ret_t bgp_attr_parse(struct peer *, struct attr *,
 					   bgp_size_t, struct bgp_nlri *,
 					   struct bgp_nlri *);
-extern struct attr_extra *bgp_attr_extra_get(struct attr *);
-extern void bgp_attr_extra_free(struct attr *);
 extern void bgp_attr_dup(struct attr *, struct attr *);
-extern void bgp_attr_deep_dup(struct attr *, struct attr *);
-extern void bgp_attr_deep_free(struct attr *);
+extern void bgp_attr_undup(struct attr *new, struct attr *old);
 extern struct attr *bgp_attr_intern(struct attr *attr);
 extern void bgp_attr_unintern_sub(struct attr *);
 extern void bgp_attr_unintern(struct attr **);
 extern void bgp_attr_flush(struct attr *);
-extern struct attr *bgp_attr_default_set(struct attr *attr, u_char);
-extern struct attr *bgp_attr_default_intern(u_char);
-extern struct attr *bgp_attr_aggregate_intern(struct bgp *, u_char,
+extern struct attr *bgp_attr_default_set(struct attr *attr, uint8_t);
+extern struct attr *bgp_attr_aggregate_intern(struct bgp *, uint8_t,
 					      struct aspath *,
 					      struct community *, int as_set,
-					      u_char);
+					      uint8_t);
 extern bgp_size_t bgp_packet_attribute(struct bgp *bgp, struct peer *,
 				       struct stream *, struct attr *,
 				       struct bpacket_attr_vec_arr *vecarr,
 				       struct prefix *, afi_t, safi_t,
 				       struct peer *, struct prefix_rd *,
-				       u_char *, int, u_int32_t);
+				       mpls_label_t *, uint32_t, int, uint32_t);
 extern void bgp_dump_routes_attr(struct stream *, struct attr *,
 				 struct prefix *);
 extern int attrhash_cmp(const void *, const void *);
@@ -261,9 +300,9 @@ struct bgp_attr_parser_args {
 	bgp_size_t length; /* attribute data length; */
 	bgp_size_t total;  /* total length, inc header */
 	struct attr *attr;
-	u_int8_t type;
-	u_int8_t flags;
-	u_char *startp;
+	uint8_t type;
+	uint8_t flags;
+	uint8_t *startp;
 };
 extern int bgp_mp_reach_parse(struct bgp_attr_parser_args *args,
 			      struct bgp_nlri *);
@@ -281,14 +320,15 @@ extern void bgp_attr_flush_encap(struct attr *attr);
  * one for each NLRI that needs to be encoded into the UPDATE message, and
  * finally the _end() function.
  */
-extern size_t bgp_packet_mpattr_start(struct stream *s, afi_t afi, safi_t safi,
-				      afi_t nh_afi,
+extern size_t bgp_packet_mpattr_start(struct stream *s, struct peer *peer,
+				      afi_t afi, safi_t safi,
 				      struct bpacket_attr_vec_arr *vecarr,
 				      struct attr *attr);
 extern void bgp_packet_mpattr_prefix(struct stream *s, afi_t afi, safi_t safi,
 				     struct prefix *p, struct prefix_rd *prd,
-				     u_char *tag, int addpath_encode,
-				     u_int32_t addpath_tx_id, struct attr *);
+				     mpls_label_t *label, uint32_t num_labels,
+				     int addpath_encode, uint32_t addpath_tx_id,
+				     struct attr *);
 extern size_t bgp_packet_mpattr_prefix_size(afi_t afi, safi_t safi,
 					    struct prefix *p);
 extern void bgp_packet_mpattr_end(struct stream *s, size_t sizep);
@@ -297,12 +337,12 @@ extern size_t bgp_packet_mpunreach_start(struct stream *s, afi_t afi,
 					 safi_t safi);
 extern void bgp_packet_mpunreach_prefix(struct stream *s, struct prefix *p,
 					afi_t afi, safi_t safi,
-					struct prefix_rd *prd, u_char *tag, int,
-					u_int32_t, struct attr *);
+					struct prefix_rd *prd, mpls_label_t *,
+					uint32_t, int, uint32_t, struct attr *);
 extern void bgp_packet_mpunreach_end(struct stream *s, size_t attrlen_pnt);
 
-static inline int bgp_rmap_nhop_changed(u_int32_t out_rmap_flags,
-					u_int32_t in_rmap_flags)
+static inline int bgp_rmap_nhop_changed(uint32_t out_rmap_flags,
+					uint32_t in_rmap_flags)
 {
 	return ((CHECK_FLAG(out_rmap_flags, BATTR_RMAP_NEXTHOP_PEER_ADDRESS)
 		 || CHECK_FLAG(out_rmap_flags, BATTR_RMAP_NEXTHOP_UNCHANGED)
@@ -317,4 +357,9 @@ static inline int bgp_rmap_nhop_changed(u_int32_t out_rmap_flags,
 			: 0);
 }
 
+static inline uint32_t mac_mobility_seqnum(struct attr *attr)
+{
+	return (attr) ? attr->mm_seqnum : 0;
+}
+
 #endif /* _QUAGGA_BGP_ATTR_H */
