@@ -15,10 +15,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -102,7 +101,7 @@ static void bgp_bfd_peer_sendmsg(struct peer *peer, int command)
 
 	bfd_info = (struct bfd_info *)peer->bfd_info;
 
-	if (peer->bgp && (peer->bgp->inst_type == BGP_INSTANCE_TYPE_VRF))
+	if (peer->bgp->inst_type == BGP_INSTANCE_TYPE_VRF)
 		vrf_id = peer->bgp->vrf_id;
 
 	if (command == ZEBRA_BFD_DEST_DEREGISTER) {
@@ -303,13 +302,13 @@ static int bgp_bfd_dest_update(int command, struct zclient *zclient,
 		prefix2str(&dp, buf[0], sizeof(buf[0]));
 		if (ifp) {
 			zlog_debug(
-				"Zebra: vrf %d interface %s bfd destination %s %s",
+				"Zebra: vrf %u interface %s bfd destination %s %s",
 				vrf_id, ifp->name, buf[0],
 				bfd_get_status_str(status));
 		} else {
 			prefix2str(&sp, buf[1], sizeof(buf[1]));
 			zlog_debug(
-				"Zebra: vrf %d source %s bfd destination %s %s",
+				"Zebra: vrf %u source %s bfd destination %s %s",
 				vrf_id, buf[1], buf[0],
 				bfd_get_status_str(status));
 		}
@@ -384,8 +383,8 @@ static int bgp_bfd_dest_update(int command, struct zclient *zclient,
 /*
  * bgp_bfd_peer_param_set - Set the configured BFD paramter values for peer.
  */
-static int bgp_bfd_peer_param_set(struct peer *peer, u_int32_t min_rx,
-				  u_int32_t min_tx, u_int8_t detect_mult,
+static int bgp_bfd_peer_param_set(struct peer *peer, uint32_t min_rx,
+				  uint32_t min_tx, uint8_t detect_mult,
 				  int defaults)
 {
 	struct peer_group *group;
@@ -510,25 +509,24 @@ void bgp_bfd_peer_config_write(struct vty *vty, struct peer *peer, char *addr)
 	bfd_info = (struct bfd_info *)peer->bfd_info;
 
 	if (CHECK_FLAG(bfd_info->flags, BFD_FLAG_PARAM_CFG))
-		vty_out(vty, " neighbor %s bfd %d %d %d%s", addr,
+		vty_out(vty, " neighbor %s bfd %d %d %d\n", addr,
 			bfd_info->detect_mult, bfd_info->required_min_rx,
-			bfd_info->desired_min_tx, VTY_NEWLINE);
+			bfd_info->desired_min_tx);
 
 	if (bfd_info->type != BFD_TYPE_NOT_CONFIGURED)
-		vty_out(vty, " neighbor %s bfd %s%s", addr,
+		vty_out(vty, " neighbor %s bfd %s\n", addr,
 			(bfd_info->type == BFD_TYPE_MULTIHOP) ? "multihop"
-							      : "singlehop",
-			VTY_NEWLINE);
+							      : "singlehop");
 
 	if (!CHECK_FLAG(bfd_info->flags, BFD_FLAG_PARAM_CFG)
 	    && (bfd_info->type == BFD_TYPE_NOT_CONFIGURED))
-		vty_out(vty, " neighbor %s bfd%s", addr, VTY_NEWLINE);
+		vty_out(vty, " neighbor %s bfd\n", addr);
 }
 
 /*
  * bgp_bfd_show_info - Show the peer BFD information.
  */
-void bgp_bfd_show_info(struct vty *vty, struct peer *peer, u_char use_json,
+void bgp_bfd_show_info(struct vty *vty, struct peer *peer, uint8_t use_json,
 		       json_object *json_neigh)
 {
 	bfd_show_info(vty, (struct bfd_info *)peer->bfd_info,
@@ -548,7 +546,7 @@ DEFUN (neighbor_bfd,
 
 	peer = peer_and_group_lookup_vty(vty, argv[idx_peer]->arg);
 	if (!peer)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	ret = bgp_bfd_peer_param_set(peer, BFD_DEF_MIN_RX, BFD_DEF_MIN_TX,
 				     BFD_DEF_DETECT_MULT, 1);
@@ -573,14 +571,14 @@ DEFUN (neighbor_bfd_param,
 	int idx_number_2 = 4;
 	int idx_number_3 = 5;
 	struct peer *peer;
-	u_int32_t rx_val;
-	u_int32_t tx_val;
-	u_int8_t dm_val;
+	uint32_t rx_val;
+	uint32_t tx_val;
+	uint8_t dm_val;
 	int ret;
 
 	peer = peer_and_group_lookup_vty(vty, argv[idx_peer]->arg);
 	if (!peer)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	if ((ret = bfd_validate_param(
 		     vty, argv[idx_number_1]->arg, argv[idx_number_2]->arg,
@@ -612,14 +610,14 @@ DEFUN_HIDDEN (neighbor_bfd_type,
 
 	peer = peer_and_group_lookup_vty(vty, argv[idx_peer]->arg);
 	if (!peer)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
-	if (!strcmp(argv[idx_hop]->arg, "singlehop"))
+	if (strmatch(argv[idx_hop]->text, "singlehop"))
 		type = BFD_TYPE_SINGLEHOP;
-	else if (!strcmp(argv[idx_hop]->arg, "multihop"))
+	else if (strmatch(argv[idx_hop]->text, "multihop"))
 		type = BFD_TYPE_MULTIHOP;
 	else
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	ret = bgp_bfd_peer_param_type_set(peer, type);
 	if (ret != 0)
@@ -645,7 +643,7 @@ DEFUN (no_neighbor_bfd,
 
 	peer = peer_and_group_lookup_vty(vty, argv[idx_peer]->arg);
 	if (!peer)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	ret = bgp_bfd_peer_param_unset(peer);
 	if (ret != 0)
@@ -671,7 +669,7 @@ DEFUN_HIDDEN (no_neighbor_bfd_type,
 
 	peer = peer_and_group_lookup_vty(vty, argv[idx_peer]->arg);
 	if (!peer)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	if (!peer->bfd_info)
 		return 0;
