@@ -1,22 +1,22 @@
 /* BGP community-list and extcommunity-list.
-   Copyright (C) 1999 Kunihiro Ishiguro
-
-This file is part of GNU Zebra.
-
-GNU Zebra is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
-
-GNU Zebra is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with GNU Zebra; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+ * Copyright (C) 1999 Kunihiro Ishiguro
+ *
+ * This file is part of GNU Zebra.
+ *
+ * GNU Zebra is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ *
+ * GNU Zebra is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -25,6 +25,7 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "memory.h"
 #include "queue.h"
 #include "filter.h"
+#include "stream.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_community.h"
@@ -333,13 +334,13 @@ community_list_entry_lookup(struct community_list *list, const void *arg,
 static char *community_str_get(struct community *com, int i)
 {
 	int len;
-	u_int32_t comval;
-	u_int16_t as;
-	u_int16_t val;
+	uint32_t comval;
+	uint16_t as;
+	uint16_t val;
 	char *str;
 	char *pnt;
 
-	memcpy(&comval, com_nthval(com, i), sizeof(u_int32_t));
+	memcpy(&comval, com_nthval(com, i), sizeof(uint32_t));
 	comval = ntohl(comval);
 
 	switch (comval) {
@@ -355,6 +356,9 @@ static char *community_str_get(struct community *com, int i)
 	case COMMUNITY_LOCAL_AS:
 		len = strlen(" local-AS");
 		break;
+	case COMMUNITY_GSHUT:
+		len = strlen(" graceful-shutdown");
+		break;
 	default:
 		len = strlen(" 65536:65535");
 		break;
@@ -380,6 +384,10 @@ static char *community_str_get(struct community *com, int i)
 		strcpy(pnt, "local-AS");
 		pnt += strlen("local-AS");
 		break;
+	case COMMUNITY_GSHUT:
+		strcpy(pnt, "graceful-shutdown");
+		pnt += strlen("graceful-shutdown");
+		break;
 	default:
 		as = (comval >> 16) & 0xFFFF;
 		val = comval & 0xFFFF;
@@ -443,11 +451,11 @@ static int community_regexp_match(struct community *com, regex_t *reg)
 static char *lcommunity_str_get(struct lcommunity *lcom, int i)
 {
 	struct lcommunity_val lcomval;
-	u_int32_t globaladmin;
-	u_int32_t localdata1;
-	u_int32_t localdata2;
+	uint32_t globaladmin;
+	uint32_t localdata1;
+	uint32_t localdata2;
 	char *str;
-	u_char *ptr;
+	uint8_t *ptr;
 	char *pnt;
 
 	ptr = lcom->val + (i * LCOMMUNITY_SIZE);
@@ -457,21 +465,11 @@ static char *lcommunity_str_get(struct lcommunity *lcom, int i)
 	/* Allocate memory.  48 bytes taken off bgp_lcommunity.c */
 	str = pnt = XMALLOC(MTYPE_LCOMMUNITY_STR, 48);
 
-	ptr = (u_char *)lcomval.val;
-	globaladmin = (*ptr++ << 24);
-	globaladmin |= (*ptr++ << 16);
-	globaladmin |= (*ptr++ << 8);
-	globaladmin |= (*ptr++);
-
-	localdata1 = (*ptr++ << 24);
-	localdata1 |= (*ptr++ << 16);
-	localdata1 |= (*ptr++ << 8);
-	localdata1 |= (*ptr++);
-
-	localdata2 = (*ptr++ << 24);
-	localdata2 |= (*ptr++ << 16);
-	localdata2 |= (*ptr++ << 8);
-	localdata2 |= (*ptr++);
+	ptr = (uint8_t *)lcomval.val;
+	ptr = ptr_get_be32(ptr, &globaladmin);
+	ptr = ptr_get_be32(ptr, &localdata1);
+	ptr = ptr_get_be32(ptr, &localdata2);
+	(void)ptr; /* consume value */
 
 	sprintf(pnt, "%u:%u:%u", globaladmin, localdata1, localdata2);
 	pnt += strlen(pnt);
@@ -514,7 +512,7 @@ static int lcommunity_regexp_match(struct lcommunity *com, regex_t *reg)
 	if (com == NULL || com->size == 0)
 		str = "";
 	else
-		str = lcommunity_str(com);
+		str = lcommunity_str(com, false);
 
 	/* Regular expression match.  */
 	if (regexec(reg, str, 0, NULL, 0) == 0)
@@ -551,7 +549,7 @@ static struct community *
 community_regexp_delete (struct community *com, regex_t * reg)
 {
   int i;
-  u_int32_t comval;
+  uint32_t comval;
   /* Maximum is "65535:65535" + '\0'. */
   char c[12];
   const char *str;
@@ -562,7 +560,7 @@ community_regexp_delete (struct community *com, regex_t * reg)
   i = 0;
   while (i < com->size)
     {
-      memcpy (&comval, com_nthval (com, i), sizeof (u_int32_t));
+      memcpy (&comval, com_nthval (com, i), sizeof (uint32_t));
       comval = ntohl (comval);
 
       switch (comval)
@@ -696,15 +694,14 @@ struct community *community_list_match_delete(struct community *com,
 					      struct community_list *list)
 {
 	struct community_entry *entry;
-	u_int32_t val;
-	u_int32_t com_index_to_delete[com->size];
+	uint32_t val;
+	uint32_t com_index_to_delete[com->size];
 	int delete_index = 0;
 	int i;
 
 	/* Loop over each community value and evaluate each against the
 	 * community-list.  If we need to delete a community value add its index
-	 * to
-	 * com_index_to_delete.
+	 * to com_index_to_delete.
 	 */
 	for (i = 0; i < com->size; i++) {
 		val = community_val_get(com, i);
@@ -855,7 +852,7 @@ int community_list_set(struct community_list_handler *ch, const char *name,
 
 /* Unset community-list */
 int community_list_unset(struct community_list_handler *ch, const char *name,
-			 const char *str, int direct, int style, int delete_all)
+			 const char *str, int direct, int style)
 {
 	struct community_entry *entry = NULL;
 	struct community_list *list;
@@ -867,16 +864,14 @@ int community_list_unset(struct community_list_handler *ch, const char *name,
 		return COMMUNITY_LIST_ERR_CANT_FIND_LIST;
 
 	/* Delete all of entry belongs to this community-list.  */
-	if (delete_all) {
+	if (!str) {
 		community_list_delete(list);
 		route_map_notify_dependencies(name, RMAP_EVENT_CLIST_DELETED);
 		return 0;
 	}
 
-	if (style == COMMUNITY_LIST_STANDARD) {
-		if (str)
-			com = community_str2com(str);
-	}
+	if (style == COMMUNITY_LIST_STANDARD)
+		com = community_str2com(str);
 
 	if (com) {
 		entry = community_list_entry_lookup(list, com, direct);
@@ -898,15 +893,14 @@ struct lcommunity *lcommunity_list_match_delete(struct lcommunity *lcom,
 						struct community_list *list)
 {
 	struct community_entry *entry;
-	u_int32_t com_index_to_delete[lcom->size];
-	u_char *ptr;
+	uint32_t com_index_to_delete[lcom->size];
+	uint8_t *ptr;
 	int delete_index = 0;
 	int i;
 
 	/* Loop over each lcommunity value and evaluate each against the
 	 * community-list.  If we need to delete a community value add its index
-	 * to
-	 * com_index_to_delete.
+	 * to com_index_to_delete.
 	 */
 	for (i = 0; i < lcom->size; i++) {
 		ptr = lcom->val + (i * LCOMMUNITY_SIZE);
@@ -992,13 +986,8 @@ int lcommunity_list_set(struct community_list_handler *ch, const char *name,
 	entry->any = (str ? 0 : 1);
 	entry->u.lcom = lcom;
 	entry->reg = regex;
-	if (lcom)
-		entry->config = lcommunity_lcom2str(
-			lcom, LCOMMUNITY_FORMAT_COMMUNITY_LIST);
-	else if (regex)
-		entry->config = XSTRDUP(MTYPE_COMMUNITY_LIST_CONFIG, str);
-	else
-		entry->config = NULL;
+	entry->config =
+		(regex ? XSTRDUP(MTYPE_COMMUNITY_LIST_CONFIG, str) : NULL);
 
 	/* Do not put duplicated community entry.  */
 	if (community_list_dup_check(list, entry))
@@ -1085,15 +1074,13 @@ int extcommunity_list_set(struct community_list_handler *ch, const char *name,
 		}
 	}
 
-	if (str) {
-		if (style == EXTCOMMUNITY_LIST_STANDARD)
-			ecom = ecommunity_str2com(str, 0, 1);
-		else
-			regex = bgp_regcomp(str);
+	if (style == EXTCOMMUNITY_LIST_STANDARD)
+		ecom = ecommunity_str2com(str, 0, 1);
+	else
+		regex = bgp_regcomp(str);
 
-		if (!ecom && !regex)
-			return COMMUNITY_LIST_ERR_MALFORMED_VAL;
-	}
+	if (!ecom && !regex)
+		return COMMUNITY_LIST_ERR_MALFORMED_VAL;
 
 	if (ecom)
 		ecom->str =
@@ -1108,8 +1095,7 @@ int extcommunity_list_set(struct community_list_handler *ch, const char *name,
 			ecom, ECOMMUNITY_FORMAT_COMMUNITY_LIST, 0);
 	else if (regex)
 		entry->config = XSTRDUP(MTYPE_COMMUNITY_LIST_CONFIG, str);
-	else
-		entry->config = NULL;
+
 	entry->u.ecom = ecom;
 	entry->reg = regex;
 
@@ -1124,11 +1110,13 @@ int extcommunity_list_set(struct community_list_handler *ch, const char *name,
 	return 0;
 }
 
-/* Unset extcommunity-list.  When str is NULL, delete all of
-   extcommunity-list entry belongs to the specified name.  */
+/* Unset extcommunity-list.
+ *
+ * When str is NULL, delete all extcommunity-list entries belonging to the
+ * specified name.
+ */
 int extcommunity_list_unset(struct community_list_handler *ch, const char *name,
-			    const char *str, int direct, int style,
-			    int delete_all)
+			    const char *str, int direct, int style)
 {
 	struct community_entry *entry = NULL;
 	struct community_list *list;
@@ -1140,16 +1128,14 @@ int extcommunity_list_unset(struct community_list_handler *ch, const char *name,
 		return COMMUNITY_LIST_ERR_CANT_FIND_LIST;
 
 	/* Delete all of entry belongs to this extcommunity-list.  */
-	if (delete_all) {
+	if (!str) {
 		community_list_delete(list);
 		route_map_notify_dependencies(name, RMAP_EVENT_ECLIST_DELETED);
 		return 0;
 	}
 
-	if (style == EXTCOMMUNITY_LIST_STANDARD) {
-		if (str)
-			ecom = ecommunity_str2com(str, 0, 1);
-	}
+	if (style == EXTCOMMUNITY_LIST_STANDARD)
+		ecom = ecommunity_str2com(str, 0, 1);
 
 	if (ecom) {
 		entry = community_list_entry_lookup(list, ecom, direct);
