@@ -1,22 +1,22 @@
 /* BGP flap dampening
-   Copyright (C) 2001 IP Infusion Inc.
-
-This file is part of GNU Zebra.
-
-GNU Zebra is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
-
-GNU Zebra is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with GNU Zebra; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+ * Copyright (C) 2001 IP Infusion Inc.
+ *
+ * This file is part of GNU Zebra.
+ *
+ * GNU Zebra is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ *
+ * GNU Zebra is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 #include <math.h>
@@ -112,8 +112,8 @@ static int bgp_reuse_timer(struct thread *t)
 	time_t t_now, t_diff;
 
 	damp->t_reuse = NULL;
-	damp->t_reuse = thread_add_timer(bm->master, bgp_reuse_timer, NULL,
-					 DELTA_REUSE);
+	thread_add_timer(bm->master, bgp_reuse_timer, NULL, DELTA_REUSE,
+			 &damp->t_reuse);
 
 	t_now = bgp_clock();
 
@@ -233,7 +233,7 @@ int bgp_damp_withdraw(struct bgp_info *binfo, struct bgp_node *rn, afi_t afi,
 	/* Remove the route from a reuse list if it is on one.  */
 	if (CHECK_FLAG(bdi->binfo->flags, BGP_INFO_DAMPED)) {
 		/* If decay rate isn't equal to 0, reinsert brn. */
-		if (bdi->penalty != last_penalty) {
+		if (bdi->penalty != last_penalty && bdi->index >= 0) {
 			bgp_reuse_list_delete(bdi);
 			bgp_reuse_list_add(bdi);
 		}
@@ -436,9 +436,8 @@ int bgp_damp_enable(struct bgp *bgp, afi_t afi, safi_t safi, time_t half,
 	bgp_damp_parameter_set(half, reuse, suppress, max);
 
 	/* Register reuse timer.  */
-	if (!damp->t_reuse)
-		damp->t_reuse = thread_add_timer(bm->master, bgp_reuse_timer,
-						 NULL, DELTA_REUSE);
+	thread_add_timer(bm->master, bgp_reuse_timer, NULL, DELTA_REUSE,
+			 &damp->t_reuse);
 
 	return 0;
 }
@@ -508,23 +507,23 @@ void bgp_config_write_damp(struct vty *vty)
 	    && bgp_damp_cfg.reuse_limit == DEFAULT_REUSE
 	    && bgp_damp_cfg.suppress_value == DEFAULT_SUPPRESS
 	    && bgp_damp_cfg.max_suppress_time == bgp_damp_cfg.half_life * 4)
-		vty_out(vty, " bgp dampening%s", VTY_NEWLINE);
+		vty_out(vty, " bgp dampening\n");
 	else if (bgp_damp_cfg.half_life != DEFAULT_HALF_LIFE * 60
 		 && bgp_damp_cfg.reuse_limit == DEFAULT_REUSE
 		 && bgp_damp_cfg.suppress_value == DEFAULT_SUPPRESS
 		 && bgp_damp_cfg.max_suppress_time
 			    == bgp_damp_cfg.half_life * 4)
-		vty_out(vty, " bgp dampening %lld%s",
-			bgp_damp_cfg.half_life / 60LL, VTY_NEWLINE);
+		vty_out(vty, " bgp dampening %lld\n",
+			bgp_damp_cfg.half_life / 60LL);
 	else
-		vty_out(vty, " bgp dampening %lld %d %d %lld%s",
+		vty_out(vty, " bgp dampening %lld %d %d %lld\n",
 			bgp_damp_cfg.half_life / 60LL, bgp_damp_cfg.reuse_limit,
 			bgp_damp_cfg.suppress_value,
-			bgp_damp_cfg.max_suppress_time / 60LL, VTY_NEWLINE);
+			bgp_damp_cfg.max_suppress_time / 60LL);
 }
 
 static const char *bgp_get_reuse_time(unsigned int penalty, char *buf,
-				      size_t len, u_char use_json,
+				      size_t len, uint8_t use_json,
 				      json_object *json)
 {
 	time_t reuse_time = 0;
@@ -543,9 +542,7 @@ static const char *bgp_get_reuse_time(unsigned int penalty, char *buf,
 	} else
 		reuse_time = 0;
 
-/* Making formatted timer strings. */
-#define ONE_DAY_SECOND 60*60*24
-#define ONE_WEEK_SECOND 60*60*24*7
+	/* Making formatted timer strings. */
 	if (reuse_time == 0) {
 		if (use_json)
 			json_object_int_add(json, "reuseTimerMsecs", 0);
@@ -639,12 +636,12 @@ void bgp_damp_info_vty(struct vty *vty, struct bgp_info *binfo,
 						   BGP_UPTIME_LEN, 0,
 						   json_path));
 
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "\n");
 	}
 }
 
 const char *bgp_damp_reuse_time_vty(struct vty *vty, struct bgp_info *binfo,
-				    char *timebuf, size_t len, u_char use_json,
+				    char *timebuf, size_t len, uint8_t use_json,
 				    json_object *json)
 {
 	struct bgp_damp_info *bdi;
@@ -676,25 +673,22 @@ int bgp_show_dampening_parameters(struct vty *vty, afi_t afi, safi_t safi)
 	bgp = bgp_get_default();
 
 	if (bgp == NULL) {
-		vty_out(vty, "No BGP process is configured%s", VTY_NEWLINE);
+		vty_out(vty, "No BGP process is configured\n");
 		return CMD_WARNING;
 	}
 
 	if (CHECK_FLAG(bgp->af_flags[afi][safi], BGP_CONFIG_DAMPENING)) {
-		vty_out(vty, "Half-life time: %lld min%s",
-			(long long)damp->half_life / 60, VTY_NEWLINE);
-		vty_out(vty, "Reuse penalty: %d%s", damp->reuse_limit,
-			VTY_NEWLINE);
-		vty_out(vty, "Suppress penalty: %d%s", damp->suppress_value,
-			VTY_NEWLINE);
-		vty_out(vty, "Max suppress time: %lld min%s",
-			(long long)damp->max_suppress_time / 60, VTY_NEWLINE);
-		vty_out(vty, "Max supress penalty: %u%s", damp->ceiling,
-			VTY_NEWLINE);
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "Half-life time: %lld min\n",
+			(long long)damp->half_life / 60);
+		vty_out(vty, "Reuse penalty: %d\n", damp->reuse_limit);
+		vty_out(vty, "Suppress penalty: %d\n", damp->suppress_value);
+		vty_out(vty, "Max suppress time: %lld min\n",
+			(long long)damp->max_suppress_time / 60);
+		vty_out(vty, "Max supress penalty: %u\n", damp->ceiling);
+		vty_out(vty, "\n");
 	} else
-		vty_out(vty, "dampening not enabled for %s%s",
-			afi == AFI_IP ? "IPv4" : "IPv6", VTY_NEWLINE);
+		vty_out(vty, "dampening not enabled for %s\n",
+			afi == AFI_IP ? "IPv4" : "IPv6");
 
 	return CMD_SUCCESS;
 }
