@@ -1,22 +1,22 @@
 /* BGP Extended Communities Attribute
-   Copyright (C) 2000 Kunihiro Ishiguro <kunihiro@zebra.org>
-
-This file is part of GNU Zebra.
-
-GNU Zebra is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
-
-GNU Zebra is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with GNU Zebra; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+ * Copyright (C) 2000 Kunihiro Ishiguro <kunihiro@zebra.org>
+ *
+ * This file is part of GNU Zebra.
+ *
+ * GNU Zebra is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ *
+ * GNU Zebra is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -26,11 +26,21 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "command.h"
 #include "queue.h"
 #include "filter.h"
+#include "jhash.h"
+#include "stream.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_ecommunity.h"
 #include "bgpd/bgp_lcommunity.h"
 #include "bgpd/bgp_aspath.h"
+#include "bgpd/bgp_flowspec_private.h"
+#include "bgpd/bgp_pbr.h"
+
+/* struct used to dump the rate contained in FS set traffic-rate EC */
+union traffic_rate {
+	float rate_float;
+	uint8_t rate_byte[4];
+};
 
 /* Hash of community attribute. */
 static struct hash *ecomhash;
@@ -66,7 +76,7 @@ static void ecommunity_hash_free(struct ecommunity *ecom)
    else return 0.  */
 int ecommunity_add_val(struct ecommunity *ecom, struct ecommunity_val *eval)
 {
-	u_int8_t *p;
+	uint8_t *p;
 	int ret;
 	int c;
 
@@ -124,7 +134,7 @@ struct ecommunity *ecommunity_uniq_sort(struct ecommunity *ecom)
 }
 
 /* Parse Extended Communites Attribute in BGP packet.  */
-struct ecommunity *ecommunity_parse(u_int8_t *pnt, u_short length)
+struct ecommunity *ecommunity_parse(uint8_t *pnt, unsigned short length)
 {
 	struct ecommunity tmp;
 	struct ecommunity *new;
@@ -234,22 +244,8 @@ unsigned int ecommunity_hash_make(void *arg)
 {
 	const struct ecommunity *ecom = arg;
 	int size = ecom->size * ECOMMUNITY_SIZE;
-	u_int8_t *pnt = ecom->val;
-	unsigned int key = 0;
-	int c;
 
-	for (c = 0; c < size; c += ECOMMUNITY_SIZE) {
-		key += pnt[c];
-		key += pnt[c + 1];
-		key += pnt[c + 2];
-		key += pnt[c + 3];
-		key += pnt[c + 4];
-		key += pnt[c + 5];
-		key += pnt[c + 6];
-		key += pnt[c + 7];
-	}
-
-	return key;
+	return jhash(ecom->val, size, 0x564321ab);
 }
 
 /* Compare two Extended Communities Attribute structure.  */
@@ -272,7 +268,8 @@ int ecommunity_cmp(const void *arg1, const void *arg2)
 /* Initialize Extended Comminities related hash. */
 void ecommunity_init(void)
 {
-	ecomhash = hash_create(ecommunity_hash_make, ecommunity_cmp);
+	ecomhash = hash_create(ecommunity_hash_make, ecommunity_cmp,
+			       "BGP ecommunity hash");
 }
 
 void ecommunity_finish(void)
@@ -290,6 +287,52 @@ enum ecommunity_token {
 	ecommunity_token_val,
 };
 
+/*
+ * Encode BGP extended community from passed values. Supports types
+ * defined in RFC 4360 and well-known sub-types.
+ */
+static int ecommunity_encode(uint8_t type, uint8_t sub_type, int trans, as_t as,
+			     struct in_addr ip, uint32_t val,
+			     struct ecommunity_val *eval)
+{
+	assert(eval);
+	if (type == ECOMMUNITY_ENCODE_AS) {
+		if (as > BGP_AS_MAX)
+			return -1;
+	} else if (type == ECOMMUNITY_ENCODE_IP
+		   || type == ECOMMUNITY_ENCODE_AS4) {
+		if (val > UINT16_MAX)
+			return -1;
+	}
+
+	/* Fill in the values. */
+	eval->val[0] = type;
+	if (!trans)
+		eval->val[0] |= ECOMMUNITY_FLAG_NON_TRANSITIVE;
+	eval->val[1] = sub_type;
+	if (type == ECOMMUNITY_ENCODE_AS) {
+		eval->val[2] = (as >> 8) & 0xff;
+		eval->val[3] = as & 0xff;
+		eval->val[4] = (val >> 24) & 0xff;
+		eval->val[5] = (val >> 16) & 0xff;
+		eval->val[6] = (val >> 8) & 0xff;
+		eval->val[7] = val & 0xff;
+	} else if (type == ECOMMUNITY_ENCODE_IP) {
+		memcpy(&eval->val[2], &ip, sizeof(struct in_addr));
+		eval->val[6] = (val >> 8) & 0xff;
+		eval->val[7] = val & 0xff;
+	} else {
+		eval->val[2] = (as >> 24) & 0xff;
+		eval->val[3] = (as >> 16) & 0xff;
+		eval->val[4] = (as >> 8) & 0xff;
+		eval->val[5] = as & 0xff;
+		eval->val[6] = (val >> 8) & 0xff;
+		eval->val[7] = val & 0xff;
+	}
+
+	return 0;
+}
+
 /* Get next Extended Communities token from the string. */
 static const char *ecommunity_gettoken(const char *str,
 				       struct ecommunity_val *eval,
@@ -303,7 +346,8 @@ static const char *ecommunity_gettoken(const char *str,
 	char *endptr;
 	struct in_addr ip;
 	as_t as = 0;
-	u_int32_t val = 0;
+	uint32_t val = 0;
+	uint8_t ecomm_type;
 	char buf[INET_ADDRSTRLEN + 1];
 
 	/* Skip white space. */
@@ -418,39 +462,15 @@ static const char *ecommunity_gettoken(const char *str,
 	if (!digit || !separator)
 		goto error;
 
-	/* Encode result into routing distinguisher.  */
-	if (dot) {
-		if (val > UINT16_MAX)
-			goto error;
-
-		eval->val[0] = ECOMMUNITY_ENCODE_IP;
-		eval->val[1] = 0;
-		memcpy(&eval->val[2], &ip, sizeof(struct in_addr));
-		eval->val[6] = (val >> 8) & 0xff;
-		eval->val[7] = val & 0xff;
-	} else if (as > BGP_AS_MAX) {
-		if (val > UINT16_MAX)
-			goto error;
-
-		eval->val[0] = ECOMMUNITY_ENCODE_AS4;
-		eval->val[1] = 0;
-		eval->val[2] = (as >> 24) & 0xff;
-		eval->val[3] = (as >> 16) & 0xff;
-		eval->val[4] = (as >> 8) & 0xff;
-		eval->val[5] = as & 0xff;
-		eval->val[6] = (val >> 8) & 0xff;
-		eval->val[7] = val & 0xff;
-	} else {
-		eval->val[0] = ECOMMUNITY_ENCODE_AS;
-		eval->val[1] = 0;
-
-		eval->val[2] = (as >> 8) & 0xff;
-		eval->val[3] = as & 0xff;
-		eval->val[4] = (val >> 24) & 0xff;
-		eval->val[5] = (val >> 16) & 0xff;
-		eval->val[6] = (val >> 8) & 0xff;
-		eval->val[7] = val & 0xff;
-	}
+	/* Encode result into extended community.  */
+	if (dot)
+		ecomm_type = ECOMMUNITY_ENCODE_IP;
+	else if (as > BGP_AS_MAX)
+		ecomm_type = ECOMMUNITY_ENCODE_AS4;
+	else
+		ecomm_type = ECOMMUNITY_ENCODE_AS;
+	if (ecommunity_encode(ecomm_type, 0, 1, as, ip, val, eval))
+		goto error;
 	*token = ecommunity_token_val;
 	return p;
 
@@ -535,6 +555,60 @@ struct ecommunity *ecommunity_str2com(const char *str, int type,
 	return ecom;
 }
 
+static int ecommunity_rt_soo_str(char *buf, uint8_t *pnt, int type,
+				 int sub_type, int format)
+{
+	int len = 0;
+	const char *prefix;
+
+	/* For parse Extended Community attribute tupple. */
+	struct ecommunity_as eas;
+	struct ecommunity_ip eip;
+
+
+	/* Determine prefix for string, if any. */
+	switch (format) {
+	case ECOMMUNITY_FORMAT_COMMUNITY_LIST:
+		prefix = (sub_type == ECOMMUNITY_ROUTE_TARGET ? "rt " : "soo ");
+		break;
+	case ECOMMUNITY_FORMAT_DISPLAY:
+		prefix = (sub_type == ECOMMUNITY_ROUTE_TARGET ? "RT:" : "SoO:");
+		break;
+	case ECOMMUNITY_FORMAT_ROUTE_MAP:
+		prefix = "";
+		break;
+	default:
+		prefix = "";
+		break;
+	}
+
+	/* Put string into buffer.  */
+	if (type == ECOMMUNITY_ENCODE_AS4) {
+		pnt = ptr_get_be32(pnt, &eas.as);
+		eas.val = (*pnt++ << 8);
+		eas.val |= (*pnt++);
+
+		len = sprintf(buf, "%s%u:%u", prefix, eas.as, eas.val);
+	} else if (type == ECOMMUNITY_ENCODE_AS) {
+		eas.as = (*pnt++ << 8);
+		eas.as |= (*pnt++);
+		pnt = ptr_get_be32(pnt, &eas.val);
+
+		len = sprintf(buf, "%s%u:%u", prefix, eas.as, eas.val);
+	} else if (type == ECOMMUNITY_ENCODE_IP) {
+		memcpy(&eip.ip, pnt, 4);
+		pnt += 4;
+		eip.val = (*pnt++ << 8);
+		eip.val |= (*pnt++);
+
+		len = sprintf(buf, "%s%s:%u", prefix, inet_ntoa(eip.ip),
+			      eip.val);
+	}
+	(void)pnt; /* consume value */
+
+	return len;
+}
+
 /* Convert extended community attribute to string.
 
    Due to historical reason of industry standard implementation, there
@@ -562,28 +636,16 @@ struct ecommunity *ecommunity_str2com(const char *str, int type,
 char *ecommunity_ecom2str(struct ecommunity *ecom, int format, int filter)
 {
 	int i;
-	u_int8_t *pnt;
-	int encode = 0;
-	int type = 0;
-#define ECOMMUNITY_STR_DEFAULT_LEN  27
+	uint8_t *pnt;
+	uint8_t type = 0;
+	uint8_t sub_type = 0;
+#define ECOMMUNITY_STR_DEFAULT_LEN  64
 	int str_size;
 	int str_pnt;
 	char *str_buf;
-	const char *prefix;
 	int len = 0;
 	int first = 1;
 
-	/* For parse Extended Community attribute tupple. */
-	struct ecommunity_as {
-		as_t as;
-		u_int32_t val;
-	} eas;
-
-	struct ecommunity_ip {
-		struct in_addr ip;
-		u_int16_t val;
-	} eip;
-
 	if (ecom->size == 0) {
 		str_buf = XMALLOC(MTYPE_ECOMMUNITY_STR, 1);
 		str_buf[0] = '\0';
@@ -597,6 +659,8 @@ char *ecommunity_ecom2str(struct ecommunity *ecom, int format, int filter)
 	str_pnt = 0;
 
 	for (i = 0; i < ecom->size; i++) {
+		int unk_ecom = 0;
+
 		/* Make it sure size is enough.  */
 		while (str_pnt + ECOMMUNITY_STR_DEFAULT_LEN >= str_size) {
 			str_size *= 2;
@@ -605,133 +669,148 @@ char *ecommunity_ecom2str(struct ecommunity *ecom, int format, int filter)
 		}
 
 		/* Space between each value.  */
-		if (!first)
+		if (!first) {
 			str_buf[str_pnt++] = ' ';
+			len++;
+		}
 
 		pnt = ecom->val + (i * 8);
 
 		/* High-order octet of type. */
-		encode = *pnt++;
+		type = *pnt++;
 
-		switch (encode) {
-		case ECOMMUNITY_ENCODE_AS:
-		case ECOMMUNITY_ENCODE_IP:
-		case ECOMMUNITY_ENCODE_AS4:
-			break;
-
-		case ECOMMUNITY_ENCODE_OPAQUE:
-			if (filter == ECOMMUNITY_ROUTE_TARGET) {
+		if (type == ECOMMUNITY_ENCODE_AS || type == ECOMMUNITY_ENCODE_IP
+		    || type == ECOMMUNITY_ENCODE_AS4) {
+			/* Low-order octet of type. */
+			sub_type = *pnt++;
+			if (sub_type != ECOMMUNITY_ROUTE_TARGET
+			    && sub_type != ECOMMUNITY_SITE_ORIGIN)
+				unk_ecom = 1;
+			else
+				len = ecommunity_rt_soo_str(str_buf + str_pnt,
+							    pnt, type, sub_type,
+							    format);
+		} else if (type == ECOMMUNITY_ENCODE_OPAQUE) {
+			if (filter == ECOMMUNITY_ROUTE_TARGET)
 				continue;
-			}
 			if (*pnt == ECOMMUNITY_OPAQUE_SUBTYPE_ENCAP) {
 				uint16_t tunneltype;
 				memcpy(&tunneltype, pnt + 5, 2);
 				tunneltype = ntohs(tunneltype);
 				len = sprintf(str_buf + str_pnt, "ET:%d",
 					      tunneltype);
-				str_pnt += len;
-				first = 0;
+			} else if (*pnt == ECOMMUNITY_EVPN_SUBTYPE_DEF_GW) {
+				len = sprintf(str_buf + str_pnt,
+					      "Default Gateway");
+			} else
+				unk_ecom = 1;
+		} else if (type == ECOMMUNITY_ENCODE_EVPN) {
+			if (filter == ECOMMUNITY_ROUTE_TARGET)
 				continue;
-			}
-			len = sprintf(str_buf + str_pnt, "?");
-			str_pnt += len;
-			first = 0;
-			continue;
-		case ECOMMUNITY_ENCODE_EVPN:
-			if (filter == ECOMMUNITY_ROUTE_TARGET) {
-				continue;
-			}
-			if (*pnt == ECOMMUNITY_SITE_ORIGIN) {
-				char macaddr[6];
+			if (*pnt == ECOMMUNITY_EVPN_SUBTYPE_ROUTERMAC) {
+				struct ethaddr rmac;
 				pnt++;
-				memcpy(&macaddr, pnt, 6);
+				memcpy(&rmac, pnt, ETH_ALEN);
 				len = sprintf(
 					str_buf + str_pnt,
-					"EVPN:%02x:%02x:%02x:%02x:%02x:%02x",
-					macaddr[0], macaddr[1], macaddr[2],
-					macaddr[3], macaddr[4], macaddr[5]);
-				str_pnt += len;
-				first = 0;
-				continue;
-			}
-			len = sprintf(str_buf + str_pnt, "?");
-			str_pnt += len;
-			first = 0;
-			continue;
-		default:
-			len = sprintf(str_buf + str_pnt, "?");
-			str_pnt += len;
-			first = 0;
-			continue;
+					"Rmac:%02x:%02x:%02x:%02x:%02x:%02x",
+					(uint8_t)rmac.octet[0],
+					(uint8_t)rmac.octet[1],
+					(uint8_t)rmac.octet[2],
+					(uint8_t)rmac.octet[3],
+					(uint8_t)rmac.octet[4],
+					(uint8_t)rmac.octet[5]);
+			} else if (*pnt
+				   == ECOMMUNITY_EVPN_SUBTYPE_MACMOBILITY) {
+				uint32_t seqnum;
+				uint8_t flags = *++pnt;
+
+				memcpy(&seqnum, pnt + 2, 4);
+				seqnum = ntohl(seqnum);
+				if (flags
+				    & ECOMMUNITY_EVPN_SUBTYPE_MACMOBILITY_FLAG_STICKY)
+					len = sprintf(str_buf + str_pnt,
+						      "MM:%u, sticky MAC",
+						      seqnum);
+				else
+					len = sprintf(str_buf + str_pnt,
+						      "MM:%u", seqnum);
+			} else
+				unk_ecom = 1;
+		} else if (type == ECOMMUNITY_ENCODE_REDIRECT_IP_NH) {
+			sub_type = *pnt++;
+			if (sub_type == ECOMMUNITY_REDIRECT_IP_NH) {
+				len = sprintf(
+					str_buf + str_pnt,
+					"FS:redirect IP 0x%x", *(pnt+5));
+			} else
+				unk_ecom = 1;
+		} else if (type == ECOMMUNITY_ENCODE_TRANS_EXP ||
+			   type == ECOMMUNITY_EXTENDED_COMMUNITY_PART_2 ||
+			   type == ECOMMUNITY_EXTENDED_COMMUNITY_PART_3) {
+			sub_type = *pnt++;
+			if (sub_type == ECOMMUNITY_REDIRECT_VRF) {
+				char buf[16];
+
+				memset(buf, 0, sizeof(buf));
+				ecommunity_rt_soo_str(buf, (uint8_t *)pnt,
+						      type &
+						      ~ECOMMUNITY_ENCODE_TRANS_EXP,
+						      ECOMMUNITY_ROUTE_TARGET,
+						      ECOMMUNITY_FORMAT_DISPLAY);
+				len = snprintf(str_buf + str_pnt,
+					       str_size - len,
+					       "FS:redirect VRF %s", buf);
+			} else if (type != ECOMMUNITY_ENCODE_TRANS_EXP)
+				unk_ecom = 1;
+			else if (sub_type == ECOMMUNITY_TRAFFIC_ACTION) {
+				char action[64];
+				char *ptr = action;
+
+				if (*(pnt+3) ==
+				    1 << FLOWSPEC_TRAFFIC_ACTION_TERMINAL)
+					ptr += snprintf(ptr, sizeof(action),
+							"terminate (apply)");
+				else
+					ptr += snprintf(ptr, sizeof(action),
+						       "eval stops");
+				if (*(pnt+3) ==
+				    1 << FLOWSPEC_TRAFFIC_ACTION_SAMPLE)
+					snprintf(ptr, sizeof(action) -
+						 (size_t)(ptr-action),
+						 ", sample");
+				len = snprintf(str_buf + str_pnt,
+					       str_size - len,
+					      "FS:action %s", action);
+			} else if (sub_type == ECOMMUNITY_TRAFFIC_RATE) {
+				union traffic_rate data;
+
+				data.rate_byte[3] = *(pnt+2);
+				data.rate_byte[2] = *(pnt+3);
+				data.rate_byte[1] = *(pnt+4);
+				data.rate_byte[0] = *(pnt+5);
+				len = sprintf(
+					str_buf + str_pnt,
+					"FS:rate %f", data.rate_float);
+			} else if (sub_type == ECOMMUNITY_TRAFFIC_MARKING) {
+				len = sprintf(
+					str_buf + str_pnt,
+					"FS:marking %u", *(pnt+5));
+			} else
+				unk_ecom = 1;
+		} else {
+			sub_type = *pnt++;
+			unk_ecom = 1;
 		}
 
-		/* Low-order octet of type. */
-		type = *pnt++;
-		if (type != ECOMMUNITY_ROUTE_TARGET
-		    && type != ECOMMUNITY_SITE_ORIGIN) {
-			len = sprintf(str_buf + str_pnt, "?");
-			str_pnt += len;
-			first = 0;
-			continue;
-		}
+		if (unk_ecom)
+			len = sprintf(str_buf + str_pnt, "UNK:%d, %d",
+				      type, sub_type);
 
-		switch (format) {
-		case ECOMMUNITY_FORMAT_COMMUNITY_LIST:
-			prefix = (type == ECOMMUNITY_ROUTE_TARGET ? "rt "
-								  : "soo ");
-			break;
-		case ECOMMUNITY_FORMAT_DISPLAY:
-			prefix = (type == ECOMMUNITY_ROUTE_TARGET ? "RT:"
-								  : "SoO:");
-			break;
-		case ECOMMUNITY_FORMAT_ROUTE_MAP:
-			prefix = "";
-			break;
-		default:
-			prefix = "";
-			break;
-		}
-
-		/* Put string into buffer.  */
-		if (encode == ECOMMUNITY_ENCODE_AS4) {
-			eas.as = (*pnt++ << 24);
-			eas.as |= (*pnt++ << 16);
-			eas.as |= (*pnt++ << 8);
-			eas.as |= (*pnt++);
-
-			eas.val = (*pnt++ << 8);
-			eas.val |= (*pnt++);
-
-			len = sprintf(str_buf + str_pnt, "%s%u:%u", prefix,
-				      eas.as, eas.val);
-			str_pnt += len;
-			first = 0;
-		}
-		if (encode == ECOMMUNITY_ENCODE_AS) {
-			eas.as = (*pnt++ << 8);
-			eas.as |= (*pnt++);
-
-			eas.val = (*pnt++ << 24);
-			eas.val |= (*pnt++ << 16);
-			eas.val |= (*pnt++ << 8);
-			eas.val |= (*pnt++);
-
-			len = sprintf(str_buf + str_pnt, "%s%u:%u", prefix,
-				      eas.as, eas.val);
-			str_pnt += len;
-			first = 0;
-		} else if (encode == ECOMMUNITY_ENCODE_IP) {
-			memcpy(&eip.ip, pnt, 4);
-			pnt += 4;
-			eip.val = (*pnt++ << 8);
-			eip.val |= (*pnt++);
-
-			len = sprintf(str_buf + str_pnt, "%s%s:%u", prefix,
-				      inet_ntoa(eip.ip), eip.val);
-			str_pnt += len;
-			first = 0;
-		}
+		str_pnt += len;
+		first = 0;
 	}
+
 	return str_buf;
 }
 
@@ -769,7 +848,7 @@ int ecommunity_match(const struct ecommunity *ecom1,
 extern struct ecommunity_val *ecommunity_lookup(const struct ecommunity *ecom,
 						uint8_t type, uint8_t subtype)
 {
-	u_int8_t *p;
+	uint8_t *p;
 	int c;
 
 	/* If the value already exists in the structure return 0.  */
@@ -790,7 +869,7 @@ extern struct ecommunity_val *ecommunity_lookup(const struct ecommunity *ecom,
 extern int ecommunity_strip(struct ecommunity *ecom, uint8_t type,
 			    uint8_t subtype)
 {
-	u_int8_t *p;
+	uint8_t *p;
 	int c, found = 0;
 	/* When this is fist value, just add it.  */
 	if (ecom == NULL || ecom->val == NULL) {
@@ -822,3 +901,88 @@ extern int ecommunity_strip(struct ecommunity *ecom, uint8_t type,
 	ecom->val = p;
 	return 1;
 }
+
+/*
+ * Remove specified extended community value from extended community.
+ * Returns 1 if value was present (and hence, removed), 0 otherwise.
+ */
+int ecommunity_del_val(struct ecommunity *ecom, struct ecommunity_val *eval)
+{
+	uint8_t *p;
+	int c, found = 0;
+
+	/* Make sure specified value exists. */
+	if (ecom == NULL || ecom->val == NULL)
+		return 0;
+	c = 0;
+	for (p = ecom->val; c < ecom->size; p += ECOMMUNITY_SIZE, c++) {
+		if (!memcmp(p, eval->val, ECOMMUNITY_SIZE)) {
+			found = 1;
+			break;
+		}
+	}
+	if (found == 0)
+		return 0;
+
+	/* Delete the selected value */
+	ecom->size--;
+	p = XMALLOC(MTYPE_ECOMMUNITY_VAL, ecom->size * ECOMMUNITY_SIZE);
+	if (c != 0)
+		memcpy(p, ecom->val, c * ECOMMUNITY_SIZE);
+	if ((ecom->size - c) != 0)
+		memcpy(p + (c)*ECOMMUNITY_SIZE,
+		       ecom->val + (c + 1) * ECOMMUNITY_SIZE,
+		       (ecom->size - c) * ECOMMUNITY_SIZE);
+	XFREE(MTYPE_ECOMMUNITY_VAL, ecom->val);
+	ecom->val = p;
+	return 1;
+}
+
+int ecommunity_fill_pbr_action(struct ecommunity_val *ecom_eval,
+			       struct bgp_pbr_entry_action *api)
+{
+	if (ecom_eval->val[1] == ECOMMUNITY_TRAFFIC_RATE) {
+		api->action = ACTION_TRAFFICRATE;
+		api->u.r.rate_info[3] = ecom_eval->val[4];
+		api->u.r.rate_info[2] = ecom_eval->val[5];
+		api->u.r.rate_info[1] = ecom_eval->val[6];
+		api->u.r.rate_info[0] = ecom_eval->val[7];
+	} else if (ecom_eval->val[1] == ECOMMUNITY_TRAFFIC_ACTION) {
+		api->action = ACTION_TRAFFIC_ACTION;
+		/* else distribute code is set by default */
+		if (ecom_eval->val[5] & (1 << FLOWSPEC_TRAFFIC_ACTION_TERMINAL))
+			api->u.za.filter |= TRAFFIC_ACTION_TERMINATE;
+		else
+			api->u.za.filter |= TRAFFIC_ACTION_DISTRIBUTE;
+		if (ecom_eval->val[5] == 1 << FLOWSPEC_TRAFFIC_ACTION_SAMPLE)
+			api->u.za.filter |= TRAFFIC_ACTION_SAMPLE;
+
+	} else if (ecom_eval->val[1] == ECOMMUNITY_TRAFFIC_MARKING) {
+		api->action = ACTION_MARKING;
+		api->u.marking_dscp = ecom_eval->val[7];
+	} else if (ecom_eval->val[1] == ECOMMUNITY_REDIRECT_VRF) {
+		/* must use external function */
+		return 0;
+	} else if (ecom_eval->val[1] == ECOMMUNITY_REDIRECT_IP_NH) {
+		/* see draft-ietf-idr-flowspec-redirect-ip-02
+		 * Q1: how come a ext. community can host ipv6 address
+		 * Q2 : from cisco documentation:
+		 * Announces the reachability of one or more flowspec NLRI.
+		 * When a BGP speaker receives an UPDATE message with the
+		 * redirect-to-IP extended community, it is expected to
+		 * create a traffic filtering rule for every flow-spec
+		 * NLRI in the message that has this path as its best
+		 * path. The filter entry matches the IP packets
+		 * described in the NLRI field and redirects them or
+		 * copies them towards the IPv4 or IPv6 address specified
+		 * in the 'Network Address of Next- Hop'
+		 * field of the associated MP_REACH_NLRI.
+		 */
+		struct ecommunity_ip *ip_ecom = (struct ecommunity_ip *)
+			ecom_eval + 2;
+
+		api->u.zr.redirect_ip_v4 = ip_ecom->ip;
+	} else
+		return -1;
+	return 0;
+}
