@@ -1,23 +1,23 @@
 /* BGP-4 Finite State Machine
-   From RFC1771 [A Border Gateway Protocol 4 (BGP-4)]
-   Copyright (C) 1996, 97, 98 Kunihiro Ishiguro
-
-This file is part of GNU Zebra.
-
-GNU Zebra is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
-
-GNU Zebra is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with GNU Zebra; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+ * From RFC1771 [A Border Gateway Protocol 4 (BGP-4)]
+ * Copyright (C) 1996, 97, 98 Kunihiro Ishiguro
+ *
+ * This file is part of GNU Zebra.
+ *
+ * GNU Zebra is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ *
+ * GNU Zebra is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -27,6 +27,7 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "thread.h"
 #include "log.h"
 #include "stream.h"
+#include "ringbuf.h"
 #include "memory.h"
 #include "plist.h"
 #include "workqueue.h"
@@ -49,6 +50,8 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "bgpd/bgp_nht.h"
 #include "bgpd/bgp_bfd.h"
 #include "bgpd/bgp_memory.h"
+#include "bgpd/bgp_keepalives.h"
+#include "bgpd/bgp_io.h"
 
 DEFINE_HOOK(peer_backward_transition, (struct peer * peer), (peer))
 DEFINE_HOOK(peer_established, (struct peer * peer), (peer))
@@ -86,7 +89,6 @@ int bgp_event(struct thread *);
 static int bgp_start_timer(struct thread *);
 static int bgp_connect_timer(struct thread *);
 static int bgp_holdtime_timer(struct thread *);
-static int bgp_keepalive_timer(struct thread *);
 
 /* BGP FSM functions. */
 static int bgp_start(struct peer *);
@@ -125,25 +127,72 @@ static struct peer *peer_xfer_conn(struct peer *from_peer)
 			   from_peer->host, from_peer, from_peer->fd, peer,
 			   peer->fd);
 
-	BGP_WRITE_OFF(peer->t_write);
-	BGP_READ_OFF(peer->t_read);
-	BGP_WRITE_OFF(from_peer->t_write);
-	BGP_READ_OFF(from_peer->t_read);
+	bgp_writes_off(peer);
+	bgp_reads_off(peer);
+	bgp_writes_off(from_peer);
+	bgp_reads_off(from_peer);
 
 	BGP_TIMER_OFF(peer->t_routeadv);
+	BGP_TIMER_OFF(peer->t_connect);
+	BGP_TIMER_OFF(peer->t_connect_check_r);
+	BGP_TIMER_OFF(peer->t_connect_check_w);
 	BGP_TIMER_OFF(from_peer->t_routeadv);
+	BGP_TIMER_OFF(from_peer->t_connect);
+	BGP_TIMER_OFF(from_peer->t_connect_check_r);
+	BGP_TIMER_OFF(from_peer->t_connect_check_w);
+	BGP_TIMER_OFF(from_peer->t_process_packet);
 
-	fd = peer->fd;
-	peer->fd = from_peer->fd;
-	from_peer->fd = fd;
-	stream_reset(peer->ibuf);
-	stream_fifo_clean(peer->obuf);
-	stream_fifo_clean(from_peer->obuf);
+	/*
+	 * At this point in time, it is possible that there are packets pending
+	 * on various buffers. Those need to be transferred or dropped,
+	 * otherwise we'll get spurious failures during session establishment.
+	 */
+	pthread_mutex_lock(&peer->io_mtx);
+	pthread_mutex_lock(&from_peer->io_mtx);
+	{
+		fd = peer->fd;
+		peer->fd = from_peer->fd;
+		from_peer->fd = fd;
+
+		stream_fifo_clean(peer->ibuf);
+		stream_fifo_clean(peer->obuf);
+
+		/*
+		 * this should never happen, since bgp_process_packet() is the
+		 * only task that sets and unsets the current packet and it
+		 * runs in our pthread.
+		 */
+		if (peer->curr) {
+			zlog_err(
+				"[%s] Dropping pending packet on connection transfer:",
+				peer->host);
+			uint16_t type = stream_getc_from(peer->curr,
+							 BGP_MARKER_SIZE + 2);
+			bgp_dump_packet(peer, type, peer->curr);
+			stream_free(peer->curr);
+			peer->curr = NULL;
+		}
+
+		// copy each packet from old peer's output queue to new peer
+		while (from_peer->obuf->head)
+			stream_fifo_push(peer->obuf,
+					 stream_fifo_pop(from_peer->obuf));
+
+		// copy each packet from old peer's input queue to new peer
+		while (from_peer->ibuf->head)
+			stream_fifo_push(peer->ibuf,
+					 stream_fifo_pop(from_peer->ibuf));
+
+		ringbuf_wipe(peer->ibuf_work);
+		ringbuf_copy(peer->ibuf_work, from_peer->ibuf_work,
+			     ringbuf_remain(from_peer->ibuf_work));
+	}
+	pthread_mutex_unlock(&from_peer->io_mtx);
+	pthread_mutex_unlock(&peer->io_mtx);
 
 	peer->as = from_peer->as;
 	peer->v_holdtime = from_peer->v_holdtime;
 	peer->v_keepalive = from_peer->v_keepalive;
-	peer->routeadv = from_peer->routeadv;
 	peer->v_routeadv = from_peer->v_routeadv;
 	peer->v_gr_restart = from_peer->v_gr_restart;
 	peer->cap = from_peer->cap;
@@ -181,22 +230,15 @@ static struct peer *peer_xfer_conn(struct peer *from_peer)
 		from_peer->domainname = NULL;
 	}
 
-	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++) {
-			peer->af_flags[afi][safi] =
-				from_peer->af_flags[afi][safi];
-			peer->af_sflags[afi][safi] =
-				from_peer->af_sflags[afi][safi];
-			peer->af_cap[afi][safi] = from_peer->af_cap[afi][safi];
-			peer->afc_nego[afi][safi] =
-				from_peer->afc_nego[afi][safi];
-			peer->afc_adv[afi][safi] =
-				from_peer->afc_adv[afi][safi];
-			peer->afc_recv[afi][safi] =
-				from_peer->afc_recv[afi][safi];
-			peer->orf_plist[afi][safi] =
-				from_peer->orf_plist[afi][safi];
-		}
+	FOREACH_AFI_SAFI (afi, safi) {
+		peer->af_flags[afi][safi] = from_peer->af_flags[afi][safi];
+		peer->af_sflags[afi][safi] = from_peer->af_sflags[afi][safi];
+		peer->af_cap[afi][safi] = from_peer->af_cap[afi][safi];
+		peer->afc_nego[afi][safi] = from_peer->afc_nego[afi][safi];
+		peer->afc_adv[afi][safi] = from_peer->afc_adv[afi][safi];
+		peer->afc_recv[afi][safi] = from_peer->afc_recv[afi][safi];
+		peer->orf_plist[afi][safi] = from_peer->orf_plist[afi][safi];
+	}
 
 	if (bgp_getsockname(peer) < 0) {
 		zlog_err(
@@ -223,12 +265,16 @@ static struct peer *peer_xfer_conn(struct peer *from_peer)
 		}
 	}
 
-	BGP_READ_ON(peer->t_read, bgp_read, peer->fd);
-	BGP_WRITE_ON(peer->t_write, bgp_write, peer->fd);
 
+	// Note: peer_xfer_stats() must be called with I/O turned OFF
 	if (from_peer)
 		peer_xfer_stats(peer, from_peer);
 
+	bgp_reads_on(peer);
+	bgp_writes_on(peer);
+	thread_add_timer_msec(bm->master, bgp_process_packet, peer, 0,
+			      &peer->t_process_packet);
+
 	return (peer);
 }
 
@@ -242,7 +288,8 @@ void bgp_timer_set(struct peer *peer)
 		/* First entry point of peer's finite state machine.  In Idle
 		   status start timer is on unless peer is shutdown or peer is
 		   inactive.  All other timer must be turned off */
-		if (BGP_PEER_START_SUPPRESSED(peer) || !peer_active(peer)) {
+		if (BGP_PEER_START_SUPPRESSED(peer) || !peer_active(peer)
+		    || peer->bgp->vrf_id == VRF_UNKNOWN) {
 			BGP_TIMER_OFF(peer->t_start);
 		} else {
 			BGP_TIMER_ON(peer->t_start, bgp_start_timer,
@@ -250,7 +297,7 @@ void bgp_timer_set(struct peer *peer)
 		}
 		BGP_TIMER_OFF(peer->t_connect);
 		BGP_TIMER_OFF(peer->t_holdtime);
-		BGP_TIMER_OFF(peer->t_keepalive);
+		bgp_keepalives_off(peer);
 		BGP_TIMER_OFF(peer->t_routeadv);
 		break;
 
@@ -262,7 +309,7 @@ void bgp_timer_set(struct peer *peer)
 		BGP_TIMER_ON(peer->t_connect, bgp_connect_timer,
 			     peer->v_connect);
 		BGP_TIMER_OFF(peer->t_holdtime);
-		BGP_TIMER_OFF(peer->t_keepalive);
+		bgp_keepalives_off(peer);
 		BGP_TIMER_OFF(peer->t_routeadv);
 		break;
 
@@ -279,7 +326,7 @@ void bgp_timer_set(struct peer *peer)
 				     peer->v_connect);
 		}
 		BGP_TIMER_OFF(peer->t_holdtime);
-		BGP_TIMER_OFF(peer->t_keepalive);
+		bgp_keepalives_off(peer);
 		BGP_TIMER_OFF(peer->t_routeadv);
 		break;
 
@@ -293,7 +340,7 @@ void bgp_timer_set(struct peer *peer)
 		} else {
 			BGP_TIMER_OFF(peer->t_holdtime);
 		}
-		BGP_TIMER_OFF(peer->t_keepalive);
+		bgp_keepalives_off(peer);
 		BGP_TIMER_OFF(peer->t_routeadv);
 		break;
 
@@ -306,12 +353,11 @@ void bgp_timer_set(struct peer *peer)
 		   timer and KeepAlive timers are not started. */
 		if (peer->v_holdtime == 0) {
 			BGP_TIMER_OFF(peer->t_holdtime);
-			BGP_TIMER_OFF(peer->t_keepalive);
+			bgp_keepalives_off(peer);
 		} else {
 			BGP_TIMER_ON(peer->t_holdtime, bgp_holdtime_timer,
 				     peer->v_holdtime);
-			BGP_TIMER_ON(peer->t_keepalive, bgp_keepalive_timer,
-				     peer->v_keepalive);
+			bgp_keepalives_on(peer);
 		}
 		BGP_TIMER_OFF(peer->t_routeadv);
 		break;
@@ -326,24 +372,23 @@ void bgp_timer_set(struct peer *peer)
 		   and keepalive must be turned off. */
 		if (peer->v_holdtime == 0) {
 			BGP_TIMER_OFF(peer->t_holdtime);
-			BGP_TIMER_OFF(peer->t_keepalive);
+			bgp_keepalives_off(peer);
 		} else {
 			BGP_TIMER_ON(peer->t_holdtime, bgp_holdtime_timer,
 				     peer->v_holdtime);
-			BGP_TIMER_ON(peer->t_keepalive, bgp_keepalive_timer,
-				     peer->v_keepalive);
+			bgp_keepalives_on(peer);
 		}
 		break;
 	case Deleted:
 		BGP_TIMER_OFF(peer->t_gr_restart);
 		BGP_TIMER_OFF(peer->t_gr_stale);
 		BGP_TIMER_OFF(peer->t_pmax_restart);
-		/* fallthru */
+	/* fallthru */
 	case Clearing:
 		BGP_TIMER_OFF(peer->t_start);
 		BGP_TIMER_OFF(peer->t_connect);
 		BGP_TIMER_OFF(peer->t_holdtime);
-		BGP_TIMER_OFF(peer->t_keepalive);
+		bgp_keepalives_off(peer);
 		BGP_TIMER_OFF(peer->t_routeadv);
 		break;
 	}
@@ -374,6 +419,10 @@ static int bgp_connect_timer(struct thread *thread)
 	int ret;
 
 	peer = THREAD_ARG(thread);
+
+	assert(!peer->t_write);
+	assert(!peer->t_read);
+
 	peer->t_connect = NULL;
 
 	if (bgp_debug_neighbor_events(peer))
@@ -409,24 +458,6 @@ static int bgp_holdtime_timer(struct thread *thread)
 	return 0;
 }
 
-/* BGP keepalive fire ! */
-static int bgp_keepalive_timer(struct thread *thread)
-{
-	struct peer *peer;
-
-	peer = THREAD_ARG(thread);
-	peer->t_keepalive = NULL;
-
-	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s [FSM] Timer (keepalive timer expire)",
-			   peer->host);
-
-	THREAD_VAL(thread) = KeepAlive_timer_expired;
-	bgp_event(thread); /* bgp_event unlocks peer */
-
-	return 0;
-}
-
 int bgp_routeadv_timer(struct thread *thread)
 {
 	struct peer *peer;
@@ -440,7 +471,8 @@ int bgp_routeadv_timer(struct thread *thread)
 
 	peer->synctime = bgp_clock();
 
-	BGP_WRITE_ON(peer->t_write, bgp_write, peer->fd);
+	thread_add_timer_msec(bm->master, bgp_generate_updgrp_packets, peer, 0,
+			      &peer->t_generate_updgrp_packets);
 
 	/* MRAI timer will be started again when FIFO is built, no need to
 	 * do it here.
@@ -488,7 +520,7 @@ static int bgp_graceful_restart_timer_expire(struct thread *thread)
 
 	/* NSF delete stale route */
 	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_RESERVED_4; safi++)
+		for (safi = SAFI_UNICAST; safi <= SAFI_MPLS_VPN; safi++)
 			if (peer->nsf[afi][safi])
 				bgp_clear_stale_route(peer, afi, safi);
 
@@ -521,7 +553,7 @@ static int bgp_graceful_stale_timer_expire(struct thread *thread)
 
 	/* NSF delete stale route */
 	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_RESERVED_4; safi++)
+		for (safi = SAFI_UNICAST; safi <= SAFI_MPLS_VPN; safi++)
 			if (peer->nsf[afi][safi])
 				bgp_clear_stale_route(peer, afi, safi);
 
@@ -647,7 +679,9 @@ void bgp_adjust_routeadv(struct peer *peer)
 			BGP_TIMER_OFF(peer->t_routeadv);
 
 		peer->synctime = bgp_clock();
-		BGP_WRITE_ON(peer->t_write, bgp_write, peer->fd);
+		thread_add_timer_msec(bm->master, bgp_generate_updgrp_packets,
+				      peer, 0,
+				      &peer->t_generate_updgrp_packets);
 		return;
 	}
 
@@ -728,8 +762,8 @@ int bgp_maxmed_onstartup_active(struct bgp *bgp)
 
 void bgp_maxmed_update(struct bgp *bgp)
 {
-	u_char maxmed_active;
-	u_int32_t maxmed_value;
+	uint8_t maxmed_active;
+	uint32_t maxmed_value;
 
 	if (bgp->v_maxmed_admin) {
 		maxmed_active = 1;
@@ -776,9 +810,8 @@ static void bgp_maxmed_onstartup_begin(struct bgp *bgp)
 	zlog_info("Begin maxmed onstartup mode - timer %d seconds",
 		  bgp->v_maxmed_onstartup);
 
-	THREAD_TIMER_ON(bm->master, bgp->t_maxmed_onstartup,
-			bgp_maxmed_onstartup_timer, bgp,
-			bgp->v_maxmed_onstartup);
+	thread_add_timer(bm->master, bgp_maxmed_onstartup_timer, bgp,
+			 bgp->v_maxmed_onstartup, &bgp->t_maxmed_onstartup);
 
 	if (!bgp->v_maxmed_admin) {
 		bgp->maxmed_active = 1;
@@ -840,13 +873,12 @@ static void bgp_update_delay_begin(struct bgp *bgp)
 		peer->update_delay_over = 0;
 
 	/* Start the update-delay timer */
-	THREAD_TIMER_ON(bm->master, bgp->t_update_delay, bgp_update_delay_timer,
-			bgp, bgp->v_update_delay);
+	thread_add_timer(bm->master, bgp_update_delay_timer, bgp,
+			 bgp->v_update_delay, &bgp->t_update_delay);
 
 	if (bgp->v_establish_wait != bgp->v_update_delay)
-		THREAD_TIMER_ON(bm->master, bgp->t_establish_wait,
-				bgp_establish_wait_timer, bgp,
-				bgp->v_establish_wait);
+		thread_add_timer(bm->master, bgp_establish_wait_timer, bgp,
+				 bgp->v_establish_wait, &bgp->t_establish_wait);
 
 	quagga_timestamp(3, bgp->update_delay_begin_time,
 			 sizeof(bgp->update_delay_begin_time));
@@ -1023,8 +1055,8 @@ int bgp_stop(struct peer *peer)
 			UNSET_FLAG(peer->sflags, PEER_STATUS_NSF_MODE);
 
 			for (afi = AFI_IP; afi < AFI_MAX; afi++)
-				for (safi = SAFI_UNICAST;
-				     safi < SAFI_RESERVED_4; safi++)
+				for (safi = SAFI_UNICAST; safi <= SAFI_MPLS_VPN;
+				     safi++)
 					peer->nsf[afi][safi] = 0;
 		}
 
@@ -1044,27 +1076,41 @@ int bgp_stop(struct peer *peer)
 		bgp_bfd_deregister_peer(peer);
 	}
 
-	/* Stop read and write threads when exists. */
-	BGP_READ_OFF(peer->t_read);
-	BGP_WRITE_OFF(peer->t_write);
+	/* stop keepalives */
+	bgp_keepalives_off(peer);
+
+	/* Stop read and write threads. */
+	bgp_writes_off(peer);
+	bgp_reads_off(peer);
+
+	THREAD_OFF(peer->t_connect_check_r);
+	THREAD_OFF(peer->t_connect_check_w);
 
 	/* Stop all timers. */
 	BGP_TIMER_OFF(peer->t_start);
 	BGP_TIMER_OFF(peer->t_connect);
 	BGP_TIMER_OFF(peer->t_holdtime);
-	BGP_TIMER_OFF(peer->t_keepalive);
 	BGP_TIMER_OFF(peer->t_routeadv);
 
-	/* Stream reset. */
-	peer->packet_size = 0;
-
 	/* Clear input and output buffer.  */
-	if (peer->ibuf)
-		stream_reset(peer->ibuf);
-	if (peer->work)
-		stream_reset(peer->work);
-	if (peer->obuf)
-		stream_fifo_clean(peer->obuf);
+	pthread_mutex_lock(&peer->io_mtx);
+	{
+		if (peer->ibuf)
+			stream_fifo_clean(peer->ibuf);
+		if (peer->obuf)
+			stream_fifo_clean(peer->obuf);
+
+		if (peer->ibuf_work)
+			ringbuf_wipe(peer->ibuf_work);
+		if (peer->obuf_work)
+			stream_reset(peer->obuf_work);
+
+		if (peer->curr) {
+			stream_free(peer->curr);
+			peer->curr = NULL;
+		}
+	}
+	pthread_mutex_unlock(&peer->io_mtx);
 
 	/* Close of file descriptor. */
 	if (peer->fd >= 0) {
@@ -1072,33 +1118,31 @@ int bgp_stop(struct peer *peer)
 		peer->fd = -1;
 	}
 
-	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++) {
-			/* Reset all negotiated variables */
-			peer->afc_nego[afi][safi] = 0;
-			peer->afc_adv[afi][safi] = 0;
-			peer->afc_recv[afi][safi] = 0;
+	FOREACH_AFI_SAFI (afi, safi) {
+		/* Reset all negotiated variables */
+		peer->afc_nego[afi][safi] = 0;
+		peer->afc_adv[afi][safi] = 0;
+		peer->afc_recv[afi][safi] = 0;
 
-			/* peer address family capability flags*/
-			peer->af_cap[afi][safi] = 0;
+		/* peer address family capability flags*/
+		peer->af_cap[afi][safi] = 0;
 
-			/* peer address family status flags*/
-			peer->af_sflags[afi][safi] = 0;
+		/* peer address family status flags*/
+		peer->af_sflags[afi][safi] = 0;
 
-			/* Received ORF prefix-filter */
-			peer->orf_plist[afi][safi] = NULL;
+		/* Received ORF prefix-filter */
+		peer->orf_plist[afi][safi] = NULL;
 
-			if ((peer->status == OpenConfirm)
-			    || (peer->status == Established)) {
-				/* ORF received prefix-filter pnt */
-				sprintf(orf_name, "%s.%d.%d", peer->host, afi,
-					safi);
-				prefix_bgp_orf_remove_all(afi, orf_name);
-			}
+		if ((peer->status == OpenConfirm)
+		    || (peer->status == Established)) {
+			/* ORF received prefix-filter pnt */
+			sprintf(orf_name, "%s.%d.%d", peer->host, afi, safi);
+			prefix_bgp_orf_remove_all(afi, orf_name);
 		}
+	}
 
 	/* Reset keepalive and holdtime */
-	if (CHECK_FLAG(peer->config, PEER_CONFIG_TIMER)) {
+	if (CHECK_FLAG(peer->flags, PEER_FLAG_TIMER)) {
 		peer->v_keepalive = peer->keepalive;
 		peer->v_holdtime = peer->holdtime;
 	} else {
@@ -1114,9 +1158,11 @@ int bgp_stop(struct peer *peer)
   /* Reset prefix count */
   peer->pcount[AFI_IP][SAFI_UNICAST] = 0;
   peer->pcount[AFI_IP][SAFI_MULTICAST] = 0;
+  peer->pcount[AFI_IP][SAFI_LABELED_UNICAST] = 0;
   peer->pcount[AFI_IP][SAFI_MPLS_VPN] = 0;
   peer->pcount[AFI_IP6][SAFI_UNICAST] = 0;
   peer->pcount[AFI_IP6][SAFI_MULTICAST] = 0;
+  peer->pcount[AFI_IP6][SAFI_LABELED_UNICAST] = 0;
 #endif /* 0 */
 
 	if (!CHECK_FLAG(peer->flags, PEER_FLAG_CONFIG_NODE)
@@ -1152,7 +1198,8 @@ static int bgp_stop_with_error(struct peer *peer)
 
 
 /* something went wrong, send notify and tear down */
-static int bgp_stop_with_notify(struct peer *peer, u_char code, u_char sub_code)
+static int bgp_stop_with_notify(struct peer *peer, uint8_t code,
+				uint8_t sub_code)
 {
 	/* Send notify to remote peer */
 	bgp_notify_send(peer, code, sub_code);
@@ -1170,6 +1217,61 @@ static int bgp_stop_with_notify(struct peer *peer, u_char code, u_char sub_code)
 	return (bgp_stop(peer));
 }
 
+/**
+ * Determines whether a TCP session has successfully established for a peer and
+ * events as appropriate.
+ *
+ * This function is called when setting up a new session. After connect() is
+ * called on the peer's socket (in bgp_start()), the fd is passed to poll()
+ * to wait for connection success or failure. When poll() returns, this
+ * function is called to evaluate the result.
+ *
+ * Due to differences in behavior of poll() on Linux and BSD - specifically,
+ * the value of .revents in the case of a closed connection - this function is
+ * scheduled both for a read and a write event. The write event is triggered
+ * when the connection is established. A read event is triggered when the
+ * connection is closed. Thus we need to cancel whichever one did not occur.
+ */
+static int bgp_connect_check(struct thread *thread)
+{
+	int status;
+	socklen_t slen;
+	int ret;
+	struct peer *peer;
+
+	peer = THREAD_ARG(thread);
+	assert(!CHECK_FLAG(peer->thread_flags, PEER_THREAD_READS_ON));
+	assert(!CHECK_FLAG(peer->thread_flags, PEER_THREAD_WRITES_ON));
+	assert(!peer->t_read);
+	assert(!peer->t_write);
+
+	THREAD_OFF(peer->t_connect_check_r);
+	THREAD_OFF(peer->t_connect_check_w);
+
+	/* Check file descriptor. */
+	slen = sizeof(status);
+	ret = getsockopt(peer->fd, SOL_SOCKET, SO_ERROR, (void *)&status,
+			 &slen);
+
+	/* If getsockopt is fail, this is fatal error. */
+	if (ret < 0) {
+		zlog_info("can't get sockopt for nonblocking connect");
+		BGP_EVENT_ADD(peer, TCP_fatal_error);
+		return -1;
+	}
+
+	/* When status is 0 then TCP connection is established. */
+	if (status == 0) {
+		BGP_EVENT_ADD(peer, TCP_connection_open);
+		return 1;
+	} else {
+		if (bgp_debug_neighbor_events(peer))
+			zlog_debug("%s [Event] Connect failed (%s)", peer->host,
+				   safe_strerror(errno));
+		BGP_EVENT_ADD(peer, TCP_connection_open_failed);
+		return 0;
+	}
+}
 
 /* TCP connection open.  Next we send open message to remote peer. And
    add read thread for reading open message. */
@@ -1187,10 +1289,11 @@ static int bgp_connect_success(struct peer *peer)
 			 __FUNCTION__, peer->host, peer->fd);
 		bgp_notify_send(peer, BGP_NOTIFY_FSM_ERR,
 				0); /* internal error */
+		bgp_writes_on(peer);
 		return -1;
 	}
 
-	BGP_READ_ON(peer->t_read, bgp_read, peer->fd);
+	bgp_reads_on(peer);
 
 	if (bgp_debug_neighbor_events(peer)) {
 		char buf1[SU_ADDRSTRLEN];
@@ -1274,6 +1377,14 @@ int bgp_start(struct peer *peer)
 		return 0;
 	}
 
+	if (peer->bgp->vrf_id == VRF_UNKNOWN) {
+		if (bgp_debug_neighbor_events(peer))
+			zlog_err(
+				"%s [FSM] In a VRF that is not initialised yet",
+				peer->host);
+		return -1;
+	}
+
 	/* Register to be notified on peer up */
 	if (peer->sort == BGP_PEER_EBGP && peer->ttl == 1
 	    && !CHECK_FLAG(peer->flags, PEER_FLAG_DISABLE_CONNECTED_CHECK)
@@ -1282,7 +1393,7 @@ int bgp_start(struct peer *peer)
 	else
 		connected = 0;
 
-	if (!bgp_find_or_add_nexthop(peer->bgp,
+	if (!bgp_find_or_add_nexthop(peer->bgp, peer->bgp,
 				     family2afi(peer->su.sa.sa_family), NULL,
 				     peer, connected)) {
 #if defined(HAVE_CUMULUS)
@@ -1294,6 +1405,10 @@ int bgp_start(struct peer *peer)
 #endif
 	}
 
+	assert(!peer->t_write);
+	assert(!peer->t_read);
+	assert(!CHECK_FLAG(peer->thread_flags, PEER_THREAD_WRITES_ON));
+	assert(!CHECK_FLAG(peer->thread_flags, PEER_THREAD_READS_ON));
 	status = bgp_connect(peer);
 
 	switch (status) {
@@ -1321,8 +1436,19 @@ int bgp_start(struct peer *peer)
 				 peer->fd);
 			return -1;
 		}
-		BGP_READ_ON(peer->t_read, bgp_read, peer->fd);
-		BGP_WRITE_ON(peer->t_write, bgp_write, peer->fd);
+		/*
+		 * - when the socket becomes ready, poll() will signify POLLOUT
+		 * - if it fails to connect, poll() will signify POLLHUP
+		 * - POLLHUP is handled as a 'read' event by thread.c
+		 *
+		 * therefore, we schedule both a read and a write event with
+		 * bgp_connect_check() as the handler for each and cancel the
+		 * unused event in that function.
+		 */
+		thread_add_read(bm->master, bgp_connect_check, peer, peer->fd,
+				&peer->t_connect_check_r);
+		thread_add_write(bm->master, bgp_connect_check, peer, peer->fd,
+				 &peer->t_connect_check_w);
 		break;
 	}
 	return 0;
@@ -1349,13 +1475,6 @@ static int bgp_fsm_open(struct peer *peer)
 	return 0;
 }
 
-/* Keepalive send to peer. */
-static int bgp_fsm_keepalive_expire(struct peer *peer)
-{
-	bgp_keepalive_send(peer);
-	return 0;
-}
-
 /* FSM error, unexpected event.  This is error of BGP connection. So cut the
    peer and change to Idle status. */
 static int bgp_fsm_event_error(struct peer *peer)
@@ -1376,8 +1495,12 @@ static int bgp_fsm_holdtime_expire(struct peer *peer)
 	return bgp_stop_with_notify(peer, BGP_NOTIFY_HOLD_ERR, 0);
 }
 
-/* Status goes to Established.  Send keepalive packet then make first
-   update information. */
+/**
+ * Transition to Established state.
+ *
+ * Convert peer from stub to full fledged peer, set some timers, and generate
+ * initial updates.
+ */
 static int bgp_establish(struct peer *peer)
 {
 	afi_t afi;
@@ -1394,9 +1517,8 @@ static int bgp_establish(struct peer *peer)
 	}
 
 	if (other == peer)
-		ret =
-			1; /* bgp_establish specific code when xfer_conn
-			      happens. */
+		ret = 1; /* bgp_establish specific code when xfer_conn
+			    happens. */
 
 	/* Reset capability open status flag. */
 	if (!CHECK_FLAG(peer->sflags, PEER_STATUS_CAPABILITY_OPEN))
@@ -1425,7 +1547,7 @@ static int bgp_establish(struct peer *peer)
 	/* graceful restart */
 	UNSET_FLAG(peer->sflags, PEER_STATUS_NSF_WAIT);
 	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_RESERVED_4; safi++) {
+		for (safi = SAFI_UNICAST; safi <= SAFI_MPLS_VPN; safi++) {
 			if (peer->afc_nego[afi][safi]
 			    && CHECK_FLAG(peer->cap, PEER_CAP_RESTART_ADV)
 			    && CHECK_FLAG(peer->af_cap[afi][safi],
@@ -1467,42 +1589,40 @@ static int bgp_establish(struct peer *peer)
 
 	hook_call(peer_established, peer);
 
-	/* Reset uptime, send keepalive, send current table. */
+	/* Reset uptime, turn on keepalives, send current table. */
+	if (!peer->v_holdtime)
+		bgp_keepalives_on(peer);
+
 	peer->uptime = bgp_clock();
 
 	/* Send route-refresh when ORF is enabled */
-	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++)
+	FOREACH_AFI_SAFI (afi, safi) {
+		if (CHECK_FLAG(peer->af_cap[afi][safi],
+			       PEER_CAP_ORF_PREFIX_SM_ADV)) {
 			if (CHECK_FLAG(peer->af_cap[afi][safi],
-				       PEER_CAP_ORF_PREFIX_SM_ADV)) {
-				if (CHECK_FLAG(peer->af_cap[afi][safi],
-					       PEER_CAP_ORF_PREFIX_RM_RCV))
-					bgp_route_refresh_send(
-						peer, afi, safi,
-						ORF_TYPE_PREFIX,
-						REFRESH_IMMEDIATE, 0);
-				else if (
-					CHECK_FLAG(
-						peer->af_cap[afi][safi],
-						PEER_CAP_ORF_PREFIX_RM_OLD_RCV))
-					bgp_route_refresh_send(
-						peer, afi, safi,
-						ORF_TYPE_PREFIX_OLD,
-						REFRESH_IMMEDIATE, 0);
-			}
+				       PEER_CAP_ORF_PREFIX_RM_RCV))
+				bgp_route_refresh_send(peer, afi, safi,
+						       ORF_TYPE_PREFIX,
+						       REFRESH_IMMEDIATE, 0);
+			else if (CHECK_FLAG(peer->af_cap[afi][safi],
+					    PEER_CAP_ORF_PREFIX_RM_OLD_RCV))
+				bgp_route_refresh_send(peer, afi, safi,
+						       ORF_TYPE_PREFIX_OLD,
+						       REFRESH_IMMEDIATE, 0);
+		}
+	}
 
 	/* First update is deferred until ORF or ROUTE-REFRESH is received */
-	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++)
+	FOREACH_AFI_SAFI (afi, safi) {
+		if (CHECK_FLAG(peer->af_cap[afi][safi],
+			       PEER_CAP_ORF_PREFIX_RM_ADV))
 			if (CHECK_FLAG(peer->af_cap[afi][safi],
-				       PEER_CAP_ORF_PREFIX_RM_ADV))
-				if (CHECK_FLAG(peer->af_cap[afi][safi],
-					       PEER_CAP_ORF_PREFIX_SM_RCV)
-				    || CHECK_FLAG(
-					       peer->af_cap[afi][safi],
-					       PEER_CAP_ORF_PREFIX_SM_OLD_RCV))
-					SET_FLAG(peer->af_sflags[afi][safi],
-						 PEER_STATUS_ORF_WAIT_REFRESH);
+				       PEER_CAP_ORF_PREFIX_SM_RCV)
+			    || CHECK_FLAG(peer->af_cap[afi][safi],
+					  PEER_CAP_ORF_PREFIX_SM_OLD_RCV))
+				SET_FLAG(peer->af_sflags[afi][safi],
+					 PEER_STATUS_ORF_WAIT_REFRESH);
+	}
 
 	bgp_announce_peer(peer);
 
@@ -1537,11 +1657,6 @@ static int bgp_establish(struct peer *peer)
 /* Keepalive packet is received. */
 static int bgp_fsm_keepalive(struct peer *peer)
 {
-	bgp_update_implicit_eors(peer);
-
-	/* peer count update */
-	peer->keepalive_in++;
-
 	BGP_TIMER_OFF(peer->t_holdtime);
 	return 0;
 }
@@ -1714,9 +1829,8 @@ static const struct {
 		{bgp_stop, Clearing},      /* TCP_fatal_error              */
 		{bgp_stop, Clearing},      /* ConnectRetry_timer_expired   */
 		{bgp_fsm_holdtime_expire, Clearing}, /* Hold_Timer_expired */
-		{bgp_fsm_keepalive_expire,
-		 Established},	/* KeepAlive_timer_expired      */
-		{bgp_stop, Clearing}, /* Receive_OPEN_message         */
+		{bgp_ignore, Established}, /* KeepAlive_timer_expired      */
+		{bgp_stop, Clearing},      /* Receive_OPEN_message         */
 		{bgp_fsm_keepalive,
 		 Established}, /* Receive_KEEPALIVE_message    */
 		{bgp_fsm_update, Established}, /* Receive_UPDATE_message */
@@ -1783,6 +1897,9 @@ int bgp_event_update(struct peer *peer, int event)
 	int passive_conn = 0;
 	int dyn_nbr;
 
+	/* default return code */
+	ret = FSM_PEER_NOOP;
+
 	other = peer->doppelganger;
 	passive_conn =
 		(CHECK_FLAG(peer->sflags, PEER_STATUS_ACCEPT_PEER)) ? 1 : 0;
@@ -1804,37 +1921,56 @@ int bgp_event_update(struct peer *peer, int event)
 	if (FSM[peer->status - 1][event - 1].func)
 		ret = (*(FSM[peer->status - 1][event - 1].func))(peer);
 
-	/* When function do not want proceed next job return -1. */
 	if (ret >= 0) {
 		if (ret == 1 && next == Established) {
 			/* The case when doppelganger swap accurred in
 			   bgp_establish.
 			   Update the peer pointer accordingly */
+			ret = FSM_PEER_TRANSFERRED;
 			peer = other;
 		}
 
 		/* If status is changed. */
-		if (next != peer->status)
+		if (next != peer->status) {
 			bgp_fsm_change_status(peer, next);
 
+			/*
+			 * If we're going to ESTABLISHED then we executed a
+			 * peer transfer. In this case we can either return
+			 * FSM_PEER_TRANSITIONED or FSM_PEER_TRANSFERRED.
+			 * Opting for TRANSFERRED since transfer implies
+			 * session establishment.
+			 */
+			if (ret != FSM_PEER_TRANSFERRED)
+				ret = FSM_PEER_TRANSITIONED;
+		}
+
 		/* Make sure timer is set. */
 		bgp_timer_set(peer);
 
-	} else if (!dyn_nbr && !passive_conn && peer->bgp) {
-		/* If we got a return value of -1, that means there was an
-		 * error, restart
-		 * the FSM. If the peer structure was deleted
+	} else {
+		/*
+		 * If we got a return value of -1, that means there was an
+		 * error, restart the FSM. Since bgp_stop() was called on the
+		 * peer. only a few fields are safe to access here. In any case
+		 * we need to indicate that the peer was stopped in the return
+		 * code.
 		 */
-		zlog_err(
-			"%s [FSM] Failure handling event %s in state %s, "
-			"prior events %s, %s, fd %d",
-			peer->host, bgp_event_str[peer->cur_event],
-			lookup_msg(bgp_status_msg, peer->status, NULL),
-			bgp_event_str[peer->last_event],
-			bgp_event_str[peer->last_major_event], peer->fd);
-		bgp_stop(peer);
-		bgp_fsm_change_status(peer, Idle);
-		bgp_timer_set(peer);
+		if (!dyn_nbr && !passive_conn && peer->bgp) {
+			zlog_err(
+				"%s [FSM] Failure handling event %s in state %s, "
+				"prior events %s, %s, fd %d",
+				peer->host, bgp_event_str[peer->cur_event],
+				lookup_msg(bgp_status_msg, peer->status, NULL),
+				bgp_event_str[peer->last_event],
+				bgp_event_str[peer->last_major_event],
+				peer->fd);
+			bgp_stop(peer);
+			bgp_fsm_change_status(peer, Idle);
+			bgp_timer_set(peer);
+		}
+		ret = FSM_PEER_STOPPED;
 	}
+
 	return ret;
 }
