@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with Quagga; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -47,7 +46,7 @@
  * Record maximum-paths configuration for BGP instance
  */
 int bgp_maximum_paths_set(struct bgp *bgp, afi_t afi, safi_t safi, int peertype,
-			  u_int16_t maxpaths, u_int16_t options)
+			  uint16_t maxpaths, uint16_t options)
 {
 	if (!bgp || (afi >= AFI_MAX) || (safi >= SAFI_MAX))
 		return -1;
@@ -93,6 +92,26 @@ int bgp_maximum_paths_unset(struct bgp *bgp, afi_t afi, safi_t safi,
 	return 0;
 }
 
+/*
+ * bgp_interface_same
+ *
+ * Return true if ifindex for ifp1 and ifp2 are the same, else return false.
+ */
+static int bgp_interface_same(struct interface *ifp1, struct interface *ifp2)
+{
+	if (!ifp1 && !ifp2)
+		return 1;
+
+	if (!ifp1 && ifp2)
+		return 0;
+
+	if (ifp1 && !ifp2)
+		return 0;
+
+	return (ifp1->ifindex == ifp2->ifindex);
+}
+
+
 /*
  * bgp_info_nexthop_cmp
  *
@@ -102,36 +121,40 @@ int bgp_maximum_paths_unset(struct bgp *bgp, afi_t afi, safi_t safi,
  */
 int bgp_info_nexthop_cmp(struct bgp_info *bi1, struct bgp_info *bi2)
 {
-	struct attr_extra *ae1, *ae2;
 	int compare;
-
-	ae1 = bi1->attr->extra;
-	ae2 = bi2->attr->extra;
+	struct in6_addr addr1, addr2;
 
 	compare = IPV4_ADDR_CMP(&bi1->attr->nexthop, &bi2->attr->nexthop);
-	if (!compare && ae1 && ae2) {
-		if (ae1->mp_nexthop_len == ae2->mp_nexthop_len) {
-			switch (ae1->mp_nexthop_len) {
+	if (!compare) {
+		if (bi1->attr->mp_nexthop_len == bi2->attr->mp_nexthop_len) {
+			switch (bi1->attr->mp_nexthop_len) {
 			case BGP_ATTR_NHLEN_IPV4:
 			case BGP_ATTR_NHLEN_VPNV4:
 				compare = IPV4_ADDR_CMP(
-					&ae1->mp_nexthop_global_in,
-					&ae2->mp_nexthop_global_in);
+					&bi1->attr->mp_nexthop_global_in,
+					&bi2->attr->mp_nexthop_global_in);
 				break;
 			case BGP_ATTR_NHLEN_IPV6_GLOBAL:
 			case BGP_ATTR_NHLEN_VPNV6_GLOBAL:
-				compare =
-					IPV6_ADDR_CMP(&ae1->mp_nexthop_global,
-						      &ae2->mp_nexthop_global);
+				compare = IPV6_ADDR_CMP(
+					&bi1->attr->mp_nexthop_global,
+					&bi2->attr->mp_nexthop_global);
 				break;
 			case BGP_ATTR_NHLEN_IPV6_GLOBAL_AND_LL:
-				compare =
-					IPV6_ADDR_CMP(&ae1->mp_nexthop_global,
-						      &ae2->mp_nexthop_global);
+				addr1 = (bi1->attr->mp_nexthop_prefer_global)
+						? bi1->attr->mp_nexthop_global
+						: bi1->attr->mp_nexthop_local;
+				addr2 = (bi2->attr->mp_nexthop_prefer_global)
+						? bi2->attr->mp_nexthop_global
+						: bi2->attr->mp_nexthop_local;
+
+				if (!bi1->attr->mp_nexthop_prefer_global
+				    && !bi2->attr->mp_nexthop_prefer_global)
+					compare = !bgp_interface_same(
+						bi1->peer->ifp, bi2->peer->ifp);
+
 				if (!compare)
-					compare = IPV6_ADDR_CMP(
-						&ae1->mp_nexthop_local,
-						&ae2->mp_nexthop_local);
+					compare = IPV6_ADDR_CMP(&addr1, &addr2);
 				break;
 			}
 		}
@@ -140,13 +163,14 @@ int bgp_info_nexthop_cmp(struct bgp_info *bi1, struct bgp_info *bi2)
 		 * link-local
 		 * nexthops but another IPv6 peer only sends you global
 		 */
-		else if (ae1->mp_nexthop_len == BGP_ATTR_NHLEN_IPV6_GLOBAL
-			 || ae1->mp_nexthop_len
+		else if (bi1->attr->mp_nexthop_len == BGP_ATTR_NHLEN_IPV6_GLOBAL
+			 || bi1->attr->mp_nexthop_len
 				    == BGP_ATTR_NHLEN_IPV6_GLOBAL_AND_LL) {
-			compare = IPV6_ADDR_CMP(&ae1->mp_nexthop_global,
-						&ae2->mp_nexthop_global);
+			compare = IPV6_ADDR_CMP(&bi1->attr->mp_nexthop_global,
+						&bi2->attr->mp_nexthop_global);
 			if (!compare) {
-				if (ae1->mp_nexthop_len < ae2->mp_nexthop_len)
+				if (bi1->attr->mp_nexthop_len
+				    < bi2->attr->mp_nexthop_len)
 					compare = -1;
 				else
 					compare = 1;
@@ -346,7 +370,7 @@ struct bgp_info *bgp_info_mpath_first(struct bgp_info *binfo)
  *
  * Given the bestpath bgp_info, return the number of multipath entries
  */
-u_int32_t bgp_info_mpath_count(struct bgp_info *binfo)
+uint32_t bgp_info_mpath_count(struct bgp_info *binfo)
 {
 	if (!binfo->mpath)
 		return 0;
@@ -358,7 +382,7 @@ u_int32_t bgp_info_mpath_count(struct bgp_info *binfo)
  *
  * Sets the count of multipaths into bestpath's mpath element
  */
-static void bgp_info_mpath_count_set(struct bgp_info *binfo, u_int32_t count)
+static void bgp_info_mpath_count_set(struct bgp_info *binfo, uint32_t count)
 {
 	struct bgp_info_mpath *mpath;
 	if (!count && !binfo->mpath)
@@ -408,7 +432,7 @@ void bgp_info_mpath_update(struct bgp_node *rn, struct bgp_info *new_best,
 			   struct bgp_info *old_best, struct list *mp_list,
 			   struct bgp_maxpaths_cfg *mpath_cfg)
 {
-	u_int16_t maxpaths, mpath_count, old_mpath_count;
+	uint16_t maxpaths, mpath_count, old_mpath_count;
 	struct listnode *mp_node, *mp_next_node;
 	struct bgp_info *cur_mpath, *new_mpath, *next_mpath, *prev_mpath;
 	int mpath_changed, debug;
@@ -568,6 +592,8 @@ void bgp_info_mpath_update(struct bgp_node *rn, struct bgp_info *new_best,
 			 */
 			new_mpath = listgetdata(mp_node);
 			list_delete_node(mp_list, mp_node);
+			assert(new_mpath);
+			assert(prev_mpath);
 			if ((mpath_count < maxpaths) && (new_mpath != new_best)
 			    && bgp_info_nexthop_cmp(prev_mpath, new_mpath)) {
 				if (new_mpath == next_mpath)
@@ -653,11 +679,10 @@ void bgp_info_mpath_aggregate_update(struct bgp_info *new_best,
 	struct aspath *aspath;
 	struct aspath *asmerge;
 	struct attr *new_attr, *old_attr;
-	u_char origin;
+	uint8_t origin;
 	struct community *community, *commerge;
 	struct ecommunity *ecomm, *ecommerge;
 	struct lcommunity *lcomm, *lcommerge;
-	struct attr_extra *ae;
 	struct attr attr = {0};
 
 	if (old_best && (old_best != new_best)
@@ -688,11 +713,10 @@ void bgp_info_mpath_aggregate_update(struct bgp_info *new_best,
 		origin = attr.origin;
 		community =
 			attr.community ? community_dup(attr.community) : NULL;
-		ae = attr.extra;
-		ecomm = (ae && ae->ecommunity) ? ecommunity_dup(ae->ecommunity)
-					       : NULL;
-		lcomm = (ae && ae->lcommunity) ? lcommunity_dup(ae->lcommunity)
-					       : NULL;
+		ecomm = (attr.ecommunity) ? ecommunity_dup(attr.ecommunity)
+					  : NULL;
+		lcomm = (attr.lcommunity) ? lcommunity_dup(attr.lcommunity)
+					  : NULL;
 
 		for (mpinfo = bgp_info_mpath_first(new_best); mpinfo;
 		     mpinfo = bgp_info_mpath_next(mpinfo)) {
@@ -717,24 +741,27 @@ void bgp_info_mpath_aggregate_update(struct bgp_info *new_best,
 						mpinfo->attr->community);
 			}
 
-			ae = mpinfo->attr->extra;
-			if (ae && ae->ecommunity) {
+			if (mpinfo->attr->ecommunity) {
 				if (ecomm) {
 					ecommerge = ecommunity_merge(
-						ecomm, ae->ecommunity);
+						ecomm,
+						mpinfo->attr->ecommunity);
 					ecomm = ecommunity_uniq_sort(ecommerge);
 					ecommunity_free(&ecommerge);
 				} else
-					ecomm = ecommunity_dup(ae->ecommunity);
+					ecomm = ecommunity_dup(
+						mpinfo->attr->ecommunity);
 			}
-			if (ae && ae->lcommunity) {
+			if (mpinfo->attr->lcommunity) {
 				if (lcomm) {
 					lcommerge = lcommunity_merge(
-						lcomm, ae->lcommunity);
+						lcomm,
+						mpinfo->attr->lcommunity);
 					lcomm = lcommunity_uniq_sort(lcommerge);
 					lcommunity_free(&lcommerge);
 				} else
-					lcomm = lcommunity_dup(ae->lcommunity);
+					lcomm = lcommunity_dup(
+						mpinfo->attr->lcommunity);
 			}
 		}
 
@@ -745,22 +772,22 @@ void bgp_info_mpath_aggregate_update(struct bgp_info *new_best,
 			attr.flag |= ATTR_FLAG_BIT(BGP_ATTR_COMMUNITIES);
 		}
 		if (ecomm) {
-			ae = bgp_attr_extra_get(&attr);
-			ae->ecommunity = ecomm;
+			attr.ecommunity = ecomm;
 			attr.flag |= ATTR_FLAG_BIT(BGP_ATTR_EXT_COMMUNITIES);
 		}
+		if (lcomm) {
+			attr.lcommunity = lcomm;
+			attr.flag |= ATTR_FLAG_BIT(BGP_ATTR_LARGE_COMMUNITIES);
+		}
 
 		/* Zap multipath attr nexthop so we set nexthop to self */
 		attr.nexthop.s_addr = 0;
-		if (attr.extra)
-			memset(&attr.extra->mp_nexthop_global, 0,
-			       sizeof(struct in6_addr));
+		memset(&attr.mp_nexthop_global, 0, sizeof(struct in6_addr));
 
 		/* TODO: should we set ATOMIC_AGGREGATE and AGGREGATOR? */
 	}
 
 	new_attr = bgp_attr_intern(&attr);
-	bgp_attr_extra_free(&attr);
 
 	if (new_attr != bgp_info_mpath_attr(new_best)) {
 		if ((old_attr = bgp_info_mpath_attr(new_best)))
