@@ -1,22 +1,22 @@
 /* BGP open message handling
-   Copyright (C) 1998, 1999 Kunihiro Ishiguro
-
-This file is part of GNU Zebra.
-
-GNU Zebra is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
-
-GNU Zebra is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with GNU Zebra; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+ * Copyright (C) 1998, 1999 Kunihiro Ishiguro
+ *
+ * This file is part of GNU Zebra.
+ *
+ * GNU Zebra is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ *
+ * GNU Zebra is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -51,8 +51,8 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
    Next, if we send capability to the peer we want to set my capabilty
    inforation at each peer. */
 
-void bgp_capability_vty_out(struct vty *vty, struct peer *peer, u_char use_json,
-			    json_object *json_neigh)
+void bgp_capability_vty_out(struct vty *vty, struct peer *peer,
+			    uint8_t use_json, json_object *json_neigh)
 {
 	char *pnt;
 	char *end;
@@ -122,6 +122,12 @@ void bgp_capability_vty_out(struct vty *vty, struct peer *peer, u_char use_json,
 						"capabilityErrorMultiProtocolSafi",
 						"multicast");
 					break;
+				case SAFI_LABELED_UNICAST:
+					json_object_string_add(
+						json_cap,
+						"capabilityErrorMultiProtocolSafi",
+						"labeled-unicast");
+					break;
 				case SAFI_MPLS_VPN:
 					json_object_string_add(
 						json_cap,
@@ -140,6 +146,12 @@ void bgp_capability_vty_out(struct vty *vty, struct peer *peer, u_char use_json,
 						"capabilityErrorMultiProtocolSafi",
 						"EVPN");
 					break;
+				case SAFI_FLOWSPEC:
+					json_object_string_add(
+						json_cap,
+						"capabilityErrorMultiProtocolSafi",
+						"flowspec");
+					break;
 				default:
 					json_object_int_add(
 						json_cap,
@@ -172,12 +184,18 @@ void bgp_capability_vty_out(struct vty *vty, struct peer *peer, u_char use_json,
 				case SAFI_MULTICAST:
 					vty_out(vty, "SAFI Multicast");
 					break;
+				case SAFI_LABELED_UNICAST:
+					vty_out(vty, "SAFI Labeled-unicast");
+					break;
 				case SAFI_MPLS_VPN:
 					vty_out(vty, "SAFI MPLS-labeled VPN");
 					break;
 				case SAFI_ENCAP:
 					vty_out(vty, "SAFI ENCAP");
 					break;
+				case SAFI_FLOWSPEC:
+					vty_out(vty, "SAFI FLOWSPEC");
+					break;
 				case SAFI_EVPN:
 					vty_out(vty, "SAFI EVPN");
 					break;
@@ -186,7 +204,7 @@ void bgp_capability_vty_out(struct vty *vty, struct peer *peer, u_char use_json,
 						mpc.safi);
 					break;
 				}
-				vty_out(vty, "%s", VTY_NEWLINE);
+				vty_out(vty, "\n");
 			}
 		} else if (hdr->code >= 128) {
 			if (use_json)
@@ -262,8 +280,8 @@ static int bgp_capability_mp(struct peer *peer, struct capability_header *hdr)
 }
 
 static void bgp_capability_orf_not_support(struct peer *peer, iana_afi_t afi,
-					   safi_t safi, u_char type,
-					   u_char mode)
+					   iana_safi_t safi, uint8_t type,
+					   uint8_t mode)
 {
 	if (bgp_debug_neighbor_events(peer))
 		zlog_debug(
@@ -285,25 +303,27 @@ static int bgp_capability_orf_entry(struct peer *peer,
 				    struct capability_header *hdr)
 {
 	struct stream *s = BGP_INPUT(peer);
-	struct capability_orf_entry entry;
+	struct capability_mp_data mpc;
+	uint8_t num;
 	iana_afi_t pkt_afi;
 	afi_t afi;
-	safi_t pkt_safi, safi;
-	u_char type;
-	u_char mode;
-	u_int16_t sm_cap = 0; /* capability send-mode receive */
-	u_int16_t rm_cap = 0; /* capability receive-mode receive */
+	iana_safi_t pkt_safi;
+	safi_t safi;
+	uint8_t type;
+	uint8_t mode;
+	uint16_t sm_cap = 0; /* capability send-mode receive */
+	uint16_t rm_cap = 0; /* capability receive-mode receive */
 	int i;
 
 	/* ORF Entry header */
-	bgp_capability_mp_data(s, &entry.mpc);
-	entry.num = stream_getc(s);
-	pkt_afi = entry.mpc.afi;
-	pkt_safi = entry.mpc.safi;
+	bgp_capability_mp_data(s, &mpc);
+	num = stream_getc(s);
+	pkt_afi = mpc.afi;
+	pkt_safi = mpc.safi;
 
 	if (bgp_debug_neighbor_events(peer))
 		zlog_debug("%s ORF Cap entry for afi/safi: %u/%u", peer->host,
-			   entry.mpc.afi, entry.mpc.safi);
+			   mpc.afi, mpc.safi);
 
 	/* Convert AFI, SAFI to internal values, check. */
 	if (bgp_map_afi_safi_iana2int(pkt_afi, pkt_safi, &afi, &safi)) {
@@ -314,21 +334,21 @@ static int bgp_capability_orf_entry(struct peer *peer,
 		return 0;
 	}
 
-	entry.mpc.afi = pkt_afi;
-	entry.mpc.safi = safi;
+	mpc.afi = pkt_afi;
+	mpc.safi = safi;
 
 	/* validate number field */
-	if (CAPABILITY_CODE_ORF_LEN + (entry.num * 2) > hdr->length) {
+	if (CAPABILITY_CODE_ORF_LEN + (num * 2) > hdr->length) {
 		zlog_info(
 			"%s ORF Capability entry length error,"
 			" Cap length %u, num %u",
-			peer->host, hdr->length, entry.num);
+			peer->host, hdr->length, num);
 		bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
 				BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 		return -1;
 	}
 
-	for (i = 0; i < entry.num; i++) {
+	for (i = 0; i < num; i++) {
 		type = stream_getc(s);
 		mode = stream_getc(s);
 
@@ -422,7 +442,7 @@ static int bgp_capability_restart(struct peer *peer,
 				  struct capability_header *caphdr)
 {
 	struct stream *s = BGP_INPUT(peer);
-	u_int16_t restart_flag_time;
+	uint16_t restart_flag_time;
 	size_t end = stream_get_getp(s) + caphdr->length;
 
 	/* Verify length is a multiple of 4 */
@@ -456,8 +476,8 @@ static int bgp_capability_restart(struct peer *peer,
 		afi_t afi;
 		safi_t safi;
 		iana_afi_t pkt_afi = stream_getw(s);
-		safi_t pkt_safi = stream_getc(s);
-		u_char flag = stream_getc(s);
+		iana_safi_t pkt_safi = stream_getc(s);
+		uint8_t flag = stream_getc(s);
 
 		/* Convert AFI, SAFI to internal values, check. */
 		if (bgp_map_afi_safi_iana2int(pkt_afi, pkt_safi, &afi, &safi)) {
@@ -533,8 +553,8 @@ static int bgp_capability_addpath(struct peer *peer,
 		afi_t afi;
 		safi_t safi;
 		iana_afi_t pkt_afi = stream_getw(s);
-		safi_t pkt_safi = stream_getc(s);
-		u_char send_receive = stream_getc(s);
+		iana_safi_t pkt_safi = stream_getc(s);
+		uint8_t send_receive = stream_getc(s);
 
 		if (bgp_debug_neighbor_events(peer))
 			zlog_debug(
@@ -590,7 +610,8 @@ static int bgp_capability_enhe(struct peer *peer, struct capability_header *hdr)
 	while (stream_get_getp(s) + 6 <= end) {
 		iana_afi_t pkt_afi = stream_getw(s);
 		afi_t afi;
-		safi_t safi, pkt_safi = stream_getw(s);
+		iana_safi_t pkt_safi = stream_getw(s);
+		safi_t safi;
 		iana_afi_t pkt_nh_afi = stream_getw(s);
 		afi_t nh_afi;
 
@@ -614,12 +635,14 @@ static int bgp_capability_enhe(struct peer *peer, struct capability_header *hdr)
 		 * the Nexthop AFI being IPv6. A future spec may introduce other
 		 * possibilities, so we ignore other values with a log. Also,
 		 * only
-		 * Unicast SAFI is currently supported (and expected).
+		 * SAFI_UNICAST and SAFI_LABELED_UNICAST are currently supported
+		 * (and expected).
 		 */
 		nh_afi = afi_iana2int(pkt_nh_afi);
 
-		if (afi != AFI_IP || safi != SAFI_UNICAST
-		    || nh_afi != AFI_IP6) {
+		if (afi != AFI_IP || nh_afi != AFI_IP6
+		    || !(safi == SAFI_UNICAST
+			 || safi == SAFI_LABELED_UNICAST)) {
 			zlog_warn(
 				"%s Unexpected afi/safi/next-hop afi: %u/%u/%u "
 				"in Extended Next-hop capability, ignoring",
@@ -645,7 +668,7 @@ static int bgp_capability_hostname(struct peer *peer,
 	struct stream *s = BGP_INPUT(peer);
 	char str[BGP_MAX_HOSTNAME + 1];
 	size_t end = stream_get_getp(s) + hdr->length;
-	u_char len;
+	uint8_t len;
 
 	SET_FLAG(peer->cap, PEER_CAP_HOSTNAME_RCV);
 
@@ -774,7 +797,7 @@ static const size_t cap_modsizes[] = {
  *                           capabilities were encountered.
  */
 static int bgp_capability_parse(struct peer *peer, size_t length,
-				int *mp_capability, u_char **error)
+				int *mp_capability, uint8_t **error)
 {
 	int ret;
 	struct stream *s = BGP_INPUT(peer);
@@ -784,7 +807,7 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 
 	while (stream_get_getp(s) < end) {
 		size_t start;
-		u_char *sp = stream_pnt(s);
+		uint8_t *sp = stream_pnt(s);
 		struct capability_header caphdr;
 
 		ret = 0;
@@ -977,7 +1000,7 @@ static int strict_capability_same(struct peer *peer)
 /* peek into option, stores ASN to *as4 if the AS4 capability was found.
  * Returns  0 if no as4 found, as4cap value otherwise.
  */
-as_t peek_for_as4_capability(struct peer *peer, u_char length)
+as_t peek_for_as4_capability(struct peer *peer, uint8_t length)
 {
 	struct stream *s = BGP_INPUT(peer);
 	size_t orig_getp = stream_get_getp(s);
@@ -993,8 +1016,8 @@ as_t peek_for_as4_capability(struct peer *peer, u_char length)
 	 * correctly formatted options.
 	 */
 	while (stream_get_getp(s) < end) {
-		u_char opt_type;
-		u_char opt_length;
+		uint8_t opt_type;
+		uint8_t opt_length;
 
 		/* Check the length. */
 		if (stream_get_getp(s) + 2 > end)
@@ -1050,11 +1073,11 @@ end:
  *
  * @param[out] mp_capability @see bgp_capability_parse() for semantics.
  */
-int bgp_open_option_parse(struct peer *peer, u_char length, int *mp_capability)
+int bgp_open_option_parse(struct peer *peer, uint8_t length, int *mp_capability)
 {
 	int ret = 0;
-	u_char *error;
-	u_char error_data[BGP_MAX_PACKET_SIZE];
+	uint8_t *error;
+	uint8_t error_data[BGP_MAX_PACKET_SIZE];
 	struct stream *s = BGP_INPUT(peer);
 	size_t end = stream_get_getp(s) + length;
 
@@ -1065,8 +1088,8 @@ int bgp_open_option_parse(struct peer *peer, u_char length, int *mp_capability)
 			   peer->host, length);
 
 	while (stream_get_getp(s) < end) {
-		u_char opt_type;
-		u_char opt_length;
+		uint8_t opt_type;
+		uint8_t opt_length;
 
 		/* Must have at least an OPEN option header */
 		if (STREAM_READABLE(s) < 2) {
@@ -1149,12 +1172,16 @@ int bgp_open_option_parse(struct peer *peer, u_char length, int *mp_capability)
 	    && !CHECK_FLAG(peer->flags, PEER_FLAG_OVERRIDE_CAPABILITY)) {
 		if (!peer->afc_nego[AFI_IP][SAFI_UNICAST]
 		    && !peer->afc_nego[AFI_IP][SAFI_MULTICAST]
+		    && !peer->afc_nego[AFI_IP][SAFI_LABELED_UNICAST]
 		    && !peer->afc_nego[AFI_IP][SAFI_MPLS_VPN]
 		    && !peer->afc_nego[AFI_IP][SAFI_ENCAP]
+		    && !peer->afc_nego[AFI_IP][SAFI_FLOWSPEC]
 		    && !peer->afc_nego[AFI_IP6][SAFI_UNICAST]
 		    && !peer->afc_nego[AFI_IP6][SAFI_MULTICAST]
+		    && !peer->afc_nego[AFI_IP6][SAFI_LABELED_UNICAST]
 		    && !peer->afc_nego[AFI_IP6][SAFI_MPLS_VPN]
 		    && !peer->afc_nego[AFI_IP6][SAFI_ENCAP]
+		    && !peer->afc_nego[AFI_IP6][SAFI_FLOWSPEC]
 		    && !peer->afc_nego[AFI_L2VPN][SAFI_EVPN]) {
 			zlog_err(
 				"%s [Error] Configured AFI/SAFIs do not "
@@ -1176,16 +1203,16 @@ int bgp_open_option_parse(struct peer *peer, u_char length, int *mp_capability)
 }
 
 static void bgp_open_capability_orf(struct stream *s, struct peer *peer,
-				    afi_t afi, safi_t safi, u_char code)
+				    afi_t afi, safi_t safi, uint8_t code)
 {
-	u_char cap_len;
-	u_char orf_len;
+	uint8_t cap_len;
+	uint8_t orf_len;
 	unsigned long capp;
 	unsigned long orfp;
 	unsigned long numberp;
 	int number_of_orfs = 0;
 	iana_afi_t pkt_afi;
-	safi_t pkt_safi;
+	iana_safi_t pkt_safi;
 
 	/* Convert AFI, SAFI to values for packet. */
 	bgp_map_afi_safi_int2iana(afi, safi, &pkt_afi, &pkt_safi);
@@ -1246,15 +1273,15 @@ static void bgp_open_capability_orf(struct stream *s, struct peer *peer,
 /* Fill in capability open option to the packet. */
 void bgp_open_capability(struct stream *s, struct peer *peer)
 {
-	u_char len;
+	uint8_t len;
 	unsigned long cp, capp, rcapp;
 	iana_afi_t pkt_afi;
 	afi_t afi;
-	safi_t safi, pkt_safi;
+	safi_t safi;
+	iana_safi_t pkt_safi;
 	as_t local_as;
-	u_int32_t restart_time;
-	u_char afi_safi_count = 0;
-	struct utsname names;
+	uint32_t restart_time;
+	uint8_t afi_safi_count = 0;
 	int adv_addpath_tx = 0;
 
 	/* Remember current pointer for Opt Parm Len. */
@@ -1269,47 +1296,51 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 		return;
 
 	/* MP capability for configured AFI, SAFI */
-	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++) {
-			if (peer->afc[afi][safi]) {
-				/* Convert AFI, SAFI to values for packet. */
-				bgp_map_afi_safi_int2iana(afi, safi, &pkt_afi,
-							  &pkt_safi);
+	FOREACH_AFI_SAFI (afi, safi) {
+		if (peer->afc[afi][safi]) {
+			/* Convert AFI, SAFI to values for packet. */
+			bgp_map_afi_safi_int2iana(afi, safi, &pkt_afi,
+						  &pkt_safi);
 
-				peer->afc_adv[afi][safi] = 1;
+			peer->afc_adv[afi][safi] = 1;
+			stream_putc(s, BGP_OPEN_OPT_CAP);
+			stream_putc(s, CAPABILITY_CODE_MP_LEN + 2);
+			stream_putc(s, CAPABILITY_CODE_MP);
+			stream_putc(s, CAPABILITY_CODE_MP_LEN);
+			stream_putw(s, pkt_afi);
+			stream_putc(s, 0);
+			stream_putc(s, pkt_safi);
+
+			/* Extended nexthop capability - currently
+			 * supporting RFC-5549 for
+			 * Link-Local peering only
+			 */
+			if (CHECK_FLAG(peer->flags, PEER_FLAG_CAPABILITY_ENHE)
+			    && peer->su.sa.sa_family == AF_INET6
+			    && IN6_IS_ADDR_LINKLOCAL(&peer->su.sin6.sin6_addr)
+			    && afi == AFI_IP
+			    && (safi == SAFI_UNICAST
+				|| safi == SAFI_LABELED_UNICAST)) {
+				/* RFC 5549 Extended Next Hop Encoding
+				 */
+				SET_FLAG(peer->cap, PEER_CAP_ENHE_ADV);
 				stream_putc(s, BGP_OPEN_OPT_CAP);
-				stream_putc(s, CAPABILITY_CODE_MP_LEN + 2);
-				stream_putc(s, CAPABILITY_CODE_MP);
-				stream_putc(s, CAPABILITY_CODE_MP_LEN);
+				stream_putc(s, CAPABILITY_CODE_ENHE_LEN + 2);
+				stream_putc(s, CAPABILITY_CODE_ENHE);
+				stream_putc(s, CAPABILITY_CODE_ENHE_LEN);
+
+				SET_FLAG(peer->af_cap[AFI_IP][safi],
+					 PEER_CAP_ENHE_AF_ADV);
 				stream_putw(s, pkt_afi);
-				stream_putc(s, 0);
-				stream_putc(s, pkt_safi);
+				stream_putw(s, pkt_safi);
+				stream_putw(s, afi_int2iana(AFI_IP6));
+
+				if (CHECK_FLAG(peer->af_cap[afi][safi],
+					       PEER_CAP_ENHE_AF_RCV))
+					SET_FLAG(peer->af_cap[afi][safi],
+						 PEER_CAP_ENHE_AF_NEGO);
 			}
 		}
-
-	/* Extended nexthop capability - currently supporting RFC-5549 for
-	 * Link-Local peering only
-	 */
-	if (CHECK_FLAG(peer->flags, PEER_FLAG_CAPABILITY_ENHE)
-	    && peer->su.sa.sa_family == AF_INET6
-	    && IN6_IS_ADDR_LINKLOCAL(&peer->su.sin6.sin6_addr)) {
-		/* RFC 5549 Extended Next Hop Encoding */
-		SET_FLAG(peer->cap, PEER_CAP_ENHE_ADV);
-		stream_putc(s, BGP_OPEN_OPT_CAP);
-		stream_putc(s, CAPABILITY_CODE_ENHE_LEN + 2);
-		stream_putc(s, CAPABILITY_CODE_ENHE);
-		stream_putc(s, CAPABILITY_CODE_ENHE_LEN);
-		/* Currently supporting for SAFI_UNICAST only */
-		SET_FLAG(peer->af_cap[AFI_IP][SAFI_UNICAST],
-			 PEER_CAP_ENHE_AF_ADV);
-		stream_putw(s, AFI_IP);
-		stream_putw(s, SAFI_UNICAST);
-		stream_putw(s, AFI_IP6);
-
-		if (CHECK_FLAG(peer->af_cap[AFI_IP][SAFI_UNICAST],
-			       PEER_CAP_ENHE_AF_RCV))
-			SET_FLAG(peer->af_cap[AFI_IP][SAFI_UNICAST],
-				 PEER_CAP_ENHE_AF_NEGO);
 	}
 
 	/* Route refresh. */
@@ -1336,21 +1367,20 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 	stream_putl(s, local_as);
 
 	/* AddPath */
-	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++)
-			if (peer->afc[afi][safi]) {
-				afi_safi_count++;
+	FOREACH_AFI_SAFI (afi, safi) {
+		if (peer->afc[afi][safi]) {
+			afi_safi_count++;
 
-				/* Only advertise addpath TX if a feature that
-				 * will use it is
-				 * configured */
-				if (CHECK_FLAG(peer->af_flags[afi][safi],
-					       PEER_FLAG_ADDPATH_TX_ALL_PATHS)
-				    || CHECK_FLAG(
-					       peer->af_flags[afi][safi],
-					       PEER_FLAG_ADDPATH_TX_BESTPATH_PER_AS))
-					adv_addpath_tx = 1;
-			}
+			/* Only advertise addpath TX if a feature that
+			 * will use it is
+			 * configured */
+			if (CHECK_FLAG(peer->af_flags[afi][safi],
+				       PEER_FLAG_ADDPATH_TX_ALL_PATHS)
+			    || CHECK_FLAG(peer->af_flags[afi][safi],
+					  PEER_FLAG_ADDPATH_TX_BESTPATH_PER_AS))
+				adv_addpath_tx = 1;
+		}
+	}
 
 	SET_FLAG(peer->cap, PEER_CAP_ADDPATH_ADV);
 	stream_putc(s, BGP_OPEN_OPT_CAP);
@@ -1358,46 +1388,43 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 	stream_putc(s, CAPABILITY_CODE_ADDPATH);
 	stream_putc(s, CAPABILITY_CODE_ADDPATH_LEN * afi_safi_count);
 
-	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++)
-			if (peer->afc[afi][safi]) {
-				/* Convert AFI, SAFI to values for packet. */
-				bgp_map_afi_safi_int2iana(afi, safi, &pkt_afi,
-							  &pkt_safi);
+	FOREACH_AFI_SAFI (afi, safi) {
+		if (peer->afc[afi][safi]) {
+			/* Convert AFI, SAFI to values for packet. */
+			bgp_map_afi_safi_int2iana(afi, safi, &pkt_afi,
+						  &pkt_safi);
 
-				stream_putw(s, pkt_afi);
-				stream_putc(s, pkt_safi);
+			stream_putw(s, pkt_afi);
+			stream_putc(s, pkt_safi);
 
-				if (adv_addpath_tx) {
-					stream_putc(s,
-						    BGP_ADDPATH_RX
-							    | BGP_ADDPATH_TX);
-					SET_FLAG(peer->af_cap[afi][safi],
-						 PEER_CAP_ADDPATH_AF_RX_ADV);
-					SET_FLAG(peer->af_cap[afi][safi],
-						 PEER_CAP_ADDPATH_AF_TX_ADV);
-				} else {
-					stream_putc(s, BGP_ADDPATH_RX);
-					SET_FLAG(peer->af_cap[afi][safi],
-						 PEER_CAP_ADDPATH_AF_RX_ADV);
-					UNSET_FLAG(peer->af_cap[afi][safi],
-						   PEER_CAP_ADDPATH_AF_TX_ADV);
-				}
+			if (adv_addpath_tx) {
+				stream_putc(s, BGP_ADDPATH_RX | BGP_ADDPATH_TX);
+				SET_FLAG(peer->af_cap[afi][safi],
+					 PEER_CAP_ADDPATH_AF_RX_ADV);
+				SET_FLAG(peer->af_cap[afi][safi],
+					 PEER_CAP_ADDPATH_AF_TX_ADV);
+			} else {
+				stream_putc(s, BGP_ADDPATH_RX);
+				SET_FLAG(peer->af_cap[afi][safi],
+					 PEER_CAP_ADDPATH_AF_RX_ADV);
+				UNSET_FLAG(peer->af_cap[afi][safi],
+					   PEER_CAP_ADDPATH_AF_TX_ADV);
 			}
+		}
+	}
 
 	/* ORF capability. */
-	for (afi = AFI_IP; afi < AFI_MAX; afi++)
-		for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++)
-			if (CHECK_FLAG(peer->af_flags[afi][safi],
-				       PEER_FLAG_ORF_PREFIX_SM)
-			    || CHECK_FLAG(peer->af_flags[afi][safi],
-					  PEER_FLAG_ORF_PREFIX_RM)) {
-				bgp_open_capability_orf(
-					s, peer, afi, safi,
-					CAPABILITY_CODE_ORF_OLD);
-				bgp_open_capability_orf(s, peer, afi, safi,
-							CAPABILITY_CODE_ORF);
-			}
+	FOREACH_AFI_SAFI (afi, safi) {
+		if (CHECK_FLAG(peer->af_flags[afi][safi],
+			       PEER_FLAG_ORF_PREFIX_SM)
+		    || CHECK_FLAG(peer->af_flags[afi][safi],
+				  PEER_FLAG_ORF_PREFIX_RM)) {
+			bgp_open_capability_orf(s, peer, afi, safi,
+						CAPABILITY_CODE_ORF_OLD);
+			bgp_open_capability_orf(s, peer, afi, safi,
+						CAPABILITY_CODE_ORF);
+		}
+	}
 
 	/* Dynamic capability. */
 	if (CHECK_FLAG(peer->flags, PEER_FLAG_DYNAMIC_CAPABILITY)) {
@@ -1413,8 +1440,7 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 	}
 
 	/* Hostname capability */
-	uname(&names);
-	if (names.nodename[0] != '\0') {
+	if (cmd_hostname_get()) {
 		SET_FLAG(peer->cap, PEER_CAP_HOSTNAME_ADV);
 		stream_putc(s, BGP_OPEN_OPT_CAP);
 		rcapp = stream_get_endp(s); /* Ptr to length placeholder */
@@ -1422,26 +1448,21 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 		stream_putc(s, CAPABILITY_CODE_FQDN);
 		capp = stream_get_endp(s);
 		stream_putc(s, 0); /* dummy len for now */
-		len = strlen(names.nodename);
+		len = strlen(cmd_hostname_get());
 		if (len > BGP_MAX_HOSTNAME)
 			len = BGP_MAX_HOSTNAME;
 
 		stream_putc(s, len);
-		stream_put(s, names.nodename, len);
-#ifdef HAVE_STRUCT_UTSNAME_DOMAINNAME
-		if ((names.domainname[0] != '\0')
-		    && (strcmp(names.domainname, "(none)") != 0)) {
-			len = strlen(names.domainname);
+		stream_put(s, cmd_hostname_get(), len);
+		if (cmd_domainname_get()) {
+			len = strlen(cmd_domainname_get());
 			if (len > BGP_MAX_HOSTNAME)
 				len = BGP_MAX_HOSTNAME;
 
 			stream_putc(s, len);
-			stream_put(s, names.domainname, len);
+			stream_put(s, cmd_domainname_get(), len);
 		} else
-#endif
-		{
 			stream_putc(s, 0); /* 0 length */
-		}
 
 		/* Set the lengths straight */
 		len = stream_get_endp(s) - rcapp - 1;
@@ -1450,14 +1471,10 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 		stream_putc_at(s, capp, len);
 
 		if (bgp_debug_neighbor_events(peer))
-#ifdef HAVE_STRUCT_UTSNAME_DOMAINNAME
 			zlog_debug(
 				"%s Sending hostname cap with hn = %s, dn = %s",
-				peer->host, names.nodename, names.domainname);
-#else
-			zlog_debug("%s Sending hostname cap with hn = %s",
-				   peer->host, names.nodename);
-#endif
+				peer->host, cmd_hostname_get(),
+				cmd_domainname_get());
 	}
 
 	/* Sending base graceful-restart capability irrespective of the config
@@ -1480,22 +1497,21 @@ void bgp_open_capability(struct stream *s, struct peer *peer)
 	   config
 	   is present */
 	if (bgp_flag_check(peer->bgp, BGP_FLAG_GRACEFUL_RESTART)) {
-		for (afi = AFI_IP; afi < AFI_MAX; afi++)
-			for (safi = SAFI_UNICAST; safi < SAFI_MAX; safi++)
-				if (peer->afc[afi][safi]) {
-					/* Convert AFI, SAFI to values for
-					 * packet. */
-					bgp_map_afi_safi_int2iana(
-						afi, safi, &pkt_afi, &pkt_safi);
-					stream_putw(s, pkt_afi);
-					stream_putc(s, pkt_safi);
-					if (bgp_flag_check(
-						    peer->bgp,
-						    BGP_FLAG_GR_PRESERVE_FWD))
-						stream_putc(s, RESTART_F_BIT);
-					else
-						stream_putc(s, 0);
-				}
+		FOREACH_AFI_SAFI (afi, safi) {
+			if (peer->afc[afi][safi]) {
+				/* Convert AFI, SAFI to values for
+				 * packet. */
+				bgp_map_afi_safi_int2iana(afi, safi, &pkt_afi,
+							  &pkt_safi);
+				stream_putw(s, pkt_afi);
+				stream_putc(s, pkt_safi);
+				if (bgp_flag_check(peer->bgp,
+						   BGP_FLAG_GR_PRESERVE_FWD))
+					stream_putc(s, RESTART_F_BIT);
+				else
+					stream_putc(s, 0);
+			}
+		}
 	}
 
 	/* Total Graceful restart capability Len. */
