@@ -19,10 +19,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -54,6 +53,7 @@
 #include "bgpd/bgp_updgrp.h"
 #include "bgpd/bgp_route.h"
 #include "bgpd/bgp_filter.h"
+#include "bgpd/bgp_io.h"
 
 /********************
  * PRIVATE FUNCTIONS
@@ -85,7 +85,8 @@ static void sync_init(struct update_subgroup *subgrp)
 	BGP_ADV_FIFO_INIT(&subgrp->sync->update);
 	BGP_ADV_FIFO_INIT(&subgrp->sync->withdraw);
 	BGP_ADV_FIFO_INIT(&subgrp->sync->withdraw_low);
-	subgrp->hash = hash_create(baa_hash_key, baa_hash_cmp);
+	subgrp->hash =
+		hash_create(baa_hash_key, baa_hash_cmp, "BGP SubGroup Hash");
 
 	/* We use a larger buffer for subgrp->work in the event that:
 	 * - We RX a BGP_UPDATE where the attributes alone are just
@@ -548,8 +549,7 @@ static int update_group_show_walkcb(struct update_group *updgrp, void *arg)
 		return CMD_SUCCESS;
 
 	if (ctx->subgrp_id) {
-		UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-		{
+		UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 			if (ctx->subgrp_id && (ctx->subgrp_id != subgrp->id))
 				continue;
 			else {
@@ -570,17 +570,16 @@ static int update_group_show_walkcb(struct update_group *updgrp, void *arg)
 
 	vty = ctx->vty;
 
-	vty_out(vty, "Update-group %" PRIu64 ":%s", updgrp->id, VTY_NEWLINE);
+	vty_out(vty, "Update-group %" PRIu64 ":\n", updgrp->id);
 	vty_out(vty, "  Created: %s", timestamp_string(updgrp->uptime));
 	filter = &updgrp->conf->filter[updgrp->afi][updgrp->safi];
 	if (filter->map[RMAP_OUT].name)
-		vty_out(vty, "  Outgoing route map: %s%s%s",
+		vty_out(vty, "  Outgoing route map: %s%s\n",
 			filter->map[RMAP_OUT].map ? "X" : "",
-			filter->map[RMAP_OUT].name, VTY_NEWLINE);
-	vty_out(vty, "  MRAI value (seconds): %d%s", updgrp->conf->v_routeadv,
-		VTY_NEWLINE);
+			filter->map[RMAP_OUT].name);
+	vty_out(vty, "  MRAI value (seconds): %d\n", updgrp->conf->v_routeadv);
 	if (updgrp->conf->change_local_as)
-		vty_out(vty, "  Local AS %u%s%s%s",
+		vty_out(vty, "  Local AS %u%s%s\n",
 			updgrp->conf->change_local_as,
 			CHECK_FLAG(updgrp->conf->flags,
 				   PEER_FLAG_LOCAL_AS_NO_PREPEND)
@@ -589,67 +588,54 @@ static int update_group_show_walkcb(struct update_group *updgrp, void *arg)
 			CHECK_FLAG(updgrp->conf->flags,
 				   PEER_FLAG_LOCAL_AS_REPLACE_AS)
 				? " replace-as"
-				: "",
-			VTY_NEWLINE);
+				: "");
 
-	UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-	{
+	UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 		if (ctx->subgrp_id && (ctx->subgrp_id != subgrp->id))
 			continue;
-		vty_out(vty, "%s", VTY_NEWLINE);
-		vty_out(vty, "  Update-subgroup %" PRIu64 ":%s", subgrp->id,
-			VTY_NEWLINE);
+		vty_out(vty, "\n");
+		vty_out(vty, "  Update-subgroup %" PRIu64 ":\n", subgrp->id);
 		vty_out(vty, "    Created: %s",
 			timestamp_string(subgrp->uptime));
 
 		if (subgrp->split_from.update_group_id
 		    || subgrp->split_from.subgroup_id) {
-			vty_out(vty, "    Split from group id: %" PRIu64 "%s",
-				subgrp->split_from.update_group_id,
-				VTY_NEWLINE);
+			vty_out(vty, "    Split from group id: %" PRIu64 "\n",
+				subgrp->split_from.update_group_id);
 			vty_out(vty,
-				"    Split from subgroup id: %" PRIu64 "%s",
-				subgrp->split_from.subgroup_id, VTY_NEWLINE);
+				"    Split from subgroup id: %" PRIu64 "\n",
+				subgrp->split_from.subgroup_id);
 		}
 
-		vty_out(vty, "    Join events: %u%s", subgrp->join_events,
-			VTY_NEWLINE);
-		vty_out(vty, "    Prune events: %u%s", subgrp->prune_events,
-			VTY_NEWLINE);
-		vty_out(vty, "    Merge events: %u%s", subgrp->merge_events,
-			VTY_NEWLINE);
-		vty_out(vty, "    Split events: %u%s", subgrp->split_events,
-			VTY_NEWLINE);
-		vty_out(vty, "    Update group switch events: %u%s",
-			subgrp->updgrp_switch_events, VTY_NEWLINE);
-		vty_out(vty, "    Peer refreshes combined: %u%s",
-			subgrp->peer_refreshes_combined, VTY_NEWLINE);
-		vty_out(vty, "    Merge checks triggered: %u%s",
-			subgrp->merge_checks_triggered, VTY_NEWLINE);
-		vty_out(vty, "    Version: %" PRIu64 "%s", subgrp->version,
-			VTY_NEWLINE);
-		vty_out(vty, "    Packet queue length: %d%s",
-			bpacket_queue_length(SUBGRP_PKTQ(subgrp)), VTY_NEWLINE);
-		vty_out(vty, "    Total packets enqueued: %u%s",
-			subgroup_total_packets_enqueued(subgrp), VTY_NEWLINE);
-		vty_out(vty, "    Packet queue high watermark: %d%s",
-			bpacket_queue_hwm_length(SUBGRP_PKTQ(subgrp)),
-			VTY_NEWLINE);
-		vty_out(vty, "    Adj-out list count: %u%s", subgrp->adj_count,
-			VTY_NEWLINE);
-		vty_out(vty, "    Advertise list: %s%s",
-			advertise_list_is_empty(subgrp) ? "empty" : "not empty",
-			VTY_NEWLINE);
-		vty_out(vty, "    Flags: %s%s",
+		vty_out(vty, "    Join events: %u\n", subgrp->join_events);
+		vty_out(vty, "    Prune events: %u\n", subgrp->prune_events);
+		vty_out(vty, "    Merge events: %u\n", subgrp->merge_events);
+		vty_out(vty, "    Split events: %u\n", subgrp->split_events);
+		vty_out(vty, "    Update group switch events: %u\n",
+			subgrp->updgrp_switch_events);
+		vty_out(vty, "    Peer refreshes combined: %u\n",
+			subgrp->peer_refreshes_combined);
+		vty_out(vty, "    Merge checks triggered: %u\n",
+			subgrp->merge_checks_triggered);
+		vty_out(vty, "    Version: %" PRIu64 "\n", subgrp->version);
+		vty_out(vty, "    Packet queue length: %d\n",
+			bpacket_queue_length(SUBGRP_PKTQ(subgrp)));
+		vty_out(vty, "    Total packets enqueued: %u\n",
+			subgroup_total_packets_enqueued(subgrp));
+		vty_out(vty, "    Packet queue high watermark: %d\n",
+			bpacket_queue_hwm_length(SUBGRP_PKTQ(subgrp)));
+		vty_out(vty, "    Adj-out list count: %u\n", subgrp->adj_count);
+		vty_out(vty, "    Advertise list: %s\n",
+			advertise_list_is_empty(subgrp) ? "empty"
+							: "not empty");
+		vty_out(vty, "    Flags: %s\n",
 			CHECK_FLAG(subgrp->flags, SUBGRP_FLAG_NEEDS_REFRESH)
 				? "R"
-				: "",
-			VTY_NEWLINE);
+				: "");
 		if (subgrp->peer_count > 0) {
-			vty_out(vty, "    Peers:%s", VTY_NEWLINE);
-			SUBGRP_FOREACH_PEER(subgrp, paf)
-			vty_out(vty, "      - %s%s", paf->peer->host,
-				VTY_NEWLINE);
+			vty_out(vty, "    Peers:\n");
+			SUBGRP_FOREACH_PEER (subgrp, paf)
+				vty_out(vty, "      - %s\n", paf->peer->host);
 		}
 	}
 	return UPDWALK_CONTINUE;
@@ -667,12 +653,11 @@ static int updgrp_show_packet_queue_walkcb(struct update_group *updgrp,
 	struct vty *vty;
 
 	vty = ctx->vty;
-	UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-	{
+	UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 		if (ctx->subgrp_id && (ctx->subgrp_id != subgrp->id))
 			continue;
-		vty_out(vty, "update group %" PRIu64 ", subgroup %" PRIu64 "%s",
-			updgrp->id, subgrp->id, VTY_NEWLINE);
+		vty_out(vty, "update group %" PRIu64 ", subgroup %" PRIu64 "\n",
+			updgrp->id, subgrp->id);
 		bpacket_queue_show_vty(SUBGRP_PKTQ(subgrp), vty);
 	}
 	return UPDWALK_CONTINUE;
@@ -971,8 +956,7 @@ static struct update_subgroup *update_subgroup_find(struct update_group *updgrp,
 	if (!peer_established(PAF_PEER(paf)))
 		return NULL;
 
-	UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-	{
+	UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 		if (subgrp->version != version
 		    || CHECK_FLAG(subgrp->sflags,
 				  SUBGRP_STATUS_DEFAULT_ORIGINATE))
@@ -1121,8 +1105,7 @@ int update_subgroup_check_merge(struct update_subgroup *subgrp,
 	/*
 	 * Look for a subgroup to merge into.
 	 */
-	UPDGRP_FOREACH_SUBGRP(subgrp->update_group, target)
-	{
+	UPDGRP_FOREACH_SUBGRP (subgrp->update_group, target) {
 		if (update_subgroup_can_merge_into(subgrp, target))
 			break;
 	}
@@ -1168,8 +1151,9 @@ int update_subgroup_trigger_merge_check(struct update_subgroup *subgrp,
 	if (!force && !update_subgroup_ready_for_merge(subgrp))
 		return 0;
 
-	subgrp->t_merge_check = thread_add_background(
-		bm->master, update_subgroup_merge_check_thread_cb, subgrp, 0);
+	subgrp->t_merge_check = NULL;
+	thread_add_timer_msec(bm->master, update_subgroup_merge_check_thread_cb,
+			      subgrp, 0, &subgrp->t_merge_check);
 
 	SUBGRP_INCR_STAT(subgrp, merge_checks_triggered);
 
@@ -1188,8 +1172,7 @@ static void update_subgroup_copy_adj_out(struct update_subgroup *source,
 {
 	struct bgp_adj_out *aout, *aout_copy;
 
-	SUBGRP_FOREACH_ADJ(source, aout)
-	{
+	SUBGRP_FOREACH_ADJ (source, aout) {
 		/*
 		 * Copy the adj out.
 		 */
@@ -1372,15 +1355,13 @@ static int updgrp_policy_update_walkcb(struct update_group *updgrp, void *arg)
 	 * refresh.
 	 */
 	if (ctx->policy_event_start_flag) {
-		UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-		{
+		UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 			update_subgroup_set_needs_refresh(subgrp, 1);
 		}
 		return UPDWALK_CONTINUE;
 	}
 
-	UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-	{
+	UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 		if (changed) {
 			if (bgp_debug_update(NULL, NULL, updgrp, 0))
 				zlog_debug(
@@ -1419,8 +1400,8 @@ static int update_group_periodic_merge_walkcb(struct update_group *updgrp,
 	struct update_subgroup *tmp_subgrp;
 	const char *reason = arg;
 
-	UPDGRP_FOREACH_SUBGRP_SAFE(updgrp, subgrp, tmp_subgrp)
-	update_subgroup_check_merge(subgrp, reason);
+	UPDGRP_FOREACH_SUBGRP_SAFE (updgrp, subgrp, tmp_subgrp)
+		update_subgroup_check_merge(subgrp, reason);
 	return UPDWALK_CONTINUE;
 }
 
@@ -1561,17 +1542,17 @@ void update_bgp_group_init(struct bgp *bgp)
 {
 	int afid;
 
-	AF_FOREACH(afid)
-	bgp->update_groups[afid] =
-		hash_create(updgrp_hash_key_make, updgrp_hash_cmp);
+	AF_FOREACH (afid)
+		bgp->update_groups[afid] =
+			hash_create(updgrp_hash_key_make, updgrp_hash_cmp,
+				    "BGP Update Group Hash");
 }
 
 void update_bgp_group_free(struct bgp *bgp)
 {
 	int afid;
 
-	AF_FOREACH(afid)
-	{
+	AF_FOREACH (afid) {
 		if (bgp->update_groups[afid]) {
 			hash_free(bgp->update_groups[afid]);
 			bgp->update_groups[afid] = NULL;
@@ -1597,28 +1578,27 @@ void update_group_show(struct bgp *bgp, afi_t afi, safi_t safi, struct vty *vty,
  */
 void update_group_show_stats(struct bgp *bgp, struct vty *vty)
 {
-	vty_out(vty, "Update groups created: %u%s",
-		bgp->update_group_stats.updgrps_created, VTY_NEWLINE);
-	vty_out(vty, "Update groups deleted: %u%s",
-		bgp->update_group_stats.updgrps_deleted, VTY_NEWLINE);
-	vty_out(vty, "Update subgroups created: %u%s",
-		bgp->update_group_stats.subgrps_created, VTY_NEWLINE);
-	vty_out(vty, "Update subgroups deleted: %u%s",
-		bgp->update_group_stats.subgrps_deleted, VTY_NEWLINE);
-	vty_out(vty, "Join events: %u%s", bgp->update_group_stats.join_events,
-		VTY_NEWLINE);
-	vty_out(vty, "Prune events: %u%s", bgp->update_group_stats.prune_events,
-		VTY_NEWLINE);
-	vty_out(vty, "Merge events: %u%s", bgp->update_group_stats.merge_events,
-		VTY_NEWLINE);
-	vty_out(vty, "Split events: %u%s", bgp->update_group_stats.split_events,
-		VTY_NEWLINE);
-	vty_out(vty, "Update group switch events: %u%s",
-		bgp->update_group_stats.updgrp_switch_events, VTY_NEWLINE);
-	vty_out(vty, "Peer route refreshes combined: %u%s",
-		bgp->update_group_stats.peer_refreshes_combined, VTY_NEWLINE);
-	vty_out(vty, "Merge checks triggered: %u%s",
-		bgp->update_group_stats.merge_checks_triggered, VTY_NEWLINE);
+	vty_out(vty, "Update groups created: %u\n",
+		bgp->update_group_stats.updgrps_created);
+	vty_out(vty, "Update groups deleted: %u\n",
+		bgp->update_group_stats.updgrps_deleted);
+	vty_out(vty, "Update subgroups created: %u\n",
+		bgp->update_group_stats.subgrps_created);
+	vty_out(vty, "Update subgroups deleted: %u\n",
+		bgp->update_group_stats.subgrps_deleted);
+	vty_out(vty, "Join events: %u\n", bgp->update_group_stats.join_events);
+	vty_out(vty, "Prune events: %u\n",
+		bgp->update_group_stats.prune_events);
+	vty_out(vty, "Merge events: %u\n",
+		bgp->update_group_stats.merge_events);
+	vty_out(vty, "Split events: %u\n",
+		bgp->update_group_stats.split_events);
+	vty_out(vty, "Update group switch events: %u\n",
+		bgp->update_group_stats.updgrp_switch_events);
+	vty_out(vty, "Peer route refreshes combined: %u\n",
+		bgp->update_group_stats.peer_refreshes_combined);
+	vty_out(vty, "Merge checks triggered: %u\n",
+		bgp->update_group_stats.merge_checks_triggered);
 }
 
 /*
@@ -1750,8 +1730,7 @@ void update_group_walk(struct bgp *bgp, updgrp_walkcb cb, void *ctx)
 	afi_t afi;
 	safi_t safi;
 
-	FOREACH_AFI_SAFI(afi, safi)
-	{
+	FOREACH_AFI_SAFI (afi, safi) {
 		update_group_af_walk(bgp, afi, safi, cb, ctx);
 	}
 }
@@ -1773,8 +1752,7 @@ update_group_default_originate_route_map_walkcb(struct update_group *updgrp,
 	afi_t afi;
 	safi_t safi;
 
-	UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-	{
+	UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 		peer = SUBGRP_PEER(subgrp);
 		afi = SUBGRP_AFI(subgrp);
 		safi = SUBGRP_SAFI(subgrp);
@@ -1834,8 +1812,7 @@ void peer_af_announce_route(struct peer_af *paf, int combine)
 		 */
 		all_pending = 1;
 
-		SUBGRP_FOREACH_PEER(subgrp, cur_paf)
-		{
+		SUBGRP_FOREACH_PEER (subgrp, cur_paf) {
 			if (cur_paf == paf)
 				continue;
 
@@ -1870,8 +1847,7 @@ void peer_af_announce_route(struct peer_af *paf, int combine)
 	 *
 	 * First stop refresh timers on all the other peers.
 	 */
-	SUBGRP_FOREACH_PEER(subgrp, cur_paf)
-	{
+	SUBGRP_FOREACH_PEER (subgrp, cur_paf) {
 		if (cur_paf == paf)
 			continue;
 
@@ -1894,18 +1870,17 @@ void subgroup_trigger_write(struct update_subgroup *subgrp)
 {
 	struct peer_af *paf;
 
-#if 0
-  if (bgp_debug_update(NULL, NULL, subgrp->update_group, 0))
-    zlog_debug("u%llu:s%llu scheduling write thread for peers",
-               subgrp->update_group->id, subgrp->id);
-#endif
-	SUBGRP_FOREACH_PEER(subgrp, paf)
-	{
-		if (paf->peer->status == Established) {
-			BGP_PEER_WRITE_ON(paf->peer->t_write, bgp_write,
-					  paf->peer->fd, paf->peer);
-		}
-	}
+	/*
+	 * For each peer in the subgroup, schedule a job to pull packets from
+	 * the subgroup output queue into their own output queue. This action
+	 * will trigger a write job on the I/O thread.
+	 */
+	SUBGRP_FOREACH_PEER (subgrp, paf)
+		if (paf->peer->status == Established)
+			thread_add_timer_msec(
+				bm->master, bgp_generate_updgrp_packets,
+				paf->peer, 0,
+				&paf->peer->t_generate_updgrp_packets);
 }
 
 int update_group_clear_update_dbg(struct update_group *updgrp, void *arg)
