@@ -21,10 +21,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -58,7 +57,7 @@
 
 static inline struct bgp_adj_out *adj_lookup(struct bgp_node *rn,
 					     struct update_subgroup *subgrp,
-					     u_int32_t addpath_tx_id)
+					     uint32_t addpath_tx_id)
 {
 	struct bgp_adj_out *adj;
 	struct peer *peer;
@@ -114,8 +113,15 @@ static int group_announce_route_walkcb(struct update_group *updgrp, void *arg)
 	peer = UPDGRP_PEER(updgrp);
 	addpath_capable = bgp_addpath_encode_tx(peer, afi, safi);
 
-	UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-	{
+	if (BGP_DEBUG(update, UPDATE_OUT)) {
+		char buf_prefix[PREFIX_STRLEN];
+		prefix2str(&ctx->rn->p, buf_prefix, sizeof(buf_prefix));
+		zlog_debug("%s: afi=%s, safi=%s, p=%s", __func__, afi2str(afi),
+			   safi2str(safi), buf_prefix);
+	}
+
+
+	UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 
 		/*
 		 * Skip the subgroups that have coalesce timer running. We will
@@ -201,7 +207,7 @@ static int group_announce_route_walkcb(struct update_group *updgrp, void *arg)
 }
 
 static void subgrp_show_adjq_vty(struct update_subgroup *subgrp,
-				 struct vty *vty, u_int8_t flags)
+				 struct vty *vty, uint8_t flags)
 {
 	struct bgp_table *table;
 	struct bgp_adj_out *adj;
@@ -225,19 +231,15 @@ static void subgrp_show_adjq_vty(struct update_subgroup *subgrp,
 				if (header1) {
 					vty_out(vty,
 						"BGP table version is %" PRIu64
-						", local router ID is %s%s",
+						", local router ID is %s\n",
 						table->version,
-						inet_ntoa(bgp->router_id),
-						VTY_NEWLINE);
-					vty_out(vty, BGP_SHOW_SCODE_HEADER,
-						VTY_NEWLINE, VTY_NEWLINE);
-					vty_out(vty, BGP_SHOW_OCODE_HEADER,
-						VTY_NEWLINE, VTY_NEWLINE);
+						inet_ntoa(bgp->router_id));
+					vty_out(vty, BGP_SHOW_SCODE_HEADER);
+					vty_out(vty, BGP_SHOW_OCODE_HEADER);
 					header1 = 0;
 				}
 				if (header2) {
-					vty_out(vty, BGP_SHOW_HEADER,
-						VTY_NEWLINE);
+					vty_out(vty, BGP_SHOW_HEADER);
 					header2 = 0;
 				}
 				if ((flags & UPDWALK_FLAGS_ADVQUEUE) && adj->adv
@@ -257,8 +259,7 @@ static void subgrp_show_adjq_vty(struct update_subgroup *subgrp,
 				}
 			}
 	if (output_count != 0)
-		vty_out(vty, "%sTotal number of prefixes %ld%s", VTY_NEWLINE,
-			output_count, VTY_NEWLINE);
+		vty_out(vty, "\nTotal number of prefixes %ld\n", output_count);
 }
 
 static int updgrp_show_adj_walkcb(struct update_group *updgrp, void *arg)
@@ -268,19 +269,18 @@ static int updgrp_show_adj_walkcb(struct update_group *updgrp, void *arg)
 	struct vty *vty;
 
 	vty = ctx->vty;
-	UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-	{
+	UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 		if (ctx->subgrp_id && (ctx->subgrp_id != subgrp->id))
 			continue;
-		vty_out(vty, "update group %" PRIu64 ", subgroup %" PRIu64 "%s",
-			updgrp->id, subgrp->id, VTY_NEWLINE);
+		vty_out(vty, "update group %" PRIu64 ", subgroup %" PRIu64 "\n",
+			updgrp->id, subgrp->id);
 		subgrp_show_adjq_vty(subgrp, vty, ctx->flags);
 	}
 	return UPDWALK_CONTINUE;
 }
 
 static void updgrp_show_adj(struct bgp *bgp, afi_t afi, safi_t safi,
-			    struct vty *vty, uint64_t id, u_int8_t flags)
+			    struct vty *vty, uint64_t id, uint8_t flags)
 {
 	struct updwalk_context ctx;
 	memset(&ctx, 0, sizeof(ctx));
@@ -317,8 +317,7 @@ static int subgroup_coalesce_timer(struct thread *thread)
 		struct peer_af *paf;
 		struct peer *peer;
 
-		SUBGRP_FOREACH_PEER(subgrp, paf)
-		{
+		SUBGRP_FOREACH_PEER (subgrp, paf) {
 			peer = PAF_PEER(paf);
 			BGP_TIMER_OFF(peer->t_routeadv);
 			BGP_TIMER_ON(peer->t_routeadv, bgp_routeadv_timer, 0);
@@ -332,8 +331,7 @@ static int update_group_announce_walkcb(struct update_group *updgrp, void *arg)
 {
 	struct update_subgroup *subgrp;
 
-	UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-	{
+	UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 		subgroup_announce_all(subgrp);
 	}
 
@@ -354,8 +352,7 @@ static int update_group_announce_rrc_walkcb(struct update_group *updgrp,
 
 	/* Only announce if this is a group of route-reflector-clients */
 	if (CHECK_FLAG(peer->af_flags[afi][safi], PEER_FLAG_REFLECTOR_CLIENT)) {
-		UPDGRP_FOREACH_SUBGRP(updgrp, subgrp)
-		{
+		UPDGRP_FOREACH_SUBGRP (updgrp, subgrp) {
 			subgroup_announce_all(subgrp);
 		}
 	}
@@ -373,7 +370,7 @@ static int update_group_announce_rrc_walkcb(struct update_group *updgrp,
  */
 struct bgp_adj_out *bgp_adj_out_alloc(struct update_subgroup *subgrp,
 				      struct bgp_node *rn,
-				      u_int32_t addpath_tx_id)
+				      uint32_t addpath_tx_id)
 {
 	struct bgp_adj_out *adj;
 
@@ -474,8 +471,7 @@ void bgp_adj_out_set_subgroup(struct bgp_node *rn,
 	if (BGP_ADV_FIFO_EMPTY(&subgrp->sync->update)) {
 		struct peer_af *paf;
 
-		SUBGRP_FOREACH_PEER(subgrp, paf)
-		{
+		SUBGRP_FOREACH_PEER (subgrp, paf) {
 			bgp_adjust_routeadv(PAF_PEER(paf));
 		}
 	}
@@ -491,11 +487,11 @@ void bgp_adj_out_set_subgroup(struct bgp_node *rn,
  */
 void bgp_adj_out_unset_subgroup(struct bgp_node *rn,
 				struct update_subgroup *subgrp, char withdraw,
-				u_int32_t addpath_tx_id)
+				uint32_t addpath_tx_id)
 {
 	struct bgp_adj_out *adj;
 	struct bgp_advertise *adv;
-	char trigger_write;
+	bool trigger_write;
 
 	if (DISABLE_BGP_ANNOUNCE)
 		return;
@@ -514,17 +510,13 @@ void bgp_adj_out_unset_subgroup(struct bgp_node *rn,
 			adv->adj = adj;
 
 			/* Note if we need to trigger a packet write */
-			if (BGP_ADV_FIFO_EMPTY(&subgrp->sync->withdraw))
-				trigger_write = 1;
-			else
-				trigger_write = 0;
+			trigger_write =
+				BGP_ADV_FIFO_EMPTY(&subgrp->sync->withdraw);
 
 			/* Add to synchronization entry for withdraw
 			 * announcement.  */
 			BGP_ADV_FIFO_ADD(&subgrp->sync->withdraw, &adv->fifo);
 
-			/* Schedule packet write, if FIFO is getting its first
-			 * entry. */
 			if (trigger_write)
 				subgroup_trigger_write(subgrp);
 		} else {
@@ -562,8 +554,7 @@ void subgroup_clear_table(struct update_subgroup *subgrp)
 {
 	struct bgp_adj_out *aout, *taout;
 
-	SUBGRP_FOREACH_ADJ_SAFE(subgrp, aout, taout)
-	{
+	SUBGRP_FOREACH_ADJ_SAFE (subgrp, aout, taout) {
 		struct bgp_node *rn = aout->rn;
 		bgp_adj_out_remove_subgroup(rn, aout, subgrp);
 		bgp_unlock_node(rn);
@@ -579,7 +570,6 @@ void subgroup_announce_table(struct update_subgroup *subgrp,
 	struct bgp_node *rn;
 	struct bgp_info *ri;
 	struct attr attr;
-	struct attr_extra extra;
 	struct peer *peer;
 	afi_t afi;
 	safi_t safi;
@@ -590,6 +580,9 @@ void subgroup_announce_table(struct update_subgroup *subgrp,
 	safi = SUBGRP_SAFI(subgrp);
 	addpath_capable = bgp_addpath_encode_tx(peer, afi, safi);
 
+	if (safi == SAFI_LABELED_UNICAST)
+		safi = SAFI_UNICAST;
+
 	if (!table)
 		table = peer->bgp->rib[afi][safi];
 
@@ -598,17 +591,14 @@ void subgroup_announce_table(struct update_subgroup *subgrp,
 			  PEER_FLAG_DEFAULT_ORIGINATE))
 		subgroup_default_originate(subgrp, 0);
 
-	/* It's initialized in bgp_announce_check() */
-	attr.extra = &extra;
-
 	for (rn = bgp_table_top(table); rn; rn = bgp_route_next(rn))
 		for (ri = rn->info; ri; ri = ri->next)
 
 			if (CHECK_FLAG(ri->flags, BGP_INFO_SELECTED)
 			    || (addpath_capable
 				&& bgp_addpath_tx_path(peer, afi, safi, ri))) {
-				if (subgroup_announce_check(ri, subgrp, &rn->p,
-							    &attr))
+				if (subgroup_announce_check(rn, ri, subgrp,
+							    &rn->p, &attr))
 					bgp_adj_out_set_subgroup(rn, subgrp,
 								 &attr, ri);
 				else
@@ -698,23 +688,22 @@ void subgroup_default_originate(struct update_subgroup *subgrp, int withdraw)
 
 	bgp_attr_default_set(&attr, BGP_ORIGIN_IGP);
 	aspath = attr.aspath;
+
 	attr.local_pref = bgp->default_local_pref;
 
-	if (afi == AFI_IP)
-		str2prefix("0.0.0.0/0", &p);
-	else if (afi == AFI_IP6) {
-		struct attr_extra *ae = attr.extra;
-
-		str2prefix("::/0", &p);
+	memset(&p, 0, sizeof(p));
+	p.family = afi2family(afi);
+	p.prefixlen = 0;
 
+	if ((afi == AFI_IP6) || peer_cap_enhe(peer, afi, safi)) {
 		/* IPv6 global nexthop must be included. */
-		ae->mp_nexthop_len = BGP_ATTR_NHLEN_IPV6_GLOBAL;
+		attr.mp_nexthop_len = BGP_ATTR_NHLEN_IPV6_GLOBAL;
 
 		/* If the peer is on shared nextwork and we have link-local
 		   nexthop set it. */
 		if (peer->shared_network
 		    && !IN6_IS_ADDR_UNSPECIFIED(&peer->nexthop.v6_local))
-			ae->mp_nexthop_len = BGP_ATTR_NHLEN_IPV6_GLOBAL_AND_LL;
+			attr.mp_nexthop_len = BGP_ATTR_NHLEN_IPV6_GLOBAL_AND_LL;
 	}
 
 	if (peer->default_rmap[afi][safi].name) {
@@ -723,12 +712,10 @@ void subgroup_default_originate(struct update_subgroup *subgrp, int withdraw)
 		     rn = bgp_route_next(rn)) {
 			for (ri = rn->info; ri; ri = ri->next) {
 				struct attr dummy_attr;
-				struct attr_extra dummy_extra;
 				struct bgp_info info;
 
 				/* Provide dummy so the route-map can't modify
 				 * the attributes */
-				dummy_attr.extra = &dummy_extra;
 				bgp_attr_dup(&dummy_attr, ri->attr);
 				info.peer = ri->peer;
 				info.attr = &dummy_attr;
@@ -760,6 +747,11 @@ void subgroup_default_originate(struct update_subgroup *subgrp, int withdraw)
 	} else {
 		if (!CHECK_FLAG(subgrp->sflags,
 				SUBGRP_STATUS_DEFAULT_ORIGINATE)) {
+
+			if (bgp_flag_check(bgp, BGP_FLAG_GRACEFUL_SHUTDOWN)) {
+				bgp_attr_add_gshut_community(&attr);
+			}
+
 			SET_FLAG(subgrp->sflags,
 				 SUBGRP_STATUS_DEFAULT_ORIGINATE);
 			subgroup_default_update_packet(subgrp, &attr, from);
@@ -771,10 +763,9 @@ void subgroup_default_originate(struct update_subgroup *subgrp, int withdraw)
 			 * clear adj_out for the 0.0.0.0/0 prefix in the BGP
 			 * table.
 			 */
-			if (afi == AFI_IP)
-				str2prefix("0.0.0.0/0", &p);
-			else
-				str2prefix("::/0", &p);
+			memset(&p, 0, sizeof(p));
+			p.family = afi2family(afi);
+			p.prefixlen = 0;
 
 			rn = bgp_afi_node_get(bgp->rib[afi][safi], afi, safi,
 					      &p, NULL);
@@ -784,7 +775,6 @@ void subgroup_default_originate(struct update_subgroup *subgrp, int withdraw)
 		}
 	}
 
-	bgp_attr_extra_free(&attr);
 	aspath_unintern(&aspath);
 }
 
@@ -817,9 +807,9 @@ void subgroup_announce_all(struct update_subgroup *subgrp)
 	 * We should wait for the coalesce timer. Arm the timer if not done.
 	 */
 	if (!subgrp->t_coalesce) {
-		THREAD_TIMER_MSEC_ON(bm->master, subgrp->t_coalesce,
-				     subgroup_coalesce_timer, subgrp,
-				     subgrp->v_coalesce);
+		thread_add_timer_msec(bm->master, subgroup_coalesce_timer,
+				      subgrp, subgrp->v_coalesce,
+				      &subgrp->t_coalesce);
 	}
 }
 
