@@ -19,10 +19,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -43,6 +42,7 @@
 #include "workqueue.h"
 #include "hash.h"
 #include "queue.h"
+#include "mpls.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_debug.h"
@@ -54,6 +54,7 @@
 #include "bgpd/bgp_nexthop.h"
 #include "bgpd/bgp_nht.h"
 #include "bgpd/bgp_mplsvpn.h"
+#include "bgpd/bgp_label.h"
 
 /********************
  * PRIVATE FUNCTIONS
@@ -370,13 +371,11 @@ void bpacket_queue_show_vty(struct bpacket_queue *q, struct vty *vty)
 
 	pkt = bpacket_queue_first(q);
 	while (pkt) {
-		vty_out(vty, "  Packet %p ver %u buffer %p%s", pkt, pkt->ver,
-			pkt->buffer, VTY_NEWLINE);
+		vty_out(vty, "  Packet %p ver %u buffer %p\n", pkt, pkt->ver,
+			pkt->buffer);
 
-		LIST_FOREACH(paf, &(pkt->peers), pkt_train)
-		{
-			vty_out(vty, "      - %s%s", paf->peer->host,
-				VTY_NEWLINE);
+		LIST_FOREACH (paf, &(pkt->peers), pkt_train) {
+			vty_out(vty, "      - %s\n", paf->peer->host);
 		}
 		pkt = bpacket_next(pkt);
 	}
@@ -397,15 +396,14 @@ struct stream *bpacket_reformat_for_peer(struct bpacket *pkt,
 
 	vec = &pkt->arr.entries[BGP_ATTR_VEC_NH];
 	if (CHECK_FLAG(vec->flags, BPKT_ATTRVEC_FLAGS_UPDATED)) {
-		u_int8_t nhlen;
+		uint8_t nhlen;
 		afi_t nhafi = AFI_MAX; /* NH AFI is based on nhlen! */
 		int route_map_sets_nh;
 		nhlen = stream_getc_from(s, vec->offset);
-		if (paf->afi == AFI_IP || paf->afi == AFI_IP6) {
+		if (peer_cap_enhe(peer, paf->afi, paf->safi))
+			nhafi = AFI_IP6;
+		else
 			nhafi = BGP_NEXTHOP_AFI_FROM_NHLEN(nhlen);
-			if (peer_cap_enhe(peer))
-				nhafi = AFI_IP6;
-		}
 
 		if (nhafi == AFI_IP) {
 			struct in_addr v4nh, *mod_v4nh;
@@ -431,6 +429,7 @@ struct stream *bpacket_reformat_for_peer(struct bpacket *pkt,
 				zlog_warn(
 					"%s: %s: invalid MP nexthop length (AFI IP): %u",
 					__func__, peer->host, nhlen);
+				stream_free(s);
 				return NULL;
 			}
 
@@ -468,6 +467,7 @@ struct stream *bpacket_reformat_for_peer(struct bpacket *pkt,
 				nh_modified = 1;
 			} else if (
 				peer->sort == BGP_PEER_EBGP
+				&& paf->safi != SAFI_EVPN
 				&& (bgp_multiaccess_check_v4(v4nh, peer) == 0)
 				&& !CHECK_FLAG(
 					   vec->flags,
@@ -535,6 +535,7 @@ struct stream *bpacket_reformat_for_peer(struct bpacket *pkt,
 				zlog_warn(
 					"%s: %s: invalid MP nexthop length (AFI IP6): %u",
 					__func__, peer->host, nhlen);
+				stream_free(s);
 				return NULL;
 			}
 
@@ -606,10 +607,32 @@ struct stream *bpacket_reformat_for_peer(struct bpacket *pkt,
 							  buf, BUFSIZ),
 						(nhlen == 24 ? " and RD" : ""));
 			}
+		} else if (paf->afi == AFI_L2VPN) {
+			struct in_addr v4nh, *mod_v4nh;
+			int nh_modified = 0;
+
+			stream_get_from(&v4nh, s, vec->offset + 1, 4);
+			mod_v4nh = &v4nh;
+
+			/* No route-map changes allowed for EVPN nexthops. */
+			if (!v4nh.s_addr) {
+				mod_v4nh = &peer->nexthop.v4;
+				nh_modified = 1;
+			}
+
+			if (nh_modified)
+				stream_put_in_addr_at(s, vec->offset + 1,
+						      mod_v4nh);
+
+			if (bgp_debug_update(peer, NULL, NULL, 0))
+				zlog_debug("u%" PRIu64 ":s%" PRIu64
+					   " %s send UPDATE w/ nexthop %s",
+					   PAF_SUBGRP(paf)->update_group->id,
+					   PAF_SUBGRP(paf)->id, peer->host,
+					   inet_ntoa(*mod_v4nh));
 		}
 	}
 
-	bgp_packet_add(peer, s);
 	return s;
 }
 
@@ -675,8 +698,11 @@ struct bpacket *subgroup_update_packet(struct update_subgroup *subgrp)
 	int send_attr_printed = 0;
 	int num_pfx = 0;
 	int addpath_encode = 0;
-	u_int32_t addpath_tx_id = 0;
+	int addpath_overhead = 0;
+	uint32_t addpath_tx_id = 0;
 	struct prefix_rd *prd = NULL;
+	mpls_label_t label = MPLS_INVALID_LABEL, *label_pnt = NULL;
+	uint32_t num_labels = 0;
 
 	if (!subgrp)
 		return NULL;
@@ -684,7 +710,6 @@ struct bpacket *subgroup_update_packet(struct update_subgroup *subgrp)
 	if (bpacket_queue_is_full(SUBGRP_INST(subgrp), SUBGRP_PKTQ(subgrp)))
 		return NULL;
 
-
 	peer = SUBGRP_PEER(subgrp);
 	afi = SUBGRP_AFI(subgrp);
 	safi = SUBGRP_SAFI(subgrp);
@@ -696,6 +721,7 @@ struct bpacket *subgroup_update_packet(struct update_subgroup *subgrp)
 	bpacket_attr_vec_arr_reset(&vecarr);
 
 	addpath_encode = bgp_addpath_encode_tx(peer, afi, safi);
+	addpath_overhead = addpath_encode ? BGP_ADDPATH_ID_LEN : 0;
 
 	adv = BGP_ADV_FIFO_HEAD(&subgrp->sync->update);
 	while (adv) {
@@ -707,8 +733,9 @@ struct bpacket *subgroup_update_packet(struct update_subgroup *subgrp)
 
 		space_remaining = STREAM_CONCAT_REMAIN(s, snlri, STREAM_SIZE(s))
 				  - BGP_MAX_PACKET_SIZE_OVERFLOW;
-		space_needed = BGP_NLRI_LENGTH + bgp_packet_mpattr_prefix_size(
-							 afi, safi, &rn->p);
+		space_needed =
+			BGP_NLRI_LENGTH + addpath_overhead
+			+ bgp_packet_mpattr_prefix_size(afi, safi, &rn->p);
 
 		/* When remaining space can't include NLRI and it's length.  */
 		if (space_remaining < space_needed)
@@ -747,14 +774,14 @@ struct bpacket *subgroup_update_packet(struct update_subgroup *subgrp)
 			 * attr. */
 			total_attr_len = bgp_packet_attribute(
 				NULL, peer, s, adv->baa->attr, &vecarr, NULL,
-				afi, safi, from, NULL, NULL, 0, 0);
+				afi, safi, from, NULL, NULL, 0, 0, 0);
 
 			space_remaining =
 				STREAM_CONCAT_REMAIN(s, snlri, STREAM_SIZE(s))
 				- BGP_MAX_PACKET_SIZE_OVERFLOW;
-			space_needed =
-				BGP_NLRI_LENGTH + bgp_packet_mpattr_prefix_size(
-							  afi, safi, &rn->p);
+			space_needed = BGP_NLRI_LENGTH + addpath_overhead
+				       + bgp_packet_mpattr_prefix_size(
+						 afi, safi, &rn->p);
 
 			/* If the attributes alone do not leave any room for
 			 * NLRI then
@@ -776,35 +803,39 @@ struct bpacket *subgroup_update_packet(struct update_subgroup *subgrp)
 			    || BGP_DEBUG(update, UPDATE_PREFIX)) {
 				memset(send_attr_str, 0, BUFSIZ);
 				send_attr_printed = 0;
-				bgp_dump_attr(peer, adv->baa->attr,
-					      send_attr_str, BUFSIZ);
+				bgp_dump_attr(adv->baa->attr, send_attr_str,
+					      BUFSIZ);
 			}
 		}
 
 		if ((afi == AFI_IP && safi == SAFI_UNICAST)
-		    && !peer_cap_enhe(peer))
+		    && !peer_cap_enhe(peer, afi, safi))
 			stream_put_prefix_addpath(s, &rn->p, addpath_encode,
 						  addpath_tx_id);
 		else {
 			/* Encode the prefix in MP_REACH_NLRI attribute */
-			u_char *tag = NULL;
-
 			if (rn->prn)
 				prd = (struct prefix_rd *)&rn->prn->p;
-			if (binfo && binfo->extra)
-				tag = binfo->extra->tag;
+
+			if (safi == SAFI_LABELED_UNICAST) {
+				label = bgp_adv_label(rn, binfo, peer, afi,
+						      safi);
+				label_pnt = &label;
+				num_labels = 1;
+			} else if (binfo && binfo->extra) {
+				label_pnt = &binfo->extra->label[0];
+				num_labels = binfo->extra->num_labels;
+			}
 
 			if (stream_empty(snlri))
 				mpattrlen_pos = bgp_packet_mpattr_start(
-					snlri, afi, safi,
-					(peer_cap_enhe(peer)
-						 ? AFI_IP6
-						 : AFI_MAX), /* get from NH */
-					&vecarr, adv->baa->attr);
+					snlri, peer, afi, safi, &vecarr,
+					adv->baa->attr);
 
 			bgp_packet_mpattr_prefix(snlri, afi, safi, &rn->p, prd,
-						 tag, addpath_encode,
-						 addpath_tx_id, adv->baa->attr);
+						 label_pnt, num_labels,
+						 addpath_encode, addpath_tx_id,
+						 adv->baa->attr);
 		}
 
 		num_pfx++;
@@ -817,15 +848,29 @@ struct bpacket *subgroup_update_packet(struct update_subgroup *subgrp)
 					   " send UPDATE w/ attr: %s",
 					   subgrp->update_group->id, subgrp->id,
 					   send_attr_str);
+				if (!stream_empty(snlri)) {
+					iana_afi_t pkt_afi;
+					iana_safi_t pkt_safi;
+
+					pkt_afi = afi_int2iana(afi);
+					pkt_safi = safi_int2iana(safi);
+					zlog_debug(
+						"u%" PRIu64 ":s%" PRIu64
+						" send MP_REACH for afi/safi %d/%d",
+						subgrp->update_group->id,
+						subgrp->id, pkt_afi, pkt_safi);
+				}
+
 				send_attr_printed = 1;
 			}
 
+			bgp_debug_rdpfxpath2str(afi, safi, prd, &rn->p,
+						label_pnt, num_labels,
+						addpath_encode, addpath_tx_id,
+						pfx_buf, sizeof(pfx_buf));
 			zlog_debug("u%" PRIu64 ":s%" PRIu64 " send UPDATE %s",
 				   subgrp->update_group->id, subgrp->id,
-				   bgp_debug_rdpfxpath2str(
-					   prd, &rn->p, addpath_encode,
-					   addpath_tx_id, pfx_buf,
-					   sizeof(pfx_buf)));
+				   pfx_buf);
 		}
 
 		/* Synchnorize attribute.  */
@@ -856,7 +901,7 @@ struct bpacket *subgroup_update_packet(struct update_subgroup *subgrp)
 		bgp_packet_set_size(packet);
 		if (bgp_debug_update(NULL, NULL, subgrp->update_group, 0))
 			zlog_debug("u%" PRIu64 ":s%" PRIu64
-				   " UPDATE len %zd numpfx %d",
+				   " send UPDATE len %zd numpfx %d",
 				   subgrp->update_group->id, subgrp->id,
 				   (stream_get_endp(packet)
 				    - stream_get_getp(packet)),
@@ -892,14 +937,15 @@ struct bpacket *subgroup_withdraw_packet(struct update_subgroup *subgrp)
 	size_t mp_start = 0;
 	size_t attrlen_pos = 0;
 	size_t mplen_pos = 0;
-	u_char first_time = 1;
+	uint8_t first_time = 1;
 	afi_t afi;
 	safi_t safi;
 	int space_remaining = 0;
 	int space_needed = 0;
 	int num_pfx = 0;
 	int addpath_encode = 0;
-	u_int32_t addpath_tx_id = 0;
+	int addpath_overhead = 0;
+	uint32_t addpath_tx_id = 0;
 	struct prefix_rd *prd = NULL;
 
 
@@ -915,6 +961,7 @@ struct bpacket *subgroup_withdraw_packet(struct update_subgroup *subgrp)
 	s = subgrp->work;
 	stream_reset(s);
 	addpath_encode = bgp_addpath_encode_tx(peer, afi, safi);
+	addpath_overhead = addpath_encode ? BGP_ADDPATH_ID_LEN : 0;
 
 	while ((adv = BGP_ADV_FIFO_HEAD(&subgrp->sync->withdraw)) != NULL) {
 		assert(adv->rn);
@@ -923,9 +970,9 @@ struct bpacket *subgroup_withdraw_packet(struct update_subgroup *subgrp)
 		addpath_tx_id = adj->addpath_tx_id;
 
 		space_remaining =
-			STREAM_REMAIN(s) - BGP_MAX_PACKET_SIZE_OVERFLOW;
+			STREAM_WRITEABLE(s) - BGP_MAX_PACKET_SIZE_OVERFLOW;
 		space_needed =
-			BGP_NLRI_LENGTH + BGP_TOTAL_ATTR_LEN
+			BGP_NLRI_LENGTH + addpath_overhead + BGP_TOTAL_ATTR_LEN
 			+ bgp_packet_mpattr_prefix_size(afi, safi, &rn->p);
 
 		if (space_remaining < space_needed)
@@ -938,7 +985,7 @@ struct bpacket *subgroup_withdraw_packet(struct update_subgroup *subgrp)
 			first_time = 0;
 
 		if (afi == AFI_IP && safi == SAFI_UNICAST
-		    && !peer_cap_enhe(peer))
+		    && !peer_cap_enhe(peer, afi, safi))
 			stream_put_prefix_addpath(s, &rn->p, addpath_encode,
 						  addpath_tx_id);
 		else {
@@ -947,6 +994,12 @@ struct bpacket *subgroup_withdraw_packet(struct update_subgroup *subgrp)
 
 			/* If first time, format the MP_UNREACH header */
 			if (first_time) {
+				iana_afi_t pkt_afi;
+				iana_safi_t pkt_safi;
+
+				pkt_afi = afi_int2iana(afi);
+				pkt_safi = safi_int2iana(safi);
+
 				attrlen_pos = stream_get_endp(s);
 				/* total attr length = 0 for now. reevaluate
 				 * later */
@@ -954,10 +1007,17 @@ struct bpacket *subgroup_withdraw_packet(struct update_subgroup *subgrp)
 				mp_start = stream_get_endp(s);
 				mplen_pos = bgp_packet_mpunreach_start(s, afi,
 								       safi);
+				if (bgp_debug_update(NULL, NULL,
+						     subgrp->update_group, 0))
+					zlog_debug(
+						"u%" PRIu64 ":s%" PRIu64
+						" send MP_UNREACH for afi/safi %d/%d",
+						subgrp->update_group->id,
+						subgrp->id, pkt_afi, pkt_safi);
 			}
 
 			bgp_packet_mpunreach_prefix(s, &rn->p, afi, safi, prd,
-						    NULL, addpath_encode,
+						    NULL, 0, addpath_encode,
 						    addpath_tx_id, NULL);
 		}
 
@@ -966,13 +1026,13 @@ struct bpacket *subgroup_withdraw_packet(struct update_subgroup *subgrp)
 		if (bgp_debug_update(NULL, &rn->p, subgrp->update_group, 0)) {
 			char pfx_buf[BGP_PRD_PATH_STRLEN];
 
+			bgp_debug_rdpfxpath2str(afi, safi, prd, &rn->p, NULL, 0,
+						addpath_encode, addpath_tx_id,
+						pfx_buf, sizeof(pfx_buf));
 			zlog_debug("u%" PRIu64 ":s%" PRIu64
 				   " send UPDATE %s -- unreachable",
 				   subgrp->update_group->id, subgrp->id,
-				   bgp_debug_rdpfxpath2str(
-					   prd, &rn->p, addpath_encode,
-					   addpath_tx_id, pfx_buf,
-					   sizeof(pfx_buf)));
+				   pfx_buf);
 		}
 
 		subgrp->scount--;
@@ -983,7 +1043,7 @@ struct bpacket *subgroup_withdraw_packet(struct update_subgroup *subgrp)
 
 	if (!stream_empty(s)) {
 		if (afi == AFI_IP && safi == SAFI_UNICAST
-		    && !peer_cap_enhe(peer)) {
+		    && !peer_cap_enhe(peer, afi, safi)) {
 			unfeasible_len = stream_get_endp(s) - BGP_HEADER_SIZE
 					 - BGP_UNFEASIBLE_LEN;
 			stream_putw_at(s, BGP_HEADER_SIZE, unfeasible_len);
@@ -999,7 +1059,7 @@ struct bpacket *subgroup_withdraw_packet(struct update_subgroup *subgrp)
 		bgp_packet_set_size(s);
 		if (bgp_debug_update(NULL, NULL, subgrp->update_group, 0))
 			zlog_debug("u%" PRIu64 ":s%" PRIu64
-				   " UPDATE (withdraw) len %zd numpfx %d",
+				   " send UPDATE (withdraw) len %zd numpfx %d",
 				   subgrp->update_group->id, subgrp->id,
 				   (stream_get_endp(s) - stream_get_getp(s)),
 				   num_pfx);
@@ -1037,10 +1097,9 @@ void subgroup_default_update_packet(struct update_subgroup *subgrp,
 	bpacket_attr_vec_arr_reset(&vecarr);
 	addpath_encode = bgp_addpath_encode_tx(peer, afi, safi);
 
-	if (afi == AFI_IP)
-		str2prefix("0.0.0.0/0", &p);
-	else
-		str2prefix("::/0", &p);
+	memset(&p, 0, sizeof(p));
+	p.family = afi2family(afi);
+	p.prefixlen = 0;
 
 	/* Logging the attribute. */
 	if (bgp_debug_update(NULL, &p, subgrp->update_group, 0)) {
@@ -1048,14 +1107,14 @@ void subgroup_default_update_packet(struct update_subgroup *subgrp,
 		char buf[PREFIX_STRLEN];
 		/* ' with addpath ID '          17
 		 * max strlen of uint32       + 10
-		 * +/- (in case of idiocy)    +  1
+		 * +/- (just in case)         +  1
 		 * null terminator            +  1
 		 * ============================ 29 */
 		char tx_id_buf[30];
 
 		attrstr[0] = '\0';
 
-		bgp_dump_attr(peer, attr, attrstr, BUFSIZ);
+		bgp_dump_attr(attr, attrstr, BUFSIZ);
 
 		if (addpath_encode)
 			snprintf(tx_id_buf, sizeof(tx_id_buf),
@@ -1081,13 +1140,14 @@ void subgroup_default_update_packet(struct update_subgroup *subgrp,
 	stream_putw(s, 0);
 	total_attr_len = bgp_packet_attribute(
 		NULL, peer, s, attr, &vecarr, &p, afi, safi, from, NULL, NULL,
-		addpath_encode, BGP_ADDPATH_TX_ID_FOR_DEFAULT_ORIGINATE);
+		0, addpath_encode, BGP_ADDPATH_TX_ID_FOR_DEFAULT_ORIGINATE);
 
 	/* Set Total Path Attribute Length. */
 	stream_putw_at(s, pos, total_attr_len);
 
 	/* NLRI set. */
-	if (p.family == AF_INET && safi == SAFI_UNICAST && !peer_cap_enhe(peer))
+	if (p.family == AF_INET && safi == SAFI_UNICAST
+	    && !peer_cap_enhe(peer, afi, safi))
 		stream_put_prefix_addpath(
 			s, &p, addpath_encode,
 			BGP_ADDPATH_TX_ID_FOR_DEFAULT_ORIGINATE);
@@ -1122,16 +1182,15 @@ void subgroup_default_withdraw_packet(struct update_subgroup *subgrp)
 	safi = SUBGRP_SAFI(subgrp);
 	addpath_encode = bgp_addpath_encode_tx(peer, afi, safi);
 
-	if (afi == AFI_IP)
-		str2prefix("0.0.0.0/0", &p);
-	else
-		str2prefix("::/0", &p);
+	memset(&p, 0, sizeof(p));
+	p.family = afi2family(afi);
+	p.prefixlen = 0;
 
 	if (bgp_debug_update(NULL, &p, subgrp->update_group, 0)) {
 		char buf[PREFIX_STRLEN];
 		/* ' with addpath ID '          17
 		 * max strlen of uint32       + 10
-		 * +/- (in case of idiocy)    +  1
+		 * +/- (just in case)         +  1
 		 * null terminator            +  1
 		 * ============================ 29 */
 		char tx_id_buf[30];
@@ -1158,7 +1217,7 @@ void subgroup_default_withdraw_packet(struct update_subgroup *subgrp)
 
 	/* Withdrawn Routes. */
 	if (p.family == AF_INET && safi == SAFI_UNICAST
-	    && !peer_cap_enhe(peer)) {
+	    && !peer_cap_enhe(peer, afi, safi)) {
 		stream_put_prefix_addpath(
 			s, &p, addpath_encode,
 			BGP_ADDPATH_TX_ID_FOR_DEFAULT_ORIGINATE);
@@ -1176,7 +1235,7 @@ void subgroup_default_withdraw_packet(struct update_subgroup *subgrp)
 		mp_start = stream_get_endp(s);
 		mplen_pos = bgp_packet_mpunreach_start(s, afi, safi);
 		bgp_packet_mpunreach_prefix(
-			s, &p, afi, safi, NULL, NULL, addpath_encode,
+			s, &p, afi, safi, NULL, NULL, 0, addpath_encode,
 			BGP_ADDPATH_TX_ID_FOR_DEFAULT_ORIGINATE, NULL);
 
 		/* Set the mp_unreach attr's length */
