@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -51,6 +50,13 @@
 #include "bgpd/rfapi/rfapi_vty.h"
 #include "bgpd/rfapi/vnc_debug.h"
 
+
+static void vnc_direct_add_rn_group_rd(struct bgp *bgp,
+				       struct rfapi_nve_group_cfg *rfg,
+				       struct route_node *rn, struct attr *attr,
+				       afi_t afi,
+				       struct rfapi_descriptor *irfd);
+
 /***********************************************************************
  * Export methods that set nexthop to CE (from 5226 roo EC) BEGIN
  ***********************************************************************/
@@ -72,7 +78,6 @@ static void encap_attr_export_ce(struct attr *new, struct attr *orig,
 	 */
 	memset(new, 0, sizeof(struct attr));
 	bgp_attr_dup(new, orig);
-	bgp_attr_extra_get(new);
 
 	/*
 	 * Set nexthop
@@ -80,17 +85,13 @@ static void encap_attr_export_ce(struct attr *new, struct attr *orig,
 	switch (use_nexthop->family) {
 	case AF_INET:
 		new->nexthop = use_nexthop->u.prefix4;
-		new->extra->mp_nexthop_len = 4; /* bytes */
+		new->mp_nexthop_len = 4; /* bytes */
 		new->flag |= ATTR_FLAG_BIT(BGP_ATTR_NEXT_HOP);
 		break;
 
 	case AF_INET6:
-		if (!new->extra) {
-			new->extra = XCALLOC(MTYPE_ATTR_EXTRA,
-					     sizeof(struct attr_extra));
-		}
-		new->extra->mp_nexthop_global = use_nexthop->u.prefix6;
-		new->extra->mp_nexthop_len = 16; /* bytes */
+		new->mp_nexthop_global = use_nexthop->u.prefix6;
+		new->mp_nexthop_len = 16; /* bytes */
 		break;
 
 	default:
@@ -126,7 +127,6 @@ static void encap_attr_export_ce(struct attr *new, struct attr *orig,
 	 *
 	 * Caller should, after using the attr, call:
 	 *  - bgp_attr_flush() to free non-interned parts
-	 *  - call bgp_attr_extra_free() to free extra
 	 */
 }
 
@@ -136,8 +136,8 @@ static int getce(struct bgp *bgp, struct attr *attr, struct prefix *pfx_ce)
 	int i;
 	uint16_t localadmin = bgp->rfapi_cfg->resolve_nve_roo_local_admin;
 
-	for (ecp = attr->extra->ecommunity->val, i = 0;
-	     i < attr->extra->ecommunity->size; ++i, ecp += ECOMMUNITY_SIZE) {
+	for (ecp = attr->ecommunity->val, i = 0; i < attr->ecommunity->size;
+	     ++i, ecp += ECOMMUNITY_SIZE) {
 
 		if (VNC_DEBUG(EXPORT_BGP_GETCE)) {
 			vnc_zlog_debug_any(
@@ -290,14 +290,12 @@ void vnc_direct_bgp_add_route_ce(struct bgp *bgp, struct route_node *rn,
 				      prefix, RMAP_BGP, &info);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
-			bgp_attr_extra_free(&hattr);
 			return;
 		}
 	}
 
 	iattr = bgp_attr_intern(&hattr);
 	bgp_attr_flush(&hattr);
-	bgp_attr_extra_free(&hattr);
 
 	/*
 	 * Rule: disallow route-map alteration of next-hop, because it
@@ -318,7 +316,7 @@ void vnc_direct_bgp_add_route_ce(struct bgp *bgp, struct route_node *rn,
 		   iattr,	   /* bgp_update copies this attr */
 		   afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
 		   BGP_ROUTE_REDISTRIBUTE, NULL, /* RD not used for unicast */
-		   NULL,			 /* tag not used for unicast */
+		   NULL, 0,			 /* tag not used for unicast */
 		   0, NULL);			 /* EVPN not used */
 	bgp_attr_unintern(&iattr);
 }
@@ -399,7 +397,7 @@ void vnc_direct_bgp_del_route_ce(struct bgp *bgp, struct route_node *rn,
 		     NULL,		  /* attr, ignored */
 		     afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
 		     BGP_ROUTE_REDISTRIBUTE, NULL, /* RD not used for unicast */
-		     NULL, NULL); /* tag not used for unicast */
+		     NULL, 0, NULL); /* tag not used for unicast */
 }
 
 static void vnc_direct_bgp_vpn_enable_ce(struct bgp *bgp, afi_t afi)
@@ -438,14 +436,11 @@ static void vnc_direct_bgp_vpn_enable_ce(struct bgp *bgp, afi_t afi)
 			continue;
 
 		{
-			char prefixstr[BUFSIZ];
+			char prefixstr[PREFIX_STRLEN];
 
-			prefixstr[0] = 0;
-			inet_ntop(rn->p.family, &rn->p.u.prefix, prefixstr,
-				  BUFSIZ);
-			vnc_zlog_debug_verbose("%s: checking prefix %s/%d",
-					       __func__, prefixstr,
-					       rn->p.prefixlen);
+			prefix2str(&rn->p, prefixstr, sizeof(prefixstr));
+			vnc_zlog_debug_verbose("%s: checking prefix %s",
+					       __func__, prefixstr);
 		}
 
 		for (ri = rn->info; ri; ri = ri->next) {
@@ -502,7 +497,7 @@ static void vnc_direct_bgp_vpn_disable_ce(struct bgp *bgp, afi_t afi)
 					ZEBRA_ROUTE_VNC_DIRECT,
 					BGP_ROUTE_REDISTRIBUTE,
 					NULL, /* RD not used for unicast */
-					NULL,
+					NULL, 0,
 					NULL); /* tag not used for unicast */
 			}
 		}
@@ -531,15 +526,13 @@ static struct ecommunity *vnc_route_origin_ecom(struct route_node *rn)
 
 		struct ecommunity_val roec;
 
-		switch (BGP_MP_NEXTHOP_FAMILY(
-			bi->attr->extra->mp_nexthop_len)) {
+		switch (BGP_MP_NEXTHOP_FAMILY(bi->attr->mp_nexthop_len)) {
 		case AF_INET:
 			memset(&roec, 0, sizeof(roec));
 			roec.val[0] = 0x01;
 			roec.val[1] = 0x03;
 			memcpy(roec.val + 2,
-			       &bi->attr->extra->mp_nexthop_global_in.s_addr,
-			       4);
+			       &bi->attr->mp_nexthop_global_in.s_addr, 4);
 			roec.val[6] = 0;
 			roec.val[7] = 0;
 			ecommunity_add_val(new, &roec);
@@ -604,14 +597,13 @@ encap_attr_export(struct attr *new, struct attr *orig,
 	} else {
 		use_nexthop = &orig_nexthop;
 		orig_nexthop.family =
-			BGP_MP_NEXTHOP_FAMILY(orig->extra->mp_nexthop_len);
+			BGP_MP_NEXTHOP_FAMILY(orig->mp_nexthop_len);
 		if (orig_nexthop.family == AF_INET) {
 			orig_nexthop.prefixlen = 32;
-			orig_nexthop.u.prefix4 =
-				orig->extra->mp_nexthop_global_in;
+			orig_nexthop.u.prefix4 = orig->mp_nexthop_global_in;
 		} else if (orig_nexthop.family == AF_INET6) {
 			orig_nexthop.prefixlen = 128;
-			orig_nexthop.u.prefix6 = orig->extra->mp_nexthop_global;
+			orig_nexthop.u.prefix6 = orig->mp_nexthop_global;
 		} else {
 			return -1; /* FAIL - can't compute nexthop */
 		}
@@ -630,17 +622,13 @@ encap_attr_export(struct attr *new, struct attr *orig,
 	switch (use_nexthop->family) {
 	case AF_INET:
 		new->nexthop = use_nexthop->u.prefix4;
-		new->extra->mp_nexthop_len = 4; /* bytes */
+		new->mp_nexthop_len = 4; /* bytes */
 		new->flag |= ATTR_FLAG_BIT(BGP_ATTR_NEXT_HOP);
 		break;
 
 	case AF_INET6:
-		if (!new->extra) {
-			new->extra = XCALLOC(MTYPE_ATTR_EXTRA,
-					     sizeof(struct attr_extra));
-		}
-		new->extra->mp_nexthop_global = use_nexthop->u.prefix6;
-		new->extra->mp_nexthop_len = 16; /* bytes */
+		new->mp_nexthop_global = use_nexthop->u.prefix6;
+		new->mp_nexthop_len = 16; /* bytes */
 		break;
 
 	default:
@@ -648,20 +636,18 @@ encap_attr_export(struct attr *new, struct attr *orig,
 		break;
 	}
 
-	bgp_attr_extra_get(new);
 	if (rn) {
 		ecom_ro = vnc_route_origin_ecom(rn);
 	} else {
-		/* TBD test/assert for IPv6 */
+		/* TBD  use lcom for IPv6 */
 		ecom_ro = vnc_route_origin_ecom_single(&use_nexthop->u.prefix4);
 	}
-	if (new->extra->ecommunity) {
-		if (ecom_ro) {
-			new->extra->ecommunity = ecommunity_merge(
-				ecom_ro, new->extra->ecommunity);
-		}
+	if (new->ecommunity) {
+		if (ecom_ro)
+			new->ecommunity =
+				ecommunity_merge(ecom_ro, new->ecommunity);
 	} else {
-		new->extra->ecommunity = ecom_ro;
+		new->ecommunity = ecom_ro;
 	}
 	if (ecom_ro) {
 		new->flag |= ATTR_FLAG_BIT(BGP_ATTR_EXT_COMMUNITIES);
@@ -695,7 +681,6 @@ encap_attr_export(struct attr *new, struct attr *orig,
 	 *
 	 * Caller should, after using the attr, call:
 	 *  - bgp_attr_flush() to free non-interned parts
-	 *  - call bgp_attr_extra_free() to free extra
 	 */
 
 	return 0;
@@ -773,7 +758,7 @@ void vnc_direct_bgp_add_prefix(struct bgp *bgp,
 		/*
 		 * if no NVEs currently associated with this group, skip it
 		 */
-		if (!rfgn->rfg->nves)
+		if (rfgn->rfg->type != RFAPI_GROUP_CFG_VRF && !rfgn->rfg->nves)
 			continue;
 
 		/*
@@ -787,6 +772,15 @@ void vnc_direct_bgp_add_prefix(struct bgp *bgp,
 				continue;
 		}
 
+		if (rfgn->rfg->type == RFAPI_GROUP_CFG_VRF) {
+			vnc_direct_add_rn_group_rd(bgp, rfgn->rfg, rn, &attr,
+						   afi, rfgn->rfg->rfd);
+			/*
+			 * yuck!
+			 *  - but consistent with rest of function
+			 */
+			continue;
+		}
 		/*
 		 * For each NVE that is assigned to the export nve group,
 		 * generate
@@ -794,77 +788,12 @@ void vnc_direct_bgp_add_prefix(struct bgp *bgp,
 		 */
 		for (ln = listhead(rfgn->rfg->nves); ln;
 		     ln = listnextnode(ln)) {
-
-			struct prefix nhp;
-			struct rfapi_descriptor *irfd;
-			struct bgp_info info;
-			struct attr hattr;
-			struct attr *iattr;
-
-			irfd = listgetdata(ln);
-
-			if (rfapiRaddr2Qprefix(&irfd->vn_addr, &nhp))
-				continue;
-
-			/*
-			 * Construct new attribute set with NVE's VN addr as
-			 * nexthop and without Tunnel Encap attr
-			 */
-			if (encap_attr_export(&hattr, &attr, &nhp, rn))
-				continue;
-
-			if (VNC_DEBUG(EXPORT_BGP_DIRECT_ADD)) {
-				vnc_zlog_debug_any("%s: attr follows",
-						   __func__);
-				rfapiPrintAttrPtrs(NULL, &attr);
-				vnc_zlog_debug_any("%s: hattr follows",
-						   __func__);
-				rfapiPrintAttrPtrs(NULL, &hattr);
-			}
-
-			if (rfgn->rfg->routemap_export_bgp) {
-				route_map_result_t ret;
-				info.peer = irfd->peer;
-				info.attr = &hattr;
-				ret = route_map_apply(
-					rfgn->rfg->routemap_export_bgp, &rn->p,
-					RMAP_BGP, &info);
-				if (ret == RMAP_DENYMATCH) {
-					bgp_attr_flush(&hattr);
-					bgp_attr_extra_free(&hattr);
-					vnc_zlog_debug_verbose(
-						"%s: route map says DENY, so not calling bgp_update",
-						__func__);
-					continue;
-				}
-			}
-
-			if (VNC_DEBUG(EXPORT_BGP_DIRECT_ADD)) {
-				vnc_zlog_debug_any(
-					"%s: hattr after route_map_apply:",
-					__func__);
-				rfapiPrintAttrPtrs(NULL, &hattr);
-			}
-
-			iattr = bgp_attr_intern(&hattr);
-			bgp_attr_flush(&hattr);
-			bgp_attr_extra_free(&hattr);
-
-			bgp_update(irfd->peer, &rn->p, /* prefix */
-				   0,		       /* addpath_id */
-				   iattr, /* bgp_update copies it */
-				   afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
-				   BGP_ROUTE_REDISTRIBUTE,
-				   NULL,     /* RD not used for unicast */
-				   NULL,     /* tag not used for unicast */
-				   0, NULL); /* EVPN not used */
-
-			bgp_attr_unintern(&iattr);
+			vnc_direct_add_rn_group_rd(bgp, rfgn->rfg, rn, &attr,
+						   afi, listgetdata(ln));
 		}
 	}
 
 	aspath_unintern(&attr.aspath);
-	bgp_attr_extra_free(&attr);
 }
 
 /*
@@ -925,7 +854,7 @@ void vnc_direct_bgp_del_prefix(struct bgp *bgp,
 		/*
 		 * if no NVEs currently associated with this group, skip it
 		 */
-		if (!rfgn->rfg->nves)
+		if (rfgn->rfg->type != RFAPI_GROUP_CFG_VRF && !rfgn->rfg->nves)
 			continue;
 
 		/*
@@ -935,6 +864,29 @@ void vnc_direct_bgp_del_prefix(struct bgp *bgp,
 		if (import_table != rfgn->rfg->rfapi_import_table)
 			continue;
 
+		if (rfgn->rfg->type == RFAPI_GROUP_CFG_VRF) {
+			struct prefix nhp;
+			struct rfapi_descriptor *irfd;
+
+			irfd = rfgn->rfg->rfd;
+
+			if (rfapiRaddr2Qprefix(&irfd->vn_addr, &nhp))
+				continue;
+
+			bgp_withdraw(irfd->peer, &rn->p, /* prefix */
+				     0,			 /* addpath_id */
+				     NULL,		 /* attr, ignored */
+				     afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
+				     BGP_ROUTE_REDISTRIBUTE,
+				     NULL, /* RD not used for unicast */
+				     NULL, 0,
+				     NULL); /* tag not used for unicast */
+			/*
+			 * yuck!
+			 *  - but consistent with rest of function
+			 */
+			continue;
+		}
 		/*
 		 * For each NVE that is assigned to the export nve group,
 		 * generate
@@ -956,8 +908,9 @@ void vnc_direct_bgp_del_prefix(struct bgp *bgp,
 				     NULL,		 /* attr, ignored */
 				     afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
 				     BGP_ROUTE_REDISTRIBUTE,
-				     NULL,	/* RD not used for unicast */
-				     NULL, NULL); /* tag not used for unicast */
+				     NULL, /* RD not used for unicast */
+				     NULL, 0,
+				     NULL); /* tag not used for unicast */
 		}
 	}
 }
@@ -1077,36 +1030,30 @@ void vnc_direct_bgp_add_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 							&info);
 						if (ret == RMAP_DENYMATCH) {
 							bgp_attr_flush(&hattr);
-							bgp_attr_extra_free(
-								&hattr);
 							continue;
 						}
 					}
 
 					iattr = bgp_attr_intern(&hattr);
 					bgp_attr_flush(&hattr);
-					bgp_attr_extra_free(&hattr);
-
-					bgp_update(irfd->peer,
-						   &rn->p, /* prefix */
-						   0,      /* addpath_id */
-						   iattr,  /* bgp_update copies
-							      it */
-						   afi, SAFI_UNICAST,
-						   ZEBRA_ROUTE_VNC_DIRECT,
-						   BGP_ROUTE_REDISTRIBUTE,
-						   NULL,     /* RD not used for
-								unicast */
-						   NULL,     /* tag not used for
-								unicast */
-						   0, NULL); /* EVPN not used */
+					bgp_update(
+						irfd->peer, &rn->p, /* prefix */
+						0,     /* addpath_id */
+						iattr, /* bgp_update copies
+							  it */
+						afi, SAFI_UNICAST,
+						ZEBRA_ROUTE_VNC_DIRECT,
+						BGP_ROUTE_REDISTRIBUTE, NULL,
+						/* RD not used for unicast */
+						NULL,
+						/* tag not used for unicast */
+						0, 0, NULL); /* EVPN not used */
 
 					bgp_attr_unintern(&iattr);
 				}
 			}
 
 			aspath_unintern(&attr.aspath);
-			bgp_attr_extra_free(&attr);
 		}
 	}
 }
@@ -1194,7 +1141,7 @@ void vnc_direct_bgp_del_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 						     BGP_ROUTE_REDISTRIBUTE,
 						     NULL, /* RD not used for
 							      unicast */
-						     NULL, NULL); /* tag not
+						     NULL, 0, NULL); /* tag not
 								     used for
 								     unicast */
 				}
@@ -1203,6 +1150,119 @@ void vnc_direct_bgp_del_nve(struct bgp *bgp, struct rfapi_descriptor *rfd)
 	}
 }
 
+static void vnc_direct_add_rn_group_rd(struct bgp *bgp,
+				       struct rfapi_nve_group_cfg *rfg,
+				       struct route_node *rn, struct attr *attr,
+				       afi_t afi, struct rfapi_descriptor *irfd)
+{
+	struct prefix nhp;
+	struct bgp_info info;
+	struct attr hattr;
+	struct attr *iattr;
+
+	if (irfd == NULL && rfg->type != RFAPI_GROUP_CFG_VRF) {
+		/* need new rfapi_handle, for peer strcture
+		 * -- based on vnc_add_vrf_prefi */
+		assert(rfg->rfd == NULL);
+
+		if (!rfg->rt_export_list || !rfg->rfapi_import_table) {
+			vnc_zlog_debug_verbose(
+				"%s: VRF \"%s\" is missing RT import/export configuration.\n",
+				__func__, rfg->name);
+			return;
+		}
+		if (!rfg->rd.prefixlen) {
+			vnc_zlog_debug_verbose(
+				"%s: VRF \"%s\" is missing RD configuration.\n",
+				__func__, rfg->name);
+			return;
+		}
+		if (rfg->label > MPLS_LABEL_MAX) {
+			vnc_zlog_debug_verbose(
+				"%s: VRF \"%s\" is missing defaul label configuration.\n",
+				__func__, rfg->name);
+			return;
+		}
+
+		irfd = XCALLOC(MTYPE_RFAPI_DESC,
+			       sizeof(struct rfapi_descriptor));
+		irfd->bgp = bgp;
+		rfg->rfd = irfd;
+		/*
+		 * leave most fields empty as will get from (dynamic) config
+		 * when needed
+		 */
+		irfd->default_tunneltype_option.type = BGP_ENCAP_TYPE_MPLS;
+		irfd->cookie = rfg;
+		if (rfg->vn_prefix.family
+		    && !CHECK_FLAG(rfg->flags, RFAPI_RFG_VPN_NH_SELF)) {
+			rfapiQprefix2Raddr(&rfg->vn_prefix, &irfd->vn_addr);
+		} else {
+			memset(&irfd->vn_addr, 0, sizeof(struct rfapi_ip_addr));
+			irfd->vn_addr.addr_family = AF_INET;
+			irfd->vn_addr.addr.v4 = bgp->router_id;
+		}
+		irfd->un_addr = irfd->vn_addr; /* sigh, need something in UN for
+						lookups */
+		vnc_zlog_debug_verbose("%s: Opening RFD for VRF %s", __func__,
+				       rfg->name);
+		rfapi_init_and_open(bgp, irfd, rfg);
+	}
+
+	if (irfd == NULL || rfapiRaddr2Qprefix(&irfd->vn_addr, &nhp))
+		return;
+
+	/*
+	 * Construct new attribute set with NVE's VN
+	 * addr as
+	 * nexthop and without Tunnel Encap attr
+	 */
+	if (encap_attr_export(&hattr, attr, &nhp, rn))
+		return;
+
+	if (VNC_DEBUG(EXPORT_BGP_DIRECT_ADD)) {
+		vnc_zlog_debug_any("%s: attr follows", __func__);
+		rfapiPrintAttrPtrs(NULL, attr);
+		vnc_zlog_debug_any("%s: hattr follows", __func__);
+		rfapiPrintAttrPtrs(NULL, &hattr);
+	}
+
+	if (rfg->routemap_export_bgp) {
+		route_map_result_t ret;
+
+		info.peer = irfd->peer;
+		info.attr = &hattr;
+		ret = route_map_apply(rfg->routemap_export_bgp, &rn->p,
+				      RMAP_BGP, &info);
+		if (ret == RMAP_DENYMATCH) {
+			bgp_attr_flush(&hattr);
+			vnc_zlog_debug_verbose(
+				"%s: route map says DENY, so not calling bgp_update",
+				__func__);
+			return;
+		}
+	}
+
+	if (VNC_DEBUG(EXPORT_BGP_DIRECT_ADD)) {
+		vnc_zlog_debug_any("%s: hattr after route_map_apply:",
+				   __func__);
+		rfapiPrintAttrPtrs(NULL, &hattr);
+	}
+	iattr = bgp_attr_intern(&hattr);
+	bgp_attr_flush(&hattr);
+
+	bgp_update(irfd->peer, &rn->p, /* prefix */
+		   0,		       /* addpath_id */
+		   iattr,	      /* bgp_update copies it */
+		   afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
+		   BGP_ROUTE_REDISTRIBUTE, NULL, /* RD not used for unicast */
+		   NULL,			 /* tag not used for unicast */
+		   0, 0, NULL);			 /* EVPN not used */
+
+	bgp_attr_unintern(&iattr);
+
+	return;
+}
 
 /*
  * Caller is responsible for ensuring that the specified nve-group
@@ -1233,8 +1293,7 @@ static void vnc_direct_bgp_add_group_afi(struct bgp *bgp,
 		return;
 	}
 
-	if (!rfg->nves) {
-		/* avoid segfault below if list doesn't exist */
+	if (!rfg->nves && rfg->type != RFAPI_GROUP_CFG_VRF) {
 		vnc_zlog_debug_verbose("%s: no NVEs in this group", __func__);
 		return;
 	}
@@ -1261,7 +1320,15 @@ static void vnc_direct_bgp_add_group_afi(struct bgp *bgp,
 
 					continue;
 			}
-
+			if (rfg->type == RFAPI_GROUP_CFG_VRF) {
+				vnc_direct_add_rn_group_rd(bgp, rfg, rn, &attr,
+							   afi, rfg->rfd);
+				/*
+				 * yuck!
+				 *  - but consistent with rest of function
+				 */
+				continue;
+			}
 			/*
 			 * For each NVE that is assigned to the export nve
 			 * group, generate
@@ -1269,61 +1336,14 @@ static void vnc_direct_bgp_add_group_afi(struct bgp *bgp,
 			 */
 			for (ln = listhead(rfg->nves); ln;
 			     ln = listnextnode(ln)) {
-
-				struct prefix nhp;
-				struct rfapi_descriptor *irfd;
-				struct bgp_info info;
-				struct attr hattr;
-				struct attr *iattr;
-
-				irfd = listgetdata(ln);
-
-				if (rfapiRaddr2Qprefix(&irfd->vn_addr, &nhp))
-					continue;
-
-				/*
-				 * Construct new attribute set with NVE's VN
-				 * addr as
-				 * nexthop and without Tunnel Encap attr
-				 */
-				if (encap_attr_export(&hattr, &attr, &nhp, rn))
-					continue;
-
-				if (rfg->routemap_export_bgp) {
-					route_map_result_t ret;
-					info.peer = irfd->peer;
-					info.attr = &hattr;
-					ret = route_map_apply(
-						rfg->routemap_export_bgp,
-						&rn->p, RMAP_BGP, &info);
-					if (ret == RMAP_DENYMATCH) {
-						bgp_attr_flush(&hattr);
-						bgp_attr_extra_free(&hattr);
-						continue;
-					}
-				}
-
-				iattr = bgp_attr_intern(&hattr);
-				bgp_attr_flush(&hattr);
-				bgp_attr_extra_free(&hattr);
-
-				bgp_update(irfd->peer, &rn->p, /* prefix */
-					   0,		       /* addpath_id */
-					   iattr, /* bgp_update copies it */
-					   afi, SAFI_UNICAST,
-					   ZEBRA_ROUTE_VNC_DIRECT,
-					   BGP_ROUTE_REDISTRIBUTE,
-					   NULL, /* RD not used for unicast */
-					   NULL, /* tag not used for unicast */
-					   0, NULL); /* EVPN not used */
-
-				bgp_attr_unintern(&iattr);
+				vnc_direct_add_rn_group_rd(bgp, rfg, rn, &attr,
+							   afi,
+							   listgetdata(ln));
 			}
 		}
 	}
 
 	aspath_unintern(&attr.aspath);
-	bgp_attr_extra_free(&attr);
 }
 
 
@@ -1337,6 +1357,21 @@ void vnc_direct_bgp_add_group(struct bgp *bgp, struct rfapi_nve_group_cfg *rfg)
 	vnc_direct_bgp_add_group_afi(bgp, rfg, AFI_IP6);
 }
 
+static void vnc_direct_del_rn_group_rd(struct bgp *bgp,
+				       struct rfapi_nve_group_cfg *rfg,
+				       struct route_node *rn, afi_t afi,
+				       struct rfapi_descriptor *irfd)
+{
+	if (irfd == NULL)
+		return;
+	bgp_withdraw(irfd->peer, &rn->p, /* prefix */
+		     0,			 /* addpath_id */
+		     NULL,		 /* attr, ignored */
+		     afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT,
+		     BGP_ROUTE_REDISTRIBUTE, NULL, /* RD not used for unicast */
+		     NULL, 0, NULL); /* tag not used for unicast */
+	return;
+}
 
 /*
  * Caller is responsible for ensuring that the specified nve-group
@@ -1362,8 +1397,7 @@ static void vnc_direct_bgp_del_group_afi(struct bgp *bgp,
 	assert(afi == AFI_IP || afi == AFI_IP6);
 	rt = import_table->imported_vpn[afi];
 
-	if (!rfg->nves) {
-		/* avoid segfault below if list does not exist */
+	if (!rfg->nves && rfg->type != RFAPI_GROUP_CFG_VRF) {
 		vnc_zlog_debug_verbose("%s: no NVEs in this group", __func__);
 		return;
 	}
@@ -1371,40 +1405,29 @@ static void vnc_direct_bgp_del_group_afi(struct bgp *bgp,
 	/*
 	 * Walk the NVE-Group's VNC Import table
 	 */
-	for (rn = route_top(rt); rn; rn = route_next(rn)) {
-
+	for (rn = route_top(rt); rn; rn = route_next(rn))
 		if (rn->info) {
+			if (rfg->type == RFAPI_GROUP_CFG_VRF)
+				vnc_direct_del_rn_group_rd(bgp, rfg, rn, afi,
+							   rfg->rfd);
+			else {
+				struct listnode *ln;
 
-			struct listnode *ln;
-
-			/*
-			 * For each NVE that is assigned to the export nve
-			 * group, generate
-			 * a route with that NVE as its next hop
-			 */
-			for (ln = listhead(rfg->nves); ln;
-			     ln = listnextnode(ln)) {
-
-				struct rfapi_descriptor *irfd;
-
-				irfd = listgetdata(ln);
-
-				bgp_withdraw(
-					irfd->peer, &rn->p, /* prefix */
-					0,		    /* addpath_id */
-					NULL,		    /* attr, ignored */
-					afi, SAFI_UNICAST,
-					ZEBRA_ROUTE_VNC_DIRECT,
-					BGP_ROUTE_REDISTRIBUTE,
-					NULL, /* RD not used for unicast */
-					NULL,
-					NULL); /* tag not used for unicast */
+				/*
+				 * For each NVE that is assigned to the export
+				 * nve
+				 * group, generate
+				 * a route with that NVE as its next hop
+				 */
+				for (ln = listhead(rfg->nves); ln;
+				     ln = listnextnode(ln))
+					vnc_direct_del_rn_group_rd(
+						bgp, rfg, rn, afi,
+						listgetdata(ln));
 			}
 		}
-	}
 }
 
-
 /*
  * Caller is responsible for ensuring that the specified nve-group
  * was actually part of the list of exported nve groups.
@@ -1469,7 +1492,7 @@ static void vnc_direct_bgp_unexport_table(afi_t afi, struct route_table *rt,
 						     BGP_ROUTE_REDISTRIBUTE,
 						     NULL, /* RD not used for
 							      unicast */
-						     NULL, NULL); /* tag not
+						     NULL, 0, NULL); /* tag not
 								     used for
 								     unicast,
 								     EVPN
@@ -1501,10 +1524,15 @@ static void import_table_to_nve_list_direct_bgp(struct bgp *bgp,
 		/*
 		 * If this NVE-Group's import table matches the current one
 		 */
-		if (rfgn->rfg && rfgn->rfg->nves
-		    && rfgn->rfg->rfapi_import_table == it) {
-
-			nve_group_to_nve_list(rfgn->rfg, nves, family);
+		if (rfgn->rfg && rfgn->rfg->rfapi_import_table == it) {
+			if (rfgn->rfg->nves)
+				nve_group_to_nve_list(rfgn->rfg, nves, family);
+			else if (rfgn->rfg->rfd
+				 && rfgn->rfg->type == RFAPI_GROUP_CFG_VRF) {
+				if (!*nves)
+					*nves = list_new();
+				listnode_add(*nves, rfgn->rfg->rfd);
+			}
 		}
 	}
 }
@@ -1573,7 +1601,7 @@ void vnc_direct_bgp_vpn_disable(struct bgp *bgp, afi_t afi)
 		if (nve_list) {
 			vnc_direct_bgp_unexport_table(
 				afi, it->imported_vpn[afi], nve_list);
-			list_free(nve_list);
+			list_delete_and_null(&nve_list);
 		}
 	}
 }
@@ -1656,14 +1684,12 @@ void vnc_direct_bgp_rh_add_route(struct bgp *bgp, afi_t afi,
 				      &info);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
-			bgp_attr_extra_free(&hattr);
 			return;
 		}
 	}
 
 	iattr = bgp_attr_intern(&hattr);
 	bgp_attr_flush(&hattr);
-	bgp_attr_extra_free(&hattr);
 
 	/*
 	 * record route information that we will need to expire
@@ -1688,8 +1714,8 @@ void vnc_direct_bgp_rh_add_route(struct bgp *bgp, afi_t afi,
 		   iattr,	/* bgp_update copies this attr */
 		   afi, SAFI_UNICAST, ZEBRA_ROUTE_VNC_DIRECT_RH,
 		   BGP_ROUTE_REDISTRIBUTE, NULL, /* RD not used for unicast */
-		   NULL,     /* tag not used for unicast, EVPN neither */
-		   0, NULL); /* EVPN not used */
+		   NULL,	/* tag not used for unicast, EVPN neither */
+		   0, 0, NULL); /* EVPN not used */
 	bgp_attr_unintern(&iattr);
 }
 
@@ -1704,7 +1730,8 @@ static int vncExportWithdrawTimer(struct thread *t)
 		     NULL,			  /* attr, ignored */
 		     family2afi(eti->node->p.family), SAFI_UNICAST, eti->type,
 		     eti->subtype, NULL, /* RD not used for unicast */
-		     NULL, NULL); /* tag not used for unicast, EVPN neither */
+		     NULL, 0,
+		     NULL); /* tag not used for unicast, EVPN neither */
 
 	/*
 	 * Free the eti
@@ -1753,8 +1780,9 @@ void vnc_direct_bgp_rh_del_route(struct bgp *bgp, afi_t afi,
 			  ZEBRA_ROUTE_VNC_DIRECT_RH, BGP_ROUTE_REDISTRIBUTE);
 
 	if (!eti->timer && eti->lifetime <= INT32_MAX) {
-		eti->timer = thread_add_timer(
-			bm->master, vncExportWithdrawTimer, eti, eti->lifetime);
+		eti->timer = NULL;
+		thread_add_timer(bm->master, vncExportWithdrawTimer, eti,
+				 eti->lifetime, &eti->timer);
 		vnc_zlog_debug_verbose(
 			"%s: set expiration timer for %u seconds", __func__,
 			eti->lifetime);
@@ -1810,6 +1838,9 @@ void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 		/* This is the per-RD table of prefixes */
 		table = prn->info;
 
+		if (!table)
+			continue;
+
 		for (rn = bgp_table_top(table); rn; rn = bgp_route_next(rn)) {
 
 			/*
@@ -1819,14 +1850,12 @@ void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 				continue;
 
 			{
-				char prefixstr[BUFSIZ];
+				char prefixstr[PREFIX_STRLEN];
 
-				prefixstr[0] = 0;
-				inet_ntop(rn->p.family, &rn->p.u.prefix,
-					  prefixstr, BUFSIZ);
-				vnc_zlog_debug_verbose(
-					"%s: checking prefix %s/%d", __func__,
-					prefixstr, rn->p.prefixlen);
+				prefix2str(&rn->p, prefixstr,
+					   sizeof(prefixstr));
+				vnc_zlog_debug_verbose("%s: checking prefix %s",
+						       __func__, prefixstr);
 			}
 
 			/*
@@ -1882,8 +1911,6 @@ void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 							&info);
 						if (ret == RMAP_DENYMATCH) {
 							bgp_attr_flush(&hattr);
-							bgp_attr_extra_free(
-								&hattr);
 							vnc_zlog_debug_verbose(
 								"%s:   route map says DENY",
 								__func__);
@@ -1893,7 +1920,6 @@ void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 
 					iattr = bgp_attr_intern(&hattr);
 					bgp_attr_flush(&hattr);
-					bgp_attr_extra_free(&hattr);
 
 					/*
 					 * record route information that we will
@@ -1922,20 +1948,20 @@ void vnc_direct_bgp_rh_vpn_enable(struct bgp *bgp, afi_t afi)
 						"%s: calling bgp_update",
 						__func__);
 
-					bgp_update(ri->peer,
-						   &rn->p, /* prefix */
-						   0,      /* addpath_id */
-						   iattr,  /* bgp_update copies
-							      it */
-						   AFI_IP, SAFI_UNICAST,
-						   ZEBRA_ROUTE_VNC_DIRECT_RH,
-						   BGP_ROUTE_REDISTRIBUTE,
-						   NULL,     /* RD not used for
-								unicast */
-						   NULL,     /* tag not used for
-								unicast, EVPN
-								neither */
-						   0, NULL); /* EVPN not used */
+					bgp_update(
+						ri->peer, &rn->p, /* prefix */
+						0,     /* addpath_id */
+						iattr, /* bgp_update copies
+							  it */
+						AFI_IP, SAFI_UNICAST,
+						ZEBRA_ROUTE_VNC_DIRECT_RH,
+						BGP_ROUTE_REDISTRIBUTE, NULL,
+						/* RD not used for unicast */
+						NULL,
+						/* tag not used for unicast,
+						   or EVPN */
+						0, 0, NULL); /* EVPN not used */
+
 					bgp_attr_unintern(&iattr);
 				}
 			}
@@ -1996,7 +2022,7 @@ void vnc_direct_bgp_rh_vpn_disable(struct bgp *bgp, afi_t afi)
 					     ZEBRA_ROUTE_VNC_DIRECT_RH,
 					     BGP_ROUTE_REDISTRIBUTE,
 					     NULL, /* RD not used for unicast */
-					     NULL, NULL); /* tag not used for
+					     NULL, 0, NULL); /* tag not used for
 							     unicast, EVPN
 							     neither */
 			}
@@ -2023,6 +2049,9 @@ void vnc_direct_bgp_rh_reexport(struct bgp *bgp, afi_t afi)
  */
 void vnc_export_bgp_enable(struct bgp *bgp, afi_t afi)
 {
+	if (!bgp->rfapi_cfg)
+		return;
+
 	switch (bgp->rfapi_cfg->flags & BGP_VNC_CONFIG_EXPORT_BGP_MODE_BITS) {
 	case BGP_VNC_CONFIG_EXPORT_BGP_MODE_NONE:
 		break;
@@ -2043,6 +2072,9 @@ void vnc_export_bgp_enable(struct bgp *bgp, afi_t afi)
 
 void vnc_export_bgp_disable(struct bgp *bgp, afi_t afi)
 {
+	if (!bgp->rfapi_cfg)
+		return;
+
 	switch (bgp->rfapi_cfg->flags & BGP_VNC_CONFIG_EXPORT_BGP_MODE_BITS) {
 	case BGP_VNC_CONFIG_EXPORT_BGP_MODE_NONE:
 		break;
