@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /*
@@ -125,14 +124,14 @@ struct prefix_bag {
 	struct bgp_info *ubi; /* unicast route */
 };
 
-static const u_char maskbit[] = {0x00, 0x80, 0xc0, 0xe0, 0xf0,
-				 0xf8, 0xfc, 0xfe, 0xff};
+static const uint8_t maskbit[] = {0x00, 0x80, 0xc0, 0xe0, 0xf0,
+				  0xf8, 0xfc, 0xfe, 0xff};
 
 int vnc_prefix_cmp(void *pfx1, void *pfx2)
 {
 	int offset;
 	int shift;
-	u_char mask;
+	uint8_t mask;
 
 	struct prefix *p1 = pfx1;
 	struct prefix *p2 = pfx2;
@@ -155,8 +154,8 @@ int vnc_prefix_cmp(void *pfx1, void *pfx2)
 	}
 
 	/* Set both prefix's head pointer. */
-	const u_char *pp1 = (const u_char *)&p1->u.prefix;
-	const u_char *pp2 = (const u_char *)&p2->u.prefix;
+	const uint8_t *pp1 = (const uint8_t *)&p1->u.prefix;
+	const uint8_t *pp2 = (const uint8_t *)&p2->u.prefix;
 
 	while (offset--) {
 		if (*pp1 < *pp2)
@@ -206,15 +205,15 @@ static void print_rhn_list(const char *tag1, const char *tag2)
 
 	/* XXX uses secret knowledge of skiplist structure */
 	for (p = sl->header->forward[0]; p; p = p->forward[0]) {
-		char kbuf[BUFSIZ];
-		char hbuf[BUFSIZ];
-		char ubuf[BUFSIZ];
+		char kbuf[PREFIX_STRLEN];
+		char hbuf[PREFIX_STRLEN];
+		char ubuf[PREFIX_STRLEN];
 
 		pb = p->value;
 
-		prefix2str(p->key, kbuf, BUFSIZ);
-		prefix2str(&pb->hpfx, hbuf, BUFSIZ);
-		prefix2str(&pb->upfx, ubuf, BUFSIZ);
+		prefix2str(p->key, kbuf, sizeof(kbuf));
+		prefix2str(&pb->hpfx, hbuf, sizeof(hbuf));
+		prefix2str(&pb->upfx, ubuf, sizeof(ubuf));
 
 		vnc_zlog_debug_verbose(
 			"RHN Entry %d (q=%p): kpfx=%s, upfx=%s, hpfx=%s, ubi=%p",
@@ -260,14 +259,11 @@ static void vnc_rhnck(char *tag)
 		 * pfx */
 		assert(!vnc_prefix_cmp(&pb->hpfx, pkey));
 		if (vnc_prefix_cmp(&pb->hpfx, &pfx_orig_nexthop)) {
-			char str_onh[BUFSIZ];
-			char str_nve_pfx[BUFSIZ];
+			char str_onh[PREFIX_STRLEN];
+			char str_nve_pfx[PREFIX_STRLEN];
 
-			prefix2str(&pfx_orig_nexthop, str_onh, BUFSIZ);
-			str_onh[BUFSIZ - 1] = 0;
-
-			prefix2str(&pb->hpfx, str_nve_pfx, BUFSIZ);
-			str_nve_pfx[BUFSIZ - 1] = 0;
+			prefix2str(&pfx_orig_nexthop, str_onh, sizeof(str_onh));
+			prefix2str(&pb->hpfx, str_nve_pfx, sizeof(str_nve_pfx));
 
 			vnc_zlog_debug_verbose(
 				"%s: %s: FATAL: resolve_nve_nexthop list item bi nexthop %s != nve pfx %s",
@@ -371,7 +367,6 @@ static int process_unicast_route(struct bgp *bgp,		 /* in */
 		ret = route_map_apply(rmap, prefix, RMAP_BGP, &info);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
-			bgp_attr_extra_free(&hattr);
 			vnc_zlog_debug_verbose(
 				"%s: route map \"%s\" says DENY, returning",
 				__func__, rmap->name);
@@ -385,8 +380,8 @@ static int process_unicast_route(struct bgp *bgp,		 /* in */
 	 */
 	rfapiUnicastNexthop2Prefix(afi, &hattr, unicast_nexthop);
 
-	if (hattr.extra && hattr.extra->ecommunity)
-		*ecom = ecommunity_dup(hattr.extra->ecommunity);
+	if (hattr.ecommunity)
+		*ecom = ecommunity_dup(hattr.ecommunity);
 	else
 		*ecom = ecommunity_new();
 
@@ -394,7 +389,6 @@ static int process_unicast_route(struct bgp *bgp,		 /* in */
 	 * Done with hattr, clean up
 	 */
 	bgp_attr_flush(&hattr);
-	bgp_attr_extra_free(&hattr);
 
 	/*
 	 * Add EC that carries original NH of iBGP route (2 bytes = magic
@@ -479,17 +473,16 @@ static void vnc_import_bgp_add_route_mode_resolve_nve_one_bi(
 		plifetime = &lifetime;
 	}
 
-	if (bi->attr && bi->attr->extra) {
-		encaptlvs = bi->attr->extra->vnc_subtlvs;
-		if (bi->attr->extra->encap_tunneltype != BGP_ENCAP_TYPE_RESERVED
-		    && bi->attr->extra->encap_tunneltype
-			       != BGP_ENCAP_TYPE_MPLS) {
+	if (bi->attr) {
+		encaptlvs = bi->attr->vnc_subtlvs;
+		if (bi->attr->encap_tunneltype != BGP_ENCAP_TYPE_RESERVED
+		    && bi->attr->encap_tunneltype != BGP_ENCAP_TYPE_MPLS) {
 			if (opt != NULL)
 				opt->next = &optary[cur_opt];
 			opt = &optary[cur_opt++];
 			memset(opt, 0, sizeof(struct rfapi_un_option));
 			opt->type = RFAPI_UN_OPTION_TYPE_TUNNELTYPE;
-			opt->v.tunnel.type = bi->attr->extra->encap_tunneltype;
+			opt->v.tunnel.type = bi->attr->encap_tunneltype;
 			/* TBD parse bi->attr->extra->encap_subtlvs */
 		}
 	} else {
@@ -498,11 +491,11 @@ static void vnc_import_bgp_add_route_mode_resolve_nve_one_bi(
 
 	struct ecommunity *new_ecom = ecommunity_dup(ecom);
 
-	if (bi->attr && bi->attr->extra && bi->attr->extra->ecommunity)
-		ecommunity_merge(new_ecom, bi->attr->extra->ecommunity);
+	if (bi->attr && bi->attr->ecommunity)
+		ecommunity_merge(new_ecom, bi->attr->ecommunity);
 
 	if (bi->extra)
-		label = decode_label(bi->extra->tag);
+		label = decode_label(&bi->extra->label[0]);
 
 	add_vnc_route(&vncHDResolveNve, bgp, SAFI_MPLS_VPN,
 		      prefix,	  /* unicast route prefix */
@@ -533,10 +526,9 @@ static void vnc_import_bgp_add_route_mode_resolve_nve_one_rd(
 		return;
 
 	{
-		char str_nh[BUFSIZ];
+		char str_nh[PREFIX_STRLEN];
 
-		prefix2str(ubi_nexthop, str_nh, BUFSIZ);
-		str_nh[BUFSIZ - 1] = 0;
+		prefix2str(ubi_nexthop, str_nh, sizeof(str_nh));
 
 		vnc_zlog_debug_verbose("%s: ubi_nexthop=%s", __func__, str_nh);
 	}
@@ -577,19 +569,17 @@ static void vnc_import_bgp_add_route_mode_resolve_nve(
 	struct bgp_node *bnp; /* prd table node */
 
 	/*debugging */
-	{
-		char str_pfx[BUFSIZ];
-		char str_nh[BUFSIZ];
+	if (VNC_DEBUG(VERBOSE)) {
+		char str_pfx[PREFIX_STRLEN];
+		char str_nh[PREFIX_STRLEN];
 		struct prefix nh;
 
-		prefix2str(prefix, str_pfx, BUFSIZ);
-		str_pfx[BUFSIZ - 1] = 0;
+		prefix2str(prefix, str_pfx, sizeof(str_pfx));
 
 		nh.prefixlen = 0;
 		rfapiUnicastNexthop2Prefix(afi, info->attr, &nh);
 		if (nh.prefixlen) {
-			prefix2str(&nh, str_nh, BUFSIZ);
-			str_nh[BUFSIZ - 1] = 0;
+			prefix2str(&nh, str_nh, sizeof(str_nh));
 		} else {
 			str_nh[0] = '?';
 			str_nh[1] = 0;
@@ -722,11 +712,9 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 	uint32_t *med = NULL;
 
 	{
-		char buf[BUFSIZ];
+		char buf[PREFIX_STRLEN];
 
-		buf[0] = 0;
-		prefix2str(prefix, buf, BUFSIZ);
-		buf[BUFSIZ - 1] = 0;
+		prefix2str(prefix, buf, sizeof(buf));
 		vnc_zlog_debug_verbose("%s(prefix=%s) entry", __func__, buf);
 	}
 
@@ -793,11 +781,9 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 	}
 
 	if (VNC_DEBUG(IMPORT_BGP_ADD_ROUTE)) {
-		char buf[BUFSIZ];
+		char buf[PREFIX_STRLEN];
 
-		buf[0] = 0;
-		prefix2str(vn_pfx, buf, BUFSIZ);
-		buf[BUFSIZ - 1] = 0;
+		prefix2str(vn_pfx, buf, sizeof(buf));
 		vnc_zlog_debug_any("%s vn_pfx=%s", __func__, buf);
 	}
 
@@ -829,7 +815,6 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 		ret = route_map_apply(rmap, prefix, RMAP_BGP, &info);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
-			bgp_attr_extra_free(&hattr);
 			vnc_zlog_debug_verbose(
 				"%s: route map \"%s\" says DENY, returning",
 				__func__, rmap->name);
@@ -839,7 +824,6 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 
 	iattr = bgp_attr_intern(&hattr);
 	bgp_attr_flush(&hattr);
-	bgp_attr_extra_free(&hattr);
 
 	/* Now iattr is an allocated interned attr */
 
@@ -862,8 +846,8 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 		memset(&prd, 0, sizeof(prd));
 		rfapi_set_autord_from_vn(&prd, &vnaddr);
 
-		if (iattr && iattr->extra && iattr->extra->ecommunity)
-			ecom = ecommunity_dup(iattr->extra->ecommunity);
+		if (iattr && iattr->ecommunity)
+			ecom = ecommunity_dup(iattr->ecommunity);
 	}
 
 	local_pref = calc_local_pref(iattr, peer);
@@ -873,11 +857,9 @@ static void vnc_import_bgp_add_route_mode_plain(struct bgp *bgp,
 	}
 
 	if (VNC_DEBUG(IMPORT_BGP_ADD_ROUTE)) {
-		char buf[BUFSIZ];
+		char buf[PREFIX_STRLEN];
 
-		buf[0] = 0;
-		rfapiRfapiIpAddr2Str(&vnaddr, buf, BUFSIZ);
-		buf[BUFSIZ - 1] = 0;
+		rfapiRfapiIpAddr2Str(&vnaddr, buf, sizeof(buf));
 		vnc_zlog_debug_any("%s: setting vnaddr to %s", __func__, buf);
 	}
 
@@ -916,11 +898,9 @@ vnc_import_bgp_add_route_mode_nvegroup(struct bgp *bgp, struct prefix *prefix,
 	uint32_t local_pref;
 
 	{
-		char buf[BUFSIZ];
+		char buf[PREFIX_STRLEN];
 
-		buf[0] = 0;
-		prefix2str(prefix, buf, BUFSIZ);
-		buf[BUFSIZ - 1] = 0;
+		prefix2str(prefix, buf, sizeof(buf));
 		vnc_zlog_debug_verbose("%s(prefix=%s) entry", __func__, buf);
 	}
 
@@ -1003,11 +983,9 @@ vnc_import_bgp_add_route_mode_nvegroup(struct bgp *bgp, struct prefix *prefix,
 	}
 
 	if (VNC_DEBUG(IMPORT_BGP_ADD_ROUTE)) {
-		char buf[BUFSIZ];
+		char buf[PREFIX_STRLEN];
 
-		buf[0] = 0;
-		prefix2str(vn_pfx, buf, BUFSIZ);
-		buf[BUFSIZ - 1] = 0;
+		prefix2str(vn_pfx, buf, sizeof(buf));
 		vnc_zlog_debug_any("%s vn_pfx=%s", __func__, buf);
 	}
 
@@ -1039,7 +1017,6 @@ vnc_import_bgp_add_route_mode_nvegroup(struct bgp *bgp, struct prefix *prefix,
 		ret = route_map_apply(rmap, prefix, RMAP_BGP, &info);
 		if (ret == RMAP_DENYMATCH) {
 			bgp_attr_flush(&hattr);
-			bgp_attr_extra_free(&hattr);
 			vnc_zlog_debug_verbose(
 				"%s: route map \"%s\" says DENY, returning",
 				__func__, rmap->name);
@@ -1049,7 +1026,6 @@ vnc_import_bgp_add_route_mode_nvegroup(struct bgp *bgp, struct prefix *prefix,
 
 	iattr = bgp_attr_intern(&hattr);
 	bgp_attr_flush(&hattr);
-	bgp_attr_extra_free(&hattr);
 
 	/* Now iattr is an allocated interned attr */
 
@@ -1076,8 +1052,8 @@ vnc_import_bgp_add_route_mode_nvegroup(struct bgp *bgp, struct prefix *prefix,
 		else
 			ecom = ecommunity_new();
 
-		if (iattr && iattr->extra && iattr->extra->ecommunity)
-			ecom = ecommunity_merge(ecom, iattr->extra->ecommunity);
+		if (iattr && iattr->ecommunity)
+			ecom = ecommunity_merge(ecom, iattr->ecommunity);
 	}
 
 	local_pref = calc_local_pref(iattr, peer);
@@ -1191,7 +1167,8 @@ static void vnc_import_bgp_del_route_mode_nvegroup(struct bgp *bgp,
 
 	assert(afi);
 
-	assert((rfg = bgp->rfapi_cfg->rfg_redist));
+	rfg = bgp->rfapi_cfg->rfg_redist;
+	assert(rfg);
 
 	/*
 	 * Compute VN address
@@ -1305,11 +1282,9 @@ static void vnc_import_bgp_del_route_mode_resolve_nve_one_rd(
 		return;
 
 	{
-		char str_nh[BUFSIZ];
-
-		prefix2str(ubi_nexthop, str_nh, BUFSIZ);
-		str_nh[BUFSIZ - 1] = 0;
+		char str_nh[PREFIX_STRLEN];
 
+		prefix2str(ubi_nexthop, str_nh, sizeof(str_nh));
 		vnc_zlog_debug_verbose("%s: ubi_nexthop=%s", __func__, str_nh);
 	}
 
@@ -1458,7 +1433,7 @@ void vnc_import_bgp_add_vnc_host_route_mode_resolve_nve(
 		return;
 	}
 
-	if (bgp && bgp->rfapi)
+	if (bgp->rfapi)
 		sl = bgp->rfapi->resolve_nve_nexthop;
 
 	if (!sl) {
@@ -1485,11 +1460,11 @@ void vnc_import_bgp_add_vnc_host_route_mode_resolve_nve(
 		       sizeof(struct prefix)); /* keep valgrind happy */
 
 		if (VNC_DEBUG(IMPORT_BGP_ADD_ROUTE)) {
-			char hbuf[BUFSIZ];
-			char ubuf[BUFSIZ];
+			char hbuf[PREFIX_STRLEN];
+			char ubuf[PREFIX_STRLEN];
 
-			prefix2str(&pb->hpfx, hbuf, BUFSIZ);
-			prefix2str(&pb->upfx, ubuf, BUFSIZ);
+			prefix2str(&pb->hpfx, hbuf, sizeof(hbuf));
+			prefix2str(&pb->upfx, ubuf, sizeof(ubuf));
 
 			vnc_zlog_debug_any(
 				"%s: examining RHN Entry (q=%p): upfx=%s, hpfx=%s, ubi=%p",
@@ -1517,14 +1492,12 @@ void vnc_import_bgp_add_vnc_host_route_mode_resolve_nve(
 		 * Sanity check
 		 */
 		if (vnc_prefix_cmp(&pfx_unicast_nexthop, prefix)) {
-			char str_unh[BUFSIZ];
-			char str_nve_pfx[BUFSIZ];
+			char str_unh[PREFIX_STRLEN];
+			char str_nve_pfx[PREFIX_STRLEN];
 
-			prefix2str(&pfx_unicast_nexthop, str_unh, BUFSIZ);
-			str_unh[BUFSIZ - 1] = 0;
-
-			prefix2str(prefix, str_nve_pfx, BUFSIZ);
-			str_nve_pfx[BUFSIZ - 1] = 0;
+			prefix2str(&pfx_unicast_nexthop, str_unh,
+				   sizeof(str_unh));
+			prefix2str(prefix, str_nve_pfx, sizeof(str_nve_pfx));
 
 			vnc_zlog_debug_verbose(
 				"%s: FATAL: resolve_nve_nexthop list item bi nexthop %s != nve pfx %s",
@@ -1543,9 +1516,9 @@ void vnc_import_bgp_add_vnc_host_route_mode_resolve_nve(
 #if DEBUG_RHN_LIST
 		/* debug */
 		{
-			char pbuf[BUFSIZ];
+			char pbuf[PREFIX_STRLEN];
 
-			prefix2str(prefix, pbuf, BUFSIZ);
+			prefix2str(prefix, pbuf, sizeof(pbuf));
 
 			vnc_zlog_debug_verbose(
 				"%s: advancing past RHN Entry (q=%p): with prefix %s",
@@ -1573,10 +1546,9 @@ void vnc_import_bgp_del_vnc_host_route_mode_resolve_nve(
 	int rc;
 
 	{
-		char str_pfx[BUFSIZ];
+		char str_pfx[PREFIX_STRLEN];
 
-		prefix2str(prefix, str_pfx, BUFSIZ);
-		str_pfx[BUFSIZ - 1] = 0;
+		prefix2str(prefix, str_pfx, sizeof(str_pfx));
 
 		vnc_zlog_debug_verbose("%s(bgp=%p, nve prefix=%s)", __func__,
 				       bgp, str_pfx);
@@ -1605,7 +1577,7 @@ void vnc_import_bgp_del_vnc_host_route_mode_resolve_nve(
 		return;
 	}
 
-	if (bgp && bgp->rfapi)
+	if (bgp->rfapi)
 		sl = bgp->rfapi->resolve_nve_nexthop;
 
 	if (!sl) {
@@ -1644,14 +1616,12 @@ void vnc_import_bgp_del_vnc_host_route_mode_resolve_nve(
 		 * Sanity check
 		 */
 		if (vnc_prefix_cmp(&pfx_unicast_nexthop, prefix)) {
-			char str_unh[BUFSIZ];
-			char str_nve_pfx[BUFSIZ];
+			char str_unh[PREFIX_STRLEN];
+			char str_nve_pfx[PREFIX_STRLEN];
 
-			prefix2str(&pfx_unicast_nexthop, str_unh, BUFSIZ);
-			str_unh[BUFSIZ - 1] = 0;
-
-			prefix2str(prefix, str_nve_pfx, BUFSIZ);
-			str_nve_pfx[BUFSIZ - 1] = 0;
+			prefix2str(&pfx_unicast_nexthop, str_unh,
+				   sizeof(str_unh));
+			prefix2str(prefix, str_nve_pfx, sizeof(str_nve_pfx));
 
 			vnc_zlog_debug_verbose(
 				"%s: FATAL: resolve_nve_nexthop list item bi nexthop %s != nve pfx %s",
@@ -1792,7 +1762,7 @@ static void vnc_import_bgp_exterior_add_route_it(
 			     bi_interior = bi_interior->next) {
 				struct prefix_rd *prd;
 				struct attr new_attr;
-				u_int32_t label = 0;
+				uint32_t label = 0;
 
 				if (!is_usable_interior_route(bi_interior))
 					continue;
@@ -1813,7 +1783,7 @@ static void vnc_import_bgp_exterior_add_route_it(
 					prd = &bi_interior->extra->vnc.import
 						       .rd;
 					label = decode_label(
-						bi_interior->extra->tag);
+						&bi_interior->extra->label[0]);
 				} else
 					prd = NULL;
 
@@ -1834,8 +1804,6 @@ static void vnc_import_bgp_exterior_add_route_it(
 					prefix, NULL, afi, prd, &new_attr,
 					ZEBRA_ROUTE_BGP_DIRECT_EXT,
 					BGP_ROUTE_REDISTRIBUTE, &label);
-
-				bgp_attr_extra_free(&new_attr);
 			}
 
 			if (have_usable_route) {
@@ -1973,7 +1941,7 @@ void vnc_import_bgp_exterior_del_route(
 			for (bi_interior = rn->info; bi_interior;
 			     bi_interior = bi_interior->next) {
 				struct prefix_rd *prd;
-				u_int32_t label = 0;
+				uint32_t label = 0;
 
 				if (!is_usable_interior_route(bi_interior))
 					continue;
@@ -1990,7 +1958,7 @@ void vnc_import_bgp_exterior_del_route(
 					prd = &bi_interior->extra->vnc.import
 						       .rd;
 					label = decode_label(
-						bi_interior->extra->tag);
+						&bi_interior->extra->label[0]);
 				} else
 					prd = NULL;
 
@@ -2102,11 +2070,9 @@ void vnc_import_bgp_exterior_add_route_interior(
 
 	/*debugging */
 	{
-		char str_pfx[BUFSIZ];
-
-		prefix2str(&rn_interior->p, str_pfx, BUFSIZ);
-		str_pfx[BUFSIZ - 1] = 0;
+		char str_pfx[PREFIX_STRLEN];
 
+		prefix2str(&rn_interior->p, str_pfx, sizeof(str_pfx));
 		vnc_zlog_debug_verbose("%s: interior prefix=%s, bi type=%d",
 				       __func__, str_pfx, bi_interior->type);
 	}
@@ -2136,7 +2102,7 @@ void vnc_import_bgp_exterior_add_route_interior(
 
 			struct prefix_rd *prd;
 			struct attr new_attr;
-			u_int32_t label = 0;
+			uint32_t label = 0;
 
 
 			++count; /* debugging */
@@ -2146,7 +2112,8 @@ void vnc_import_bgp_exterior_add_route_interior(
 
 			if (bi_interior->extra) {
 				prd = &bi_interior->extra->vnc.import.rd;
-				label = decode_label(bi_interior->extra->tag);
+				label = decode_label(
+					&bi_interior->extra->label[0]);
 			} else
 				prd = NULL;
 
@@ -2168,8 +2135,6 @@ void vnc_import_bgp_exterior_add_route_interior(
 				pfx_exterior, NULL, afi, prd, &new_attr,
 				ZEBRA_ROUTE_BGP_DIRECT_EXT,
 				BGP_ROUTE_REDISTRIBUTE, &label);
-
-			bgp_attr_extra_free(&new_attr);
 		}
 		vnc_zlog_debug_verbose(
 			"%s: finished constructing exteriors based on existing monitors",
@@ -2229,7 +2194,7 @@ void vnc_import_bgp_exterior_add_route_interior(
 				struct bgp_info *bi;
 				struct prefix_rd *prd;
 				struct attr new_attr;
-				u_int32_t label = 0;
+				uint32_t label = 0;
 
 				/* do pull-down */
 
@@ -2261,7 +2226,7 @@ void vnc_import_bgp_exterior_add_route_interior(
 					if (bi->extra) {
 						prd = &bi->extra->vnc.import.rd;
 						label = decode_label(
-							bi->extra->tag);
+							&bi->extra->label[0]);
 					} else
 						prd = NULL;
 
@@ -2283,7 +2248,7 @@ void vnc_import_bgp_exterior_add_route_interior(
 					prd = &bi_interior->extra->vnc.import
 						       .rd;
 					label = decode_label(
-						bi_interior->extra->tag);
+						&bi_interior->extra->label[0]);
 				} else
 					prd = NULL;
 
@@ -2305,8 +2270,6 @@ void vnc_import_bgp_exterior_add_route_interior(
 					pfx_exterior, NULL, afi, prd, &new_attr,
 					ZEBRA_ROUTE_BGP_DIRECT_EXT,
 					BGP_ROUTE_REDISTRIBUTE, &label);
-
-				bgp_attr_extra_free(&new_attr);
 			}
 		}
 
@@ -2350,11 +2313,10 @@ void vnc_import_bgp_exterior_add_route_interior(
 				     (void **)&pfx_exterior, &cursor)) {
 
 		struct prefix pfx_nexthop;
-		char buf[BUFSIZ];
+		char buf[PREFIX_STRLEN];
 		afi_t afi_exterior = family2afi(pfx_exterior->family);
 
 		prefix2str(pfx_exterior, buf, sizeof(buf));
-		buf[sizeof(buf) - 1] = 0;
 		vnc_zlog_debug_verbose(
 			"%s: checking exterior orphan at prefix %s", __func__,
 			buf);
@@ -2374,7 +2336,7 @@ void vnc_import_bgp_exterior_add_route_interior(
 
 			struct prefix_rd *prd;
 			struct attr new_attr;
-			u_int32_t label = 0;
+			uint32_t label = 0;
 
 			/* do pull-down */
 
@@ -2406,7 +2368,8 @@ void vnc_import_bgp_exterior_add_route_interior(
 			 */
 			if (bi_interior->extra) {
 				prd = &bi_interior->extra->vnc.import.rd;
-				label = decode_label(bi_interior->extra->tag);
+				label = decode_label(
+					&bi_interior->extra->label[0]);
 			} else
 				prd = NULL;
 
@@ -2428,8 +2391,6 @@ void vnc_import_bgp_exterior_add_route_interior(
 				pfx_exterior, NULL, afi, prd, &new_attr,
 				ZEBRA_ROUTE_BGP_DIRECT_EXT,
 				BGP_ROUTE_REDISTRIBUTE, &label);
-
-			bgp_attr_extra_free(&new_attr);
 		}
 	}
 	if (list_adopted) {
@@ -2440,7 +2401,7 @@ void vnc_import_bgp_exterior_add_route_interior(
 			skiplist_delete(it->monitor_exterior_orphans,
 					bi_exterior, NULL);
 		}
-		list_delete(list_adopted);
+		list_delete_and_null(&list_adopted);
 	}
 }
 
@@ -2494,10 +2455,9 @@ void vnc_import_bgp_exterior_del_route_interior(
 
 	/*debugging */
 	{
-		char str_pfx[BUFSIZ];
+		char str_pfx[PREFIX_STRLEN];
 
-		prefix2str(&rn_interior->p, str_pfx, BUFSIZ);
-		str_pfx[BUFSIZ - 1] = 0;
+		prefix2str(&rn_interior->p, str_pfx, sizeof(str_pfx));
 
 		vnc_zlog_debug_verbose("%s: interior prefix=%s, bi type=%d",
 				       __func__, str_pfx, bi_interior->type);
@@ -2516,11 +2476,11 @@ void vnc_import_bgp_exterior_del_route_interior(
 				&cursor)) {
 
 		struct prefix_rd *prd;
-		u_int32_t label = 0;
+		uint32_t label = 0;
 
 		if (bi_interior->extra) {
 			prd = &bi_interior->extra->vnc.import.rd;
-			label = decode_label(bi_interior->extra->tag);
+			label = decode_label(&bi_interior->extra->label[0]);
 		} else
 			prd = NULL;
 
@@ -2589,14 +2549,15 @@ void vnc_import_bgp_exterior_del_route_interior(
 
 				struct prefix_rd *prd;
 				struct attr new_attr;
-				u_int32_t label = 0;
+				uint32_t label = 0;
 
 				if (bi->type == ZEBRA_ROUTE_BGP_DIRECT_EXT)
 					continue;
 
 				if (bi->extra) {
 					prd = &bi->extra->vnc.import.rd;
-					label = decode_label(bi->extra->tag);
+					label = decode_label(
+						&bi->extra->label[0]);
 				} else
 					prd = NULL;
 
@@ -2618,8 +2579,6 @@ void vnc_import_bgp_exterior_del_route_interior(
 					pfx_exterior, NULL, afi, prd, &new_attr,
 					ZEBRA_ROUTE_BGP_DIRECT_EXT,
 					BGP_ROUTE_REDISTRIBUTE, &label);
-
-				bgp_attr_extra_free(&new_attr);
 			}
 
 		} else {
@@ -2653,14 +2612,14 @@ void vnc_import_bgp_add_route(struct bgp *bgp, struct prefix *prefix,
 {
 	afi_t afi = family2afi(prefix->family);
 
-	{
+	if (VNC_DEBUG(VERBOSE)) {
 		struct prefix pfx_nexthop;
-		char buf[BUFSIZ];
-		char buf_nh[BUFSIZ];
+		char buf[PREFIX_STRLEN];
+		char buf_nh[PREFIX_STRLEN];
 
-		prefix2str(prefix, buf, BUFSIZ);
+		prefix2str(prefix, buf, sizeof(buf));
 		rfapiUnicastNexthop2Prefix(afi, info->attr, &pfx_nexthop);
-		prefix2str(&pfx_nexthop, buf_nh, BUFSIZ);
+		prefix2str(&pfx_nexthop, buf_nh, sizeof(buf_nh));
 
 		vnc_zlog_debug_verbose("%s: pfx %s, nh %s", __func__, buf,
 				       buf_nh);
@@ -2725,12 +2684,12 @@ void vnc_import_bgp_del_route(struct bgp *bgp, struct prefix *prefix,
 
 	{
 		struct prefix pfx_nexthop;
-		char buf[BUFSIZ];
-		char buf_nh[BUFSIZ];
+		char buf[PREFIX_STRLEN];
+		char buf_nh[PREFIX_STRLEN];
 
-		prefix2str(prefix, buf, BUFSIZ);
+		prefix2str(prefix, buf, sizeof(buf));
 		rfapiUnicastNexthop2Prefix(afi, info->attr, &pfx_nexthop);
-		prefix2str(&pfx_nexthop, buf_nh, BUFSIZ);
+		prefix2str(&pfx_nexthop, buf_nh, sizeof(buf_nh));
 
 		vnc_zlog_debug_verbose("%s: pfx %s, nh %s", __func__, buf,
 				       buf_nh);
