@@ -14,10 +14,10 @@
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
-
+ *
  * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <zebra.h>
 #ifdef GNU_LINUX
@@ -26,10 +26,6 @@
 #include <netinet/if_ether.h>
 #endif
 
-#ifndef ETHER_ADDR_LEN
-#define	ETHER_ADDR_LEN	ETHERADDRL
-#endif
-
 #include "log.h"
 #include "memory.h"
 #include "vrf.h"
@@ -48,8 +44,8 @@
 #include "isisd/isis_common.h"
 #include "isisd/isis_flags.h"
 #include "isisd/isis_circuit.h"
-#include "isisd/isis_tlv.h"
 #include "isisd/isis_lsp.h"
+#include "isisd/isis_lsp_hash.h"
 #include "isisd/isis_pdu.h"
 #include "isisd/isis_network.h"
 #include "isisd/isis_misc.h"
@@ -60,6 +56,7 @@
 #include "isisd/isis_csm.h"
 #include "isisd/isis_events.h"
 #include "isisd/isis_te.h"
+#include "isisd/isis_mt.h"
 
 DEFINE_QOBJ_TYPE(isis_circuit)
 
@@ -99,6 +96,8 @@ struct isis_circuit *isis_circuit_new()
 
 	circuit->mtc = mpls_te_circuit_new();
 
+	circuit_mt_init(circuit);
+
 	QOBJ_REG(circuit, isis_circuit);
 
 	return circuit;
@@ -113,6 +112,8 @@ void isis_circuit_del(struct isis_circuit *circuit)
 
 	isis_circuit_if_unbind(circuit, circuit->interface);
 
+	circuit_mt_finish(circuit);
+
 	/* and lastly the circuit itself */
 	XFREE(MTYPE_ISIS_CIRCUIT, circuit);
 
@@ -208,7 +209,7 @@ void isis_circuit_add_addr(struct isis_circuit *circuit,
 	struct prefix_ipv6 *ipv6;
 
 	if (connected->address->family == AF_INET) {
-		u_int32_t addr = connected->address->u.prefix4.s_addr;
+		uint32_t addr = connected->address->u.prefix4.s_addr;
 		addr = ntohl(addr);
 		if (IPV4_NET0(addr) || IPV4_NET127(addr) || IN_CLASSD(addr)
 		    || IPV4_LINKLOCAL(addr))
@@ -233,8 +234,8 @@ void isis_circuit_add_addr(struct isis_circuit *circuit,
 
 #ifdef EXTREME_DEBUG
 		prefix2str(connected->address, buf, sizeof(buf));
-		zlog_debug("Added IP address %s to circuit %d", buf,
-			   circuit->circuit_id);
+		zlog_debug("Added IP address %s to circuit %s", buf,
+			   circuit->interface->name);
 #endif /* EXTREME_DEBUG */
 	}
 	if (connected->address->family == AF_INET6) {
@@ -264,8 +265,8 @@ void isis_circuit_add_addr(struct isis_circuit *circuit,
 
 #ifdef EXTREME_DEBUG
 		prefix2str(connected->address, buf, sizeof(buf));
-		zlog_debug("Added IPv6 address %s to circuit %d", buf,
-			   circuit->circuit_id);
+		zlog_debug("Added IPv6 address %s to circuit %s", buf,
+			   circuit->interface->name);
 #endif /* EXTREME_DEBUG */
 	}
 	return;
@@ -299,9 +300,9 @@ void isis_circuit_del_addr(struct isis_circuit *circuit,
 		} else {
 			prefix2str(connected->address, buf, sizeof(buf));
 			zlog_warn(
-				"Nonexistant ip address %s removal attempt from \
-                      circuit %d",
-				buf, circuit->circuit_id);
+				"Nonexistent ip address %s removal attempt from \
+                      circuit %s",
+				buf, circuit->interface->name);
 			zlog_warn("Current ip addresses on %s:",
 				  circuit->interface->name);
 			for (ALL_LIST_ELEMENTS_RO(circuit->ip_addrs, node,
@@ -348,9 +349,9 @@ void isis_circuit_del_addr(struct isis_circuit *circuit,
 		if (!found) {
 			prefix2str(connected->address, buf, sizeof(buf));
 			zlog_warn(
-				"Nonexitant ip address %s removal attempt from \
-		      circuit %d",
-				buf, circuit->circuit_id);
+				"Nonexistent ip address %s removal attempt from \
+		      circuit %s",
+				buf, circuit->interface->name);
 			zlog_warn("Current ip addresses on %s:",
 				  circuit->interface->name);
 			for (ALL_LIST_ELEMENTS_RO(circuit->ipv6_link, node,
@@ -376,7 +377,7 @@ void isis_circuit_del_addr(struct isis_circuit *circuit,
 	return;
 }
 
-static uint8_t isis_circuit_id_gen(struct interface *ifp)
+static uint8_t isis_circuit_id_gen(struct isis *isis, struct interface *ifp)
 {
 	/* Circuit ids MUST be unique for any broadcast circuits. Otherwise,
 	 * Pseudo-Node LSPs cannot be generated correctly.
@@ -397,10 +398,12 @@ static uint8_t isis_circuit_id_gen(struct interface *ifp)
 	}
 
 	if (i == 256) {
-		zlog_warn("Could not allocate a circuit id for '%s'", ifp->name);
+		zlog_warn("Could not allocate a circuit id for '%s'",
+			  ifp->name);
 		return 0;
 	}
 
+	_ISIS_SET_FLAG(isis->circuit_ids_used, id);
 	return id;
 }
 
@@ -409,11 +412,7 @@ void isis_circuit_if_add(struct isis_circuit *circuit, struct interface *ifp)
 	struct listnode *node, *nnode;
 	struct connected *conn;
 
-	circuit->circuit_id = isis_circuit_id_gen(ifp);
-	_ISIS_SET_FLAG(isis->circuit_ids_used, circuit->circuit_id);
-
 	isis_circuit_if_bind(circuit, ifp);
-	/*  isis_circuit_update_addrs (circuit, ifp); */
 
 	if (if_is_broadcast(ifp)) {
 		if (circuit->circ_type_config == CIRCUIT_T_P2P)
@@ -438,8 +437,6 @@ void isis_circuit_if_add(struct isis_circuit *circuit, struct interface *ifp)
 
 	for (ALL_LIST_ELEMENTS(ifp->connected, node, nnode, conn))
 		isis_circuit_add_addr(circuit, conn);
-
-	return;
 }
 
 void isis_circuit_if_del(struct isis_circuit *circuit, struct interface *ifp)
@@ -455,27 +452,20 @@ void isis_circuit_if_del(struct isis_circuit *circuit, struct interface *ifp)
 
 	if (circuit->ip_addrs) {
 		assert(listcount(circuit->ip_addrs) == 0);
-		list_delete(circuit->ip_addrs);
-		circuit->ip_addrs = NULL;
+		list_delete_and_null(&circuit->ip_addrs);
 	}
 
 	if (circuit->ipv6_link) {
 		assert(listcount(circuit->ipv6_link) == 0);
-		list_delete(circuit->ipv6_link);
-		circuit->ipv6_link = NULL;
+		list_delete_and_null(&circuit->ipv6_link);
 	}
 
 	if (circuit->ipv6_non_link) {
 		assert(listcount(circuit->ipv6_non_link) == 0);
-		list_delete(circuit->ipv6_non_link);
-		circuit->ipv6_non_link = NULL;
+		list_delete_and_null(&circuit->ipv6_non_link);
 	}
 
 	circuit->circ_type = CIRCUIT_T_UNKNOWN;
-	_ISIS_CLEAR_FLAG(isis->circuit_ids_used, circuit->circuit_id);
-	circuit->circuit_id = 0;
-
-	return;
 }
 
 void isis_circuit_if_bind(struct isis_circuit *circuit, struct interface *ifp)
@@ -556,12 +546,13 @@ void isis_circuit_stream(struct isis_circuit *circuit, struct stream **stream)
 
 void isis_circuit_prepare(struct isis_circuit *circuit)
 {
-#ifdef GNU_LINUX
-	THREAD_READ_ON(master, circuit->t_read, isis_receive, circuit,
-		       circuit->fd);
+#if ISIS_METHOD != ISIS_METHOD_DLPI
+	thread_add_read(master, isis_receive, circuit, circuit->fd,
+			&circuit->t_read);
 #else
-	THREAD_TIMER_MSEC_ON(master, circuit->t_read, isis_receive, circuit,
-			     listcount(circuit->area->circuit_list) * 100);
+	thread_add_timer_msec(master, isis_receive, circuit,
+			      listcount(circuit->area->circuit_list) * 100,
+			      &circuit->t_read);
 #endif
 }
 
@@ -588,6 +579,12 @@ int isis_circuit_up(struct isis_circuit *circuit)
 	}
 
 	if (circuit->circ_type == CIRCUIT_T_BROADCAST) {
+		circuit->circuit_id = isis_circuit_id_gen(isis, circuit->interface);
+		if (!circuit->circuit_id) {
+			zlog_err("There are already 255 broadcast circuits active!");
+			return ISIS_ERROR;
+		}
+
 		/*
 		 * Get the Hardware Address
 		 */
@@ -617,12 +614,14 @@ int isis_circuit_up(struct isis_circuit *circuit)
 		/* 8.4.1 a) commence sending of IIH PDUs */
 
 		if (circuit->is_type & IS_LEVEL_1) {
-			thread_add_event(master, send_lan_l1_hello, circuit, 0);
+			thread_add_event(master, send_lan_l1_hello, circuit, 0,
+					 NULL);
 			circuit->u.bc.lan_neighs[0] = list_new();
 		}
 
 		if (circuit->is_type & IS_LEVEL_2) {
-			thread_add_event(master, send_lan_l2_hello, circuit, 0);
+			thread_add_event(master, send_lan_l2_hello, circuit, 0,
+					 NULL);
 			circuit->u.bc.lan_neighs[1] = list_new();
 		}
 
@@ -632,31 +631,33 @@ int isis_circuit_up(struct isis_circuit *circuit)
 		/* 8.4.1 d) */
 		/* dr election will commence in... */
 		if (circuit->is_type & IS_LEVEL_1)
-			THREAD_TIMER_ON(master, circuit->u.bc.t_run_dr[0],
-					isis_run_dr_l1, circuit,
-					2 * circuit->hello_interval[0]);
+			thread_add_timer(master, isis_run_dr_l1, circuit,
+					 2 * circuit->hello_interval[0],
+					 &circuit->u.bc.t_run_dr[0]);
 		if (circuit->is_type & IS_LEVEL_2)
-			THREAD_TIMER_ON(master, circuit->u.bc.t_run_dr[1],
-					isis_run_dr_l2, circuit,
-					2 * circuit->hello_interval[1]);
-	} else {
+			thread_add_timer(master, isis_run_dr_l2, circuit,
+					 2 * circuit->hello_interval[1],
+					 &circuit->u.bc.t_run_dr[1]);
+	} else if (circuit->circ_type == CIRCUIT_T_P2P) {
 		/* initializing the hello send threads
 		 * for a ptp IF
 		 */
 		circuit->u.p2p.neighbor = NULL;
-		thread_add_event(master, send_p2p_hello, circuit, 0);
+		thread_add_event(master, send_p2p_hello, circuit, 0, NULL);
 	}
 
 	/* initializing PSNP timers */
 	if (circuit->is_type & IS_LEVEL_1)
-		THREAD_TIMER_ON(
-			master, circuit->t_send_psnp[0], send_l1_psnp, circuit,
-			isis_jitter(circuit->psnp_interval[0], PSNP_JITTER));
+		thread_add_timer(
+			master, send_l1_psnp, circuit,
+			isis_jitter(circuit->psnp_interval[0], PSNP_JITTER),
+			&circuit->t_send_psnp[0]);
 
 	if (circuit->is_type & IS_LEVEL_2)
-		THREAD_TIMER_ON(
-			master, circuit->t_send_psnp[1], send_l2_psnp, circuit,
-			isis_jitter(circuit->psnp_interval[1], PSNP_JITTER));
+		thread_add_timer(
+			master, send_l2_psnp, circuit,
+			isis_jitter(circuit->psnp_interval[1], PSNP_JITTER),
+			&circuit->t_send_psnp[1]);
 
 	/* unified init for circuits; ignore warnings below this level */
 	retv = isis_sock_init(circuit);
@@ -672,38 +673,37 @@ int isis_circuit_up(struct isis_circuit *circuit)
 	isis_circuit_prepare(circuit);
 
 	circuit->lsp_queue = list_new();
-	circuit->lsp_queue_last_cleared = time(NULL);
+	circuit->lsp_hash = isis_lsp_hash_new();
+	circuit->lsp_queue_last_push[0] = circuit->lsp_queue_last_push[1] =
+		monotime(NULL);
 
 	return ISIS_OK;
 }
 
 void isis_circuit_down(struct isis_circuit *circuit)
 {
-	if (circuit->state != C_STATE_UP)
-		return;
-
 	/* Clear the flags for all the lsps of the circuit. */
 	isis_circuit_update_all_srmflags(circuit, 0);
 
 	if (circuit->circ_type == CIRCUIT_T_BROADCAST) {
 		/* destroy neighbour lists */
 		if (circuit->u.bc.lan_neighs[0]) {
-			list_delete(circuit->u.bc.lan_neighs[0]);
+			list_delete_and_null(&circuit->u.bc.lan_neighs[0]);
 			circuit->u.bc.lan_neighs[0] = NULL;
 		}
 		if (circuit->u.bc.lan_neighs[1]) {
-			list_delete(circuit->u.bc.lan_neighs[1]);
+			list_delete_and_null(&circuit->u.bc.lan_neighs[1]);
 			circuit->u.bc.lan_neighs[1] = NULL;
 		}
 		/* destroy adjacency databases */
 		if (circuit->u.bc.adjdb[0]) {
 			circuit->u.bc.adjdb[0]->del = isis_delete_adj;
-			list_delete(circuit->u.bc.adjdb[0]);
+			list_delete_and_null(&circuit->u.bc.adjdb[0]);
 			circuit->u.bc.adjdb[0] = NULL;
 		}
 		if (circuit->u.bc.adjdb[1]) {
 			circuit->u.bc.adjdb[1]->del = isis_delete_adj;
-			list_delete(circuit->u.bc.adjdb[1]);
+			list_delete_and_null(&circuit->u.bc.adjdb[1]);
 			circuit->u.bc.adjdb[1] = NULL;
 		}
 		if (circuit->u.bc.is_dr[0]) {
@@ -726,6 +726,9 @@ void isis_circuit_down(struct isis_circuit *circuit)
 		THREAD_TIMER_OFF(circuit->u.bc.t_refresh_pseudo_lsp[1]);
 		circuit->lsp_regenerate_pending[0] = 0;
 		circuit->lsp_regenerate_pending[1] = 0;
+
+		_ISIS_CLEAR_FLAG(isis->circuit_ids_used, circuit->circuit_id);
+		circuit->circuit_id = 0;
 	} else if (circuit->circ_type == CIRCUIT_T_P2P) {
 		isis_delete_adj(circuit->u.p2p.neighbor);
 		circuit->u.p2p.neighbor = NULL;
@@ -737,19 +740,25 @@ void isis_circuit_down(struct isis_circuit *circuit)
 	THREAD_TIMER_OFF(circuit->t_send_csnp[1]);
 	THREAD_TIMER_OFF(circuit->t_send_psnp[0]);
 	THREAD_TIMER_OFF(circuit->t_send_psnp[1]);
+	THREAD_OFF(circuit->t_send_lsp);
 	THREAD_OFF(circuit->t_read);
 
 	if (circuit->lsp_queue) {
-		circuit->lsp_queue->del = NULL;
-		list_delete(circuit->lsp_queue);
-		circuit->lsp_queue = NULL;
+		list_delete_and_null(&circuit->lsp_queue);
+	}
+
+	if (circuit->lsp_hash) {
+		isis_lsp_hash_free(circuit->lsp_hash);
+		circuit->lsp_hash = NULL;
 	}
 
 	/* send one gratuitous hello to spead up convergence */
-	if (circuit->is_type & IS_LEVEL_1)
-		send_hello(circuit, IS_LEVEL_1);
-	if (circuit->is_type & IS_LEVEL_2)
-		send_hello(circuit, IS_LEVEL_2);
+	if (circuit->state == C_STATE_UP) {
+		if (circuit->is_type & IS_LEVEL_1)
+			send_hello(circuit, IS_LEVEL_1);
+		if (circuit->is_type & IS_LEVEL_2)
+			send_hello(circuit, IS_LEVEL_2);
+	}
 
 	circuit->upadjcount[0] = 0;
 	circuit->upadjcount[1] = 0;
@@ -799,7 +808,7 @@ void isis_circuit_print_vty(struct isis_circuit *circuit, struct vty *vty,
 		vty_out(vty, "%-9s", circuit_state2string(circuit->state));
 		vty_out(vty, "%-9s", circuit_type2string(circuit->circ_type));
 		vty_out(vty, "%-9s", circuit_t2string(circuit->is_type));
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "\n");
 	}
 
 	if (detail == ISIS_UI_LEVEL_DETAIL) {
@@ -815,16 +824,16 @@ void isis_circuit_print_vty(struct isis_circuit *circuit, struct vty *vty,
 		else
 			vty_out(vty, ", Active");
 		vty_out(vty, ", Circuit Id: 0x%x", circuit->circuit_id);
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "\n");
 		vty_out(vty, "    Type: %s",
 			circuit_type2string(circuit->circ_type));
 		vty_out(vty, ", Level: %s", circuit_t2string(circuit->is_type));
 		if (circuit->circ_type == CIRCUIT_T_BROADCAST)
 			vty_out(vty, ", SNPA: %-10s",
 				snpa_print(circuit->u.bc.snpa));
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "\n");
 		if (circuit->is_type & IS_LEVEL_1) {
-			vty_out(vty, "    Level-1 Information:%s", VTY_NEWLINE);
+			vty_out(vty, "    Level-1 Information:\n");
 			if (circuit->area->newmetric)
 				vty_out(vty, "      Metric: %d",
 					circuit->te_metric[0]);
@@ -832,35 +841,33 @@ void isis_circuit_print_vty(struct isis_circuit *circuit, struct vty *vty,
 				vty_out(vty, "      Metric: %d",
 					circuit->metric[0]);
 			if (!circuit->is_passive) {
-				vty_out(vty, ", Active neighbors: %u%s",
-					circuit->upadjcount[0], VTY_NEWLINE);
+				vty_out(vty, ", Active neighbors: %u\n",
+					circuit->upadjcount[0]);
 				vty_out(vty,
 					"      Hello interval: %u, "
-					"Holddown count: %u %s%s",
+					"Holddown count: %u %s\n",
 					circuit->hello_interval[0],
 					circuit->hello_multiplier[0],
 					(circuit->pad_hellos ? "(pad)"
-							     : "(no-pad)"),
-					VTY_NEWLINE);
+							     : "(no-pad)"));
 				vty_out(vty,
 					"      CNSP interval: %u, "
-					"PSNP interval: %u%s",
+					"PSNP interval: %u\n",
 					circuit->csnp_interval[0],
-					circuit->psnp_interval[0], VTY_NEWLINE);
+					circuit->psnp_interval[0]);
 				if (circuit->circ_type == CIRCUIT_T_BROADCAST)
 					vty_out(vty,
-						"      LAN Priority: %u, %s%s",
+						"      LAN Priority: %u, %s\n",
 						circuit->priority[0],
 						(circuit->u.bc.is_dr[0]
 							 ? "is DIS"
-							 : "is not DIS"),
-						VTY_NEWLINE);
+							 : "is not DIS"));
 			} else {
-				vty_out(vty, "%s", VTY_NEWLINE);
+				vty_out(vty, "\n");
 			}
 		}
 		if (circuit->is_type & IS_LEVEL_2) {
-			vty_out(vty, "    Level-2 Information:%s", VTY_NEWLINE);
+			vty_out(vty, "    Level-2 Information:\n");
 			if (circuit->area->newmetric)
 				vty_out(vty, "      Metric: %d",
 					circuit->te_metric[1]);
@@ -868,125 +875,114 @@ void isis_circuit_print_vty(struct isis_circuit *circuit, struct vty *vty,
 				vty_out(vty, "      Metric: %d",
 					circuit->metric[1]);
 			if (!circuit->is_passive) {
-				vty_out(vty, ", Active neighbors: %u%s",
-					circuit->upadjcount[1], VTY_NEWLINE);
+				vty_out(vty, ", Active neighbors: %u\n",
+					circuit->upadjcount[1]);
 				vty_out(vty,
 					"      Hello interval: %u, "
-					"Holddown count: %u %s%s",
+					"Holddown count: %u %s\n",
 					circuit->hello_interval[1],
 					circuit->hello_multiplier[1],
 					(circuit->pad_hellos ? "(pad)"
-							     : "(no-pad)"),
-					VTY_NEWLINE);
+							     : "(no-pad)"));
 				vty_out(vty,
 					"      CNSP interval: %u, "
-					"PSNP interval: %u%s",
+					"PSNP interval: %u\n",
 					circuit->csnp_interval[1],
-					circuit->psnp_interval[1], VTY_NEWLINE);
+					circuit->psnp_interval[1]);
 				if (circuit->circ_type == CIRCUIT_T_BROADCAST)
 					vty_out(vty,
-						"      LAN Priority: %u, %s%s",
+						"      LAN Priority: %u, %s\n",
 						circuit->priority[1],
 						(circuit->u.bc.is_dr[1]
 							 ? "is DIS"
-							 : "is not DIS"),
-						VTY_NEWLINE);
+							 : "is not DIS"));
 			} else {
-				vty_out(vty, "%s", VTY_NEWLINE);
+				vty_out(vty, "\n");
 			}
 		}
 		if (circuit->ip_addrs && listcount(circuit->ip_addrs) > 0) {
-			vty_out(vty, "    IP Prefix(es):%s", VTY_NEWLINE);
+			vty_out(vty, "    IP Prefix(es):\n");
 			for (ALL_LIST_ELEMENTS_RO(circuit->ip_addrs, node,
 						  ip_addr)) {
 				prefix2str(ip_addr, buf, sizeof(buf)),
-					vty_out(vty, "      %s%s", buf,
-						VTY_NEWLINE);
+					vty_out(vty, "      %s\n", buf);
 			}
 		}
 		if (circuit->ipv6_link && listcount(circuit->ipv6_link) > 0) {
-			vty_out(vty, "    IPv6 Link-Locals:%s", VTY_NEWLINE);
+			vty_out(vty, "    IPv6 Link-Locals:\n");
 			for (ALL_LIST_ELEMENTS_RO(circuit->ipv6_link, node,
 						  ip_addr)) {
 				prefix2str(ip_addr, (char *)buf, BUFSIZ),
-					vty_out(vty, "      %s%s", buf,
-						VTY_NEWLINE);
+					vty_out(vty, "      %s\n", buf);
 			}
 		}
 		if (circuit->ipv6_non_link
 		    && listcount(circuit->ipv6_non_link) > 0) {
-			vty_out(vty, "    IPv6 Prefixes:%s", VTY_NEWLINE);
+			vty_out(vty, "    IPv6 Prefixes:\n");
 			for (ALL_LIST_ELEMENTS_RO(circuit->ipv6_non_link, node,
 						  ip_addr)) {
 				prefix2str(ip_addr, (char *)buf, BUFSIZ),
-					vty_out(vty, "      %s%s", buf,
-						VTY_NEWLINE);
+					vty_out(vty, "      %s\n", buf);
 			}
 		}
 
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "\n");
 	}
 	return;
 }
 
 int isis_interface_config_write(struct vty *vty)
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	int write = 0;
-	struct listnode *node, *node2;
+	struct listnode *node;
 	struct interface *ifp;
 	struct isis_area *area;
 	struct isis_circuit *circuit;
 	int i;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp)) {
-		if (ifp->ifindex == IFINDEX_DELETED)
-			continue;
-
+	FOR_ALL_INTERFACES (vrf, ifp) {
 		/* IF name */
-		vty_out(vty, "interface %s%s", ifp->name, VTY_NEWLINE);
+		vty_frame(vty, "interface %s\n", ifp->name);
 		write++;
 		/* IF desc */
 		if (ifp->desc) {
-			vty_out(vty, " description %s%s", ifp->desc,
-				VTY_NEWLINE);
+			vty_out(vty, " description %s\n", ifp->desc);
 			write++;
 		}
 		/* ISIS Circuit */
-		for (ALL_LIST_ELEMENTS_RO(isis->area_list, node2, area)) {
+		for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area)) {
 			circuit =
 				circuit_lookup_by_ifp(ifp, area->circuit_list);
 			if (circuit == NULL)
 				continue;
 			if (circuit->ip_router) {
-				vty_out(vty, " ip router isis %s%s",
-					area->area_tag, VTY_NEWLINE);
+				vty_out(vty, " ip router isis %s\n",
+					area->area_tag);
 				write++;
 			}
 			if (circuit->is_passive) {
-				vty_out(vty, " isis passive%s", VTY_NEWLINE);
+				vty_out(vty, " isis passive\n");
 				write++;
 			}
 			if (circuit->circ_type_config == CIRCUIT_T_P2P) {
-				vty_out(vty, " isis network point-to-point%s",
-					VTY_NEWLINE);
+				vty_out(vty, " isis network point-to-point\n");
 				write++;
 			}
 			if (circuit->ipv6_router) {
-				vty_out(vty, " ipv6 router isis %s%s",
-					area->area_tag, VTY_NEWLINE);
+				vty_out(vty, " ipv6 router isis %s\n",
+					area->area_tag);
 				write++;
 			}
 
 			/* ISIS - circuit type */
 			if (circuit->is_type == IS_LEVEL_1) {
-				vty_out(vty, " isis circuit-type level-1%s",
-					VTY_NEWLINE);
+				vty_out(vty, " isis circuit-type level-1\n");
 				write++;
 			} else {
 				if (circuit->is_type == IS_LEVEL_2) {
 					vty_out(vty,
-						" isis circuit-type level-2-only%s",
-						VTY_NEWLINE);
+						" isis circuit-type level-2-only\n");
 					write++;
 				}
 			}
@@ -996,9 +992,8 @@ int isis_interface_config_write(struct vty *vty)
 			    == circuit->csnp_interval[1]) {
 				if (circuit->csnp_interval[0]
 				    != DEFAULT_CSNP_INTERVAL) {
-					vty_out(vty, " isis csnp-interval %d%s",
-						circuit->csnp_interval[0],
-						VTY_NEWLINE);
+					vty_out(vty, " isis csnp-interval %d\n",
+						circuit->csnp_interval[0]);
 					write++;
 				}
 			} else {
@@ -1006,10 +1001,10 @@ int isis_interface_config_write(struct vty *vty)
 					if (circuit->csnp_interval[i]
 					    != DEFAULT_CSNP_INTERVAL) {
 						vty_out(vty,
-							" isis csnp-interval %d level-%d%s",
+							" isis csnp-interval %d level-%d\n",
 							circuit->csnp_interval
 								[i],
-							i + 1, VTY_NEWLINE);
+							i + 1);
 						write++;
 					}
 				}
@@ -1020,9 +1015,8 @@ int isis_interface_config_write(struct vty *vty)
 			    == circuit->psnp_interval[1]) {
 				if (circuit->psnp_interval[0]
 				    != DEFAULT_PSNP_INTERVAL) {
-					vty_out(vty, " isis psnp-interval %d%s",
-						circuit->psnp_interval[0],
-						VTY_NEWLINE);
+					vty_out(vty, " isis psnp-interval %d\n",
+						circuit->psnp_interval[0]);
 					write++;
 				}
 			} else {
@@ -1030,10 +1024,10 @@ int isis_interface_config_write(struct vty *vty)
 					if (circuit->psnp_interval[i]
 					    != DEFAULT_PSNP_INTERVAL) {
 						vty_out(vty,
-							" isis psnp-interval %d level-%d%s",
+							" isis psnp-interval %d level-%d\n",
 							circuit->psnp_interval
 								[i],
-							i + 1, VTY_NEWLINE);
+							i + 1);
 						write++;
 					}
 				}
@@ -1042,8 +1036,12 @@ int isis_interface_config_write(struct vty *vty)
 			/* ISIS - Hello padding - Defaults to true so only
 			 * display if false */
 			if (circuit->pad_hellos == 0) {
-				vty_out(vty, " no isis hello padding%s",
-					VTY_NEWLINE);
+				vty_out(vty, " no isis hello padding\n");
+				write++;
+			}
+
+			if (circuit->disable_threeway_adj) {
+				vty_out(vty, " no isis three-way-handshake\n");
 				write++;
 			}
 
@@ -1053,9 +1051,8 @@ int isis_interface_config_write(struct vty *vty)
 				if (circuit->hello_interval[0]
 				    != DEFAULT_HELLO_INTERVAL) {
 					vty_out(vty,
-						" isis hello-interval %d%s",
-						circuit->hello_interval[0],
-						VTY_NEWLINE);
+						" isis hello-interval %d\n",
+						circuit->hello_interval[0]);
 					write++;
 				}
 			} else {
@@ -1063,10 +1060,10 @@ int isis_interface_config_write(struct vty *vty)
 					if (circuit->hello_interval[i]
 					    != DEFAULT_HELLO_INTERVAL) {
 						vty_out(vty,
-							" isis hello-interval %d level-%d%s",
+							" isis hello-interval %d level-%d\n",
 							circuit->hello_interval
 								[i],
-							i + 1, VTY_NEWLINE);
+							i + 1);
 						write++;
 					}
 				}
@@ -1078,9 +1075,8 @@ int isis_interface_config_write(struct vty *vty)
 				if (circuit->hello_multiplier[0]
 				    != DEFAULT_HELLO_MULTIPLIER) {
 					vty_out(vty,
-						" isis hello-multiplier %d%s",
-						circuit->hello_multiplier[0],
-						VTY_NEWLINE);
+						" isis hello-multiplier %d\n",
+						circuit->hello_multiplier[0]);
 					write++;
 				}
 			} else {
@@ -1088,10 +1084,10 @@ int isis_interface_config_write(struct vty *vty)
 					if (circuit->hello_multiplier[i]
 					    != DEFAULT_HELLO_MULTIPLIER) {
 						vty_out(vty,
-							" isis hello-multiplier %d level-%d%s",
+							" isis hello-multiplier %d level-%d\n",
 							circuit->hello_multiplier
 								[i],
-							i + 1, VTY_NEWLINE);
+							i + 1);
 						write++;
 					}
 				}
@@ -1100,9 +1096,8 @@ int isis_interface_config_write(struct vty *vty)
 			/* ISIS - Priority */
 			if (circuit->priority[0] == circuit->priority[1]) {
 				if (circuit->priority[0] != DEFAULT_PRIORITY) {
-					vty_out(vty, " isis priority %d%s",
-						circuit->priority[0],
-						VTY_NEWLINE);
+					vty_out(vty, " isis priority %d\n",
+						circuit->priority[0]);
 					write++;
 				}
 			} else {
@@ -1110,9 +1105,9 @@ int isis_interface_config_write(struct vty *vty)
 					if (circuit->priority[i]
 					    != DEFAULT_PRIORITY) {
 						vty_out(vty,
-							" isis priority %d level-%d%s",
+							" isis priority %d level-%d\n",
 							circuit->priority[i],
-							i + 1, VTY_NEWLINE);
+							i + 1);
 						write++;
 					}
 				}
@@ -1122,9 +1117,8 @@ int isis_interface_config_write(struct vty *vty)
 			if (circuit->te_metric[0] == circuit->te_metric[1]) {
 				if (circuit->te_metric[0]
 				    != DEFAULT_CIRCUIT_METRIC) {
-					vty_out(vty, " isis metric %d%s",
-						circuit->te_metric[0],
-						VTY_NEWLINE);
+					vty_out(vty, " isis metric %d\n",
+						circuit->te_metric[0]);
 					write++;
 				}
 			} else {
@@ -1132,25 +1126,26 @@ int isis_interface_config_write(struct vty *vty)
 					if (circuit->te_metric[i]
 					    != DEFAULT_CIRCUIT_METRIC) {
 						vty_out(vty,
-							" isis metric %d level-%d%s",
+							" isis metric %d level-%d\n",
 							circuit->te_metric[i],
-							i + 1, VTY_NEWLINE);
+							i + 1);
 						write++;
 					}
 				}
 			}
 			if (circuit->passwd.type == ISIS_PASSWD_TYPE_HMAC_MD5) {
-				vty_out(vty, " isis password md5 %s%s",
-					circuit->passwd.passwd, VTY_NEWLINE);
+				vty_out(vty, " isis password md5 %s\n",
+					circuit->passwd.passwd);
 				write++;
 			} else if (circuit->passwd.type
 				   == ISIS_PASSWD_TYPE_CLEARTXT) {
-				vty_out(vty, " isis password clear %s%s",
-					circuit->passwd.passwd, VTY_NEWLINE);
+				vty_out(vty, " isis password clear %s\n",
+					circuit->passwd.passwd);
 				write++;
 			}
+			write += circuit_write_mt_settings(circuit, vty);
 		}
-		vty_out(vty, "!%s", VTY_NEWLINE);
+		vty_endframe(vty, "!\n");
 	}
 
 	return write;
@@ -1175,7 +1170,6 @@ void isis_circuit_af_set(struct isis_circuit *circuit, bool ip_router,
 	struct isis_area *area = circuit->area;
 	bool change = circuit->ip_router != ip_router
 		      || circuit->ipv6_router != ipv6_router;
-	bool was_enabled = !!circuit->area;
 
 	area->ip_circuits += ip_router - circuit->ip_router;
 	area->ipv6_circuits += ipv6_router - circuit->ipv6_router;
@@ -1189,19 +1183,17 @@ void isis_circuit_af_set(struct isis_circuit *circuit, bool ip_router,
 
 	if (!ip_router && !ipv6_router)
 		isis_csm_state_change(ISIS_DISABLE, circuit, area);
-	else if (!was_enabled)
-		isis_csm_state_change(ISIS_ENABLE, circuit, area);
 	else
 		lsp_regenerate_schedule(circuit->area, circuit->is_type, 0);
 }
 
-int isis_circuit_passive_set(struct isis_circuit *circuit, bool passive)
+ferr_r isis_circuit_passive_set(struct isis_circuit *circuit, bool passive)
 {
 	if (circuit->is_passive == passive)
-		return 0;
+		return ferr_ok();
 
 	if (if_is_loopback(circuit->interface) && !passive)
-		return -1;
+		return ferr_cfg_invalid("loopback is always passive");
 
 	if (circuit->state != C_STATE_UP) {
 		circuit->is_passive = passive;
@@ -1212,82 +1204,85 @@ int isis_circuit_passive_set(struct isis_circuit *circuit, bool passive)
 		isis_csm_state_change(ISIS_ENABLE, circuit, area);
 	}
 
-	return 0;
+	return ferr_ok();
 }
 
-int isis_circuit_metric_set(struct isis_circuit *circuit, int level, int metric)
+ferr_r isis_circuit_metric_set(struct isis_circuit *circuit, int level,
+			       int metric)
 {
 	assert(level == IS_LEVEL_1 || level == IS_LEVEL_2);
 	if (metric > MAX_WIDE_LINK_METRIC)
-		return -1;
+		return ferr_cfg_invalid("metric %d too large for wide metric",
+					metric);
 	if (circuit->area && circuit->area->oldmetric
 	    && metric > MAX_NARROW_LINK_METRIC)
-		return -1;
+		return ferr_cfg_invalid("metric %d too large for narrow metric",
+					metric);
 
 	circuit->te_metric[level - 1] = metric;
 	circuit->metric[level - 1] = metric;
 
 	if (circuit->area)
 		lsp_regenerate_schedule(circuit->area, level, 0);
-	return 0;
+	return ferr_ok();
 }
 
-int isis_circuit_passwd_unset(struct isis_circuit *circuit)
+ferr_r isis_circuit_passwd_unset(struct isis_circuit *circuit)
 {
 	memset(&circuit->passwd, 0, sizeof(circuit->passwd));
-	return 0;
+	return ferr_ok();
 }
 
 static int isis_circuit_passwd_set(struct isis_circuit *circuit,
-				   u_char passwd_type, const char *passwd)
+				   uint8_t passwd_type, const char *passwd)
 {
 	int len;
 
 	if (!passwd)
-		return -1;
+		return ferr_code_bug("no circuit password given");
 
 	len = strlen(passwd);
 	if (len > 254)
-		return -1;
+		return ferr_code_bug(
+			"circuit password too long (max 254 chars)");
 
 	circuit->passwd.len = len;
 	strncpy((char *)circuit->passwd.passwd, passwd, 255);
 	circuit->passwd.type = passwd_type;
-	return 0;
+	return ferr_ok();
 }
 
-int isis_circuit_passwd_cleartext_set(struct isis_circuit *circuit,
-				      const char *passwd)
+ferr_r isis_circuit_passwd_cleartext_set(struct isis_circuit *circuit,
+					 const char *passwd)
 {
 	return isis_circuit_passwd_set(circuit, ISIS_PASSWD_TYPE_CLEARTXT,
 				       passwd);
 }
 
-int isis_circuit_passwd_hmac_md5_set(struct isis_circuit *circuit,
-				     const char *passwd)
+ferr_r isis_circuit_passwd_hmac_md5_set(struct isis_circuit *circuit,
+					const char *passwd)
 {
 	return isis_circuit_passwd_set(circuit, ISIS_PASSWD_TYPE_HMAC_MD5,
 				       passwd);
 }
+
 struct cmd_node interface_node = {
 	INTERFACE_NODE, "%s(config-if)# ", 1,
 };
 
-int isis_circuit_circ_type_set(struct isis_circuit *circuit, int circ_type)
+ferr_r isis_circuit_circ_type_set(struct isis_circuit *circuit, int circ_type)
 {
+	if (circuit->circ_type == circ_type)
+		return ferr_ok();
+
 	/* Changing the network type to/of loopback or unknown interfaces
 	 * is not supported. */
 	if (circ_type == CIRCUIT_T_UNKNOWN || circ_type == CIRCUIT_T_LOOPBACK
 	    || circuit->circ_type == CIRCUIT_T_LOOPBACK) {
-		if (circuit->circ_type != circ_type)
-			return -1;
-		else
-			return 0;
+		return ferr_cfg_invalid(
+			"cannot change network type on unknown interface");
 	}
 
-	if (circuit->circ_type == circ_type)
-		return 0;
-
 	if (circuit->state != C_STATE_UP) {
 		circuit->circ_type = circ_type;
 		circuit->circ_type_config = circ_type;
@@ -1295,14 +1290,30 @@ int isis_circuit_circ_type_set(struct isis_circuit *circuit, int circ_type)
 		struct isis_area *area = circuit->area;
 		if (circ_type == CIRCUIT_T_BROADCAST
 		    && !if_is_broadcast(circuit->interface))
-			return -1;
+			return ferr_cfg_reality(
+				"cannot configure non-broadcast interface for broadcast operation");
 
 		isis_csm_state_change(ISIS_DISABLE, circuit, area);
 		circuit->circ_type = circ_type;
 		circuit->circ_type_config = circ_type;
 		isis_csm_state_change(ISIS_ENABLE, circuit, area);
 	}
-	return 0;
+	return ferr_ok();
+}
+
+int isis_circuit_mt_enabled_set(struct isis_circuit *circuit, uint16_t mtid,
+				bool enabled)
+{
+	struct isis_circuit_mt_setting *setting;
+
+	setting = circuit_get_mt_setting(circuit, mtid);
+	if (setting->enabled != enabled) {
+		setting->enabled = enabled;
+		lsp_regenerate_schedule(circuit->area, IS_LEVEL_1 | IS_LEVEL_2,
+					0);
+	}
+
+	return CMD_SUCCESS;
 }
 
 int isis_if_new_hook(struct interface *ifp)
@@ -1326,8 +1337,8 @@ int isis_if_delete_hook(struct interface *ifp)
 void isis_circuit_init()
 {
 	/* Initialize Zebra interface data structure */
-	if_add_hook(IF_NEW_HOOK, isis_if_new_hook);
-	if_add_hook(IF_DELETE_HOOK, isis_if_delete_hook);
+	hook_register_prio(if_add, 0, isis_if_new_hook);
+	hook_register_prio(if_del, 0, isis_if_delete_hook);
 
 	/* Install interface node */
 	install_node(&interface_node, isis_interface_config_write);
@@ -1335,3 +1346,57 @@ void isis_circuit_init()
 
 	isis_vty_init();
 }
+
+void isis_circuit_schedule_lsp_send(struct isis_circuit *circuit)
+{
+	if (circuit->t_send_lsp)
+		return;
+	circuit->t_send_lsp =
+		thread_add_event(master, send_lsp, circuit, 0, NULL);
+}
+
+void isis_circuit_queue_lsp(struct isis_circuit *circuit, struct isis_lsp *lsp)
+{
+	if (isis_lsp_hash_lookup(circuit->lsp_hash, lsp))
+		return;
+
+	listnode_add(circuit->lsp_queue, lsp);
+	isis_lsp_hash_add(circuit->lsp_hash, lsp);
+	isis_circuit_schedule_lsp_send(circuit);
+}
+
+void isis_circuit_lsp_queue_clean(struct isis_circuit *circuit)
+{
+	if (!circuit->lsp_queue)
+		return;
+
+	list_delete_all_node(circuit->lsp_queue);
+	isis_lsp_hash_clean(circuit->lsp_hash);
+}
+
+void isis_circuit_cancel_queued_lsp(struct isis_circuit *circuit,
+				    struct isis_lsp *lsp)
+{
+	if (!circuit->lsp_queue)
+		return;
+
+	listnode_delete(circuit->lsp_queue, lsp);
+	isis_lsp_hash_release(circuit->lsp_hash, lsp);
+}
+
+struct isis_lsp *isis_circuit_lsp_queue_pop(struct isis_circuit *circuit)
+{
+	if (!circuit->lsp_queue)
+		return NULL;
+
+	struct listnode *node = listhead(circuit->lsp_queue);
+	if (!node)
+		return NULL;
+
+	struct isis_lsp *rv = listgetdata(node);
+
+	list_delete_node(circuit->lsp_queue, node);
+	isis_lsp_hash_release(circuit->lsp_hash, rv);
+
+	return rv;
+}
