@@ -12,10 +12,10 @@
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
-
+ *
  * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -37,7 +37,6 @@
 #include "isisd/isis_flags.h"
 #include "isisd/isis_misc.h"
 #include "isisd/isis_circuit.h"
-#include "isisd/isis_tlv.h"
 #include "isisd/isisd.h"
 #include "isisd/isis_lsp.h"
 #include "isisd/isis_route.h"
@@ -65,17 +64,6 @@ static afi_t afi_for_redist_protocol(int protocol)
 	return AFI_IP;
 }
 
-static int is_default(struct prefix *p)
-{
-	if (p->family == AF_INET)
-		if (p->u.prefix4.s_addr == 0 && p->prefixlen == 0)
-			return 1;
-	if (p->family == AF_INET6)
-		if (IN6_IS_ADDR_UNSPECIFIED(&p->u.prefix6) && p->prefixlen == 0)
-			return 1;
-	return 0;
-}
-
 static struct route_table *get_ext_info(struct isis *i, int family)
 {
 	int protocol = redist_protocol(family);
@@ -196,8 +184,7 @@ static void isis_redist_update_ext_reach(struct isis_area *area, int level,
 	route_map_result_t map_ret;
 
 	memcpy(&area_info, info, sizeof(area_info));
-	if (redist->metric != 0xffffffff)
-		area_info.metric = redist->metric;
+	area_info.metric = redist->metric;
 
 	if (redist->map_name) {
 		map_ret =
@@ -251,7 +238,7 @@ static void isis_redist_ensure_default(struct isis *isis, int family)
 }
 
 /* Handle notification about route being added */
-void isis_redist_add(int type, struct prefix *p, u_char distance,
+void isis_redist_add(int type, struct prefix *p, uint8_t distance,
 		     uint32_t metric)
 {
 	int family = p->family;
@@ -266,8 +253,8 @@ void isis_redist_add(int type, struct prefix *p, u_char distance,
 	char debug_buf[BUFSIZ];
 	prefix2str(p, debug_buf, sizeof(debug_buf));
 
-	zlog_debug("%s: New route %s from %s.", __func__, debug_buf,
-		   zebra_route_string(type));
+	zlog_debug("%s: New route %s from %s: distance %d.", __func__,
+		   debug_buf, zebra_route_string(type), distance);
 
 	if (!ei_table) {
 		zlog_warn("%s: External information table not initialized.",
@@ -287,7 +274,7 @@ void isis_redist_add(int type, struct prefix *p, u_char distance,
 	info->distance = distance;
 	info->metric = metric;
 
-	if (is_default(p))
+	if (is_default_prefix(p))
 		type = DEFAULT_ROUTE;
 
 	for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area))
@@ -317,7 +304,7 @@ void isis_redist_delete(int type, struct prefix *p)
 	zlog_debug("%s: Removing route %s from %s.", __func__, debug_buf,
 		   zebra_route_string(type));
 
-	if (is_default(p)) {
+	if (is_default_prefix(p)) {
 		/* Don't remove default route but add synthetic route for use
 		 * by "default-information originate always". Areas without the
 		 * "always" setting will ignore routes with origin
@@ -455,7 +442,7 @@ static void isis_redist_set(struct isis_area *area, int level, int family,
 		info = rn->info;
 
 		if (type == DEFAULT_ROUTE) {
-			if (!is_default(&rn->p))
+			if (!is_default_prefix(&rn->p))
 				continue;
 		} else {
 			if (info->origin != type)
@@ -491,7 +478,7 @@ static void isis_redist_unset(struct isis_area *area, int level, int family,
 		info = rn->info;
 
 		if (type == DEFAULT_ROUTE) {
-			if (!is_default(&rn->p))
+			if (!is_default_prefix(&rn->p))
 				continue;
 		} else {
 			if (info->origin != type)
@@ -552,39 +539,36 @@ DEFUN (isis_redistribute,
 	int afi;
 	int type;
 	int level;
-	unsigned long metric;
+	unsigned long metric = 0;
 	const char *routemap = NULL;
 
 	family = str2family(argv[idx_afi]->text);
 	if (family < 0)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	afi = family2afi(family);
 	if (!afi)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	type = proto_redistnum(afi, argv[idx_protocol]->text);
 	if (type < 0)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	if (!strcmp("level-1", argv[idx_level]->arg))
 		level = 1;
 	else if (!strcmp("level-2", argv[idx_level]->arg))
 		level = 2;
 	else
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	if ((area->is_type & level) != level) {
-		vty_out(vty, "Node is not a level-%d IS%s", level, VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Node is not a level-%d IS\n", level);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
-	metric = 0xffffffff;
-	routemap = NULL;
-
 	if (argc > idx_metric_rmap + 1) {
 		if (argv[idx_metric_rmap + 1]->arg[0] == '\0')
-			return CMD_WARNING;
+			return CMD_WARNING_CONFIG_FAILED;
 
 		if (strmatch(argv[idx_metric_rmap]->text, "metric")) {
 			char *endp;
@@ -592,7 +576,7 @@ DEFUN (isis_redistribute,
 					 10);
 
 			if (*endp != '\0')
-				return CMD_WARNING;
+				return CMD_WARNING_CONFIG_FAILED;
 		} else {
 			routemap = argv[idx_metric_rmap + 1]->arg;
 		}
@@ -624,15 +608,15 @@ DEFUN (no_isis_redistribute,
 
 	family = str2family(argv[idx_afi]->arg);
 	if (family < 0)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	afi = family2afi(family);
 	if (!afi)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	type = proto_redistnum(afi, argv[idx_protocol]->text);
 	if (type < 0)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	level = strmatch("level-1", argv[idx_level]->text) ? 1 : 2;
 
@@ -663,18 +647,18 @@ DEFUN (isis_default_originate,
 	int family;
 	int originate_type = DEFAULT_ORIGINATE;
 	int level;
-	unsigned long metric = 0xffffffff;
+	unsigned long metric = 0;
 	const char *routemap = NULL;
 
 	family = str2family(argv[idx_afi]->text);
 	if (family < 0)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	level = strmatch("level-1", argv[idx_level]->text) ? 1 : 2;
 
 	if ((area->is_type & level) != level) {
-		vty_out(vty, "Node is not a level-%d IS%s", level, VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Node is not a level-%d IS\n", level);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (argc > idx_always && strmatch(argv[idx_always]->text, "always")) {
@@ -692,11 +676,9 @@ DEFUN (isis_default_originate,
 
 	if (family == AF_INET6 && originate_type != DEFAULT_ORIGINATE_ALWAYS) {
 		vty_out(vty,
-			"Zebra doesn't implement default-originate for IPv6 yet%s",
-			VTY_NEWLINE);
+			"Zebra doesn't implement default-originate for IPv6 yet\n");
 		vty_out(vty,
-			"so use with care or use default-originate always.%s",
-			VTY_NEWLINE);
+			"so use with care or use default-originate always.\n");
 	}
 
 	isis_redist_set(area, level, family, DEFAULT_ROUTE, metric, routemap,
@@ -723,14 +705,14 @@ DEFUN (no_isis_default_originate,
 
 	family = str2family(argv[idx_afi]->text);
 	if (family < 0)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	if (strmatch("level-1", argv[idx_level]->text))
 		level = 1;
 	else if (strmatch("level-2", argv[idx_level]->text))
 		level = 2;
 	else
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	isis_redist_unset(area, level, family, DEFAULT_ROUTE);
 	return 0;
@@ -762,11 +744,11 @@ int isis_redist_config_write(struct vty *vty, struct isis_area *area,
 				continue;
 			vty_out(vty, " redistribute %s %s level-%d", family_str,
 				zebra_route_string(type), level);
-			if (redist->metric != 0xffffffff)
+			if (redist->metric)
 				vty_out(vty, " metric %u", redist->metric);
 			if (redist->map_name)
 				vty_out(vty, " route-map %s", redist->map_name);
-			vty_out(vty, "%s", VTY_NEWLINE);
+			vty_out(vty, "\n");
 			write++;
 		}
 	}
@@ -780,11 +762,11 @@ int isis_redist_config_write(struct vty *vty, struct isis_area *area,
 			family_str, level);
 		if (redist->redist == DEFAULT_ORIGINATE_ALWAYS)
 			vty_out(vty, " always");
-		if (redist->metric != 0xffffffff)
+		if (redist->metric)
 			vty_out(vty, " metric %u", redist->metric);
 		if (redist->map_name)
 			vty_out(vty, " route-map %s", redist->map_name);
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "\n");
 		write++;
 	}
 
