@@ -18,10 +18,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -42,6 +41,7 @@
 #include "md5.h"
 #include "sockunion.h"
 #include "network.h"
+#include "sbuf.h"
 
 #include "isisd/dict.h"
 #include "isisd/isis_constants.h"
@@ -49,7 +49,6 @@
 #include "isisd/isis_flags.h"
 #include "isisd/isis_circuit.h"
 #include "isisd/isisd.h"
-#include "isisd/isis_tlv.h"
 #include "isisd/isis_lsp.h"
 #include "isisd/isis_pdu.h"
 #include "isisd/isis_dynhn.h"
@@ -101,9 +100,9 @@ struct mpls_te_circuit *mpls_te_circuit_new()
 /* Copy SUB TLVs parameters into a buffer - No space verification are performed
  */
 /* Caller must verify before that there is enough free space in the buffer */
-u_char add_te_subtlvs(u_char *buf, struct mpls_te_circuit *mtc)
+uint8_t add_te_subtlvs(uint8_t *buf, struct mpls_te_circuit *mtc)
 {
-	u_char size, *tlvs = buf;
+	uint8_t size, *tlvs = buf;
 
 	zlog_debug("ISIS MPLS-TE: Add TE Sub TLVs to buffer");
 
@@ -224,6 +223,9 @@ u_char add_te_subtlvs(u_char *buf, struct mpls_te_circuit *mtc)
 		tlvs += size;
 	}
 
+	/* Add before this line any other parsing of TLV */
+	(void)tlvs;
+
 	/* Update SubTLVs length */
 	mtc->length = subtlvs_len(mtc);
 
@@ -233,7 +235,7 @@ u_char add_te_subtlvs(u_char *buf, struct mpls_te_circuit *mtc)
 }
 
 /* Compute total Sub-TLVs size */
-u_char subtlvs_len(struct mpls_te_circuit *mtc)
+uint8_t subtlvs_len(struct mpls_te_circuit *mtc)
 {
 	int length = 0;
 
@@ -307,14 +309,14 @@ u_char subtlvs_len(struct mpls_te_circuit *mtc)
 		return 0;
 	}
 
-	mtc->length = (u_char)length;
+	mtc->length = (uint8_t)length;
 
 	return mtc->length;
 }
 
 /* Following are various functions to set MPLS TE parameters */
 static void set_circuitparams_admin_grp(struct mpls_te_circuit *mtc,
-					u_int32_t admingrp)
+					uint32_t admingrp)
 {
 	SUBTLV_TYPE(mtc->admin_grp) = TE_SUBTLV_ADMIN_GRP;
 	SUBTLV_LEN(mtc->admin_grp) = SUBTLV_DEF_SIZE;
@@ -323,8 +325,8 @@ static void set_circuitparams_admin_grp(struct mpls_te_circuit *mtc,
 }
 
 static void __attribute__((unused))
-set_circuitparams_llri(struct mpls_te_circuit *mtc, u_int32_t local,
-		       u_int32_t remote)
+set_circuitparams_llri(struct mpls_te_circuit *mtc, uint32_t local,
+		       uint32_t remote)
 {
 	SUBTLV_TYPE(mtc->llri) = TE_SUBTLV_LLRI;
 	SUBTLV_LEN(mtc->llri) = TE_SUBTLV_LLRI_SIZE;
@@ -379,7 +381,7 @@ static void set_circuitparams_unrsv_bw(struct mpls_te_circuit *mtc,
 }
 
 static void set_circuitparams_te_metric(struct mpls_te_circuit *mtc,
-					u_int32_t te_metric)
+					uint32_t te_metric)
 {
 	SUBTLV_TYPE(mtc->te_metric) = TE_SUBTLV_TE_METRIC;
 	SUBTLV_LEN(mtc->te_metric) = TE_SUBTLV_TE_METRIC_SIZE;
@@ -390,7 +392,7 @@ static void set_circuitparams_te_metric(struct mpls_te_circuit *mtc,
 }
 
 static void set_circuitparams_inter_as(struct mpls_te_circuit *mtc,
-				       struct in_addr addr, u_int32_t as)
+				       struct in_addr addr, uint32_t as)
 {
 
 	/* Set the Remote ASBR IP address and then the associated AS number */
@@ -417,9 +419,9 @@ static void unset_circuitparams_inter_as(struct mpls_te_circuit *mtc)
 }
 
 static void set_circuitparams_av_delay(struct mpls_te_circuit *mtc,
-				       u_int32_t delay, u_char anormal)
+				       uint32_t delay, uint8_t anormal)
 {
-	u_int32_t tmp;
+	uint32_t tmp;
 	/* Note that TLV-length field is the size of array. */
 	SUBTLV_TYPE(mtc->av_delay) = TE_SUBTLV_AV_DELAY;
 	SUBTLV_LEN(mtc->av_delay) = SUBTLV_DEF_SIZE;
@@ -431,10 +433,10 @@ static void set_circuitparams_av_delay(struct mpls_te_circuit *mtc,
 }
 
 static void set_circuitparams_mm_delay(struct mpls_te_circuit *mtc,
-				       u_int32_t low, u_int32_t high,
-				       u_char anormal)
+				       uint32_t low, uint32_t high,
+				       uint8_t anormal)
 {
-	u_int32_t tmp;
+	uint32_t tmp;
 	/* Note that TLV-length field is the size of array. */
 	SUBTLV_TYPE(mtc->mm_delay) = TE_SUBTLV_MM_DELAY;
 	SUBTLV_LEN(mtc->mm_delay) = TE_SUBTLV_MM_DELAY_SIZE;
@@ -447,7 +449,7 @@ static void set_circuitparams_mm_delay(struct mpls_te_circuit *mtc,
 }
 
 static void set_circuitparams_delay_var(struct mpls_te_circuit *mtc,
-					u_int32_t jitter)
+					uint32_t jitter)
 {
 	/* Note that TLV-length field is the size of array. */
 	SUBTLV_TYPE(mtc->delay_var) = TE_SUBTLV_DELAY_VAR;
@@ -457,9 +459,9 @@ static void set_circuitparams_delay_var(struct mpls_te_circuit *mtc,
 }
 
 static void set_circuitparams_pkt_loss(struct mpls_te_circuit *mtc,
-				       u_int32_t loss, u_char anormal)
+				       uint32_t loss, uint8_t anormal)
 {
-	u_int32_t tmp;
+	uint32_t tmp;
 	/* Note that TLV-length field is the size of array. */
 	SUBTLV_TYPE(mtc->pkt_loss) = TE_SUBTLV_PKT_LOSS;
 	SUBTLV_LEN(mtc->pkt_loss) = SUBTLV_DEF_SIZE;
@@ -547,13 +549,10 @@ void isis_link_params_update(struct isis_circuit *circuit,
 		if ((SUBTLV_TYPE(mtc->rmt_ipaddr) == 0)
 		    && (circuit->circ_type == CIRCUIT_T_P2P)) {
 			struct isis_adjacency *adj = circuit->u.p2p.neighbor;
-			if (adj->ipv4_addrs != NULL
-			    && listcount(adj->ipv4_addrs) != 0) {
-				struct in_addr *ip_addr;
-				ip_addr = (struct in_addr *)listgetdata(
-					(struct listnode *)listhead(
-						adj->ipv4_addrs));
-				set_circuitparams_rmt_ipaddr(mtc, *ip_addr);
+			if (adj && adj->adj_state == ISIS_ADJ_UP
+			    && adj->ipv4_address_count) {
+				set_circuitparams_rmt_ipaddr(
+					mtc, adj->ipv4_addresses[0]);
 			}
 		}
 
@@ -671,412 +670,402 @@ void isis_mpls_te_update(struct interface *ifp)
  * Followings are vty session control functions.
  *------------------------------------------------------------------------*/
 
-static u_char show_vty_subtlv_admin_grp(struct vty *vty,
-					struct te_subtlv_admin_grp *tlv)
+static uint8_t print_subtlv_admin_grp(struct sbuf *buf, int indent,
+				      struct te_subtlv_admin_grp *tlv)
 {
-
-	if (vty != NULL)
-		vty_out(vty, "    Administrative Group: 0x%x%s",
-			(u_int32_t)ntohl(tlv->value), VTY_NEWLINE);
-	else
-		zlog_debug("      Administrative Group: 0x%x",
-			   (u_int32_t)ntohl(tlv->value));
-
+	sbuf_push(buf, indent, "Administrative Group: 0x%" PRIx32 "\n",
+		  ntohl(tlv->value));
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_llri(struct vty *vty, struct te_subtlv_llri *tlv)
+static uint8_t print_subtlv_llri(struct sbuf *buf, int indent,
+				 struct te_subtlv_llri *tlv)
 {
-	if (vty != NULL) {
-		vty_out(vty, "    Link Local  ID: %d%s",
-			(u_int32_t)ntohl(tlv->local), VTY_NEWLINE);
-		vty_out(vty, "    Link Remote ID: %d%s",
-			(u_int32_t)ntohl(tlv->remote), VTY_NEWLINE);
-	} else {
-		zlog_debug("      Link Local  ID: %d",
-			   (u_int32_t)ntohl(tlv->local));
-		zlog_debug("      Link Remote ID: %d",
-			   (u_int32_t)ntohl(tlv->remote));
-	}
+	sbuf_push(buf, indent, "Link Local  ID: %" PRIu32 "\n",
+		  ntohl(tlv->local));
+	sbuf_push(buf, indent, "Link Remote ID: %" PRIu32 "\n",
+		  ntohl(tlv->remote));
 
 	return (SUBTLV_HDR_SIZE + TE_SUBTLV_LLRI_SIZE);
 }
 
-static u_char show_vty_subtlv_local_ipaddr(struct vty *vty,
-					   struct te_subtlv_local_ipaddr *tlv)
+static uint8_t print_subtlv_local_ipaddr(struct sbuf *buf, int indent,
+					 struct te_subtlv_local_ipaddr *tlv)
 {
-	if (vty != NULL)
-		vty_out(vty, "    Local Interface IP Address(es): %s%s",
-			inet_ntoa(tlv->value), VTY_NEWLINE);
-	else
-		zlog_debug("      Local Interface IP Address(es): %s",
-			   inet_ntoa(tlv->value));
+	sbuf_push(buf, indent, "Local Interface IP Address(es): %s\n",
+		  inet_ntoa(tlv->value));
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_rmt_ipaddr(struct vty *vty,
-					 struct te_subtlv_rmt_ipaddr *tlv)
+static uint8_t print_subtlv_rmt_ipaddr(struct sbuf *buf, int indent,
+				       struct te_subtlv_rmt_ipaddr *tlv)
 {
-	if (vty != NULL)
-		vty_out(vty, "    Remote Interface IP Address(es): %s%s",
-			inet_ntoa(tlv->value), VTY_NEWLINE);
-	else
-		zlog_debug("      Remote Interface IP Address(es): %s",
-			   inet_ntoa(tlv->value));
+	sbuf_push(buf, indent, "Remote Interface IP Address(es): %s\n",
+		  inet_ntoa(tlv->value));
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_max_bw(struct vty *vty,
-				     struct te_subtlv_max_bw *tlv)
+static uint8_t print_subtlv_max_bw(struct sbuf *buf, int indent,
+				   struct te_subtlv_max_bw *tlv)
 {
 	float fval;
 
 	fval = ntohf(tlv->value);
 
-	if (vty != NULL)
-		vty_out(vty, "    Maximum Bandwidth: %g (Bytes/sec)%s", fval,
-			VTY_NEWLINE);
-	else
-		zlog_debug("      Maximum Bandwidth: %g (Bytes/sec)", fval);
+	sbuf_push(buf, indent, "Maximum Bandwidth: %g (Bytes/sec)\n", fval);
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_max_rsv_bw(struct vty *vty,
-					 struct te_subtlv_max_rsv_bw *tlv)
+static uint8_t print_subtlv_max_rsv_bw(struct sbuf *buf, int indent,
+				       struct te_subtlv_max_rsv_bw *tlv)
 {
 	float fval;
 
 	fval = ntohf(tlv->value);
 
-	if (vty != NULL)
-		vty_out(vty,
-			"    Maximum Reservable Bandwidth: %g (Bytes/sec)%s",
-			fval, VTY_NEWLINE);
-	else
-		zlog_debug("      Maximum Reservable Bandwidth: %g (Bytes/sec)",
-			   fval);
+	sbuf_push(buf, indent, "Maximum Reservable Bandwidth: %g (Bytes/sec)\n",
+		  fval);
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_unrsv_bw(struct vty *vty,
-				       struct te_subtlv_unrsv_bw *tlv)
+static uint8_t print_subtlv_unrsv_bw(struct sbuf *buf, int indent,
+				     struct te_subtlv_unrsv_bw *tlv)
 {
 	float fval1, fval2;
 	int i;
 
-	if (vty != NULL)
-		vty_out(vty, "    Unreserved Bandwidth:%s", VTY_NEWLINE);
-	else
-		zlog_debug("      Unreserved Bandwidth:");
+	sbuf_push(buf, indent, "Unreserved Bandwidth:\n");
 
 	for (i = 0; i < MAX_CLASS_TYPE; i += 2) {
 		fval1 = ntohf(tlv->value[i]);
 		fval2 = ntohf(tlv->value[i + 1]);
-		if (vty != NULL)
-			vty_out(vty,
-				"      [%d]: %g (Bytes/sec),\t[%d]: %g (Bytes/sec)%s",
-				i, fval1, i + 1, fval2, VTY_NEWLINE);
-		else
-			zlog_debug(
-				"        [%d]: %g (Bytes/sec),\t[%d]: %g (Bytes/sec)",
-				i, fval1, i + 1, fval2);
+		sbuf_push(buf, indent + 2,
+			  "[%d]: %g (Bytes/sec),\t[%d]: %g (Bytes/sec)\n", i,
+			  fval1, i + 1, fval2);
 	}
 
 	return (SUBTLV_HDR_SIZE + TE_SUBTLV_UNRSV_SIZE);
 }
 
-static u_char show_vty_subtlv_te_metric(struct vty *vty,
-					struct te_subtlv_te_metric *tlv)
+static uint8_t print_subtlv_te_metric(struct sbuf *buf, int indent,
+				      struct te_subtlv_te_metric *tlv)
 {
-	u_int32_t te_metric;
+	uint32_t te_metric;
 
 	te_metric = tlv->value[2] | tlv->value[1] << 8 | tlv->value[0] << 16;
-	if (vty != NULL)
-		vty_out(vty, "    Traffic Engineering Metric: %u%s", te_metric,
-			VTY_NEWLINE);
-	else
-		zlog_debug("      Traffic Engineering Metric: %u", te_metric);
+	sbuf_push(buf, indent, "Traffic Engineering Metric: %u\n", te_metric);
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_ras(struct vty *vty, struct te_subtlv_ras *tlv)
+static uint8_t print_subtlv_ras(struct sbuf *buf, int indent,
+				struct te_subtlv_ras *tlv)
 {
-	if (vty != NULL)
-		vty_out(vty, "    Inter-AS TE Remote AS number: %u%s",
-			ntohl(tlv->value), VTY_NEWLINE);
-	else
-		zlog_debug("      Inter-AS TE Remote AS number: %u",
-			   ntohl(tlv->value));
+	sbuf_push(buf, indent, "Inter-AS TE Remote AS number: %" PRIu32 "\n",
+		  ntohl(tlv->value));
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_rip(struct vty *vty, struct te_subtlv_rip *tlv)
+static uint8_t print_subtlv_rip(struct sbuf *buf, int indent,
+				struct te_subtlv_rip *tlv)
 {
-	if (vty != NULL)
-		vty_out(vty, "    Inter-AS TE Remote ASBR IP address: %s%s",
-			inet_ntoa(tlv->value), VTY_NEWLINE);
-	else
-		zlog_debug("      Inter-AS TE Remote ASBR IP address: %s",
-			   inet_ntoa(tlv->value));
+	sbuf_push(buf, indent, "Inter-AS TE Remote ASBR IP address: %s\n",
+		  inet_ntoa(tlv->value));
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_av_delay(struct vty *vty,
-				       struct te_subtlv_av_delay *tlv)
+static uint8_t print_subtlv_av_delay(struct sbuf *buf, int indent,
+				     struct te_subtlv_av_delay *tlv)
 {
-	u_int32_t delay;
-	u_int32_t A;
+	uint32_t delay;
+	uint32_t A;
 
-	delay = (u_int32_t)ntohl(tlv->value) & TE_EXT_MASK;
-	A = (u_int32_t)ntohl(tlv->value) & TE_EXT_ANORMAL;
+	delay = (uint32_t)ntohl(tlv->value) & TE_EXT_MASK;
+	A = (uint32_t)ntohl(tlv->value) & TE_EXT_ANORMAL;
 
-	if (vty != NULL)
-		vty_out(vty, "    %s Average Link Delay: %d (micro-sec)%s",
-			A ? "Anomalous" : "Normal", delay, VTY_NEWLINE);
-	else
-		zlog_debug("      %s Average Link Delay: %d (micro-sec)",
-			   A ? "Anomalous" : "Normal", delay);
+	sbuf_push(buf, indent,
+		  "%s Average Link Delay: %" PRIu32 " (micro-sec)\n",
+		  A ? "Anomalous" : "Normal", delay);
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_mm_delay(struct vty *vty,
-				       struct te_subtlv_mm_delay *tlv)
+static uint8_t print_subtlv_mm_delay(struct sbuf *buf, int indent,
+				     struct te_subtlv_mm_delay *tlv)
 {
-	u_int32_t low, high;
-	u_int32_t A;
+	uint32_t low, high;
+	uint32_t A;
 
-	low = (u_int32_t)ntohl(tlv->low) & TE_EXT_MASK;
-	A = (u_int32_t)ntohl(tlv->low) & TE_EXT_ANORMAL;
-	high = (u_int32_t)ntohl(tlv->high) & TE_EXT_MASK;
+	low = (uint32_t)ntohl(tlv->low) & TE_EXT_MASK;
+	A = (uint32_t)ntohl(tlv->low) & TE_EXT_ANORMAL;
+	high = (uint32_t)ntohl(tlv->high) & TE_EXT_MASK;
 
-	if (vty != NULL)
-		vty_out(vty, "    %s Min/Max Link Delay: %d / %d (micro-sec)%s",
-			A ? "Anomalous" : "Normal", low, high, VTY_NEWLINE);
-	else
-		zlog_debug("      %s Min/Max Link Delay: %d / %d (micro-sec)",
-			   A ? "Anomalous" : "Normal", low, high);
+	sbuf_push(buf, indent, "%s Min/Max Link Delay: %" PRIu32 " / %" PRIu32 " (micro-sec)\n",
+		  A ? "Anomalous" : "Normal", low, high);
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_delay_var(struct vty *vty,
-					struct te_subtlv_delay_var *tlv)
+static uint8_t print_subtlv_delay_var(struct sbuf *buf, int indent,
+				      struct te_subtlv_delay_var *tlv)
 {
-	u_int32_t jitter;
+	uint32_t jitter;
 
-	jitter = (u_int32_t)ntohl(tlv->value) & TE_EXT_MASK;
+	jitter = (uint32_t)ntohl(tlv->value) & TE_EXT_MASK;
 
-	if (vty != NULL)
-		vty_out(vty, "    Delay Variation: %d (micro-sec)%s", jitter,
-			VTY_NEWLINE);
-	else
-		zlog_debug("      Delay Variation: %d (micro-sec)", jitter);
+	sbuf_push(buf, indent, "Delay Variation: %" PRIu32 " (micro-sec)\n",
+		  jitter);
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_pkt_loss(struct vty *vty,
-				       struct te_subtlv_pkt_loss *tlv)
+static uint8_t print_subtlv_pkt_loss(struct sbuf *buf, int indent,
+				     struct te_subtlv_pkt_loss *tlv)
 {
-	u_int32_t loss;
-	u_int32_t A;
+	uint32_t loss;
+	uint32_t A;
 	float fval;
 
-	loss = (u_int32_t)ntohl(tlv->value) & TE_EXT_MASK;
+	loss = (uint32_t)ntohl(tlv->value) & TE_EXT_MASK;
 	fval = (float)(loss * LOSS_PRECISION);
-	A = (u_int32_t)ntohl(tlv->value) & TE_EXT_ANORMAL;
+	A = (uint32_t)ntohl(tlv->value) & TE_EXT_ANORMAL;
 
-	if (vty != NULL)
-		vty_out(vty, "    %s Link Packet Loss: %g (%%)%s",
-			A ? "Anomalous" : "Normal", fval, VTY_NEWLINE);
-	else
-		zlog_debug("      %s Link Packet Loss: %g (%%)",
-			   A ? "Anomalous" : "Normal", fval);
+	sbuf_push(buf, indent, "%s Link Packet Loss: %g (%%)\n",
+		  A ? "Anomalous" : "Normal", fval);
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_res_bw(struct vty *vty,
-				     struct te_subtlv_res_bw *tlv)
+static uint8_t print_subtlv_res_bw(struct sbuf *buf, int indent,
+				   struct te_subtlv_res_bw *tlv)
 {
 	float fval;
 
 	fval = ntohf(tlv->value);
 
-	if (vty != NULL)
-		vty_out(vty,
-			"    Unidirectional Residual Bandwidth: %g (Bytes/sec)%s",
-			fval, VTY_NEWLINE);
-	else
-		zlog_debug(
-			"      Unidirectional Residual Bandwidth: %g (Bytes/sec)",
-			fval);
+	sbuf_push(buf, indent,
+		  "Unidirectional Residual Bandwidth: %g (Bytes/sec)\n", fval);
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_ava_bw(struct vty *vty,
-				     struct te_subtlv_ava_bw *tlv)
+static uint8_t print_subtlv_ava_bw(struct sbuf *buf, int indent,
+				   struct te_subtlv_ava_bw *tlv)
 {
 	float fval;
 
 	fval = ntohf(tlv->value);
 
-	if (vty != NULL)
-		vty_out(vty,
-			"    Unidirectional Available Bandwidth: %g (Bytes/sec)%s",
-			fval, VTY_NEWLINE);
-	else
-		zlog_debug(
-			"      Unidirectional Available Bandwidth: %g (Bytes/sec)",
-			fval);
+	sbuf_push(buf, indent,
+		  "Unidirectional Available Bandwidth: %g (Bytes/sec)\n", fval);
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_subtlv_use_bw(struct vty *vty,
-				     struct te_subtlv_use_bw *tlv)
+static uint8_t print_subtlv_use_bw(struct sbuf *buf, int indent,
+				   struct te_subtlv_use_bw *tlv)
 {
 	float fval;
 
 	fval = ntohf(tlv->value);
 
-	if (vty != NULL)
-		vty_out(vty,
-			"    Unidirectional Utilized Bandwidth: %g (Bytes/sec)%s",
-			fval, VTY_NEWLINE);
-	else
-		zlog_debug(
-			"      Unidirectional Utilized Bandwidth: %g (Bytes/sec)",
-			fval);
+	sbuf_push(buf, indent,
+		  "Unidirectional Utilized Bandwidth: %g (Bytes/sec)\n", fval);
 
 	return (SUBTLV_HDR_SIZE + SUBTLV_DEF_SIZE);
 }
 
-static u_char show_vty_unknown_tlv(struct vty *vty, struct subtlv_header *tlvh)
+static uint8_t print_unknown_tlv(struct sbuf *buf, int indent,
+				 struct subtlv_header *tlvh)
 {
 	int i, rtn = 1;
-	u_char *v = (u_char *)tlvh;
+	uint8_t *v = (uint8_t *)tlvh;
 
-	if (vty != NULL) {
-		if (tlvh->length != 0) {
-			vty_out(vty,
-				"    Unknown TLV: [type(%#.2x), length(%#.2x)]%s",
-				tlvh->type, tlvh->length, VTY_NEWLINE);
-			vty_out(vty, "       Dump: [00]");
-			rtn = 1; /* initialize end of line counter */
-			for (i = 0; i < tlvh->length; i++) {
-				vty_out(vty, " %#.2x", v[i]);
-				if (rtn == 8) {
-					vty_out(vty, "%s             [%.2x]",
-						VTY_NEWLINE, i + 1);
-					rtn = 1;
-				} else
-					rtn++;
-			}
-			vty_out(vty, "%s", VTY_NEWLINE);
-		} else
-			vty_out(vty,
-				"    Unknown TLV: [type(%#.2x), length(%#.2x)]%s",
-				tlvh->type, tlvh->length, VTY_NEWLINE);
+	if (tlvh->length != 0) {
+		sbuf_push(buf, indent,
+			  "Unknown TLV: [type(%#.2x), length(%#.2x)]\n",
+			  tlvh->type, tlvh->length);
+		sbuf_push(buf, indent + 2, "Dump: [00]");
+		rtn = 1; /* initialize end of line counter */
+		for (i = 0; i < tlvh->length; i++) {
+			sbuf_push(buf, 0, " %#.2x", v[i]);
+			if (rtn == 8) {
+				sbuf_push(buf, 0, "\n");
+				sbuf_push(buf, indent + 8, "[%.2x]", i + 1);
+				rtn = 1;
+			} else
+				rtn++;
+		}
+		sbuf_push(buf, 0, "\n");
 	} else {
-		zlog_debug("      Unknown TLV: [type(%#.2x), length(%#.2x)]",
-			   tlvh->type, tlvh->length);
+		sbuf_push(buf, indent,
+			  "Unknown TLV: [type(%#.2x), length(%#.2x)]\n",
+			  tlvh->type, tlvh->length);
 	}
 
 	return SUBTLV_SIZE(tlvh);
 }
 
 /* Main Show function */
-void mpls_te_print_detail(struct vty *vty, struct te_is_neigh *te)
+void mpls_te_print_detail(struct sbuf *buf, int indent,
+			  uint8_t *subtlvs, uint8_t subtlv_len)
 {
-	struct subtlv_header *tlvh;
-	u_int16_t sum = 0;
+	struct subtlv_header *tlvh = (struct subtlv_header *)subtlvs;
+	uint16_t sum = 0;
 
-	zlog_debug("ISIS MPLS-TE: Show database TE detail");
+	for (; sum < subtlv_len;
+	     tlvh = (struct subtlv_header *)(subtlvs + sum)) {
+		if (subtlv_len - sum < SUBTLV_SIZE(tlvh)) {
+			sbuf_push(buf, indent, "Available data %" PRIu8 " is less than TLV size %u!\n",
+				  subtlv_len - sum, SUBTLV_SIZE(tlvh));
+			return;
+		}
 
-	tlvh = (struct subtlv_header *)te->sub_tlvs;
-
-	for (; sum < te->sub_tlvs_length; tlvh = SUBTLV_HDR_NEXT(tlvh)) {
 		switch (tlvh->type) {
 		case TE_SUBTLV_ADMIN_GRP:
-			sum += show_vty_subtlv_admin_grp(
-				vty, (struct te_subtlv_admin_grp *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Administrative Group!\n");
+				return;
+			}
+			sum += print_subtlv_admin_grp(buf, indent,
+				(struct te_subtlv_admin_grp *)tlvh);
 			break;
 		case TE_SUBTLV_LLRI:
-			sum += show_vty_subtlv_llri(
-				vty, (struct te_subtlv_llri *)tlvh);
+			if (tlvh->length != TE_SUBTLV_LLRI_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Link ID!\n");
+				return;
+			}
+			sum += print_subtlv_llri(buf, indent,
+						 (struct te_subtlv_llri *)tlvh);
 			break;
 		case TE_SUBTLV_LOCAL_IPADDR:
-			sum += show_vty_subtlv_local_ipaddr(
-				vty, (struct te_subtlv_local_ipaddr *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Local IP address!\n");
+				return;
+			}
+			sum += print_subtlv_local_ipaddr(buf, indent,
+				(struct te_subtlv_local_ipaddr *)tlvh);
 			break;
 		case TE_SUBTLV_RMT_IPADDR:
-			sum += show_vty_subtlv_rmt_ipaddr(
-				vty, (struct te_subtlv_rmt_ipaddr *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Remote Interface address!\n");
+				return;
+			}
+			sum += print_subtlv_rmt_ipaddr(buf, indent,
+				(struct te_subtlv_rmt_ipaddr *)tlvh);
 			break;
 		case TE_SUBTLV_MAX_BW:
-			sum += show_vty_subtlv_max_bw(
-				vty, (struct te_subtlv_max_bw *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Maximum Bandwidth!\n");
+				return;
+			}
+			sum += print_subtlv_max_bw(buf, indent,
+				(struct te_subtlv_max_bw *)tlvh);
 			break;
 		case TE_SUBTLV_MAX_RSV_BW:
-			sum += show_vty_subtlv_max_rsv_bw(
-				vty, (struct te_subtlv_max_rsv_bw *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Maximum Reservable Bandwidth!\n");
+				return;
+			}
+			sum += print_subtlv_max_rsv_bw(buf, indent,
+				(struct te_subtlv_max_rsv_bw *)tlvh);
 			break;
 		case TE_SUBTLV_UNRSV_BW:
-			sum += show_vty_subtlv_unrsv_bw(
-				vty, (struct te_subtlv_unrsv_bw *)tlvh);
+			if (tlvh->length != TE_SUBTLV_UNRSV_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Unreserved Bandwidth!\n");
+				return;
+			}
+			sum += print_subtlv_unrsv_bw(buf, indent,
+				(struct te_subtlv_unrsv_bw *)tlvh);
 			break;
 		case TE_SUBTLV_TE_METRIC:
-			sum += show_vty_subtlv_te_metric(
-				vty, (struct te_subtlv_te_metric *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Traffic Engineering Metric!\n");
+				return;
+			}
+			sum += print_subtlv_te_metric(buf, indent,
+				(struct te_subtlv_te_metric *)tlvh);
 			break;
 		case TE_SUBTLV_RAS:
-			sum += show_vty_subtlv_ras(
-				vty, (struct te_subtlv_ras *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Remote AS number!\n");
+				return;
+			}
+			sum += print_subtlv_ras(buf, indent,
+						(struct te_subtlv_ras *)tlvh);
 			break;
 		case TE_SUBTLV_RIP:
-			sum += show_vty_subtlv_rip(
-				vty, (struct te_subtlv_rip *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Remote ASBR IP Address!\n");
+				return;
+			}
+			sum += print_subtlv_rip(buf, indent,
+						(struct te_subtlv_rip *)tlvh);
 			break;
 		case TE_SUBTLV_AV_DELAY:
-			sum += show_vty_subtlv_av_delay(
-				vty, (struct te_subtlv_av_delay *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Average Link Delay!\n");
+				return;
+			}
+			sum += print_subtlv_av_delay(buf, indent,
+				(struct te_subtlv_av_delay *)tlvh);
 			break;
 		case TE_SUBTLV_MM_DELAY:
-			sum += show_vty_subtlv_mm_delay(
-				vty, (struct te_subtlv_mm_delay *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Min/Max Link Delay!\n");
+				return;
+			}
+			sum += print_subtlv_mm_delay(buf, indent,
+				(struct te_subtlv_mm_delay *)tlvh);
 			break;
 		case TE_SUBTLV_DELAY_VAR:
-			sum += show_vty_subtlv_delay_var(
-				vty, (struct te_subtlv_delay_var *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Delay Variation!\n");
+				return;
+			}
+			sum += print_subtlv_delay_var(buf, indent,
+				(struct te_subtlv_delay_var *)tlvh);
 			break;
 		case TE_SUBTLV_PKT_LOSS:
-			sum += show_vty_subtlv_pkt_loss(
-				vty, (struct te_subtlv_pkt_loss *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Link Packet Loss!\n");
+				return;
+			}
+			sum += print_subtlv_pkt_loss(buf, indent,
+				(struct te_subtlv_pkt_loss *)tlvh);
 			break;
 		case TE_SUBTLV_RES_BW:
-			sum += show_vty_subtlv_res_bw(
-				vty, (struct te_subtlv_res_bw *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Unidirectional Residual Bandwidth!\n");
+				return;
+			}
+			sum += print_subtlv_res_bw(buf, indent,
+				(struct te_subtlv_res_bw *)tlvh);
 			break;
 		case TE_SUBTLV_AVA_BW:
-			sum += show_vty_subtlv_ava_bw(
-				vty, (struct te_subtlv_ava_bw *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Unidirectional Available Bandwidth!\n");
+				return;
+			}
+			sum += print_subtlv_ava_bw(buf, indent,
+				(struct te_subtlv_ava_bw *)tlvh);
 			break;
 		case TE_SUBTLV_USE_BW:
-			sum += show_vty_subtlv_use_bw(
-				vty, (struct te_subtlv_use_bw *)tlvh);
+			if (tlvh->length != SUBTLV_DEF_SIZE) {
+				sbuf_push(buf, indent, "TLV size does not match expected size for Unidirectional Utilized Bandwidth!\n");
+				return;
+			}
+			sum += print_subtlv_use_bw(buf, indent,
+				(struct te_subtlv_use_bw *)tlvh);
 			break;
 		default:
-			sum += show_vty_unknown_tlv(vty, tlvh);
+			sum += print_unknown_tlv(buf, indent, tlvh);
 			break;
 		}
 	}
@@ -1087,9 +1076,9 @@ void mpls_te_print_detail(struct vty *vty, struct te_is_neigh *te)
 void isis_mpls_te_config_write_router(struct vty *vty)
 {
 	if (IS_MPLS_TE(isisMplsTE)) {
-		vty_out(vty, "  mpls-te on%s", VTY_NEWLINE);
-		vty_out(vty, "  mpls-te router-address %s%s",
-			inet_ntoa(isisMplsTE.router_id), VTY_NEWLINE);
+		vty_out(vty, "  mpls-te on\n");
+		vty_out(vty, "  mpls-te router-address %s\n",
+			inet_ntoa(isisMplsTE.router_id));
 	}
 
 	return;
@@ -1191,9 +1180,8 @@ DEFUN (isis_mpls_te_router_addr,
 	struct isis_area *area;
 
 	if (!inet_aton(argv[idx_ipv4]->arg, &value)) {
-		vty_out(vty, "Please specify Router-Addr by A.B.C.D%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Please specify Router-Addr by A.B.C.D\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	isisMplsTE.router_id.s_addr = value.s_addr;
@@ -1220,7 +1208,7 @@ DEFUN (isis_mpls_te_inter_as,
        "AREA native mode self originate INTER-AS LSP with L1 and L2 flooding scope)\n"
        "AS native mode self originate INTER-AS LSP with L2 only flooding scope\n")
 {
-	vty_out(vty, "Not yet supported%s", VTY_NEWLINE);
+	vty_out(vty, "Not yet supported\n");
 	return CMD_SUCCESS;
 }
 
@@ -1232,7 +1220,7 @@ DEFUN (no_isis_mpls_te_inter_as,
        "Disable MPLS-TE Inter-AS support\n")
 {
 
-	vty_out(vty, "Not yet supported%s", VTY_NEWLINE);
+	vty_out(vty, "Not yet supported\n");
 	return CMD_SUCCESS;
 }
 
@@ -1245,17 +1233,15 @@ DEFUN (show_isis_mpls_te_router,
        "Router information\n")
 {
 	if (IS_MPLS_TE(isisMplsTE)) {
-		vty_out(vty, "--- MPLS-TE router parameters ---%s",
-			VTY_NEWLINE);
+		vty_out(vty, "--- MPLS-TE router parameters ---\n");
 
 		if (ntohs(isisMplsTE.router_id.s_addr) != 0)
-			vty_out(vty, "  Router-Address: %s%s",
-				inet_ntoa(isisMplsTE.router_id), VTY_NEWLINE);
+			vty_out(vty, "  Router-Address: %s\n",
+				inet_ntoa(isisMplsTE.router_id));
 		else
-			vty_out(vty, "  N/A%s", VTY_NEWLINE);
+			vty_out(vty, "  N/A\n");
 	} else
-		vty_out(vty, "  MPLS-TE is disable on this router%s",
-			VTY_NEWLINE);
+		vty_out(vty, "  MPLS-TE is disable on this router\n");
 
 	return CMD_SUCCESS;
 }
@@ -1263,6 +1249,9 @@ DEFUN (show_isis_mpls_te_router,
 static void show_mpls_te_sub(struct vty *vty, struct interface *ifp)
 {
 	struct mpls_te_circuit *mtc;
+	struct sbuf buf;
+
+	sbuf_init(&buf, NULL, 0);
 
 	if ((IS_MPLS_TE(isisMplsTE))
 	    && ((mtc = lookup_mpls_params_by_ifp(ifp)) != NULL)) {
@@ -1271,54 +1260,58 @@ static void show_mpls_te_sub(struct vty *vty, struct interface *ifp)
 		if (mtc->status != enable) {
 			if (IS_INTER_AS(mtc->type)) {
 				vty_out(vty,
-					"-- Inter-AS TEv2 link parameters for %s --%s",
-					ifp->name, VTY_NEWLINE);
+					"-- Inter-AS TEv2 link parameters for %s --\n",
+					ifp->name);
 			} else {
 				/* MPLS-TE is not activate on this interface */
 				/* or this interface is passive and Inter-AS
 				 * TEv2 is not activate */
 				vty_out(vty,
-					"  %s: MPLS-TE is disabled on this interface%s",
-					ifp->name, VTY_NEWLINE);
+					"  %s: MPLS-TE is disabled on this interface\n",
+					ifp->name);
 				return;
 			}
 		} else {
-			vty_out(vty, "-- MPLS-TE link parameters for %s --%s",
-				ifp->name, VTY_NEWLINE);
+			vty_out(vty, "-- MPLS-TE link parameters for %s --\n",
+				ifp->name);
 		}
 
-		show_vty_subtlv_admin_grp(vty, &mtc->admin_grp);
+		sbuf_reset(&buf);
+		print_subtlv_admin_grp(&buf, 4, &mtc->admin_grp);
 
 		if (SUBTLV_TYPE(mtc->local_ipaddr) != 0)
-			show_vty_subtlv_local_ipaddr(vty, &mtc->local_ipaddr);
+			print_subtlv_local_ipaddr(&buf, 4, &mtc->local_ipaddr);
 		if (SUBTLV_TYPE(mtc->rmt_ipaddr) != 0)
-			show_vty_subtlv_rmt_ipaddr(vty, &mtc->rmt_ipaddr);
+			print_subtlv_rmt_ipaddr(&buf, 4, &mtc->rmt_ipaddr);
 
-		show_vty_subtlv_max_bw(vty, &mtc->max_bw);
-		show_vty_subtlv_max_rsv_bw(vty, &mtc->max_rsv_bw);
-		show_vty_subtlv_unrsv_bw(vty, &mtc->unrsv_bw);
-		show_vty_subtlv_te_metric(vty, &mtc->te_metric);
+		print_subtlv_max_bw(&buf, 4, &mtc->max_bw);
+		print_subtlv_max_rsv_bw(&buf, 4, &mtc->max_rsv_bw);
+		print_subtlv_unrsv_bw(&buf, 4, &mtc->unrsv_bw);
+		print_subtlv_te_metric(&buf, 4, &mtc->te_metric);
 
 		if (IS_INTER_AS(mtc->type)) {
 			if (SUBTLV_TYPE(mtc->ras) != 0)
-				show_vty_subtlv_ras(vty, &mtc->ras);
+				print_subtlv_ras(&buf, 4, &mtc->ras);
 			if (SUBTLV_TYPE(mtc->rip) != 0)
-				show_vty_subtlv_rip(vty, &mtc->rip);
+				print_subtlv_rip(&buf, 4, &mtc->rip);
 		}
 
-		show_vty_subtlv_av_delay(vty, &mtc->av_delay);
-		show_vty_subtlv_mm_delay(vty, &mtc->mm_delay);
-		show_vty_subtlv_delay_var(vty, &mtc->delay_var);
-		show_vty_subtlv_pkt_loss(vty, &mtc->pkt_loss);
-		show_vty_subtlv_res_bw(vty, &mtc->res_bw);
-		show_vty_subtlv_ava_bw(vty, &mtc->ava_bw);
-		show_vty_subtlv_use_bw(vty, &mtc->use_bw);
-		vty_out(vty, "---------------%s%s", VTY_NEWLINE, VTY_NEWLINE);
+		print_subtlv_av_delay(&buf, 4, &mtc->av_delay);
+		print_subtlv_mm_delay(&buf, 4, &mtc->mm_delay);
+		print_subtlv_delay_var(&buf, 4, &mtc->delay_var);
+		print_subtlv_pkt_loss(&buf, 4, &mtc->pkt_loss);
+		print_subtlv_res_bw(&buf, 4, &mtc->res_bw);
+		print_subtlv_ava_bw(&buf, 4, &mtc->ava_bw);
+		print_subtlv_use_bw(&buf, 4, &mtc->use_bw);
+
+		vty_multiline(vty, "", "%s", sbuf_buf(&buf));
+		vty_out(vty, "---------------\n\n");
 	} else {
-		vty_out(vty, "  %s: MPLS-TE is disabled on this interface%s",
-			ifp->name, VTY_NEWLINE);
+		vty_out(vty, "  %s: MPLS-TE is disabled on this interface\n",
+			ifp->name);
 	}
 
+	sbuf_free(&buf);
 	return;
 }
 
@@ -1331,13 +1324,13 @@ DEFUN (show_isis_mpls_te_interface,
        "Interface information\n"
        "Interface name\n")
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	int idx_interface = 4;
 	struct interface *ifp;
-	struct listnode *node;
 
 	/* Show All Interfaces. */
 	if (argc == 4) {
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp))
+		FOR_ALL_INTERFACES (vrf, ifp)
 			show_mpls_te_sub(vty, ifp);
 	}
 	/* Interface name is specified. */
@@ -1345,7 +1338,7 @@ DEFUN (show_isis_mpls_te_interface,
 		if ((ifp = if_lookup_by_name(argv[idx_interface]->arg,
 					     VRF_DEFAULT))
 		    == NULL)
-			vty_out(vty, "No such interface name%s", VTY_NEWLINE);
+			vty_out(vty, "No such interface name\n");
 		else
 			show_mpls_te_sub(vty, ifp);
 	}
