@@ -14,10 +14,10 @@
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
-
+ *
  * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -49,13 +49,13 @@
 #include "isisd/isis_pdu.h"
 #include "isisd/isis_misc.h"
 #include "isisd/isis_constants.h"
-#include "isisd/isis_tlv.h"
 #include "isisd/isis_lsp.h"
 #include "isisd/isis_spf.h"
 #include "isisd/isis_route.h"
 #include "isisd/isis_zebra.h"
 #include "isisd/isis_events.h"
 #include "isisd/isis_te.h"
+#include "isisd/isis_mt.h"
 
 struct isis *isis = NULL;
 
@@ -131,7 +131,7 @@ struct isis_area *isis_area_create(const char *area_tag)
 
 	area->circuit_list = list_new();
 	area->area_addrs = list_new();
-	THREAD_TIMER_ON(master, area->t_tick, lsp_tick, area, 1);
+	thread_add_timer(master, lsp_tick, area, 1, &area->t_tick);
 	flags_initialize(&area->flags);
 
 	/*
@@ -151,6 +151,8 @@ struct isis_area *isis_area_create(const char *area_tag)
 	area->lsp_frag_threshold = 90;
 	area->lsp_mtu = DEFAULT_LSP_MTU;
 
+	area_mt_init(area);
+
 	area->area_tag = strdup(area_tag);
 	listnode_add(isis->area_list, area);
 	area->isis = isis;
@@ -205,7 +207,7 @@ int isis_area_destroy(struct vty *vty, const char *area_tag)
 	area = isis_area_lookup(area_tag);
 
 	if (area == NULL) {
-		vty_out(vty, "Can't find ISIS instance %s", VTY_NEWLINE);
+		vty_out(vty, "Can't find ISIS instance \n");
 		return CMD_ERR_NO_MATCH;
 	}
 
@@ -218,8 +220,7 @@ int isis_area_destroy(struct vty *vty, const char *area_tag)
 			circuit->ipv6_router = 0;
 			isis_csm_state_change(ISIS_DISABLE, circuit, area);
 		}
-		list_delete(area->circuit_list);
-		area->circuit_list = NULL;
+		list_delete_and_null(&area->circuit_list);
 	}
 
 	if (area->lspdb[0] != NULL) {
@@ -278,6 +279,8 @@ int isis_area_destroy(struct vty *vty, const char *area_tag)
 
 	free(area->area_tag);
 
+	area_mt_finish(area);
+
 	XFREE(MTYPE_ISIS_AREA, area);
 
 	if (listcount(isis->area_list) == 0) {
@@ -288,6 +291,32 @@ int isis_area_destroy(struct vty *vty, const char *area_tag)
 	return CMD_SUCCESS;
 }
 
+static void area_set_mt_enabled(struct isis_area *area, uint16_t mtid,
+				bool enabled)
+{
+	struct isis_area_mt_setting *setting;
+
+	setting = area_get_mt_setting(area, mtid);
+	if (setting->enabled != enabled) {
+		setting->enabled = enabled;
+		lsp_regenerate_schedule(area, IS_LEVEL_1 | IS_LEVEL_2, 0);
+	}
+}
+
+static void area_set_mt_overload(struct isis_area *area, uint16_t mtid,
+				 bool overload)
+{
+	struct isis_area_mt_setting *setting;
+
+	setting = area_get_mt_setting(area, mtid);
+	if (setting->overload != overload) {
+		setting->overload = overload;
+		if (setting->enabled)
+			lsp_regenerate_schedule(area, IS_LEVEL_1 | IS_LEVEL_2,
+						0);
+	}
+}
+
 int area_net_title(struct vty *vty, const char *net_title)
 {
 	VTY_DECLVAR_CONTEXT(isis_area, area);
@@ -295,13 +324,13 @@ int area_net_title(struct vty *vty, const char *net_title)
 	struct area_addr *addrp;
 	struct listnode *node;
 
-	u_char buff[255];
+	uint8_t buff[255];
 
 	/* We check that we are not over the maximal number of addresses */
 	if (listcount(area->area_addrs) >= isis->max_area_addrs) {
 		vty_out(vty,
-			"Maximum of area addresses (%d) already reached %s",
-			isis->max_area_addrs, VTY_NEWLINE);
+			"Maximum of area addresses (%d) already reached \n",
+			isis->max_area_addrs);
 		return CMD_ERR_NOTHING_TODO;
 	}
 
@@ -314,18 +343,17 @@ int area_net_title(struct vty *vty, const char *net_title)
 #endif /* EXTREME_DEBUG */
 	if (addr->addr_len < 8 || addr->addr_len > 20) {
 		vty_out(vty,
-			"area address must be at least 8..20 octets long (%d)%s",
-			addr->addr_len, VTY_NEWLINE);
+			"area address must be at least 8..20 octets long (%d)\n",
+			addr->addr_len);
 		XFREE(MTYPE_ISIS_AREA_ADDR, addr);
-		return CMD_ERR_AMBIGUOUS;
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (addr->area_addr[addr->addr_len - 1] != 0) {
 		vty_out(vty,
-			"nsel byte (last byte) in area address must be 0%s",
-			VTY_NEWLINE);
+			"nsel byte (last byte) in area address must be 0\n");
 		XFREE(MTYPE_ISIS_AREA_ADDR, addr);
-		return CMD_ERR_AMBIGUOUS;
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (isis->sysid_set == 0) {
@@ -343,11 +371,9 @@ int area_net_title(struct vty *vty, const char *net_title)
 		 */
 		if (memcmp(isis->sysid, GETSYSID(addr), ISIS_SYS_ID_LEN)) {
 			vty_out(vty,
-				"System ID must not change when defining additional area"
-				" addresses%s",
-				VTY_NEWLINE);
+				"System ID must not change when defining additional area addresses\n");
 			XFREE(MTYPE_ISIS_AREA_ADDR, addr);
-			return CMD_ERR_AMBIGUOUS;
+			return CMD_WARNING_CONFIG_FAILED;
 		}
 
 		/* now we see that we don't already have this address */
@@ -385,14 +411,14 @@ int area_clear_net_title(struct vty *vty, const char *net_title)
 	VTY_DECLVAR_CONTEXT(isis_area, area);
 	struct area_addr addr, *addrp = NULL;
 	struct listnode *node;
-	u_char buff[255];
+	uint8_t buff[255];
 
 	addr.addr_len = dotformat2buff(buff, net_title);
 	if (addr.addr_len < 8 || addr.addr_len > 20) {
 		vty_out(vty,
-			"Unsupported area address length %d, should be 8...20 %s",
-			addr.addr_len, VTY_NEWLINE);
-		return CMD_ERR_AMBIGUOUS;
+			"Unsupported area address length %d, should be 8...20 \n",
+			addr.addr_len);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	memcpy(addr.area_addr, buff, (int)addr.addr_len);
@@ -403,8 +429,8 @@ int area_clear_net_title(struct vty *vty, const char *net_title)
 			break;
 
 	if (!addrp) {
-		vty_out(vty, "No area address %s for area %s %s", net_title,
-			area->area_tag, VTY_NEWLINE);
+		vty_out(vty, "No area address %s for area %s \n", net_title,
+			area->area_tag);
 		return CMD_ERR_NO_MATCH;
 	}
 
@@ -435,18 +461,16 @@ int show_isis_interface_common(struct vty *vty, const char *ifname, char detail)
 	struct isis_circuit *circuit;
 
 	if (!isis) {
-		vty_out(vty, "IS-IS Routing Process not enabled%s",
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS Routing Process not enabled\n");
 		return CMD_SUCCESS;
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(isis->area_list, anode, area)) {
-		vty_out(vty, "Area %s:%s", area->area_tag, VTY_NEWLINE);
+		vty_out(vty, "Area %s:\n", area->area_tag);
 
 		if (detail == ISIS_UI_LEVEL_BRIEF)
 			vty_out(vty,
-				"  Interface   CircId   State    Type     Level%s",
-				VTY_NEWLINE);
+				"  Interface   CircId   State    Type     Level\n");
 
 		for (ALL_LIST_ELEMENTS_RO(area->circuit_list, cnode, circuit))
 			if (!ifname)
@@ -504,12 +528,11 @@ int show_isis_neighbor_common(struct vty *vty, const char *id, char detail)
 	struct list *adjdb;
 	struct isis_adjacency *adj;
 	struct isis_dynhn *dynhn;
-	u_char sysid[ISIS_SYS_ID_LEN];
+	uint8_t sysid[ISIS_SYS_ID_LEN];
 	int i;
 
 	if (!isis) {
-		vty_out(vty, "IS-IS Routing Process not enabled%s",
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS Routing Process not enabled\n");
 		return CMD_SUCCESS;
 	}
 
@@ -518,8 +541,7 @@ int show_isis_neighbor_common(struct vty *vty, const char *id, char detail)
 		if (sysid2buff(sysid, id) == 0) {
 			dynhn = dynhn_find_by_name(id);
 			if (dynhn == NULL) {
-				vty_out(vty, "Invalid system id %s%s", id,
-					VTY_NEWLINE);
+				vty_out(vty, "Invalid system id %s\n", id);
 				return CMD_SUCCESS;
 			}
 			memcpy(sysid, dynhn->id, ISIS_SYS_ID_LEN);
@@ -527,13 +549,11 @@ int show_isis_neighbor_common(struct vty *vty, const char *id, char detail)
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(isis->area_list, anode, area)) {
-		vty_out(vty, "Area %s:%s", area->area_tag, VTY_NEWLINE);
+		vty_out(vty, "Area %s:\n", area->area_tag);
 
 		if (detail == ISIS_UI_LEVEL_BRIEF)
 			vty_out(vty,
-				"  System Id           Interface   L  State"
-				"        Holdtime SNPA%s",
-				VTY_NEWLINE);
+				"  System Id           Interface   L  State        Holdtime SNPA\n");
 
 		for (ALL_LIST_ELEMENTS_RO(area->circuit_list, cnode, circuit)) {
 			if (circuit->circ_type == CIRCUIT_T_BROADCAST) {
@@ -577,12 +597,11 @@ int clear_isis_neighbor_common(struct vty *vty, const char *id)
 	struct list *adjdb;
 	struct isis_adjacency *adj;
 	struct isis_dynhn *dynhn;
-	u_char sysid[ISIS_SYS_ID_LEN];
+	uint8_t sysid[ISIS_SYS_ID_LEN];
 	int i;
 
 	if (!isis) {
-		vty_out(vty, "IS-IS Routing Process not enabled%s",
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS Routing Process not enabled\n");
 		return CMD_SUCCESS;
 	}
 
@@ -591,8 +610,7 @@ int clear_isis_neighbor_common(struct vty *vty, const char *id)
 		if (sysid2buff(sysid, id) == 0) {
 			dynhn = dynhn_find_by_name(id);
 			if (dynhn == NULL) {
-				vty_out(vty, "Invalid system id %s%s", id,
-					VTY_NEWLINE);
+				vty_out(vty, "Invalid system id %s\n", id);
 				return CMD_SUCCESS;
 			}
 			memcpy(sysid, dynhn->id, ISIS_SYS_ID_LEN);
@@ -704,61 +722,53 @@ void print_debug(struct vty *vty, int flags, int onoff)
 
 	if (flags & DEBUG_ADJ_PACKETS)
 		vty_out(vty,
-			"IS-IS Adjacency related packets debugging is %s%s",
-			onoffs, VTY_NEWLINE);
+			"IS-IS Adjacency related packets debugging is %s\n",
+			onoffs);
 	if (flags & DEBUG_CHECKSUM_ERRORS)
-		vty_out(vty, "IS-IS checksum errors debugging is %s%s", onoffs,
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS checksum errors debugging is %s\n", onoffs);
 	if (flags & DEBUG_LOCAL_UPDATES)
-		vty_out(vty, "IS-IS local updates debugging is %s%s", onoffs,
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS local updates debugging is %s\n", onoffs);
 	if (flags & DEBUG_PROTOCOL_ERRORS)
-		vty_out(vty, "IS-IS protocol errors debugging is %s%s", onoffs,
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS protocol errors debugging is %s\n", onoffs);
 	if (flags & DEBUG_SNP_PACKETS)
-		vty_out(vty, "IS-IS CSNP/PSNP packets debugging is %s%s",
-			onoffs, VTY_NEWLINE);
+		vty_out(vty, "IS-IS CSNP/PSNP packets debugging is %s\n",
+			onoffs);
 	if (flags & DEBUG_SPF_EVENTS)
-		vty_out(vty, "IS-IS SPF events debugging is %s%s", onoffs,
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS SPF events debugging is %s\n", onoffs);
 	if (flags & DEBUG_SPF_STATS)
 		vty_out(vty,
-			"IS-IS SPF Timing and Statistics Data debugging is %s%s",
-			onoffs, VTY_NEWLINE);
+			"IS-IS SPF Timing and Statistics Data debugging is %s\n",
+			onoffs);
 	if (flags & DEBUG_SPF_TRIGGERS)
-		vty_out(vty, "IS-IS SPF triggering events debugging is %s%s",
-			onoffs, VTY_NEWLINE);
+		vty_out(vty, "IS-IS SPF triggering events debugging is %s\n",
+			onoffs);
 	if (flags & DEBUG_UPDATE_PACKETS)
-		vty_out(vty, "IS-IS Update related packet debugging is %s%s",
-			onoffs, VTY_NEWLINE);
+		vty_out(vty, "IS-IS Update related packet debugging is %s\n",
+			onoffs);
 	if (flags & DEBUG_RTE_EVENTS)
-		vty_out(vty, "IS-IS Route related debuggin is %s%s", onoffs,
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS Route related debuggin is %s\n", onoffs);
 	if (flags & DEBUG_EVENTS)
-		vty_out(vty, "IS-IS Event debugging is %s%s", onoffs,
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS Event debugging is %s\n", onoffs);
 	if (flags & DEBUG_PACKET_DUMP)
-		vty_out(vty, "IS-IS Packet dump debugging is %s%s", onoffs,
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS Packet dump debugging is %s\n", onoffs);
 	if (flags & DEBUG_LSP_GEN)
-		vty_out(vty, "IS-IS LSP generation debugging is %s%s", onoffs,
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS LSP generation debugging is %s\n", onoffs);
 	if (flags & DEBUG_LSP_SCHED)
-		vty_out(vty, "IS-IS LSP scheduling debugging is %s%s", onoffs,
-			VTY_NEWLINE);
+		vty_out(vty, "IS-IS LSP scheduling debugging is %s\n", onoffs);
 }
 
-DEFUN (show_debugging,
-       show_debugging_isis_cmd,
-       "show debugging isis",
-       SHOW_STR
-       "State of each debugging option\n"
-       ISIS_STR)
+DEFUN_NOSH (show_debugging,
+	    show_debugging_isis_cmd,
+	    "show debugging [isis]",
+	    SHOW_STR
+	    "State of each debugging option\n"
+	    ISIS_STR)
 {
-	if (isis->debugs) {
-		vty_out(vty, "IS-IS:%s", VTY_NEWLINE);
+	vty_out(vty, "IS-IS debugging status:\n");
+
+	if (isis->debugs)
 		print_debug(vty, isis->debugs, 1);
-	}
+
 	return CMD_SUCCESS;
 }
 
@@ -771,59 +781,59 @@ static int config_write_debug(struct vty *vty)
 	int flags = isis->debugs;
 
 	if (flags & DEBUG_ADJ_PACKETS) {
-		vty_out(vty, "debug isis adj-packets%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis adj-packets\n");
 		write++;
 	}
 	if (flags & DEBUG_CHECKSUM_ERRORS) {
-		vty_out(vty, "debug isis checksum-errors%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis checksum-errors\n");
 		write++;
 	}
 	if (flags & DEBUG_LOCAL_UPDATES) {
-		vty_out(vty, "debug isis local-updates%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis local-updates\n");
 		write++;
 	}
 	if (flags & DEBUG_PROTOCOL_ERRORS) {
-		vty_out(vty, "debug isis protocol-errors%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis protocol-errors\n");
 		write++;
 	}
 	if (flags & DEBUG_SNP_PACKETS) {
-		vty_out(vty, "debug isis snp-packets%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis snp-packets\n");
 		write++;
 	}
 	if (flags & DEBUG_SPF_EVENTS) {
-		vty_out(vty, "debug isis spf-events%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis spf-events\n");
 		write++;
 	}
 	if (flags & DEBUG_SPF_STATS) {
-		vty_out(vty, "debug isis spf-statistics%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis spf-statistics\n");
 		write++;
 	}
 	if (flags & DEBUG_SPF_TRIGGERS) {
-		vty_out(vty, "debug isis spf-triggers%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis spf-triggers\n");
 		write++;
 	}
 	if (flags & DEBUG_UPDATE_PACKETS) {
-		vty_out(vty, "debug isis update-packets%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis update-packets\n");
 		write++;
 	}
 	if (flags & DEBUG_RTE_EVENTS) {
-		vty_out(vty, "debug isis route-events%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis route-events\n");
 		write++;
 	}
 	if (flags & DEBUG_EVENTS) {
-		vty_out(vty, "debug isis events%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis events\n");
 		write++;
 	}
 	if (flags & DEBUG_PACKET_DUMP) {
-		vty_out(vty, "debug isis packet-dump%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis packet-dump\n");
 		write++;
 	}
 	if (flags & DEBUG_LSP_GEN) {
-		vty_out(vty, "debug isis lsp-gen%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis lsp-gen\n");
 		write++;
 	}
 	if (flags & DEBUG_LSP_SCHED) {
-		vty_out(vty, "debug isis lsp-sched%s", VTY_NEWLINE);
+		vty_out(vty, "debug isis lsp-sched\n");
 		write++;
 	}
 	write += spf_backoff_write_config(vty);
@@ -1221,27 +1231,6 @@ DEFUN (show_hostname,
 	return CMD_SUCCESS;
 }
 
-static void vty_out_timestr(struct vty *vty, time_t uptime)
-{
-	struct tm *tm;
-	time_t difftime = time(NULL);
-	difftime -= uptime;
-	tm = gmtime(&difftime);
-
-#define ONE_DAY_SECOND 60*60*24
-#define ONE_WEEK_SECOND 60*60*24*7
-	if (difftime < ONE_DAY_SECOND)
-		vty_out(vty, "%02d:%02d:%02d", tm->tm_hour, tm->tm_min,
-			tm->tm_sec);
-	else if (difftime < ONE_WEEK_SECOND)
-		vty_out(vty, "%dd%02dh%02dm", tm->tm_yday, tm->tm_hour,
-			tm->tm_min);
-	else
-		vty_out(vty, "%02dw%dd%02dh", tm->tm_yday / 7,
-			tm->tm_yday - ((tm->tm_yday / 7) * 7), tm->tm_hour);
-	vty_out(vty, " ago");
-}
-
 DEFUN (show_isis_spf_ietf,
        show_isis_spf_ietf_cmd,
        "show isis spf-delay-ietf",
@@ -1250,7 +1239,7 @@ DEFUN (show_isis_spf_ietf,
        "IS-IS SPF delay IETF information\n")
 {
 	if (!isis) {
-		vty_out(vty, "ISIS is not running%s", VTY_NEWLINE);
+		vty_out(vty, "ISIS is not running\n");
 		return CMD_SUCCESS;
 	}
 
@@ -1258,36 +1247,33 @@ DEFUN (show_isis_spf_ietf,
 	struct isis_area *area;
 
 	for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area)) {
-		vty_out(vty, "Area %s:%s",
-			area->area_tag ? area->area_tag : "null", VTY_NEWLINE);
+		vty_out(vty, "Area %s:\n",
+			area->area_tag ? area->area_tag : "null");
 
 		for (int level = ISIS_LEVEL1; level <= ISIS_LEVELS; level++) {
 			if ((area->is_type & level) == 0)
 				continue;
 
-			vty_out(vty, "  Level-%d:%s", level, VTY_NEWLINE);
+			vty_out(vty, "  Level-%d:\n", level);
 			vty_out(vty, "    SPF delay status: ");
 			if (area->spf_timer[level - 1]) {
 				struct timeval remain = thread_timer_remain(
 					area->spf_timer[level - 1]);
-				vty_out(vty, "Pending, due in %ld msec%s",
-					remain.tv_sec * 1000
-						+ remain.tv_usec / 1000,
-					VTY_NEWLINE);
+				vty_out(vty, "Pending, due in %lld msec\n",
+					(long long)remain.tv_sec * 1000
+						+ remain.tv_usec / 1000);
 			} else {
-				vty_out(vty, "Not scheduled%s", VTY_NEWLINE);
+				vty_out(vty, "Not scheduled\n");
 			}
 
 			if (area->spf_delay_ietf[level - 1]) {
 				vty_out(vty,
-					"    Using draft-ietf-rtgwg-backoff-algo-04%s",
-					VTY_NEWLINE);
+					"    Using draft-ietf-rtgwg-backoff-algo-04\n");
 				spf_backoff_show(
 					area->spf_delay_ietf[level - 1], vty,
 					"    ");
 			} else {
-				vty_out(vty, "    Using legacy backoff algo%s",
-					VTY_NEWLINE);
+				vty_out(vty, "    Using legacy backoff algo\n");
 			}
 		}
 	}
@@ -1301,41 +1287,38 @@ DEFUN (show_isis_summary,
 {
 	struct listnode *node, *node2;
 	struct isis_area *area;
-	struct isis_spftree *spftree;
 	int level;
 
 	if (isis == NULL) {
-		vty_out(vty, "ISIS is not running%s", VTY_NEWLINE);
+		vty_out(vty, "ISIS is not running\n");
 		return CMD_SUCCESS;
 	}
 
-	vty_out(vty, "Process Id      : %ld%s", isis->process_id, VTY_NEWLINE);
+	vty_out(vty, "Process Id      : %ld\n", isis->process_id);
 	if (isis->sysid_set)
-		vty_out(vty, "System Id       : %s%s", sysid_print(isis->sysid),
-			VTY_NEWLINE);
+		vty_out(vty, "System Id       : %s\n",
+			sysid_print(isis->sysid));
 
 	vty_out(vty, "Up time         : ");
 	vty_out_timestr(vty, isis->uptime);
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 
 	if (isis->area_list)
-		vty_out(vty, "Number of areas : %d%s", isis->area_list->count,
-			VTY_NEWLINE);
+		vty_out(vty, "Number of areas : %d\n", isis->area_list->count);
 
 	for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area)) {
-		vty_out(vty, "Area %s:%s",
-			area->area_tag ? area->area_tag : "null", VTY_NEWLINE);
+		vty_out(vty, "Area %s:\n",
+			area->area_tag ? area->area_tag : "null");
 
 		if (listcount(area->area_addrs) > 0) {
 			struct area_addr *area_addr;
 			for (ALL_LIST_ELEMENTS_RO(area->area_addrs, node2,
 						  area_addr)) {
-				vty_out(vty, "  Net: %s%s",
+				vty_out(vty, "  Net: %s\n",
 					isonet_print(area_addr->area_addr,
 						     area_addr->addr_len
 							     + ISIS_SYS_ID_LEN
-							     + 1),
-					VTY_NEWLINE);
+							     + 1));
 			}
 		}
 
@@ -1343,51 +1326,27 @@ DEFUN (show_isis_summary,
 			if ((area->is_type & level) == 0)
 				continue;
 
-			vty_out(vty, "  Level-%d:%s", level, VTY_NEWLINE);
-			spftree = area->spftree[level - 1];
+			vty_out(vty, "  Level-%d:\n", level);
 			if (area->spf_timer[level - 1])
-				vty_out(vty, "    SPF: (pending)%s",
-					VTY_NEWLINE);
+				vty_out(vty, "    SPF: (pending)\n");
 			else
-				vty_out(vty, "    SPF:%s", VTY_NEWLINE);
+				vty_out(vty, "    SPF:\n");
 
 			vty_out(vty, "      minimum interval  : %d",
 				area->min_spf_interval[level - 1]);
 			if (area->spf_delay_ietf[level - 1])
 				vty_out(vty,
 					" (not used, IETF SPF delay activated)");
-			vty_out(vty, VTY_NEWLINE);
+			vty_out(vty, "\n");
 
-			vty_out(vty, "    IPv4 route computation:%s",
-				VTY_NEWLINE);
-			vty_out(vty, "      last run elapsed  : ");
-			vty_out_timestr(vty, spftree->last_run_timestamp);
-			vty_out(vty, "%s", VTY_NEWLINE);
+			vty_out(vty, "    IPv4 route computation:\n");
+			isis_spf_print(area->spftree[level - 1], vty);
 
-			vty_out(vty, "      last run duration : %u usec%s",
-				(u_int32_t)spftree->last_run_duration,
-				VTY_NEWLINE);
-
-			vty_out(vty, "      run count         : %d%s",
-				spftree->runcount, VTY_NEWLINE);
-
-			spftree = area->spftree6[level - 1];
-			vty_out(vty, "    IPv6 route computation:%s",
-				VTY_NEWLINE);
-
-			vty_out(vty, "      last run elapsed  : ");
-			vty_out_timestr(vty, spftree->last_run_timestamp);
-			vty_out(vty, "%s", VTY_NEWLINE);
-
-			vty_out(vty, "      last run duration : %llu msec%s",
-				(unsigned long long)spftree->last_run_duration,
-				VTY_NEWLINE);
-
-			vty_out(vty, "      run count         : %d%s",
-				spftree->runcount, VTY_NEWLINE);
+			vty_out(vty, "    IPv6 route computation:\n");
+			isis_spf_print(area->spftree6[level - 1], vty);
 		}
 	}
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 
 	return CMD_SUCCESS;
 }
@@ -1415,9 +1374,9 @@ static int show_isis_database(struct vty *vty, const char *argv, int ui_level)
 	struct isis_lsp *lsp;
 	struct isis_dynhn *dynhn;
 	const char *pos = argv;
-	u_char lspid[ISIS_SYS_ID_LEN + 2];
+	uint8_t lspid[ISIS_SYS_ID_LEN + 2];
 	char sysid[255];
-	u_char number[3];
+	uint8_t number[3];
 	int level, lsp_count;
 
 	if (isis->area_list->count == 0)
@@ -1442,22 +1401,22 @@ static int show_isis_database(struct vty *vty, const char *argv, int ui_level)
 		if (strncmp(pos, "-", 1) == 0) {
 			memcpy(number, ++pos, 2);
 			lspid[ISIS_SYS_ID_LEN + 1] =
-				(u_char)strtol((char *)number, NULL, 16);
+				(uint8_t)strtol((char *)number, NULL, 16);
 			pos -= 4;
 			if (strncmp(pos, ".", 1) != 0)
-				return CMD_ERR_AMBIGUOUS;
+				return CMD_WARNING;
 		}
 		if (strncmp(pos, ".", 1) == 0) {
 			memcpy(number, ++pos, 2);
 			lspid[ISIS_SYS_ID_LEN] =
-				(u_char)strtol((char *)number, NULL, 16);
+				(uint8_t)strtol((char *)number, NULL, 16);
 			sysid[pos - argv - 1] = '\0';
 		}
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area)) {
-		vty_out(vty, "Area %s:%s",
-			area->area_tag ? area->area_tag : "null", VTY_NEWLINE);
+		vty_out(vty, "Area %s:\n",
+			area->area_tag ? area->area_tag : "null");
 
 		for (level = 0; level < ISIS_LEVELS; level++) {
 			if (area->lspdb[level]
@@ -1481,7 +1440,7 @@ static int show_isis_database(struct vty *vty, const char *argv, int ui_level)
 						lsp = lsp_search(
 							lspid,
 							area->lspdb[level]);
-					} else if (strncmp(unix_hostname(),
+					} else if (strncmp(cmd_hostname_get(),
 							   sysid, 15)
 						   == 0) {
 						memcpy(lspid, isis->sysid,
@@ -1494,14 +1453,12 @@ static int show_isis_database(struct vty *vty, const char *argv, int ui_level)
 
 				if (lsp != NULL || argv == NULL) {
 					vty_out(vty,
-						"IS-IS Level-%d link-state database:%s",
-						level + 1, VTY_NEWLINE);
+						"IS-IS Level-%d link-state database:\n",
+						level + 1);
 
 					/* print the title in all cases */
 					vty_out(vty,
-						"LSP ID                  PduLen  "
-						"SeqNumber   Chksum  Holdtime  ATT/P/OL%s",
-						VTY_NEWLINE);
+						"LSP ID                  PduLen  SeqNumber   Chksum  Holdtime  ATT/P/OL\n");
 				}
 
 				if (lsp) {
@@ -1517,9 +1474,8 @@ static int show_isis_database(struct vty *vty, const char *argv, int ui_level)
 						vty, area->lspdb[level],
 						ui_level, area->dynhostname);
 
-					vty_out(vty, "    %u LSPs%s%s",
-						lsp_count, VTY_NEWLINE,
-						VTY_NEWLINE);
+					vty_out(vty, "    %u LSPs\n\n",
+						lsp_count);
 				}
 			}
 		}
@@ -1553,7 +1509,7 @@ DEFUN_NOSH (router_isis,
        "router isis WORD",
        ROUTER_STR
        "ISO IS-IS\n"
-       "ISO Routing area tag")
+       "ISO Routing area tag\n")
 {
 	int idx_word = 2;
 	return isis_area_get(vty, argv[idx_word]->arg);
@@ -1565,7 +1521,7 @@ DEFUN_NOSH (router_isis,
 DEFUN (no_router_isis,
        no_router_isis_cmd,
        "no router isis WORD",
-       "no\n" ROUTER_STR "ISO IS-IS\n" "ISO Routing area tag")
+       "no\n" ROUTER_STR "ISO IS-IS\n" "ISO Routing area tag\n")
 {
 	int idx_word = 3;
 	return isis_area_destroy(vty, argv[idx_word]->arg);
@@ -1598,6 +1554,71 @@ DEFUN (no_net,
 	return area_clear_net_title(vty, argv[idx_word]->arg);
 }
 
+DEFUN (isis_topology,
+       isis_topology_cmd,
+       "topology " ISIS_MT_NAMES " [overload]",
+       "Configure IS-IS topologies\n"
+       ISIS_MT_DESCRIPTIONS
+       "Set overload bit for topology\n")
+{
+	VTY_DECLVAR_CONTEXT(isis_area, area);
+
+	const char *arg = argv[1]->arg;
+	uint16_t mtid = isis_str2mtid(arg);
+
+	if (area->oldmetric) {
+		vty_out(vty,
+			"Multi topology IS-IS can only be used with wide metrics\n");
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
+	if (mtid == (uint16_t)-1) {
+		vty_out(vty, "Don't know topology '%s'\n", arg);
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+	if (mtid == ISIS_MT_IPV4_UNICAST) {
+		vty_out(vty, "Cannot configure IPv4 unicast topology\n");
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
+	area_set_mt_enabled(area, mtid, true);
+	area_set_mt_overload(area, mtid, (argc == 3));
+	return CMD_SUCCESS;
+}
+
+DEFUN (no_isis_topology,
+       no_isis_topology_cmd,
+       "no topology " ISIS_MT_NAMES " [overload]",
+       NO_STR
+       "Configure IS-IS topologies\n"
+       ISIS_MT_DESCRIPTIONS
+       "Set overload bit for topology\n")
+{
+	VTY_DECLVAR_CONTEXT(isis_area, area);
+
+	const char *arg = argv[2]->arg;
+	uint16_t mtid = isis_str2mtid(arg);
+
+	if (area->oldmetric) {
+		vty_out(vty,
+			"Multi topology IS-IS can only be used with wide metrics\n");
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
+	if (mtid == (uint16_t)-1) {
+		vty_out(vty, "Don't know topology '%s'\n", arg);
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+	if (mtid == ISIS_MT_IPV4_UNICAST) {
+		vty_out(vty, "Cannot configure IPv4 unicast topology\n");
+		return CMD_WARNING_CONFIG_FAILED;
+	}
+
+	area_set_mt_enabled(area, mtid, false);
+	area_set_mt_overload(area, mtid, false);
+	return CMD_SUCCESS;
+}
+
 void isis_area_lsp_mtu_set(struct isis_area *area, unsigned int lsp_mtu)
 {
 	area->lsp_mtu = lsp_mtu;
@@ -1605,8 +1626,8 @@ void isis_area_lsp_mtu_set(struct isis_area *area, unsigned int lsp_mtu)
 }
 
 static int isis_area_passwd_set(struct isis_area *area, int level,
-				u_char passwd_type, const char *passwd,
-				u_char snp_auth)
+				uint8_t passwd_type, const char *passwd,
+				uint8_t snp_auth)
 {
 	struct isis_passwd *dest;
 	struct isis_passwd modified;
@@ -1646,14 +1667,14 @@ int isis_area_passwd_unset(struct isis_area *area, int level)
 }
 
 int isis_area_passwd_cleartext_set(struct isis_area *area, int level,
-				   const char *passwd, u_char snp_auth)
+				   const char *passwd, uint8_t snp_auth)
 {
 	return isis_area_passwd_set(area, level, ISIS_PASSWD_TYPE_CLEARTXT,
 				    passwd, snp_auth);
 }
 
 int isis_area_passwd_hmac_md5_set(struct isis_area *area, int level,
-				  const char *passwd, u_char snp_auth)
+				  const char *passwd, uint8_t snp_auth)
 {
 	return isis_area_passwd_set(area, level, ISIS_PASSWD_TYPE_HMAC_MD5,
 				    passwd, snp_auth);
@@ -1858,21 +1879,19 @@ int isis_config_write(struct vty *vty)
 
 		for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area)) {
 			/* ISIS - Area name */
-			vty_out(vty, "router isis %s%s", area->area_tag,
-				VTY_NEWLINE);
+			vty_out(vty, "router isis %s\n", area->area_tag);
 			write++;
 			/* ISIS - Net */
 			if (listcount(area->area_addrs) > 0) {
 				struct area_addr *area_addr;
 				for (ALL_LIST_ELEMENTS_RO(area->area_addrs,
 							  node2, area_addr)) {
-					vty_out(vty, " net %s%s",
+					vty_out(vty, " net %s\n",
 						isonet_print(
 							area_addr->area_addr,
 							area_addr->addr_len
 								+ ISIS_SYS_ID_LEN
-								+ 1),
-						VTY_NEWLINE);
+								+ 1));
 					write++;
 				}
 			}
@@ -1880,38 +1899,32 @@ int isis_config_write(struct vty *vty)
 			 * display if
 			 * false. */
 			if (!area->dynhostname) {
-				vty_out(vty, " no hostname dynamic%s",
-					VTY_NEWLINE);
+				vty_out(vty, " no hostname dynamic\n");
 				write++;
 			}
 			/* ISIS - Metric-Style - when true displays wide */
 			if (area->newmetric) {
 				if (!area->oldmetric)
-					vty_out(vty, " metric-style wide%s",
-						VTY_NEWLINE);
+					vty_out(vty, " metric-style wide\n");
 				else
 					vty_out(vty,
-						" metric-style transition%s",
-						VTY_NEWLINE);
+						" metric-style transition\n");
 				write++;
 			} else {
-				vty_out(vty, " metric-style narrow%s",
-					VTY_NEWLINE);
+				vty_out(vty, " metric-style narrow\n");
 				write++;
 			}
 			/* ISIS - overload-bit */
 			if (area->overload_bit) {
-				vty_out(vty, " set-overload-bit%s",
-					VTY_NEWLINE);
+				vty_out(vty, " set-overload-bit\n");
 				write++;
 			}
 			/* ISIS - Area is-type (level-1-2 is default) */
 			if (area->is_type == IS_LEVEL_1) {
-				vty_out(vty, " is-type level-1%s", VTY_NEWLINE);
+				vty_out(vty, " is-type level-1\n");
 				write++;
 			} else if (area->is_type == IS_LEVEL_2) {
-				vty_out(vty, " is-type level-2-only%s",
-					VTY_NEWLINE);
+				vty_out(vty, " is-type level-2-only\n");
 				write++;
 			}
 			write += isis_redist_config_write(vty, area, AF_INET);
@@ -1921,26 +1934,23 @@ int isis_config_write(struct vty *vty)
 			    == area->lsp_gen_interval[1]) {
 				if (area->lsp_gen_interval[0]
 				    != DEFAULT_MIN_LSP_GEN_INTERVAL) {
-					vty_out(vty, " lsp-gen-interval %d%s",
-						area->lsp_gen_interval[0],
-						VTY_NEWLINE);
+					vty_out(vty, " lsp-gen-interval %d\n",
+						area->lsp_gen_interval[0]);
 					write++;
 				}
 			} else {
 				if (area->lsp_gen_interval[0]
 				    != DEFAULT_MIN_LSP_GEN_INTERVAL) {
 					vty_out(vty,
-						" lsp-gen-interval level-1 %d%s",
-						area->lsp_gen_interval[0],
-						VTY_NEWLINE);
+						" lsp-gen-interval level-1 %d\n",
+						area->lsp_gen_interval[0]);
 					write++;
 				}
 				if (area->lsp_gen_interval[1]
 				    != DEFAULT_MIN_LSP_GEN_INTERVAL) {
 					vty_out(vty,
-						" lsp-gen-interval level-2 %d%s",
-						area->lsp_gen_interval[1],
-						VTY_NEWLINE);
+						" lsp-gen-interval level-2 %d\n",
+						area->lsp_gen_interval[1]);
 					write++;
 				}
 			}
@@ -1949,26 +1959,23 @@ int isis_config_write(struct vty *vty)
 			    == area->max_lsp_lifetime[1]) {
 				if (area->max_lsp_lifetime[0]
 				    != DEFAULT_LSP_LIFETIME) {
-					vty_out(vty, " max-lsp-lifetime %u%s",
-						area->max_lsp_lifetime[0],
-						VTY_NEWLINE);
+					vty_out(vty, " max-lsp-lifetime %u\n",
+						area->max_lsp_lifetime[0]);
 					write++;
 				}
 			} else {
 				if (area->max_lsp_lifetime[0]
 				    != DEFAULT_LSP_LIFETIME) {
 					vty_out(vty,
-						" max-lsp-lifetime level-1 %u%s",
-						area->max_lsp_lifetime[0],
-						VTY_NEWLINE);
+						" max-lsp-lifetime level-1 %u\n",
+						area->max_lsp_lifetime[0]);
 					write++;
 				}
 				if (area->max_lsp_lifetime[1]
 				    != DEFAULT_LSP_LIFETIME) {
 					vty_out(vty,
-						" max-lsp-lifetime level-2 %u%s",
-						area->max_lsp_lifetime[1],
-						VTY_NEWLINE);
+						" max-lsp-lifetime level-2 %u\n",
+						area->max_lsp_lifetime[1]);
 					write++;
 				}
 			}
@@ -1977,32 +1984,28 @@ int isis_config_write(struct vty *vty)
 				if (area->lsp_refresh[0]
 				    != DEFAULT_MAX_LSP_GEN_INTERVAL) {
 					vty_out(vty,
-						" lsp-refresh-interval %u%s",
-						area->lsp_refresh[0],
-						VTY_NEWLINE);
+						" lsp-refresh-interval %u\n",
+						area->lsp_refresh[0]);
 					write++;
 				}
 			} else {
 				if (area->lsp_refresh[0]
 				    != DEFAULT_MAX_LSP_GEN_INTERVAL) {
 					vty_out(vty,
-						" lsp-refresh-interval level-1 %u%s",
-						area->lsp_refresh[0],
-						VTY_NEWLINE);
+						" lsp-refresh-interval level-1 %u\n",
+						area->lsp_refresh[0]);
 					write++;
 				}
 				if (area->lsp_refresh[1]
 				    != DEFAULT_MAX_LSP_GEN_INTERVAL) {
 					vty_out(vty,
-						" lsp-refresh-interval level-2 %u%s",
-						area->lsp_refresh[1],
-						VTY_NEWLINE);
+						" lsp-refresh-interval level-2 %u\n",
+						area->lsp_refresh[1]);
 					write++;
 				}
 			}
 			if (area->lsp_mtu != DEFAULT_LSP_MTU) {
-				vty_out(vty, " lsp-mtu %u%s", area->lsp_mtu,
-					VTY_NEWLINE);
+				vty_out(vty, " lsp-mtu %u\n", area->lsp_mtu);
 				write++;
 			}
 
@@ -2011,26 +2014,23 @@ int isis_config_write(struct vty *vty)
 			    == area->min_spf_interval[1]) {
 				if (area->min_spf_interval[0]
 				    != MINIMUM_SPF_INTERVAL) {
-					vty_out(vty, " spf-interval %d%s",
-						area->min_spf_interval[0],
-						VTY_NEWLINE);
+					vty_out(vty, " spf-interval %d\n",
+						area->min_spf_interval[0]);
 					write++;
 				}
 			} else {
 				if (area->min_spf_interval[0]
 				    != MINIMUM_SPF_INTERVAL) {
 					vty_out(vty,
-						" spf-interval level-1 %d%s",
-						area->min_spf_interval[0],
-						VTY_NEWLINE);
+						" spf-interval level-1 %d\n",
+						area->min_spf_interval[0]);
 					write++;
 				}
 				if (area->min_spf_interval[1]
 				    != MINIMUM_SPF_INTERVAL) {
 					vty_out(vty,
-						" spf-interval level-2 %d%s",
-						area->min_spf_interval[1],
-						VTY_NEWLINE);
+						" spf-interval level-2 %d\n",
+						area->min_spf_interval[1]);
 					write++;
 				}
 			}
@@ -2038,7 +2038,7 @@ int isis_config_write(struct vty *vty)
 			/* IETF SPF interval */
 			if (area->spf_delay_ietf[0]) {
 				vty_out(vty,
-					" spf-delay-ietf init-delay %ld short-delay %ld long-delay %ld holddown %ld time-to-learn %ld%s",
+					" spf-delay-ietf init-delay %ld short-delay %ld long-delay %ld holddown %ld time-to-learn %ld\n",
 					spf_backoff_init_delay(
 						area->spf_delay_ietf[0]),
 					spf_backoff_short_delay(
@@ -2048,8 +2048,7 @@ int isis_config_write(struct vty *vty)
 					spf_backoff_holddown(
 						area->spf_delay_ietf[0]),
 					spf_backoff_timetolearn(
-						area->spf_delay_ietf[0]),
-					VTY_NEWLINE);
+						area->spf_delay_ietf[0]));
 				write++;
 			}
 
@@ -2068,7 +2067,7 @@ int isis_config_write(struct vty *vty)
 					else
 						vty_out(vty, "send-only");
 				}
-				vty_out(vty, "%s", VTY_NEWLINE);
+				vty_out(vty, "\n");
 				write++;
 			} else if (area->area_passwd.type
 				   == ISIS_PASSWD_TYPE_CLEARTXT) {
@@ -2084,7 +2083,7 @@ int isis_config_write(struct vty *vty)
 					else
 						vty_out(vty, "send-only");
 				}
-				vty_out(vty, "%s", VTY_NEWLINE);
+				vty_out(vty, "\n");
 				write++;
 			}
 			if (area->domain_passwd.type
@@ -2101,7 +2100,7 @@ int isis_config_write(struct vty *vty)
 					else
 						vty_out(vty, "send-only");
 				}
-				vty_out(vty, "%s", VTY_NEWLINE);
+				vty_out(vty, "\n");
 				write++;
 			} else if (area->domain_passwd.type
 				   == ISIS_PASSWD_TYPE_CLEARTXT) {
@@ -2117,15 +2116,16 @@ int isis_config_write(struct vty *vty)
 					else
 						vty_out(vty, "send-only");
 				}
-				vty_out(vty, "%s", VTY_NEWLINE);
+				vty_out(vty, "\n");
 				write++;
 			}
 
 			if (area->log_adj_changes) {
-				vty_out(vty, " log-adjacency-changes%s",
-					VTY_NEWLINE);
+				vty_out(vty, " log-adjacency-changes\n");
 				write++;
 			}
+
+			write += area_write_mt_settings(area, vty);
 		}
 		isis_mpls_te_config_write_router(vty);
 	}
@@ -2227,6 +2227,9 @@ void isis_init()
 	install_element(ISIS_NODE, &net_cmd);
 	install_element(ISIS_NODE, &no_net_cmd);
 
+	install_element(ISIS_NODE, &isis_topology_cmd);
+	install_element(ISIS_NODE, &no_isis_topology_cmd);
+
 	install_element(ISIS_NODE, &log_adj_changes_cmd);
 	install_element(ISIS_NODE, &no_log_adj_changes_cmd);
 
