@@ -53,6 +53,7 @@ static void		 main_imsg_send_net_sockets(int);
 static void		 main_imsg_send_net_socket(int, enum socket_type);
 static int		 main_imsg_send_config(struct ldpd_conf *);
 static void		 ldp_config_normalize(struct ldpd_conf *);
+static void		 ldp_config_reset(struct ldpd_conf *);
 static void		 ldp_config_reset_main(struct ldpd_conf *);
 static void		 ldp_config_reset_af(struct ldpd_conf *, int);
 static void		 ldp_config_reset_l2vpns(struct ldpd_conf *);
@@ -131,20 +132,13 @@ sighup(void)
 {
 	log_info("SIGHUP received");
 
-	/* reset vty_conf */
-	ldp_config_reset_main(vty_conf);
-	ldp_config_reset_l2vpns(vty_conf);
-
-	/* read configuration file without applying any changes */
-	global.sighup = 1;
-	vty_read_config(ldpd_di.config_file, config_default);
-	global.sighup = 0;
-
 	/*
-	 * Apply the new configuration all at once, this way merge_config()
-	 * will be the least disruptive as possible.
+	 * Do a full configuration reload. In other words, reset vty_conf
+	 * and build a new configuartion from scratch.
 	 */
-	ldp_reload(vty_conf);
+	ldp_config_reset(vty_conf);
+	vty_read_config(ldpd_di.config_file, config_default);
+	ldp_config_apply(NULL, vty_conf);
 }
 
 /* SIGINT / SIGTERM handler. */
@@ -212,7 +206,7 @@ main(int argc, char *argv[])
 	frr_preinit(&ldpd_di, argc, argv);
 	frr_opt_add("LEn:", longopts,
 		"      --ctl_socket   Override ctl socket path\n"
-		"-n,   --instance     Instance id\n");
+		"  -n, --instance     Instance id\n");
 
 	while (1) {
 		int opt;
@@ -262,11 +256,10 @@ main(int argc, char *argv[])
 	strlcpy(init.user, ldpd_privs.user, sizeof(init.user));
 	strlcpy(init.group, ldpd_privs.group, sizeof(init.group));
 	strlcpy(init.ctl_sock_path, ctl_sock_path, sizeof(init.ctl_sock_path));
-	strlcpy(init.zclient_serv_path, zclient_serv_path_get(),
+	strlcpy(init.zclient_serv_path, frr_zclientpath,
 	    sizeof(init.zclient_serv_path));
 
 	argc -= optind;
-	argv += optind;
 	if (argc > 0 || (lflag && eflag))
 		frr_help_exit(1);
 
@@ -277,9 +270,9 @@ main(int argc, char *argv[])
 		exit(1);
 	}
 
-	openzlog(ldpd_di.progname, "LDP", 0,
-	    LOG_CONS | LOG_NDELAY | LOG_PID, LOG_DAEMON);
-
+	if (lflag || eflag)
+		openzlog(ldpd_di.progname, "LDP", 0,
+		         LOG_CONS | LOG_NDELAY | LOG_PID, LOG_DAEMON);
 	if (lflag)
 		lde();
 	else if (eflag)
@@ -316,40 +309,30 @@ main(int argc, char *argv[])
 	ldpe_pid = start_child(PROC_LDP_ENGINE, saved_argv0,
 	    pipe_parent2ldpe[1], pipe_parent2ldpe_sync[1]);
 
-	/* drop privileges */
-	zprivs_init(&ldpd_privs);
+	master = frr_init();
 
-	/* setup signal handler */
-	signal_init(master, array_size(ldp_signals), ldp_signals);
-
-	/* thread master */
-  	master = thread_master_create();
-
-	/* library inits */
-	cmd_init(1);
 	vty_config_lockless();
-	vty_init(master);
-	vrf_init();
+	vrf_init(NULL, NULL, NULL, NULL);
 	access_list_init();
 	ldp_vty_init();
 	ldp_zebra_init(master);
 
-	/* create base configuration with sane defaults */
-	ldpd_conf = config_new_empty();
-	ldp_config_reset_main(ldpd_conf);
-
 	/*
-	 * Create vty_conf as a duplicate of the main configuration. All
-	 * configuration requests (e.g. CLI) act on vty_conf and then call
-	 * ldp_reload() to merge the changes into ldpd_conf.
+	 * Create base configuration with sane defaults. All configuration
+	 * requests (e.g. CLI) act on vty_conf and then call ldp_config_apply()
+	 * to merge the changes into ldpd_conf, which contains the actual
+	 * running configuration.
 	 */
+	ldpd_conf = config_new_empty();
 	vty_conf = config_new_empty();
-	ldp_config_reset_main(vty_conf);
 	QOBJ_REG(vty_conf, ldpd_conf);
 
 	/* read configuration file and daemonize  */
 	frr_config_fork();
 
+	/* apply configuration */
+	ldp_config_apply(NULL, vty_conf);
+
 	/* setup pipes to children */
 	if ((iev_ldpe = calloc(1, sizeof(struct imsgev))) == NULL ||
 	    (iev_ldpe_sync = calloc(1, sizeof(struct imsgev))) == NULL ||
@@ -358,33 +341,37 @@ main(int argc, char *argv[])
 		fatal(NULL);
 	imsg_init(&iev_ldpe->ibuf, pipe_parent2ldpe[0]);
 	iev_ldpe->handler_read = main_dispatch_ldpe;
-	iev_ldpe->ev_read = thread_add_read(master, iev_ldpe->handler_read,
-	    iev_ldpe, iev_ldpe->ibuf.fd);
+	iev_ldpe->ev_read = NULL;
+	thread_add_read(master, iev_ldpe->handler_read, iev_ldpe, iev_ldpe->ibuf.fd,
+			&iev_ldpe->ev_read);
 	iev_ldpe->handler_write = ldp_write_handler;
 
 	imsg_init(&iev_ldpe_sync->ibuf, pipe_parent2ldpe_sync[0]);
 	iev_ldpe_sync->handler_read = main_dispatch_ldpe;
-	iev_ldpe_sync->ev_read = thread_add_read(master,
-	    iev_ldpe_sync->handler_read, iev_ldpe_sync, iev_ldpe_sync->ibuf.fd);
+	iev_ldpe_sync->ev_read = NULL;
+	thread_add_read(master, iev_ldpe_sync->handler_read, iev_ldpe_sync, iev_ldpe_sync->ibuf.fd,
+			&iev_ldpe_sync->ev_read);
 	iev_ldpe_sync->handler_write = ldp_write_handler;
 
 	imsg_init(&iev_lde->ibuf, pipe_parent2lde[0]);
 	iev_lde->handler_read = main_dispatch_lde;
-	iev_lde->ev_read = thread_add_read(master, iev_lde->handler_read,
-	    iev_lde, iev_lde->ibuf.fd);
+	iev_lde->ev_read = NULL;
+	thread_add_read(master, iev_lde->handler_read, iev_lde, iev_lde->ibuf.fd,
+			&iev_lde->ev_read);
 	iev_lde->handler_write = ldp_write_handler;
 
 	imsg_init(&iev_lde_sync->ibuf, pipe_parent2lde_sync[0]);
 	iev_lde_sync->handler_read = main_dispatch_lde;
-	iev_lde_sync->ev_read = thread_add_read(master,
-	    iev_lde_sync->handler_read, iev_lde_sync, iev_lde_sync->ibuf.fd);
+	iev_lde_sync->ev_read = NULL;
+	thread_add_read(master, iev_lde_sync->handler_read, iev_lde_sync, iev_lde_sync->ibuf.fd,
+			&iev_lde_sync->ev_read);
 	iev_lde_sync->handler_write = ldp_write_handler;
 
 	if (main_imsg_send_ipc_sockets(&iev_ldpe->ibuf, &iev_lde->ibuf))
 		fatal("could not establish imsg links");
-	main_imsg_compose_both(IMSG_INIT, &init, sizeof(init));
 	main_imsg_compose_both(IMSG_DEBUG_UPDATE, &ldp_debug,
 	    sizeof(ldp_debug));
+	main_imsg_compose_both(IMSG_INIT, &init, sizeof(init));
 	main_imsg_send_config(ldpd_conf);
 
 	if (ldpd_conf->ipv4.flags & F_LDPD_AF_ENABLED)
@@ -404,6 +391,8 @@ ldpd_shutdown(void)
 	pid_t		 pid;
 	int		 status;
 
+	frr_early_fini();
+
 	/* close pipes */
 	msgbuf_clear(&iev_ldpe->ibuf.w);
 	close(iev_ldpe->ibuf.fd);
@@ -412,8 +401,7 @@ ldpd_shutdown(void)
 
 	config_clear(ldpd_conf);
 
-	ldp_config_reset_main(vty_conf);
-	ldp_config_reset_l2vpns(vty_conf);
+	ldp_config_reset(vty_conf);
 	QOBJ_UNREG(vty_conf);
 	free(vty_conf);
 
@@ -436,13 +424,9 @@ ldpd_shutdown(void)
 
 	vrf_terminate();
 	access_list_reset();
-	cmd_terminate();
-	vty_terminate();
 	ldp_zebra_destroy();
-	zprivs_terminate(&ldpd_privs);
-	thread_master_free(master);
-	closezlog();
 
+	frr_fini();
 	exit(0);
 }
 
@@ -465,10 +449,15 @@ start_child(enum ldpd_process p, char *argv0, int fd_async, int fd_sync)
 	}
 
 	nullfd = open("/dev/null", O_RDONLY | O_NOCTTY);
-	dup2(nullfd, 0);
-	dup2(nullfd, 1);
-	dup2(nullfd, 2);
-	close(nullfd);
+	if (nullfd == -1) {
+		zlog_err("%s: failed to open /dev/null: %s", __func__,
+			 safe_strerror(errno));
+	} else {
+		dup2(nullfd, 0);
+		dup2(nullfd, 1);
+		dup2(nullfd, 2);
+		close(nullfd);
+	}
 
 	if (dup2(fd_async, LDPD_FD_ASYNC) == -1)
 		fatal("cannot setup imsg async fd");
@@ -711,12 +700,12 @@ void
 imsg_event_add(struct imsgev *iev)
 {
 	if (iev->handler_read)
-		THREAD_READ_ON(master, iev->ev_read, iev->handler_read, iev,
-		    iev->ibuf.fd);
+		thread_add_read(master, iev->handler_read, iev, iev->ibuf.fd,
+				&iev->ev_read);
 
 	if (iev->handler_write && iev->ibuf.w.queued)
-		THREAD_WRITE_ON(master, iev->ev_write, iev->handler_write, iev,
-		    iev->ibuf.fd);
+		thread_add_write(master, iev->handler_write, iev,
+				 iev->ibuf.fd, &iev->ev_write);
 }
 
 int
@@ -742,8 +731,8 @@ void
 evbuf_event_add(struct evbuf *eb)
 {
 	if (eb->wbuf.queued)
-		THREAD_WRITE_ON(master, eb->ev, eb->handler, eb->arg,
-		    eb->wbuf.fd);
+		thread_add_write(master, eb->handler, eb->arg, eb->wbuf.fd,
+				 &eb->ev);
 }
 
 void
@@ -979,9 +968,15 @@ main_imsg_send_config(struct ldpd_conf *xconf)
 }
 
 int
-ldp_reload(struct ldpd_conf *xconf)
+ldp_config_apply(struct vty *vty, struct ldpd_conf *xconf)
 {
-	if (global.sighup)
+	/*
+	 * When reading from a configuration file (startup and sighup), we
+	 * call merge_config() only once after the whole config has been read.
+	 * This is the optimal and least disruptive way to update the running
+	 * configuration.
+	 */
+	if (vty && vty->type == VTY_FILE)
 		return (0);
 
 	ldp_config_normalize(xconf);
@@ -1058,19 +1053,30 @@ ldp_config_normalize(struct ldpd_conf *xconf)
 	}
 }
 
+static void
+ldp_config_reset(struct ldpd_conf *conf)
+{
+	ldp_config_reset_main(conf);
+	ldp_config_reset_l2vpns(conf);
+}
+
 static void
 ldp_config_reset_main(struct ldpd_conf *conf)
 {
 	struct iface		*iface;
 	struct nbr_params	*nbrp;
 
-	while ((iface = RB_ROOT(&conf->iface_tree)) != NULL) {
+	while (!RB_EMPTY(iface_head, &conf->iface_tree)) {
+		iface = RB_ROOT(iface_head, &conf->iface_tree);
+
 		QOBJ_UNREG(iface);
 		RB_REMOVE(iface_head, &conf->iface_tree, iface);
 		free(iface);
 	}
 
-	while ((nbrp = RB_ROOT(&conf->nbrp_tree)) != NULL) {
+	while (!RB_EMPTY(nbrp_head, &conf->nbrp_tree)) {
+		nbrp = RB_ROOT(nbrp_head, &conf->nbrp_tree);
+
 		QOBJ_UNREG(nbrp);
 		RB_REMOVE(nbrp_head, &conf->nbrp_tree, nbrp);
 		free(nbrp);
@@ -1126,18 +1132,25 @@ ldp_config_reset_l2vpns(struct ldpd_conf *conf)
 	struct l2vpn_if		*lif;
 	struct l2vpn_pw		*pw;
 
-	while ((l2vpn = RB_ROOT(&conf->l2vpn_tree)) != NULL) {
-		while ((lif = RB_ROOT(&l2vpn->if_tree)) != NULL) {
+	while (!RB_EMPTY(l2vpn_head, &conf->l2vpn_tree)) {
+		l2vpn = RB_ROOT(l2vpn_head, &conf->l2vpn_tree);
+		while (!RB_EMPTY(l2vpn_if_head, &l2vpn->if_tree)) {
+			lif = RB_ROOT(l2vpn_if_head, &l2vpn->if_tree);
+
 			QOBJ_UNREG(lif);
 			RB_REMOVE(l2vpn_if_head, &l2vpn->if_tree, lif);
 			free(lif);
 		}
-		while ((pw = RB_ROOT(&l2vpn->pw_tree)) != NULL) {
+		while (!RB_EMPTY(l2vpn_pw_head, &l2vpn->pw_tree)) {
+			pw = RB_ROOT(l2vpn_pw_head, &l2vpn->pw_tree);
+
 			QOBJ_UNREG(pw);
 			RB_REMOVE(l2vpn_pw_head, &l2vpn->pw_tree, pw);
 			free(pw);
 		}
-		while ((pw = RB_ROOT(&l2vpn->pw_inactive_tree)) != NULL) {
+		while (!RB_EMPTY(l2vpn_pw_head, &l2vpn->pw_inactive_tree)) {
+			pw = RB_ROOT(l2vpn_pw_head, &l2vpn->pw_inactive_tree);
+
 			QOBJ_UNREG(pw);
 			RB_REMOVE(l2vpn_pw_head, &l2vpn->pw_inactive_tree, pw);
 			free(pw);
@@ -1156,19 +1169,27 @@ ldp_clear_config(struct ldpd_conf *xconf)
 	struct nbr_params	*nbrp;
 	struct l2vpn		*l2vpn;
 
-	while ((iface = RB_ROOT(&xconf->iface_tree)) != NULL) {
+	while (!RB_EMPTY(iface_head, &xconf->iface_tree)) {
+		iface = RB_ROOT(iface_head, &xconf->iface_tree);
+
 		RB_REMOVE(iface_head, &xconf->iface_tree, iface);
 		free(iface);
 	}
-	while ((tnbr = RB_ROOT(&xconf->tnbr_tree)) != NULL) {
+	while (!RB_EMPTY(tnbr_head, &xconf->tnbr_tree)) {
+		tnbr = RB_ROOT(tnbr_head, &xconf->tnbr_tree);
+
 		RB_REMOVE(tnbr_head, &xconf->tnbr_tree, tnbr);
 		free(tnbr);
 	}
-	while ((nbrp = RB_ROOT(&xconf->nbrp_tree)) != NULL) {
+	while (!RB_EMPTY(nbrp_head, &xconf->nbrp_tree)) {
+		nbrp = RB_ROOT(nbrp_head, &xconf->nbrp_tree);
+
 		RB_REMOVE(nbrp_head, &xconf->nbrp_tree, nbrp);
 		free(nbrp);
 	}
-	while ((l2vpn = RB_ROOT(&xconf->l2vpn_tree)) != NULL) {
+	while (!RB_EMPTY(l2vpn_head, &xconf->l2vpn_tree)) {
+		l2vpn = RB_ROOT(l2vpn_head, &xconf->l2vpn_tree);
+
 		RB_REMOVE(l2vpn_head, &xconf->l2vpn_tree, l2vpn);
 		l2vpn_del(l2vpn);
 	}
@@ -1571,9 +1592,9 @@ merge_l2vpns(struct ldpd_conf *conf, struct ldpd_conf *xconf)
 		if ((l2vpn = l2vpn_find(conf, xl->name)) == NULL) {
 			COPY(l2vpn, xl);
 			RB_INSERT(l2vpn_head, &conf->l2vpn_tree, l2vpn);
-			RB_INIT(&l2vpn->if_tree);
-			RB_INIT(&l2vpn->pw_tree);
-			RB_INIT(&l2vpn->pw_inactive_tree);
+			RB_INIT(l2vpn_if_head, &l2vpn->if_tree);
+			RB_INIT(l2vpn_pw_head, &l2vpn->pw_tree);
+			RB_INIT(l2vpn_pw_head, &l2vpn->pw_inactive_tree);
 
 			switch (ldpd_process) {
 			case PROC_LDE_ENGINE:
@@ -1782,10 +1803,13 @@ config_new_empty(void)
 	if (xconf == NULL)
 		fatal(NULL);
 
-	RB_INIT(&xconf->iface_tree);
-	RB_INIT(&xconf->tnbr_tree);
-	RB_INIT(&xconf->nbrp_tree);
-	RB_INIT(&xconf->l2vpn_tree);
+	RB_INIT(iface_head, &xconf->iface_tree);
+	RB_INIT(tnbr_head, &xconf->tnbr_tree);
+	RB_INIT(nbrp_head, &xconf->nbrp_tree);
+	RB_INIT(l2vpn_head, &xconf->l2vpn_tree);
+
+	/* set default values */
+	ldp_config_reset(xconf);
 
 	return (xconf);
 }
