@@ -22,13 +22,14 @@
 #ifndef _LDPD_H_
 #define _LDPD_H_
 
-#include "openbsd-queue.h"
+#include "queue.h"
 #include "openbsd-tree.h"
 #include "imsg.h"
 #include "thread.h"
 #include "qobj.h"
 #include "prefix.h"
 #include "filter.h"
+#include "vty.h"
 #include "pw.h"
 #include "zclient.h"
 
@@ -149,8 +150,6 @@ enum imsg_type {
 	IMSG_DEBUG_UPDATE,
 	IMSG_LOG,
 	IMSG_ACL_CHECK,
-	IMSG_GET_LABEL_CHUNK,
-	IMSG_RELEASE_LABEL_CHUNK,
 	IMSG_INIT,
 	IMSG_PW_UPDATE
 };
@@ -160,7 +159,7 @@ struct ldpd_init {
 	char		 group[256];
 	char		 ctl_sock_path[MAXPATHLEN];
 	char		 zclient_serv_path[MAXPATHLEN];
-	u_short		 instance;
+	unsigned short instance;
 };
 
 union ldpd_addr {
@@ -394,7 +393,7 @@ struct l2vpn_if {
 	char			 ifname[IF_NAMESIZE];
 	unsigned int		 ifindex;
 	int			 operative;
-	uint8_t			 mac[ETHER_ADDR_LEN];
+	uint8_t			 mac[ETH_ALEN];
 	QOBJ_FIELDS
 };
 RB_HEAD(l2vpn_if_head, l2vpn_if);
@@ -523,7 +522,6 @@ struct ldpd_af_global {
 
 struct ldpd_global {
 	int			 cmd_opts;
-	int			 sighup;
 	struct in_addr		 rtr_id;
 	struct ldpd_af_global	 ipv4;
 	struct ldpd_af_global	 ipv6;
@@ -563,7 +561,7 @@ struct kif {
 	unsigned short		 ifindex;
 	int			 flags;
 	int			 operative;
-	uint8_t			 mac[ETHER_ADDR_LEN];
+	uint8_t			 mac[ETH_ALEN];
 	int			 mtu;
 };
 
@@ -719,7 +717,7 @@ struct ldpd_af_conf	*ldp_af_conf_get(struct ldpd_conf *, int);
 struct ldpd_af_global	*ldp_af_global_get(struct ldpd_global *, int);
 int			 ldp_is_dual_stack(struct ldpd_conf *);
 in_addr_t		 ldp_rtr_id_get(struct ldpd_conf *);
-int			 ldp_reload(struct ldpd_conf *);
+int			 ldp_config_apply(struct vty *, struct ldpd_conf *);
 void			 ldp_clear_config(struct ldpd_conf *);
 void			 merge_config(struct ldpd_conf *, struct ldpd_conf *);
 struct ldpd_conf	*config_new_empty(void);
