@@ -26,9 +26,11 @@
 #include "lde.h"
 #include "log.h"
 
-static __inline int	 nbr_id_compare(struct nbr *, struct nbr *);
-static __inline int	 nbr_addr_compare(struct nbr *, struct nbr *);
-static __inline int	 nbr_pid_compare(struct nbr *, struct nbr *);
+static __inline int	 nbr_id_compare(const struct nbr *, const struct nbr *);
+static __inline int	 nbr_addr_compare(const struct nbr *,
+			    const struct nbr *);
+static __inline int	 nbr_pid_compare(const struct nbr *,
+			    const struct nbr *);
 static void		 nbr_update_peerid(struct nbr *);
 static int		 nbr_ktimer(struct thread *);
 static void		 nbr_start_ktimer(struct nbr *);
@@ -39,8 +41,8 @@ static void		 nbr_start_itimeout(struct nbr *);
 static int		 nbr_idtimer(struct thread *);
 static int		 nbr_act_session_operational(struct nbr *);
 static void		 nbr_send_labelmappings(struct nbr *);
-static __inline int	 nbr_params_compare(struct nbr_params *,
-			    struct nbr_params *);
+static __inline int	 nbr_params_compare(const struct nbr_params *,
+			    const struct nbr_params *);
 
 RB_GENERATE(nbr_id_head, nbr, id_tree, nbr_id_compare)
 RB_GENERATE(nbr_addr_head, nbr, addr_tree, nbr_addr_compare)
@@ -101,13 +103,13 @@ struct nbr_addr_head nbrs_by_addr = RB_INITIALIZER(&nbrs_by_addr);
 struct nbr_pid_head nbrs_by_pid = RB_INITIALIZER(&nbrs_by_pid);
 
 static __inline int
-nbr_id_compare(struct nbr *a, struct nbr *b)
+nbr_id_compare(const struct nbr *a, const struct nbr *b)
 {
 	return (ntohl(a->id.s_addr) - ntohl(b->id.s_addr));
 }
 
 static __inline int
-nbr_addr_compare(struct nbr *a, struct nbr *b)
+nbr_addr_compare(const struct nbr *a, const struct nbr *b)
 {
 	if (a->af < b->af)
 		return (-1);
@@ -118,7 +120,7 @@ nbr_addr_compare(struct nbr *a, struct nbr *b)
 }
 
 static __inline int
-nbr_pid_compare(struct nbr *a, struct nbr *b)
+nbr_pid_compare(const struct nbr *a, const struct nbr *b)
 {
 	return (a->peerid - b->peerid);
 }
@@ -229,7 +231,7 @@ nbr_new(struct in_addr id, int af, int ds_tlv, union ldpd_addr *addr,
 	if ((nbr = calloc(1, sizeof(*nbr))) == NULL)
 		fatal(__func__);
 
-	RB_INIT(&nbr->adj_tree);
+	RB_INIT(nbr_adj_head, &nbr->adj_tree);
 	nbr->state = NBR_STA_PRESENT;
 	nbr->peerid = 0;
 	nbr->af = af;
@@ -314,7 +316,9 @@ nbr_del(struct nbr *nbr)
 	mapping_list_clr(&nbr->release_list);
 	mapping_list_clr(&nbr->abortreq_list);
 
-	while ((adj = RB_ROOT(&nbr->adj_tree)) != NULL) {
+	while (!RB_EMPTY(nbr_adj_head, &nbr->adj_tree)) {
+		adj = RB_ROOT(nbr_adj_head, &nbr->adj_tree);
+
 		adj->nbr = NULL;
 		RB_REMOVE(nbr_adj_head, &nbr->adj_tree, adj);
 	}
@@ -415,7 +419,8 @@ nbr_start_ktimer(struct nbr *nbr)
 	/* send three keepalives per period */
 	secs = nbr->keepalive / KEEPALIVE_PER_PERIOD;
 	THREAD_TIMER_OFF(nbr->keepalive_timer);
-	nbr->keepalive_timer = thread_add_timer(master, nbr_ktimer, nbr, secs);
+	nbr->keepalive_timer = NULL;
+	thread_add_timer(master, nbr_ktimer, nbr, secs, &nbr->keepalive_timer);
 }
 
 void
@@ -444,8 +449,9 @@ static void
 nbr_start_ktimeout(struct nbr *nbr)
 {
 	THREAD_TIMER_OFF(nbr->keepalive_timeout);
-	nbr->keepalive_timeout = thread_add_timer(master, nbr_ktimeout, nbr,
-	    nbr->keepalive);
+	nbr->keepalive_timeout = NULL;
+	thread_add_timer(master, nbr_ktimeout, nbr, nbr->keepalive,
+			 &nbr->keepalive_timeout);
 }
 
 void
@@ -475,7 +481,8 @@ nbr_start_itimeout(struct nbr *nbr)
 
 	secs = INIT_FSM_TIMEOUT;
 	THREAD_TIMER_OFF(nbr->init_timeout);
-	nbr->init_timeout = thread_add_timer(master, nbr_itimeout, nbr, secs);
+	nbr->init_timeout = NULL;
+	thread_add_timer(master, nbr_itimeout, nbr, secs, &nbr->init_timeout);
 }
 
 void
@@ -523,7 +530,9 @@ nbr_start_idtimer(struct nbr *nbr)
 	}
 
 	THREAD_TIMER_OFF(nbr->initdelay_timer);
-	nbr->initdelay_timer = thread_add_timer(master, nbr_idtimer, nbr, secs);
+	nbr->initdelay_timer = NULL;
+	thread_add_timer(master, nbr_idtimer, nbr, secs,
+			 &nbr->initdelay_timer);
 }
 
 void
@@ -640,8 +649,8 @@ nbr_establish_connection(struct nbr *nbr)
 	if (connect(nbr->fd, (struct sockaddr *)&remote_sa,
 	    sockaddr_len((struct sockaddr *)&remote_sa)) == -1) {
 		if (errno == EINPROGRESS) {
-			THREAD_WRITE_ON(master, nbr->ev_connect, nbr_connect_cb,
-			    nbr, nbr->fd);
+			thread_add_write(master, nbr_connect_cb, nbr, nbr->fd,
+					 &nbr->ev_connect);
 			return (0);
 		}
 		log_warn("%s: error while connecting to %s", __func__,
@@ -766,7 +775,7 @@ nbr_send_labelmappings(struct nbr *nbr)
 }
 
 static __inline int
-nbr_params_compare(struct nbr_params *a, struct nbr_params *b)
+nbr_params_compare(const struct nbr_params *a, const struct nbr_params *b)
 {
 	return (ntohl(a->lsr_id.s_addr) - ntohl(b->lsr_id.s_addr));
 }
