@@ -16,10 +16,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -29,8 +28,6 @@
 
 DEFINE_MTYPE_STATIC(LIB, CMD_MATCHSTACK, "Command Match Stack")
 
-#define MAXDEPTH 64
-
 #ifdef TRACE_MATCHER
 #define TM 1
 #else
@@ -47,8 +44,9 @@ DEFINE_MTYPE_STATIC(LIB, CMD_MATCHSTACK, "Command Match Stack")
 static int add_nexthops(struct list *, struct graph_node *,
 			struct graph_node **, size_t);
 
-static struct list *command_match_r(struct graph_node *, vector, unsigned int,
-				    struct graph_node **);
+static enum matcher_rv command_match_r(struct graph_node *, vector,
+				       unsigned int, struct graph_node **,
+				       struct list **);
 
 static int score_precedence(enum cmd_token_type);
 
@@ -71,9 +69,7 @@ static enum match_type match_ipv4(const char *);
 
 static enum match_type match_ipv4_prefix(const char *);
 
-static enum match_type match_ipv6(const char *);
-
-static enum match_type match_ipv6_prefix(const char *);
+static enum match_type match_ipv6_prefix(const char *, bool);
 
 static enum match_type match_range(struct cmd_token *, const char *);
 
@@ -81,14 +77,14 @@ static enum match_type match_word(struct cmd_token *, const char *);
 
 static enum match_type match_variable(struct cmd_token *, const char *);
 
-/* matching functions */
-static enum matcher_rv matcher_rv;
+static enum match_type match_mac(const char *, bool);
 
 enum matcher_rv command_match(struct graph *cmdgraph, vector vline,
 			      struct list **argv, const struct cmd_element **el)
 {
-	struct graph_node *stack[MAXDEPTH];
-	matcher_rv = MATCHER_NO_MATCH;
+	struct graph_node *stack[CMD_ARGC_MAX];
+	enum matcher_rv status;
+	*argv = NULL;
 
 	// prepend a dummy token to match that pesky start node
 	vector vvline = vector_init(vline->alloced + 1);
@@ -98,14 +94,16 @@ enum matcher_rv command_match(struct graph *cmdgraph, vector vline,
 	vvline->active = vline->active + 1;
 
 	struct graph_node *start = vector_slot(cmdgraph->nodes, 0);
-	if ((*argv = command_match_r(start, vvline, 0,
-				     stack))) // successful match
-	{
+	status = command_match_r(start, vvline, 0, stack, argv);
+	if (status == MATCHER_OK) { // successful match
 		struct listnode *head = listhead(*argv);
 		struct listnode *tail = listtail(*argv);
 
+		assert(head);
+		assert(tail);
+
 		// delete dummy start node
-		del_cmd_token((struct cmd_token *)head->data);
+		cmd_token_del((struct cmd_token *)head->data);
 		list_delete_node(*argv, head);
 
 		// get cmd_element out of list tail
@@ -116,6 +114,9 @@ enum matcher_rv command_match(struct graph *cmdgraph, vector vline,
 		// input, with each cmd_token->arg holding the corresponding
 		// input
 		assert(*el);
+	} else if (*argv) {
+		del_arglist(*argv);
+		*argv = NULL;
 	}
 
 	if (!*el) {
@@ -130,7 +131,7 @@ enum matcher_rv command_match(struct graph *cmdgraph, vector vline,
 	// free vector
 	vector_free(vvline);
 
-	return matcher_rv;
+	return status;
 }
 
 /**
@@ -184,11 +185,15 @@ enum matcher_rv command_match(struct graph *cmdgraph, vector vline,
  *
  * If no match was found, the return value is NULL.
  */
-static struct list *command_match_r(struct graph_node *start, vector vline,
-				    unsigned int n, struct graph_node **stack)
+static enum matcher_rv command_match_r(struct graph_node *start, vector vline,
+				       unsigned int n,
+				       struct graph_node **stack,
+				       struct list **currbest)
 {
 	assert(n < vector_active(vline));
 
+	enum matcher_rv status = MATCHER_NO_MATCH;
+
 	// get the minimum match level that can count as a full match
 	struct cmd_token *token = start->data;
 	enum match_type minmatch = min_match_level(token->type);
@@ -196,12 +201,12 @@ static struct list *command_match_r(struct graph_node *start, vector vline,
 	/* check history/stack of tokens
 	 * this disallows matching the same one more than once if there is a
 	 * circle in the graph (used for keyword arguments) */
-	if (n == MAXDEPTH)
-		return NULL;
+	if (n == CMD_ARGC_MAX)
+		return MATCHER_NO_MATCH;
 	if (!token->allowrepeat)
 		for (size_t s = 0; s < n; s++)
 			if (stack[s] == start)
-				return NULL;
+				return MATCHER_NO_MATCH;
 
 	// get the current operating input token
 	char *input_token = vector_slot(vline, n);
@@ -210,6 +215,7 @@ static struct list *command_match_r(struct graph_node *start, vector vline,
 	fprintf(stdout, "\"%-20s\" matches \"%-30s\" ? ", input_token,
 		token->text);
 	enum match_type mt = match_token(token, input_token);
+	fprintf(stdout, "type: %d ", token->type);
 	fprintf(stdout, "min: %d - ", minmatch);
 	switch (mt) {
 	case trivial_match:
@@ -232,7 +238,7 @@ static struct list *command_match_r(struct graph_node *start, vector vline,
 
 	// if we don't match this node, die
 	if (match_token(token, input_token) < minmatch)
-		return NULL;
+		return MATCHER_NO_MATCH;
 
 	stack[n] = start;
 
@@ -245,86 +251,92 @@ static struct list *command_match_r(struct graph_node *start, vector vline,
 	add_nexthops(next, start, NULL, 0);
 
 	// determine the best match
-	int ambiguous = 0;
-	struct list *currbest = NULL;
 	for (ALL_LIST_ELEMENTS_RO(next, ln, gn)) {
 		// if we've matched all input we're looking for END_TKN
 		if (n + 1 == vector_active(vline)) {
 			struct cmd_token *tok = gn->data;
 			if (tok->type == END_TKN) {
-				if (currbest) // there is more than one END_TKN
-					      // in the follow set
-				{
-					ambiguous = 1;
+				// if more than one END_TKN in the follow set
+				if (*currbest) {
+					status = MATCHER_AMBIGUOUS;
 					break;
+				} else {
+					status = MATCHER_OK;
 				}
-				currbest = list_new();
+				*currbest = list_new();
 				// node should have one child node with the
 				// element
 				struct graph_node *leaf =
 					vector_slot(gn->to, 0);
 				// last node in the list will hold the
-				// cmd_element;
-				// this is important because list_delete()
-				// expects
-				// that all nodes have the same data type, so
-				// when
-				// deleting this list the last node must be
-				// manually deleted
+				// cmd_element; this is important because
+				// list_delete() expects that all nodes have
+				// the same data type, so when deleting this
+				// list the last node must be manually deleted
 				struct cmd_element *el = leaf->data;
-				listnode_add(currbest, el);
-				currbest->del =
-					(void (*)(void *)) & del_cmd_token;
+				listnode_add(*currbest, el);
+				(*currbest)->del =
+					(void (*)(void *)) & cmd_token_del;
 				// do not break immediately; continue walking
-				// through the follow set
-				// to ensure that there is exactly one END_TKN
+				// through the follow set to ensure that there
+				// is exactly one END_TKN
 			}
 			continue;
 		}
 
 		// else recurse on candidate child node
-		struct list *result = command_match_r(gn, vline, n + 1, stack);
+		struct list *result = NULL;
+		enum matcher_rv rstat =
+			command_match_r(gn, vline, n + 1, stack, &result);
 
 		// save the best match
-		if (result && currbest) {
+		if (result && *currbest) {
 			// pick the best of two matches
 			struct list *newbest =
-				disambiguate(currbest, result, vline, n + 1);
-			// set ambiguity flag
-			ambiguous =
-				!newbest || (ambiguous && newbest == currbest);
+				disambiguate(*currbest, result, vline, n + 1);
+
+			// current best and result are ambiguous
+			if (!newbest)
+				status = MATCHER_AMBIGUOUS;
+			// current best is still the best, but ambiguous
+			else if (newbest == *currbest
+				 && status == MATCHER_AMBIGUOUS)
+				status = MATCHER_AMBIGUOUS;
+			// result is better, but also ambiguous
+			else if (newbest == result
+				 && rstat == MATCHER_AMBIGUOUS)
+				status = MATCHER_AMBIGUOUS;
+			// one or the other is superior and not ambiguous
+			else
+				status = MATCHER_OK;
+
 			// delete the unnecessary result
 			struct list *todelete =
-				((newbest && newbest == result) ? currbest
+				((newbest && newbest == result) ? *currbest
 								: result);
 			del_arglist(todelete);
 
-			currbest = newbest ? newbest : currbest;
-		} else if (result)
-			currbest = result;
-	}
-
-	if (currbest) {
-		if (ambiguous) {
-			del_arglist(currbest);
-			currbest = NULL;
-			matcher_rv = MATCHER_AMBIGUOUS;
-		} else {
-			// copy token, set arg and prepend to currbest
-			struct cmd_token *token = start->data;
-			struct cmd_token *copy = copy_cmd_token(token);
-			copy->arg = XSTRDUP(MTYPE_CMD_ARG, input_token);
-			listnode_add_before(currbest, currbest->head, copy);
-			matcher_rv = MATCHER_OK;
+			*currbest = newbest ? newbest : *currbest;
+		} else if (result) {
+			status = rstat;
+			*currbest = result;
+		} else if (!*currbest) {
+			status = MAX(rstat, status);
 		}
-	} else if (n + 1 == vector_active(vline)
-		   && matcher_rv == MATCHER_NO_MATCH)
-		matcher_rv = MATCHER_INCOMPLETE;
+	}
+	if (*currbest) {
+		// copy token, set arg and prepend to currbest
+		struct cmd_token *token = start->data;
+		struct cmd_token *copy = cmd_token_dup(token);
+		copy->arg = XSTRDUP(MTYPE_CMD_ARG, input_token);
+		listnode_add_before(*currbest, (*currbest)->head, copy);
+	} else if (n + 1 == vector_active(vline) && status == MATCHER_NO_MATCH)
+		status = MATCHER_INCOMPLETE;
 
 	// cleanup
-	list_delete(next);
+	list_delete_and_null(&next);
 
-	return currbest;
+	return status;
 }
 
 static void stack_del(void *val)
@@ -355,7 +367,7 @@ enum matcher_rv command_complete(struct graph *graph, vector vline,
 
 	unsigned int idx;
 	for (idx = 0; idx < vector_active(vline) && next->count > 0; idx++) {
-		list_delete(current);
+		list_delete_and_null(&current);
 		current = next;
 		next = list_new();
 		next->del = stack_del;
@@ -403,7 +415,7 @@ enum matcher_rv command_complete(struct graph *graph, vector vline,
 				trace_matcher("trivial_match\n");
 				if (exact_match_exists && !last_token)
 					break;
-				/* fallthru */
+			/* fallthru */
 			case exact_match:
 				trace_matcher("exact_match\n");
 				if (last_token) {
@@ -433,12 +445,12 @@ enum matcher_rv command_complete(struct graph *graph, vector vline,
 	 * next     = set of all nodes reachable from all nodes in `matched`
 	 */
 
-	matcher_rv = idx == vector_active(vline) && next->count
-			     ? MATCHER_OK
-			     : MATCHER_NO_MATCH;
+	enum matcher_rv mrv = idx == vector_active(vline) && next->count
+				      ? MATCHER_OK
+				      : MATCHER_NO_MATCH;
 
 	*completions = NULL;
-	if (!MATCHER_ERROR(matcher_rv)) {
+	if (!MATCHER_ERROR(mrv)) {
 		// extract cmd_token into list
 		*completions = list_new();
 		for (ALL_LIST_ELEMENTS_RO(next, node, gstack)) {
@@ -446,10 +458,10 @@ enum matcher_rv command_complete(struct graph *graph, vector vline,
 		}
 	}
 
-	list_delete(current);
-	list_delete(next);
+	list_delete_and_null(&current);
+	list_delete_and_null(&next);
 
-	return matcher_rv;
+	return mrv;
 }
 
 /**
@@ -540,6 +552,8 @@ static int score_precedence(enum cmd_token_type type)
 	case IPV4_PREFIX_TKN:
 	case IPV6_TKN:
 	case IPV6_PREFIX_TKN:
+	case MAC_TKN:
+	case MAC_PREFIX_TKN:
 	case RANGE_TKN:
 		return 2;
 	case WORD_TKN:
@@ -635,7 +649,7 @@ static void del_arglist(struct list *list)
 	list_delete_node(list, tail);
 
 	// delete the rest of the list as usual
-	list_delete(list);
+	list_delete_and_null(&list);
 }
 
 /*---------- token level matching functions ----------*/
@@ -654,13 +668,17 @@ static enum match_type match_token(struct cmd_token *token, char *input_token)
 	case IPV4_PREFIX_TKN:
 		return match_ipv4_prefix(input_token);
 	case IPV6_TKN:
-		return match_ipv6(input_token);
+		return match_ipv6_prefix(input_token, false);
 	case IPV6_PREFIX_TKN:
-		return match_ipv6_prefix(input_token);
+		return match_ipv6_prefix(input_token, true);
 	case RANGE_TKN:
 		return match_range(token, input_token);
 	case VARIABLE_TKN:
 		return match_variable(token, input_token);
+	case MAC_TKN:
+		return match_mac(input_token, false);
+	case MAC_PREFIX_TKN:
+		return match_mac(input_token, true);
 	case END_TKN:
 	default:
 		return no_match;
@@ -799,34 +817,19 @@ static enum match_type match_ipv4_prefix(const char *str)
 #define STATE_SLASH     6
 #define STATE_MASK      7
 
-static enum match_type match_ipv6(const char *str)
-{
-	struct sockaddr_in6 sin6_dummy;
-	int ret;
-
-	if (strspn(str, IPV6_ADDR_STR) != strlen(str))
-		return no_match;
-
-	ret = inet_pton(AF_INET6, str, &sin6_dummy.sin6_addr);
-
-	if (ret == 1)
-		return exact_match;
-
-	return no_match;
-}
-
-static enum match_type match_ipv6_prefix(const char *str)
+static enum match_type match_ipv6_prefix(const char *str, bool prefix)
 {
 	int state = STATE_START;
 	int colons = 0, nums = 0, double_colon = 0;
 	int mask;
-	const char *sp = NULL;
+	const char *sp = NULL, *start = str;
 	char *endptr = NULL;
 
 	if (str == NULL)
 		return partly_match;
 
-	if (strspn(str, IPV6_PREFIX_STR) != strlen(str))
+	if (strspn(str, prefix ? IPV6_PREFIX_STR : IPV6_ADDR_STR)
+	    != strlen(str))
 		return no_match;
 
 	while (*str != '\0' && state != STATE_MASK) {
@@ -919,6 +922,12 @@ static enum match_type match_ipv6_prefix(const char *str)
 		str++;
 	}
 
+	if (!prefix) {
+		struct sockaddr_in6 sin6_dummy;
+		int ret = inet_pton(AF_INET6, start, &sin6_dummy.sin6_addr);
+		return ret == 1 ? exact_match : partly_match;
+	}
+
 	if (state < STATE_MASK)
 		return partly_match;
 
@@ -976,3 +985,50 @@ static enum match_type match_variable(struct cmd_token *token, const char *word)
 	assert(token->type == VARIABLE_TKN);
 	return exact_match;
 }
+
+#define MAC_CHARS "ABCDEFabcdef0123456789:"
+
+static enum match_type match_mac(const char *word, bool prefix)
+{
+	/* 6 2-digit hex numbers separated by 5 colons */
+	size_t mac_explen = 6 * 2 + 5;
+	/* '/' + 2-digit integer */
+	size_t mask_len = 1 + 2;
+	unsigned int i;
+	char *eptr;
+	unsigned int maskval;
+
+	/* length check */
+	if (strlen(word) > mac_explen + (prefix ? mask_len : 0))
+		return no_match;
+
+	/* address check */
+	for (i = 0; i < mac_explen; i++) {
+		if (word[i] == '\0' || !strchr(MAC_CHARS, word[i]))
+			break;
+		if (((i + 1) % 3 == 0) != (word[i] == ':'))
+			return no_match;
+	}
+
+	/* incomplete address */
+	if (i < mac_explen && word[i] == '\0')
+		return partly_match;
+	else if (i < mac_explen)
+		return no_match;
+
+	/* mask check */
+	if (prefix && word[i] == '/') {
+		if (word[++i] == '\0')
+			return partly_match;
+
+		maskval = strtoul(&word[i], &eptr, 10);
+		if (*eptr != '\0' || maskval > 48)
+			return no_match;
+	} else if (prefix && word[i] == '\0') {
+		return partly_match;
+	} else if (prefix) {
+		return no_match;
+	}
+
+	return exact_match;
+}
