@@ -1,4 +1,3 @@
-
 /*
  * Interface functions.
  * Copyright (C) 1997, 98 Kunihiro Ishiguro
@@ -15,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -42,19 +40,20 @@ DEFINE_MTYPE_STATIC(LIB, NBR_CONNECTED, "Neighbor Connected")
 DEFINE_MTYPE(LIB, CONNECTED_LABEL, "Connected interface label")
 DEFINE_MTYPE_STATIC(LIB, IF_LINK_PARAMS, "Informational Link Parameters")
 
+static int if_cmp_func(const struct interface *, const struct interface *);
+static int if_cmp_index_func(const struct interface *ifp1,
+			     const struct interface *ifp2);
+RB_GENERATE(if_name_head, interface, name_entry, if_cmp_func);
+RB_GENERATE(if_index_head, interface, index_entry, if_cmp_index_func);
+
 DEFINE_QOBJ_TYPE(interface)
 
+DEFINE_HOOK(if_add, (struct interface * ifp), (ifp))
+DEFINE_KOOH(if_del, (struct interface * ifp), (ifp))
+
 /* List of interfaces in only the default VRF */
 int ptm_enable = 0;
 
-/* One for each program.  This structure is needed to store hooks. */
-struct if_master {
-	int (*if_new_hook)(struct interface *);
-	int (*if_delete_hook)(struct interface *);
-} if_master = {
-	0,
-};
-
 /* Compare interface names, returning an integer greater than, equal to, or
  * less than 0, (following the strcmp convention), according to the
  * relationship between ifp1 and ifp2.  Interface names consist of an
@@ -92,6 +91,8 @@ int if_cmp_name_func(char *p1, char *p2)
 		p1 += l1;
 		p2 += l1;
 
+		if (!*p1 && !*p2)
+			return 0;
 		if (!*p1)
 			return -1;
 		if (!*p2)
@@ -116,32 +117,31 @@ int if_cmp_name_func(char *p1, char *p2)
 	return 0;
 }
 
-static int if_cmp_func(struct interface *ifp1, struct interface *ifp2)
+static int if_cmp_func(const struct interface *ifp1,
+		       const struct interface *ifp2)
 {
-	return if_cmp_name_func(ifp1->name, ifp2->name);
+	return if_cmp_name_func((char *)ifp1->name, (char *)ifp2->name);
+}
+
+static int if_cmp_index_func(const struct interface *ifp1,
+			     const struct interface *ifp2)
+{
+	return ifp1->ifindex - ifp2->ifindex;
 }
 
 /* Create new interface structure. */
-struct interface *if_create(const char *name, int namelen, vrf_id_t vrf_id)
+struct interface *if_create(const char *name, vrf_id_t vrf_id)
 {
+	struct vrf *vrf = vrf_get(vrf_id, NULL);
 	struct interface *ifp;
-	struct list *intf_list = vrf_iflist_get(vrf_id);
 
 	ifp = XCALLOC(MTYPE_IF, sizeof(struct interface));
 	ifp->ifindex = IFINDEX_INTERNAL;
 
 	assert(name);
-	assert(namelen <= INTERFACE_NAMSIZ); /* Need space for '\0' at end. */
-	strncpy(ifp->name, name, namelen);
-	ifp->name[namelen] = '\0';
+	strlcpy(ifp->name, name, sizeof(ifp->name));
 	ifp->vrf_id = vrf_id;
-	if (if_lookup_by_name(ifp->name, vrf_id) == NULL)
-		listnode_add_sort(intf_list, ifp);
-	else
-		zlog_err(
-			"if_create(%s): corruption detected -- interface with this "
-			"name exists already in VRF %u!",
-			ifp->name, vrf_id);
+	IFNAME_RB_INSERT(vrf, ifp);
 	ifp->connected = list_new();
 	ifp->connected->del = (void (*)(void *))connected_free;
 
@@ -152,46 +152,36 @@ struct interface *if_create(const char *name, int namelen, vrf_id_t vrf_id)
 	SET_FLAG(ifp->status, ZEBRA_INTERFACE_LINKDETECTION);
 
 	QOBJ_REG(ifp, interface);
-
-	if (if_master.if_new_hook)
-		(*if_master.if_new_hook)(ifp);
-
+	hook_call(if_add, ifp);
 	return ifp;
 }
 
 /* Create new interface structure. */
-void if_update(struct interface *ifp, const char *name, int namelen,
-	       vrf_id_t vrf_id)
+void if_update_to_new_vrf(struct interface *ifp, vrf_id_t vrf_id)
 {
-	struct list *intf_list = vrf_iflist_get(vrf_id);
+	struct vrf *vrf;
 
 	/* remove interface from old master vrf list */
-	if (vrf_iflist(ifp->vrf_id))
-		listnode_delete(vrf_iflist(ifp->vrf_id), ifp);
+	vrf = vrf_lookup_by_id(ifp->vrf_id);
+	if (vrf) {
+		IFNAME_RB_REMOVE(vrf, ifp);
+		if (ifp->ifindex != IFINDEX_INTERNAL)
+			IFINDEX_RB_REMOVE(vrf, ifp);
+	}
 
-	assert(name);
-	assert(namelen <= INTERFACE_NAMSIZ); /* Need space for '\0' at end. */
-	strncpy(ifp->name, name, namelen);
-	ifp->name[namelen] = '\0';
 	ifp->vrf_id = vrf_id;
-	if (if_lookup_by_name(ifp->name, vrf_id) == NULL)
-		listnode_add_sort(intf_list, ifp);
-	else
-		zlog_err(
-			"if_create(%s): corruption detected -- interface with this "
-			"name exists already in VRF %u!",
-			ifp->name, vrf_id);
+	vrf = vrf_get(ifp->vrf_id, NULL);
 
-	return;
+	IFNAME_RB_INSERT(vrf, ifp);
+	if (ifp->ifindex != IFINDEX_INTERNAL)
+		IFINDEX_RB_INSERT(vrf, ifp);
 }
 
 
 /* Delete interface structure. */
 void if_delete_retain(struct interface *ifp)
 {
-	if (if_master.if_delete_hook)
-		(*if_master.if_delete_hook)(ifp);
-
+	hook_call(if_del, ifp);
 	QOBJ_UNREG(ifp);
 
 	/* Free connected address list */
@@ -204,44 +194,52 @@ void if_delete_retain(struct interface *ifp)
 /* Delete and free interface structure. */
 void if_delete(struct interface *ifp)
 {
-	listnode_delete(vrf_iflist(ifp->vrf_id), ifp);
+	struct vrf *vrf;
+
+	vrf = vrf_lookup_by_id(ifp->vrf_id);
+	assert(vrf);
+
+	IFNAME_RB_REMOVE(vrf, ifp);
+	if (ifp->ifindex != IFINDEX_INTERNAL)
+		IFINDEX_RB_REMOVE(vrf, ifp);
 
 	if_delete_retain(ifp);
 
-	list_free(ifp->connected);
-	list_free(ifp->nbr_connected);
+	list_delete_and_null(&ifp->connected);
+	list_delete_and_null(&ifp->nbr_connected);
 
 	if_link_params_free(ifp);
 
-	XFREE(MTYPE_IF, ifp);
-}
+	if (ifp->desc)
+		XFREE(MTYPE_TMP, ifp->desc);
 
-/* Add hook to interface master. */
-void if_add_hook(int type, int (*func)(struct interface *ifp))
-{
-	switch (type) {
-	case IF_NEW_HOOK:
-		if_master.if_new_hook = func;
-		break;
-	case IF_DELETE_HOOK:
-		if_master.if_delete_hook = func;
-		break;
-	default:
-		break;
-	}
+	XFREE(MTYPE_IF, ifp);
 }
 
 /* Interface existance check by index. */
 struct interface *if_lookup_by_index(ifindex_t ifindex, vrf_id_t vrf_id)
 {
-	struct listnode *node;
-	struct interface *ifp;
+	struct vrf *vrf;
+	struct interface if_tmp;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(vrf_id), node, ifp)) {
-		if (ifp->ifindex == ifindex)
-			return ifp;
+	if (vrf_id == VRF_UNKNOWN) {
+		struct interface *ifp;
+
+		RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id) {
+			ifp = if_lookup_by_index(ifindex, vrf->vrf_id);
+			if (ifp)
+				return ifp;
+		}
+
+		return NULL;
 	}
-	return NULL;
+
+	vrf = vrf_lookup_by_id(vrf_id);
+	if (!vrf)
+		return NULL;
+
+	if_tmp.ifindex = ifindex;
+	return RB_FIND(if_index_head, &vrf->ifaces_by_index, &if_tmp);
 }
 
 const char *ifindex2ifname(ifindex_t ifindex, vrf_id_t vrf_id)
@@ -265,15 +263,15 @@ ifindex_t ifname2ifindex(const char *name, vrf_id_t vrf_id)
 /* Interface existance check by interface name. */
 struct interface *if_lookup_by_name(const char *name, vrf_id_t vrf_id)
 {
-	struct listnode *node;
-	struct interface *ifp;
+	struct vrf *vrf = vrf_lookup_by_id(vrf_id);
+	struct interface if_tmp;
 
-	if (name)
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(vrf_id), node, ifp)) {
-			if (strcmp(name, ifp->name) == 0)
-				return ifp;
-		}
-	return NULL;
+	if (!vrf || !name
+	    || strnlen(name, INTERFACE_NAMSIZ) == INTERFACE_NAMSIZ)
+		return NULL;
+
+	strlcpy(if_tmp.name, name, sizeof(if_tmp.name));
+	return RB_FIND(if_name_head, &vrf->ifaces_by_name, &if_tmp);
 }
 
 struct interface *if_lookup_by_name_all_vrf(const char *name)
@@ -281,8 +279,10 @@ struct interface *if_lookup_by_name_all_vrf(const char *name)
 	struct vrf *vrf;
 	struct interface *ifp;
 
-	RB_FOREACH(vrf, vrf_id_head, &vrfs_by_id)
-	{
+	if (!name || strnlen(name, INTERFACE_NAMSIZ) == INTERFACE_NAMSIZ)
+		return NULL;
+
+	RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id) {
 		ifp = if_lookup_by_name(name, vrf->vrf_id);
 		if (ifp)
 			return ifp;
@@ -291,34 +291,17 @@ struct interface *if_lookup_by_name_all_vrf(const char *name)
 	return NULL;
 }
 
-struct interface *if_lookup_by_name_len(const char *name, size_t namelen,
-					vrf_id_t vrf_id)
-{
-	struct listnode *node;
-	struct interface *ifp;
-
-	if (namelen > INTERFACE_NAMSIZ)
-		return NULL;
-
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(vrf_id), node, ifp)) {
-		if (!memcmp(name, ifp->name, namelen)
-		    && (ifp->name[namelen] == '\0'))
-			return ifp;
-	}
-	return NULL;
-}
-
 /* Lookup interface by IPv4 address. */
 struct interface *if_lookup_exact_address(void *src, int family,
 					  vrf_id_t vrf_id)
 {
-	struct listnode *node;
+	struct vrf *vrf = vrf_lookup_by_id(vrf_id);
 	struct listnode *cnode;
 	struct interface *ifp;
 	struct prefix *p;
 	struct connected *c;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(vrf_id), node, ifp)) {
+	FOR_ALL_INTERFACES (vrf, ifp) {
 		for (ALL_LIST_ELEMENTS_RO(ifp->connected, cnode, c)) {
 			p = c->address;
 
@@ -344,7 +327,7 @@ struct interface *if_lookup_exact_address(void *src, int family,
 struct connected *if_lookup_address(void *matchaddr, int family,
 				    vrf_id_t vrf_id)
 {
-	struct listnode *node;
+	struct vrf *vrf = vrf_lookup_by_id(vrf_id);
 	struct prefix addr;
 	int bestlen = 0;
 	struct listnode *cnode;
@@ -364,7 +347,7 @@ struct connected *if_lookup_address(void *matchaddr, int family,
 
 	match = NULL;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(vrf_id), node, ifp)) {
+	FOR_ALL_INTERFACES (vrf, ifp) {
 		for (ALL_LIST_ELEMENTS_RO(ifp->connected, cnode, c)) {
 			if (c->address && (c->address->family == AF_INET)
 			    && prefix_match(CONNECTED_PREFIX(c), &addr)
@@ -380,12 +363,12 @@ struct connected *if_lookup_address(void *matchaddr, int family,
 /* Lookup interface by prefix */
 struct interface *if_lookup_prefix(struct prefix *prefix, vrf_id_t vrf_id)
 {
-	struct listnode *node;
+	struct vrf *vrf = vrf_lookup_by_id(vrf_id);
 	struct listnode *cnode;
 	struct interface *ifp;
 	struct connected *c;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(vrf_id), node, ifp)) {
+	FOR_ALL_INTERFACES (vrf, ifp) {
 		for (ALL_LIST_ELEMENTS_RO(ifp->connected, cnode, c)) {
 			if (prefix_cmp(c->address, prefix) == 0) {
 				return ifp;
@@ -397,54 +380,57 @@ struct interface *if_lookup_prefix(struct prefix *prefix, vrf_id_t vrf_id)
 
 /* Get interface by name if given name interface doesn't exist create
    one. */
-struct interface *if_get_by_name(const char *name, vrf_id_t vrf_id)
+struct interface *if_get_by_name(const char *name, vrf_id_t vrf_id, int vty)
 {
 	struct interface *ifp;
 
-	return ((ifp = if_lookup_by_name(name, vrf_id)) != NULL)
-		       ? ifp
-		       : if_create(name, strlen(name), vrf_id);
-}
-
-struct interface *if_get_by_name_len(const char *name, size_t namelen,
-				     vrf_id_t vrf_id, int vty)
-{
-	struct interface *ifp;
-	struct vrf *vrf;
-	struct listnode *node;
-
-	ifp = if_lookup_by_name_len(name, namelen, vrf_id);
+	ifp = if_lookup_by_name(name, vrf_id);
 	if (ifp)
 		return ifp;
-
-	/* Didn't find the interface on that vrf. Defined on a different one? */
-	RB_FOREACH(vrf, vrf_id_head, &vrfs_by_id)
-	{
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(vrf->vrf_id), node, ifp)) {
-			if (!memcmp(name, ifp->name, namelen)
-			    && (ifp->name[namelen] == '\0')) {
-				/* Found a match.  If the interface command was
-				 * entered in vty without a
-				 * VRF (passed as VRF_DEFAULT), accept the ifp
-				 * we found.   If a vrf was
-				 * entered and there is a mismatch, reject it if
-				 * from vty. If it came
-				 * from the kernel by way of zclient,  believe
-				 * it and update
-				 * the ifp accordingly.
-				 */
-				if (vty) {
-					if (vrf_id == VRF_DEFAULT)
-						return ifp;
-					return NULL;
-				} else {
-					if_update(ifp, name, namelen, vrf_id);
-					return ifp;
-				}
-			}
-		}
+	/* Not Found on same VRF. If the interface command
+	 * was entered in vty without a VRF (passed as VRF_DEFAULT),
+	 * accept the ifp we found. If a vrf was entered and there is
+	 * a mismatch, reject it if from vty.
+	 */
+	ifp = if_lookup_by_name_all_vrf(name);
+	if (!ifp)
+		return if_create(name, vrf_id);
+	if (vty) {
+		if (vrf_id == VRF_DEFAULT)
+			return ifp;
+		return NULL;
 	}
-	return (if_create(name, namelen, vrf_id));
+	/* if vrf backend uses NETNS, then
+	 * this should not be considered as an update
+	 * then create the new interface
+	 */
+	if (ifp->vrf_id != vrf_id && vrf_is_mapped_on_netns(vrf_id))
+		return if_create(name, vrf_id);
+	/* If it came from the kernel
+	 * or by way of zclient, believe it and update
+	 * the ifp accordingly.
+	 */
+	if_update_to_new_vrf(ifp, vrf_id);
+	return ifp;
+}
+
+void if_set_index(struct interface *ifp, ifindex_t ifindex)
+{
+	struct vrf *vrf;
+
+	vrf = vrf_lookup_by_id(ifp->vrf_id);
+	assert(vrf);
+
+	if (ifp->ifindex == ifindex)
+		return;
+
+	if (ifp->ifindex != IFINDEX_INTERNAL)
+		IFINDEX_RB_REMOVE(vrf, ifp)
+
+	ifp->ifindex = ifindex;
+
+	if (ifp->ifindex != IFINDEX_INTERNAL)
+		IFINDEX_RB_INSERT(vrf, ifp)
 }
 
 /* Does interface up ? */
@@ -490,6 +476,12 @@ int if_is_loopback(struct interface *ifp)
 	return (ifp->flags & (IFF_LOOPBACK | IFF_NOXMIT | IFF_VIRTUAL));
 }
 
+/* Check interface is VRF */
+int if_is_vrf(struct interface *ifp)
+{
+	return CHECK_FLAG(ifp->status, ZEBRA_INTERFACE_VRF_LOOPBACK);
+}
+
 /* Does this interface support broadcast ? */
 int if_is_broadcast(struct interface *ifp)
 {
@@ -570,13 +562,11 @@ static void if_dump(const struct interface *ifp)
 void if_dump_all(void)
 {
 	struct vrf *vrf;
-	struct listnode *node;
-	void *p;
+	void *ifp;
 
-	RB_FOREACH(vrf, vrf_id_head, &vrfs_by_id)
-	if (vrf->iflist != NULL)
-		for (ALL_LIST_ELEMENTS_RO(vrf->iflist, node, p))
-			if_dump(p);
+	RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id)
+		FOR_ALL_INTERFACES (vrf, ifp)
+			if_dump(ifp);
 }
 
 DEFUN (interface_desc,
@@ -631,24 +621,20 @@ DEFUN (no_interface_desc,
  *     if not:
  *     - no idea, just get the name in its entirety.
  */
-static struct interface *if_sunwzebra_get(const char *name, size_t nlen,
-					  vrf_id_t vrf_id)
+static struct interface *if_sunwzebra_get(char *name, vrf_id_t vrf_id)
 {
 	struct interface *ifp;
-	size_t seppos = 0;
+	char *cp;
 
-	if ((ifp = if_lookup_by_name_len(name, nlen, vrf_id)) != NULL)
+	if ((ifp = if_lookup_by_name(name, vrf_id)) != NULL)
 		return ifp;
 
 	/* hunt the primary interface name... */
-	while (seppos < nlen && name[seppos] != ':')
-		seppos++;
+	cp = strchr(name, ':');
+	if (cp)
+		*cp = '\0';
 
-	/* Wont catch seperator as last char, e.g. 'foo0:' but thats invalid */
-	if (seppos < nlen)
-		return if_get_by_name_len(name, seppos, vrf_id, 1);
-	else
-		return if_get_by_name_len(name, nlen, vrf_id, 1);
+	return if_get_by_name(name, vrf_id, 1);
 }
 #endif /* SUNOS_5 */
 
@@ -665,15 +651,14 @@ DEFUN (interface,
 	const char *vrfname = (argc > 2) ? argv[idx_vrf]->arg : NULL;
 
 	struct interface *ifp;
-	size_t sl;
 	vrf_id_t vrf_id = VRF_DEFAULT;
 
-	if ((sl = strlen(ifname)) > INTERFACE_NAMSIZ) {
+	if (strlen(ifname) > INTERFACE_NAMSIZ) {
 		vty_out(vty,
 			"%% Interface name %s is invalid: length exceeds "
-			"%d characters%s",
-			ifname, INTERFACE_NAMSIZ, VTY_NEWLINE);
-		return CMD_WARNING;
+			"%d characters\n",
+			ifname, INTERFACE_NAMSIZ);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	/*Pending: need proper vrf name based lookup/(possible creation of VRF)
@@ -682,15 +667,14 @@ DEFUN (interface,
 		VRF_GET_ID(vrf_id, vrfname);
 
 #ifdef SUNOS_5
-	ifp = if_sunwzebra_get(ifname, sl, vrf_id);
+	ifp = if_sunwzebra_get(ifname, vrf_id);
 #else
-	ifp = if_get_by_name_len(ifname, sl, vrf_id, 1);
+	ifp = if_get_by_name(ifname, vrf_id, 1);
 #endif /* SUNOS_5 */
 
 	if (!ifp) {
-		vty_out(vty, "%% interface %s not in %s%s", ifname, vrfname,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% interface %s not in %s\n", ifname, vrfname);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	VTY_PUSH_CONTEXT(INTERFACE_NODE, ifp);
 
@@ -705,8 +689,9 @@ DEFUN_NOSH (no_interface,
            "Interface's name\n"
            VRF_CMD_HELP_STR)
 {
+	int idx_vrf = 4;
 	const char *ifname = argv[2]->arg;
-	const char *vrfname = (argc > 3) ? argv[3]->arg : NULL;
+	const char *vrfname = (argc > 3) ? argv[idx_vrf]->arg : NULL;
 
 	// deleting interface
 	struct interface *ifp;
@@ -718,15 +703,13 @@ DEFUN_NOSH (no_interface,
 	ifp = if_lookup_by_name(ifname, vrf_id);
 
 	if (ifp == NULL) {
-		vty_out(vty, "%% Interface %s does not exist%s", ifname,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% Interface %s does not exist\n", ifname);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (CHECK_FLAG(ifp->status, ZEBRA_INTERFACE_ACTIVE)) {
-		vty_out(vty, "%% Only inactive interfaces can be deleted%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% Only inactive interfaces can be deleted\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if_delete(ifp);
@@ -734,8 +717,30 @@ DEFUN_NOSH (no_interface,
 	return CMD_SUCCESS;
 }
 
+static void if_autocomplete(vector comps, struct cmd_token *token)
+{
+	struct interface *ifp;
+	struct vrf *vrf = NULL;
+
+	RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
+		FOR_ALL_INTERFACES (vrf, ifp) {
+			vector_set(comps, XSTRDUP(MTYPE_COMPLETION, ifp->name));
+		}
+	}
+}
+
+static const struct cmd_variable_handler if_var_handlers[] = {
+	{/* "interface NAME" */
+	 .varname = "interface",
+	 .completions = if_autocomplete},
+	{.tokenname = "IFNAME", .completions = if_autocomplete},
+	{.tokenname = "INTERFACE", .completions = if_autocomplete},
+	{.completions = NULL}};
+
 void if_cmd_init(void)
 {
+	cmd_variable_handler_register(if_var_handlers);
+
 	install_element(CONFIG_NODE, &interface_cmd);
 	install_element(CONFIG_NODE, &no_interface_cmd);
 
@@ -755,7 +760,6 @@ DEFUN (show_address,
 {
   int idx_vrf = 3;
   struct listnode *node;
-  struct listnode *node2;
   struct interface *ifp;
   struct connected *ifc;
   struct prefix *p;
@@ -764,15 +768,14 @@ DEFUN (show_address,
   if (argc > 2)
     VRF_GET_ID (vrf_id, argv[idx_vrf]->arg);
 
-  for (ALL_LIST_ELEMENTS_RO (vrf_iflist (vrf_id), node, ifp))
+  FOR_ALL_INTERFACES (vrf, ifp)
     {
-      for (ALL_LIST_ELEMENTS_RO (ifp->connected, node2, ifc))
+      for (ALL_LIST_ELEMENTS_RO (ifp->connected, node, ifc))
 	{
 	  p = ifc->address;
 
 	  if (p->family == AF_INET)
-	    vty_out (vty, "%s/%d%s", inet_ntoa (p->u.prefix4), p->prefixlen,
-		     VTY_NEWLINE);
+	    vty_out (vty, "%s/%d\n", inet_ntoa (p->u.prefix4), p->prefixlen);
 	}
     }
   return CMD_SUCCESS;
@@ -787,28 +790,25 @@ DEFUN (show_address_vrf_all,
 {
   struct vrf *vrf;
   struct listnode *node;
-  struct listnode *node2;
   struct interface *ifp;
   struct connected *ifc;
   struct prefix *p;
 
   RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name)
     {
-      if (!vrf->iflist || !listcount (vrf->iflist))
+      if (RB_EMPTY (if_name_head, &vrf->ifaces_by_name))
         continue;
 
-      vty_out (vty, "%sVRF %u%s%s", VTY_NEWLINE, vrf->vrf_id, VTY_NEWLINE,
-	       VTY_NEWLINE);
+      vty_out (vty, "\nVRF %u\n\n", vrf->vrf_id);
 
-      for (ALL_LIST_ELEMENTS_RO (vrf->iflist, node, ifp))
+      FOR_ALL_INTERFACES (vrf, ifp)
         {
-          for (ALL_LIST_ELEMENTS_RO (ifp->connected, node2, ifc))
+          for (ALL_LIST_ELEMENTS_RO (ifp->connected, node, ifc))
             {
               p = ifc->address;
 
               if (p->family == AF_INET)
-                vty_out (vty, "%s/%d%s", inet_ntoa (p->u.prefix4), p->prefixlen,
-                         VTY_NEWLINE);
+                vty_out (vty, "%s/%d\n", inet_ntoa (p->u.prefix4), p->prefixlen);
             }
         }
     }
@@ -1075,7 +1075,7 @@ ifaddr_ipv4_lookup (struct in_addr *addr, ifindex_t ifindex)
       rn = route_node_lookup (ifaddr_ipv4_table, (struct prefix *) &p);
       if (! rn)
 	return NULL;
-      
+
       ifp = rn->info;
       route_unlock_node (rn);
       return ifp;
@@ -1085,36 +1085,19 @@ ifaddr_ipv4_lookup (struct in_addr *addr, ifindex_t ifindex)
 }
 #endif /* ifaddr_ipv4_table */
 
-/* Initialize interface list. */
-void if_init(struct list **intf_list)
+void if_terminate(struct vrf *vrf)
 {
-	*intf_list = list_new();
-#if 0
-  ifaddr_ipv4_table = route_table_init ();
-#endif /* ifaddr_ipv4_table */
+	struct interface *ifp;
 
-	(*intf_list)->cmp = (int (*)(void *, void *))if_cmp_func;
-}
-
-void if_terminate(struct list **intf_list)
-{
-	for (;;) {
-		struct interface *ifp;
-
-		ifp = listnode_head(*intf_list);
-		if (ifp == NULL)
-			break;
+	while (!RB_EMPTY(if_name_head, &vrf->ifaces_by_name)) {
+		ifp = RB_ROOT(if_name_head, &vrf->ifaces_by_name);
 
 		if (ifp->node) {
 			ifp->node->info = NULL;
 			route_unlock_node(ifp->node);
 		}
-
 		if_delete(ifp);
 	}
-
-	list_delete(*intf_list);
-	*intf_list = NULL;
 }
 
 const char *if_link_type_str(enum zebra_link_type llt)
