@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #define FRR_DEFINE_DESC_TABLE
@@ -42,6 +41,10 @@ DEFINE_MTYPE_STATIC(LIB, ZLOG, "Logging")
 static int logfile_fd = -1; /* Used in signal handler. */
 
 struct zlog *zlog_default = NULL;
+bool zlog_startup_stderr = true;
+
+/* lock protecting zlog_default for mt-safe zlog */
+pthread_mutex_t loglock = PTHREAD_MUTEX_INITIALIZER;
 
 const char *zlog_priority[] = {
 	"emergencies",   "alerts",	"critical",  "errors", "warnings",
@@ -173,9 +176,29 @@ static void time_print(FILE *fp, struct timestamp_control *ctl)
 }
 
 
+static void vzlog_file(struct zlog *zl, struct timestamp_control *tsctl,
+		       const char *proto_str, int record_priority, int priority,
+		       FILE *fp, const char *format, va_list args)
+{
+	va_list ac;
+
+	time_print(fp, tsctl);
+	if (record_priority)
+		fprintf(fp, "%s: ", zlog_priority[priority]);
+
+	fprintf(fp, "%s", proto_str);
+	va_copy(ac, args);
+	vfprintf(fp, format, ac);
+	va_end(ac);
+	fprintf(fp, "\n");
+	fflush(fp);
+}
+
 /* va_list version of zlog. */
 void vzlog(int priority, const char *format, va_list args)
 {
+	pthread_mutex_lock(&loglock);
+
 	char proto_str[32];
 	int original_errno = errno;
 	struct timestamp_control tsctl;
@@ -193,6 +216,7 @@ void vzlog(int priority, const char *format, va_list args)
 
 		/* In this case we return at here. */
 		errno = original_errno;
+		pthread_mutex_unlock(&loglock);
 		return;
 	}
 	tsctl.precision = zl->timestamp_precision;
@@ -211,32 +235,21 @@ void vzlog(int priority, const char *format, va_list args)
 		sprintf(proto_str, "%s: ", zl->protoname);
 
 	/* File output. */
-	if ((priority <= zl->maxlvl[ZLOG_DEST_FILE]) && zl->fp) {
-		va_list ac;
-		time_print(zl->fp, &tsctl);
-		if (zl->record_priority)
-			fprintf(zl->fp, "%s: ", zlog_priority[priority]);
-		fprintf(zl->fp, "%s", proto_str);
-		va_copy(ac, args);
-		vfprintf(zl->fp, format, ac);
-		va_end(ac);
-		fprintf(zl->fp, "\n");
-		fflush(zl->fp);
-	}
+	if ((priority <= zl->maxlvl[ZLOG_DEST_FILE]) && zl->fp)
+		vzlog_file(zl, &tsctl, proto_str, zl->record_priority, priority,
+			   zl->fp, format, args);
 
-	/* stdout output. */
-	if (priority <= zl->maxlvl[ZLOG_DEST_STDOUT]) {
-		va_list ac;
-		time_print(stdout, &tsctl);
-		if (zl->record_priority)
-			fprintf(stdout, "%s: ", zlog_priority[priority]);
-		fprintf(stdout, "%s", proto_str);
-		va_copy(ac, args);
-		vfprintf(stdout, format, ac);
-		va_end(ac);
-		fprintf(stdout, "\n");
-		fflush(stdout);
-	}
+	/* fixed-config logging to stderr while we're stating up & haven't
+	 * daemonized / reached mainloop yet
+	 *
+	 * note the "else" on stdout output -- we don't want to print the same
+	 * message to both stderr and stdout. */
+	if (zlog_startup_stderr && priority <= LOG_WARNING)
+		vzlog_file(zl, &tsctl, proto_str, 1, priority, stderr, format,
+			   args);
+	else if (priority <= zl->maxlvl[ZLOG_DEST_STDOUT])
+		vzlog_file(zl, &tsctl, proto_str, zl->record_priority, priority,
+			   stdout, format, args);
 
 	/* Terminal monitor. */
 	if (priority <= zl->maxlvl[ZLOG_DEST_MONITOR])
@@ -244,37 +257,36 @@ void vzlog(int priority, const char *format, va_list args)
 			proto_str, format, &tsctl, args);
 
 	errno = original_errno;
+	pthread_mutex_unlock(&loglock);
 }
 
 int vzlog_test(int priority)
 {
+	pthread_mutex_lock(&loglock);
+
+	int ret = 0;
+
 	struct zlog *zl = zlog_default;
 
 	/* When zlog_default is also NULL, use stderr for logging. */
-	if (zl == NULL) {
-		return 1;
-	}
-
+	if (zl == NULL)
+		ret = 1;
 	/* Syslog output */
-	if (priority <= zl->maxlvl[ZLOG_DEST_SYSLOG]) {
-		return 1;
-	}
-
+	else if (priority <= zl->maxlvl[ZLOG_DEST_SYSLOG])
+		ret = 1;
 	/* File output. */
-	if ((priority <= zl->maxlvl[ZLOG_DEST_FILE]) && zl->fp) {
-		return 1;
-	}
-
+	else if ((priority <= zl->maxlvl[ZLOG_DEST_FILE]) && zl->fp)
+		ret = 1;
 	/* stdout output. */
-	if (priority <= zl->maxlvl[ZLOG_DEST_STDOUT]) {
-		return 1;
-	}
-
+	else if (priority <= zl->maxlvl[ZLOG_DEST_STDOUT])
+		ret = 1;
 	/* Terminal monitor. */
-	if (priority <= zl->maxlvl[ZLOG_DEST_MONITOR])
-		return 1;
+	else if (priority <= zl->maxlvl[ZLOG_DEST_MONITOR])
+		ret = 1;
 
-	return 0;
+	pthread_mutex_unlock(&loglock);
+
+	return ret;
 }
 
 static char *str_append(char *dst, int len, const char *src)
@@ -284,7 +296,7 @@ static char *str_append(char *dst, int len, const char *src)
 	return dst;
 }
 
-static char *num_append(char *s, int len, u_long x)
+static char *num_append(char *s, int len, unsigned long x)
 {
 	char buf[30];
 	char *t;
@@ -300,7 +312,7 @@ static char *num_append(char *s, int len, u_long x)
 }
 
 #if defined(SA_SIGINFO) || defined(HAVE_STACK_TRACE)
-static char *hex_append(char *s, int len, u_long x)
+static char *hex_append(char *s, int len, unsigned long x)
 {
 	char buf[30];
 	char *t;
@@ -309,7 +321,7 @@ static char *hex_append(char *s, int len, u_long x)
 		return str_append(s, len, "0");
 	*(t = &buf[sizeof(buf) - 1]) = '\0';
 	while (x && (t > buf)) {
-		u_int cc = (x % 16);
+		unsigned int cc = (x % 16);
 		*--t = ((cc < 10) ? ('0' + cc) : ('a' + cc - 10));
 		x /= 16;
 	}
@@ -434,10 +446,10 @@ void zlog_signal(int signo, const char *action
 	s = num_append(LOC, now);
 #ifdef SA_SIGINFO
 	s = str_append(LOC, " (si_addr 0x");
-	s = hex_append(LOC, (u_long)(siginfo->si_addr));
+	s = hex_append(LOC, (unsigned long)(siginfo->si_addr));
 	if (program_counter) {
 		s = str_append(LOC, ", PC 0x");
-		s = hex_append(LOC, (u_long)program_counter);
+		s = hex_append(LOC, (unsigned long)program_counter);
 	}
 	s = str_append(LOC, "); ");
 #else  /* SA_SIGINFO */
@@ -478,15 +490,17 @@ void zlog_signal(int signo, const char *action
 			       );
 
 	s = buf;
-	if (!thread_current)
+	struct thread *tc;
+	tc = pthread_getspecific(thread_current);
+	if (!tc)
 		s = str_append(LOC, "no thread information available\n");
 	else {
 		s = str_append(LOC, "in thread ");
-		s = str_append(LOC, thread_current->funcname);
+		s = str_append(LOC, tc->funcname);
 		s = str_append(LOC, " scheduled from ");
-		s = str_append(LOC, thread_current->schedfrom);
+		s = str_append(LOC, tc->schedfrom);
 		s = str_append(LOC, ":");
-		s = num_append(LOC, thread_current->schedfrom_line);
+		s = num_append(LOC, tc->schedfrom_line);
 		s = str_append(LOC, "\n");
 	}
 
@@ -582,7 +596,8 @@ void zlog_backtrace_sigsafe(int priority, void *program_counter)
 					s = str_append(LOC, "[bt ");
 					s = num_append(LOC, i);
 					s = str_append(LOC, "] 0x");
-					s = hex_append(LOC, (u_long)(array[i]));
+					s = hex_append(
+						LOC, (unsigned long)(array[i]));
 				}
 				*s = '\0';
 				if (priority
@@ -667,12 +682,14 @@ ZLOG_FUNC(zlog_debug, LOG_DEBUG)
 
 void zlog_thread_info(int log_level)
 {
-	if (thread_current)
+	struct thread *tc;
+	tc = pthread_getspecific(thread_current);
+
+	if (tc)
 		zlog(log_level,
 		     "Current thread function %s, scheduled from "
 		     "file %s, line %u",
-		     thread_current->funcname, thread_current->schedfrom,
-		     thread_current->schedfrom_line);
+		     tc->funcname, tc->schedfrom, tc->schedfrom_line);
 	else
 		zlog(log_level, "Current thread not known/applicable");
 }
@@ -689,7 +706,7 @@ void _zlog_assert_failed(const char *assertion, const char *file,
 	     assertion, file, line, (function ? function : "?"));
 	zlog_backtrace(LOG_CRIT);
 	zlog_thread_info(LOG_CRIT);
-	log_memstats_stderr("log");
+	log_memstats(stderr, "log");
 	abort();
 }
 
@@ -704,11 +721,11 @@ void memory_oom(size_t size, const char *name)
 }
 
 /* Open log stream */
-void openzlog(const char *progname, const char *protoname, u_short instance,
-	      int syslog_flags, int syslog_facility)
+void openzlog(const char *progname, const char *protoname,
+	      unsigned short instance, int syslog_flags, int syslog_facility)
 {
 	struct zlog *zl;
-	u_int i;
+	unsigned int i;
 
 	zl = XCALLOC(MTYPE_ZLOG, sizeof(struct zlog));
 
@@ -725,11 +742,27 @@ void openzlog(const char *progname, const char *protoname, u_short instance,
 	zl->default_lvl = LOG_DEBUG;
 
 	openlog(progname, syslog_flags, zl->facility);
+
+	pthread_mutex_lock(&loglock);
 	zlog_default = zl;
+	pthread_mutex_unlock(&loglock);
+
+#ifdef HAVE_GLIBC_BACKTRACE
+	/* work around backtrace() using lazily resolved dynamically linked
+	 * symbols, which will otherwise cause funny breakage in the SEGV
+	 * handler.
+	 * (particularly, the dynamic linker can call malloc(), which uses locks
+	 * in programs linked with -pthread, thus can deadlock.) */
+	void *bt[4];
+	backtrace(bt, array_size(bt));
+	free(backtrace_symbols(bt, 0));
+	backtrace_symbols_fd(bt, 0, 0);
+#endif
 }
 
 void closezlog(void)
 {
+	pthread_mutex_lock(&loglock);
 	struct zlog *zl = zlog_default;
 
 	closelog();
@@ -742,24 +775,23 @@ void closezlog(void)
 
 	XFREE(MTYPE_ZLOG, zl);
 	zlog_default = NULL;
-}
-
-const char *zlog_protoname(void)
-{
-	return zlog_default ? zlog_default->protoname : "NONE";
+	pthread_mutex_unlock(&loglock);
 }
 
 /* Called from command.c. */
 void zlog_set_level(zlog_dest_t dest, int log_level)
 {
+	pthread_mutex_lock(&loglock);
 	zlog_default->maxlvl[dest] = log_level;
+	pthread_mutex_unlock(&loglock);
 }
 
 int zlog_set_file(const char *filename, int log_level)
 {
-	struct zlog *zl = zlog_default;
+	struct zlog *zl;
 	FILE *fp;
 	mode_t oldumask;
+	int ret = 1;
 
 	/* There is opend file.  */
 	zlog_reset_file();
@@ -768,21 +800,28 @@ int zlog_set_file(const char *filename, int log_level)
 	oldumask = umask(0777 & ~LOGFILE_MASK);
 	fp = fopen(filename, "a");
 	umask(oldumask);
-	if (fp == NULL)
-		return 0;
+	if (fp == NULL) {
+		ret = 0;
+	} else {
+		pthread_mutex_lock(&loglock);
+		zl = zlog_default;
 
-	/* Set flags. */
-	zl->filename = XSTRDUP(MTYPE_ZLOG, filename);
-	zl->maxlvl[ZLOG_DEST_FILE] = log_level;
-	zl->fp = fp;
-	logfile_fd = fileno(fp);
+		/* Set flags. */
+		zl->filename = XSTRDUP(MTYPE_ZLOG, filename);
+		zl->maxlvl[ZLOG_DEST_FILE] = log_level;
+		zl->fp = fp;
+		logfile_fd = fileno(fp);
+		pthread_mutex_unlock(&loglock);
+	}
 
-	return 1;
+	return ret;
 }
 
 /* Reset opend file. */
 int zlog_reset_file(void)
 {
+	pthread_mutex_lock(&loglock);
+
 	struct zlog *zl = zlog_default;
 
 	if (zl->fp)
@@ -795,14 +834,19 @@ int zlog_reset_file(void)
 		XFREE(MTYPE_ZLOG, zl->filename);
 	zl->filename = NULL;
 
+	pthread_mutex_unlock(&loglock);
+
 	return 1;
 }
 
 /* Reopen log file. */
 int zlog_rotate(void)
 {
+	pthread_mutex_lock(&loglock);
+
 	struct zlog *zl = zlog_default;
 	int level;
+	int ret = 1;
 
 	if (zl->fp)
 		fclose(zl->fp);
@@ -823,13 +867,16 @@ int zlog_rotate(void)
 			zlog_err(
 				"Log rotate failed: cannot open file %s for append: %s",
 				zl->filename, safe_strerror(save_errno));
-			return -1;
+			ret = -1;
+		} else {
+			logfile_fd = fileno(zl->fp);
+			zl->maxlvl[ZLOG_DEST_FILE] = level;
 		}
-		logfile_fd = fileno(zl->fp);
-		zl->maxlvl[ZLOG_DEST_FILE] = level;
 	}
 
-	return 1;
+	pthread_mutex_unlock(&loglock);
+
+	return ret;
 }
 
 /* Wrapper around strerror to handle case where it returns NULL. */
@@ -847,6 +894,10 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_INTERFACE_ADDRESS_DELETE),
 	DESC_ENTRY(ZEBRA_INTERFACE_UP),
 	DESC_ENTRY(ZEBRA_INTERFACE_DOWN),
+	DESC_ENTRY(ZEBRA_INTERFACE_SET_MASTER),
+	DESC_ENTRY(ZEBRA_ROUTE_ADD),
+	DESC_ENTRY(ZEBRA_ROUTE_DELETE),
+	DESC_ENTRY(ZEBRA_ROUTE_NOTIFY_OWNER),
 	DESC_ENTRY(ZEBRA_IPV4_ROUTE_ADD),
 	DESC_ENTRY(ZEBRA_IPV4_ROUTE_DELETE),
 	DESC_ENTRY(ZEBRA_IPV6_ROUTE_ADD),
@@ -873,13 +924,12 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_BFD_DEST_DEREGISTER),
 	DESC_ENTRY(ZEBRA_BFD_DEST_UPDATE),
 	DESC_ENTRY(ZEBRA_BFD_DEST_REPLAY),
-	DESC_ENTRY(ZEBRA_REDISTRIBUTE_IPV4_ADD),
-	DESC_ENTRY(ZEBRA_REDISTRIBUTE_IPV4_DEL),
-	DESC_ENTRY(ZEBRA_REDISTRIBUTE_IPV6_ADD),
-	DESC_ENTRY(ZEBRA_REDISTRIBUTE_IPV6_DEL),
+	DESC_ENTRY(ZEBRA_REDISTRIBUTE_ROUTE_ADD),
+	DESC_ENTRY(ZEBRA_REDISTRIBUTE_ROUTE_DEL),
 	DESC_ENTRY(ZEBRA_VRF_UNREGISTER),
 	DESC_ENTRY(ZEBRA_VRF_ADD),
 	DESC_ENTRY(ZEBRA_VRF_DELETE),
+	DESC_ENTRY(ZEBRA_VRF_LABEL),
 	DESC_ENTRY(ZEBRA_INTERFACE_VRF_UPDATE),
 	DESC_ENTRY(ZEBRA_BFD_CLIENT_REGISTER),
 	DESC_ENTRY(ZEBRA_INTERFACE_ENABLE_RADV),
@@ -888,27 +938,49 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_INTERFACE_LINK_PARAMS),
 	DESC_ENTRY(ZEBRA_MPLS_LABELS_ADD),
 	DESC_ENTRY(ZEBRA_MPLS_LABELS_DELETE),
-	DESC_ENTRY(ZEBRA_IPV4_NEXTHOP_ADD),
-	DESC_ENTRY(ZEBRA_IPV4_NEXTHOP_DELETE),
-	DESC_ENTRY(ZEBRA_IPV6_NEXTHOP_ADD),
-	DESC_ENTRY(ZEBRA_IPV6_NEXTHOP_DELETE),
 	DESC_ENTRY(ZEBRA_IPMR_ROUTE_STATS),
 	DESC_ENTRY(ZEBRA_LABEL_MANAGER_CONNECT),
+	DESC_ENTRY(ZEBRA_LABEL_MANAGER_CONNECT_ASYNC),
 	DESC_ENTRY(ZEBRA_GET_LABEL_CHUNK),
 	DESC_ENTRY(ZEBRA_RELEASE_LABEL_CHUNK),
+	DESC_ENTRY(ZEBRA_ADVERTISE_ALL_VNI),
+	DESC_ENTRY(ZEBRA_ADVERTISE_DEFAULT_GW),
+	DESC_ENTRY(ZEBRA_ADVERTISE_SUBNET),
+	DESC_ENTRY(ZEBRA_VNI_ADD),
+	DESC_ENTRY(ZEBRA_VNI_DEL),
+	DESC_ENTRY(ZEBRA_L3VNI_ADD),
+	DESC_ENTRY(ZEBRA_L3VNI_DEL),
+	DESC_ENTRY(ZEBRA_REMOTE_VTEP_ADD),
+	DESC_ENTRY(ZEBRA_REMOTE_VTEP_DEL),
+	DESC_ENTRY(ZEBRA_MACIP_ADD),
+	DESC_ENTRY(ZEBRA_MACIP_DEL),
+	DESC_ENTRY(ZEBRA_IP_PREFIX_ROUTE_ADD),
+	DESC_ENTRY(ZEBRA_IP_PREFIX_ROUTE_DEL),
+	DESC_ENTRY(ZEBRA_REMOTE_MACIP_ADD),
+	DESC_ENTRY(ZEBRA_REMOTE_MACIP_DEL),
 	DESC_ENTRY(ZEBRA_PW_ADD),
 	DESC_ENTRY(ZEBRA_PW_DELETE),
 	DESC_ENTRY(ZEBRA_PW_SET),
 	DESC_ENTRY(ZEBRA_PW_UNSET),
 	DESC_ENTRY(ZEBRA_PW_STATUS_UPDATE),
+	DESC_ENTRY(ZEBRA_RULE_ADD),
+	DESC_ENTRY(ZEBRA_RULE_DELETE),
+	DESC_ENTRY(ZEBRA_RULE_NOTIFY_OWNER),
+	DESC_ENTRY(ZEBRA_TABLE_MANAGER_CONNECT),
+	DESC_ENTRY(ZEBRA_GET_TABLE_CHUNK),
+	DESC_ENTRY(ZEBRA_RELEASE_TABLE_CHUNK),
+	DESC_ENTRY(ZEBRA_IPSET_CREATE),
+	DESC_ENTRY(ZEBRA_IPSET_DESTROY),
+	DESC_ENTRY(ZEBRA_IPSET_ENTRY_ADD),
+	DESC_ENTRY(ZEBRA_IPSET_ENTRY_DELETE),
 };
 #undef DESC_ENTRY
 
 static const struct zebra_desc_table unknown = {0, "unknown", '?'};
 
-static const struct zebra_desc_table *zroute_lookup(u_int zroute)
+static const struct zebra_desc_table *zroute_lookup(unsigned int zroute)
 {
-	u_int i;
+	unsigned int i;
 
 	if (zroute >= array_size(route_types)) {
 		zlog_err("unknown zebra route type: %u", zroute);
@@ -929,12 +1001,12 @@ static const struct zebra_desc_table *zroute_lookup(u_int zroute)
 	return &unknown;
 }
 
-const char *zebra_route_string(u_int zroute)
+const char *zebra_route_string(unsigned int zroute)
 {
 	return zroute_lookup(zroute)->string;
 }
 
-char zebra_route_char(u_int zroute)
+char zebra_route_char(unsigned int zroute)
 {
 	return zroute_lookup(zroute)->chr;
 }
@@ -972,6 +1044,8 @@ int proto_redistnum(int afi, const char *s)
 			return ZEBRA_ROUTE_STATIC;
 		else if (strmatch(s, "rip"))
 			return ZEBRA_ROUTE_RIP;
+		else if (strmatch(s, "eigrp"))
+			return ZEBRA_ROUTE_EIGRP;
 		else if (strmatch(s, "ospf"))
 			return ZEBRA_ROUTE_OSPF;
 		else if (strmatch(s, "isis"))
@@ -986,6 +1060,10 @@ int proto_redistnum(int afi, const char *s)
 			return ZEBRA_ROUTE_VNC_DIRECT;
 		else if (strmatch(s, "nhrp"))
 			return ZEBRA_ROUTE_NHRP;
+		else if (strmatch(s, "babel"))
+			return ZEBRA_ROUTE_BABEL;
+		else if (strmatch(s, "sharp"))
+			return ZEBRA_ROUTE_SHARP;
 	}
 	if (afi == AFI_IP6) {
 		if (strmatch(s, "kernel"))
@@ -1010,6 +1088,10 @@ int proto_redistnum(int afi, const char *s)
 			return ZEBRA_ROUTE_VNC_DIRECT;
 		else if (strmatch(s, "nhrp"))
 			return ZEBRA_ROUTE_NHRP;
+		else if (strmatch(s, "babel"))
+			return ZEBRA_ROUTE_BABEL;
+		else if (strmatch(s, "sharp"))
+			return ZEBRA_ROUTE_SHARP;
 	}
 	return -1;
 }
@@ -1019,42 +1101,52 @@ void zlog_hexdump(const void *mem, unsigned int len)
 	unsigned long i = 0;
 	unsigned int j = 0;
 	unsigned int columns = 8;
-	char buf[(len * 4) + ((len / 4) * 20) + 30];
+	/*
+	 * 19 bytes for 0xADDRESS:
+	 * 24 bytes for data; 2 chars plus a space per data byte
+	 *  1 byte for space
+	 *  8 bytes for ASCII representation
+	 *  1 byte for a newline
+	 * =====================
+	 * 53 bytes per 8 bytes of data
+	 *  1 byte for null term
+	 */
+	size_t bs = ((len / 8) + 1) * 53 + 1;
+	char buf[bs];
 	char *s = buf;
 
+	memset(buf, 0, sizeof(buf));
+
 	for (i = 0; i < len + ((len % columns) ? (columns - len % columns) : 0);
 	     i++) {
 		/* print offset */
 		if (i % columns == 0)
-			s += sprintf(s, "0x%016lx: ", (unsigned long)mem + i);
+			s += snprintf(s, bs - (s - buf),
+				      "0x%016lx: ", (unsigned long)mem + i);
 
 		/* print hex data */
 		if (i < len)
-			s += sprintf(s, "%02x ", 0xFF & ((const char *)mem)[i]);
+			s += snprintf(s, bs - (s - buf), "%02x ",
+				      0xFF & ((const char *)mem)[i]);
 
 		/* end of block, just aligning for ASCII dump */
 		else
-			s += sprintf(s, "   ");
+			s += snprintf(s, bs - (s - buf), "   ");
 
 		/* print ASCII dump */
 		if (i % columns == (columns - 1)) {
 			for (j = i - (columns - 1); j <= i; j++) {
-				if (j >= len) /* end of block, not really
-						 printing */
-					s += sprintf(s, " ");
-
-				else if (
-					isprint((int)((const char *)mem)
-							[j])) /* printable char
-								 */
-					s += sprintf(
-						s, "%c",
+				/* end of block not really printing */
+				if (j >= len)
+					s += snprintf(s, bs - (s - buf), " ");
+				else if (isprint((int)((const char *)mem)[j]))
+					s += snprintf(
+						s, bs - (s - buf), "%c",
 						0xFF & ((const char *)mem)[j]);
-
 				else /* other char */
-					s += sprintf(s, ".");
+					s += snprintf(s, bs - (s - buf), ".");
 			}
-			s += sprintf(s, "\n");
+			s += snprintf(s, bs - (s - buf), "\n");
 		}
 	}
 	zlog_debug("\n%s", buf);
