@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -27,12 +26,13 @@
 #include "sockunion.h"
 #include "memory.h"
 #include "log.h"
+#include "jhash.h"
 
 DEFINE_MTYPE_STATIC(LIB, PREFIX, "Prefix")
 
 /* Maskbit. */
-static const u_char maskbit[] = {0x00, 0x80, 0xc0, 0xe0, 0xf0,
-				 0xf8, 0xfc, 0xfe, 0xff};
+static const uint8_t maskbit[] = {0x00, 0x80, 0xc0, 0xe0, 0xf0,
+				  0xf8, 0xfc, 0xfe, 0xff};
 
 static const struct in6_addr maskbytes6[] = {
 	/* /0   */ {{{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -429,7 +429,19 @@ static const struct in6_addr maskbytes6[] = {
 
 #define MASKBIT(offset)  ((0xff << (PNBBY - (offset))) & 0xff)
 
-unsigned int prefix_bit(const u_char *prefix, const u_char prefixlen)
+int is_zero_mac(struct ethaddr *mac)
+{
+	int i = 0;
+
+	for (i = 0; i < ETH_ALEN; i++) {
+		if (mac->octet[i])
+			return 0;
+	}
+
+	return 1;
+}
+
+unsigned int prefix_bit(const uint8_t *prefix, const uint8_t prefixlen)
 {
 	unsigned int offset = prefixlen / 8;
 	unsigned int shift = 7 - (prefixlen % 8);
@@ -437,9 +449,9 @@ unsigned int prefix_bit(const u_char *prefix, const u_char prefixlen)
 	return (prefix[offset] >> shift) & 1;
 }
 
-unsigned int prefix6_bit(const struct in6_addr *prefix, const u_char prefixlen)
+unsigned int prefix6_bit(const struct in6_addr *prefix, const uint8_t prefixlen)
 {
-	return prefix_bit((const u_char *)&prefix->s6_addr, prefixlen);
+	return prefix_bit((const uint8_t *)&prefix->s6_addr, prefixlen);
 }
 
 int str2family(const char *string)
@@ -450,9 +462,26 @@ int str2family(const char *string)
 		return AF_INET6;
 	else if (!strcmp("ethernet", string))
 		return AF_ETHERNET;
+	else if (!strcmp("evpn", string))
+		return AF_EVPN;
 	return -1;
 }
 
+const char *family2str(int family)
+{
+	switch (family) {
+	case AF_INET:
+		return "IPv4";
+	case AF_INET6:
+		return "IPv6";
+	case AF_ETHERNET:
+		return "Ethernet";
+	case AF_EVPN:
+		return "Evpn";
+	}
+	return "?";
+}
+
 /* Address Famiy Identifier to Address Family converter. */
 int afi2family(afi_t afi)
 {
@@ -462,6 +491,7 @@ int afi2family(afi_t afi)
 		return AF_INET6;
 	else if (afi == AFI_L2VPN)
 		return AF_ETHERNET;
+	/* NOTE: EVPN code should NOT use this interface. */
 	return 0;
 }
 
@@ -471,7 +501,7 @@ afi_t family2afi(int family)
 		return AFI_IP;
 	else if (family == AF_INET6)
 		return AFI_IP6;
-	else if (family == AF_ETHERNET)
+	else if (family == AF_ETHERNET || family == AF_EVPN)
 		return AFI_L2VPN;
 	return 0;
 }
@@ -500,14 +530,19 @@ const char *safi2str(safi_t safi)
 		return "unicast";
 	case SAFI_MULTICAST:
 		return "multicast";
-	case SAFI_ENCAP:
-		return "encap";
 	case SAFI_MPLS_VPN:
 		return "vpn";
+	case SAFI_ENCAP:
+		return "encap";
 	case SAFI_EVPN:
 		return "evpn";
+	case SAFI_LABELED_UNICAST:
+		return "labeled-unicast";
+	case SAFI_FLOWSPEC:
+		return "flowspec";
+	default:
+		return "unknown";
 	}
-	return NULL;
 }
 
 /* If n includes p prefix then return 1 else return 0. */
@@ -515,15 +550,58 @@ int prefix_match(const struct prefix *n, const struct prefix *p)
 {
 	int offset;
 	int shift;
-	const u_char *np, *pp;
+	const uint8_t *np, *pp;
 
 	/* If n's prefix is longer than p's one return 0. */
 	if (n->prefixlen > p->prefixlen)
 		return 0;
 
+	if (n->family == AF_FLOWSPEC) {
+		/* prefixlen is unused. look at fs prefix len */
+		if (n->u.prefix_flowspec.prefixlen >
+		    p->u.prefix_flowspec.prefixlen)
+			return 0;
+
+		/* Set both prefix's head pointer. */
+		np = (const uint8_t *)&n->u.prefix_flowspec.ptr;
+		pp = (const uint8_t *)&p->u.prefix_flowspec.ptr;
+
+		offset = n->u.prefix_flowspec.prefixlen;
+
+		while (offset--)
+			if (np[offset] != pp[offset])
+				return 0;
+		return 1;
+	}
+
 	/* Set both prefix's head pointer. */
-	np = (const u_char *)&n->u.prefix;
-	pp = (const u_char *)&p->u.prefix;
+	np = (const uint8_t *)&n->u.prefix;
+	pp = (const uint8_t *)&p->u.prefix;
+
+	offset = n->prefixlen / PNBBY;
+	shift = n->prefixlen % PNBBY;
+
+	if (shift)
+		if (maskbit[shift] & (np[offset] ^ pp[offset]))
+			return 0;
+
+	while (offset--)
+		if (np[offset] != pp[offset])
+			return 0;
+	return 1;
+}
+
+/* If n includes p then return 1 else return 0. Prefix mask is not considered */
+int prefix_match_network_statement(const struct prefix *n,
+				   const struct prefix *p)
+{
+	int offset;
+	int shift;
+	const uint8_t *np, *pp;
+
+	/* Set both prefix's head pointer. */
+	np = (const uint8_t *)&n->u.prefix;
+	pp = (const uint8_t *)&p->u.prefix;
 
 	offset = n->prefixlen / PNBBY;
 	shift = n->prefixlen % PNBBY;
@@ -538,7 +616,6 @@ int prefix_match(const struct prefix *n, const struct prefix *p)
 	return 1;
 }
 
-/* Copy prefix from src to dest. */
 void prefix_copy(struct prefix *dest, const struct prefix *src)
 {
 	dest->family = src->family;
@@ -549,11 +626,26 @@ void prefix_copy(struct prefix *dest, const struct prefix *src)
 	else if (src->family == AF_INET6)
 		dest->u.prefix6 = src->u.prefix6;
 	else if (src->family == AF_ETHERNET) {
+		memcpy(&dest->u.prefix_eth, &src->u.prefix_eth,
+		       sizeof(struct ethaddr));
+	} else if (src->family == AF_EVPN) {
 		memcpy(&dest->u.prefix_evpn, &src->u.prefix_evpn,
 		       sizeof(struct evpn_addr));
 	} else if (src->family == AF_UNSPEC) {
 		dest->u.lp.id = src->u.lp.id;
 		dest->u.lp.adv_router = src->u.lp.adv_router;
+	} else if (src->family == AF_FLOWSPEC) {
+		void *temp;
+		int len;
+
+		len = src->u.prefix_flowspec.prefixlen;
+		dest->u.prefix_flowspec.prefixlen =
+			src->u.prefix_flowspec.prefixlen;
+		dest->family = src->family;
+		temp = XCALLOC(MTYPE_PREFIX_FLOWSPEC, len);
+		dest->u.prefix_flowspec.ptr = (uintptr_t)temp;
+		memcpy((void *)dest->u.prefix_flowspec.ptr,
+		       (void *)src->u.prefix_flowspec.ptr, len);
 	} else {
 		zlog_err("prefix_copy(): Unknown address family %d",
 			 src->family);
@@ -579,17 +671,29 @@ int prefix_same(const struct prefix *p1, const struct prefix *p2)
 
 	if (p1->family == p2->family && p1->prefixlen == p2->prefixlen) {
 		if (p1->family == AF_INET)
-			if (IPV4_ADDR_SAME(&p1->u.prefix4.s_addr,
-					   &p2->u.prefix4.s_addr))
+			if (IPV4_ADDR_SAME(&p1->u.prefix4, &p2->u.prefix4))
 				return 1;
 		if (p1->family == AF_INET6)
 			if (IPV6_ADDR_SAME(&p1->u.prefix6.s6_addr,
 					   &p2->u.prefix6.s6_addr))
 				return 1;
 		if (p1->family == AF_ETHERNET)
+			if (!memcmp(&p1->u.prefix_eth, &p2->u.prefix_eth,
+				    sizeof(struct ethaddr)))
+				return 1;
+		if (p1->family == AF_EVPN)
 			if (!memcmp(&p1->u.prefix_evpn, &p2->u.prefix_evpn,
 				    sizeof(struct evpn_addr)))
 				return 1;
+		if (p1->family == AF_FLOWSPEC) {
+			if (p1->u.prefix_flowspec.prefixlen !=
+			    p2->u.prefix_flowspec.prefixlen)
+				return 0;
+			if (!memcmp(&p1->u.prefix_flowspec.ptr,
+				    &p2->u.prefix_flowspec.ptr,
+				    p2->u.prefix_flowspec.prefixlen))
+				return 1;
+		}
 	}
 	return 0;
 }
@@ -610,12 +714,30 @@ int prefix_cmp(const struct prefix *p1, const struct prefix *p2)
 	int shift;
 
 	/* Set both prefix's head pointer. */
-	const u_char *pp1 = (const u_char *)&p1->u.prefix;
-	const u_char *pp2 = (const u_char *)&p2->u.prefix;
+	const uint8_t *pp1;
+	const uint8_t *pp2;
 
-	if (p1->family != p2->family || p1->prefixlen != p2->prefixlen)
+	if (p1->family != p2->family)
 		return 1;
+	if (p1->family == AF_FLOWSPEC) {
+		pp1 = (const uint8_t *)p1->u.prefix_flowspec.ptr;
+		pp2 = (const uint8_t *)p2->u.prefix_flowspec.ptr;
 
+		if (p1->u.prefix_flowspec.prefixlen !=
+		    p2->u.prefix_flowspec.prefixlen)
+			return 1;
+
+		offset = p1->u.prefix_flowspec.prefixlen;
+		while (offset--)
+			if (pp1[offset] != pp2[offset])
+				return 1;
+		return 0;
+	}
+	pp1 = (const uint8_t *)&p1->u.prefix;
+	pp2 = (const uint8_t *)&p2->u.prefix;
+
+	if (p1->prefixlen != p2->prefixlen)
+		return 1;
 	offset = p1->prefixlen / PNBBY;
 	shift = p1->prefixlen % PNBBY;
 
@@ -640,17 +762,19 @@ int prefix_common_bits(const struct prefix *p1, const struct prefix *p2)
 {
 	int pos, bit;
 	int length = 0;
-	u_char xor ;
+	uint8_t xor ;
 
 	/* Set both prefix's head pointer. */
-	const u_char *pp1 = (const u_char *)&p1->u.prefix;
-	const u_char *pp2 = (const u_char *)&p2->u.prefix;
+	const uint8_t *pp1 = (const uint8_t *)&p1->u.prefix;
+	const uint8_t *pp2 = (const uint8_t *)&p2->u.prefix;
 
 	if (p1->family == AF_INET)
 		length = IPV4_MAX_BYTELEN;
 	if (p1->family == AF_INET6)
 		length = IPV6_MAX_BYTELEN;
 	if (p1->family == AF_ETHERNET)
+		length = ETH_ALEN;
+	if (p1->family == AF_EVPN)
 		length = 8 * sizeof(struct evpn_addr);
 
 	if (p1->family != p2->family || !length)
@@ -679,6 +803,8 @@ const char *prefix_family_str(const struct prefix *p)
 		return "inet6";
 	if (p->family == AF_ETHERNET)
 		return "ether";
+	if (p->family == AF_EVPN)
+		return "evpn";
 	return "unspec";
 }
 
@@ -734,7 +860,7 @@ int str2prefix_ipv4(const char *str, struct prefix_ipv4 *p)
 		XFREE(MTYPE_TMP, cp);
 
 		/* Get prefix length. */
-		plen = (u_char)atoi(++pnt);
+		plen = (uint8_t)atoi(++pnt);
 		if (plen > IPV4_MAX_PREFIXLEN)
 			return 0;
 
@@ -755,13 +881,20 @@ int str2prefix_eth(const char *str, struct prefix_eth *p)
 	const char *str_addr = str;
 	unsigned int a[6];
 	int i;
+	bool slash = false;
+
+	if (!strcmp(str, "any")) {
+		memset(p, 0, sizeof(*p));
+		p->family = AF_ETHERNET;
+		return 1;
+	}
 
 	/* Find slash inside string. */
 	pnt = strchr(str, '/');
 
 	if (pnt) {
 		/* Get prefix length. */
-		plen = (u_char)atoi(++pnt);
+		plen = (uint8_t)atoi(++pnt);
 		if (plen > 48) {
 			ret = 0;
 			goto done;
@@ -772,6 +905,7 @@ int str2prefix_eth(const char *str, struct prefix_eth *p)
 		*(cp + (pnt - str)) = '\0';
 
 		str_addr = cp;
+		slash = true;
 	}
 
 	/* Convert string to prefix. */
@@ -786,6 +920,15 @@ int str2prefix_eth(const char *str, struct prefix_eth *p)
 	}
 	p->prefixlen = plen;
 	p->family = AF_ETHERNET;
+
+	/*
+	 * special case to allow old configurations to work
+	 * Since all zero's is implicitly meant to allow
+	 * a comparison to zero, let's assume
+	 */
+	if (!slash && is_zero_mac(&(p->eth_addr)))
+		p->prefixlen = 0;
+
 	ret = 1;
 
 done:
@@ -813,7 +956,7 @@ void masklen2ip(const int masklen, struct in_addr *netmask)
 
 /* Convert IP address's netmask into integer. We assume netmask is
    sequential one. Argument netmask should be network byte order. */
-u_char ip_masklen(struct in_addr netmask)
+uint8_t ip_masklen(struct in_addr netmask)
 {
 	uint32_t tmp = ~ntohl(netmask.s_addr);
 	if (tmp)
@@ -883,7 +1026,7 @@ int str2prefix_ipv6(const char *str, struct prefix_ipv6 *p)
 		XFREE(MTYPE_TMP, cp);
 		if (ret == 0)
 			return 0;
-		plen = (u_char)atoi(++pnt);
+		plen = (uint8_t)atoi(++pnt);
 		if (plen > IPV6_MAX_BITLEN)
 			return 0;
 		p->prefixlen = plen;
@@ -894,7 +1037,7 @@ int str2prefix_ipv6(const char *str, struct prefix_ipv6 *p)
 }
 
 /* Convert struct in6_addr netmask into integer.
- * FIXME return u_char as ip_maskleni() does. */
+ * FIXME return uint8_t as ip_maskleni() does. */
 int ip6_masklen(struct in6_addr netmask)
 {
 	int len = 0;
@@ -926,14 +1069,14 @@ void masklen2ip6(const int masklen, struct in6_addr *netmask)
 
 void apply_mask_ipv6(struct prefix_ipv6 *p)
 {
-	u_char *pnt;
+	uint8_t *pnt;
 	int index;
 	int offset;
 
 	index = p->prefixlen / 8;
 
 	if (index < 16) {
-		pnt = (u_char *)&p->prefix;
+		pnt = (uint8_t *)&p->prefix;
 		offset = p->prefixlen % 8;
 
 		pnt[index] &= maskbit[offset];
@@ -1034,7 +1177,8 @@ int prefix_blen(const struct prefix *p)
 		return IPV6_MAX_BYTELEN;
 		break;
 	case AF_ETHERNET:
-		return ETHER_ADDR_LEN;
+		return ETH_ALEN;
+		break;
 	}
 	return 0;
 }
@@ -1062,6 +1206,57 @@ int str2prefix(const char *str, struct prefix *p)
 	return 0;
 }
 
+static const char *prefixevpn2str(const struct prefix *p, char *str, int size)
+{
+	uint8_t family;
+	char buf[PREFIX2STR_BUFFER];
+	char buf2[ETHER_ADDR_STRLEN];
+
+	if (p->u.prefix_evpn.route_type == 2) {
+		if (IS_EVPN_PREFIX_IPADDR_NONE((struct prefix_evpn *)p))
+			snprintf(str, size, "[%d]:[%s]/%d",
+				 p->u.prefix_evpn.route_type,
+				 prefix_mac2str(&p->u.prefix_evpn.mac, buf2,
+						sizeof(buf2)),
+				 p->prefixlen);
+		else {
+			family = IS_EVPN_PREFIX_IPADDR_V4(
+					 (struct prefix_evpn *)p)
+					 ? AF_INET
+					 : AF_INET6;
+			snprintf(str, size, "[%d]:[%s]:[%s]/%d",
+				 p->u.prefix_evpn.route_type,
+				 prefix_mac2str(&p->u.prefix_evpn.mac, buf2,
+						sizeof(buf2)),
+				 inet_ntop(family, &p->u.prefix_evpn.ip.ip.addr,
+					   buf, PREFIX2STR_BUFFER),
+				 p->prefixlen);
+		}
+	} else if (p->u.prefix_evpn.route_type == 3) {
+		family = IS_EVPN_PREFIX_IPADDR_V4((struct prefix_evpn *)p)
+				 ? AF_INET
+				 : AF_INET6;
+		snprintf(str, size, "[%d]:[%s]/%d", p->u.prefix_evpn.route_type,
+			 inet_ntop(family, &p->u.prefix_evpn.ip.ip.addr, buf,
+				   PREFIX2STR_BUFFER),
+			 p->prefixlen);
+	} else if (p->u.prefix_evpn.route_type == 5) {
+		family = IS_EVPN_PREFIX_IPADDR_V4((struct prefix_evpn *)p)
+				 ? AF_INET
+				 : AF_INET6;
+		snprintf(str, size, "[%d]:[%u][%s/%d]/%d",
+			 p->u.prefix_evpn.route_type, p->u.prefix_evpn.eth_tag,
+			 inet_ntop(family, &p->u.prefix_evpn.ip.ip.addr, buf,
+				   PREFIX2STR_BUFFER),
+			 p->u.prefix_evpn.ip_prefix_length, p->prefixlen);
+	} else {
+		sprintf(str, "Unsupported EVPN route type %d",
+			p->u.prefix_evpn.route_type);
+	}
+
+	return str;
+}
+
 const char *prefix2str(union prefixconstptr pu, char *str, int size)
 {
 	const struct prefix *p = pu.p;
@@ -1076,29 +1271,19 @@ const char *prefix2str(union prefixconstptr pu, char *str, int size)
 		break;
 
 	case AF_ETHERNET:
-		if (p->u.prefix_evpn.route_type == 5) {
-			u_char family;
-			family = (p->u.prefix_evpn.flags
-				  & (IP_ADDR_V4 | IP_PREFIX_V4))
-					 ? AF_INET
-					 : AF_INET6;
-			snprintf(str, size, "[%d]:[%u][%s]/%d",
-				 p->u.prefix_evpn.route_type,
-				 p->u.prefix_evpn.eth_tag,
-				 inet_ntop(family, &p->u.prefix_evpn.ip.addr,
-					   buf, PREFIX2STR_BUFFER),
-				 p->prefixlen);
-		} else {
-			sprintf(str, "UNK AF_ETHER prefix");
-			snprintf(str, size, "%02x:%02x:%02x:%02x:%02x:%02x/%d",
-				 p->u.prefix_eth.octet[0],
-				 p->u.prefix_eth.octet[1],
-				 p->u.prefix_eth.octet[2],
-				 p->u.prefix_eth.octet[3],
-				 p->u.prefix_eth.octet[4],
-				 p->u.prefix_eth.octet[5], p->prefixlen);
-		}
+		snprintf(str, size, "%s/%d",
+			 prefix_mac2str(&p->u.prefix_eth, buf, sizeof(buf)),
+			 p->prefixlen);
 		break;
+
+	case AF_EVPN:
+		prefixevpn2str(p, str, size);
+		break;
+
+	case AF_FLOWSPEC:
+		sprintf(str, "FS prefix");
+		break;
+
 	default:
 		sprintf(str, "UNK prefix");
 		break;
@@ -1136,7 +1321,7 @@ int all_digit(const char *str)
 void apply_classful_mask_ipv4(struct prefix_ipv4 *p)
 {
 
-	u_int32_t destination;
+	uint32_t destination;
 
 	destination = ntohl(p->prefix.s_addr);
 
@@ -1184,8 +1369,8 @@ int netmask_str2prefix_str(const char *net_str, const char *mask_str,
 {
 	struct in_addr network;
 	struct in_addr mask;
-	u_char prefixlen;
-	u_int32_t destination;
+	uint8_t prefixlen;
+	uint32_t destination;
 	int ret;
 
 	ret = inet_aton(net_str, &network);
@@ -1273,3 +1458,35 @@ char *prefix_mac2str(const struct ethaddr *mac, char *buf, int size)
 		 (uint8_t)mac->octet[4], (uint8_t)mac->octet[5]);
 	return ptr;
 }
+
+unsigned prefix_hash_key(void *pp)
+{
+	struct prefix copy;
+
+	if (((struct prefix *)pp)->family == AF_FLOWSPEC) {
+		uint32_t len;
+		void *temp;
+
+		/* make sure *all* unused bits are zero,
+		 * particularly including alignment /
+		 * padding and unused prefix bytes.
+		 */
+		memset(&copy, 0, sizeof(copy));
+		prefix_copy(&copy, (struct prefix *)pp);
+		len = jhash((void *)copy.u.prefix_flowspec.ptr,
+			    copy.u.prefix_flowspec.prefixlen,
+			    0x55aa5a5a);
+		temp = (void *)copy.u.prefix_flowspec.ptr;
+		XFREE(MTYPE_PREFIX_FLOWSPEC, temp);
+		copy.u.prefix_flowspec.ptr = (uintptr_t)NULL;
+		return len;
+	}
+	/* make sure *all* unused bits are zero, particularly including
+	 * alignment /
+	 * padding and unused prefix bytes. */
+	memset(&copy, 0, sizeof(copy));
+	prefix_copy(&copy, (struct prefix *)pp);
+	return jhash(&copy,
+		     offsetof(struct prefix, u.prefix) + PSIZE(copy.prefixlen),
+		     0x55aa5a5a);
+}
