@@ -1,22 +1,22 @@
 /* Route map function.
-   Copyright (C) 1998, 1999 Kunihiro Ishiguro
-
-This file is part of GNU Zebra.
-
-GNU Zebra is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
-
-GNU Zebra is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with GNU Zebra; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.  */
+ * Copyright (C) 1998, 1999 Kunihiro Ishiguro
+ *
+ * This file is part of GNU Zebra.
+ *
+ * GNU Zebra is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation; either version 2, or (at your option) any
+ * later version.
+ *
+ * GNU Zebra is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -28,8 +28,8 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #include "routemap.h"
 #include "command.h"
 #include "log.h"
-#include "log_int.h"
 #include "hash.h"
+#include "libfrr.h"
 
 DEFINE_MTYPE_STATIC(LIB, ROUTE_MAP, "Route map")
 DEFINE_MTYPE(LIB, ROUTE_MAP_NAME, "Route map name")
@@ -416,21 +416,24 @@ int generic_match_add(struct vty *vty, struct route_map_index *index,
 	int ret;
 
 	ret = route_map_add_match(index, command, arg);
-	if (ret) {
-		switch (ret) {
-		case RMAP_RULE_MISSING:
-			vty_out(vty, "%% Can't find rule.%s", VTY_NEWLINE);
-			return CMD_WARNING;
-		case RMAP_COMPILE_ERROR:
-			vty_out(vty, "%% Argument is malformed.%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
+	switch (ret) {
+	case RMAP_COMPILE_SUCCESS:
+		if (type != RMAP_EVENT_MATCH_ADDED) {
+			route_map_upd8_dependency(type, arg, index->map->name);
 		}
+		break;
+	case RMAP_RULE_MISSING:
+		vty_out(vty, "%% [%s] Can't find rule.\n", frr_protonameinst);
+		return CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_ERROR:
+		vty_out(vty,
+			"%% [%s] Argument form is unsupported or malformed.\n",
+			frr_protonameinst);
+		return CMD_WARNING_CONFIG_FAILED;
+		break;
 	}
 
-	if (type != RMAP_EVENT_MATCH_ADDED) {
-		route_map_upd8_dependency(type, arg, index->map->name);
-	}
 	return CMD_SUCCESS;
 }
 
@@ -439,6 +442,7 @@ int generic_match_delete(struct vty *vty, struct route_map_index *index,
 			 route_map_event_t type)
 {
 	int ret;
+	int retval = CMD_SUCCESS;
 	char *dep_name = NULL;
 	const char *tmpstr;
 	char *rmap_name = NULL;
@@ -457,32 +461,29 @@ int generic_match_delete(struct vty *vty, struct route_map_index *index,
 	}
 
 	ret = route_map_delete_match(index, command, dep_name);
-	if (ret) {
-		switch (ret) {
-		case RMAP_RULE_MISSING:
-			vty_out(vty, "%% BGP Can't find rule.%s", VTY_NEWLINE);
-			break;
-		case RMAP_COMPILE_ERROR:
-			vty_out(vty, "%% BGP Argument is malformed.%s",
-				VTY_NEWLINE);
-			break;
-		}
-		if (dep_name)
-			XFREE(MTYPE_ROUTE_MAP_RULE, dep_name);
-		if (rmap_name)
-			XFREE(MTYPE_ROUTE_MAP_NAME, rmap_name);
-		return CMD_WARNING;
+	switch (ret) {
+	case RMAP_RULE_MISSING:
+		vty_out(vty, "%% [%s] Can't find rule.\n", frr_protonameinst);
+		retval = CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_ERROR:
+		vty_out(vty,
+			"%% [%s] Argument form is unsupported or malformed.\n",
+			frr_protonameinst);
+		retval = CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_SUCCESS:
+		if (type != RMAP_EVENT_MATCH_DELETED && dep_name)
+			route_map_upd8_dependency(type, dep_name, rmap_name);
+		break;
 	}
 
-	if (type != RMAP_EVENT_MATCH_DELETED && dep_name)
-		route_map_upd8_dependency(type, dep_name, rmap_name);
-
 	if (dep_name)
 		XFREE(MTYPE_ROUTE_MAP_RULE, dep_name);
 	if (rmap_name)
 		XFREE(MTYPE_ROUTE_MAP_NAME, rmap_name);
 
-	return CMD_SUCCESS;
+	return retval;
 }
 
 int generic_set_add(struct vty *vty, struct route_map_index *index,
@@ -491,17 +492,21 @@ int generic_set_add(struct vty *vty, struct route_map_index *index,
 	int ret;
 
 	ret = route_map_add_set(index, command, arg);
-	if (ret) {
-		switch (ret) {
-		case RMAP_RULE_MISSING:
-			vty_out(vty, "%% Can't find rule.%s", VTY_NEWLINE);
-			return CMD_WARNING;
-		case RMAP_COMPILE_ERROR:
-			vty_out(vty, "%% Argument is malformed.%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
-		}
+	switch (ret) {
+	case RMAP_RULE_MISSING:
+		vty_out(vty, "%% [%s] Can't find rule.\n", frr_protonameinst);
+		return CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_ERROR:
+		vty_out(vty,
+			"%% [%s] Argument form is unsupported or malformed.\n",
+			frr_protonameinst);
+		return CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_SUCCESS:
+		break;
 	}
+
 	return CMD_SUCCESS;
 }
 
@@ -511,17 +516,21 @@ int generic_set_delete(struct vty *vty, struct route_map_index *index,
 	int ret;
 
 	ret = route_map_delete_set(index, command, arg);
-	if (ret) {
-		switch (ret) {
-		case RMAP_RULE_MISSING:
-			vty_out(vty, "%% Can't find rule.%s", VTY_NEWLINE);
-			return CMD_WARNING;
-		case RMAP_COMPILE_ERROR:
-			vty_out(vty, "%% Argument is malformed.%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
-		}
+	switch (ret) {
+	case RMAP_RULE_MISSING:
+		vty_out(vty, "%% [%s] Can't find rule.\n", frr_protonameinst);
+		return CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_ERROR:
+		vty_out(vty,
+			"%% [%s] Argument form is unsupported or malformed.\n",
+			frr_protonameinst);
+		return CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_SUCCESS:
+		break;
 	}
+
 	return CMD_SUCCESS;
 }
 
@@ -713,7 +722,7 @@ static void route_map_delete(struct route_map *map)
 
 	/* Clear all dependencies */
 	route_map_clear_all_references(name);
-	map->deleted = 1;
+	map->deleted = true;
 	/* Execute deletion hook. */
 	if (route_map_master.delete_hook) {
 		(*route_map_master.delete_hook)(name);
@@ -753,19 +762,19 @@ int route_map_mark_updated(const char *name, int del_later)
 
 	map = route_map_lookup_by_name(name);
 
-	/* If we did not find the routemap with deleted=0 try again
-	 * with deleted=1
+	/* If we did not find the routemap with deleted=false try again
+	 * with deleted=true
 	 */
 	if (!map) {
 		memset(&tmp_map, 0, sizeof(struct route_map));
 		tmp_map.name = XSTRDUP(MTYPE_ROUTE_MAP_NAME, name);
-		tmp_map.deleted = 1;
+		tmp_map.deleted = true;
 		map = hash_lookup(route_map_master_hash, &tmp_map);
 		XFREE(MTYPE_ROUTE_MAP_NAME, tmp_map.name);
 	}
 
 	if (map) {
-		map->to_be_processed = 1;
+		map->to_be_processed = true;
 		ret = 0;
 	}
 
@@ -777,7 +786,7 @@ int route_map_clear_updated(struct route_map *map)
 	int ret = -1;
 
 	if (map) {
-		map->to_be_processed = 0;
+		map->to_be_processed = false;
 		if (map->deleted)
 			route_map_free_map(map);
 	}
@@ -844,51 +853,41 @@ static void vty_show_route_map_entry(struct vty *vty, struct route_map *map)
 	struct route_map_index *index;
 	struct route_map_rule *rule;
 
-	/* Print the name of the protocol */
-	if (zlog_default) {
-		vty_out(vty, "%s", zlog_protoname());
-		if (zlog_default->instance)
-			vty_out(vty, " %d", zlog_default->instance);
-	}
-	vty_out(vty, ":%s", VTY_NEWLINE);
+	vty_out(vty, "%s:\n", frr_protonameinst);
 
 	for (index = map->head; index; index = index->next) {
-		vty_out(vty, "route-map %s, %s, sequence %d%s", map->name,
-			route_map_type_str(index->type), index->pref,
-			VTY_NEWLINE);
+		vty_out(vty, "route-map %s, %s, sequence %d\n", map->name,
+			route_map_type_str(index->type), index->pref);
 
 		/* Description */
 		if (index->description)
-			vty_out(vty, "  Description:%s    %s%s", VTY_NEWLINE,
-				index->description, VTY_NEWLINE);
+			vty_out(vty, "  Description:\n    %s\n",
+				index->description);
 
 		/* Match clauses */
-		vty_out(vty, "  Match clauses:%s", VTY_NEWLINE);
+		vty_out(vty, "  Match clauses:\n");
 		for (rule = index->match_list.head; rule; rule = rule->next)
-			vty_out(vty, "    %s %s%s", rule->cmd->str,
-				rule->rule_str, VTY_NEWLINE);
+			vty_out(vty, "    %s %s\n", rule->cmd->str,
+				rule->rule_str);
 
-		vty_out(vty, "  Set clauses:%s", VTY_NEWLINE);
+		vty_out(vty, "  Set clauses:\n");
 		for (rule = index->set_list.head; rule; rule = rule->next)
-			vty_out(vty, "    %s %s%s", rule->cmd->str,
-				rule->rule_str, VTY_NEWLINE);
+			vty_out(vty, "    %s %s\n", rule->cmd->str,
+				rule->rule_str);
 
 		/* Call clause */
-		vty_out(vty, "  Call clause:%s", VTY_NEWLINE);
+		vty_out(vty, "  Call clause:\n");
 		if (index->nextrm)
-			vty_out(vty, "    Call %s%s", index->nextrm,
-				VTY_NEWLINE);
+			vty_out(vty, "    Call %s\n", index->nextrm);
 
 		/* Exit Policy */
-		vty_out(vty, "  Action:%s", VTY_NEWLINE);
+		vty_out(vty, "  Action:\n");
 		if (index->exitpolicy == RMAP_GOTO)
-			vty_out(vty, "    Goto %d%s", index->nextpref,
-				VTY_NEWLINE);
+			vty_out(vty, "    Goto %d\n", index->nextpref);
 		else if (index->exitpolicy == RMAP_NEXT)
-			vty_out(vty, "    Continue to next entry%s",
-				VTY_NEWLINE);
+			vty_out(vty, "    Continue to next entry\n");
 		else if (index->exitpolicy == RMAP_EXIT)
-			vty_out(vty, "    Exit routemap%s", VTY_NEWLINE);
+			vty_out(vty, "    Exit routemap\n");
 	}
 }
 
@@ -903,11 +902,8 @@ static int vty_show_route_map(struct vty *vty, const char *name)
 			vty_show_route_map_entry(vty, map);
 			return CMD_SUCCESS;
 		} else {
-			vty_out(vty, "%s", zlog_protoname());
-			if (zlog_default && zlog_default->instance)
-				vty_out(vty, " %d", zlog_default->instance);
-			vty_out(vty, ": 'route-map %s' not found%s", name,
-				VTY_NEWLINE);
+			vty_out(vty, "%s: 'route-map %s' not found\n",
+				frr_protonameinst, name);
 			return CMD_SUCCESS;
 		}
 	} else {
@@ -1222,7 +1218,7 @@ int route_map_add_match(struct route_map_index *index, const char *match_name,
 					      RMAP_EVENT_CALL_ADDED);
 	}
 
-	return 0;
+	return RMAP_COMPILE_SUCCESS;
 }
 
 /* Delete specified route match rule. */
@@ -1309,7 +1305,7 @@ int route_map_add_set(struct route_map_index *index, const char *set_name,
 		route_map_notify_dependencies(index->map->name,
 					      RMAP_EVENT_CALL_ADDED);
 	}
-	return 0;
+	return RMAP_COMPILE_SUCCESS;
 }
 
 /* Delete route map set rule. */
@@ -1581,8 +1577,9 @@ static void *route_map_dep_hash_alloc(void *p)
 
 	dep_entry = XCALLOC(MTYPE_ROUTE_MAP_DEP, sizeof(struct route_map_dep));
 	dep_entry->dep_name = XSTRDUP(MTYPE_ROUTE_MAP_NAME, dep_name);
-	dep_entry->dep_rmap_hash = hash_create(route_map_dep_hash_make_key,
-					       route_map_rmap_hash_cmp);
+	dep_entry->dep_rmap_hash =
+		hash_create_size(8, route_map_dep_hash_make_key,
+				 route_map_rmap_hash_cmp, "Route Map Dep Hash");
 	dep_entry->this_hash = NULL;
 
 	return ((void *)dep_entry);
@@ -1800,7 +1797,7 @@ DEFUN (match_interface,
 
 DEFUN (no_match_interface,
        no_match_interface_cmd,
-       "no match interface [INTERFACE]",
+       "no match interface [WORD]",
        NO_STR
        MATCH_STR
        "Match first hop interface of route\n"
@@ -2180,16 +2177,14 @@ DEFUN (set_ip_nexthop,
 
 	ret = str2sockunion(argv[idx_ipv4]->arg, &su);
 	if (ret < 0) {
-		vty_out(vty, "%% Malformed nexthop address%s", VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% Malformed nexthop address\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	if (su.sin.sin_addr.s_addr == 0
 	    || IPV4_CLASS_DE(ntohl(su.sin.sin_addr.s_addr))) {
 		vty_out(vty,
-			"%% nexthop address cannot be 0.0.0.0, multicast "
-			"or reserved%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+			"%% nexthop address cannot be 0.0.0.0, multicast or reserved\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (rmap_match_set_hook.set_ip_nexthop)
@@ -2201,24 +2196,24 @@ DEFUN (set_ip_nexthop,
 
 DEFUN (no_set_ip_nexthop,
        no_set_ip_nexthop_cmd,
-       "no set ip next-hop [<peer-address|A.B.C.D>]",
+       "no set ip next-hop [A.B.C.D]",
        NO_STR
        SET_STR
        IP_STR
        "Next hop address\n"
-       "Use peer address (for BGP only)\n"
        "IP address of next hop\n")
 {
-	int idx_peer = 4;
+	int idx = 0;
 	VTY_DECLVAR_CONTEXT(route_map_index, index);
+	const char *arg = NULL;
 
-	if (rmap_match_set_hook.no_set_ip_nexthop) {
-		if (argc <= idx_peer)
-			return rmap_match_set_hook.no_set_ip_nexthop(
-				vty, index, "ip next-hop", NULL);
+	if (argv_find(argv, argc, "A.B.C.D", &idx))
+		arg = argv[idx]->arg;
+
+	if (rmap_match_set_hook.no_set_ip_nexthop)
 		return rmap_match_set_hook.no_set_ip_nexthop(
-			vty, index, "ip next-hop", argv[idx_peer]->arg);
-	}
+			vty, index, "ip next-hop", arg);
+
 	return CMD_SUCCESS;
 }
 
@@ -2239,13 +2234,12 @@ DEFUN (set_ipv6_nexthop_local,
 
 	ret = inet_pton(AF_INET6, argv[idx_ipv6]->arg, &addr);
 	if (!ret) {
-		vty_out(vty, "%% Malformed nexthop address%s", VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% Malformed nexthop address\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	if (!IN6_IS_ADDR_LINKLOCAL(&addr)) {
-		vty_out(vty, "%% Invalid link-local nexthop address%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% Invalid link-local nexthop address\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (rmap_match_set_hook.set_ipv6_nexthop_local)
@@ -2293,9 +2287,13 @@ DEFUN (set_metric,
 	int idx_number = 2;
 	VTY_DECLVAR_CONTEXT(route_map_index, index);
 
+	const char *pass = (argv[idx_number]->type == RANGE_TKN)
+				   ? argv[idx_number]->arg
+				   : argv[idx_number]->text;
+
 	if (rmap_match_set_hook.set_metric)
 		return rmap_match_set_hook.set_metric(vty, index, "metric",
-						      argv[idx_number]->arg);
+						      pass);
 	return CMD_SUCCESS;
 }
 
@@ -2402,9 +2400,8 @@ DEFUN (no_route_map_all,
 
 	map = route_map_lookup_by_name(mapname);
 	if (map == NULL) {
-		vty_out(vty, "%% Could not find route-map %s%s", mapname,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% Could not find route-map %s\n", mapname);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	route_map_delete(map);
@@ -2428,8 +2425,9 @@ DEFUN (no_route_map,
 	struct route_map *map;
 	struct route_map_index *index;
 	char *endptr = NULL;
-	int permit =
-		argv[idx_permit_deny]->arg[0] == 'p' ? RMAP_PERMIT : RMAP_DENY;
+	int permit = strmatch(argv[idx_permit_deny]->text, "permit")
+			     ? RMAP_PERMIT
+			     : RMAP_DENY;
 	const char *prefstr = argv[idx_number]->arg;
 	const char *mapname = argv[idx_word]->arg;
 	unsigned long pref = strtoul(prefstr, &endptr, 10);
@@ -2437,17 +2435,16 @@ DEFUN (no_route_map,
 	/* Existence check. */
 	map = route_map_lookup_by_name(mapname);
 	if (map == NULL) {
-		vty_out(vty, "%% Could not find route-map %s%s", mapname,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% Could not find route-map %s\n", mapname);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	/* Lookup route map index. */
 	index = route_map_index_lookup(map, permit, pref);
 	if (index == NULL) {
-		vty_out(vty, "%% Could not find route-map entry %s %s%s",
-			mapname, prefstr, VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% Could not find route-map entry %s %s\n",
+			mapname, prefstr);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	/* Delete index from route map. */
@@ -2473,9 +2470,8 @@ DEFUN (rmap_onmatch_next,
 			/* Under a deny clause, match means it's finished. No
 			 * need to set next */
 			vty_out(vty,
-				"on-match next not supported under route-map deny%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
+				"on-match next not supported under route-map deny\n");
+			return CMD_WARNING_CONFIG_FAILED;
 		}
 		index->exitpolicy = RMAP_NEXT;
 	}
@@ -2516,22 +2512,19 @@ DEFUN (rmap_onmatch_goto,
 			/* Under a deny clause, match means it's finished. No
 			 * need to go anywhere */
 			vty_out(vty,
-				"on-match goto not supported under route-map deny%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
+				"on-match goto not supported under route-map deny\n");
+			return CMD_WARNING_CONFIG_FAILED;
 		}
 
 		if (num)
-			VTY_GET_INTEGER_RANGE("route-map index", d, num, 1,
-					      65535);
+			d = strtoul(num, NULL, 10);
 		else
 			d = index->pref + 1;
 
 		if (d <= index->pref) {
 			/* Can't allow you to do that, Dave */
-			vty_out(vty, "can't jump backwards in route-maps%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
+			vty_out(vty, "can't jump backwards in route-maps\n");
+			return CMD_WARNING_CONFIG_FAILED;
 		} else {
 			index->exitpolicy = RMAP_GOTO;
 			index->nextpref = d;
@@ -2678,37 +2671,33 @@ static int route_map_config_write(struct vty *vty)
 	for (map = route_map_master.head; map; map = map->next)
 		for (index = map->head; index; index = index->next) {
 			if (!first)
-				vty_out(vty, "!%s", VTY_NEWLINE);
+				vty_out(vty, "!\n");
 			else
 				first = 0;
 
-			vty_out(vty, "route-map %s %s %d%s", map->name,
-				route_map_type_str(index->type), index->pref,
-				VTY_NEWLINE);
+			vty_out(vty, "route-map %s %s %d\n", map->name,
+				route_map_type_str(index->type), index->pref);
 
 			if (index->description)
-				vty_out(vty, " description %s%s",
-					index->description, VTY_NEWLINE);
+				vty_out(vty, " description %s\n",
+					index->description);
 
 			for (rule = index->match_list.head; rule;
 			     rule = rule->next)
-				vty_out(vty, " match %s %s%s", rule->cmd->str,
-					rule->rule_str ? rule->rule_str : "",
-					VTY_NEWLINE);
+				vty_out(vty, " match %s %s\n", rule->cmd->str,
+					rule->rule_str ? rule->rule_str : "");
 
 			for (rule = index->set_list.head; rule;
 			     rule = rule->next)
-				vty_out(vty, " set %s %s%s", rule->cmd->str,
-					rule->rule_str ? rule->rule_str : "",
-					VTY_NEWLINE);
+				vty_out(vty, " set %s %s\n", rule->cmd->str,
+					rule->rule_str ? rule->rule_str : "");
 			if (index->nextrm)
-				vty_out(vty, " call %s%s", index->nextrm,
-					VTY_NEWLINE);
+				vty_out(vty, " call %s\n", index->nextrm);
 			if (index->exitpolicy == RMAP_GOTO)
-				vty_out(vty, " on-match goto %d%s",
-					index->nextpref, VTY_NEWLINE);
+				vty_out(vty, " on-match goto %d\n",
+					index->nextpref);
 			if (index->exitpolicy == RMAP_NEXT)
-				vty_out(vty, " on-match next%s", VTY_NEWLINE);
+				vty_out(vty, " on-match next\n");
 
 			write++;
 		}
@@ -2754,7 +2743,7 @@ void route_map_finish(void)
 	/* cleanup route_map */
 	while (route_map_master.head) {
 		struct route_map *map = route_map_master.head;
-		map->to_be_processed = 0;
+		map->to_be_processed = false;
 		route_map_delete(map);
 	}
 
@@ -2767,6 +2756,22 @@ void route_map_finish(void)
 	route_map_master_hash = NULL;
 }
 
+static void rmap_autocomplete(vector comps, struct cmd_token *token)
+{
+	struct route_map *map;
+
+	for (map = route_map_master.head; map; map = map->next)
+		vector_set(comps, XSTRDUP(MTYPE_COMPLETION, map->name));
+}
+
+static const struct cmd_variable_handler rmap_var_handlers[] = {
+	{/* "route-map WORD" */
+	 .varname = "route_map",
+	 .completions = rmap_autocomplete},
+	{.tokenname = "ROUTEMAP_NAME", .completions = rmap_autocomplete},
+	{.tokenname = "RMAP_NAME", .completions = rmap_autocomplete},
+	{.completions = NULL}};
+
 /* Initialization of route map vector. */
 void route_map_init(void)
 {
@@ -2776,11 +2781,15 @@ void route_map_init(void)
 	route_match_vec = vector_init(1);
 	route_set_vec = vector_init(1);
 	route_map_master_hash =
-		hash_create(route_map_hash_key_make, route_map_hash_cmp);
+		hash_create_size(8, route_map_hash_key_make, route_map_hash_cmp,
+				 "Route Map Master Hash");
 
 	for (i = 1; i < ROUTE_MAP_DEP_MAX; i++)
-		route_map_dep_hash[i] = hash_create(route_map_dep_hash_make_key,
-						    route_map_dep_hash_cmp);
+		route_map_dep_hash[i] = hash_create_size(
+			8, route_map_dep_hash_make_key, route_map_dep_hash_cmp,
+			"Route Map Dep Hash");
+
+	cmd_variable_handler_register(rmap_var_handlers);
 
 	/* Install route map top node. */
 	install_node(&rmap_node, route_map_config_write);
