@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -57,7 +56,7 @@ void setsockopt_so_sendbuf(const int sock, int size)
 
 int getsockopt_so_sendbuf(const int sock)
 {
-	u_int32_t optval;
+	uint32_t optval;
 	socklen_t optlen = sizeof(optval);
 	int ret = getsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *)&optval,
 			     &optlen);
@@ -383,7 +382,7 @@ int setsockopt_ipv4_multicast_if(int sock, struct in_addr if_addr,
 #endif
 }
 
-int setsockopt_ipv4_multicast_loop(int sock, u_char val)
+int setsockopt_ipv4_multicast_loop(int sock, uint8_t val)
 {
 	int ret;
 
@@ -585,9 +584,9 @@ int sockopt_tcp_signature(int sock, union sockunion *su, const char *password)
 #define TCP_MD5_AUTH_ADD 1
 #define TCP_MD5_AUTH_DEL 2
 	struct tcp_rfc2385_cmd {
-		u_int8_t command;  /* Command - Add/Delete */
-		u_int32_t address; /* IPV4 address associated */
-		u_int8_t keylen;   /* MD5 Key len (do NOT assume 0 terminated
+		uint8_t command;  /* Command - Add/Delete */
+		uint32_t address; /* IPV4 address associated */
+		uint8_t keylen;   /* MD5 Key len (do NOT assume 0 terminated
 				      ascii) */
 		void *key;	 /* MD5 Key */
 	} cmd;
