@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -35,6 +34,7 @@
 #include "vty.h"
 #include "privs.h"
 #include "network.h"
+#include "libfrr.h"
 
 #include <arpa/telnet.h>
 #include <termios.h>
@@ -91,6 +91,24 @@ char integrate_default[] = SYSCONFDIR INTEGRATE_DEFAULT_CONFIG;
 
 static int do_log_commands = 0;
 
+void vty_frame(struct vty *vty, const char *format, ...)
+{
+	va_list args;
+
+	va_start(args, format);
+	vsnprintf(vty->frame + vty->frame_pos,
+		  sizeof(vty->frame) - vty->frame_pos, format, args);
+	vty->frame_pos = strlen(vty->frame);
+	va_end(args);
+}
+
+void vty_endframe(struct vty *vty, const char *endtext)
+{
+	if (vty->frame_pos == 0 && endtext)
+		vty_out(vty, "%s", endtext);
+	vty->frame_pos = 0;
+}
+
 /* VTY standard output function. */
 int vty_out(struct vty *vty, const char *format, ...)
 {
@@ -100,6 +118,11 @@ int vty_out(struct vty *vty, const char *format, ...)
 	char buf[1024];
 	char *p = NULL;
 
+	if (vty->frame_pos) {
+		vty->frame_pos = 0;
+		vty_out(vty, "%s", vty->frame);
+	}
+
 	if (vty_shell(vty)) {
 		va_start(args, format);
 		vprintf(format, args);
@@ -136,7 +159,10 @@ int vty_out(struct vty *vty, const char *format, ...)
 			p = buf;
 
 		/* Pointer p must point out buffer. */
-		buffer_put(vty->obuf, (u_char *)p, len);
+		if (vty->type != VTY_TERM)
+			buffer_put(vty->obuf, (uint8_t *)p, len);
+		else
+			buffer_put_crlf(vty->obuf, (uint8_t *)p, len);
 
 		/* If p is not different with buf, it is allocated buffer.  */
 		if (p != buf)
@@ -235,11 +261,11 @@ void vty_hello(struct vty *vty)
 				     (s > buf) && isspace((int)*(s - 1)); s--)
 					;
 				*s = '\0';
-				vty_out(vty, "%s%s", buf, VTY_NEWLINE);
+				vty_out(vty, "%s\n", buf);
 			}
 			fclose(f);
 		} else
-			vty_out(vty, "MOTD file not found%s", VTY_NEWLINE);
+			vty_out(vty, "MOTD file not found\n");
 	} else if (host.motd)
 		vty_out(vty, "%s", host.motd);
 }
@@ -247,16 +273,8 @@ void vty_hello(struct vty *vty)
 /* Put out prompt and wait input from user. */
 static void vty_prompt(struct vty *vty)
 {
-	struct utsname names;
-	const char *hostname;
-
 	if (vty->type == VTY_TERM) {
-		hostname = host.name;
-		if (!hostname) {
-			uname(&names);
-			hostname = names.nodename;
-		}
-		vty_out(vty, cmd_prompt(vty->node), hostname);
+		vty_out(vty, cmd_prompt(vty->node), cmd_hostname_get());
 	}
 }
 
@@ -356,15 +374,13 @@ static void vty_auth(struct vty *vty, char *buf)
 		if (vty->fail >= 3) {
 			if (vty->node == AUTH_NODE) {
 				vty_out(vty,
-					"%% Bad passwords, too many failures!%s",
-					VTY_NEWLINE);
+					"%% Bad passwords, too many failures!\n");
 				vty->status = VTY_CLOSE;
 			} else {
 				/* AUTH_ENABLE_NODE */
 				vty->fail = 0;
 				vty_out(vty,
-					"%% Bad enable passwords, too many failures!%s",
-					VTY_NEWLINE);
+					"%% Bad enable passwords, too many failures!\n");
 				vty->status = VTY_CLOSE;
 			}
 		}
@@ -429,7 +445,7 @@ static int vty_command(struct vty *vty, char *buf)
 		ret = cmd_execute_command(vline, vty, NULL, 0);
 
 		/* Get the name of the protocol if any */
-		protocolname = zlog_protoname();
+		protocolname = frr_protoname;
 
 #ifdef CONSUMED_TIME_CHECK
 		GETRUSAGE(&after);
@@ -446,17 +462,17 @@ static int vty_command(struct vty *vty, char *buf)
 		switch (ret) {
 		case CMD_WARNING:
 			if (vty->type == VTY_FILE)
-				vty_out(vty, "Warning...%s", VTY_NEWLINE);
+				vty_out(vty, "Warning...\n");
 			break;
 		case CMD_ERR_AMBIGUOUS:
-			vty_out(vty, "%% Ambiguous command.%s", VTY_NEWLINE);
+			vty_out(vty, "%% Ambiguous command.\n");
 			break;
 		case CMD_ERR_NO_MATCH:
-			vty_out(vty, "%% [%s] Unknown command: %s%s",
-				protocolname, buf, VTY_NEWLINE);
+			vty_out(vty, "%% [%s] Unknown command: %s\n",
+				protocolname, buf);
 			break;
 		case CMD_ERR_INCOMPLETE:
-			vty_out(vty, "%% Command incomplete.%s", VTY_NEWLINE);
+			vty_out(vty, "%% Command incomplete.\n");
 			break;
 		}
 	cmd_free_strvec(vline);
@@ -665,7 +681,7 @@ static void vty_backward_word(struct vty *vty)
    level. */
 static void vty_down_level(struct vty *vty)
 {
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 	cmd_exit(vty);
 	vty_prompt(vty);
 	vty->cp = 0;
@@ -674,7 +690,7 @@ static void vty_down_level(struct vty *vty)
 /* When '^Z' is received from vty, move down to the enable mode. */
 static void vty_end_config(struct vty *vty)
 {
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 
 	switch (vty->node) {
 	case VIEW_NODE:
@@ -687,6 +703,7 @@ static void vty_end_config(struct vty *vty)
 	case ZEBRA_NODE:
 	case RIP_NODE:
 	case RIPNG_NODE:
+	case EIGRP_NODE:
 	case BGP_NODE:
 	case BGP_VPNV4_NODE:
 	case BGP_VPNV6_NODE:
@@ -696,10 +713,13 @@ static void vty_end_config(struct vty *vty)
 	case BGP_VNC_L2_GROUP_NODE:
 	case BGP_IPV4_NODE:
 	case BGP_IPV4M_NODE:
+	case BGP_IPV4L_NODE:
 	case BGP_IPV6_NODE:
 	case BGP_IPV6M_NODE:
 	case BGP_EVPN_NODE:
+	case BGP_IPV6L_NODE:
 	case RMAP_NODE:
+	case PBRMAP_NODE:
 	case OSPF_NODE:
 	case OSPF6_NODE:
 	case LDP_NODE:
@@ -712,9 +732,8 @@ static void vty_end_config(struct vty *vty)
 	case ISIS_NODE:
 	case KEYCHAIN_NODE:
 	case KEYCHAIN_KEY_NODE:
-	case MASC_NODE:
-	case PIM_NODE:
 	case VTY_NODE:
+	case BGP_EVPN_VNI_NODE:
 		vty_config_unlock(vty);
 		vty->node = ENABLE_NODE;
 		break;
@@ -864,24 +883,22 @@ static void vty_complete_command(struct vty *vty)
 
 	cmd_free_strvec(vline);
 
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 	switch (ret) {
 	case CMD_ERR_AMBIGUOUS:
-		vty_out(vty, "%% Ambiguous command.%s", VTY_NEWLINE);
+		vty_out(vty, "%% Ambiguous command.\n");
 		vty_prompt(vty);
 		vty_redraw_line(vty);
 		break;
 	case CMD_ERR_NO_MATCH:
-		/* vty_out (vty, "%% There is no matched command.%s",
-		 * VTY_NEWLINE); */
+		/* vty_out (vty, "%% There is no matched command.\n"); */
 		vty_prompt(vty);
 		vty_redraw_line(vty);
 		break;
 	case CMD_COMPLETE_FULL_MATCH:
 		if (!matched[0]) {
 			/* 2016-11-28 equinox -- need to debug, SEGV here */
-			vty_out(vty, "%% CLI BUG: FULL_MATCH with NULL str%s",
-				VTY_NEWLINE);
+			vty_out(vty, "%% CLI BUG: FULL_MATCH with NULL str\n");
 			vty_prompt(vty);
 			vty_redraw_line(vty);
 			break;
@@ -891,23 +908,23 @@ static void vty_complete_command(struct vty *vty)
 		vty_backward_pure_word(vty);
 		vty_insert_word_overwrite(vty, matched[0]);
 		vty_self_insert(vty, ' ');
-		XFREE(MTYPE_TMP, matched[0]);
+		XFREE(MTYPE_COMPLETION, matched[0]);
 		break;
 	case CMD_COMPLETE_MATCH:
 		vty_prompt(vty);
 		vty_redraw_line(vty);
 		vty_backward_pure_word(vty);
 		vty_insert_word_overwrite(vty, matched[0]);
-		XFREE(MTYPE_TMP, matched[0]);
+		XFREE(MTYPE_COMPLETION, matched[0]);
 		break;
 	case CMD_COMPLETE_LIST_MATCH:
 		for (i = 0; matched[i] != NULL; i++) {
 			if (i != 0 && ((i % 6) == 0))
-				vty_out(vty, "%s", VTY_NEWLINE);
+				vty_out(vty, "\n");
 			vty_out(vty, "%-10s ", matched[i]);
-			XFREE(MTYPE_TMP, matched[i]);
+			XFREE(MTYPE_COMPLETION, matched[i]);
 		}
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "\n");
 
 		vty_prompt(vty);
 		vty_redraw_line(vty);
@@ -933,8 +950,7 @@ static void vty_describe_fold(struct vty *vty, int cmd_width,
 	cmd = token->text;
 
 	if (desc_width <= 0) {
-		vty_out(vty, "  %-*s  %s%s", cmd_width, cmd, token->desc,
-			VTY_NEWLINE);
+		vty_out(vty, "  %-*s  %s\n", cmd_width, cmd, token->desc);
 		return;
 	}
 
@@ -950,12 +966,12 @@ static void vty_describe_fold(struct vty *vty, int cmd_width,
 
 		strncpy(buf, p, pos);
 		buf[pos] = '\0';
-		vty_out(vty, "  %-*s  %s%s", cmd_width, cmd, buf, VTY_NEWLINE);
+		vty_out(vty, "  %-*s  %s\n", cmd_width, cmd, buf);
 
 		cmd = "";
 	}
 
-	vty_out(vty, "  %-*s  %s%s", cmd_width, cmd, p, VTY_NEWLINE);
+	vty_out(vty, "  %-*s  %s\n", cmd_width, cmd, p);
 
 	XFREE(MTYPE_TMP, buf);
 }
@@ -980,16 +996,16 @@ static void vty_describe_command(struct vty *vty)
 
 	describe = cmd_describe_command(vline, vty, &ret);
 
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 
 	/* Ambiguous error. */
 	switch (ret) {
 	case CMD_ERR_AMBIGUOUS:
-		vty_out(vty, "%% Ambiguous command.%s", VTY_NEWLINE);
+		vty_out(vty, "%% Ambiguous command.\n");
 		goto out;
 		break;
 	case CMD_ERR_NO_MATCH:
-		vty_out(vty, "%% There is no matched command.%s", VTY_NEWLINE);
+		vty_out(vty, "%% There is no matched command.\n");
 		goto out;
 		break;
 	}
@@ -1024,28 +1040,43 @@ static void vty_describe_command(struct vty *vty)
 			}
 
 			if (!token->desc)
-				vty_out(vty, "  %-s%s", token->text,
-					VTY_NEWLINE);
+				vty_out(vty, "  %-s\n", token->text);
 			else if (desc_width >= strlen(token->desc))
-				vty_out(vty, "  %-*s  %s%s", width, token->text,
-					token->desc, VTY_NEWLINE);
+				vty_out(vty, "  %-*s  %s\n", width, token->text,
+					token->desc);
 			else
 				vty_describe_fold(vty, width, desc_width,
 						  token);
 
+			if (IS_VARYING_TOKEN(token->type)) {
+				const char *ref = vector_slot(
+					vline, vector_active(vline) - 1);
+
+				vector varcomps = vector_init(VECTOR_MIN_SIZE);
+				cmd_variable_complete(token, ref, varcomps);
+
+				if (vector_active(varcomps) > 0) {
+					char *ac = cmd_variable_comp2str(
+						varcomps, vty->width);
+					vty_out(vty, "%s\n", ac);
+					XFREE(MTYPE_TMP, ac);
+				}
+
+				vector_free(varcomps);
+			}
 #if 0
-        vty_out (vty, "  %-*s %s%s", width
+        vty_out (vty, "  %-*s %s\n", width
                  desc->cmd[0] == '.' ? desc->cmd + 1 : desc->cmd,
-                 desc->str ? desc->str : "", VTY_NEWLINE);
+                 desc->str ? desc->str : "");
 #endif /* 0 */
 		}
 
 	if ((token = token_cr)) {
 		if (!token->desc)
-			vty_out(vty, "  %-s%s", token->text, VTY_NEWLINE);
+			vty_out(vty, "  %-s\n", token->text);
 		else if (desc_width >= strlen(token->desc))
-			vty_out(vty, "  %-*s  %s%s", width, token->text,
-				token->desc, VTY_NEWLINE);
+			vty_out(vty, "  %-*s  %s\n", width, token->text,
+				token->desc);
 		else
 			vty_describe_fold(vty, width, desc_width, token);
 	}
@@ -1069,7 +1100,7 @@ static void vty_stop_input(struct vty *vty)
 {
 	vty->cp = vty->length = 0;
 	vty_clear_buf(vty);
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 
 	switch (vty->node) {
 	case VIEW_NODE:
@@ -1082,8 +1113,10 @@ static void vty_stop_input(struct vty *vty)
 	case ZEBRA_NODE:
 	case RIP_NODE:
 	case RIPNG_NODE:
+	case EIGRP_NODE:
 	case BGP_NODE:
 	case RMAP_NODE:
+	case PBRMAP_NODE:
 	case OSPF_NODE:
 	case OSPF6_NODE:
 	case LDP_NODE:
@@ -1096,8 +1129,6 @@ static void vty_stop_input(struct vty *vty)
 	case ISIS_NODE:
 	case KEYCHAIN_NODE:
 	case KEYCHAIN_KEY_NODE:
-	case MASC_NODE:
-	case PIM_NODE:
 	case VTY_NODE:
 		vty_config_unlock(vty);
 		vty->node = ENABLE_NODE;
@@ -1174,20 +1205,20 @@ static int vty_telnet_option(struct vty *vty, unsigned char *buf, int nbytes)
 			vty_out(vty, "SE ");
 			break;
 		case TELOPT_ECHO:
-			vty_out(vty, "TELOPT_ECHO %s", VTY_NEWLINE);
+			vty_out(vty, "TELOPT_ECHO \n");
 			break;
 		case TELOPT_SGA:
-			vty_out(vty, "TELOPT_SGA %s", VTY_NEWLINE);
+			vty_out(vty, "TELOPT_SGA \n");
 			break;
 		case TELOPT_NAWS:
-			vty_out(vty, "TELOPT_NAWS %s", VTY_NEWLINE);
+			vty_out(vty, "TELOPT_NAWS \n");
 			break;
 		default:
 			vty_out(vty, "%x ", buf[i]);
 			break;
 		}
 	}
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 
 #endif /* TELNET_OPTION_DEBUG */
 
@@ -1212,12 +1243,12 @@ static int vty_telnet_option(struct vty *vty, unsigned char *buf, int nbytes)
 					"RFC 1073 violation detected: telnet NAWS option "
 					"should send %d characters, but we received %lu",
 					TELNET_NAWS_SB_LEN,
-					(u_long)vty->sb_len);
+					(unsigned long)vty->sb_len);
 			else if (sizeof(vty->sb_buf) < TELNET_NAWS_SB_LEN)
 				zlog_err(
 					"Bug detected: sizeof(vty->sb_buf) %lu < %d, "
 					"too small to handle the telnet NAWS option",
-					(u_long)sizeof(vty->sb_buf),
+					(unsigned long)sizeof(vty->sb_buf),
 					TELNET_NAWS_SB_LEN);
 			else {
 				vty->width = ((vty->sb_buf[1] << 8)
@@ -1227,8 +1258,8 @@ static int vty_telnet_option(struct vty *vty, unsigned char *buf, int nbytes)
 #ifdef TELNET_OPTION_DEBUG
 				vty_out(vty,
 					"TELNET NAWS window size negotiation completed: "
-					"width %d, height %d%s",
-					vty->width, vty->height, VTY_NEWLINE);
+					"width %d, height %d\n",
+					vty->width, vty->height);
 #endif
 			}
 			break;
@@ -1465,7 +1496,7 @@ static int vty_read(struct thread *thread)
 			break;
 		case '\n':
 		case '\r':
-			vty_out(vty, "%s", VTY_NEWLINE);
+			vty_out(vty, "\n");
 			vty_execute(vty);
 			break;
 		case '\t':
@@ -1613,7 +1644,7 @@ static struct vty *vty_create(int vty_sock, union sockunion *su)
 	if (!no_password_check) {
 		/* Vty is not available if password isn't set. */
 		if (host.password == NULL && host.password_encrypt == NULL) {
-			vty_out(vty, "Vty password is not set.%s", VTY_NEWLINE);
+			vty_out(vty, "Vty password is not set.\n");
 			vty->status = VTY_CLOSE;
 			vty_close(vty);
 			return NULL;
@@ -1623,8 +1654,7 @@ static struct vty *vty_create(int vty_sock, union sockunion *su)
 	/* Say hello to the world. */
 	vty_hello(vty);
 	if (!no_password_check)
-		vty_out(vty, "%sUser Access Verification%s%s", VTY_NEWLINE,
-			VTY_NEWLINE, VTY_NEWLINE);
+		vty_out(vty, "\nUser Access Verification\n\n");
 
 	/* Setting up terminal. */
 	vty_will_echo(vty);
@@ -1646,24 +1676,82 @@ static struct vty *vty_create(int vty_sock, union sockunion *su)
 /* create vty for stdio */
 static struct termios stdio_orig_termios;
 static struct vty *stdio_vty = NULL;
-static void (*stdio_vty_atclose)(void);
+static bool stdio_termios = false;
+static void (*stdio_vty_atclose)(int isexit);
 
-static void vty_stdio_reset(void)
+static void vty_stdio_reset(int isexit)
 {
 	if (stdio_vty) {
-		tcsetattr(0, TCSANOW, &stdio_orig_termios);
+		if (stdio_termios)
+			tcsetattr(0, TCSANOW, &stdio_orig_termios);
+		stdio_termios = false;
+
 		stdio_vty = NULL;
 
 		if (stdio_vty_atclose)
-			stdio_vty_atclose();
+			stdio_vty_atclose(isexit);
 		stdio_vty_atclose = NULL;
 	}
 }
 
-struct vty *vty_stdio(void (*atclose)())
+static void vty_stdio_atexit(void)
+{
+	vty_stdio_reset(1);
+}
+
+void vty_stdio_suspend(void)
+{
+	if (!stdio_vty)
+		return;
+
+	if (stdio_vty->t_write)
+		thread_cancel(stdio_vty->t_write);
+	if (stdio_vty->t_read)
+		thread_cancel(stdio_vty->t_read);
+	if (stdio_vty->t_timeout)
+		thread_cancel(stdio_vty->t_timeout);
+
+	if (stdio_termios)
+		tcsetattr(0, TCSANOW, &stdio_orig_termios);
+	stdio_termios = false;
+}
+
+void vty_stdio_resume(void)
+{
+	if (!stdio_vty)
+		return;
+
+	if (!tcgetattr(0, &stdio_orig_termios)) {
+		struct termios termios;
+
+		termios = stdio_orig_termios;
+		termios.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR
+				     | IGNCR | ICRNL | IXON);
+		termios.c_oflag &= ~OPOST;
+		termios.c_lflag &= ~(ECHO | ECHONL | ICANON | IEXTEN);
+		termios.c_cflag &= ~(CSIZE | PARENB);
+		termios.c_cflag |= CS8;
+		tcsetattr(0, TCSANOW, &termios);
+		stdio_termios = true;
+	}
+
+	vty_prompt(stdio_vty);
+
+	/* Add read/write thread. */
+	vty_event(VTY_WRITE, 1, stdio_vty);
+	vty_event(VTY_READ, 0, stdio_vty);
+}
+
+void vty_stdio_close(void)
+{
+	if (!stdio_vty)
+		return;
+	vty_close(stdio_vty);
+}
+
+struct vty *vty_stdio(void (*atclose)(int isexit))
 {
 	struct vty *vty;
-	struct termios termios;
 
 	/* refuse creating two vtys on stdio */
 	if (stdio_vty)
@@ -1681,23 +1769,7 @@ struct vty *vty_stdio(void (*atclose)())
 	vty->v_timeout = 0;
 	strcpy(vty->address, "console");
 
-	if (!tcgetattr(0, &stdio_orig_termios)) {
-		termios = stdio_orig_termios;
-		termios.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR
-				     | IGNCR | ICRNL | IXON);
-		termios.c_oflag &= ~OPOST;
-		termios.c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
-		termios.c_cflag &= ~(CSIZE | PARENB);
-		termios.c_cflag |= CS8;
-		tcsetattr(0, TCSANOW, &termios);
-	}
-
-	vty_prompt(vty);
-
-	/* Add read/write thread. */
-	vty_event(VTY_WRITE, 1, vty);
-	vty_event(VTY_READ, 0, vty);
-
+	vty_stdio_resume();
 	return vty;
 }
 
@@ -1796,7 +1868,7 @@ static void vty_serv_sock_addrinfo(const char *hostname, unsigned short port)
 	ret = getaddrinfo(hostname, port_str, &req, &ainfo);
 
 	if (ret != 0) {
-		fprintf(stderr, "getaddrinfo failed: %s\n", gai_strerror(ret));
+		zlog_err("getaddrinfo failed: %s", gai_strerror(ret));
 		exit(1);
 	}
 
@@ -1985,7 +2057,7 @@ static int vtysh_read(struct thread *thread)
 	struct vty *vty;
 	unsigned char buf[VTY_READ_BUFSIZ];
 	unsigned char *p;
-	u_char header[4] = {0, 0, 0, 0};
+	uint8_t header[4] = {0, 0, 0, 0};
 
 	sock = THREAD_FD(thread);
 	vty = THREAD_ARG(thread);
@@ -2019,7 +2091,7 @@ static int vtysh_read(struct thread *thread)
 		/* Clear command line buffer. */
 		vty->cp = vty->length = 0;
 		vty_clear_buf(vty);
-		vty_out(vty, "%% Command is too long.%s", VTY_NEWLINE);
+		vty_out(vty, "%% Command is too long.\n");
 	} else {
 		for (p = buf; p < buf + nbytes; p++) {
 			vty->buf[vty->length++] = *p;
@@ -2042,8 +2114,8 @@ static int vtysh_read(struct thread *thread)
 				if (ret == CMD_SUSPEND)
 					break;
 
-				/* warning: watchquagga hardcodes this result
-				 * write */
+				/* warning: watchfrr hardcodes this result write
+				 */
 				header[3] = ret;
 				buffer_put(vty->obuf, header, 4);
 
@@ -2055,7 +2127,10 @@ static int vtysh_read(struct thread *thread)
 		}
 	}
 
-	vty_event(VTYSH_READ, sock, vty);
+	if (vty->status == VTY_CLOSE)
+		vty_close(vty);
+	else
+		vty_event(VTYSH_READ, sock, vty);
 
 	return 0;
 }
@@ -2125,10 +2200,9 @@ void vty_close(struct vty *vty)
 	 * additionally, we'd need to replace these fds with /dev/null. */
 	if (vty->wfd > STDERR_FILENO && vty->wfd != vty->fd)
 		close(vty->wfd);
-
-	if (vty->fd > STDERR_FILENO)
+	if (vty->fd > STDERR_FILENO) {
 		close(vty->fd);
-	else
+	} else
 		was_stdio = true;
 
 	if (vty->buf)
@@ -2144,7 +2218,7 @@ void vty_close(struct vty *vty)
 	XFREE(MTYPE_VTY, vty);
 
 	if (was_stdio)
-		vty_stdio_reset();
+		vty_stdio_reset(0);
 }
 
 /* When time out occur output message then close connection. */
@@ -2158,8 +2232,7 @@ static int vty_timeout(struct thread *thread)
 
 	/* Clear buffer*/
 	buffer_reset(vty->obuf);
-	vty_out(vty, "%sVty connection is timed out.%s", VTY_NEWLINE,
-		VTY_NEWLINE);
+	vty_out(vty, "\nVty connection is timed out.\n");
 
 	/* Close connection. */
 	vty->status = VTY_CLOSE;
@@ -2195,23 +2268,38 @@ static void vty_read_file(FILE *confp)
 
 	if (!((ret == CMD_SUCCESS) || (ret == CMD_ERR_NOTHING_TODO))) {
 		const char *message = NULL;
+		char *nl;
+
 		switch (ret) {
 		case CMD_ERR_AMBIGUOUS:
-			message =
-				"*** Error reading config: Ambiguous command.";
+			message = "Ambiguous command";
 			break;
 		case CMD_ERR_NO_MATCH:
+			message = "No such command";
+			break;
+		case CMD_WARNING:
+			message = "Command returned Warning";
+			break;
+		case CMD_WARNING_CONFIG_FAILED:
+			message = "Command returned Warning Config Failed";
+			break;
+		case CMD_ERR_INCOMPLETE:
+			message = "Command returned Incomplete";
+			break;
+		case CMD_ERR_EXEED_ARGC_MAX:
 			message =
-				"*** Error reading config: There is no such command.";
+				"Command exceeded maximum number of Arguments";
+			break;
+		default:
+			message = "Command returned unhandled error message";
 			break;
 		}
-		fprintf(stderr, "%s\n", message);
-		zlog_err("%s", message);
-		fprintf(stderr,
-			"*** Error occurred processing line %u, below:\n%s\n",
-			line_num, vty->error_buf);
-		zlog_err("*** Error occurred processing line %u, below:\n%s",
-			 line_num, vty->error_buf);
+
+		nl = strchr(vty->error_buf, '\n');
+		if (nl)
+			*nl = '\0';
+		zlog_err("ERROR: %s on config line %u: %s", message, line_num,
+			 vty->error_buf);
 	}
 
 	vty_close(vty);
@@ -2283,8 +2371,8 @@ void vty_read_config(const char *config_file, char *config_default_dir)
 	if (config_file != NULL) {
 		if (!IS_DIRECTORY_SEP(config_file[0])) {
 			if (getcwd(cwd, MAXPATHLEN) == NULL) {
-				fprintf(stderr,
-					"Failure to determine Current Working Directory %d!\n",
+				zlog_err(
+					"Failure to determine Current Working Directory %d!",
 					errno);
 				exit(1);
 			}
@@ -2298,18 +2386,16 @@ void vty_read_config(const char *config_file, char *config_default_dir)
 		confp = fopen(fullpath, "r");
 
 		if (confp == NULL) {
-			fprintf(stderr,
-				"%s: failed to open configuration file %s: %s\n",
-				__func__, fullpath, safe_strerror(errno));
+			zlog_err("%s: failed to open configuration file %s: %s",
+				 __func__, fullpath, safe_strerror(errno));
 
 			confp = vty_use_backup_config(fullpath);
 			if (confp)
-				fprintf(stderr,
-					"WARNING: using backup configuration file!\n");
+				zlog_warn(
+					"WARNING: using backup configuration file!");
 			else {
-				fprintf(stderr,
-					"can't open configuration file [%s]\n",
-					config_file);
+				zlog_err("can't open configuration file [%s]",
+					 config_file);
 				exit(1);
 			}
 		}
@@ -2343,20 +2429,18 @@ void vty_read_config(const char *config_file, char *config_default_dir)
 #endif /* VTYSH */
 		confp = fopen(config_default_dir, "r");
 		if (confp == NULL) {
-			fprintf(stderr,
-				"%s: failed to open configuration file %s: %s\n",
-				__func__, config_default_dir,
-				safe_strerror(errno));
+			zlog_err("%s: failed to open configuration file %s: %s",
+				 __func__, config_default_dir,
+				 safe_strerror(errno));
 
 			confp = vty_use_backup_config(config_default_dir);
 			if (confp) {
-				fprintf(stderr,
-					"WARNING: using backup configuration file!\n");
+				zlog_warn(
+					"WARNING: using backup configuration file!");
 				fullpath = config_default_dir;
 			} else {
-				fprintf(stderr,
-					"can't open configuration file [%s]\n",
-					config_default_dir);
+				zlog_err("can't open configuration file [%s]",
+					 config_default_dir);
 				goto tmp_free_and_out;
 			}
 		} else
@@ -2457,44 +2541,47 @@ static struct thread_master *vty_master;
 
 static void vty_event(enum event event, int sock, struct vty *vty)
 {
-	struct thread *vty_serv_thread;
+	struct thread *vty_serv_thread = NULL;
 
 	switch (event) {
 	case VTY_SERV:
-		vty_serv_thread =
-			thread_add_read(vty_master, vty_accept, vty, sock);
+		vty_serv_thread = thread_add_read(vty_master, vty_accept, vty,
+						  sock, NULL);
 		vector_set_index(Vvty_serv_thread, sock, vty_serv_thread);
 		break;
 #ifdef VTYSH
 	case VTYSH_SERV:
-		vty_serv_thread =
-			thread_add_read(vty_master, vtysh_accept, vty, sock);
+		vty_serv_thread = thread_add_read(vty_master, vtysh_accept, vty,
+						  sock, NULL);
 		vector_set_index(Vvty_serv_thread, sock, vty_serv_thread);
 		break;
 	case VTYSH_READ:
-		vty->t_read =
-			thread_add_read(vty_master, vtysh_read, vty, sock);
+		vty->t_read = NULL;
+		thread_add_read(vty_master, vtysh_read, vty, sock,
+				&vty->t_read);
 		break;
 	case VTYSH_WRITE:
-		vty->t_write =
-			thread_add_write(vty_master, vtysh_write, vty, sock);
+		vty->t_write = NULL;
+		thread_add_write(vty_master, vtysh_write, vty, sock,
+				 &vty->t_write);
 		break;
 #endif /* VTYSH */
 	case VTY_READ:
-		vty->t_read = thread_add_read(vty_master, vty_read, vty, sock);
+		vty->t_read = NULL;
+		thread_add_read(vty_master, vty_read, vty, sock, &vty->t_read);
 
 		/* Time out treatment. */
 		if (vty->v_timeout) {
 			if (vty->t_timeout)
 				thread_cancel(vty->t_timeout);
-			vty->t_timeout = thread_add_timer(
-				vty_master, vty_timeout, vty, vty->v_timeout);
+			vty->t_timeout = NULL;
+			thread_add_timer(vty_master, vty_timeout, vty,
+					 vty->v_timeout, &vty->t_timeout);
 		}
 		break;
 	case VTY_WRITE:
-		if (!vty->t_write)
-			vty->t_write = thread_add_write(vty_master, vty_flush,
-							vty, sock);
+		thread_add_write(vty_master, vty_flush, vty, sock,
+				 &vty->t_write);
 		break;
 	case VTY_TIMEOUT_RESET:
 		if (vty->t_timeout) {
@@ -2502,8 +2589,9 @@ static void vty_event(enum event event, int sock, struct vty *vty)
 			vty->t_timeout = NULL;
 		}
 		if (vty->v_timeout) {
-			vty->t_timeout = thread_add_timer(
-				vty_master, vty_timeout, vty, vty->v_timeout);
+			vty->t_timeout = NULL;
+			thread_add_timer(vty_master, vty_timeout, vty,
+					 vty->v_timeout, &vty->t_timeout);
 		}
 		break;
 	}
@@ -2519,9 +2607,8 @@ DEFUN_NOSH (config_who,
 
 	for (i = 0; i < vector_active(vtyvec); i++)
 		if ((v = vector_slot(vtyvec, i)) != NULL)
-			vty_out(vty, "%svty[%d] connected from %s.%s",
-				v->config ? "*" : " ", i, v->address,
-				VTY_NEWLINE);
+			vty_out(vty, "%svty[%d] connected from %s.\n",
+				v->config ? "*" : " ", i, v->address);
 	return CMD_SUCCESS;
 }
 
@@ -2619,9 +2706,8 @@ DEFUN (no_vty_access_class,
 	const char *accesslist = (argc == 3) ? argv[idx_word]->arg : NULL;
 	if (!vty_accesslist_name
 	    || (argc == 3 && strcmp(vty_accesslist_name, accesslist))) {
-		vty_out(vty, "Access-class is not currently applied to vty%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Access-class is not currently applied to vty\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	XFREE(MTYPE_VTY, vty_accesslist_name);
@@ -2663,9 +2749,8 @@ DEFUN (no_vty_ipv6_access_class,
 	if (!vty_ipv6_accesslist_name
 	    || (argc == 4 && strcmp(vty_ipv6_accesslist_name, accesslist))) {
 		vty_out(vty,
-			"IPv6 access-class is not currently applied to vty%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+			"IPv6 access-class is not currently applied to vty\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	XFREE(MTYPE_VTY, vty_ipv6_accesslist_name);
@@ -2763,7 +2848,7 @@ DEFUN_NOSH (show_history,
 		}
 
 		if (vty->hist[index] != NULL)
-			vty_out(vty, "  %s%s", vty->hist[index], VTY_NEWLINE);
+			vty_out(vty, "  %s\n", vty->hist[index]);
 
 		index++;
 	}
@@ -2785,29 +2870,28 @@ DEFUN (log_commands,
 /* Display current configuration. */
 static int vty_config_write(struct vty *vty)
 {
-	vty_out(vty, "line vty%s", VTY_NEWLINE);
+	vty_out(vty, "line vty\n");
 
 	if (vty_accesslist_name)
-		vty_out(vty, " access-class %s%s", vty_accesslist_name,
-			VTY_NEWLINE);
+		vty_out(vty, " access-class %s\n", vty_accesslist_name);
 
 	if (vty_ipv6_accesslist_name)
-		vty_out(vty, " ipv6 access-class %s%s",
-			vty_ipv6_accesslist_name, VTY_NEWLINE);
+		vty_out(vty, " ipv6 access-class %s\n",
+			vty_ipv6_accesslist_name);
 
 	/* exec-timeout */
 	if (vty_timeout_val != VTY_TIMEOUT_DEFAULT)
-		vty_out(vty, " exec-timeout %ld %ld%s", vty_timeout_val / 60,
-			vty_timeout_val % 60, VTY_NEWLINE);
+		vty_out(vty, " exec-timeout %ld %ld\n", vty_timeout_val / 60,
+			vty_timeout_val % 60);
 
 	/* login */
 	if (no_password_check)
-		vty_out(vty, " no login%s", VTY_NEWLINE);
+		vty_out(vty, " no login\n");
 
 	if (do_log_commands)
-		vty_out(vty, "log commands%s", VTY_NEWLINE);
+		vty_out(vty, "log commands\n");
 
-	vty_out(vty, "!%s", VTY_NEWLINE);
+	vty_out(vty, "!\n");
 
 	return CMD_SUCCESS;
 }
@@ -2865,13 +2949,12 @@ static void vty_save_cwd(void)
 		 * Hence not worrying about it too much.
 		 */
 		if (!chdir(SYSCONFDIR)) {
-			fprintf(stderr, "Failure to chdir to %s, errno: %d\n",
-				SYSCONFDIR, errno);
+			zlog_err("Failure to chdir to %s, errno: %d",
+				 SYSCONFDIR, errno);
 			exit(-1);
 		}
 		if (getcwd(cwd, MAXPATHLEN) == NULL) {
-			fprintf(stderr, "Failure to getcwd, errno: %d\n",
-				errno);
+			zlog_err("Failure to getcwd, errno: %d", errno);
 			exit(-1);
 		}
 	}
@@ -2910,7 +2993,7 @@ void vty_init(struct thread_master *master_thread)
 
 	vty_master = master_thread;
 
-	atexit(vty_stdio_reset);
+	atexit(vty_stdio_atexit);
 
 	/* Initilize server thread vector. */
 	Vvty_serv_thread = vector_init(VECTOR_MIN_SIZE);
@@ -2954,25 +3037,3 @@ void vty_terminate(void)
 		Vvty_serv_thread = NULL;
 	}
 }
-
-/* Utility functions to get arguments from commands generated
-   by the xml2cli.pl script. */
-const char *vty_get_arg_value(struct vty_arg *args[], const char *arg)
-{
-	while (*args) {
-		if (strcmp((*args)->name, arg) == 0)
-			return (*args)->value;
-		args++;
-	}
-	return NULL;
-}
-
-struct vty_arg *vty_get_arg(struct vty_arg *args[], const char *arg)
-{
-	while (*args) {
-		if (strcmp((*args)->name, arg) == 0)
-			return *args;
-		args++;
-	}
-	return NULL;
-}
