@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -153,7 +152,7 @@ int ospf6_abr_originate_summary_to_area(struct ospf6_route *route,
 	struct ospf6_inter_prefix_lsa *prefix_lsa;
 	struct ospf6_inter_router_lsa *router_lsa;
 	struct ospf6_route_table *summary_table = NULL;
-	u_int16_t type;
+	uint16_t type;
 	char buf[PREFIX2STR_BUFFER];
 	int is_debug = 0;
 
@@ -387,27 +386,20 @@ int ospf6_abr_originate_summary_to_area(struct ospf6_route *route,
 	}
 
 	/* Check filter-list */
-	if (PREFIX_NAME_OUT(area)) {
-		if (PREFIX_LIST_OUT(area) == NULL)
-			PREFIX_LIST_OUT(area) = prefix_list_lookup(
-				AFI_IP6, PREFIX_NAME_OUT(area));
-
-		if (PREFIX_LIST_OUT(area))
-			if (prefix_list_apply(PREFIX_LIST_OUT(area),
-					      &route->prefix)
-			    != PREFIX_PERMIT) {
-				if (is_debug) {
-					inet_ntop(AF_INET,
-						  &(ADV_ROUTER_IN_PREFIX(
-							  &route->prefix)),
-						  buf, sizeof(buf));
-					zlog_debug(
-						"prefix %s was denied by filter-list out",
-						buf);
-				}
-				return 0;
+	if (PREFIX_LIST_OUT(area))
+		if (prefix_list_apply(PREFIX_LIST_OUT(area), &route->prefix)
+		    != PREFIX_PERMIT) {
+			if (is_debug) {
+				inet_ntop(
+					AF_INET,
+					&(ADV_ROUTER_IN_PREFIX(&route->prefix)),
+					buf, sizeof(buf));
+				zlog_debug(
+					"prefix %s was denied by filter-list out",
+					buf);
 			}
-	}
+			return 0;
+		}
 
 	/* the route is going to be originated. store it in area's summary_table
 	 */
@@ -514,11 +506,11 @@ void ospf6_abr_range_reset_cost(struct ospf6 *ospf6)
 			OSPF6_ABR_RANGE_CLEAR_COST(range);
 }
 
-static inline u_int32_t ospf6_abr_range_compute_cost(struct ospf6_route *range,
-						     struct ospf6 *o)
+static inline uint32_t ospf6_abr_range_compute_cost(struct ospf6_route *range,
+						    struct ospf6 *o)
 {
 	struct ospf6_route *ro;
-	u_int32_t cost = 0;
+	uint32_t cost = 0;
 
 	for (ro = ospf6_route_match_head(&range->prefix, o->route_table); ro;
 	     ro = ospf6_route_match_next(&range->prefix, ro)) {
@@ -532,7 +524,7 @@ static inline u_int32_t ospf6_abr_range_compute_cost(struct ospf6_route *range,
 }
 
 static inline int
-ospf6_abr_range_summary_needs_update(struct ospf6_route *range, u_int32_t cost)
+ospf6_abr_range_summary_needs_update(struct ospf6_route *range, uint32_t cost)
 {
 	int redo_summary = 0;
 
@@ -568,7 +560,7 @@ ospf6_abr_range_summary_needs_update(struct ospf6_route *range, u_int32_t cost)
 
 static void ospf6_abr_range_update(struct ospf6_route *range)
 {
-	u_int32_t cost = 0;
+	uint32_t cost = 0;
 	struct listnode *node, *nnode;
 	struct ospf6_area *oa;
 	int summary_orig = 0;
@@ -694,15 +686,16 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 	struct ospf6_route_table *table = NULL;
 	struct ospf6_route *range, *route, *old = NULL;
 	struct ospf6_route *abr_entry;
-	u_char type = 0;
+	uint8_t type = 0;
 	char options[3] = {0, 0, 0};
-	u_int8_t prefix_options = 0;
-	u_int32_t cost = 0;
-	u_char router_bits = 0;
+	uint8_t prefix_options = 0;
+	uint32_t cost = 0;
+	uint8_t router_bits = 0;
 	char buf[PREFIX2STR_BUFFER];
 	int is_debug = 0;
 	struct ospf6_inter_prefix_lsa *prefix_lsa = NULL;
 	struct ospf6_inter_router_lsa *router_lsa = NULL;
+	struct ospf6_path *path;
 
 	memset(&prefix, 0, sizeof(prefix));
 
@@ -793,6 +786,9 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 	/* (3) if the prefix is equal to an active configured address range */
 	/*     or if the NU bit is set in the prefix */
 	if (lsa->header->type == htons(OSPF6_LSTYPE_INTER_PREFIX)) {
+		/* must have been set in previous block */
+		assert(prefix_lsa);
+
 		range = ospf6_route_lookup(&prefix, oa->range_table);
 		if (range) {
 			if (is_debug)
@@ -874,22 +870,15 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 	}
 
 	/* Check input prefix-list */
-	if (PREFIX_NAME_IN(oa)) {
-		if (PREFIX_LIST_IN(oa) == NULL)
-			PREFIX_LIST_IN(oa) =
-				prefix_list_lookup(AFI_IP6, PREFIX_NAME_IN(oa));
-
-		if (PREFIX_LIST_IN(oa))
-			if (prefix_list_apply(PREFIX_LIST_IN(oa), &prefix)
-			    != PREFIX_PERMIT) {
-				if (is_debug)
-					zlog_debug(
-						"Prefix was denied by prefix-list");
-				if (old)
-					ospf6_route_remove(old, table);
-				return;
-			}
-	}
+	if (PREFIX_LIST_IN(oa))
+		if (prefix_list_apply(PREFIX_LIST_IN(oa), &prefix)
+		    != PREFIX_PERMIT) {
+			if (is_debug)
+				zlog_debug("Prefix was denied by prefix-list");
+			if (old)
+				ospf6_route_remove(old, table);
+			return;
+		}
 
 	/* (5),(6): the path preference is handled by the sorting
 	   in the routing table. Always install the path by substituting
@@ -915,6 +904,10 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 
 	ospf6_route_copy_nexthops(route, abr_entry);
 
+	path = ospf6_path_dup(&route->path);
+	ospf6_copy_nexthops(path->nh_list, abr_entry->nh_list);
+	listnode_add_sort(route->paths, path);
+
 	/* (7) If the routes are identical, copy the next hops over to existing
 	   route. ospf6's route table implementation will otherwise string both
 	   routes, but keep the older one as the best route since the routes
@@ -924,6 +917,13 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 
 	if (old && (ospf6_route_cmp(route, old) == 0)) {
 		ospf6_route_merge_nexthops(old, route);
+
+		if (is_debug)
+			zlog_debug("%s: Update route: %s old cost %u new cost %u nh count %u",
+				   __PRETTY_FUNCTION__,
+				   buf, old->path.cost, route->path.cost,
+				   listcount(route->nh_list));
+
 		/* Update RIB/FIB */
 		if (table->hook_add)
 			(*table->hook_add)(old);
@@ -932,17 +932,19 @@ void ospf6_abr_examin_summary(struct ospf6_lsa *lsa, struct ospf6_area *oa)
 		ospf6_route_delete(route);
 	} else {
 		if (is_debug)
-			zlog_debug("Install route: %s", buf);
+			zlog_debug("%s: Install route: %s cost %u nh count %u",
+				   __PRETTY_FUNCTION__, buf, route->path.cost,
+				   listcount(route->nh_list));
 		/* ospf6_ia_add_nw_route (table, &prefix, route); */
 		ospf6_route_add(route, table);
 	}
 }
 
-void ospf6_abr_examin_brouter(u_int32_t router_id)
+void ospf6_abr_examin_brouter(uint32_t router_id)
 {
 	struct ospf6_lsa *lsa;
 	struct ospf6_area *oa;
-	u_int16_t type;
+	uint16_t type;
 
 	if (ospf6_is_router_abr(ospf6))
 		oa = ospf6->backbone;
@@ -958,29 +960,25 @@ void ospf6_abr_examin_brouter(u_int32_t router_id)
 		return;
 
 	type = htons(OSPF6_LSTYPE_INTER_ROUTER);
-	for (lsa = ospf6_lsdb_type_router_head(type, router_id, oa->lsdb); lsa;
-	     lsa = ospf6_lsdb_type_router_next(type, router_id, lsa))
+	for (ALL_LSDB_TYPED_ADVRTR(oa->lsdb, type, router_id, lsa))
 		ospf6_abr_examin_summary(lsa, oa);
 
 	type = htons(OSPF6_LSTYPE_INTER_PREFIX);
-	for (lsa = ospf6_lsdb_type_router_head(type, router_id, oa->lsdb); lsa;
-	     lsa = ospf6_lsdb_type_router_next(type, router_id, lsa))
+	for (ALL_LSDB_TYPED_ADVRTR(oa->lsdb, type, router_id, lsa))
 		ospf6_abr_examin_summary(lsa, oa);
 }
 
 void ospf6_abr_reimport(struct ospf6_area *oa)
 {
 	struct ospf6_lsa *lsa;
-	u_int16_t type;
+	uint16_t type;
 
 	type = htons(OSPF6_LSTYPE_INTER_ROUTER);
-	for (lsa = ospf6_lsdb_type_head(type, oa->lsdb); lsa;
-	     lsa = ospf6_lsdb_type_next(type, lsa))
+	for (ALL_LSDB_TYPED(oa->lsdb, type, lsa))
 		ospf6_abr_examin_summary(lsa, oa);
 
 	type = htons(OSPF6_LSTYPE_INTER_PREFIX);
-	for (lsa = ospf6_lsdb_type_head(type, oa->lsdb); lsa;
-	     lsa = ospf6_lsdb_type_next(type, lsa))
+	for (ALL_LSDB_TYPED(oa->lsdb, type, lsa))
 		ospf6_abr_examin_summary(lsa, oa);
 }
 
@@ -1033,17 +1031,16 @@ static int ospf6_inter_area_prefix_lsa_show(struct vty *vty,
 	prefix_lsa = (struct ospf6_inter_prefix_lsa *)OSPF6_LSA_HEADER_END(
 		lsa->header);
 
-	vty_out(vty, "     Metric: %lu%s",
-		(u_long)OSPF6_ABR_SUMMARY_METRIC(prefix_lsa), VNL);
+	vty_out(vty, "     Metric: %lu\n",
+		(unsigned long)OSPF6_ABR_SUMMARY_METRIC(prefix_lsa));
 
 	ospf6_prefix_options_printbuf(prefix_lsa->prefix.prefix_options, buf,
 				      sizeof(buf));
-	vty_out(vty, "     Prefix Options: %s%s", buf, VNL);
+	vty_out(vty, "     Prefix Options: %s\n", buf);
 
-	vty_out(vty, "     Prefix: %s%s",
+	vty_out(vty, "     Prefix: %s\n",
 		ospf6_inter_area_prefix_lsa_get_prefix_str(lsa, buf,
-							   sizeof(buf), 0),
-		VNL);
+							   sizeof(buf), 0));
 
 	return 0;
 }
@@ -1077,12 +1074,12 @@ static int ospf6_inter_area_router_lsa_show(struct vty *vty,
 		lsa->header);
 
 	ospf6_options_printbuf(router_lsa->options, buf, sizeof(buf));
-	vty_out(vty, "     Options: %s%s", buf, VNL);
-	vty_out(vty, "     Metric: %lu%s",
-		(u_long)OSPF6_ABR_SUMMARY_METRIC(router_lsa), VNL);
+	vty_out(vty, "     Options: %s\n", buf);
+	vty_out(vty, "     Metric: %lu\n",
+		(unsigned long)OSPF6_ABR_SUMMARY_METRIC(router_lsa));
 
 	inet_ntop(AF_INET, &router_lsa->router_id, buf, sizeof(buf));
-	vty_out(vty, "     Destination Router ID: %s%s", buf, VNL);
+	vty_out(vty, "     Destination Router ID: %s\n", buf);
 
 	return 0;
 }
@@ -1116,7 +1113,7 @@ DEFUN (no_debug_ospf6_abr,
 int config_write_ospf6_debug_abr(struct vty *vty)
 {
 	if (IS_OSPF6_DEBUG_ABR)
-		vty_out(vty, "debug ospf6 abr%s", VNL);
+		vty_out(vty, "debug ospf6 abr\n");
 	return 0;
 }
 
@@ -1129,20 +1126,20 @@ void install_element_ospf6_debug_abr(void)
 }
 
 struct ospf6_lsa_handler inter_prefix_handler = {
-	OSPF6_LSTYPE_INTER_PREFIX,
-	"Inter-Prefix",
-	"IAP",
-	ospf6_inter_area_prefix_lsa_show,
-	ospf6_inter_area_prefix_lsa_get_prefix_str,
-};
+	.lh_type = OSPF6_LSTYPE_INTER_PREFIX,
+	.lh_name = "Inter-Prefix",
+	.lh_short_name = "IAP",
+	.lh_show = ospf6_inter_area_prefix_lsa_show,
+	.lh_get_prefix_str = ospf6_inter_area_prefix_lsa_get_prefix_str,
+	.lh_debug = 0};
 
 struct ospf6_lsa_handler inter_router_handler = {
-	OSPF6_LSTYPE_INTER_ROUTER,
-	"Inter-Router",
-	"IAR",
-	ospf6_inter_area_router_lsa_show,
-	ospf6_inter_area_router_lsa_get_prefix_str,
-};
+	.lh_type = OSPF6_LSTYPE_INTER_ROUTER,
+	.lh_name = "Inter-Router",
+	.lh_short_name = "IAR",
+	.lh_show = ospf6_inter_area_router_lsa_show,
+	.lh_get_prefix_str = ospf6_inter_area_router_lsa_get_prefix_str,
+	.lh_debug = 0};
 
 void ospf6_abr_init(void)
 {
