@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -46,6 +45,8 @@
 #include "ospf6_asbr.h"
 #include "ospf6d.h"
 
+DEFINE_MTYPE_STATIC(OSPF6D, OSPF6_PLISTNAME, "Prefix list name")
+
 int ospf6_area_cmp(void *va, void *vb)
 {
 	struct ospf6_area *oa = (struct ospf6_area *)va;
@@ -60,8 +61,9 @@ static void ospf6_area_lsdb_hook_add(struct ospf6_lsa *lsa)
 	case OSPF6_LSTYPE_ROUTER:
 	case OSPF6_LSTYPE_NETWORK:
 		if (IS_OSPF6_DEBUG_EXAMIN_TYPE(lsa->header->type)) {
-			zlog_debug("Examin %s", lsa->name);
-			zlog_debug("Schedule SPF Calculation for %s",
+			zlog_debug("%s Examin LSA %s", __PRETTY_FUNCTION__,
+				   lsa->name);
+			zlog_debug(" Schedule SPF Calculation for %s",
 				   OSPF6_AREA(lsa->lsdb->data)->name);
 		}
 		ospf6_spf_schedule(
@@ -116,7 +118,9 @@ static void ospf6_area_lsdb_hook_remove(struct ospf6_lsa *lsa)
 
 static void ospf6_area_route_hook_add(struct ospf6_route *route)
 {
-	struct ospf6_route *copy = ospf6_route_copy(route);
+	struct ospf6_route *copy;
+
+	copy = ospf6_route_copy(route);
 	ospf6_route_add(copy, ospf6->route_table);
 }
 
@@ -188,14 +192,29 @@ static void ospf6_area_no_summary_unset(struct ospf6 *ospf6,
 	}
 }
 
-/* Make new area structure */
-struct ospf6_area *ospf6_area_create(u_int32_t area_id, struct ospf6 *o)
+/**
+ * Make new area structure.
+ *
+ * @param area_id - ospf6 area ID
+ * @param o - ospf6 instance
+ * @param df - display format for area ID
+ */
+struct ospf6_area *ospf6_area_create(uint32_t area_id, struct ospf6 *o, int df)
 {
 	struct ospf6_area *oa;
 
 	oa = XCALLOC(MTYPE_OSPF6_AREA, sizeof(struct ospf6_area));
 
-	inet_ntop(AF_INET, &area_id, oa->name, sizeof(oa->name));
+	switch (df) {
+	case OSPF6_AREA_FMT_DECIMAL:
+		snprintf(oa->name, sizeof(oa->name), "%u", ntohl(area_id));
+		break;
+	default:
+	case OSPF6_AREA_FMT_DOTTEDQUAD:
+		inet_ntop(AF_INET, &area_id, oa->name, sizeof(oa->name));
+		break;
+	}
+
 	oa->area_id = area_id;
 	oa->if_list = list_new();
 
@@ -203,6 +222,7 @@ struct ospf6_area *ospf6_area_create(u_int32_t area_id, struct ospf6 *o)
 	oa->lsdb->hook_add = ospf6_area_lsdb_hook_add;
 	oa->lsdb->hook_remove = ospf6_area_lsdb_hook_remove;
 	oa->lsdb_self = ospf6_lsdb_create(oa);
+	oa->temp_router_lsa_lsdb = ospf6_lsdb_create(oa);
 
 	oa->spf_table = OSPF6_ROUTE_TABLE_CREATE(AREA, SPF_RESULTS);
 	oa->spf_table->scope = oa;
@@ -218,6 +238,7 @@ struct ospf6_area *ospf6_area_create(u_int32_t area_id, struct ospf6 *o)
 	oa->summary_prefix->scope = oa;
 	oa->summary_router = OSPF6_ROUTE_TABLE_CREATE(AREA, SUMMARY_ROUTERS);
 	oa->summary_router->scope = oa;
+	oa->router_lsa_size_limit = 1024 + 256;
 
 	/* set default options */
 	if (CHECK_FLAG(o->flag, OSPF6_STUB_ROUTER)) {
@@ -256,10 +277,11 @@ void ospf6_area_delete(struct ospf6_area *oa)
 	for (ALL_LIST_ELEMENTS_RO(oa->if_list, n, oi))
 		oi->area = NULL;
 
-	list_delete(oa->if_list);
+	list_delete_and_null(&oa->if_list);
 
 	ospf6_lsdb_delete(oa->lsdb);
 	ospf6_lsdb_delete(oa->lsdb_self);
+	ospf6_lsdb_delete(oa->temp_router_lsa_lsdb);
 
 	ospf6_spf_table_finish(oa->spf_table);
 	ospf6_route_table_delete(oa->spf_table);
@@ -276,7 +298,7 @@ void ospf6_area_delete(struct ospf6_area *oa)
 	XFREE(MTYPE_OSPF6_AREA, oa);
 }
 
-struct ospf6_area *ospf6_area_lookup(u_int32_t area_id, struct ospf6 *ospf6)
+struct ospf6_area *ospf6_area_lookup(uint32_t area_id, struct ospf6 *ospf6)
 {
 	struct ospf6_area *oa;
 	struct listnode *n;
@@ -288,15 +310,6 @@ struct ospf6_area *ospf6_area_lookup(u_int32_t area_id, struct ospf6 *ospf6)
 	return (struct ospf6_area *)NULL;
 }
 
-static struct ospf6_area *ospf6_area_get(u_int32_t area_id, struct ospf6 *o)
-{
-	struct ospf6_area *oa;
-	oa = ospf6_area_lookup(area_id, o);
-	if (oa == NULL)
-		oa = ospf6_area_create(area_id, o);
-	return oa;
-}
-
 void ospf6_area_enable(struct ospf6_area *oa)
 {
 	struct listnode *node, *nnode;
@@ -338,47 +351,49 @@ void ospf6_area_show(struct vty *vty, struct ospf6_area *oa)
 	unsigned long result;
 
 	if (!IS_AREA_STUB(oa))
-		vty_out(vty, " Area %s%s", oa->name, VNL);
+		vty_out(vty, " Area %s\n", oa->name);
 	else {
 		if (oa->no_summary) {
-			vty_out(vty, " Area %s[Stub, No Summary]%s", oa->name,
-				VNL);
+			vty_out(vty, " Area %s[Stub, No Summary]\n", oa->name);
 		} else {
-			vty_out(vty, " Area %s[Stub]%s", oa->name, VNL);
+			vty_out(vty, " Area %s[Stub]\n", oa->name);
 		}
 	}
-	vty_out(vty, "     Number of Area scoped LSAs is %u%s", oa->lsdb->count,
-		VNL);
+	vty_out(vty, "     Number of Area scoped LSAs is %u\n",
+		oa->lsdb->count);
 
 	vty_out(vty, "     Interface attached to this area:");
 	for (ALL_LIST_ELEMENTS_RO(oa->if_list, i, oi))
 		vty_out(vty, " %s", oi->interface->name);
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 
 	if (oa->ts_spf.tv_sec || oa->ts_spf.tv_usec) {
 		result = monotime_since(&oa->ts_spf, NULL);
 		if (result / TIMER_SECOND_MICRO > 0) {
-			vty_out(vty, "SPF last executed %ld.%lds ago%s",
+			vty_out(vty, "SPF last executed %ld.%lds ago\n",
 				result / TIMER_SECOND_MICRO,
-				result % TIMER_SECOND_MICRO, VTY_NEWLINE);
+				result % TIMER_SECOND_MICRO);
 		} else {
-			vty_out(vty, "SPF last executed %ldus ago%s", result,
-				VTY_NEWLINE);
+			vty_out(vty, "SPF last executed %ldus ago\n", result);
 		}
 	} else
-		vty_out(vty, "SPF has not been run%s", VTY_NEWLINE);
+		vty_out(vty, "SPF has not been run\n");
 }
 
 
 #define OSPF6_CMD_AREA_GET(str, oa)                                            \
 	{                                                                      \
 		char *ep;                                                      \
-		u_int32_t area_id = htonl(strtol(str, &ep, 10));               \
+		uint32_t area_id = htonl(strtoul(str, &ep, 10));               \
 		if (*ep && inet_pton(AF_INET, str, &area_id) != 1) {           \
-			vty_out(vty, "Malformed Area-ID: %s%s", str, VNL);     \
+			vty_out(vty, "Malformed Area-ID: %s\n", str);          \
 			return CMD_SUCCESS;                                    \
 		}                                                              \
-		oa = ospf6_area_get(area_id, ospf6);                           \
+		int format = !*ep ? OSPF6_AREA_FMT_DECIMAL                     \
+				  : OSPF6_AREA_FMT_DOTTEDQUAD;                 \
+		oa = ospf6_area_lookup(area_id, ospf6);                        \
+		if (oa == NULL)                                                \
+			oa = ospf6_area_create(area_id, ospf6, format);        \
 	}
 
 DEFUN (area_range,
@@ -401,14 +416,14 @@ DEFUN (area_range,
 	struct ospf6_area *oa;
 	struct prefix prefix;
 	struct ospf6_route *range;
-	u_int32_t cost = OSPF_AREA_RANGE_COST_UNSPEC;
+	uint32_t cost = OSPF_AREA_RANGE_COST_UNSPEC;
 
 	OSPF6_CMD_AREA_GET(argv[idx_ipv4]->arg, oa);
 
 	ret = str2prefix(argv[idx_ipv6_prefixlen]->arg, &prefix);
 	if (ret != 1 || prefix.family != AF_INET6) {
-		vty_out(vty, "Malformed argument: %s%s",
-			argv[idx_ipv6_prefixlen]->arg, VNL);
+		vty_out(vty, "Malformed argument: %s\n",
+			argv[idx_ipv6_prefixlen]->arg);
 		return CMD_SUCCESS;
 	}
 
@@ -427,8 +442,7 @@ DEFUN (area_range,
 		} else if (strmatch(argv[idx_type]->text, "advertise")) {
 			UNSET_FLAG(range->flag, OSPF6_ROUTE_DO_NOT_ADVERTISE);
 		} else {
-			VTY_GET_INTEGER_RANGE("cost", cost, argv[5]->arg, 0,
-					      OSPF_LS_INFINITY);
+			cost = strtoul(argv[5]->arg, NULL, 10);
 			UNSET_FLAG(range->flag, OSPF6_ROUTE_DO_NOT_ADVERTISE);
 		}
 	}
@@ -474,15 +488,14 @@ DEFUN (no_area_range,
 
 	ret = str2prefix(argv[idx_ipv6]->arg, &prefix);
 	if (ret != 1 || prefix.family != AF_INET6) {
-		vty_out(vty, "Malformed argument: %s%s", argv[idx_ipv6]->arg,
-			VNL);
+		vty_out(vty, "Malformed argument: %s\n", argv[idx_ipv6]->arg);
 		return CMD_SUCCESS;
 	}
 
 	range = ospf6_route_lookup(&prefix, oa->range_table);
 	if (range == NULL) {
-		vty_out(vty, "Range %s does not exists.%s", argv[idx_ipv6]->arg,
-			VNL);
+		vty_out(vty, "Range %s does not exists.\n",
+			argv[idx_ipv6]->arg);
 		return CMD_SUCCESS;
 	}
 
@@ -527,27 +540,27 @@ void ospf6_area_config_write(struct vty *vty)
 					vty_out(vty, " cost %d",
 						range->path.u.cost_config);
 			}
-			vty_out(vty, "%s", VNL);
+			vty_out(vty, "\n");
 		}
 		if (IS_AREA_STUB(oa)) {
 			if (oa->no_summary)
-				vty_out(vty, " area %s stub no-summary%s",
-					oa->name, VNL);
+				vty_out(vty, " area %s stub no-summary\n",
+					oa->name);
 			else
-				vty_out(vty, " area %s stub%s", oa->name, VNL);
+				vty_out(vty, " area %s stub\n", oa->name);
 		}
 		if (PREFIX_NAME_IN(oa))
-			vty_out(vty, " area %s filter-list prefix %s in%s",
-				oa->name, PREFIX_NAME_IN(oa), VNL);
+			vty_out(vty, " area %s filter-list prefix %s in\n",
+				oa->name, PREFIX_NAME_IN(oa));
 		if (PREFIX_NAME_OUT(oa))
-			vty_out(vty, " area %s filter-list prefix %s out%s",
-				oa->name, PREFIX_NAME_OUT(oa), VNL);
+			vty_out(vty, " area %s filter-list prefix %s out\n",
+				oa->name, PREFIX_NAME_OUT(oa));
 		if (IMPORT_NAME(oa))
-			vty_out(vty, " area %s import-list %s%s", oa->name,
-				IMPORT_NAME(oa), VNL);
+			vty_out(vty, " area %s import-list %s\n", oa->name,
+				IMPORT_NAME(oa));
 		if (EXPORT_NAME(oa))
-			vty_out(vty, " area %s export-list %s%s", oa->name,
-				EXPORT_NAME(oa), VNL);
+			vty_out(vty, " area %s export-list %s\n", oa->name,
+				EXPORT_NAME(oa));
 	}
 }
 
@@ -574,17 +587,15 @@ DEFUN (area_filter_list,
 	plist = prefix_list_lookup(AFI_IP6, plistname);
 	if (strmatch(inout, "in")) {
 		PREFIX_LIST_IN(area) = plist;
-		if (PREFIX_NAME_IN(area))
-			free(PREFIX_NAME_IN(area));
-
-		PREFIX_NAME_IN(area) = strdup(plistname);
+		XFREE(MTYPE_OSPF6_PLISTNAME, PREFIX_NAME_IN(area));
+		PREFIX_NAME_IN(area) =
+			XSTRDUP(MTYPE_OSPF6_PLISTNAME, plistname);
 		ospf6_abr_reimport(area);
 	} else {
 		PREFIX_LIST_OUT(area) = plist;
-		if (PREFIX_NAME_OUT(area))
-			free(PREFIX_NAME_OUT(area));
-
-		PREFIX_NAME_OUT(area) = strdup(plistname);
+		XFREE(MTYPE_OSPF6_PLISTNAME, PREFIX_NAME_OUT(area));
+		PREFIX_NAME_OUT(area) =
+			XSTRDUP(MTYPE_OSPF6_PLISTNAME, plistname);
 		ospf6_abr_enable_area(area);
 	}
 
@@ -617,27 +628,37 @@ DEFUN (no_area_filter_list,
 				return CMD_SUCCESS;
 
 		PREFIX_LIST_IN(area) = NULL;
-		if (PREFIX_NAME_IN(area))
-			free(PREFIX_NAME_IN(area));
-
-		PREFIX_NAME_IN(area) = NULL;
+		XFREE(MTYPE_OSPF6_PLISTNAME, PREFIX_NAME_IN(area));
 		ospf6_abr_reimport(area);
 	} else {
 		if (PREFIX_NAME_OUT(area))
 			if (!strmatch(PREFIX_NAME_OUT(area), plistname))
 				return CMD_SUCCESS;
 
-		PREFIX_LIST_OUT(area) = NULL;
-		if (PREFIX_NAME_OUT(area))
-			free(PREFIX_NAME_OUT(area));
-
-		PREFIX_NAME_OUT(area) = NULL;
+		XFREE(MTYPE_OSPF6_PLISTNAME, PREFIX_NAME_OUT(area));
 		ospf6_abr_enable_area(area);
 	}
 
 	return CMD_SUCCESS;
 }
 
+void ospf6_area_plist_update(struct prefix_list *plist, int add)
+{
+	struct ospf6_area *oa;
+	struct listnode *n;
+	const char *name = prefix_list_name(plist);
+
+	if (!ospf6)
+		return;
+
+	for (ALL_LIST_ELEMENTS_RO(ospf6->area_list, n, oa)) {
+		if (PREFIX_NAME_IN(oa) && !strcmp(PREFIX_NAME_IN(oa), name))
+			PREFIX_LIST_IN(oa) = add ? plist : NULL;
+		if (PREFIX_NAME_OUT(oa) && !strcmp(PREFIX_NAME_OUT(oa), name))
+			PREFIX_LIST_OUT(oa) = add ? plist : NULL;
+	}
+}
+
 DEFUN (area_import_list,
        area_import_list_cmd,
        "area A.B.C.D import-list NAME",
@@ -766,8 +787,8 @@ DEFUN (show_ipv6_ospf6_spf_tree,
 	for (ALL_LIST_ELEMENTS_RO(ospf6->area_list, node, oa)) {
 		route = ospf6_route_lookup(&prefix, oa->spf_table);
 		if (route == NULL) {
-			vty_out(vty, "LS entry for root not found in area %s%s",
-				oa->name, VNL);
+			vty_out(vty, "LS entry for root not found in area %s\n",
+				oa->name);
 			continue;
 		}
 		root = (struct ospf6_vertex *)route->route_option;
@@ -789,7 +810,7 @@ DEFUN (show_ipv6_ospf6_area_spf_tree,
        "Show SPF tree\n")
 {
 	int idx_ipv4 = 4;
-	u_int32_t area_id;
+	uint32_t area_id;
 	struct ospf6_area *oa;
 	struct ospf6_vertex *root;
 	struct ospf6_route *route;
@@ -800,20 +821,19 @@ DEFUN (show_ipv6_ospf6_area_spf_tree,
 	ospf6_linkstate_prefix(ospf6->router_id, htonl(0), &prefix);
 
 	if (inet_pton(AF_INET, argv[idx_ipv4]->arg, &area_id) != 1) {
-		vty_out(vty, "Malformed Area-ID: %s%s", argv[idx_ipv4]->arg,
-			VNL);
+		vty_out(vty, "Malformed Area-ID: %s\n", argv[idx_ipv4]->arg);
 		return CMD_SUCCESS;
 	}
 	oa = ospf6_area_lookup(area_id, ospf6);
 	if (oa == NULL) {
-		vty_out(vty, "No such Area: %s%s", argv[idx_ipv4]->arg, VNL);
+		vty_out(vty, "No such Area: %s\n", argv[idx_ipv4]->arg);
 		return CMD_SUCCESS;
 	}
 
 	route = ospf6_route_lookup(&prefix, oa->spf_table);
 	if (route == NULL) {
-		vty_out(vty, "LS entry for root not found in area %s%s",
-			oa->name, VNL);
+		vty_out(vty, "LS entry for root not found in area %s\n",
+			oa->name);
 		return CMD_SUCCESS;
 	}
 	root = (struct ospf6_vertex *)route->route_option;
@@ -836,12 +856,12 @@ DEFUN (show_ipv6_ospf6_simulate_spf_tree_root,
 {
 	int idx_ipv4 = 5;
 	int idx_ipv4_2 = 7;
-	u_int32_t area_id;
+	uint32_t area_id;
 	struct ospf6_area *oa;
 	struct ospf6_vertex *root;
 	struct ospf6_route *route;
 	struct prefix prefix;
-	u_int32_t router_id;
+	uint32_t router_id;
 	struct ospf6_route_table *spf_table;
 	unsigned char tmp_debug_ospf6_spf = 0;
 
@@ -851,13 +871,12 @@ DEFUN (show_ipv6_ospf6_simulate_spf_tree_root,
 	ospf6_linkstate_prefix(router_id, htonl(0), &prefix);
 
 	if (inet_pton(AF_INET, argv[idx_ipv4_2]->arg, &area_id) != 1) {
-		vty_out(vty, "Malformed Area-ID: %s%s", argv[idx_ipv4_2]->arg,
-			VNL);
+		vty_out(vty, "Malformed Area-ID: %s\n", argv[idx_ipv4_2]->arg);
 		return CMD_SUCCESS;
 	}
 	oa = ospf6_area_lookup(area_id, ospf6);
 	if (oa == NULL) {
-		vty_out(vty, "No such Area: %s%s", argv[idx_ipv4_2]->arg, VNL);
+		vty_out(vty, "No such Area: %s\n", argv[idx_ipv4_2]->arg);
 		return CMD_SUCCESS;
 	}
 
@@ -899,9 +918,8 @@ DEFUN (ospf6_area_stub,
 
 	if (!ospf6_area_stub_set(ospf6, area)) {
 		vty_out(vty,
-			"First deconfigure all virtual link through this area%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+			"First deconfigure all virtual link through this area\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	ospf6_area_no_summary_unset(ospf6, area);
@@ -925,9 +943,8 @@ DEFUN (ospf6_area_stub_no_summary,
 
 	if (!ospf6_area_stub_set(ospf6, area)) {
 		vty_out(vty,
-			"First deconfigure all virtual link through this area%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+			"First deconfigure all virtual link through this area\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	ospf6_area_no_summary_set(ospf6, area);
