@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -106,8 +105,9 @@ void ospf6_lsa_originate(struct ospf6_lsa *lsa)
 	lsdb_self = ospf6_get_scoped_lsdb_self(lsa);
 	ospf6_lsdb_add(ospf6_lsa_copy(lsa), lsdb_self);
 
-	lsa->refresh = thread_add_timer(master, ospf6_lsa_refresh, lsa,
-					OSPF_LS_REFRESH_TIME);
+	lsa->refresh = NULL;
+	thread_add_timer(master, ospf6_lsa_refresh, lsa, OSPF_LS_REFRESH_TIME,
+			 &lsa->refresh);
 
 	if (IS_OSPF6_DEBUG_LSA_TYPE(lsa->header->type)
 	    || IS_OSPF6_DEBUG_ORIGINATE_TYPE(lsa->header->type)) {
@@ -156,6 +156,28 @@ void ospf6_lsa_purge(struct ospf6_lsa *lsa)
 	ospf6_lsa_premature_aging(lsa);
 }
 
+/* Puring Multi Link-State IDs LSAs:
+ * Same Advertising Router with Multiple Link-State IDs
+ * LSAs, purging require to traverse all Link-State IDs
+ */
+void ospf6_lsa_purge_multi_ls_id(struct ospf6_area *oa, struct ospf6_lsa *lsa)
+{
+	int ls_id = 0;
+	struct ospf6_lsa *lsa_next;
+	uint16_t type;
+
+	type = lsa->header->type;
+
+	ospf6_lsa_purge(lsa);
+
+	lsa_next = ospf6_lsdb_lookup(type, htonl(++ls_id),
+				     oa->ospf6->router_id, oa->lsdb);
+	while (lsa_next) {
+		ospf6_lsa_purge(lsa_next);
+		lsa_next = ospf6_lsdb_lookup(type, htonl(++ls_id),
+					     oa->ospf6->router_id, oa->lsdb);
+	}
+}
 
 void ospf6_increment_retrans_count(struct ospf6_lsa *lsa)
 {
@@ -192,10 +214,6 @@ void ospf6_install_lsa(struct ospf6_lsa *lsa)
 	struct timeval now;
 	struct ospf6_lsa *old;
 
-	if (IS_OSPF6_DEBUG_LSA_TYPE(lsa->header->type)
-	    || IS_OSPF6_DEBUG_EXAMIN_TYPE(lsa->header->type))
-		zlog_debug("Install LSA: %s", lsa->name);
-
 	/* Remove the old instance from all neighbors' Link state
 	   retransmission list (RFC2328 13.2 last paragraph) */
 	old = ospf6_lsdb_lookup(lsa->header->type, lsa->header->id,
@@ -207,11 +225,13 @@ void ospf6_install_lsa(struct ospf6_lsa *lsa)
 	}
 
 	monotime(&now);
-	if (!OSPF6_LSA_IS_MAXAGE(lsa))
-		lsa->expire = thread_add_timer(
-			master, ospf6_lsa_expire, lsa,
-			OSPF_LSA_MAXAGE + lsa->birth.tv_sec - now.tv_sec);
-	else
+	if (!OSPF6_LSA_IS_MAXAGE(lsa)) {
+		lsa->expire = NULL;
+		thread_add_timer(master, ospf6_lsa_expire, lsa,
+				 OSPF_LSA_MAXAGE + lsa->birth.tv_sec
+					 - now.tv_sec,
+				 &lsa->expire);
+	} else
 		lsa->expire = NULL;
 
 	if (OSPF6_LSA_IS_SEQWRAP(lsa)
@@ -235,6 +255,12 @@ void ospf6_install_lsa(struct ospf6_lsa *lsa)
 		ospf6_lsa_checksum(lsa->header);
 	}
 
+	if (IS_OSPF6_DEBUG_LSA_TYPE(lsa->header->type)
+	    || IS_OSPF6_DEBUG_EXAMIN_TYPE(lsa->header->type))
+		zlog_debug("%s Install LSA: %s age %d seqnum %x in LSDB.",
+			   __PRETTY_FUNCTION__, lsa->name,
+			   ntohs(lsa->header->age), ntohl(lsa->header->seqnum));
+
 	/* actually install */
 	lsa->installed = now;
 	ospf6_lsdb_add(lsa, lsa->lsdb);
@@ -244,9 +270,8 @@ void ospf6_install_lsa(struct ospf6_lsa *lsa)
 
 /* RFC2740 section 3.5.2. Sending Link State Update packets */
 /* RFC2328 section 13.3 Next step in the flooding procedure */
-static void ospf6_flood_interface(struct ospf6_neighbor *from,
-				  struct ospf6_lsa *lsa,
-				  struct ospf6_interface *oi)
+void ospf6_flood_interface(struct ospf6_neighbor *from, struct ospf6_lsa *lsa,
+			   struct ospf6_interface *oi)
 {
 	struct listnode *node, *nnode;
 	struct ospf6_neighbor *on;
@@ -341,16 +366,25 @@ static void ospf6_flood_interface(struct ospf6_neighbor *from,
 			continue;
 		}
 
-		/* (d) add retrans-list, schedule retransmission */
-		if (is_debug)
-			zlog_debug("Add retrans-list of this neighbor");
-		ospf6_increment_retrans_count(lsa);
-		ospf6_lsdb_add(ospf6_lsa_copy(lsa), on->retrans_list);
-		if (on->thread_send_lsupdate == NULL)
-			on->thread_send_lsupdate = thread_add_timer(
-				master, ospf6_lsupdate_send_neighbor, on,
-				on->ospf6_if->rxmt_interval);
-		retrans_added++;
+		if (ospf6->inst_shutdown) {
+			if (is_debug)
+				zlog_debug(
+					"%s: Send LSA %s (age %d) update now",
+					__PRETTY_FUNCTION__, lsa->name,
+					ntohs(lsa->header->age));
+			ospf6_lsupdate_send_neighbor_now(on, lsa);
+			continue;
+		} else {
+			/* (d) add retrans-list, schedule retransmission */
+			if (is_debug)
+				zlog_debug("Add retrans-list of this neighbor");
+			ospf6_increment_retrans_count(lsa);
+			ospf6_lsdb_add(ospf6_lsa_copy(lsa), on->retrans_list);
+			thread_add_timer(master, ospf6_lsupdate_send_neighbor,
+					 on, on->ospf6_if->rxmt_interval,
+					 &on->thread_send_lsupdate);
+			retrans_added++;
+		}
 	}
 
 	/* (2) examin next interface if not added to retrans-list */
@@ -390,15 +424,15 @@ static void ospf6_flood_interface(struct ospf6_neighbor *from,
 	if ((oi->type == OSPF_IFTYPE_BROADCAST)
 	    || (oi->type == OSPF_IFTYPE_POINTOPOINT)) {
 		ospf6_lsdb_add(ospf6_lsa_copy(lsa), oi->lsupdate_list);
-		if (oi->thread_send_lsupdate == NULL)
-			oi->thread_send_lsupdate = thread_add_event(
-				master, ospf6_lsupdate_send_interface, oi, 0);
+		thread_add_event(master, ospf6_lsupdate_send_interface, oi, 0,
+				 &oi->thread_send_lsupdate);
 	} else {
 		/* reschedule retransmissions to all neighbors */
 		for (ALL_LIST_ELEMENTS(oi->neighbor_list, node, nnode, on)) {
 			THREAD_OFF(on->thread_send_lsupdate);
-			on->thread_send_lsupdate = thread_add_event(
-				master, ospf6_lsupdate_send_neighbor, on, 0);
+			on->thread_send_lsupdate = NULL;
+			thread_add_event(master, ospf6_lsupdate_send_neighbor,
+					 on, 0, &on->thread_send_lsupdate);
 		}
 	}
 }
@@ -547,10 +581,8 @@ static void ospf6_acknowledge_lsa_bdrouter(struct ospf6_lsa *lsa,
 					"Delayed acknowledgement (BDR & MoreRecent & from DR)");
 			/* Delayed acknowledgement */
 			ospf6_lsdb_add(ospf6_lsa_copy(lsa), oi->lsack_list);
-			if (oi->thread_send_lsack == NULL)
-				oi->thread_send_lsack = thread_add_timer(
-					master, ospf6_lsack_send_interface, oi,
-					3);
+			thread_add_timer(master, ospf6_lsack_send_interface, oi,
+					 3, &oi->thread_send_lsack);
 		} else {
 			if (is_debug)
 				zlog_debug(
@@ -570,10 +602,8 @@ static void ospf6_acknowledge_lsa_bdrouter(struct ospf6_lsa *lsa,
 					"Delayed acknowledgement (BDR & Duplicate & ImpliedAck & from DR)");
 			/* Delayed acknowledgement */
 			ospf6_lsdb_add(ospf6_lsa_copy(lsa), oi->lsack_list);
-			if (oi->thread_send_lsack == NULL)
-				oi->thread_send_lsack = thread_add_timer(
-					master, ospf6_lsack_send_interface, oi,
-					3);
+			thread_add_timer(master, ospf6_lsack_send_interface, oi,
+					 3, &oi->thread_send_lsack);
 		} else {
 			if (is_debug)
 				zlog_debug(
@@ -590,9 +620,8 @@ static void ospf6_acknowledge_lsa_bdrouter(struct ospf6_lsa *lsa,
 		if (is_debug)
 			zlog_debug("Direct acknowledgement (BDR & Duplicate)");
 		ospf6_lsdb_add(ospf6_lsa_copy(lsa), from->lsack_list);
-		if (from->thread_send_lsack == NULL)
-			from->thread_send_lsack = thread_add_event(
-				master, ospf6_lsack_send_neighbor, from, 0);
+		thread_add_event(master, ospf6_lsack_send_neighbor, from, 0,
+				 &from->thread_send_lsack);
 		return;
 	}
 
@@ -633,9 +662,8 @@ static void ospf6_acknowledge_lsa_allother(struct ospf6_lsa *lsa,
 				"Delayed acknowledgement (AllOther & MoreRecent)");
 		/* Delayed acknowledgement */
 		ospf6_lsdb_add(ospf6_lsa_copy(lsa), oi->lsack_list);
-		if (oi->thread_send_lsack == NULL)
-			oi->thread_send_lsack = thread_add_timer(
-				master, ospf6_lsack_send_interface, oi, 3);
+		thread_add_timer(master, ospf6_lsack_send_interface, oi, 3,
+				 &oi->thread_send_lsack);
 		return;
 	}
 
@@ -658,9 +686,8 @@ static void ospf6_acknowledge_lsa_allother(struct ospf6_lsa *lsa,
 			zlog_debug(
 				"Direct acknowledgement (AllOther & Duplicate)");
 		ospf6_lsdb_add(ospf6_lsa_copy(lsa), from->lsack_list);
-		if (from->thread_send_lsack == NULL)
-			from->thread_send_lsack = thread_add_event(
-				master, ospf6_lsack_send_neighbor, from, 0);
+		thread_add_event(master, ospf6_lsack_send_neighbor, from, 0,
+				 &from->thread_send_lsack);
 		return;
 	}
 
@@ -793,9 +820,8 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 		/* a) Acknowledge back to neighbor (Direct acknowledgement,
 		 * 13.5) */
 		ospf6_lsdb_add(ospf6_lsa_copy(new), from->lsack_list);
-		if (from->thread_send_lsack == NULL)
-			from->thread_send_lsack = thread_add_event(
-				master, ospf6_lsack_send_neighbor, from, 0);
+		thread_add_event(master, ospf6_lsack_send_neighbor, from, 0,
+				 &from->thread_send_lsack);
 
 		/* b) Discard */
 		ospf6_lsa_delete(new);
@@ -813,6 +839,18 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 				zlog_debug("Received is duplicated LSA");
 			SET_FLAG(new->flag, OSPF6_LSA_DUPLICATE);
 		}
+		if (old->header->adv_router
+			    == from->ospf6_if->area->ospf6->router_id
+		    && OSPF6_LSA_IS_MAXAGE(new)) {
+			ospf6_acknowledge_lsa(new, ismore_recent, from);
+			ospf6_lsa_delete(new);
+			if (is_debug)
+				zlog_debug(
+					"%s: Received is self orig MAXAGE LSA %s, discard (ismore_recent %d)",
+					__PRETTY_FUNCTION__, old->name,
+					ismore_recent);
+			return;
+		}
 	}
 
 	/* if no database copy or received is more recent */
@@ -882,8 +920,9 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 					"Newer instance of the self-originated LSA");
 				zlog_debug("Schedule reorigination");
 			}
-			new->refresh = thread_add_event(
-				master, ospf6_lsa_refresh, new, 0);
+			new->refresh = NULL;
+			thread_add_event(master, ospf6_lsa_refresh, new, 0,
+					 &new->refresh);
 		}
 
 		return;
@@ -902,7 +941,7 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 		}
 
 		/* BadLSReq */
-		thread_add_event(master, bad_lsreq, from, 0);
+		thread_add_event(master, bad_lsreq, from, 0, NULL);
 
 		ospf6_lsa_delete(new);
 		return;
@@ -965,14 +1004,33 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 					"Send back directly and then discard");
 			}
 
+			/* Neighbor router sent recent age for LSA,
+			 * Router could be restarted while current copy is
+			 * MAXAGEd and not removed.*/
+			if (OSPF6_LSA_IS_MAXAGE(old)
+			    && !OSPF6_LSA_IS_MAXAGE(new)) {
+
+				if (is_debug)
+					zlog_debug(
+						"%s: Current copy of LSA %s is MAXAGE, but new has recent Age.",
+						old->name, __PRETTY_FUNCTION__);
+
+				ospf6_lsa_purge(old);
+				if (new->header->adv_router
+				    != from->ospf6_if->area->ospf6->router_id)
+					ospf6_flood(from, new);
+
+				ospf6_install_lsa(new);
+				return;
+			}
+
 			/* XXX, MinLSArrival check !? RFC 2328 13 (8) */
 
 			ospf6_lsdb_add(ospf6_lsa_copy(old),
 				       from->lsupdate_list);
-			if (from->thread_send_lsupdate == NULL)
-				from->thread_send_lsupdate = thread_add_event(
-					master, ospf6_lsupdate_send_neighbor,
-					from, 0);
+			thread_add_event(master, ospf6_lsupdate_send_neighbor,
+					 from, 0, &from->thread_send_lsupdate);
+
 			ospf6_lsa_delete(new);
 			return;
 		}
@@ -980,7 +1038,6 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 	}
 }
 
-
 DEFUN (debug_ospf6_flooding,
        debug_ospf6_flooding_cmd,
        "debug ospf6 flooding",
@@ -1009,7 +1066,7 @@ DEFUN (no_debug_ospf6_flooding,
 int config_write_ospf6_debug_flood(struct vty *vty)
 {
 	if (IS_OSPF6_DEBUG_FLOODING)
-		vty_out(vty, "debug ospf6 flooding%s", VNL);
+		vty_out(vty, "debug ospf6 flooding\n");
 	return 0;
 }
 
