@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -102,7 +101,7 @@ static void ospf6_interface_lsdb_hook_remove(struct ospf6_lsa *lsa)
 	ospf6_interface_lsdb_hook(lsa, ospf6_lsremove_to_spf_reason(lsa));
 }
 
-static u_char ospf6_default_iftype(struct interface *ifp)
+static uint8_t ospf6_default_iftype(struct interface *ifp)
 {
 	if (if_is_pointopoint(ifp))
 		return OSPF_IFTYPE_POINTOPOINT;
@@ -112,21 +111,30 @@ static u_char ospf6_default_iftype(struct interface *ifp)
 		return OSPF_IFTYPE_BROADCAST;
 }
 
-static u_int32_t ospf6_interface_get_cost(struct ospf6_interface *oi)
+static uint32_t ospf6_interface_get_cost(struct ospf6_interface *oi)
 {
 	/* If all else fails, use default OSPF cost */
-	u_int32_t cost;
-	u_int32_t bw, refbw;
+	uint32_t cost;
+	uint32_t bw, refbw;
+
+	/* interface speed and bw can be 0 in some platforms,
+	 * use ospf default bw. If bw is configured then it would
+	 * be used.
+	 */
+	if (!oi->interface->bandwidth && oi->interface->speed) {
+		bw = oi->interface->speed;
+	} else {
+		bw = oi->interface->bandwidth ? oi->interface->bandwidth
+					      : OSPF6_INTERFACE_BANDWIDTH;
+	}
 
-	bw = oi->interface->bandwidth ? oi->interface->bandwidth
-				      : OSPF6_INTERFACE_BANDWIDTH;
 	refbw = ospf6 ? ospf6->ref_bandwidth : OSPF6_REFERENCE_BANDWIDTH;
 
 	/* A specifed ip ospf cost overrides a calculated one. */
 	if (CHECK_FLAG(oi->flag, OSPF6_INTERFACE_NOAUTOCOST))
 		cost = oi->cost;
 	else {
-		cost = (u_int32_t)((double)refbw / (double)bw + (double)0.5);
+		cost = (uint32_t)((double)refbw / (double)bw + (double)0.5);
 		if (cost < 1)
 			cost = 1;
 		else if (cost > UINT32_MAX)
@@ -153,7 +161,7 @@ static void ospf6_interface_force_recalculate_cost(struct ospf6_interface *oi)
 
 static void ospf6_interface_recalculate_cost(struct ospf6_interface *oi)
 {
-	u_int32_t newcost;
+	uint32_t newcost;
 
 	newcost = ospf6_interface_get_cost(oi);
 	if (newcost == oi->cost)
@@ -193,6 +201,7 @@ struct ospf6_interface *ospf6_interface_create(struct interface *ifp)
 	oi->state = OSPF6_INTERFACE_DOWN;
 	oi->flag = 0;
 	oi->mtu_ignore = 0;
+	oi->c_ifmtu = 0;
 
 	/* Try to adjust I/O buffer size with IfMtu */
 	oi->ifmtu = ifp->mtu6;
@@ -238,7 +247,7 @@ void ospf6_interface_delete(struct ospf6_interface *oi)
 	for (ALL_LIST_ELEMENTS(oi->neighbor_list, node, nnode, on))
 		ospf6_neighbor_delete(on);
 
-	list_delete(oi->neighbor_list);
+	list_delete_and_null(&oi->neighbor_list);
 
 	THREAD_OFF(oi->thread_send_hello);
 	THREAD_OFF(oi->thread_send_lsupdate);
@@ -292,6 +301,7 @@ void ospf6_interface_disable(struct ospf6_interface *oi)
 	THREAD_OFF(oi->thread_network_lsa);
 	THREAD_OFF(oi->thread_link_lsa);
 	THREAD_OFF(oi->thread_intra_prefix_lsa);
+	THREAD_OFF(oi->thread_as_extern_lsa);
 }
 
 static struct in6_addr *
@@ -339,31 +349,10 @@ void ospf6_interface_if_add(struct interface *ifp)
 	ospf6_interface_state_update(oi->interface);
 }
 
-void ospf6_interface_if_del(struct interface *ifp)
-{
-	struct ospf6_interface *oi;
-
-	oi = (struct ospf6_interface *)ifp->info;
-	if (oi == NULL)
-		return;
-
-	/* interface stop */
-	if (oi->area)
-		thread_execute(master, interface_down, oi, 0);
-
-	listnode_delete(oi->area->if_list, oi);
-	oi->area = (struct ospf6_area *)NULL;
-
-	/* cut link */
-	oi->interface = NULL;
-	ifp->info = NULL;
-
-	ospf6_interface_delete(oi);
-}
-
 void ospf6_interface_state_update(struct interface *ifp)
 {
 	struct ospf6_interface *oi;
+	unsigned int iobuflen;
 
 	oi = (struct ospf6_interface *)ifp->info;
 	if (oi == NULL)
@@ -373,12 +362,34 @@ void ospf6_interface_state_update(struct interface *ifp)
 	if (CHECK_FLAG(oi->flag, OSPF6_INTERFACE_DISABLE))
 		return;
 
+	/* Adjust the mtu values if the kernel told us something new */
+	if (ifp->mtu6 != oi->ifmtu) {
+		/* If nothing configured, accept it and check for buffer size */
+		if (!oi->c_ifmtu) {
+			oi->ifmtu = ifp->mtu6;
+			iobuflen = ospf6_iobuf_size(ifp->mtu6);
+			if (oi->ifmtu > iobuflen) {
+				if (IS_OSPF6_DEBUG_INTERFACE)
+					zlog_debug(
+						"Interface %s: IfMtu is adjusted to I/O buffer size: %d.",
+						ifp->name, iobuflen);
+				oi->ifmtu = iobuflen;
+			}
+		} else if (oi->c_ifmtu > ifp->mtu6) {
+			oi->ifmtu = ifp->mtu6;
+			zlog_warn(
+				"Configured mtu %u on %s overridden by kernel %u",
+				oi->c_ifmtu, ifp->name, ifp->mtu6);
+		} else
+			oi->ifmtu = oi->c_ifmtu;
+	}
+
 	if (if_is_operative(ifp)
 	    && (ospf6_interface_get_linklocal_address(oi->interface)
 		|| if_is_loopback(oi->interface)))
-		thread_add_event(master, interface_up, oi, 0);
+		thread_add_event(master, interface_up, oi, 0, NULL);
 	else
-		thread_add_event(master, interface_down, oi, 0);
+		thread_add_event(master, interface_down, oi, 0, NULL);
 
 	return;
 }
@@ -461,10 +472,10 @@ void ospf6_interface_connected_route_update(struct interface *ifp)
 	OSPF6_INTRA_PREFIX_LSA_SCHEDULE_STUB(oi->area);
 }
 
-static void ospf6_interface_state_change(u_char next_state,
+static void ospf6_interface_state_change(uint8_t next_state,
 					 struct ospf6_interface *oi)
 {
-	u_char prev_state;
+	uint8_t prev_state;
 
 	prev_state = oi->state;
 	oi->state = next_state;
@@ -500,6 +511,7 @@ static void ospf6_interface_state_change(u_char next_state,
 		OSPF6_NETWORK_LSA_EXECUTE(oi);
 		OSPF6_INTRA_PREFIX_LSA_EXECUTE_TRANSIT(oi);
 		OSPF6_INTRA_PREFIX_LSA_SCHEDULE_STUB(oi->area);
+		OSPF6_INTRA_PREFIX_LSA_EXECUTE_TRANSIT(oi);
 	} else if (prev_state == OSPF6_INTERFACE_DR
 		   || next_state == OSPF6_INTERFACE_DR) {
 		OSPF6_NETWORK_LSA_SCHEDULE(oi);
@@ -576,12 +588,12 @@ static struct ospf6_neighbor *better_drouter(struct ospf6_neighbor *a,
 	return a;
 }
 
-static u_char dr_election(struct ospf6_interface *oi)
+static uint8_t dr_election(struct ospf6_interface *oi)
 {
 	struct listnode *node, *nnode;
 	struct ospf6_neighbor *on, *drouter, *bdrouter, myself;
 	struct ospf6_neighbor *best_drouter, *best_bdrouter;
-	u_char next_state = 0;
+	uint8_t next_state = 0;
 
 	drouter = bdrouter = NULL;
 	best_drouter = best_bdrouter = NULL;
@@ -655,7 +667,7 @@ static u_char dr_election(struct ospf6_interface *oi)
 			if (on->state < OSPF6_NEIGHBOR_TWOWAY)
 				continue;
 			/* Schedule AdjOK. */
-			thread_add_event(master, adj_ok, on, 0);
+			thread_add_event(master, adj_ok, on, 0, NULL);
 		}
 	}
 
@@ -723,7 +735,7 @@ int interface_up(struct thread *thread)
 				"Scheduling %s for sso retry, trial count: %d",
 				oi->interface->name, oi->sso_try_cnt);
 			thread_add_timer(master, interface_up, oi,
-					 OSPF6_INTERFACE_SSO_RETRY_INT);
+					 OSPF6_INTERFACE_SSO_RETRY_INT, NULL);
 		}
 		return 0;
 	}
@@ -734,9 +746,11 @@ int interface_up(struct thread *thread)
 
 	/* Schedule Hello */
 	if (!CHECK_FLAG(oi->flag, OSPF6_INTERFACE_PASSIVE)
-	    && !if_is_loopback(oi->interface))
-		oi->thread_send_hello =
-			thread_add_event(master, ospf6_hello_send, oi, 0);
+	    && !if_is_loopback(oi->interface)) {
+		oi->thread_send_hello = NULL;
+		thread_add_event(master, ospf6_hello_send, oi, 0,
+				 &oi->thread_send_hello);
+	}
 
 	/* decide next interface state */
 	if ((if_is_pointopoint(oi->interface))
@@ -746,7 +760,8 @@ int interface_up(struct thread *thread)
 		ospf6_interface_state_change(OSPF6_INTERFACE_DROTHER, oi);
 	else {
 		ospf6_interface_state_change(OSPF6_INTERFACE_WAITING, oi);
-		thread_add_timer(master, wait_timer, oi, oi->dead_interval);
+		thread_add_timer(master, wait_timer, oi, oi->dead_interval,
+				 NULL);
 	}
 
 	return 0;
@@ -865,58 +880,57 @@ static int ospf6_interface_show(struct vty *vty, struct interface *ifp)
 	else
 		type = "UNKNOWN";
 
-	vty_out(vty, "%s is %s, type %s%s", ifp->name,
-		updown[if_is_operative(ifp)], type, VNL);
-	vty_out(vty, "  Interface ID: %d%s", ifp->ifindex, VNL);
+	vty_out(vty, "%s is %s, type %s\n", ifp->name,
+		updown[if_is_operative(ifp)], type);
+	vty_out(vty, "  Interface ID: %d\n", ifp->ifindex);
 
 	if (ifp->info == NULL) {
-		vty_out(vty, "   OSPF not enabled on this interface%s", VNL);
+		vty_out(vty, "   OSPF not enabled on this interface\n");
 		return 0;
 	} else
 		oi = (struct ospf6_interface *)ifp->info;
 
-	vty_out(vty, "  Internet Address:%s", VNL);
+	vty_out(vty, "  Internet Address:\n");
 
 	for (ALL_LIST_ELEMENTS_RO(ifp->connected, i, c)) {
 		p = c->address;
 		prefix2str(p, strbuf, sizeof(strbuf));
 		switch (p->family) {
 		case AF_INET:
-			vty_out(vty, "    inet : %s%s", strbuf, VNL);
+			vty_out(vty, "    inet : %s\n", strbuf);
 			break;
 		case AF_INET6:
-			vty_out(vty, "    inet6: %s%s", strbuf, VNL);
+			vty_out(vty, "    inet6: %s\n", strbuf);
 			break;
 		default:
-			vty_out(vty, "    ???  : %s%s", strbuf, VNL);
+			vty_out(vty, "    ???  : %s\n", strbuf);
 			break;
 		}
 	}
 
 	if (oi->area) {
 		vty_out(vty,
-			"  Instance ID %d, Interface MTU %d (autodetect: %d)%s",
-			oi->instance_id, oi->ifmtu, ifp->mtu6, VNL);
-		vty_out(vty, "  MTU mismatch detection: %s%s",
-			oi->mtu_ignore ? "disabled" : "enabled", VNL);
+			"  Instance ID %d, Interface MTU %d (autodetect: %d)\n",
+			oi->instance_id, oi->ifmtu, ifp->mtu6);
+		vty_out(vty, "  MTU mismatch detection: %s\n",
+			oi->mtu_ignore ? "disabled" : "enabled");
 		inet_ntop(AF_INET, &oi->area->area_id, strbuf, sizeof(strbuf));
-		vty_out(vty, "  Area ID %s, Cost %u%s", strbuf, oi->cost, VNL);
+		vty_out(vty, "  Area ID %s, Cost %u\n", strbuf, oi->cost);
 	} else
-		vty_out(vty, "  Not Attached to Area%s", VNL);
+		vty_out(vty, "  Not Attached to Area\n");
 
-	vty_out(vty, "  State %s, Transmit Delay %d sec, Priority %d%s",
+	vty_out(vty, "  State %s, Transmit Delay %d sec, Priority %d\n",
 		ospf6_interface_state_str[oi->state], oi->transdelay,
-		oi->priority, VNL);
-	vty_out(vty, "  Timer intervals configured:%s", VNL);
-	vty_out(vty, "   Hello %d, Dead %d, Retransmit %d%s",
-		oi->hello_interval, oi->dead_interval, oi->rxmt_interval, VNL);
+		oi->priority);
+	vty_out(vty, "  Timer intervals configured:\n");
+	vty_out(vty, "   Hello %d, Dead %d, Retransmit %d\n",
+		oi->hello_interval, oi->dead_interval, oi->rxmt_interval);
 
 	inet_ntop(AF_INET, &oi->drouter, drouter, sizeof(drouter));
 	inet_ntop(AF_INET, &oi->bdrouter, bdrouter, sizeof(bdrouter));
-	vty_out(vty, "  DR: %s BDR: %s%s", drouter, bdrouter, VNL);
+	vty_out(vty, "  DR: %s BDR: %s\n", drouter, bdrouter);
 
-	vty_out(vty, "  Number of I/F scoped LSAs is %u%s", oi->lsdb->count,
-		VNL);
+	vty_out(vty, "  Number of I/F scoped LSAs is %u\n", oi->lsdb->count);
 
 	monotime(&now);
 
@@ -925,23 +939,21 @@ static int ospf6_interface_show(struct vty *vty, struct interface *ifp)
 		timersub(&oi->thread_send_lsupdate->u.sands, &now, &res);
 	timerstring(&res, duration, sizeof(duration));
 	vty_out(vty,
-		"    %d Pending LSAs for LSUpdate in Time %s [thread %s]%s",
+		"    %d Pending LSAs for LSUpdate in Time %s [thread %s]\n",
 		oi->lsupdate_list->count, duration,
-		(oi->thread_send_lsupdate ? "on" : "off"), VNL);
-	for (lsa = ospf6_lsdb_head(oi->lsupdate_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa))
-		vty_out(vty, "      %s%s", lsa->name, VNL);
+		(oi->thread_send_lsupdate ? "on" : "off"));
+	for (ALL_LSDB(oi->lsupdate_list, lsa))
+		vty_out(vty, "      %s\n", lsa->name);
 
 	timerclear(&res);
 	if (oi->thread_send_lsack)
 		timersub(&oi->thread_send_lsack->u.sands, &now, &res);
 	timerstring(&res, duration, sizeof(duration));
-	vty_out(vty, "    %d Pending LSAs for LSAck in Time %s [thread %s]%s",
+	vty_out(vty, "    %d Pending LSAs for LSAck in Time %s [thread %s]\n",
 		oi->lsack_list->count, duration,
-		(oi->thread_send_lsack ? "on" : "off"), VNL);
-	for (lsa = ospf6_lsdb_head(oi->lsack_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa))
-		vty_out(vty, "      %s%s", lsa->name, VNL);
+		(oi->thread_send_lsack ? "on" : "off"));
+	for (ALL_LSDB(oi->lsack_list, lsa))
+		vty_out(vty, "      %s\n", lsa->name);
 	ospf6_bfd_show_info(vty, oi->bfd_info, 1);
 	return 0;
 }
@@ -956,26 +968,116 @@ DEFUN (show_ipv6_ospf6_interface,
        INTERFACE_STR
        IFNAME_STR)
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	int idx_ifname = 4;
 	struct interface *ifp;
-	struct listnode *i;
 
 	if (argc == 5) {
 		ifp = if_lookup_by_name(argv[idx_ifname]->arg, VRF_DEFAULT);
 		if (ifp == NULL) {
-			vty_out(vty, "No such Interface: %s%s",
-				argv[idx_ifname]->arg, VNL);
+			vty_out(vty, "No such Interface: %s\n",
+				argv[idx_ifname]->arg);
 			return CMD_WARNING;
 		}
 		ospf6_interface_show(vty, ifp);
 	} else {
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), i, ifp))
+		FOR_ALL_INTERFACES (vrf, ifp)
 			ospf6_interface_show(vty, ifp);
 	}
 
 	return CMD_SUCCESS;
 }
 
+static int ospf6_interface_show_traffic(struct vty *vty, uint32_t vrf_id,
+					struct interface *intf_ifp,
+					int display_once)
+{
+	struct interface *ifp;
+	struct vrf *vrf = NULL;
+	struct ospf6_interface *oi = NULL;
+
+	vrf = vrf_lookup_by_id(vrf_id);
+
+	if (!display_once) {
+		vty_out(vty, "\n");
+		vty_out(vty, "%-12s%-17s%-17s%-17s%-17s%-17s\n", "Interface",
+			"    HELLO", "    DB-Desc", "   LS-Req", "   LS-Update",
+			"   LS-Ack");
+		vty_out(vty, "%-10s%-18s%-18s%-17s%-17s%-17s\n", "",
+			"      Rx/Tx", "     Rx/Tx", "    Rx/Tx", "    Rx/Tx",
+			"    Rx/Tx");
+		vty_out(vty,
+			"--------------------------------------------------------------------------------------------\n");
+	}
+
+	if (intf_ifp == NULL) {
+		FOR_ALL_INTERFACES (vrf, ifp) {
+			if (ifp->info)
+				oi = (struct ospf6_interface *)ifp->info;
+			else
+				continue;
+
+			vty_out(vty,
+				"%-10s %8u/%-8u %7u/%-7u %7u/%-7u %7u/%-7u %7u/%-7u\n",
+				oi->interface->name, oi->hello_in,
+				oi->hello_out, oi->db_desc_in, oi->db_desc_out,
+				oi->ls_req_in, oi->ls_req_out, oi->ls_upd_in,
+				oi->ls_upd_out, oi->ls_ack_in, oi->ls_ack_out);
+		}
+	} else {
+		oi = intf_ifp->info;
+		if (oi == NULL)
+			return CMD_WARNING;
+
+		vty_out(vty,
+			"%-10s %8u/%-8u %7u/%-7u %7u/%-7u %7u/%-7u %7u/%-7u\n",
+			oi->interface->name, oi->hello_in, oi->hello_out,
+			oi->db_desc_in, oi->db_desc_out, oi->ls_req_in,
+			oi->ls_req_out, oi->ls_upd_in, oi->ls_upd_out,
+			oi->ls_ack_in, oi->ls_ack_out);
+	}
+
+	return CMD_SUCCESS;
+}
+
+/* show interface */
+DEFUN (show_ipv6_ospf6_interface_traffic,
+       show_ipv6_ospf6_interface_traffic_cmd,
+       "show ipv6 ospf6 interface traffic [IFNAME]",
+       SHOW_STR
+       IP6_STR
+       OSPF6_STR
+       INTERFACE_STR
+       "Protocol Packet counters\n"
+       IFNAME_STR)
+{
+	int idx_ifname = 0;
+	int display_once = 0;
+	char *intf_name = NULL;
+	struct interface *ifp = NULL;
+
+	if (argv_find(argv, argc, "IFNAME", &idx_ifname)) {
+		intf_name = argv[idx_ifname]->arg;
+		ifp = if_lookup_by_name(intf_name, VRF_DEFAULT);
+		if (ifp == NULL) {
+			vty_out(vty, "No such Interface: %s\n", intf_name);
+			return CMD_WARNING;
+		}
+		if (ifp->info == NULL) {
+			vty_out(vty,
+				"   OSPF not enabled on this interface %s\n",
+				intf_name);
+			return 0;
+		}
+	}
+
+	ospf6_interface_show_traffic(vty, VRF_DEFAULT, ifp, display_once);
+
+
+	return CMD_SUCCESS;
+}
+
+
 DEFUN (show_ipv6_ospf6_interface_ifname_prefix,
        show_ipv6_ospf6_interface_ifname_prefix_cmd,
        "show ipv6 ospf6 interface IFNAME prefix [<X:X::X:X|X:X::X:X/M>] [<match|detail>]",
@@ -997,15 +1099,14 @@ DEFUN (show_ipv6_ospf6_interface_ifname_prefix,
 
 	ifp = if_lookup_by_name(argv[idx_ifname]->arg, VRF_DEFAULT);
 	if (ifp == NULL) {
-		vty_out(vty, "No such Interface: %s%s", argv[idx_ifname]->arg,
-			VNL);
+		vty_out(vty, "No such Interface: %s\n", argv[idx_ifname]->arg);
 		return CMD_WARNING;
 	}
 
 	oi = ifp->info;
 	if (oi == NULL) {
-		vty_out(vty, "OSPFv3 is not enabled on %s%s",
-			argv[idx_ifname]->arg, VNL);
+		vty_out(vty, "OSPFv3 is not enabled on %s\n",
+			argv[idx_ifname]->arg);
 		return CMD_WARNING;
 	}
 
@@ -1028,12 +1129,12 @@ DEFUN (show_ipv6_ospf6_interface_prefix,
        OSPF6_ROUTE_MATCH_STR
        "Display details of the prefixes\n")
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	int idx_prefix = 5;
-	struct listnode *i;
 	struct ospf6_interface *oi;
 	struct interface *ifp;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), i, ifp)) {
+	FOR_ALL_INTERFACES (vrf, ifp) {
 		oi = (struct ospf6_interface *)ifp->info;
 		if (oi == NULL)
 			continue;
@@ -1071,32 +1172,32 @@ DEFUN (ipv6_ospf6_ifmtu,
 
 	ifmtu = strtol(argv[idx_number]->arg, NULL, 10);
 
-	if (oi->ifmtu == ifmtu)
+	if (oi->c_ifmtu == ifmtu)
 		return CMD_SUCCESS;
 
 	if (ifp->mtu6 != 0 && ifp->mtu6 < ifmtu) {
 		vty_out(vty,
-			"%s's ospf6 ifmtu cannot go beyond physical mtu (%d)%s",
-			ifp->name, ifp->mtu6, VNL);
-		return CMD_WARNING;
+			"%s's ospf6 ifmtu cannot go beyond physical mtu (%d)\n",
+			ifp->name, ifp->mtu6);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (oi->ifmtu < ifmtu) {
 		iobuflen = ospf6_iobuf_size(ifmtu);
 		if (iobuflen < ifmtu) {
 			vty_out(vty,
-				"%s's ifmtu is adjusted to I/O buffer size (%d).%s",
-				ifp->name, iobuflen, VNL);
-			oi->ifmtu = iobuflen;
+				"%s's ifmtu is adjusted to I/O buffer size (%d).\n",
+				ifp->name, iobuflen);
+			oi->ifmtu = oi->c_ifmtu = iobuflen;
 		} else
-			oi->ifmtu = ifmtu;
+			oi->ifmtu = oi->c_ifmtu = ifmtu;
 	} else
-		oi->ifmtu = ifmtu;
+		oi->ifmtu = oi->c_ifmtu = ifmtu;
 
 	/* re-establish adjacencies */
 	for (ALL_LIST_ELEMENTS(oi->neighbor_list, node, nnode, on)) {
 		THREAD_OFF(on->inactivity_timer);
-		thread_add_event(master, inactivity_timer, on, 0);
+		thread_add_event(master, inactivity_timer, on, 0, NULL);
 	}
 
 	return CMD_SUCCESS;
@@ -1104,11 +1205,12 @@ DEFUN (ipv6_ospf6_ifmtu,
 
 DEFUN (no_ipv6_ospf6_ifmtu,
        no_ipv6_ospf6_ifmtu_cmd,
-       "no ipv6 ospf6 ifmtu",
+       "no ipv6 ospf6 ifmtu [(1-65535)]",
        NO_STR
        IP6_STR
        OSPF6_STR
        "Interface MTU\n"
+       "OSPFv3 Interface MTU\n"
        )
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
@@ -1128,18 +1230,20 @@ DEFUN (no_ipv6_ospf6_ifmtu,
 		iobuflen = ospf6_iobuf_size(ifp->mtu);
 		if (iobuflen < ifp->mtu) {
 			vty_out(vty,
-				"%s's ifmtu is adjusted to I/O buffer size (%d).%s",
-				ifp->name, iobuflen, VNL);
+				"%s's ifmtu is adjusted to I/O buffer size (%d).\n",
+				ifp->name, iobuflen);
 			oi->ifmtu = iobuflen;
 		} else
 			oi->ifmtu = ifp->mtu;
 	} else
 		oi->ifmtu = ifp->mtu;
 
+	oi->c_ifmtu = 0;
+
 	/* re-establish adjacencies */
 	for (ALL_LIST_ELEMENTS(oi->neighbor_list, node, nnode, on)) {
 		THREAD_OFF(on->inactivity_timer);
-		thread_add_event(master, inactivity_timer, on, 0);
+		thread_add_event(master, inactivity_timer, on, 0, NULL);
 	}
 
 	return CMD_SUCCESS;
@@ -1151,8 +1255,7 @@ DEFUN (ipv6_ospf6_cost,
        IP6_STR
        OSPF6_STR
        "Interface cost\n"
-       "Outgoing metric of this interface\n"
-       )
+       "Outgoing metric of this interface\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	int idx_number = 3;
@@ -1169,8 +1272,8 @@ DEFUN (ipv6_ospf6_cost,
 	lcost = strtol(argv[idx_number]->arg, NULL, 10);
 
 	if (lcost > UINT32_MAX) {
-		vty_out(vty, "Cost %ld is out of range%s", lcost, VNL);
-		return CMD_WARNING;
+		vty_out(vty, "Cost %ld is out of range\n", lcost);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (oi->cost == lcost)
@@ -1186,12 +1289,12 @@ DEFUN (ipv6_ospf6_cost,
 
 DEFUN (no_ipv6_ospf6_cost,
        no_ipv6_ospf6_cost_cmd,
-       "no ipv6 ospf6 cost",
+       "no ipv6 ospf6 cost [(1-65535)]",
        NO_STR
        IP6_STR
        OSPF6_STR
        "Calculate interface cost from bandwidth\n"
-       )
+       "Outgoing metric of this interface\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct ospf6_interface *oi;
@@ -1221,13 +1324,12 @@ DEFUN (auto_cost_reference_bandwidth,
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
 	struct listnode *i, *j;
-	u_int32_t refbw;
+	uint32_t refbw;
 
 	refbw = strtol(argv[idx_number]->arg, NULL, 10);
 	if (refbw < 1 || refbw > 4294967) {
-		vty_out(vty, "reference-bandwidth value is invalid%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "reference-bandwidth value is invalid\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	/* If reference bandwidth is changed. */
@@ -1273,8 +1375,7 @@ DEFUN (ipv6_ospf6_hellointerval,
        IP6_STR
        OSPF6_STR
        "Time between HELLO packets\n"
-       SECONDS_STR
-       )
+       SECONDS_STR)
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	int idx_number = 3;
@@ -1286,10 +1387,21 @@ DEFUN (ipv6_ospf6_hellointerval,
 		oi = ospf6_interface_create(ifp);
 	assert(oi);
 
-	oi->hello_interval = strtol(argv[idx_number]->arg, NULL, 10);
+	oi->hello_interval = strmatch(argv[0]->text, "no")
+				     ? OSPF_HELLO_INTERVAL_DEFAULT
+				     : strtoul(argv[idx_number]->arg, NULL, 10);
 	return CMD_SUCCESS;
 }
 
+ALIAS (ipv6_ospf6_hellointerval,
+       no_ipv6_ospf6_hellointerval_cmd,
+       "no ipv6 ospf6 hello-interval [(1-65535)]",
+       NO_STR
+       IP6_STR
+       OSPF6_STR
+       "Time between HELLO packets\n"
+       SECONDS_STR)
+
 /* interface variable set command */
 DEFUN (ipv6_ospf6_deadinterval,
        ipv6_ospf6_deadinterval_cmd,
@@ -1297,8 +1409,7 @@ DEFUN (ipv6_ospf6_deadinterval,
        IP6_STR
        OSPF6_STR
        "Interval time after which a neighbor is declared down\n"
-       SECONDS_STR
-       )
+       SECONDS_STR)
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	int idx_number = 3;
@@ -1310,10 +1421,21 @@ DEFUN (ipv6_ospf6_deadinterval,
 		oi = ospf6_interface_create(ifp);
 	assert(oi);
 
-	oi->dead_interval = strtol(argv[idx_number]->arg, NULL, 10);
+	oi->dead_interval = strmatch(argv[0]->arg, "no")
+				    ? OSPF_ROUTER_DEAD_INTERVAL_DEFAULT
+				    : strtoul(argv[idx_number]->arg, NULL, 10);
 	return CMD_SUCCESS;
 }
 
+ALIAS (ipv6_ospf6_deadinterval,
+       no_ipv6_ospf6_deadinterval_cmd,
+       "no ipv6 ospf6 dead-interval [(1-65535)]",
+       NO_STR
+       IP6_STR
+       OSPF6_STR
+       "Interval time after which a neighbor is declared down\n"
+       SECONDS_STR)
+
 /* interface variable set command */
 DEFUN (ipv6_ospf6_transmitdelay,
        ipv6_ospf6_transmitdelay_cmd,
@@ -1333,10 +1455,21 @@ DEFUN (ipv6_ospf6_transmitdelay,
 		oi = ospf6_interface_create(ifp);
 	assert(oi);
 
-	oi->transdelay = strtol(argv[idx_number]->arg, NULL, 10);
+	oi->transdelay = strmatch(argv[0]->text, "no")
+				 ? OSPF6_INTERFACE_TRANSDELAY
+				 : strtoul(argv[idx_number]->arg, NULL, 10);
 	return CMD_SUCCESS;
 }
 
+ALIAS (ipv6_ospf6_transmitdelay,
+       no_ipv6_ospf6_transmitdelay_cmd,
+       "no ipv6 ospf6 transmit-delay [(1-3600)]",
+       NO_STR
+       IP6_STR
+       OSPF6_STR
+       "Link state transmit delay\n"
+       SECONDS_STR)
+
 /* interface variable set command */
 DEFUN (ipv6_ospf6_retransmitinterval,
        ipv6_ospf6_retransmitinterval_cmd,
@@ -1344,8 +1477,7 @@ DEFUN (ipv6_ospf6_retransmitinterval,
        IP6_STR
        OSPF6_STR
        "Time between retransmitting lost link state advertisements\n"
-       SECONDS_STR
-       )
+       SECONDS_STR)
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	int idx_number = 3;
@@ -1357,10 +1489,21 @@ DEFUN (ipv6_ospf6_retransmitinterval,
 		oi = ospf6_interface_create(ifp);
 	assert(oi);
 
-	oi->rxmt_interval = strtol(argv[idx_number]->arg, NULL, 10);
+	oi->rxmt_interval = strmatch(argv[0]->text, "no")
+				    ? OSPF_RETRANSMIT_INTERVAL_DEFAULT
+				    : strtoul(argv[idx_number]->arg, NULL, 10);
 	return CMD_SUCCESS;
 }
 
+ALIAS (ipv6_ospf6_retransmitinterval,
+       no_ipv6_ospf6_retransmitinterval_cmd,
+       "no ipv6 ospf6 retransmit-interval [(1-65535)]",
+       NO_STR
+       IP6_STR
+       OSPF6_STR
+       "Time between retransmitting lost link state advertisements\n"
+       SECONDS_STR)
+
 /* interface variable set command */
 DEFUN (ipv6_ospf6_priority,
        ipv6_ospf6_priority_cmd,
@@ -1368,8 +1511,7 @@ DEFUN (ipv6_ospf6_priority,
        IP6_STR
        OSPF6_STR
        "Router priority\n"
-       "Priority value\n"
-       )
+       "Priority value\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	int idx_number = 3;
@@ -1381,7 +1523,9 @@ DEFUN (ipv6_ospf6_priority,
 		oi = ospf6_interface_create(ifp);
 	assert(oi);
 
-	oi->priority = strtol(argv[idx_number]->arg, NULL, 10);
+	oi->priority = strmatch(argv[0]->text, "no")
+			       ? OSPF6_INTERFACE_PRIORITY
+			       : strtoul(argv[idx_number]->arg, NULL, 10);
 
 	if (oi->area && (oi->state == OSPF6_INTERFACE_DROTHER
 			 || oi->state == OSPF6_INTERFACE_BDR
@@ -1391,14 +1535,22 @@ DEFUN (ipv6_ospf6_priority,
 	return CMD_SUCCESS;
 }
 
+ALIAS (ipv6_ospf6_priority,
+       no_ipv6_ospf6_priority_cmd,
+       "no ipv6 ospf6 priority [(0-255)]",
+       NO_STR
+       IP6_STR
+       OSPF6_STR
+       "Router priority\n"
+       "Priority value\n")
+
 DEFUN (ipv6_ospf6_instance,
        ipv6_ospf6_instance_cmd,
        "ipv6 ospf6 instance-id (0-255)",
        IP6_STR
        OSPF6_STR
        "Instance ID for this interface\n"
-       "Instance ID value\n"
-       )
+       "Instance ID value\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	int idx_number = 3;
@@ -1410,10 +1562,21 @@ DEFUN (ipv6_ospf6_instance,
 		oi = ospf6_interface_create(ifp);
 	assert(oi);
 
-	oi->instance_id = strtol(argv[idx_number]->arg, NULL, 10);
+	oi->instance_id = strmatch(argv[0]->text, "no")
+				  ? OSPF6_INTERFACE_INSTANCE_ID
+				  : strtoul(argv[idx_number]->arg, NULL, 10);
 	return CMD_SUCCESS;
 }
 
+ALIAS (ipv6_ospf6_instance,
+       no_ipv6_ospf6_instance_cmd,
+       "no ipv6 ospf6 instance-id [(0-255)]",
+       NO_STR
+       IP6_STR
+       OSPF6_STR
+       "Instance ID for this interface\n"
+       "Instance ID value\n")
+
 DEFUN (ipv6_ospf6_passive,
        ipv6_ospf6_passive_cmd,
        "ipv6 ospf6 passive",
@@ -1439,7 +1602,7 @@ DEFUN (ipv6_ospf6_passive,
 
 	for (ALL_LIST_ELEMENTS(oi->neighbor_list, node, nnode, on)) {
 		THREAD_OFF(on->inactivity_timer);
-		thread_add_event(master, inactivity_timer, on, 0);
+		thread_add_event(master, inactivity_timer, on, 0, NULL);
 	}
 
 	return CMD_SUCCESS;
@@ -1465,8 +1628,9 @@ DEFUN (no_ipv6_ospf6_passive,
 
 	UNSET_FLAG(oi->flag, OSPF6_INTERFACE_PASSIVE);
 	THREAD_OFF(oi->thread_send_hello);
-	oi->thread_send_hello =
-		thread_add_event(master, ospf6_hello_send, oi, 0);
+	oi->thread_send_hello = NULL;
+	thread_add_event(master, ospf6_hello_send, oi, 0,
+			 &oi->thread_send_hello);
 
 	return CMD_SUCCESS;
 }
@@ -1556,13 +1720,13 @@ DEFUN (ipv6_ospf6_advertise_prefix_list,
 
 DEFUN (no_ipv6_ospf6_advertise_prefix_list,
        no_ipv6_ospf6_advertise_prefix_list_cmd,
-       "no ipv6 ospf6 advertise prefix-list",
+       "no ipv6 ospf6 advertise prefix-list [WORD]",
        NO_STR
        IP6_STR
        OSPF6_STR
        "Advertising options\n"
        "Filter prefix using prefix-list\n"
-       )
+       "Prefix list name\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct ospf6_interface *oi;
@@ -1573,10 +1737,8 @@ DEFUN (no_ipv6_ospf6_advertise_prefix_list,
 		oi = ospf6_interface_create(ifp);
 	assert(oi);
 
-	if (oi->plist_name) {
+	if (oi->plist_name)
 		XFREE(MTYPE_CFG_PLIST_NAME, oi->plist_name);
-		oi->plist_name = NULL;
-	}
 
 	ospf6_interface_connected_route_update(oi->interface);
 
@@ -1626,21 +1788,21 @@ DEFUN (ipv6_ospf6_network,
 	}
 
 	/* Reset the interface */
-	thread_add_event(master, interface_down, oi, 0);
-	thread_add_event(master, interface_up, oi, 0);
+	thread_add_event(master, interface_down, oi, 0, NULL);
+	thread_add_event(master, interface_up, oi, 0, NULL);
 
 	return CMD_SUCCESS;
 }
 
 DEFUN (no_ipv6_ospf6_network,
        no_ipv6_ospf6_network_cmd,
-       "no ipv6 ospf6 network",
+       "no ipv6 ospf6 network [<broadcast|point-to-point>]",
        NO_STR
        IP6_STR
        OSPF6_STR
-       "Network type\n"
-       "Default to whatever interface type system specifies"
-       )
+       "Set default network type\n"
+       "Specify OSPF6 broadcast network\n"
+       "Specify OSPF6 point-to-point network\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct ospf6_interface *oi;
@@ -1660,80 +1822,77 @@ DEFUN (no_ipv6_ospf6_network,
 	oi->type = type;
 
 	/* Reset the interface */
-	thread_add_event(master, interface_down, oi, 0);
-	thread_add_event(master, interface_up, oi, 0);
+	thread_add_event(master, interface_down, oi, 0, NULL);
+	thread_add_event(master, interface_up, oi, 0, NULL);
 
 	return CMD_SUCCESS;
 }
 
 static int config_write_ospf6_interface(struct vty *vty)
 {
-	struct listnode *i;
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct ospf6_interface *oi;
 	struct interface *ifp;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), i, ifp)) {
+	FOR_ALL_INTERFACES (vrf, ifp) {
 		oi = (struct ospf6_interface *)ifp->info;
 		if (oi == NULL)
 			continue;
 
-		vty_out(vty, "interface %s%s", oi->interface->name, VNL);
+		vty_frame(vty, "interface %s\n", oi->interface->name);
 
 		if (ifp->desc)
-			vty_out(vty, " description %s%s", ifp->desc, VNL);
-		if (ifp->mtu6 != oi->ifmtu)
-			vty_out(vty, " ipv6 ospf6 ifmtu %d%s", oi->ifmtu, VNL);
+			vty_out(vty, " description %s\n", ifp->desc);
+		if (oi->c_ifmtu)
+			vty_out(vty, " ipv6 ospf6 ifmtu %d\n", oi->c_ifmtu);
 
 		if (CHECK_FLAG(oi->flag, OSPF6_INTERFACE_NOAUTOCOST))
-			vty_out(vty, " ipv6 ospf6 cost %d%s", oi->cost, VNL);
+			vty_out(vty, " ipv6 ospf6 cost %d\n", oi->cost);
 
 		if (oi->hello_interval != OSPF6_INTERFACE_HELLO_INTERVAL)
-			vty_out(vty, " ipv6 ospf6 hello-interval %d%s",
-				oi->hello_interval, VNL);
+			vty_out(vty, " ipv6 ospf6 hello-interval %d\n",
+				oi->hello_interval);
 
 		if (oi->dead_interval != OSPF6_INTERFACE_DEAD_INTERVAL)
-			vty_out(vty, " ipv6 ospf6 dead-interval %d%s",
-				oi->dead_interval, VNL);
+			vty_out(vty, " ipv6 ospf6 dead-interval %d\n",
+				oi->dead_interval);
 
 		if (oi->rxmt_interval != OSPF6_INTERFACE_RXMT_INTERVAL)
-			vty_out(vty, " ipv6 ospf6 retransmit-interval %d%s",
-				oi->rxmt_interval, VNL);
+			vty_out(vty, " ipv6 ospf6 retransmit-interval %d\n",
+				oi->rxmt_interval);
 
 		if (oi->priority != OSPF6_INTERFACE_PRIORITY)
-			vty_out(vty, " ipv6 ospf6 priority %d%s", oi->priority,
-				VNL);
+			vty_out(vty, " ipv6 ospf6 priority %d\n", oi->priority);
 
 		if (oi->transdelay != OSPF6_INTERFACE_TRANSDELAY)
-			vty_out(vty, " ipv6 ospf6 transmit-delay %d%s",
-				oi->transdelay, VNL);
+			vty_out(vty, " ipv6 ospf6 transmit-delay %d\n",
+				oi->transdelay);
 
 		if (oi->instance_id != OSPF6_INTERFACE_INSTANCE_ID)
-			vty_out(vty, " ipv6 ospf6 instance-id %d%s",
-				oi->instance_id, VNL);
+			vty_out(vty, " ipv6 ospf6 instance-id %d\n",
+				oi->instance_id);
 
 		if (oi->plist_name)
-			vty_out(vty, " ipv6 ospf6 advertise prefix-list %s%s",
-				oi->plist_name, VNL);
+			vty_out(vty, " ipv6 ospf6 advertise prefix-list %s\n",
+				oi->plist_name);
 
 		if (CHECK_FLAG(oi->flag, OSPF6_INTERFACE_PASSIVE))
-			vty_out(vty, " ipv6 ospf6 passive%s", VNL);
+			vty_out(vty, " ipv6 ospf6 passive\n");
 
 		if (oi->mtu_ignore)
-			vty_out(vty, " ipv6 ospf6 mtu-ignore%s", VNL);
+			vty_out(vty, " ipv6 ospf6 mtu-ignore\n");
 
 		if (oi->type != ospf6_default_iftype(ifp)) {
 			if (oi->type == OSPF_IFTYPE_POINTOPOINT)
 				vty_out(vty,
-					" ipv6 ospf6 network point-to-point%s",
-					VNL);
+					" ipv6 ospf6 network point-to-point\n");
 			else if (oi->type == OSPF_IFTYPE_BROADCAST)
-				vty_out(vty, " ipv6 ospf6 network broadcast%s",
-					VNL);
+				vty_out(vty, " ipv6 ospf6 network broadcast\n");
 		}
 
 		ospf6_bfd_write_config(vty, oi);
 
-		vty_out(vty, "!%s", VNL);
+		vty_endframe(vty, "!\n");
 	}
 	return 0;
 }
@@ -1752,17 +1911,25 @@ void ospf6_interface_init(void)
 	install_element(VIEW_NODE, &show_ipv6_ospf6_interface_ifname_cmd);
 	install_element(VIEW_NODE,
 			&show_ipv6_ospf6_interface_ifname_prefix_cmd);
+	install_element(VIEW_NODE, &show_ipv6_ospf6_interface_traffic_cmd);
 
 	install_element(INTERFACE_NODE, &ipv6_ospf6_cost_cmd);
 	install_element(INTERFACE_NODE, &no_ipv6_ospf6_cost_cmd);
 	install_element(INTERFACE_NODE, &ipv6_ospf6_ifmtu_cmd);
 	install_element(INTERFACE_NODE, &no_ipv6_ospf6_ifmtu_cmd);
+
 	install_element(INTERFACE_NODE, &ipv6_ospf6_deadinterval_cmd);
 	install_element(INTERFACE_NODE, &ipv6_ospf6_hellointerval_cmd);
 	install_element(INTERFACE_NODE, &ipv6_ospf6_priority_cmd);
 	install_element(INTERFACE_NODE, &ipv6_ospf6_retransmitinterval_cmd);
 	install_element(INTERFACE_NODE, &ipv6_ospf6_transmitdelay_cmd);
 	install_element(INTERFACE_NODE, &ipv6_ospf6_instance_cmd);
+	install_element(INTERFACE_NODE, &no_ipv6_ospf6_deadinterval_cmd);
+	install_element(INTERFACE_NODE, &no_ipv6_ospf6_hellointerval_cmd);
+	install_element(INTERFACE_NODE, &no_ipv6_ospf6_priority_cmd);
+	install_element(INTERFACE_NODE, &no_ipv6_ospf6_retransmitinterval_cmd);
+	install_element(INTERFACE_NODE, &no_ipv6_ospf6_transmitdelay_cmd);
+	install_element(INTERFACE_NODE, &no_ipv6_ospf6_instance_cmd);
 
 	install_element(INTERFACE_NODE, &ipv6_ospf6_passive_cmd);
 	install_element(INTERFACE_NODE, &no_ipv6_ospf6_passive_cmd);
@@ -1799,8 +1966,8 @@ static void ospf6_interface_clear(struct vty *vty, struct interface *ifp)
 		zlog_debug("Interface %s: clear by reset", ifp->name);
 
 	/* Reset the interface */
-	thread_add_event(master, interface_down, oi, 0);
-	thread_add_event(master, interface_up, oi, 0);
+	thread_add_event(master, interface_down, oi, 0, NULL);
+	thread_add_event(master, interface_up, oi, 0, NULL);
 }
 
 /* Clear interface */
@@ -1814,21 +1981,21 @@ DEFUN (clear_ipv6_ospf6_interface,
        IFNAME_STR
        )
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	int idx_ifname = 4;
 	struct interface *ifp;
-	struct listnode *node;
 
 	if (argc == 4) /* Clear all the ospfv3 interfaces. */
 	{
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp))
+		FOR_ALL_INTERFACES (vrf, ifp)
 			ospf6_interface_clear(vty, ifp);
 	} else /* Interface name is specified. */
 	{
 		if ((ifp = if_lookup_by_name(argv[idx_ifname]->arg,
 					     VRF_DEFAULT))
 		    == NULL) {
-			vty_out(vty, "No such Interface: %s%s",
-				argv[idx_ifname]->arg, VNL);
+			vty_out(vty, "No such Interface: %s\n",
+				argv[idx_ifname]->arg);
 			return CMD_WARNING;
 		}
 		ospf6_interface_clear(vty, ifp);
@@ -1870,7 +2037,7 @@ DEFUN (no_debug_ospf6_interface,
 int config_write_ospf6_debug_interface(struct vty *vty)
 {
 	if (IS_OSPF6_DEBUG_INTERFACE)
-		vty_out(vty, "debug ospf6 interface%s", VNL);
+		vty_out(vty, "debug ospf6 interface\n");
 	return 0;
 }
 
