@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -50,9 +49,10 @@
 #include "ospf6_spf.h"
 
 unsigned char conf_debug_ospf6_brouter = 0;
-u_int32_t conf_debug_ospf6_brouter_specific_router_id;
-u_int32_t conf_debug_ospf6_brouter_specific_area_id;
+uint32_t conf_debug_ospf6_brouter_specific_router_id;
+uint32_t conf_debug_ospf6_brouter_specific_area_id;
 
+#define MAX_LSA_PAYLOAD   (1024 + 256)
 /******************************/
 /* RFC2740 3.4.3.1 Router-LSA */
 /******************************/
@@ -105,7 +105,7 @@ static int ospf6_router_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 
 	ospf6_capability_printbuf(router_lsa->bits, bits, sizeof(bits));
 	ospf6_options_printbuf(router_lsa->options, options, sizeof(options));
-	vty_out(vty, "    Bits: %s Options: %s%s", bits, options, VNL);
+	vty_out(vty, "    Bits: %s Options: %s\n", bits, options);
 
 	start = (char *)router_lsa + sizeof(struct ospf6_router_lsa);
 	end = (char *)lsa->header + ntohs(lsa->header->length);
@@ -126,20 +126,17 @@ static int ospf6_router_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 			snprintf(name, sizeof(name), "Unknown (%#x)",
 				 lsdesc->type);
 
-		vty_out(vty, "    Type: %s Metric: %d%s", name,
-			ntohs(lsdesc->metric), VNL);
-		vty_out(vty, "    Interface ID: %s%s",
+		vty_out(vty, "    Type: %s Metric: %d\n", name,
+			ntohs(lsdesc->metric));
+		vty_out(vty, "    Interface ID: %s\n",
 			inet_ntop(AF_INET, &lsdesc->interface_id, buf,
-				  sizeof(buf)),
-			VNL);
-		vty_out(vty, "    Neighbor Interface ID: %s%s",
+				  sizeof(buf)));
+		vty_out(vty, "    Neighbor Interface ID: %s\n",
 			inet_ntop(AF_INET, &lsdesc->neighbor_interface_id, buf,
-				  sizeof(buf)),
-			VNL);
-		vty_out(vty, "    Neighbor Router ID: %s%s",
+				  sizeof(buf)));
+		vty_out(vty, "    Neighbor Router ID: %s\n",
 			inet_ntop(AF_INET, &lsdesc->neighbor_router_id, buf,
-				  sizeof(buf)),
-			VNL);
+				  sizeof(buf)));
 	}
 	return 0;
 }
@@ -192,15 +189,15 @@ int ospf6_router_lsa_originate(struct thread *thread)
 	struct ospf6_lsa_header *lsa_header;
 	struct ospf6_lsa *lsa;
 
-	u_int32_t link_state_id = 0;
+	uint32_t link_state_id = 0;
 	struct listnode *node, *nnode;
 	struct listnode *j;
 	struct ospf6_interface *oi;
 	struct ospf6_neighbor *on, *drouter = NULL;
 	struct ospf6_router_lsa *router_lsa;
 	struct ospf6_router_lsdesc *lsdesc;
-	u_int16_t type;
-	u_int32_t router;
+	uint16_t type;
+	uint32_t router;
 	int count;
 
 	oa = (struct ospf6_area *)THREAD_ARG(thread);
@@ -252,6 +249,41 @@ int ospf6_router_lsa_originate(struct thread *thread)
 				return 0;
 			}
 
+			/* Fill LSA Header */
+			lsa_header->age = 0;
+			lsa_header->type = htons(OSPF6_LSTYPE_ROUTER);
+			lsa_header->id = htonl(link_state_id);
+			lsa_header->adv_router = oa->ospf6->router_id;
+			lsa_header->seqnum = ospf6_new_ls_seqnum(
+				lsa_header->type, lsa_header->id,
+				lsa_header->adv_router, oa->lsdb);
+			lsa_header->length =
+				htons((caddr_t)lsdesc - (caddr_t)buffer);
+
+			/* LSA checksum */
+			ospf6_lsa_checksum(lsa_header);
+
+			/* create LSA */
+			lsa = ospf6_lsa_create(lsa_header);
+
+			/* Originate */
+			ospf6_lsa_originate_area(lsa, oa);
+
+			/* Reset Buffer to fill next Router LSA */
+			memset(buffer, 0, sizeof(buffer));
+			lsa_header = (struct ospf6_lsa_header *)buffer;
+			router_lsa =
+				(struct ospf6_router_lsa
+					 *)((caddr_t)lsa_header
+					    + sizeof(struct ospf6_lsa_header));
+
+			ospf6_router_lsa_options_set(oa, router_lsa);
+
+			/* describe links for each interfaces */
+			lsdesc = (struct ospf6_router_lsdesc
+					  *)((caddr_t)router_lsa
+					     + sizeof(struct ospf6_router_lsa));
+
 			link_state_id++;
 		}
 
@@ -336,8 +368,7 @@ int ospf6_router_lsa_originate(struct thread *thread)
 	type = ntohs(OSPF6_LSTYPE_ROUTER);
 	router = oa->ospf6->router_id;
 	count = 0;
-	for (lsa = ospf6_lsdb_type_router_head(type, router, oa->lsdb); lsa;
-	     lsa = ospf6_lsdb_type_router_next(type, router, lsa)) {
+	for (ALL_LSDB_TYPED_ADVRTR(oa->lsdb, type, router, lsa)) {
 		if (ntohl(lsa->header->id) < link_state_id)
 			continue;
 		ospf6_lsa_purge(lsa);
@@ -402,7 +433,7 @@ static int ospf6_network_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 					     + sizeof(struct ospf6_lsa_header));
 
 	ospf6_options_printbuf(network_lsa->options, options, sizeof(options));
-	vty_out(vty, "     Options: %s%s", options, VNL);
+	vty_out(vty, "     Options: %s\n", options);
 
 	start = (char *)network_lsa + sizeof(struct ospf6_network_lsa);
 	end = (char *)lsa->header + ntohs(lsa->header->length);
@@ -411,7 +442,7 @@ static int ospf6_network_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 	     current += sizeof(struct ospf6_network_lsdesc)) {
 		lsdesc = (struct ospf6_network_lsdesc *)current;
 		inet_ntop(AF_INET, &lsdesc->router_id, buf, sizeof(buf));
-		vty_out(vty, "     Attached Router: %s%s", buf, VNL);
+		vty_out(vty, "     Attached Router: %s\n", buf);
 	}
 	return 0;
 }
@@ -430,7 +461,7 @@ int ospf6_network_lsa_originate(struct thread *thread)
 	struct ospf6_neighbor *on;
 	struct ospf6_link_lsa *link_lsa;
 	struct listnode *i;
-	u_int16_t type;
+	uint16_t type;
 
 	oi = (struct ospf6_interface *)THREAD_ARG(thread);
 	oi->thread_network_lsa = NULL;
@@ -489,8 +520,7 @@ int ospf6_network_lsa_originate(struct thread *thread)
 	/* Collect the interface's Link-LSAs to describe
 	   network's optional capabilities */
 	type = htons(OSPF6_LSTYPE_LINK);
-	for (lsa = ospf6_lsdb_type_head(type, oi->lsdb); lsa;
-	     lsa = ospf6_lsdb_type_next(type, lsa)) {
+	for (ALL_LSDB_TYPED(oi->lsdb, type, lsa)) {
 		link_lsa = (struct ospf6_link_lsa
 				    *)((caddr_t)lsa->header
 				       + sizeof(struct ospf6_lsa_header));
@@ -613,10 +643,10 @@ static int ospf6_link_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 	inet_ntop(AF_INET6, &link_lsa->linklocal_addr, buf, sizeof(buf));
 	prefixnum = ntohl(link_lsa->prefix_num);
 
-	vty_out(vty, "     Priority: %d Options: %s%s", link_lsa->priority,
-		options, VNL);
-	vty_out(vty, "     LinkLocal Address: %s%s", buf, VNL);
-	vty_out(vty, "     Number of Prefix: %d%s", prefixnum, VNL);
+	vty_out(vty, "     Priority: %d Options: %s\n", link_lsa->priority,
+		options);
+	vty_out(vty, "     LinkLocal Address: %s\n", buf);
+	vty_out(vty, "     Number of Prefix: %d\n", prefixnum);
 
 	start = (char *)link_lsa + sizeof(struct ospf6_link_lsa);
 	end = (char *)lsa->header + ntohs(lsa->header->length);
@@ -639,15 +669,15 @@ static int ospf6_link_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 		nu = (CHECK_FLAG(prefix->prefix_options, OSPF6_PREFIX_OPTION_NU)
 			      ? "NU"
 			      : "--");
-		vty_out(vty, "     Prefix Options: %s|%s|%s|%s%s", p, mc, la,
-			nu, VNL);
+		vty_out(vty, "     Prefix Options: %s|%s|%s|%s\n", p, mc, la,
+			nu);
 
 		memset(&in6, 0, sizeof(in6));
 		memcpy(&in6, OSPF6_PREFIX_BODY(prefix),
 		       OSPF6_PREFIX_SPACE(prefix->prefix_length));
 		inet_ntop(AF_INET6, &in6, buf, sizeof(buf));
-		vty_out(vty, "     Prefix: %s/%d%s", buf, prefix->prefix_length,
-			VNL);
+		vty_out(vty, "     Prefix: %s/%d\n", buf,
+			prefix->prefix_length);
 	}
 
 	return 0;
@@ -817,14 +847,13 @@ static int ospf6_intra_prefix_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 
 	prefixnum = ntohs(intra_prefix_lsa->prefix_num);
 
-	vty_out(vty, "     Number of Prefix: %d%s", prefixnum, VNL);
+	vty_out(vty, "     Number of Prefix: %d\n", prefixnum);
 
 	inet_ntop(AF_INET, &intra_prefix_lsa->ref_id, id, sizeof(id));
 	inet_ntop(AF_INET, &intra_prefix_lsa->ref_adv_router, adv_router,
 		  sizeof(adv_router));
-	vty_out(vty, "     Reference: %s Id: %s Adv: %s%s",
-		ospf6_lstype_name(intra_prefix_lsa->ref_type), id, adv_router,
-		VNL);
+	vty_out(vty, "     Reference: %s Id: %s Adv: %s\n",
+		ospf6_lstype_name(intra_prefix_lsa->ref_type), id, adv_router);
 
 	start = (char *)intra_prefix_lsa
 		+ sizeof(struct ospf6_intra_prefix_lsa);
@@ -848,15 +877,15 @@ static int ospf6_intra_prefix_lsa_show(struct vty *vty, struct ospf6_lsa *lsa)
 		nu = (CHECK_FLAG(prefix->prefix_options, OSPF6_PREFIX_OPTION_NU)
 			      ? "NU"
 			      : "--");
-		vty_out(vty, "     Prefix Options: %s|%s|%s|%s%s", p, mc, la,
-			nu, VNL);
+		vty_out(vty, "     Prefix Options: %s|%s|%s|%s\n", p, mc, la,
+			nu);
 
 		memset(&in6, 0, sizeof(in6));
 		memcpy(&in6, OSPF6_PREFIX_BODY(prefix),
 		       OSPF6_PREFIX_SPACE(prefix->prefix_length));
 		inet_ntop(AF_INET6, &in6, buf, sizeof(buf));
-		vty_out(vty, "     Prefix: %s/%d%s", buf, prefix->prefix_length,
-			VNL);
+		vty_out(vty, "     Prefix: %s/%d\n", buf,
+			prefix->prefix_length);
 	}
 
 	return 0;
@@ -868,7 +897,7 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 
 	char buffer[OSPF6_MAX_LSASIZE];
 	struct ospf6_lsa_header *lsa_header;
-	struct ospf6_lsa *old, *lsa;
+	struct ospf6_lsa *old, *lsa, *old_next = NULL;
 
 	struct ospf6_intra_prefix_lsa *intra_prefix_lsa;
 	struct ospf6_interface *oi;
@@ -880,6 +909,7 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 	unsigned short prefix_num = 0;
 	char buf[PREFIX2STR_BUFFER];
 	struct ospf6_route_table *route_advertise;
+	int ls_id = 0;
 
 	oa = (struct ospf6_area *)THREAD_ARG(thread);
 	oa->thread_intra_prefix_lsa = NULL;
@@ -889,8 +919,21 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 				oa->ospf6->router_id, oa->lsdb);
 
 	if (!IS_AREA_ENABLED(oa)) {
-		if (old)
+		if (old) {
 			ospf6_lsa_purge(old);
+			/* find previous LSA */
+			old_next = ospf6_lsdb_lookup(
+				htons(OSPF6_LSTYPE_INTRA_PREFIX),
+				htonl(++ls_id), oa->ospf6->router_id, oa->lsdb);
+
+			while (old_next) {
+				ospf6_lsa_purge(old_next);
+				old_next = ospf6_lsdb_lookup(
+					htons(OSPF6_LSTYPE_INTRA_PREFIX),
+					htonl(++ls_id), oa->ospf6->router_id,
+					oa->lsdb);
+			}
+		}
 		return 0;
 	}
 
@@ -952,25 +995,99 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 	}
 
 	if (route_advertise->count == 0) {
-		if (old)
+		if (old) {
+			ls_id = 0;
 			ospf6_lsa_purge(old);
+			/* find previous LSA */
+			old_next = ospf6_lsdb_lookup(
+				htons(OSPF6_LSTYPE_INTRA_PREFIX),
+				htonl(++ls_id), oa->ospf6->router_id, oa->lsdb);
+
+			while (old_next) {
+				ospf6_lsa_purge(old_next);
+				old_next = ospf6_lsdb_lookup(
+					htons(OSPF6_LSTYPE_INTRA_PREFIX),
+					htonl(++ls_id), oa->ospf6->router_id,
+					oa->lsdb);
+			}
+		}
 		ospf6_route_table_delete(route_advertise);
 		return 0;
 	}
 
+	/* Neighbor change to FULL, if INTRA-AREA-PREFIX LSA
+	 * has not change, Flush old LSA and Re-Originate INP,
+	 * as ospf6_flood() checks if LSA is same as DB,
+	 * it won't be updated to neighbor's DB.
+	 */
+	if (oa->intra_prefix_originate) {
+		if (IS_OSPF6_DEBUG_ORIGINATE(INTRA_PREFIX))
+			zlog_debug("%s: Re-originate intra prefix LSA, Current full nbrs %u",
+				   __PRETTY_FUNCTION__, oa->full_nbrs);
+		if (old)
+			ospf6_lsa_purge_multi_ls_id(oa, old);
+		oa->intra_prefix_originate = 0;
+	}
+
 	/* put prefixes to advertise */
 	prefix_num = 0;
 	op = (struct ospf6_prefix *)((caddr_t)intra_prefix_lsa
 				     + sizeof(struct ospf6_intra_prefix_lsa));
 	for (route = ospf6_route_head(route_advertise); route;
 	     route = ospf6_route_best_next(route)) {
+		if (((caddr_t)op - (caddr_t)lsa_header) > MAX_LSA_PAYLOAD) {
+
+			intra_prefix_lsa->prefix_num = htons(prefix_num);
+
+			/* Fill LSA Header */
+			lsa_header->age = 0;
+			lsa_header->type = htons(OSPF6_LSTYPE_INTRA_PREFIX);
+			lsa_header->id = htonl(ls_id++);
+			lsa_header->adv_router = oa->ospf6->router_id;
+			lsa_header->seqnum = ospf6_new_ls_seqnum(
+				lsa_header->type, lsa_header->id,
+				lsa_header->adv_router, oa->lsdb);
+			lsa_header->length =
+				htons((caddr_t)op - (caddr_t)lsa_header);
+
+			/* LSA checksum */
+			ospf6_lsa_checksum(lsa_header);
+
+			/* Create LSA */
+			lsa = ospf6_lsa_create(lsa_header);
+
+			/* Originate */
+			ospf6_lsa_originate_area(lsa, oa);
+
+			/* Prepare next buffer */
+			memset(buffer, 0, sizeof(buffer));
+			lsa_header = (struct ospf6_lsa_header *)buffer;
+			intra_prefix_lsa =
+				(struct ospf6_intra_prefix_lsa
+					 *)((caddr_t)lsa_header
+					    + sizeof(struct ospf6_lsa_header));
+
+			/* Fill Intra-Area-Prefix-LSA */
+			intra_prefix_lsa->ref_type = htons(OSPF6_LSTYPE_ROUTER);
+			intra_prefix_lsa->ref_id = htonl(0);
+			intra_prefix_lsa->ref_adv_router = oa->ospf6->router_id;
+
+			/* Put next set of prefixes to advertise */
+			prefix_num = 0;
+			op = (struct ospf6_prefix
+				      *)((caddr_t)intra_prefix_lsa
+					 + sizeof(struct
+						  ospf6_intra_prefix_lsa));
+		}
+
 		op->prefix_length = route->prefix.prefixlen;
 		op->prefix_options = route->path.prefix_options;
 		op->prefix_metric = htons(route->path.cost);
 		memcpy(OSPF6_PREFIX_BODY(op), &route->prefix.u.prefix6,
 		       OSPF6_PREFIX_SPACE(op->prefix_length));
-		op = OSPF6_PREFIX_NEXT(op);
 		prefix_num++;
+
+		op = OSPF6_PREFIX_NEXT(op);
 	}
 
 	ospf6_route_table_delete(route_advertise);
@@ -987,7 +1104,7 @@ int ospf6_intra_prefix_lsa_originate_stub(struct thread *thread)
 	/* Fill LSA Header */
 	lsa_header->age = 0;
 	lsa_header->type = htons(OSPF6_LSTYPE_INTRA_PREFIX);
-	lsa_header->id = htonl(0);
+	lsa_header->id = htonl(ls_id++);
 	lsa_header->adv_router = oa->ospf6->router_id;
 	lsa_header->seqnum =
 		ospf6_new_ls_seqnum(lsa_header->type, lsa_header->id,
@@ -1025,7 +1142,7 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 	struct ospf6_route_table *route_advertise;
 	struct ospf6_link_lsa *link_lsa;
 	char *start, *end, *current;
-	u_int16_t type;
+	uint16_t type;
 	char buf[PREFIX2STR_BUFFER];
 
 	oi = (struct ospf6_interface *)THREAD_ARG(thread);
@@ -1086,8 +1203,7 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 	route_advertise = ospf6_route_table_create(0, 0);
 
 	type = ntohs(OSPF6_LSTYPE_LINK);
-	for (lsa = ospf6_lsdb_type_head(type, oi->lsdb); lsa;
-	     lsa = ospf6_lsdb_type_next(type, lsa)) {
+	for (ALL_LSDB_TYPED(oi->lsdb, type, lsa)) {
 		if (OSPF6_LSA_IS_MAXAGE(lsa))
 			continue;
 
@@ -1198,24 +1314,259 @@ int ospf6_intra_prefix_lsa_originate_transit(struct thread *thread)
 	return 0;
 }
 
+void ospf6_intra_prefix_route_ecmp_path(struct ospf6_area *oa,
+					struct ospf6_route *old,
+					struct ospf6_route *route)
+{
+	struct ospf6_route *old_route;
+	struct ospf6_path *ecmp_path, *o_path = NULL;
+	struct listnode *anode, *anext;
+	struct listnode *nnode, *rnode, *rnext;
+	struct ospf6_nexthop *nh, *rnh;
+	char buf[PREFIX2STR_BUFFER];
+	bool route_found = false;
+
+	/* check for old entry match with new route origin,
+	 * delete old entry.
+	 */
+	for (old_route = old; old_route; old_route = old_route->next) {
+		bool route_updated = false;
+
+		if (!ospf6_route_is_same(old_route, route) ||
+			(old_route->path.type != route->path.type))
+			continue;
+
+		/* Current and New route has same origin,
+		 * delete old entry.
+		 */
+		for (ALL_LIST_ELEMENTS(old_route->paths, anode, anext,
+						  o_path)) {
+			/* Check old route path and route has same
+			 * origin.
+			 */
+			if (o_path->area_id != route->path.area_id ||
+			    (memcmp(&(o_path)->origin, &(route)->path.origin,
+				   sizeof(struct ospf6_ls_origin)) != 0))
+				continue;
+
+			/* Cost is not same then delete current path */
+			if (o_path->cost == route->path.cost)
+				continue;
+
+			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
+				prefix2str(&old_route->prefix, buf,
+					   sizeof(buf));
+				zlog_debug("%s: route %s cost old %u new %u is not same, replace route",
+					   __PRETTY_FUNCTION__, buf,
+					   o_path->cost, route->path.cost);
+			}
+
+			/* Remove selected current rout path's nh from
+			 * effective nh list.
+			 */
+			for (ALL_LIST_ELEMENTS_RO(o_path->nh_list, nnode, nh)) {
+				for (ALL_LIST_ELEMENTS(old_route->nh_list,
+							rnode, rnext, rnh)) {
+					if (!ospf6_nexthop_is_same(rnh, nh))
+						continue;
+					listnode_delete(old_route->nh_list,
+								rnh);
+					ospf6_nexthop_delete(rnh);
+					route_updated = true;
+				}
+			}
+
+			listnode_delete(old_route->paths, o_path);
+			ospf6_path_free(o_path);
+
+			/* Current route's path (adv_router info) is similar
+			 * to route being added.
+			 * Replace current route's path with paths list head.
+			 * Update FIB with effective NHs.
+			 */
+			if (listcount(old_route->paths)) {
+				if (old_route->path.origin.id ==
+					route->path.origin.id &&
+					old_route->path.origin.adv_router ==
+						route->path.origin.adv_router) {
+					struct ospf6_path *h_path;
+
+					h_path = (struct ospf6_path *)
+					listgetdata(listhead(old_route->paths));
+					old_route->path.origin.type =
+						h_path->origin.type;
+					old_route->path.origin.id =
+						h_path->origin.id;
+					old_route->path.origin.adv_router =
+						h_path->origin.adv_router;
+				}
+
+				if (route_updated) {
+					for (ALL_LIST_ELEMENTS(old_route->paths,
+							anode, anext, o_path)) {
+						ospf6_merge_nexthops(
+							old_route->nh_list,
+							o_path->nh_list);
+					}
+					/* Update ospf6 route table and
+					 * RIB/FIB with effective
+					 * nh_list
+					 */
+					if (oa->route_table->hook_add)
+						(*oa->route_table->hook_add)
+							(old_route);
+					break;
+				}
+			} else {
+				if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
+					prefix2str(&old_route->prefix, buf,
+						   sizeof(buf));
+					zlog_debug("%s: route %s old cost %u new cost %u, delete old entry.",
+						   __PRETTY_FUNCTION__, buf,
+						   old_route->path.cost,
+						   route->path.cost);
+				}
+				ospf6_route_remove(old_route,
+						   oa->route_table);
+				break;
+			}
+		}
+		if (route_updated)
+			break;
+	}
+
+	for (old_route = old; old_route; old_route = old_route->next) {
+
+		if (!ospf6_route_is_same(old_route, route) ||
+			(old_route->path.type != route->path.type))
+			continue;
+
+		/* Old Route and New Route have Equal Cost, Merge NHs */
+		if (old_route->path.cost == route->path.cost) {
+			route_found = true;
+
+			/* check if this path exists already in
+			 * route->paths list, if so, replace nh_list.
+			 */
+			for (ALL_LIST_ELEMENTS_RO(old_route->paths, anode,
+						  o_path)) {
+				if (o_path->area_id == route->path.area_id &&
+				    (memcmp(&(o_path)->origin,
+					&(route)->path.origin,
+					sizeof(struct ospf6_ls_origin)) == 0))
+					break;
+			}
+			/* If path is not found in old_route paths's list,
+			 * add a new path to route paths list and merge
+			 * nexthops in route->path->nh_list.
+			 * Otherwise replace existing path's nh_list.
+			 */
+			if (o_path == NULL) {
+				ecmp_path = ospf6_path_dup(&route->path);
+
+				/* Add a nh_list to new ecmp path */
+				ospf6_copy_nexthops(ecmp_path->nh_list,
+						    route->nh_list);
+				/* Merge nexthop to existing route's nh_list */
+				ospf6_route_merge_nexthops(old_route, route);
+				/* Add the new path to route's path list */
+				listnode_add_sort(old_route->paths, ecmp_path);
+
+				UNSET_FLAG(old_route->flag, OSPF6_ROUTE_REMOVE);
+				SET_FLAG(old_route->flag, OSPF6_ROUTE_CHANGE);
+				/* Update RIB/FIB */
+				if (oa->route_table->hook_add)
+					(*oa->route_table->hook_add)
+						(old_route);
+				if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
+					prefix2str(&route->prefix, buf,
+						   sizeof(buf));
+					zlog_debug("%s: route %s %p another path added with nh %u, effective paths %u nh %u",
+						__PRETTY_FUNCTION__, buf,
+						(void *)old_route,
+						listcount(ecmp_path->nh_list),
+						old_route->paths ?
+						listcount(old_route->paths)
+						: 0,
+						listcount(old_route->nh_list));
+				}
+			} else {
+				for (ALL_LIST_ELEMENTS_RO(o_path->nh_list,
+							  nnode, nh)) {
+					for (ALL_LIST_ELEMENTS(
+							old_route->nh_list,
+							rnode, rnext, rnh)) {
+						if (!ospf6_nexthop_is_same(rnh,
+									   nh))
+							continue;
+
+						listnode_delete(
+							old_route->nh_list,
+							rnh);
+						ospf6_nexthop_delete(rnh);
+					}
+				}
+				list_delete_all_node(o_path->nh_list);
+				ospf6_copy_nexthops(o_path->nh_list,
+					    route->nh_list);
+
+				/* Merge nexthop to existing route's nh_list */
+				ospf6_route_merge_nexthops(old_route,
+							   route);
+
+				if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
+					prefix2str(&route->prefix,
+						   buf, sizeof(buf));
+					zlog_debug("%s: existing route %s %p with effective paths %u nh count %u",
+						   __PRETTY_FUNCTION__, buf,
+						   (void *)old_route,
+						   listcount(old_route->paths),
+						   old_route->nh_list ?
+						   listcount(old_route->nh_list)
+						   : 0);
+				}
+
+				UNSET_FLAG(old_route->flag, OSPF6_ROUTE_REMOVE);
+				SET_FLAG(old_route->flag, OSPF6_ROUTE_CHANGE);
+				/* Update ospf6 route table and RIB/FIB */
+				if (oa->route_table->hook_add)
+					(*oa->route_table->hook_add)
+						(old_route);
+			}
+			/* Delete the new route its info added to existing
+			 * route.
+			 */
+			ospf6_route_delete(route);
+
+			break;
+		}
+	}
+
+	if (!route_found) {
+		/* Add new route to existing node in ospf6 route table. */
+		ospf6_route_add(route, oa->route_table);
+	}
+}
+
 void ospf6_intra_prefix_lsa_add(struct ospf6_lsa *lsa)
 {
 	struct ospf6_area *oa;
 	struct ospf6_intra_prefix_lsa *intra_prefix_lsa;
 	struct prefix ls_prefix;
-	struct ospf6_route *route, *ls_entry;
+	struct ospf6_route *route, *ls_entry, *old;
 	int prefix_num;
 	struct ospf6_prefix *op;
 	char *start, *current, *end;
 	char buf[PREFIX2STR_BUFFER];
 	struct interface *ifp;
 	int direct_connect = 0;
+	struct ospf6_path *path;
 
 	if (OSPF6_LSA_IS_MAXAGE(lsa))
 		return;
 
 	if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX))
-		zlog_debug("%s found", lsa->name);
+		zlog_debug("%s: LSA %s found", __PRETTY_FUNCTION__, lsa->name);
 
 	oa = OSPF6_AREA(lsa->lsdb->data);
 
@@ -1224,7 +1575,7 @@ void ospf6_intra_prefix_lsa_add(struct ospf6_lsa *lsa)
 			lsa->header);
 	if (intra_prefix_lsa->ref_type == htons(OSPF6_LSTYPE_ROUTER))
 		ospf6_linkstate_prefix(intra_prefix_lsa->ref_adv_router,
-				       htonl(0), &ls_prefix);
+				       intra_prefix_lsa->ref_id, &ls_prefix);
 	else if (intra_prefix_lsa->ref_type == htons(OSPF6_LSTYPE_NETWORK))
 		ospf6_linkstate_prefix(intra_prefix_lsa->ref_adv_router,
 				       intra_prefix_lsa->ref_id, &ls_prefix);
@@ -1301,12 +1652,31 @@ void ospf6_intra_prefix_lsa_add(struct ospf6_lsa *lsa)
 			ospf6_route_copy_nexthops(route, ls_entry);
 		}
 
-		if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
-			prefix2str(&route->prefix, buf, sizeof(buf));
-			zlog_debug("  add %s", buf);
-		}
+		path = ospf6_path_dup(&route->path);
+		ospf6_copy_nexthops(path->nh_list, route->path.nh_list);
+		listnode_add_sort(route->paths, path);
 
-		ospf6_route_add(route, oa->route_table);
+		old = ospf6_route_lookup(&route->prefix, oa->route_table);
+		if (old && (ospf6_route_cmp(route, old) == 0)) {
+			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
+				prefix2str(&route->prefix, buf, sizeof(buf));
+				zlog_debug(" Update route: %s old cost %u new cost %u nh count %u paths %u",
+					   buf,
+					   old->path.cost, route->path.cost,
+					   listcount(route->nh_list),
+					   listcount(route->paths));
+			}
+			ospf6_intra_prefix_route_ecmp_path(oa, old, route);
+		} else {
+			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
+				prefix2str(&route->prefix, buf, sizeof(buf));
+				zlog_debug("  route %s add with cost %u nh %u paths %u",
+					   buf, route->path.cost,
+					   listcount(route->nh_list),
+					   listcount(route->paths));
+			}
+			ospf6_route_add(route, oa->route_table);
+		}
 		prefix_num--;
 	}
 
@@ -1319,14 +1689,15 @@ void ospf6_intra_prefix_lsa_remove(struct ospf6_lsa *lsa)
 	struct ospf6_area *oa;
 	struct ospf6_intra_prefix_lsa *intra_prefix_lsa;
 	struct prefix prefix;
-	struct ospf6_route *route, *nroute;
+	struct ospf6_route *route, *nroute, *route_to_del;
 	int prefix_num;
 	struct ospf6_prefix *op;
 	char *start, *current, *end;
 	char buf[PREFIX2STR_BUFFER];
 
 	if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX))
-		zlog_debug("%s disappearing", lsa->name);
+		zlog_debug("%s: %s disappearing", __PRETTY_FUNCTION__,
+			   lsa->name);
 
 	oa = OSPF6_AREA(lsa->lsdb->data);
 
@@ -1346,6 +1717,22 @@ void ospf6_intra_prefix_lsa_remove(struct ospf6_lsa *lsa)
 			break;
 		prefix_num--;
 
+		route_to_del = ospf6_route_create();
+
+		memset(&route_to_del->prefix, 0, sizeof(struct prefix));
+		route_to_del->prefix.family = AF_INET6;
+		route_to_del->prefix.prefixlen = op->prefix_length;
+		ospf6_prefix_in6_addr(&route_to_del->prefix.u.prefix6, op);
+
+		route_to_del->type = OSPF6_DEST_TYPE_NETWORK;
+		route_to_del->path.origin.type = lsa->header->type;
+		route_to_del->path.origin.id = lsa->header->id;
+		route_to_del->path.origin.adv_router = lsa->header->adv_router;
+		route_to_del->path.prefix_options = op->prefix_options;
+		route_to_del->path.area_id = oa->area_id;
+		route_to_del->path.type = OSPF6_PATH_TYPE_INTRA;
+		route_to_del->path.metric_type = 1;
+
 		memset(&prefix, 0, sizeof(struct prefix));
 		prefix.family = AF_INET6;
 		prefix.prefixlen = op->prefix_length;
@@ -1365,20 +1752,162 @@ void ospf6_intra_prefix_lsa_remove(struct ospf6_lsa *lsa)
 				continue;
 			if (route->path.type != OSPF6_PATH_TYPE_INTRA)
 				continue;
-			if (route->path.origin.type != lsa->header->type
-			    || route->path.origin.id != lsa->header->id
-			    || route->path.origin.adv_router
-				       != lsa->header->adv_router)
+			/* Route has multiple ECMP paths, remove matching
+			 * path. Update current route's effective nh list
+			 * after removal of one of the path.
+			 */
+			if (listcount(route->paths) > 1) {
+				struct listnode *anode, *anext;
+				struct listnode *nnode, *rnode, *rnext;
+				struct ospf6_nexthop *nh, *rnh;
+				struct ospf6_path *o_path;
+				bool nh_updated = false;
+
+				/* Iterate all paths of route to find maching
+				 * with LSA remove info.
+				 * If route->path is same, replace
+				 * from paths list.
+				 */
+				for (ALL_LIST_ELEMENTS(route->paths, anode,
+						       anext, o_path)) {
+					if ((o_path->origin.type !=
+					     lsa->header->type) ||
+					    (o_path->origin.adv_router !=
+					     lsa->header->adv_router) ||
+					    (o_path->origin.id !=
+					     lsa->header->id))
+						continue;
+
+					if (IS_OSPF6_DEBUG_EXAMIN
+						(INTRA_PREFIX)) {
+						prefix2str(&prefix, buf,
+							   sizeof(buf));
+						zlog_debug(
+							"%s: route %s path found with cost %u nh %u to remove.",
+							__PRETTY_FUNCTION__,
+							buf, o_path->cost,
+							listcount(
+							o_path->nh_list));
+					}
+					/* Remove old route from global
+					 * ospf6 route table.
+					 * nh_update section will add
+					 * back with effective nh.
+					 */
+					if (oa->route_table->hook_remove)
+						(*oa->route_table->hook_remove)
+							(route);
+					/* Remove found path's nh_list from
+					 * the route's nh_list.
+					 */
+					for (ALL_LIST_ELEMENTS_RO(
+							o_path->nh_list,
+								nnode, nh)) {
+						for (ALL_LIST_ELEMENTS(
+							route->nh_list,
+							rnode, rnext, rnh)) {
+							if (
+							!ospf6_nexthop_is_same(
+								rnh, nh))
+								continue;
+							listnode_delete(
+								route->nh_list,
+								rnh);
+							ospf6_nexthop_delete(
+									rnh);
+						}
+					}
+					/* Delete the path from route's
+					 * path list
+					 */
+					listnode_delete(route->paths, o_path);
+					ospf6_path_free(o_path);
+					nh_updated = true;
+					break;
+				}
+
+				if (nh_updated) {
+
+					/* Iterate all paths and merge nexthop,
+					 * unlesss any of the nexthop similar to
+					 * ones deleted as part of path
+					 * deletion.
+					 */
+					for (ALL_LIST_ELEMENTS(route->paths,
+							anode, anext, o_path)) {
+						ospf6_merge_nexthops(
+								route->nh_list,
+							     o_path->nh_list);
+					}
+
+					if (IS_OSPF6_DEBUG_EXAMIN(
+								INTRA_PREFIX)) {
+						prefix2str(&route->prefix, buf,
+							   sizeof(buf));
+						assert(route->nh_list);
+						zlog_debug("%s: route %s update paths %u nh %u"
+							, __PRETTY_FUNCTION__,
+							buf,
+							listcount(route->paths),
+							listcount(
+							route->nh_list));
+					}
+
+					/* route's primary path is similar
+					 * to LSA, replace route's primary
+					 * path with route's paths list
+					 * head.
+					 */
+					if ((route->path.origin.id ==
+							lsa->header->id) &&
+						(route->path.origin.adv_router
+						 == lsa->header->adv_router)) {
+						struct ospf6_path *h_path;
+
+						h_path = (struct ospf6_path *)
+						listgetdata(listhead(
+								route->paths));
+						route->path.origin.type =
+							h_path->origin.type;
+						route->path.origin.id =
+							h_path->origin.id;
+						route->path.origin.adv_router =
+						h_path->origin.adv_router;
+					}
+
+					/* Update Global Route table and
+					 * RIB/FIB with effective
+					 * nh_list
+					 */
+					if (oa->route_table->hook_add)
+						(*oa->route_table->hook_add)
+							(route);
+				}
 				continue;
 
-			if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
-				prefix2str(&route->prefix, buf, sizeof(buf));
-				zlog_debug("remove %s", buf);
+			} else {
+
+				if (route->path.origin.type != lsa->header->type
+				    || route->path.origin.id != lsa->header->id
+				    || route->path.origin.adv_router
+				    != lsa->header->adv_router)
+					continue;
+
+				if (IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX)) {
+					prefix2str(&route->prefix, buf,
+						   sizeof(buf));
+					zlog_debug("route remove %s with path %u cost %u nh %u",
+						   buf, route->path.type,
+						   route->path.cost,
+						   listcount(route->nh_list));
+				}
+				ospf6_route_remove(route, oa->route_table);
 			}
-			ospf6_route_remove(route, oa->route_table);
 		}
 		if (route)
 			ospf6_route_unlock(route);
+
+		ospf6_route_delete(route_to_del);
 	}
 
 	if (current != end && IS_OSPF6_DEBUG_EXAMIN(INTRA_PREFIX))
@@ -1388,7 +1917,7 @@ void ospf6_intra_prefix_lsa_remove(struct ospf6_lsa *lsa)
 void ospf6_intra_route_calculation(struct ospf6_area *oa)
 {
 	struct ospf6_route *route, *nroute;
-	u_int16_t type;
+	uint16_t type;
 	struct ospf6_lsa *lsa;
 	void (*hook_add)(struct ospf6_route *) = NULL;
 	void (*hook_remove)(struct ospf6_route *) = NULL;
@@ -1406,8 +1935,7 @@ void ospf6_intra_route_calculation(struct ospf6_area *oa)
 		route->flag = OSPF6_ROUTE_REMOVE;
 
 	type = htons(OSPF6_LSTYPE_INTRA_PREFIX);
-	for (lsa = ospf6_lsdb_type_head(type, oa->lsdb); lsa;
-	     lsa = ospf6_lsdb_type_next(type, lsa))
+	for (ALL_LSDB_TYPED(oa->lsdb, type, lsa))
 		ospf6_intra_prefix_lsa_add(lsa);
 
 	oa->route_table->hook_add = hook_add;
@@ -1442,7 +1970,7 @@ void ospf6_intra_route_calculation(struct ospf6_area *oa)
 
 static void ospf6_brouter_debug_print(struct ospf6_route *brouter)
 {
-	u_int32_t brouter_id;
+	uint32_t brouter_id;
 	char brouter_name[16];
 	char area_name[16];
 	char destination[64];
@@ -1498,7 +2026,7 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 	struct ospf6_route *brouter, *nbrouter, *copy;
 	void (*hook_add)(struct ospf6_route *) = NULL;
 	void (*hook_remove)(struct ospf6_route *) = NULL;
-	u_int32_t brouter_id;
+	uint32_t brouter_id;
 	char brouter_name[16];
 
 	if (IS_OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_ID(oa->area_id))
@@ -1515,8 +2043,10 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 		brouter_id = ADV_ROUTER_IN_PREFIX(&brouter->prefix);
 		inet_ntop(AF_INET, &brouter_id, brouter_name,
 			  sizeof(brouter_name));
+
 		if (brouter->path.area_id != oa->area_id)
 			continue;
+
 		SET_FLAG(brouter->flag, OSPF6_ROUTE_REMOVE);
 
 		if (IS_OSPF6_DEBUG_BROUTER_SPECIFIC_ROUTER_ID(brouter_id)
@@ -1535,8 +2065,10 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 
 		if (brouter->type != OSPF6_DEST_TYPE_LINKSTATE)
 			continue;
+
 		if (ospf6_linkstate_prefix_id(&brouter->prefix) != htonl(0))
 			continue;
+
 		if (!CHECK_FLAG(brouter->path.router_bits, OSPF6_ROUTER_BIT_E)
 		    && !CHECK_FLAG(brouter->path.router_bits,
 				   OSPF6_ROUTER_BIT_B))
@@ -1588,6 +2120,29 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 		if (CHECK_FLAG(brouter->flag, OSPF6_ROUTE_WAS_REMOVED))
 			continue;
 
+		/* After iterating spf_table for all routers including
+		 * intra brouter, clear mark for remove flag for
+		 * inter border router if its adv router present in
+		 * SPF table.
+		 */
+		if (brouter->path.type == OSPF6_PATH_TYPE_INTER) {
+			struct prefix adv_prefix;
+
+			ospf6_linkstate_prefix(brouter->path.origin.adv_router,
+					       htonl(0), &adv_prefix);
+
+			if (ospf6_route_lookup(&adv_prefix, oa->spf_table)) {
+				if (IS_OSPF6_DEBUG_BROUTER) {
+					zlog_debug("%s: keep inter brouter %s as adv router 0x%x found in spf",
+						   __PRETTY_FUNCTION__,
+						   brouter_name,
+					brouter->path.origin.adv_router);
+					ospf6_brouter_debug_print(brouter);
+				}
+				UNSET_FLAG(brouter->flag, OSPF6_ROUTE_REMOVE);
+			}
+		}
+
 		if (CHECK_FLAG(brouter->flag, OSPF6_ROUTE_REMOVE)
 		    && CHECK_FLAG(brouter->flag, OSPF6_ROUTE_ADD)) {
 			UNSET_FLAG(brouter->flag, OSPF6_ROUTE_REMOVE);
@@ -1639,21 +2194,37 @@ void ospf6_intra_brouter_calculation(struct ospf6_area *oa)
 			  oa->name);
 }
 
-struct ospf6_lsa_handler router_handler = {OSPF6_LSTYPE_ROUTER, "Router", "Rtr",
-					   ospf6_router_lsa_show,
-					   ospf6_router_lsa_get_nbr_id};
+struct ospf6_lsa_handler router_handler = {.lh_type = OSPF6_LSTYPE_ROUTER,
+					   .lh_name = "Router",
+					   .lh_short_name = "Rtr",
+					   .lh_show = ospf6_router_lsa_show,
+					   .lh_get_prefix_str =
+						   ospf6_router_lsa_get_nbr_id,
+					   .lh_debug = 0};
 
-struct ospf6_lsa_handler network_handler = {OSPF6_LSTYPE_NETWORK, "Network",
-					    "Net", ospf6_network_lsa_show,
-					    ospf6_network_lsa_get_ar_id};
+struct ospf6_lsa_handler network_handler = {.lh_type = OSPF6_LSTYPE_NETWORK,
+					    .lh_name = "Network",
+					    .lh_short_name = "Net",
+					    .lh_show = ospf6_network_lsa_show,
+					    .lh_get_prefix_str =
+						    ospf6_network_lsa_get_ar_id,
+					    .lh_debug = 0};
 
-struct ospf6_lsa_handler link_handler = {OSPF6_LSTYPE_LINK, "Link", "Lnk",
-					 ospf6_link_lsa_show,
-					 ospf6_link_lsa_get_prefix_str};
+struct ospf6_lsa_handler link_handler = {.lh_type = OSPF6_LSTYPE_LINK,
+					 .lh_name = "Link",
+					 .lh_short_name = "Lnk",
+					 .lh_show = ospf6_link_lsa_show,
+					 .lh_get_prefix_str =
+						 ospf6_link_lsa_get_prefix_str,
+					 .lh_debug = 0};
 
 struct ospf6_lsa_handler intra_prefix_handler = {
-	OSPF6_LSTYPE_INTRA_PREFIX, "Intra-Prefix", "INP",
-	ospf6_intra_prefix_lsa_show, ospf6_intra_prefix_lsa_get_prefix_str};
+	.lh_type = OSPF6_LSTYPE_INTRA_PREFIX,
+	.lh_name = "Intra-Prefix",
+	.lh_short_name = "INP",
+	.lh_show = ospf6_intra_prefix_lsa_show,
+	.lh_get_prefix_str = ospf6_intra_prefix_lsa_get_prefix_str,
+	.lh_debug = 0};
 
 void ospf6_intra_init(void)
 {
@@ -1699,7 +2270,7 @@ DEFUN (debug_ospf6_brouter_router,
       )
 {
 	int idx_ipv4 = 4;
-	u_int32_t router_id;
+	uint32_t router_id;
 	inet_pton(AF_INET, argv[idx_ipv4]->arg, &router_id);
 	OSPF6_DEBUG_BROUTER_SPECIFIC_ROUTER_ON(router_id);
 	return CMD_SUCCESS;
@@ -1730,7 +2301,7 @@ DEFUN (debug_ospf6_brouter_area,
       )
 {
 	int idx_ipv4 = 4;
-	u_int32_t area_id;
+	uint32_t area_id;
 	inet_pton(AF_INET, argv[idx_ipv4]->arg, &area_id);
 	OSPF6_DEBUG_BROUTER_SPECIFIC_AREA_ON(area_id);
 	return CMD_SUCCESS;
@@ -1754,18 +2325,16 @@ int config_write_ospf6_debug_brouter(struct vty *vty)
 {
 	char buf[16];
 	if (IS_OSPF6_DEBUG_BROUTER)
-		vty_out(vty, "debug ospf6 border-routers%s", VNL);
+		vty_out(vty, "debug ospf6 border-routers\n");
 	if (IS_OSPF6_DEBUG_BROUTER_SPECIFIC_ROUTER) {
 		inet_ntop(AF_INET, &conf_debug_ospf6_brouter_specific_router_id,
 			  buf, sizeof(buf));
-		vty_out(vty, "debug ospf6 border-routers router-id %s%s", buf,
-			VNL);
+		vty_out(vty, "debug ospf6 border-routers router-id %s\n", buf);
 	}
 	if (IS_OSPF6_DEBUG_BROUTER_SPECIFIC_AREA) {
 		inet_ntop(AF_INET, &conf_debug_ospf6_brouter_specific_area_id,
 			  buf, sizeof(buf));
-		vty_out(vty, "debug ospf6 border-routers area-id %s%s", buf,
-			VNL);
+		vty_out(vty, "debug ospf6 border-routers area-id %s\n", buf);
 	}
 	return 0;
 }
