@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -60,7 +59,7 @@ static const struct message ospf6_message_type_str[] = {
 
 /* Minimum (besides the standard OSPF packet header) lengths for OSPF
    packets of particular types, offset is the "type" field. */
-const u_int16_t ospf6_packet_minlen[OSPF6_MESSAGE_TYPE_ALL] = {
+const uint16_t ospf6_packet_minlen[OSPF6_MESSAGE_TYPE_ALL] = {
 	0,
 	OSPF6_HELLO_MIN_SIZE,
 	OSPF6_DB_DESC_MIN_SIZE,
@@ -70,7 +69,7 @@ const u_int16_t ospf6_packet_minlen[OSPF6_MESSAGE_TYPE_ALL] = {
 
 /* Minimum (besides the standard LSA header) lengths for LSAs of particular
    types, offset is the "LSA function code" portion of "LSA type" field. */
-const u_int16_t ospf6_lsa_minlen[OSPF6_LSTYPE_SIZE] = {
+const uint16_t ospf6_lsa_minlen[OSPF6_LSTYPE_SIZE] = {
 	0,
 	/* 0x2001 */ OSPF6_ROUTER_LSA_MIN_SIZE,
 	/* 0x2002 */ OSPF6_NETWORK_LSA_MIN_SIZE,
@@ -114,15 +113,15 @@ void ospf6_hello_print(struct ospf6_header *oh)
 	ospf6_options_printbuf(hello->options, options, sizeof(options));
 
 	zlog_debug("    I/F-Id:%ld Priority:%d Option:%s",
-		   (u_long)ntohl(hello->interface_id), hello->priority,
+		   (unsigned long)ntohl(hello->interface_id), hello->priority,
 		   options);
 	zlog_debug("    HelloInterval:%hu DeadInterval:%hu",
 		   ntohs(hello->hello_interval), ntohs(hello->dead_interval));
 	zlog_debug("    DR:%s BDR:%s", drouter, bdrouter);
 
 	for (p = (char *)((caddr_t)hello + sizeof(struct ospf6_hello));
-	     p + sizeof(u_int32_t) <= OSPF6_MESSAGE_END(oh);
-	     p += sizeof(u_int32_t)) {
+	     p + sizeof(uint32_t) <= OSPF6_MESSAGE_END(oh);
+	     p += sizeof(uint32_t)) {
 		inet_ntop(AF_INET, (void *)p, neighbor, sizeof(neighbor));
 		zlog_debug("    Neighbor: %s", neighbor);
 	}
@@ -150,7 +149,7 @@ void ospf6_dbdesc_print(struct ospf6_header *oh)
 		   (CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_IBIT) ? "I" : "-"),
 		   (CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_MBIT) ? "M" : "-"),
 		   (CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_MSBIT) ? "m" : "s"),
-		   (u_long)ntohl(dbdesc->seqnum));
+		   (unsigned long)ntohl(dbdesc->seqnum));
 
 	for (p = (char *)((caddr_t)dbdesc + sizeof(struct ospf6_dbdesc));
 	     p + sizeof(struct ospf6_lsa_header) <= OSPF6_MESSAGE_END(oh);
@@ -185,7 +184,7 @@ void ospf6_lsreq_print(struct ospf6_header *oh)
 void ospf6_lsupdate_print(struct ospf6_header *oh)
 {
 	struct ospf6_lsupdate *lsupdate;
-	u_long num;
+	unsigned long num;
 	char *p;
 
 	ospf6_header_print(oh);
@@ -279,9 +278,9 @@ static void ospf6_hello_recv(struct in6_addr *src, struct in6_addr *dst,
 
 	/* TwoWay check */
 	for (p = (char *)((caddr_t)hello + sizeof(struct ospf6_hello));
-	     p + sizeof(u_int32_t) <= OSPF6_MESSAGE_END(oh);
-	     p += sizeof(u_int32_t)) {
-		u_int32_t *router_id = (u_int32_t *)p;
+	     p + sizeof(uint32_t) <= OSPF6_MESSAGE_END(oh);
+	     p += sizeof(uint32_t)) {
+		uint32_t *router_id = (uint32_t *)p;
 
 		if (*router_id == oi->area->ospf6->router_id)
 			twoway++;
@@ -322,6 +321,8 @@ static void ospf6_hello_recv(struct in6_addr *src, struct in6_addr *dst,
 			backupseen++;
 	}
 
+	oi->hello_in++;
+
 	/* Execute neighbor events */
 	thread_execute(master, hello_received, on, 0);
 	if (twoway)
@@ -331,9 +332,9 @@ static void ospf6_hello_recv(struct in6_addr *src, struct in6_addr *dst,
 
 	/* Schedule interface events */
 	if (backupseen)
-		thread_add_event(master, backup_seen, oi, 0);
+		thread_add_event(master, backup_seen, oi, 0, NULL);
 	if (neighborchange)
-		thread_add_event(master, neighbor_change, oi, 0);
+		thread_add_event(master, neighbor_change, oi, 0, NULL);
 
 	if (neighbor_ifindex_change && on->state == OSPF6_NEIGHBOR_FULL)
 		OSPF6_ROUTER_LSA_SCHEDULE(oi->area);
@@ -403,21 +404,24 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 		if (CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_MSBIT)) {
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 				zlog_debug("Master/Slave bit mismatch");
-			thread_add_event(master, seqnumber_mismatch, on, 0);
+			thread_add_event(master, seqnumber_mismatch, on, 0,
+					 NULL);
 			return;
 		}
 
 		if (CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_IBIT)) {
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 				zlog_debug("Initialize bit mismatch");
-			thread_add_event(master, seqnumber_mismatch, on, 0);
+			thread_add_event(master, seqnumber_mismatch, on, 0,
+					 NULL);
 			return;
 		}
 
 		if (memcmp(on->options, dbdesc->options, sizeof(on->options))) {
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 				zlog_debug("Option field mismatch");
-			thread_add_event(master, seqnumber_mismatch, on, 0);
+			thread_add_event(master, seqnumber_mismatch, on, 0,
+					 NULL);
 			return;
 		}
 
@@ -425,8 +429,9 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 				zlog_debug(
 					"Sequence number mismatch (%#lx expected)",
-					(u_long)on->dbdesc_seqnum);
-			thread_add_event(master, seqnumber_mismatch, on, 0);
+					(unsigned long)on->dbdesc_seqnum);
+			thread_add_event(master, seqnumber_mismatch, on, 0,
+					 NULL);
 			return;
 		}
 		break;
@@ -446,7 +451,7 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 		if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 			zlog_debug("Not duplicate dbdesc in state %s",
 				   ospf6_neighbor_state_str[on->state]);
-		thread_add_event(master, seqnumber_mismatch, on, 0);
+		thread_add_event(master, seqnumber_mismatch, on, 0, NULL);
 		return;
 
 	default:
@@ -490,7 +495,8 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 				zlog_debug(
 					"SeqNumMismatch (E-bit mismatch), discard");
 			ospf6_lsa_delete(his);
-			thread_add_event(master, seqnumber_mismatch, on, 0);
+			thread_add_event(master, seqnumber_mismatch, on, 0,
+					 NULL);
 			return;
 		}
 
@@ -517,19 +523,21 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 	on->dbdesc_seqnum++;
 
 	/* schedule send lsreq */
-	if (on->request_list->count && (on->thread_send_lsreq == NULL))
-		on->thread_send_lsreq =
-			thread_add_event(master, ospf6_lsreq_send, on, 0);
+	if (on->request_list->count)
+		thread_add_event(master, ospf6_lsreq_send, on, 0,
+				 &on->thread_send_lsreq);
 
 	THREAD_OFF(on->thread_send_dbdesc);
 
 	/* More bit check */
 	if (!CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_MBIT)
 	    && !CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_MBIT))
-		thread_add_event(master, exchange_done, on, 0);
-	else
-		on->thread_send_dbdesc = thread_add_event(
-			master, ospf6_dbdesc_send_newone, on, 0);
+		thread_add_event(master, exchange_done, on, 0, NULL);
+	else {
+		on->thread_send_dbdesc = NULL;
+		thread_add_event(master, ospf6_dbdesc_send_newone, on, 0,
+				 &on->thread_send_dbdesc);
+	}
 
 	/* save last received dbdesc */
 	memcpy(&on->dbdesc_last, dbdesc, sizeof(struct ospf6_dbdesc));
@@ -605,29 +613,33 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 				zlog_debug(
 					"Duplicated dbdesc causes retransmit");
 			THREAD_OFF(on->thread_send_dbdesc);
-			on->thread_send_dbdesc = thread_add_event(
-				master, ospf6_dbdesc_send, on, 0);
+			on->thread_send_dbdesc = NULL;
+			thread_add_event(master, ospf6_dbdesc_send, on, 0,
+					 &on->thread_send_dbdesc);
 			return;
 		}
 
 		if (!CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_MSBIT)) {
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 				zlog_debug("Master/Slave bit mismatch");
-			thread_add_event(master, seqnumber_mismatch, on, 0);
+			thread_add_event(master, seqnumber_mismatch, on, 0,
+					 NULL);
 			return;
 		}
 
 		if (CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_IBIT)) {
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 				zlog_debug("Initialize bit mismatch");
-			thread_add_event(master, seqnumber_mismatch, on, 0);
+			thread_add_event(master, seqnumber_mismatch, on, 0,
+					 NULL);
 			return;
 		}
 
 		if (memcmp(on->options, dbdesc->options, sizeof(on->options))) {
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 				zlog_debug("Option field mismatch");
-			thread_add_event(master, seqnumber_mismatch, on, 0);
+			thread_add_event(master, seqnumber_mismatch, on, 0,
+					 NULL);
 			return;
 		}
 
@@ -635,8 +647,9 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 				zlog_debug(
 					"Sequence number mismatch (%#lx expected)",
-					(u_long)on->dbdesc_seqnum + 1);
-			thread_add_event(master, seqnumber_mismatch, on, 0);
+					(unsigned long)on->dbdesc_seqnum + 1);
+			thread_add_event(master, seqnumber_mismatch, on, 0,
+					 NULL);
 			return;
 		}
 		break;
@@ -651,15 +664,15 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 				zlog_debug(
 					"Duplicated dbdesc causes retransmit");
 			THREAD_OFF(on->thread_send_dbdesc);
-			on->thread_send_dbdesc = thread_add_event(
-				master, ospf6_dbdesc_send, on, 0);
+			thread_add_event(master, ospf6_dbdesc_send, on, 0,
+					 &on->thread_send_dbdesc);
 			return;
 		}
 
 		if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 			zlog_debug("Not duplicate dbdesc in state %s",
 				   ospf6_neighbor_state_str[on->state]);
-		thread_add_event(master, seqnumber_mismatch, on, 0);
+		thread_add_event(master, seqnumber_mismatch, on, 0, NULL);
 		return;
 
 	default:
@@ -699,7 +712,8 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 				zlog_debug("E-bit mismatch with LSA Headers");
 			ospf6_lsa_delete(his);
-			thread_add_event(master, seqnumber_mismatch, on, 0);
+			thread_add_event(master, seqnumber_mismatch, on, 0,
+					 NULL);
 			return;
 		}
 
@@ -719,13 +733,13 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 	on->dbdesc_seqnum = ntohl(dbdesc->seqnum);
 
 	/* schedule send lsreq */
-	if ((on->thread_send_lsreq == NULL) && (on->request_list->count))
-		on->thread_send_lsreq =
-			thread_add_event(master, ospf6_lsreq_send, on, 0);
+	if (on->request_list->count)
+		thread_add_event(master, ospf6_lsreq_send, on, 0,
+				 &on->thread_send_lsreq);
 
 	THREAD_OFF(on->thread_send_dbdesc);
-	on->thread_send_dbdesc =
-		thread_add_event(master, ospf6_dbdesc_send_newone, on, 0);
+	thread_add_event(master, ospf6_dbdesc_send_newone, on, 0,
+			 &on->thread_send_dbdesc);
 
 	/* save last received dbdesc */
 	memcpy(&on->dbdesc_last, dbdesc, sizeof(struct ospf6_dbdesc));
@@ -764,6 +778,8 @@ static void ospf6_dbdesc_recv(struct in6_addr *src, struct in6_addr *dst,
 		dbdesc->reserved2 = 0;
 	}
 
+	oi->db_desc_in++;
+
 	if (ntohl(oh->router_id) < ntohl(ospf6->router_id))
 		ospf6_dbdesc_recv_master(oh, on);
 	else if (ntohl(ospf6->router_id) < ntohl(oh->router_id))
@@ -799,6 +815,8 @@ static void ospf6_lsreq_recv(struct in6_addr *src, struct in6_addr *dst,
 		return;
 	}
 
+	oi->ls_req_in++;
+
 	/* Process each request */
 	for (p = (char *)((caddr_t)oh + sizeof(struct ospf6_header));
 	     p + sizeof(struct ospf6_lsreq_entry) <= OSPF6_MESSAGE_END(oh);
@@ -835,7 +853,7 @@ static void ospf6_lsreq_recv(struct in6_addr *src, struct in6_addr *dst,
 					ospf6_lstype_name(e->type), id,
 					adv_router);
 			}
-			thread_add_event(master, bad_lsreq, on, 0);
+			thread_add_event(master, bad_lsreq, on, 0, NULL);
 			return;
 		}
 
@@ -846,8 +864,8 @@ static void ospf6_lsreq_recv(struct in6_addr *src, struct in6_addr *dst,
 
 	/* schedule send lsupdate */
 	THREAD_OFF(on->thread_send_lsupdate);
-	on->thread_send_lsupdate =
-		thread_add_event(master, ospf6_lsupdate_send_neighbor, on, 0);
+	thread_add_event(master, ospf6_lsupdate_send_neighbor, on, 0,
+			 &on->thread_send_lsupdate);
 }
 
 /* Verify, that the specified memory area contains exactly N valid IPv6
@@ -855,12 +873,12 @@ static void ospf6_lsreq_recv(struct in6_addr *src, struct in6_addr *dst,
 static unsigned ospf6_prefixes_examin(
 	struct ospf6_prefix *current, /* start of buffer    */
 	unsigned length,
-	const u_int32_t req_num_pfxs /* always compared with the actual number
+	const uint32_t req_num_pfxs /* always compared with the actual number
 					of prefixes */
-	)
+)
 {
-	u_char requested_pfx_bytes;
-	u_int32_t real_num_pfxs = 0;
+	uint8_t requested_pfx_bytes;
+	uint32_t real_num_pfxs = 0;
 
 	while (length) {
 		if (length < OSPF6_PREFIX_MIN_SIZE) {
@@ -911,15 +929,15 @@ static unsigned ospf6_prefixes_examin(
    LSA type in network byte order, uses in host byte order and passes to
    ospf6_lstype_name() in network byte order again. */
 static unsigned ospf6_lsa_examin(struct ospf6_lsa_header *lsah,
-				 const u_int16_t lsalen,
-				 const u_char headeronly)
+				 const uint16_t lsalen,
+				 const uint8_t headeronly)
 {
 	struct ospf6_intra_prefix_lsa *intra_prefix_lsa;
 	struct ospf6_as_external_lsa *as_external_lsa;
 	struct ospf6_link_lsa *link_lsa;
 	unsigned exp_length;
-	u_int8_t ltindex;
-	u_int16_t lsatype;
+	uint8_t ltindex;
+	uint16_t lsatype;
 
 	/* In case an additional minimum length constraint is defined for
 	   current
@@ -1085,16 +1103,16 @@ static unsigned ospf6_lsa_examin(struct ospf6_lsa_header *lsah,
    of deeper-level checks. */
 static unsigned
 ospf6_lsaseq_examin(struct ospf6_lsa_header *lsah, /* start of buffered data */
-		    size_t length, const u_char headeronly,
+		    size_t length, const uint8_t headeronly,
 		    /* When declared_num_lsas is not 0, compare it to the real
 		       number of LSAs
 		       and treat the difference as an error. */
-		    const u_int32_t declared_num_lsas)
+		    const uint32_t declared_num_lsas)
 {
-	u_int32_t counted_lsas = 0;
+	uint32_t counted_lsas = 0;
 
 	while (length) {
-		u_int16_t lsalen;
+		uint16_t lsalen;
 		if (length < OSPF6_LSA_HEADER_SIZE) {
 			if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_UNKNOWN,
 						   RECV))
@@ -1187,8 +1205,10 @@ static unsigned ospf6_packet_examin(struct ospf6_header *oh,
 	if (bytesonwire != ntohs(oh->length)) {
 		if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_UNKNOWN, RECV))
 			zlog_debug(
-				"%s: packet length error (%u real, %u declared)",
-				__func__, bytesonwire, ntohs(oh->length));
+				"%s: %s packet length error (%u real, %u declared)",
+				__func__, lookup_msg(ospf6_message_type_str,
+						     oh->type, NULL),
+				bytesonwire, ntohs(oh->length));
 		return MSG_NG;
 	}
 	/* version check */
@@ -1356,6 +1376,8 @@ static void ospf6_lsupdate_recv(struct in6_addr *src, struct in6_addr *dst,
 	lsupdate = (struct ospf6_lsupdate *)((caddr_t)oh
 					     + sizeof(struct ospf6_header));
 
+	oi->ls_upd_in++;
+
 	/* Process LSAs */
 	for (p = (char *)((caddr_t)lsupdate + sizeof(struct ospf6_lsupdate));
 	     p < OSPF6_MESSAGE_END(oh)
@@ -1393,6 +1415,8 @@ static void ospf6_lsack_recv(struct in6_addr *src, struct in6_addr *dst,
 		return;
 	}
 
+	oi->ls_ack_in++;
+
 	for (p = (char *)((caddr_t)oh + sizeof(struct ospf6_header));
 	     p + sizeof(struct ospf6_lsa_header) <= OSPF6_MESSAGE_END(oh);
 	     p += sizeof(struct ospf6_lsa_header)) {
@@ -1466,13 +1490,13 @@ static void ospf6_lsack_recv(struct in6_addr *src, struct in6_addr *dst,
 	assert(p == OSPF6_MESSAGE_END(oh));
 }
 
-static u_char *recvbuf = NULL;
-static u_char *sendbuf = NULL;
+static uint8_t *recvbuf = NULL;
+static uint8_t *sendbuf = NULL;
 static unsigned int iobuflen = 0;
 
 int ospf6_iobuf_size(unsigned int size)
 {
-	u_char *recvnew, *sendnew;
+	uint8_t *recvnew, *sendnew;
 
 	if (size <= iobuflen)
 		return iobuflen;
@@ -1527,7 +1551,7 @@ int ospf6_receive(struct thread *thread)
 
 	/* add next read thread */
 	sockfd = THREAD_FD(thread);
-	thread_add_read(master, ospf6_receive, NULL, sockfd);
+	thread_add_read(master, ospf6_receive, NULL, sockfd, NULL);
 
 	/* initialize */
 	memset(&src, 0, sizeof(src));
@@ -1549,7 +1573,8 @@ int ospf6_receive(struct thread *thread)
 	oi = ospf6_interface_lookup_by_ifindex(ifindex);
 	if (oi == NULL || oi->area == NULL
 	    || CHECK_FLAG(oi->flag, OSPF6_INTERFACE_DISABLE)) {
-		zlog_debug("Message received on disabled interface");
+		if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_UNKNOWN, RECV))
+			zlog_debug("Message received on disabled interface");
 		return 0;
 	}
 	if (CHECK_FLAG(oi->flag, OSPF6_INTERFACE_PASSIVE)) {
@@ -1703,7 +1728,7 @@ int ospf6_hello_send(struct thread *thread)
 	struct ospf6_interface *oi;
 	struct ospf6_header *oh;
 	struct ospf6_hello *hello;
-	u_char *p;
+	uint8_t *p;
 	struct listnode *node, *nnode;
 	struct ospf6_neighbor *on;
 
@@ -1724,8 +1749,8 @@ int ospf6_hello_send(struct thread *thread)
 	}
 
 	/* set next thread */
-	oi->thread_send_hello = thread_add_timer(master, ospf6_hello_send, oi,
-						 oi->hello_interval);
+	thread_add_timer(master, ospf6_hello_send, oi, oi->hello_interval,
+			 &oi->thread_send_hello);
 
 	memset(sendbuf, 0, iobuflen);
 	oh = (struct ospf6_header *)sendbuf;
@@ -1742,13 +1767,13 @@ int ospf6_hello_send(struct thread *thread)
 	hello->drouter = oi->drouter;
 	hello->bdrouter = oi->bdrouter;
 
-	p = (u_char *)((caddr_t)hello + sizeof(struct ospf6_hello));
+	p = (uint8_t *)((caddr_t)hello + sizeof(struct ospf6_hello));
 
 	for (ALL_LIST_ELEMENTS(oi->neighbor_list, node, nnode, on)) {
 		if (on->state < OSPF6_NEIGHBOR_INIT)
 			continue;
 
-		if (p - sendbuf + sizeof(u_int32_t) > ospf6_packet_max(oi)) {
+		if (p - sendbuf + sizeof(uint32_t) > ospf6_packet_max(oi)) {
 			if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_HELLO,
 						   SEND))
 				zlog_debug(
@@ -1756,13 +1781,15 @@ int ospf6_hello_send(struct thread *thread)
 			break;
 		}
 
-		memcpy(p, &on->router_id, sizeof(u_int32_t));
-		p += sizeof(u_int32_t);
+		memcpy(p, &on->router_id, sizeof(uint32_t));
+		p += sizeof(uint32_t);
 	}
 
 	oh->type = OSPF6_MESSAGE_TYPE_HELLO;
 	oh->length = htons(p - sendbuf);
 
+	oi->hello_out++;
+
 	ospf6_send(oi->linklocal_addr, &allspfrouters6, oi, oh);
 	return 0;
 }
@@ -1772,7 +1799,7 @@ int ospf6_dbdesc_send(struct thread *thread)
 	struct ospf6_neighbor *on;
 	struct ospf6_header *oh;
 	struct ospf6_dbdesc *dbdesc;
-	u_char *p;
+	uint8_t *p;
 	struct ospf6_lsa *lsa;
 	struct in6_addr *dst;
 
@@ -1789,9 +1816,9 @@ int ospf6_dbdesc_send(struct thread *thread)
 
 	/* set next thread if master */
 	if (CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_MSBIT))
-		on->thread_send_dbdesc =
-			thread_add_timer(master, ospf6_dbdesc_send, on,
-					 on->ospf6_if->rxmt_interval);
+		thread_add_timer(master, ospf6_dbdesc_send, on,
+				 on->ospf6_if->rxmt_interval,
+				 &on->thread_send_dbdesc);
 
 	memset(sendbuf, 0, iobuflen);
 	oh = (struct ospf6_header *)sendbuf;
@@ -1812,10 +1839,9 @@ int ospf6_dbdesc_send(struct thread *thread)
 	dbdesc->seqnum = htonl(on->dbdesc_seqnum);
 
 	/* if this is not initial one, set LSA headers in dbdesc */
-	p = (u_char *)((caddr_t)dbdesc + sizeof(struct ospf6_dbdesc));
+	p = (uint8_t *)((caddr_t)dbdesc + sizeof(struct ospf6_dbdesc));
 	if (!CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_IBIT)) {
-		for (lsa = ospf6_lsdb_head(on->dbdesc_list); lsa;
-		     lsa = ospf6_lsdb_next(lsa)) {
+		for (ALL_LSDB(on->dbdesc_list, lsa)) {
 			ospf6_lsa_age_update_to_send(lsa,
 						     on->ospf6_if->transdelay);
 
@@ -1839,6 +1865,8 @@ int ospf6_dbdesc_send(struct thread *thread)
 	else
 		dst = &on->linklocal_addr;
 
+	on->ospf6_if->db_desc_out++;
+
 	ospf6_send(on->ospf6_if->linklocal_addr, dst, on->ospf6_if, oh);
 
 	return 0;
@@ -1857,8 +1885,7 @@ int ospf6_dbdesc_send_newone(struct thread *thread)
 	   structure)
 	   so that ospf6_send_dbdesc () can send those LSAs */
 	size = sizeof(struct ospf6_lsa_header) + sizeof(struct ospf6_dbdesc);
-	for (lsa = ospf6_lsdb_head(on->summary_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->summary_list, lsa)) {
 		if (size + sizeof(struct ospf6_lsa_header)
 		    > ospf6_packet_max(on->ospf6_if)) {
 			ospf6_lsdb_lsa_unlock(lsa);
@@ -1877,7 +1904,7 @@ int ospf6_dbdesc_send_newone(struct thread *thread)
 	if (!CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_MSBIT) && /* Slave */
 	    !CHECK_FLAG(on->dbdesc_last.bits, OSPF6_DBDESC_MBIT)
 	    && !CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_MBIT))
-		thread_add_event(master, exchange_done, on, 0);
+		thread_add_event(master, exchange_done, on, 0, NULL);
 
 	thread_execute(master, ospf6_dbdesc_send, on, 0);
 	return 0;
@@ -1888,7 +1915,7 @@ int ospf6_lsreq_send(struct thread *thread)
 	struct ospf6_neighbor *on;
 	struct ospf6_header *oh;
 	struct ospf6_lsreq_entry *e;
-	u_char *p;
+	uint8_t *p;
 	struct ospf6_lsa *lsa, *last_req;
 
 	on = (struct ospf6_neighbor *)THREAD_ARG(thread);
@@ -1906,7 +1933,7 @@ int ospf6_lsreq_send(struct thread *thread)
 
 	/* schedule loading_done if request list is empty */
 	if (on->request_list->count == 0) {
-		thread_add_event(master, loading_done, on, 0);
+		thread_add_event(master, loading_done, on, 0, NULL);
 		return 0;
 	}
 
@@ -1915,9 +1942,8 @@ int ospf6_lsreq_send(struct thread *thread)
 	last_req = NULL;
 
 	/* set Request entries in lsreq */
-	p = (u_char *)((caddr_t)oh + sizeof(struct ospf6_header));
-	for (lsa = ospf6_lsdb_head(on->request_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	p = (uint8_t *)((caddr_t)oh + sizeof(struct ospf6_header));
+	for (ALL_LSDB(on->request_list, lsa)) {
 		/* MTU check */
 		if (p - sendbuf + sizeof(struct ospf6_lsreq_entry)
 		    > ospf6_packet_max(on->ospf6_if)) {
@@ -1944,6 +1970,8 @@ int ospf6_lsreq_send(struct thread *thread)
 	oh->type = OSPF6_MESSAGE_TYPE_LSREQ;
 	oh->length = htons(p - sendbuf);
 
+	on->ospf6_if->ls_req_out++;
+
 	if (on->ospf6_if->state == OSPF6_INTERFACE_POINTTOPOINT)
 		ospf6_send(on->ospf6_if->linklocal_addr, &allspfrouters6,
 			   on->ospf6_if, oh);
@@ -1953,20 +1981,52 @@ int ospf6_lsreq_send(struct thread *thread)
 
 	/* set next thread */
 	if (on->request_list->count != 0) {
-		on->thread_send_lsreq =
-			thread_add_timer(master, ospf6_lsreq_send, on,
-					 on->ospf6_if->rxmt_interval);
+		on->thread_send_lsreq = NULL;
+		thread_add_timer(master, ospf6_lsreq_send, on,
+				 on->ospf6_if->rxmt_interval,
+				 &on->thread_send_lsreq);
 	}
 
 	return 0;
 }
 
+static void ospf6_send_lsupdate(struct ospf6_neighbor *on,
+				struct ospf6_interface *oi,
+				struct ospf6_header *oh)
+{
+
+	if (on) {
+		on->ospf6_if->ls_upd_out++;
+
+		if ((on->ospf6_if->state == OSPF6_INTERFACE_POINTTOPOINT)
+		    || (on->ospf6_if->state == OSPF6_INTERFACE_DR)
+		    || (on->ospf6_if->state == OSPF6_INTERFACE_BDR)) {
+			ospf6_send(on->ospf6_if->linklocal_addr,
+				   &allspfrouters6, on->ospf6_if, oh);
+		} else {
+			ospf6_send(on->ospf6_if->linklocal_addr,
+				   &on->linklocal_addr, on->ospf6_if, oh);
+		}
+	} else if (oi) {
+
+		oi->ls_upd_out++;
+
+		if ((oi->state == OSPF6_INTERFACE_POINTTOPOINT)
+		    || (oi->state == OSPF6_INTERFACE_DR)
+		    || (oi->state == OSPF6_INTERFACE_BDR)) {
+			ospf6_send(oi->linklocal_addr, &allspfrouters6, oi, oh);
+		} else {
+			ospf6_send(oi->linklocal_addr, &alldrouters6, oi, oh);
+		}
+	}
+}
+
 int ospf6_lsupdate_send_neighbor(struct thread *thread)
 {
 	struct ospf6_neighbor *on;
 	struct ospf6_header *oh;
 	struct ospf6_lsupdate *lsupdate;
-	u_char *p;
+	uint8_t *p;
 	int lsa_cnt;
 	struct ospf6_lsa *lsa;
 
@@ -1988,18 +2048,34 @@ int ospf6_lsupdate_send_neighbor(struct thread *thread)
 	lsupdate = (struct ospf6_lsupdate *)((caddr_t)oh
 					     + sizeof(struct ospf6_header));
 
-	p = (u_char *)((caddr_t)lsupdate + sizeof(struct ospf6_lsupdate));
+	p = (uint8_t *)((caddr_t)lsupdate + sizeof(struct ospf6_lsupdate));
 	lsa_cnt = 0;
 
 	/* lsupdate_list lists those LSA which doesn't need to be
 	   retransmitted. remove those from the list */
-	for (lsa = ospf6_lsdb_head(on->lsupdate_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->lsupdate_list, lsa)) {
 		/* MTU check */
 		if ((p - sendbuf + (unsigned int)OSPF6_LSA_SIZE(lsa->header))
 		    > ospf6_packet_max(on->ospf6_if)) {
-			ospf6_lsdb_lsa_unlock(lsa);
-			break;
+			if (lsa_cnt) {
+				oh->type = OSPF6_MESSAGE_TYPE_LSUPDATE;
+				oh->length = htons(p - sendbuf);
+				lsupdate->lsa_number = htonl(lsa_cnt);
+
+				ospf6_send_lsupdate(on, NULL, oh);
+
+				memset(sendbuf, 0, iobuflen);
+				oh = (struct ospf6_header *)sendbuf;
+				lsupdate = (struct ospf6_lsupdate
+						    *)((caddr_t)oh
+						       + sizeof(struct
+								ospf6_header));
+
+				p = (uint8_t *)((caddr_t)lsupdate
+						+ sizeof(struct
+							 ospf6_lsupdate));
+				lsa_cnt = 0;
+			}
 		}
 
 		ospf6_lsa_age_update_to_send(lsa, on->ospf6_if->transdelay);
@@ -2015,15 +2091,7 @@ int ospf6_lsupdate_send_neighbor(struct thread *thread)
 		oh->type = OSPF6_MESSAGE_TYPE_LSUPDATE;
 		oh->length = htons(p - sendbuf);
 		lsupdate->lsa_number = htonl(lsa_cnt);
-
-		if ((on->ospf6_if->state == OSPF6_INTERFACE_POINTTOPOINT)
-		    || (on->ospf6_if->state == OSPF6_INTERFACE_DR)
-		    || (on->ospf6_if->state == OSPF6_INTERFACE_BDR))
-			ospf6_send(on->ospf6_if->linklocal_addr,
-				   &allspfrouters6, on->ospf6_if, oh);
-		else
-			ospf6_send(on->ospf6_if->linklocal_addr,
-				   &on->linklocal_addr, on->ospf6_if, oh);
+		ospf6_send_lsupdate(on, NULL, oh);
 	}
 
 	/* The addresses used for retransmissions are different from those sent
@@ -2034,16 +2102,40 @@ int ospf6_lsupdate_send_neighbor(struct thread *thread)
 	oh = (struct ospf6_header *)sendbuf;
 	lsupdate = (struct ospf6_lsupdate *)((caddr_t)oh
 					     + sizeof(struct ospf6_header));
-	p = (u_char *)((caddr_t)lsupdate + sizeof(struct ospf6_lsupdate));
+	p = (uint8_t *)((caddr_t)lsupdate + sizeof(struct ospf6_lsupdate));
 	lsa_cnt = 0;
 
-	for (lsa = ospf6_lsdb_head(on->retrans_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->retrans_list, lsa)) {
 		/* MTU check */
 		if ((p - sendbuf + (unsigned int)OSPF6_LSA_SIZE(lsa->header))
 		    > ospf6_packet_max(on->ospf6_if)) {
-			ospf6_lsdb_lsa_unlock(lsa);
-			break;
+			if (lsa_cnt) {
+				oh->type = OSPF6_MESSAGE_TYPE_LSUPDATE;
+				oh->length = htons(p - sendbuf);
+				lsupdate->lsa_number = htonl(lsa_cnt);
+
+				if (on->ospf6_if->state
+				    == OSPF6_INTERFACE_POINTTOPOINT) {
+					ospf6_send(on->ospf6_if->linklocal_addr,
+						   &allspfrouters6,
+						   on->ospf6_if, oh);
+				} else {
+					ospf6_send(on->ospf6_if->linklocal_addr,
+						   &on->linklocal_addr,
+						   on->ospf6_if, oh);
+				}
+
+				memset(sendbuf, 0, iobuflen);
+				oh = (struct ospf6_header *)sendbuf;
+				lsupdate = (struct ospf6_lsupdate
+						    *)((caddr_t)oh
+						       + sizeof(struct
+								ospf6_header));
+				p = (uint8_t *)((caddr_t)lsupdate
+						+ sizeof(struct
+							 ospf6_lsupdate));
+				lsa_cnt = 0;
+			}
 		}
 
 		ospf6_lsa_age_update_to_send(lsa, on->ospf6_if->transdelay);
@@ -2065,13 +2157,50 @@ int ospf6_lsupdate_send_neighbor(struct thread *thread)
 				   &on->linklocal_addr, on->ospf6_if, oh);
 	}
 
-	if (on->lsupdate_list->count != 0)
-		on->thread_send_lsupdate = thread_add_event(
-			master, ospf6_lsupdate_send_neighbor, on, 0);
-	else if (on->retrans_list->count != 0)
-		on->thread_send_lsupdate =
-			thread_add_timer(master, ospf6_lsupdate_send_neighbor,
-					 on, on->ospf6_if->rxmt_interval);
+	if (on->lsupdate_list->count != 0) {
+		on->thread_send_lsupdate = NULL;
+		thread_add_event(master, ospf6_lsupdate_send_neighbor, on, 0,
+				 &on->thread_send_lsupdate);
+	} else if (on->retrans_list->count != 0) {
+		on->thread_send_lsupdate = NULL;
+		thread_add_timer(master, ospf6_lsupdate_send_neighbor, on,
+				 on->ospf6_if->rxmt_interval,
+				 &on->thread_send_lsupdate);
+	}
+	return 0;
+}
+
+int ospf6_lsupdate_send_neighbor_now(struct ospf6_neighbor *on,
+				     struct ospf6_lsa *lsa)
+{
+	struct ospf6_header *oh;
+	struct ospf6_lsupdate *lsupdate;
+	uint8_t *p;
+	int lsa_cnt = 0;
+
+	memset(sendbuf, 0, iobuflen);
+	oh = (struct ospf6_header *)sendbuf;
+	lsupdate = (struct ospf6_lsupdate *)((caddr_t)oh
+					     + sizeof(struct ospf6_header));
+
+	p = (uint8_t *)((caddr_t)lsupdate + sizeof(struct ospf6_lsupdate));
+	ospf6_lsa_age_update_to_send(lsa, on->ospf6_if->transdelay);
+	memcpy(p, lsa->header, OSPF6_LSA_SIZE(lsa->header));
+	p += OSPF6_LSA_SIZE(lsa->header);
+	lsa_cnt++;
+
+	oh->type = OSPF6_MESSAGE_TYPE_LSUPDATE;
+	oh->length = htons(p - sendbuf);
+	lsupdate->lsa_number = htonl(lsa_cnt);
+
+	if (IS_OSPF6_DEBUG_FLOODING
+	    || IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_LSUPDATE, SEND))
+		zlog_debug("%s: Send lsupdate with lsa %s (age %u)",
+			   __PRETTY_FUNCTION__, lsa->name,
+			   ntohs(lsa->header->age));
+
+	ospf6_send_lsupdate(on, NULL, oh);
+
 	return 0;
 }
 
@@ -2080,7 +2209,7 @@ int ospf6_lsupdate_send_interface(struct thread *thread)
 	struct ospf6_interface *oi;
 	struct ospf6_header *oh;
 	struct ospf6_lsupdate *lsupdate;
-	u_char *p;
+	uint8_t *p;
 	int lsa_cnt;
 	struct ospf6_lsa *lsa;
 
@@ -2105,16 +2234,37 @@ int ospf6_lsupdate_send_interface(struct thread *thread)
 	lsupdate = (struct ospf6_lsupdate *)((caddr_t)oh
 					     + sizeof(struct ospf6_header));
 
-	p = (u_char *)((caddr_t)lsupdate + sizeof(struct ospf6_lsupdate));
+	p = (uint8_t *)((caddr_t)lsupdate + sizeof(struct ospf6_lsupdate));
 	lsa_cnt = 0;
 
-	for (lsa = ospf6_lsdb_head(oi->lsupdate_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(oi->lsupdate_list, lsa)) {
 		/* MTU check */
 		if ((p - sendbuf + ((unsigned int)OSPF6_LSA_SIZE(lsa->header)))
 		    > ospf6_packet_max(oi)) {
-			ospf6_lsdb_lsa_unlock(lsa);
-			break;
+			if (lsa_cnt) {
+				oh->type = OSPF6_MESSAGE_TYPE_LSUPDATE;
+				oh->length = htons(p - sendbuf);
+				lsupdate->lsa_number = htonl(lsa_cnt);
+
+				ospf6_send_lsupdate(NULL, oi, oh);
+				if (IS_OSPF6_DEBUG_MESSAGE(
+					    OSPF6_MESSAGE_TYPE_LSUPDATE, SEND))
+					zlog_debug("%s: LSUpdate length %d",
+						   __PRETTY_FUNCTION__,
+						   ntohs(oh->length));
+
+				memset(sendbuf, 0, iobuflen);
+				oh = (struct ospf6_header *)sendbuf;
+				lsupdate = (struct ospf6_lsupdate
+						    *)((caddr_t)oh
+						       + sizeof(struct
+								ospf6_header));
+
+				p = (uint8_t *)((caddr_t)lsupdate
+						+ sizeof(struct
+							 ospf6_lsupdate));
+				lsa_cnt = 0;
+			}
 		}
 
 		ospf6_lsa_age_update_to_send(lsa, oi->transdelay);
@@ -2127,22 +2277,17 @@ int ospf6_lsupdate_send_interface(struct thread *thread)
 	}
 
 	if (lsa_cnt) {
-		lsupdate->lsa_number = htonl(lsa_cnt);
-
 		oh->type = OSPF6_MESSAGE_TYPE_LSUPDATE;
 		oh->length = htons(p - sendbuf);
+		lsupdate->lsa_number = htonl(lsa_cnt);
 
-		if ((oi->state == OSPF6_INTERFACE_POINTTOPOINT)
-		    || (oi->state == OSPF6_INTERFACE_DR)
-		    || (oi->state == OSPF6_INTERFACE_BDR))
-			ospf6_send(oi->linklocal_addr, &allspfrouters6, oi, oh);
-		else
-			ospf6_send(oi->linklocal_addr, &alldrouters6, oi, oh);
+		ospf6_send_lsupdate(NULL, oi, oh);
 	}
 
 	if (oi->lsupdate_list->count > 0) {
-		oi->thread_send_lsupdate = thread_add_event(
-			master, ospf6_lsupdate_send_interface, oi, 0);
+		oi->thread_send_lsupdate = NULL;
+		thread_add_event(master, ospf6_lsupdate_send_interface, oi, 0,
+				 &oi->thread_send_lsupdate);
 	}
 
 	return 0;
@@ -2152,7 +2297,7 @@ int ospf6_lsack_send_neighbor(struct thread *thread)
 {
 	struct ospf6_neighbor *on;
 	struct ospf6_header *oh;
-	u_char *p;
+	uint8_t *p;
 	struct ospf6_lsa *lsa;
 	int lsa_cnt = 0;
 
@@ -2174,21 +2319,30 @@ int ospf6_lsack_send_neighbor(struct thread *thread)
 	memset(sendbuf, 0, iobuflen);
 	oh = (struct ospf6_header *)sendbuf;
 
-	p = (u_char *)((caddr_t)oh + sizeof(struct ospf6_header));
+	p = (uint8_t *)((caddr_t)oh + sizeof(struct ospf6_header));
 
-	for (lsa = ospf6_lsdb_head(on->lsack_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->lsack_list, lsa)) {
 		/* MTU check */
 		if (p - sendbuf + sizeof(struct ospf6_lsa_header)
 		    > ospf6_packet_max(on->ospf6_if)) {
 			/* if we run out of packet size/space here,
 			   better to try again soon. */
-			THREAD_OFF(on->thread_send_lsack);
-			on->thread_send_lsack = thread_add_event(
-				master, ospf6_lsack_send_neighbor, on, 0);
+			if (lsa_cnt) {
+				oh->type = OSPF6_MESSAGE_TYPE_LSACK;
+				oh->length = htons(p - sendbuf);
 
-			ospf6_lsdb_lsa_unlock(lsa);
-			break;
+				on->ospf6_if->ls_ack_out++;
+
+				ospf6_send(on->ospf6_if->linklocal_addr,
+					   &on->linklocal_addr, on->ospf6_if,
+					   oh);
+
+				memset(sendbuf, 0, iobuflen);
+				oh = (struct ospf6_header *)sendbuf;
+				p = (uint8_t *)((caddr_t)oh
+						+ sizeof(struct ospf6_header));
+				lsa_cnt = 0;
+			}
 		}
 
 		ospf6_lsa_age_update_to_send(lsa, on->ospf6_if->transdelay);
@@ -2204,14 +2358,15 @@ int ospf6_lsack_send_neighbor(struct thread *thread)
 		oh->type = OSPF6_MESSAGE_TYPE_LSACK;
 		oh->length = htons(p - sendbuf);
 
+		on->ospf6_if->ls_ack_out++;
+
 		ospf6_send(on->ospf6_if->linklocal_addr, &on->linklocal_addr,
 			   on->ospf6_if, oh);
 	}
 
-	if (on->thread_send_lsack == NULL && on->lsack_list->count > 0) {
-		on->thread_send_lsack = thread_add_event(
-			master, ospf6_lsack_send_neighbor, on, 0);
-	}
+	if (on->lsack_list->count > 0)
+		thread_add_event(master, ospf6_lsack_send_neighbor, on, 0,
+				 &on->thread_send_lsack);
 
 	return 0;
 }
@@ -2220,7 +2375,7 @@ int ospf6_lsack_send_interface(struct thread *thread)
 {
 	struct ospf6_interface *oi;
 	struct ospf6_header *oh;
-	u_char *p;
+	uint8_t *p;
 	struct ospf6_lsa *lsa;
 	int lsa_cnt = 0;
 
@@ -2243,18 +2398,17 @@ int ospf6_lsack_send_interface(struct thread *thread)
 	memset(sendbuf, 0, iobuflen);
 	oh = (struct ospf6_header *)sendbuf;
 
-	p = (u_char *)((caddr_t)oh + sizeof(struct ospf6_header));
+	p = (uint8_t *)((caddr_t)oh + sizeof(struct ospf6_header));
 
-	for (lsa = ospf6_lsdb_head(oi->lsack_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(oi->lsack_list, lsa)) {
 		/* MTU check */
 		if (p - sendbuf + sizeof(struct ospf6_lsa_header)
 		    > ospf6_packet_max(oi)) {
 			/* if we run out of packet size/space here,
 			   better to try again soon. */
 			THREAD_OFF(oi->thread_send_lsack);
-			oi->thread_send_lsack = thread_add_event(
-				master, ospf6_lsack_send_interface, oi, 0);
+			thread_add_event(master, ospf6_lsack_send_interface, oi,
+					 0, &oi->thread_send_lsack);
 
 			ospf6_lsdb_lsa_unlock(lsa);
 			break;
@@ -2281,10 +2435,9 @@ int ospf6_lsack_send_interface(struct thread *thread)
 			ospf6_send(oi->linklocal_addr, &alldrouters6, oi, oh);
 	}
 
-	if (oi->thread_send_lsack == NULL && oi->lsack_list->count > 0) {
-		oi->thread_send_lsack = thread_add_event(
-			master, ospf6_lsack_send_interface, oi, 0);
-	}
+	if (oi->lsack_list->count > 0)
+		thread_add_event(master, ospf6_lsack_send_interface, oi, 0,
+				 &oi->thread_send_lsack);
 
 	return 0;
 }
@@ -2416,38 +2569,37 @@ int config_write_ospf6_debug_message(struct vty *vty)
 	}
 
 	if (s == 0x3f && r == 0x3f) {
-		vty_out(vty, "debug ospf6 message all%s", VNL);
+		vty_out(vty, "debug ospf6 message all\n");
 		return 0;
 	}
 
 	if (s == 0x3f && r == 0) {
-		vty_out(vty, "debug ospf6 message all send%s", VNL);
+		vty_out(vty, "debug ospf6 message all send\n");
 		return 0;
 	} else if (s == 0 && r == 0x3f) {
-		vty_out(vty, "debug ospf6 message all recv%s", VNL);
+		vty_out(vty, "debug ospf6 message all recv\n");
 		return 0;
 	}
 
 	/* Unknown message is logged by default */
 	if (!IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_UNKNOWN, SEND)
 	    && !IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_UNKNOWN, RECV))
-		vty_out(vty, "no debug ospf6 message unknown%s", VNL);
+		vty_out(vty, "no debug ospf6 message unknown\n");
 	else if (!IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_UNKNOWN, SEND))
-		vty_out(vty, "no debug ospf6 message unknown send%s", VNL);
+		vty_out(vty, "no debug ospf6 message unknown send\n");
 	else if (!IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_UNKNOWN, RECV))
-		vty_out(vty, "no debug ospf6 message unknown recv%s", VNL);
+		vty_out(vty, "no debug ospf6 message unknown recv\n");
 
 	for (i = 1; i < 6; i++) {
 		if (IS_OSPF6_DEBUG_MESSAGE(i, SEND)
 		    && IS_OSPF6_DEBUG_MESSAGE(i, RECV))
-			vty_out(vty, "debug ospf6 message %s%s", type_str[i],
-				VNL);
+			vty_out(vty, "debug ospf6 message %s\n", type_str[i]);
 		else if (IS_OSPF6_DEBUG_MESSAGE(i, SEND))
-			vty_out(vty, "debug ospf6 message %s send%s",
-				type_str[i], VNL);
+			vty_out(vty, "debug ospf6 message %s send\n",
+				type_str[i]);
 		else if (IS_OSPF6_DEBUG_MESSAGE(i, RECV))
-			vty_out(vty, "debug ospf6 message %s recv%s",
-				type_str[i], VNL);
+			vty_out(vty, "debug ospf6 message %s recv\n",
+				type_str[i]);
 	}
 
 	return 0;
