@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -63,7 +62,7 @@ int ospf6_neighbor_cmp(void *va, void *vb)
 	return (ntohl(ona->router_id) < ntohl(onb->router_id) ? -1 : 1);
 }
 
-struct ospf6_neighbor *ospf6_neighbor_lookup(u_int32_t router_id,
+struct ospf6_neighbor *ospf6_neighbor_lookup(uint32_t router_id,
 					     struct ospf6_interface *oi)
 {
 	struct listnode *n;
@@ -77,7 +76,7 @@ struct ospf6_neighbor *ospf6_neighbor_lookup(u_int32_t router_id,
 }
 
 /* create ospf6_neighbor */
-struct ospf6_neighbor *ospf6_neighbor_create(u_int32_t router_id,
+struct ospf6_neighbor *ospf6_neighbor_create(uint32_t router_id,
 					     struct ospf6_interface *oi)
 {
 	struct ospf6_neighbor *on;
@@ -120,8 +119,7 @@ void ospf6_neighbor_delete(struct ospf6_neighbor *on)
 
 	ospf6_lsdb_remove_all(on->summary_list);
 	ospf6_lsdb_remove_all(on->request_list);
-	for (lsa = ospf6_lsdb_head(on->retrans_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->retrans_list, lsa)) {
 		ospf6_decrement_retrans_count(lsa);
 		ospf6_lsdb_remove(lsa, on->retrans_list);
 	}
@@ -149,10 +147,10 @@ void ospf6_neighbor_delete(struct ospf6_neighbor *on)
 	XFREE(MTYPE_OSPF6_NEIGHBOR, on);
 }
 
-static void ospf6_neighbor_state_change(u_char next_state,
+static void ospf6_neighbor_state_change(uint8_t next_state,
 					struct ospf6_neighbor *on, int event)
 {
-	u_char prev_state;
+	uint8_t prev_state;
 
 	prev_state = on->state;
 	on->state = next_state;
@@ -190,7 +188,20 @@ static void ospf6_neighbor_state_change(u_char next_state,
 			OSPF6_NETWORK_LSA_SCHEDULE(on->ospf6_if);
 			OSPF6_INTRA_PREFIX_LSA_SCHEDULE_TRANSIT(on->ospf6_if);
 		}
+		if (next_state == OSPF6_NEIGHBOR_FULL)
+			on->ospf6_if->area->intra_prefix_originate = 1;
+
 		OSPF6_INTRA_PREFIX_LSA_SCHEDULE_STUB(on->ospf6_if->area);
+
+		if ((prev_state == OSPF6_NEIGHBOR_LOADING ||
+		     prev_state == OSPF6_NEIGHBOR_EXCHANGE) &&
+		    next_state == OSPF6_NEIGHBOR_FULL) {
+			OSPF6_AS_EXTERN_LSA_SCHEDULE(on->ospf6_if);
+			on->ospf6_if->area->full_nbrs++;
+		}
+
+		if (prev_state == OSPF6_NEIGHBOR_FULL)
+			on->ospf6_if->area->full_nbrs--;
 	}
 
 	if ((prev_state == OSPF6_NEIGHBOR_EXCHANGE
@@ -230,8 +241,9 @@ int hello_received(struct thread *thread)
 
 	/* reset Inactivity Timer */
 	THREAD_OFF(on->inactivity_timer);
-	on->inactivity_timer = thread_add_timer(master, inactivity_timer, on,
-						on->ospf6_if->dead_interval);
+	on->inactivity_timer = NULL;
+	thread_add_timer(master, inactivity_timer, on,
+			 on->ospf6_if->dead_interval, &on->inactivity_timer);
 
 	if (on->state <= OSPF6_NEIGHBOR_DOWN)
 		ospf6_neighbor_state_change(OSPF6_NEIGHBOR_INIT, on,
@@ -253,7 +265,7 @@ int twoway_received(struct thread *thread)
 	if (IS_OSPF6_DEBUG_NEIGHBOR(EVENT))
 		zlog_debug("Neighbor Event %s: *2Way-Received*", on->name);
 
-	thread_add_event(master, neighbor_change, on->ospf6_if, 0);
+	thread_add_event(master, neighbor_change, on->ospf6_if, 0, NULL);
 
 	if (!need_adjacency(on)) {
 		ospf6_neighbor_state_change(OSPF6_NEIGHBOR_TWOWAY, on,
@@ -268,8 +280,9 @@ int twoway_received(struct thread *thread)
 	SET_FLAG(on->dbdesc_bits, OSPF6_DBDESC_IBIT);
 
 	THREAD_OFF(on->thread_send_dbdesc);
-	on->thread_send_dbdesc =
-		thread_add_event(master, ospf6_dbdesc_send, on, 0);
+	on->thread_send_dbdesc = NULL;
+	thread_add_event(master, ospf6_dbdesc_send, on, 0,
+			 &on->thread_send_dbdesc);
 
 	return 0;
 }
@@ -291,15 +304,13 @@ int negotiation_done(struct thread *thread)
 	/* clear ls-list */
 	ospf6_lsdb_remove_all(on->summary_list);
 	ospf6_lsdb_remove_all(on->request_list);
-	for (lsa = ospf6_lsdb_head(on->retrans_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->retrans_list, lsa)) {
 		ospf6_decrement_retrans_count(lsa);
 		ospf6_lsdb_remove(lsa, on->retrans_list);
 	}
 
 	/* Interface scoped LSAs */
-	for (lsa = ospf6_lsdb_head(on->ospf6_if->lsdb); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->ospf6_if->lsdb, lsa)) {
 		if (OSPF6_LSA_IS_MAXAGE(lsa)) {
 			ospf6_increment_retrans_count(lsa);
 			ospf6_lsdb_add(ospf6_lsa_copy(lsa), on->retrans_list);
@@ -308,8 +319,7 @@ int negotiation_done(struct thread *thread)
 	}
 
 	/* Area scoped LSAs */
-	for (lsa = ospf6_lsdb_head(on->ospf6_if->area->lsdb); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->ospf6_if->area->lsdb, lsa)) {
 		if (OSPF6_LSA_IS_MAXAGE(lsa)) {
 			ospf6_increment_retrans_count(lsa);
 			ospf6_lsdb_add(ospf6_lsa_copy(lsa), on->retrans_list);
@@ -318,8 +328,7 @@ int negotiation_done(struct thread *thread)
 	}
 
 	/* AS scoped LSAs */
-	for (lsa = ospf6_lsdb_head(on->ospf6_if->area->ospf6->lsdb); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->ospf6_if->area->ospf6->lsdb, lsa)) {
 		if (OSPF6_LSA_IS_MAXAGE(lsa)) {
 			ospf6_increment_retrans_count(lsa);
 			ospf6_lsdb_add(ospf6_lsa_copy(lsa), on->retrans_list);
@@ -362,9 +371,8 @@ int exchange_done(struct thread *thread)
 		ospf6_neighbor_state_change(OSPF6_NEIGHBOR_LOADING, on,
 					    OSPF6_NEIGHBOR_EVENT_EXCHANGE_DONE);
 
-		if (on->thread_send_lsreq == NULL)
-			on->thread_send_lsreq = thread_add_event(
-				master, ospf6_lsreq_send, on, 0);
+		thread_add_event(master, ospf6_lsreq_send, on, 0,
+				 &on->thread_send_lsreq);
 	}
 
 	return 0;
@@ -381,12 +389,13 @@ void ospf6_check_nbr_loading(struct ospf6_neighbor *on)
 	if ((on->state == OSPF6_NEIGHBOR_LOADING)
 	    || (on->state == OSPF6_NEIGHBOR_EXCHANGE)) {
 		if (on->request_list->count == 0)
-			thread_add_event(master, loading_done, on, 0);
+			thread_add_event(master, loading_done, on, 0, NULL);
 		else if (on->last_ls_req == NULL) {
 			if (on->thread_send_lsreq != NULL)
 				THREAD_OFF(on->thread_send_lsreq);
-			on->thread_send_lsreq = thread_add_event(
-				master, ospf6_lsreq_send, on, 0);
+			on->thread_send_lsreq = NULL;
+			thread_add_event(master, ospf6_lsreq_send, on, 0,
+					 &on->thread_send_lsreq);
 		}
 	}
 }
@@ -431,16 +440,16 @@ int adj_ok(struct thread *thread)
 		SET_FLAG(on->dbdesc_bits, OSPF6_DBDESC_IBIT);
 
 		THREAD_OFF(on->thread_send_dbdesc);
-		on->thread_send_dbdesc =
-			thread_add_event(master, ospf6_dbdesc_send, on, 0);
+		on->thread_send_dbdesc = NULL;
+		thread_add_event(master, ospf6_dbdesc_send, on, 0,
+				 &on->thread_send_dbdesc);
 
 	} else if (on->state >= OSPF6_NEIGHBOR_EXSTART && !need_adjacency(on)) {
 		ospf6_neighbor_state_change(OSPF6_NEIGHBOR_TWOWAY, on,
 					    OSPF6_NEIGHBOR_EVENT_ADJ_OK);
 		ospf6_lsdb_remove_all(on->summary_list);
 		ospf6_lsdb_remove_all(on->request_list);
-		for (lsa = ospf6_lsdb_head(on->retrans_list); lsa;
-		     lsa = ospf6_lsdb_next(lsa)) {
+		for (ALL_LSDB(on->retrans_list, lsa)) {
 			ospf6_decrement_retrans_count(lsa);
 			ospf6_lsdb_remove(lsa, on->retrans_list);
 		}
@@ -471,8 +480,7 @@ int seqnumber_mismatch(struct thread *thread)
 
 	ospf6_lsdb_remove_all(on->summary_list);
 	ospf6_lsdb_remove_all(on->request_list);
-	for (lsa = ospf6_lsdb_head(on->retrans_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->retrans_list, lsa)) {
 		ospf6_decrement_retrans_count(lsa);
 		ospf6_lsdb_remove(lsa, on->retrans_list);
 	}
@@ -480,8 +488,9 @@ int seqnumber_mismatch(struct thread *thread)
 	THREAD_OFF(on->thread_send_dbdesc);
 	on->dbdesc_seqnum++; /* Incr seqnum as per RFC2328, sec 10.3 */
 
-	on->thread_send_dbdesc =
-		thread_add_event(master, ospf6_dbdesc_send, on, 0);
+	on->thread_send_dbdesc = NULL;
+	thread_add_event(master, ospf6_dbdesc_send, on, 0,
+			 &on->thread_send_dbdesc);
 
 	return 0;
 }
@@ -508,8 +517,7 @@ int bad_lsreq(struct thread *thread)
 
 	ospf6_lsdb_remove_all(on->summary_list);
 	ospf6_lsdb_remove_all(on->request_list);
-	for (lsa = ospf6_lsdb_head(on->retrans_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->retrans_list, lsa)) {
 		ospf6_decrement_retrans_count(lsa);
 		ospf6_lsdb_remove(lsa, on->retrans_list);
 	}
@@ -517,8 +525,9 @@ int bad_lsreq(struct thread *thread)
 	THREAD_OFF(on->thread_send_dbdesc);
 	on->dbdesc_seqnum++; /* Incr seqnum as per RFC2328, sec 10.3 */
 
-	on->thread_send_dbdesc =
-		thread_add_event(master, ospf6_dbdesc_send, on, 0);
+	on->thread_send_dbdesc = NULL;
+	thread_add_event(master, ospf6_dbdesc_send, on, 0,
+			 &on->thread_send_dbdesc);
 
 	return 0;
 }
@@ -539,12 +548,11 @@ int oneway_received(struct thread *thread)
 
 	ospf6_neighbor_state_change(OSPF6_NEIGHBOR_INIT, on,
 				    OSPF6_NEIGHBOR_EVENT_ONEWAY_RCVD);
-	thread_add_event(master, neighbor_change, on->ospf6_if, 0);
+	thread_add_event(master, neighbor_change, on->ospf6_if, 0, NULL);
 
 	ospf6_lsdb_remove_all(on->summary_list);
 	ospf6_lsdb_remove_all(on->request_list);
-	for (lsa = ospf6_lsdb_head(on->retrans_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa)) {
+	for (ALL_LSDB(on->retrans_list, lsa)) {
 		ospf6_decrement_retrans_count(lsa);
 		ospf6_lsdb_remove(lsa, on->retrans_list);
 	}
@@ -573,7 +581,7 @@ int inactivity_timer(struct thread *thread)
 
 	ospf6_neighbor_state_change(OSPF6_NEIGHBOR_DOWN, on,
 				    OSPF6_NEIGHBOR_EVENT_INACTIVITY_TIMER);
-	thread_add_event(master, neighbor_change, on->ospf6_if, 0);
+	thread_add_event(master, neighbor_change, on->ospf6_if, 0, NULL);
 
 	listnode_delete(on->ospf6_if->neighbor_list, on);
 	ospf6_neighbor_delete(on);
@@ -629,15 +637,15 @@ static void ospf6_neighbor_show(struct vty *vty, struct ospf6_neighbor *on)
 	timerstring(&res, duration, sizeof(duration));
 
 	/*
-	vty_out (vty, "%-15s %3d %11s %6s/%-12s %11s %s[%s]%s",
+	vty_out (vty, "%-15s %3d %11s %6s/%-12s %11s %s[%s]\n",
 		 "Neighbor ID", "Pri", "DeadTime", "State", "", "Duration",
-		 "I/F", "State", VNL);
+		 "I/F", "State");
 	*/
 
-	vty_out(vty, "%-15s %3d %11s %8s/%-12s %11s %s[%s]%s", router_id,
+	vty_out(vty, "%-15s %3d %11s %8s/%-12s %11s %s[%s]\n", router_id,
 		on->priority, deadtime, ospf6_neighbor_state_str[on->state],
 		nstate, duration, on->ospf6_if->interface->name,
-		ospf6_interface_state_str[on->ospf6_if->state], VNL);
+		ospf6_interface_state_str[on->ospf6_if->state]);
 }
 
 static void ospf6_neighbor_show_drchoice(struct vty *vty,
@@ -649,9 +657,9 @@ static void ospf6_neighbor_show_drchoice(struct vty *vty,
 	struct timeval now, res;
 
 	/*
-	    vty_out (vty, "%-15s %6s/%-11s %-15s %-15s %s[%s]%s",
+	    vty_out (vty, "%-15s %6s/%-11s %-15s %-15s %s[%s]\n",
 		     "RouterID", "State", "Duration", "DR", "BDR", "I/F",
-		     "State", VNL);
+		     "State");
 	*/
 
 	inet_ntop(AF_INET, &on->router_id, router_id, sizeof(router_id));
@@ -662,10 +670,10 @@ static void ospf6_neighbor_show_drchoice(struct vty *vty,
 	timersub(&now, &on->last_changed, &res);
 	timerstring(&res, duration, sizeof(duration));
 
-	vty_out(vty, "%-15s %8s/%-11s %-15s %-15s %s[%s]%s", router_id,
+	vty_out(vty, "%-15s %8s/%-11s %-15s %-15s %s[%s]\n", router_id,
 		ospf6_neighbor_state_str[on->state], duration, drouter,
 		bdrouter, on->ospf6_if->interface->name,
-		ospf6_interface_state_str[on->ospf6_if->state], VNL);
+		ospf6_interface_state_str[on->ospf6_if->state]);
 }
 
 static void ospf6_neighbor_show_detail(struct vty *vty,
@@ -685,86 +693,76 @@ static void ospf6_neighbor_show_detail(struct vty *vty,
 	timersub(&now, &on->last_changed, &res);
 	timerstring(&res, duration, sizeof(duration));
 
-	vty_out(vty, " Neighbor %s%s", on->name, VNL);
-	vty_out(vty, "    Area %s via interface %s (ifindex %d)%s",
+	vty_out(vty, " Neighbor %s\n", on->name);
+	vty_out(vty, "    Area %s via interface %s (ifindex %d)\n",
 		on->ospf6_if->area->name, on->ospf6_if->interface->name,
-		on->ospf6_if->interface->ifindex, VNL);
-	vty_out(vty, "    His IfIndex: %d Link-local address: %s%s",
-		on->ifindex, linklocal_addr, VNL);
-	vty_out(vty, "    State %s for a duration of %s%s",
-		ospf6_neighbor_state_str[on->state], duration, VNL);
-	vty_out(vty, "    His choice of DR/BDR %s/%s, Priority %d%s", drouter,
-		bdrouter, on->priority, VNL);
-	vty_out(vty, "    DbDesc status: %s%s%s SeqNum: %#lx%s",
+		on->ospf6_if->interface->ifindex);
+	vty_out(vty, "    His IfIndex: %d Link-local address: %s\n",
+		on->ifindex, linklocal_addr);
+	vty_out(vty, "    State %s for a duration of %s\n",
+		ospf6_neighbor_state_str[on->state], duration);
+	vty_out(vty, "    His choice of DR/BDR %s/%s, Priority %d\n", drouter,
+		bdrouter, on->priority);
+	vty_out(vty, "    DbDesc status: %s%s%s SeqNum: %#lx\n",
 		(CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_IBIT) ? "Initial "
 								: ""),
 		(CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_MBIT) ? "More " : ""),
 		(CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_MSBIT) ? "Master"
 								 : "Slave"),
-		(u_long)ntohl(on->dbdesc_seqnum), VNL);
+		(unsigned long)ntohl(on->dbdesc_seqnum));
 
-	vty_out(vty, "    Summary-List: %d LSAs%s", on->summary_list->count,
-		VNL);
-	for (lsa = ospf6_lsdb_head(on->summary_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa))
-		vty_out(vty, "      %s%s", lsa->name, VNL);
+	vty_out(vty, "    Summary-List: %d LSAs\n", on->summary_list->count);
+	for (ALL_LSDB(on->summary_list, lsa))
+		vty_out(vty, "      %s\n", lsa->name);
 
-	vty_out(vty, "    Request-List: %d LSAs%s", on->request_list->count,
-		VNL);
-	for (lsa = ospf6_lsdb_head(on->request_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa))
-		vty_out(vty, "      %s%s", lsa->name, VNL);
+	vty_out(vty, "    Request-List: %d LSAs\n", on->request_list->count);
+	for (ALL_LSDB(on->request_list, lsa))
+		vty_out(vty, "      %s\n", lsa->name);
 
-	vty_out(vty, "    Retrans-List: %d LSAs%s", on->retrans_list->count,
-		VNL);
-	for (lsa = ospf6_lsdb_head(on->retrans_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa))
-		vty_out(vty, "      %s%s", lsa->name, VNL);
+	vty_out(vty, "    Retrans-List: %d LSAs\n", on->retrans_list->count);
+	for (ALL_LSDB(on->retrans_list, lsa))
+		vty_out(vty, "      %s\n", lsa->name);
 
 	timerclear(&res);
 	if (on->thread_send_dbdesc)
 		timersub(&on->thread_send_dbdesc->u.sands, &now, &res);
 	timerstring(&res, duration, sizeof(duration));
-	vty_out(vty, "    %d Pending LSAs for DbDesc in Time %s [thread %s]%s",
+	vty_out(vty, "    %d Pending LSAs for DbDesc in Time %s [thread %s]\n",
 		on->dbdesc_list->count, duration,
-		(on->thread_send_dbdesc ? "on" : "off"), VNL);
-	for (lsa = ospf6_lsdb_head(on->dbdesc_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa))
-		vty_out(vty, "      %s%s", lsa->name, VNL);
+		(on->thread_send_dbdesc ? "on" : "off"));
+	for (ALL_LSDB(on->dbdesc_list, lsa))
+		vty_out(vty, "      %s\n", lsa->name);
 
 	timerclear(&res);
 	if (on->thread_send_lsreq)
 		timersub(&on->thread_send_lsreq->u.sands, &now, &res);
 	timerstring(&res, duration, sizeof(duration));
-	vty_out(vty, "    %d Pending LSAs for LSReq in Time %s [thread %s]%s",
+	vty_out(vty, "    %d Pending LSAs for LSReq in Time %s [thread %s]\n",
 		on->request_list->count, duration,
-		(on->thread_send_lsreq ? "on" : "off"), VNL);
-	for (lsa = ospf6_lsdb_head(on->request_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa))
-		vty_out(vty, "      %s%s", lsa->name, VNL);
+		(on->thread_send_lsreq ? "on" : "off"));
+	for (ALL_LSDB(on->request_list, lsa))
+		vty_out(vty, "      %s\n", lsa->name);
 
 	timerclear(&res);
 	if (on->thread_send_lsupdate)
 		timersub(&on->thread_send_lsupdate->u.sands, &now, &res);
 	timerstring(&res, duration, sizeof(duration));
 	vty_out(vty,
-		"    %d Pending LSAs for LSUpdate in Time %s [thread %s]%s",
+		"    %d Pending LSAs for LSUpdate in Time %s [thread %s]\n",
 		on->lsupdate_list->count, duration,
-		(on->thread_send_lsupdate ? "on" : "off"), VNL);
-	for (lsa = ospf6_lsdb_head(on->lsupdate_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa))
-		vty_out(vty, "      %s%s", lsa->name, VNL);
+		(on->thread_send_lsupdate ? "on" : "off"));
+	for (ALL_LSDB(on->lsupdate_list, lsa))
+		vty_out(vty, "      %s\n", lsa->name);
 
 	timerclear(&res);
 	if (on->thread_send_lsack)
 		timersub(&on->thread_send_lsack->u.sands, &now, &res);
 	timerstring(&res, duration, sizeof(duration));
-	vty_out(vty, "    %d Pending LSAs for LSAck in Time %s [thread %s]%s",
+	vty_out(vty, "    %d Pending LSAs for LSAck in Time %s [thread %s]\n",
 		on->lsack_list->count, duration,
-		(on->thread_send_lsack ? "on" : "off"), VNL);
-	for (lsa = ospf6_lsdb_head(on->lsack_list); lsa;
-	     lsa = ospf6_lsdb_next(lsa))
-		vty_out(vty, "      %s%s", lsa->name, VNL);
+		(on->thread_send_lsack ? "on" : "off"));
+	for (ALL_LSDB(on->lsack_list, lsa))
+		vty_out(vty, "      %s\n", lsa->name);
 
 	ospf6_bfd_show_info(vty, on->bfd_info, 0);
 }
@@ -797,12 +795,12 @@ DEFUN (show_ipv6_ospf6_neighbor,
 	}
 
 	if (showfunc == ospf6_neighbor_show)
-		vty_out(vty, "%-15s %3s %11s %8s/%-12s %11s %s[%s]%s",
+		vty_out(vty, "%-15s %3s %11s %8s/%-12s %11s %s[%s]\n",
 			"Neighbor ID", "Pri", "DeadTime", "State", "IfState",
-			"Duration", "I/F", "State", VNL);
+			"Duration", "I/F", "State");
 	else if (showfunc == ospf6_neighbor_show_drchoice)
-		vty_out(vty, "%-15s %8s/%-11s %-15s %-15s %s[%s]%s", "RouterID",
-			"State", "Duration", "DR", "BDR", "I/F", "State", VNL);
+		vty_out(vty, "%-15s %8s/%-11s %-15s %-15s %s[%s]\n", "RouterID",
+			"State", "Duration", "DR", "BDR", "I/F", "State");
 
 	for (ALL_LIST_ELEMENTS_RO(ospf6->area_list, i, oa))
 		for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi))
@@ -829,14 +827,14 @@ DEFUN (show_ipv6_ospf6_neighbor_one,
 	struct ospf6_area *oa;
 	struct listnode *i, *j, *k;
 	void (*showfunc)(struct vty *, struct ospf6_neighbor *);
-	u_int32_t router_id;
+	uint32_t router_id;
 
 	OSPF6_CMD_CHECK_RUNNING();
 	showfunc = ospf6_neighbor_show_detail;
 
 	if ((inet_pton(AF_INET, argv[idx_ipv4]->arg, &router_id)) != 1) {
-		vty_out(vty, "Router-ID is not parsable: %s%s",
-			argv[idx_ipv4]->arg, VNL);
+		vty_out(vty, "Router-ID is not parsable: %s\n",
+			argv[idx_ipv4]->arg);
 		return CMD_SUCCESS;
 	}
 
@@ -912,7 +910,7 @@ DEFUN (no_debug_ospf6,
        DEBUG_STR
        OSPF6_STR)
 {
-	u_int i;
+	unsigned int i;
 	struct ospf6_lsa_handler *handler = NULL;
 
 	OSPF6_DEBUG_ABR_OFF();
@@ -953,11 +951,11 @@ DEFUN (no_debug_ospf6,
 int config_write_ospf6_debug_neighbor(struct vty *vty)
 {
 	if (IS_OSPF6_DEBUG_NEIGHBOR(STATE) && IS_OSPF6_DEBUG_NEIGHBOR(EVENT))
-		vty_out(vty, "debug ospf6 neighbor%s", VNL);
+		vty_out(vty, "debug ospf6 neighbor\n");
 	else if (IS_OSPF6_DEBUG_NEIGHBOR(STATE))
-		vty_out(vty, "debug ospf6 neighbor state%s", VNL);
+		vty_out(vty, "debug ospf6 neighbor state\n");
 	else if (IS_OSPF6_DEBUG_NEIGHBOR(EVENT))
-		vty_out(vty, "debug ospf6 neighbor event%s", VNL);
+		vty_out(vty, "debug ospf6 neighbor event\n");
 	return 0;
 }
 
