@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 /* Shortest Path First calculation for OSPFv3 */
@@ -111,37 +110,43 @@ static struct ospf6_vertex *ospf6_vertex_create(struct ospf6_lsa *lsa)
 					   sizeof(struct ospf6_vertex));
 
 	/* type */
-	if (ntohs(lsa->header->type) == OSPF6_LSTYPE_ROUTER)
+	if (ntohs(lsa->header->type) == OSPF6_LSTYPE_ROUTER) {
 		v->type = OSPF6_VERTEX_TYPE_ROUTER;
-	else if (ntohs(lsa->header->type) == OSPF6_LSTYPE_NETWORK)
+		/* Router LSA use Link ID 0 as base in vertex_id */
+		ospf6_linkstate_prefix(lsa->header->adv_router, htonl(0),
+				       &v->vertex_id);
+	} else if (ntohs(lsa->header->type) == OSPF6_LSTYPE_NETWORK) {
 		v->type = OSPF6_VERTEX_TYPE_NETWORK;
-	else
+		/* vertex_id */
+		ospf6_linkstate_prefix(lsa->header->adv_router, lsa->header->id,
+				       &v->vertex_id);
+	} else
 		assert(0);
 
-	/* vertex_id */
-	ospf6_linkstate_prefix(lsa->header->adv_router, lsa->header->id,
-			       &v->vertex_id);
-
 	/* name */
 	ospf6_linkstate_prefix2str(&v->vertex_id, v->name, sizeof(v->name));
 
 	if (IS_OSPF6_DEBUG_SPF(PROCESS))
-		zlog_debug("%s: Creating vertex %s of type %s", __func__,
-			   v->name,
+		zlog_debug("%s: Creating vertex %s of type %s (0x%04hx) lsa %s",
+			   __func__, v->name,
 			   ((ntohs(lsa->header->type) == OSPF6_LSTYPE_ROUTER)
 				    ? "Router"
-				    : "N/W"));
+				    : "N/W"),
+			   ntohs(lsa->header->type), lsa->name);
+
 
 	/* Associated LSA */
 	v->lsa = lsa;
 
 	/* capability bits + options */
-	v->capability = *(u_char *)(OSPF6_LSA_HEADER_END(lsa->header));
-	v->options[0] = *(u_char *)(OSPF6_LSA_HEADER_END(lsa->header) + 1);
-	v->options[1] = *(u_char *)(OSPF6_LSA_HEADER_END(lsa->header) + 2);
-	v->options[2] = *(u_char *)(OSPF6_LSA_HEADER_END(lsa->header) + 3);
+	v->capability = *(uint8_t *)(OSPF6_LSA_HEADER_END(lsa->header));
+	v->options[0] = *(uint8_t *)(OSPF6_LSA_HEADER_END(lsa->header) + 1);
+	v->options[1] = *(uint8_t *)(OSPF6_LSA_HEADER_END(lsa->header) + 2);
+	v->options[2] = *(uint8_t *)(OSPF6_LSA_HEADER_END(lsa->header) + 3);
 
 	v->nh_list = list_new();
+	v->nh_list->cmp = (int (*)(void *, void *))ospf6_nexthop_cmp;
+	v->nh_list->del = (void (*)(void *))ospf6_nexthop_delete;
 
 	v->parent = NULL;
 	v->child_list = list_new();
@@ -152,19 +157,17 @@ static struct ospf6_vertex *ospf6_vertex_create(struct ospf6_lsa *lsa)
 
 static void ospf6_vertex_delete(struct ospf6_vertex *v)
 {
-	ospf6_free_nexthops(v->nh_list);
-	list_delete(v->nh_list);
-
-	list_delete(v->child_list);
+	list_delete_and_null(&v->nh_list);
+	list_delete_and_null(&v->child_list);
 	XFREE(MTYPE_OSPF6_VERTEX, v);
 }
 
 static struct ospf6_lsa *ospf6_lsdesc_lsa(caddr_t lsdesc,
 					  struct ospf6_vertex *v)
 {
-	struct ospf6_lsa *lsa;
-	u_int16_t type = 0;
-	u_int32_t id = 0, adv_router = 0;
+	struct ospf6_lsa *lsa = NULL;
+	uint16_t type = 0;
+	uint32_t id = 0, adv_router = 0;
 
 	if (VERTEX_IS_TYPE(NETWORK, v)) {
 		type = htons(OSPF6_LSTYPE_ROUTER);
@@ -182,17 +185,22 @@ static struct ospf6_lsa *ospf6_lsdesc_lsa(caddr_t lsdesc,
 		}
 	}
 
-	lsa = ospf6_lsdb_lookup(type, id, adv_router, v->area->lsdb);
-
+	if (type == htons(OSPF6_LSTYPE_NETWORK))
+		lsa = ospf6_lsdb_lookup(type, id, adv_router, v->area->lsdb);
+	else
+		lsa = ospf6_create_single_router_lsa(v->area, v->area->lsdb,
+						     adv_router);
 	if (IS_OSPF6_DEBUG_SPF(PROCESS)) {
 		char ibuf[16], abuf[16];
 		inet_ntop(AF_INET, &id, ibuf, sizeof(ibuf));
 		inet_ntop(AF_INET, &adv_router, abuf, sizeof(abuf));
 		if (lsa)
-			zlog_debug("  Link to: %s", lsa->name);
+			zlog_debug("  Link to: %s len %u, V %s", lsa->name,
+				   ntohs(lsa->header->length), v->name);
 		else
-			zlog_debug("  Link to: [%s Id:%s Adv:%s] No LSA",
-				   ospf6_lstype_name(type), ibuf, abuf);
+			zlog_debug("  Link to: [%s Id:%s Adv:%s] No LSA , V %s",
+				   ospf6_lstype_name(type), ibuf, abuf,
+				   v->name);
 	}
 
 	return lsa;
@@ -242,7 +250,8 @@ static char *ospf6_lsdesc_backlink(struct ospf6_lsa *lsa, caddr_t lsdesc,
 	}
 
 	if (IS_OSPF6_DEBUG_SPF(PROCESS))
-		zlog_debug("  Backlink %s", (found ? "OK" : "FAIL"));
+		zlog_debug("Vertex %s Lsa %s Backlink %s", v->name, lsa->name,
+			   (found ? "OK" : "FAIL"));
 
 	return found;
 }
@@ -253,8 +262,8 @@ static void ospf6_nexthop_calc(struct ospf6_vertex *w, struct ospf6_vertex *v,
 	int i;
 	ifindex_t ifindex;
 	struct ospf6_interface *oi;
-	u_int16_t type;
-	u_int32_t adv_router;
+	uint16_t type;
+	uint32_t adv_router;
 	struct ospf6_lsa *lsa;
 	struct ospf6_link_lsa *link_lsa;
 	char buf[64];
@@ -281,8 +290,7 @@ static void ospf6_nexthop_calc(struct ospf6_vertex *w, struct ospf6_vertex *v,
 			      : ROUTER_LSDESC_GET_NBR_ROUTERID(lsdesc));
 
 	i = 0;
-	for (lsa = ospf6_lsdb_type_router_head(type, adv_router, oi->lsdb); lsa;
-	     lsa = ospf6_lsdb_type_router_next(type, adv_router, lsa)) {
+	for (ALL_LSDB_TYPED_ADVRTR(oi->lsdb, type, adv_router, lsa)) {
 		if (VERTEX_IS_TYPE(ROUTER, v)
 		    && htonl(ROUTER_LSDESC_GET_NBR_IFID(lsdesc))
 			       != lsa->header->id)
@@ -310,10 +318,11 @@ static int ospf6_spf_install(struct ospf6_vertex *v,
 {
 	struct ospf6_route *route, *parent_route;
 	struct ospf6_vertex *prev;
+	char pbuf[PREFIX2STR_BUFFER];
 
 	if (IS_OSPF6_DEBUG_SPF(PROCESS))
-		zlog_debug("SPF install %s hops %d cost %d", v->name, v->hops,
-			   v->cost);
+		zlog_debug("SPF install %s (lsa %s) hops %d cost %d", v->name,
+			   v->lsa->name, v->hops, v->cost);
 
 	route = ospf6_route_lookup(&v->vertex_id, result_table);
 	if (route && route->path.cost < v->cost) {
@@ -324,15 +333,30 @@ static int ospf6_spf_install(struct ospf6_vertex *v,
 		ospf6_vertex_delete(v);
 		return -1;
 	} else if (route && route->path.cost == v->cost) {
-		if (IS_OSPF6_DEBUG_SPF(PROCESS))
-			zlog_debug("  another path found, merge");
-
+		if (IS_OSPF6_DEBUG_SPF(PROCESS)) {
+			prefix2str(&route->prefix, pbuf, sizeof(pbuf));
+			zlog_debug(
+				"  another path found to route %s lsa %s, merge",
+				pbuf, v->lsa->name);
+		}
 		ospf6_spf_merge_nexthops_to_route(route, v);
 
 		prev = (struct ospf6_vertex *)route->route_option;
 		assert(prev->hops <= v->hops);
-		ospf6_vertex_delete(v);
 
+		if ((VERTEX_IS_TYPE(ROUTER, v)
+		     && route->path.origin.id != v->lsa->header->id)) {
+			if (IS_OSPF6_DEBUG_SPF(PROCESS)) {
+				zlog_debug(
+					"%s: V lsa %s id %u, route id %u are different",
+					__PRETTY_FUNCTION__, v->lsa->name,
+					ntohl(v->lsa->header->id),
+					ntohl(route->path.origin.id));
+			}
+			return 0;
+		}
+
+		ospf6_vertex_delete(v);
 		return -1;
 	}
 
@@ -432,7 +456,7 @@ void ospf6_spf_reason_string(unsigned int reason, char *buf, int size)
 
 /* RFC2328 16.1.  Calculating the shortest-path tree for an area */
 /* RFC2740 3.8.1.  Calculating the shortest path tree for an area */
-void ospf6_spf_calculation(u_int32_t router_id,
+void ospf6_spf_calculation(uint32_t router_id,
 			   struct ospf6_route_table *result_table,
 			   struct ospf6_area *oa)
 {
@@ -447,8 +471,7 @@ void ospf6_spf_calculation(u_int32_t router_id,
 
 	/* Install the calculating router itself as the root of the SPF tree */
 	/* construct root vertex */
-	lsa = ospf6_lsdb_lookup(htons(OSPF6_LSTYPE_ROUTER), htonl(0), router_id,
-				oa->lsdb_self);
+	lsa = ospf6_create_single_router_lsa(oa, oa->lsdb_self, router_id);
 	if (lsa == NULL) {
 		if (IS_OSPF6_DEBUG_SPF(PROCESS))
 			zlog_debug("%s: No router LSA for area %s\n", __func__,
@@ -464,6 +487,7 @@ void ospf6_spf_calculation(u_int32_t router_id,
 	root->area = oa;
 	root->cost = 0;
 	root->hops = 0;
+	root->link_id = lsa->header->id;
 	inet_pton(AF_INET6, "::1", &address);
 
 	/* Actually insert root to the candidate-list as the only candidate */
@@ -510,8 +534,8 @@ void ospf6_spf_calculation(u_int32_t router_id,
 				w->hops =
 					v->hops
 					+ (VERTEX_IS_TYPE(NETWORK, w) ? 0 : 1);
-			} else /* NETWORK */
-			{
+			} else {
+				/* NETWORK */
 				w->cost = v->cost;
 				w->hops = v->hops + 1;
 			}
@@ -523,9 +547,9 @@ void ospf6_spf_calculation(u_int32_t router_id,
 					ROUTER_LSDESC_GET_IFID(lsdesc), NULL);
 			else if (w->hops == 1 && v->hops == 0)
 				ospf6_nexthop_calc(w, v, lsdesc);
-			else {
+			else
 				ospf6_copy_nexthops(w->nh_list, v->nh_list);
-			}
+
 
 			/* add new candidate to the candidate_list */
 			if (IS_OSPF6_DEBUG_SPF(PROCESS))
@@ -536,8 +560,11 @@ void ospf6_spf_calculation(u_int32_t router_id,
 		}
 	}
 
+
 	pqueue_delete(candidate_list);
 
+	ospf6_remove_temp_router_lsa(oa);
+
 	oa->spf_calculation++;
 }
 
@@ -579,6 +606,7 @@ static int ospf6_spf_calculation_thread(struct thread *t)
 
 	/* execute SPF calculation */
 	monotime(&start);
+	ospf6->ts_spf = start;
 
 	if (ospf6_is_router_abr(ospf6))
 		ospf6_abr_range_reset_cost(ospf6);
@@ -588,6 +616,7 @@ static int ospf6_spf_calculation_thread(struct thread *t)
 		if (oa == ospf6->backbone)
 			continue;
 
+		monotime(&oa->ts_spf);
 		if (IS_OSPF6_DEBUG_SPF(PROCESS))
 			zlog_debug("SPF calculation for Area %s", oa->name);
 		if (IS_OSPF6_DEBUG_SPF(DATABASE))
@@ -601,6 +630,7 @@ static int ospf6_spf_calculation_thread(struct thread *t)
 	}
 
 	if (ospf6->backbone) {
+		monotime(&ospf6->backbone->ts_spf);
 		if (IS_OSPF6_DEBUG_SPF(PROCESS))
 			zlog_debug("SPF calculation for Backbone area %s",
 				   ospf6->backbone->name);
@@ -635,6 +665,7 @@ static int ospf6_spf_calculation_thread(struct thread *t)
 		"Reason: %s\n",
 		areas_processed, (long long)runtime.tv_sec,
 		(long long)runtime.tv_usec, rbuf);
+
 	ospf6->last_spf_reason = ospf6->spf_reason;
 	ospf6_reset_spf_reason(ospf6);
 	return 0;
@@ -699,8 +730,9 @@ void ospf6_spf_schedule(struct ospf6 *ospf6, unsigned int reason)
 
 	zlog_info("SPF: Scheduled in %ld msec", delay);
 
-	ospf6->t_spf_calc = thread_add_timer_msec(
-		master, ospf6_spf_calculation_thread, ospf6, delay);
+	ospf6->t_spf_calc = NULL;
+	thread_add_timer_msec(master, ospf6_spf_calculation_thread, ospf6,
+			      delay, &ospf6->t_spf_calc);
 }
 
 void ospf6_spf_display_subtree(struct vty *vty, const char *prefix, int rest,
@@ -713,12 +745,12 @@ void ospf6_spf_display_subtree(struct vty *vty, const char *prefix, int rest,
 	int restnum;
 
 	/* "prefix" is the space prefix of the display line */
-	vty_out(vty, "%s+-%s [%d]%s", prefix, v->name, v->cost, VNL);
+	vty_out(vty, "%s+-%s [%d]\n", prefix, v->name, v->cost);
 
 	len = strlen(prefix) + 4;
 	next_prefix = (char *)malloc(len);
 	if (next_prefix == NULL) {
-		vty_out(vty, "malloc failed%s", VNL);
+		vty_out(vty, "malloc failed\n");
 		return;
 	}
 	snprintf(next_prefix, len, "%s%s", prefix, (rest ? "|  " : "   "));
@@ -852,12 +884,9 @@ DEFUN (ospf6_timers_throttle_spf,
 	int idx_number_3 = 5;
 	unsigned int delay, hold, max;
 
-	VTY_GET_INTEGER_RANGE("SPF delay timer", delay, argv[idx_number]->arg,
-			      0, 600000);
-	VTY_GET_INTEGER_RANGE("SPF hold timer", hold, argv[idx_number_2]->arg,
-			      0, 600000);
-	VTY_GET_INTEGER_RANGE("SPF max-hold timer", max,
-			      argv[idx_number_3]->arg, 0, 600000);
+	delay = strtoul(argv[idx_number]->arg, NULL, 10);
+	hold = strtoul(argv[idx_number_2]->arg, NULL, 10);
+	max = strtoul(argv[idx_number_3]->arg, NULL, 10);
 
 	return ospf6_timers_spf_set(vty, delay, hold, max);
 }
@@ -882,11 +911,11 @@ DEFUN (no_ospf6_timers_throttle_spf,
 int config_write_ospf6_debug_spf(struct vty *vty)
 {
 	if (IS_OSPF6_DEBUG_SPF(PROCESS))
-		vty_out(vty, "debug ospf6 spf process%s", VNL);
+		vty_out(vty, "debug ospf6 spf process\n");
 	if (IS_OSPF6_DEBUG_SPF(TIME))
-		vty_out(vty, "debug ospf6 spf time%s", VNL);
+		vty_out(vty, "debug ospf6 spf time\n");
 	if (IS_OSPF6_DEBUG_SPF(DATABASE))
-		vty_out(vty, "debug ospf6 spf database%s", VNL);
+		vty_out(vty, "debug ospf6 spf database\n");
 	return 0;
 }
 
@@ -896,9 +925,9 @@ void ospf6_spf_config_write(struct vty *vty)
 	if (ospf6->spf_delay != OSPF_SPF_DELAY_DEFAULT
 	    || ospf6->spf_holdtime != OSPF_SPF_HOLDTIME_DEFAULT
 	    || ospf6->spf_max_holdtime != OSPF_SPF_MAX_HOLDTIME_DEFAULT)
-		vty_out(vty, " timers throttle spf %d %d %d%s",
+		vty_out(vty, " timers throttle spf %d %d %d\n",
 			ospf6->spf_delay, ospf6->spf_holdtime,
-			ospf6->spf_max_holdtime, VTY_NEWLINE);
+			ospf6->spf_max_holdtime);
 }
 
 void install_element_ospf6_debug_spf(void)
@@ -922,3 +951,156 @@ void ospf6_spf_init(void)
 	install_element(OSPF6_NODE, &ospf6_timers_throttle_spf_cmd);
 	install_element(OSPF6_NODE, &no_ospf6_timers_throttle_spf_cmd);
 }
+
+/* Create Aggregated Large Router-LSA from multiple Link-State IDs
+ * RFC 5340 A 4.3:
+ * When more than one router-LSA is received from a single router,
+ * the links are processed as if concatenated into a single LSA.*/
+struct ospf6_lsa *ospf6_create_single_router_lsa(struct ospf6_area *area,
+						 struct ospf6_lsdb *lsdb,
+						 uint32_t adv_router)
+{
+	struct ospf6_lsa *lsa = NULL;
+	struct ospf6_lsa *rtr_lsa = NULL;
+	struct ospf6_lsa_header *lsa_header = NULL;
+	uint8_t *new_header = NULL;
+	const struct route_node *end = NULL;
+	uint16_t lsa_length, total_lsa_length = 0, num_lsa = 0;
+	uint16_t type = 0;
+	char ifbuf[16];
+	uint32_t interface_id;
+	caddr_t lsd;
+
+	lsa_length = sizeof(struct ospf6_lsa_header)
+		     + sizeof(struct ospf6_router_lsa);
+	total_lsa_length = lsa_length;
+	type = htons(OSPF6_LSTYPE_ROUTER);
+
+	/* First check Aggregated LSA formed earlier in Cache */
+	lsa = ospf6_lsdb_lookup(type, htonl(0), adv_router,
+				area->temp_router_lsa_lsdb);
+	if (lsa)
+		return lsa;
+
+	inet_ntop(AF_INET, &adv_router, ifbuf, sizeof(ifbuf));
+
+	/* Determine total LSA length from all link state ids */
+	end = ospf6_lsdb_head(lsdb, 2, type, adv_router, &rtr_lsa);
+	while (rtr_lsa) {
+		lsa = rtr_lsa;
+		if (OSPF6_LSA_IS_MAXAGE(rtr_lsa)) {
+			rtr_lsa = ospf6_lsdb_next(end, rtr_lsa);
+			continue;
+		}
+		lsa_header = (struct ospf6_lsa_header *)rtr_lsa->header;
+		total_lsa_length += (ntohs(lsa_header->length) - lsa_length);
+		num_lsa++;
+		rtr_lsa = ospf6_lsdb_next(end, rtr_lsa);
+	}
+	if (IS_OSPF6_DEBUG_SPF(PROCESS))
+		zlog_debug("%s: adv_router %s num_lsa %u to convert.",
+			   __PRETTY_FUNCTION__, ifbuf, num_lsa);
+	if (num_lsa == 1)
+		return lsa;
+
+	if (num_lsa == 0) {
+		if (IS_OSPF6_DEBUG_SPF(PROCESS))
+			zlog_debug("%s: adv_router %s not found in LSDB.",
+				   __PRETTY_FUNCTION__, ifbuf);
+		return NULL;
+	}
+
+	/* Allocate memory for this LSA */
+	new_header = XMALLOC(MTYPE_OSPF6_LSA_HEADER, total_lsa_length);
+	if (!new_header)
+		return NULL;
+
+	/* LSA information structure */
+	lsa = (struct ospf6_lsa *)XCALLOC(MTYPE_OSPF6_LSA,
+					  sizeof(struct ospf6_lsa));
+	if (!lsa) {
+		free(new_header);
+		return NULL;
+	}
+
+	lsa->header = (struct ospf6_lsa_header *)new_header;
+
+	lsa->lsdb = area->temp_router_lsa_lsdb;
+
+	/* Fill Larger LSA Payload */
+	end = ospf6_lsdb_head(lsdb, 2, type, adv_router, &rtr_lsa);
+
+	/*
+	 * We assume at this point in time that rtr_lsa is
+	 * a valid pointer.
+	 */
+	assert(rtr_lsa);
+	if (!OSPF6_LSA_IS_MAXAGE(rtr_lsa)) {
+		/* Append first Link State ID LSA */
+		lsa_header = (struct ospf6_lsa_header *)rtr_lsa->header;
+		memcpy(new_header, lsa_header, ntohs(lsa_header->length));
+		/* Assign new lsa length as aggregated length. */
+		((struct ospf6_lsa_header *)new_header)->length =
+			htons(total_lsa_length);
+		new_header += ntohs(lsa_header->length);
+		num_lsa--;
+	}
+
+	/* Print LSA Name */
+	ospf6_lsa_printbuf(lsa, lsa->name, sizeof(lsa->name));
+
+	rtr_lsa = ospf6_lsdb_next(end, rtr_lsa);
+	while (rtr_lsa) {
+		if (OSPF6_LSA_IS_MAXAGE(rtr_lsa)) {
+			rtr_lsa = ospf6_lsdb_next(end, rtr_lsa);
+			continue;
+		}
+
+		if (IS_OSPF6_DEBUG_SPF(PROCESS)) {
+			lsd = OSPF6_LSA_HEADER_END(rtr_lsa->header) + 4;
+			interface_id = ROUTER_LSDESC_GET_IFID(lsd);
+			inet_ntop(AF_INET, &interface_id, ifbuf, sizeof(ifbuf));
+			zlog_debug(
+				"%s: Next Router LSA %s to aggreat with len %u interface_id %s",
+				__PRETTY_FUNCTION__, rtr_lsa->name,
+				ntohs(lsa_header->length), ifbuf);
+		}
+
+		/* Append Next Link State ID LSA */
+		lsa_header = (struct ospf6_lsa_header *)rtr_lsa->header;
+		memcpy(new_header, (OSPF6_LSA_HEADER_END(rtr_lsa->header) + 4),
+		       (ntohs(lsa_header->length) - lsa_length));
+		new_header += (ntohs(lsa_header->length) - lsa_length);
+		num_lsa--;
+
+		rtr_lsa = ospf6_lsdb_next(end, rtr_lsa);
+	}
+
+	/* Calculate birth of this lsa */
+	ospf6_lsa_age_set(lsa);
+
+	/* Store Aggregated LSA into area temp lsdb */
+	ospf6_lsdb_add(lsa, area->temp_router_lsa_lsdb);
+
+	if (IS_OSPF6_DEBUG_SPF(PROCESS))
+		zlog_debug("%s: LSA %s id %u type 0%x len %u num_lsa %u",
+			   __PRETTY_FUNCTION__, lsa->name,
+			   ntohl(lsa->header->id), ntohs(lsa->header->type),
+			   ntohs(lsa->header->length), num_lsa);
+
+	return lsa;
+}
+
+void ospf6_remove_temp_router_lsa(struct ospf6_area *area)
+{
+	struct ospf6_lsa *lsa = NULL;
+
+	for (ALL_LSDB(area->temp_router_lsa_lsdb, lsa)) {
+		if (IS_OSPF6_DEBUG_SPF(PROCESS))
+			zlog_debug(
+				"%s Remove LSA %s lsa->lock %u lsdb count %u",
+				__PRETTY_FUNCTION__, lsa->name, lsa->lock,
+				area->temp_router_lsa_lsdb->count);
+		ospf6_lsdb_remove(lsa, area->temp_router_lsa_lsdb);
+	}
+}
