@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -54,6 +53,8 @@ DEFINE_QOBJ_TYPE(ospf6)
 
 /* global ospf6d variable */
 struct ospf6 *ospf6;
+static struct ospf6_master ospf6_master;
+struct ospf6_master *om6;
 
 static void ospf6_disable(struct ospf6 *o);
 
@@ -73,7 +74,7 @@ static void ospf6_top_lsdb_hook_remove(struct ospf6_lsa *lsa)
 {
 	switch (ntohs(lsa->header->type)) {
 	case OSPF6_LSTYPE_AS_EXTERNAL:
-		ospf6_asbr_lsa_remove(lsa);
+		ospf6_asbr_lsa_remove(lsa, NULL);
 		break;
 
 	default:
@@ -96,6 +97,18 @@ static void ospf6_top_route_hook_remove(struct ospf6_route *route)
 
 static void ospf6_top_brouter_hook_add(struct ospf6_route *route)
 {
+	if (IS_OSPF6_DEBUG_EXAMIN(AS_EXTERNAL)) {
+		uint32_t brouter_id;
+		char brouter_name[16];
+
+		brouter_id = ADV_ROUTER_IN_PREFIX(&route->prefix);
+		inet_ntop(AF_INET, &brouter_id, brouter_name,
+			  sizeof(brouter_name));
+		zlog_debug("%s: brouter %s add with adv router %x nh count %u",
+			   __PRETTY_FUNCTION__, brouter_name,
+			   route->path.origin.adv_router,
+			   listcount(route->nh_list));
+	}
 	ospf6_abr_examin_brouter(ADV_ROUTER_IN_PREFIX(&route->prefix));
 	ospf6_asbr_lsentry_add(route);
 	ospf6_abr_originate_summary(route);
@@ -103,6 +116,17 @@ static void ospf6_top_brouter_hook_add(struct ospf6_route *route)
 
 static void ospf6_top_brouter_hook_remove(struct ospf6_route *route)
 {
+	if (IS_OSPF6_DEBUG_EXAMIN(AS_EXTERNAL)) {
+		uint32_t brouter_id;
+		char brouter_name[16];
+
+		brouter_id = ADV_ROUTER_IN_PREFIX(&route->prefix);
+		inet_ntop(AF_INET, &brouter_id, brouter_name,
+			  sizeof(brouter_name));
+		zlog_debug("%s: brouter %s del with nh count %u",
+			   __PRETTY_FUNCTION__, brouter_name,
+			   listcount(route->nh_list));
+	}
 	route->flag |= OSPF6_ROUTE_REMOVE;
 	ospf6_abr_examin_brouter(ADV_ROUTER_IN_PREFIX(&route->prefix));
 	ospf6_asbr_lsentry_remove(route);
@@ -167,13 +191,15 @@ void ospf6_delete(struct ospf6 *o)
 	struct ospf6_area *oa;
 
 	QOBJ_UNREG(o);
+
+	ospf6_flush_self_originated_lsas_now();
 	ospf6_disable(ospf6);
 
 	for (ALL_LIST_ELEMENTS(o->area_list, node, nnode, oa))
 		ospf6_area_delete(oa);
 
 
-	list_delete(o->area_list);
+	list_delete_and_null(&o->area_list);
 
 	ospf6_lsdb_delete(o->lsdb);
 	ospf6_lsdb_delete(o->lsdb_self);
@@ -211,9 +237,17 @@ static void ospf6_disable(struct ospf6 *o)
 		THREAD_OFF(o->maxage_remover);
 		THREAD_OFF(o->t_spf_calc);
 		THREAD_OFF(o->t_ase_calc);
+		THREAD_OFF(o->t_distribute_update);
 	}
 }
 
+void ospf6_master_init(void)
+{
+	memset(&ospf6_master, 0, sizeof(struct ospf6_master));
+
+	om6 = &ospf6_master;
+}
+
 static int ospf6_maxage_remover(struct thread *thread)
 {
 	struct ospf6 *o = (struct ospf6 *)THREAD_ARG(thread);
@@ -263,10 +297,21 @@ static int ospf6_maxage_remover(struct thread *thread)
 
 void ospf6_maxage_remove(struct ospf6 *o)
 {
-	if (o && !o->maxage_remover)
-		o->maxage_remover =
-			thread_add_timer(master, ospf6_maxage_remover, o,
-					 OSPF_LSA_MAXAGE_REMOVE_DELAY_DEFAULT);
+	if (o)
+		thread_add_timer(master, ospf6_maxage_remover, o,
+				 OSPF_LSA_MAXAGE_REMOVE_DELAY_DEFAULT,
+				 &o->maxage_remover);
+}
+
+void ospf6_router_id_update(void)
+{
+	if (!ospf6)
+		return;
+
+	if (ospf6->router_id_static != 0)
+		ospf6->router_id = ospf6->router_id_static;
+	else
+		ospf6->router_id = om6->zebra_router_id;
 }
 
 /* start ospf6 */
@@ -276,9 +321,11 @@ DEFUN_NOSH (router_ospf6,
        ROUTER_STR
        OSPF6_STR)
 {
-	if (ospf6 == NULL)
+	if (ospf6 == NULL) {
 		ospf6 = ospf6_create();
-
+		if (ospf6->router_id == 0)
+			ospf6_router_id_update();
+	}
 	/* set current ospf point. */
 	VTY_PUSH_CONTEXT(OSPF6_NODE, ospf6);
 
@@ -294,7 +341,7 @@ DEFUN (no_router_ospf6,
        OSPF6_STR)
 {
 	if (ospf6 == NULL)
-		vty_out(vty, "OSPFv3 is not configured%s", VNL);
+		vty_out(vty, "OSPFv3 is not configured\n");
 	else {
 		ospf6_delete(ospf6);
 		ospf6 = NULL;
@@ -307,31 +354,87 @@ DEFUN (no_router_ospf6,
 }
 
 /* change Router_ID commands. */
-DEFUN (ospf6_router_id,
-       ospf6_router_id_cmd,
-       "router-id A.B.C.D",
-       "Configure OSPF Router-ID\n"
-       V4NOTATION_STR)
+DEFUN(ospf6_router_id,
+      ospf6_router_id_cmd,
+      "ospf6 router-id A.B.C.D",
+      OSPF6_STR
+      "Configure OSPF6 Router-ID\n"
+      V4NOTATION_STR)
 {
 	VTY_DECLVAR_CONTEXT(ospf6, o);
-	int idx_ipv4 = 1;
+	int idx = 0;
 	int ret;
-	u_int32_t router_id;
+	const char *router_id_str;
+	uint32_t router_id;
+	struct ospf6_area *oa;
+	struct listnode *node;
 
-	ret = inet_pton(AF_INET, argv[idx_ipv4]->arg, &router_id);
+	argv_find(argv, argc, "A.B.C.D", &idx);
+	router_id_str = argv[idx]->arg;
+
+	ret = inet_pton(AF_INET, router_id_str, &router_id);
 	if (ret == 0) {
-		vty_out(vty, "malformed OSPF Router-ID: %s%s",
-			argv[idx_ipv4]->arg, VNL);
+		vty_out(vty, "malformed OSPF Router-ID: %s\n", router_id_str);
 		return CMD_SUCCESS;
 	}
 
 	o->router_id_static = router_id;
-	if (o->router_id == 0)
-		o->router_id = router_id;
+
+	for (ALL_LIST_ELEMENTS_RO(o->area_list, node, oa)) {
+		if (oa->full_nbrs) {
+			vty_out(vty,
+				"For this router-id change to take effect,"
+				" save config and restart ospf6d\n");
+			return CMD_SUCCESS;
+		}
+	}
+
+	o->router_id = router_id;
 
 	return CMD_SUCCESS;
 }
 
+DEFUN(no_ospf6_router_id,
+      no_ospf6_router_id_cmd,
+      "no ospf6 router-id [A.B.C.D]",
+      NO_STR OSPF6_STR
+      "Configure OSPF6 Router-ID\n"
+      V4NOTATION_STR)
+{
+	VTY_DECLVAR_CONTEXT(ospf6, o);
+	struct ospf6_area *oa;
+	struct listnode *node;
+
+	o->router_id_static = 0;
+
+	for (ALL_LIST_ELEMENTS_RO(o->area_list, node, oa)) {
+		if (oa->full_nbrs) {
+			vty_out(vty,
+				"For this router-id change to take effect,"
+				" save config and restart ospf6d\n");
+			return CMD_SUCCESS;
+		}
+	}
+	o->router_id = 0;
+	if (o->router_id_zebra.s_addr)
+		o->router_id = (uint32_t)o->router_id_zebra.s_addr;
+
+	return CMD_SUCCESS;
+}
+
+#if defined(VERSION_TYPE_DEV) && CONFDATE > 20180828
+CPP_NOTICE("ospf6: `router-id A.B.C.D` deprecated 2017/08/28")
+#endif
+ALIAS_HIDDEN(ospf6_router_id, ospf6_router_id_hdn_cmd, "router-id A.B.C.D",
+	     "Configure OSPF6 Router-ID\n" V4NOTATION_STR)
+
+#if defined(VERSION_TYPE_DEV) && CONFDATE > 20180828
+CPP_NOTICE("ospf6: `no router-id A.B.C.D` deprecated 2017/08/28")
+#endif
+ALIAS_HIDDEN(no_ospf6_router_id, no_ospf6_router_id_hdn_cmd,
+	     "no router-id [A.B.C.D]",
+	     NO_STR "Configure OSPF6 Router-ID\n" V4NOTATION_STR)
+
 DEFUN (ospf6_log_adjacency_changes,
        ospf6_log_adjacency_changes_cmd,
        "log-adjacency-changes",
@@ -395,7 +498,7 @@ DEFUN (ospf6_timers_lsa,
 	int idx_number = 3;
 	unsigned int minarrival;
 
-	VTY_GET_INTEGER("LSA min-arrival", minarrival, argv[idx_number]->arg);
+	minarrival = strtoul(argv[idx_number]->arg, NULL, 10);
 	ospf->lsa_minarrival = minarrival;
 
 	return CMD_SUCCESS;
@@ -415,8 +518,7 @@ DEFUN (no_ospf6_timers_lsa,
 	unsigned int minarrival;
 
 	if (argc == 5) {
-		VTY_GET_INTEGER("LSA min-arrival", minarrival,
-				argv[idx_number]->arg);
+		minarrival = strtoul(argv[idx_number]->arg, NULL, 10);
 
 		if (ospf->lsa_minarrival != minarrival
 		    || minarrival == OSPF_MIN_LS_ARRIVAL)
@@ -471,6 +573,10 @@ DEFUN (ospf6_distance_ospf6,
 	VTY_DECLVAR_CONTEXT(ospf6, o);
 	int idx = 0;
 
+	o->distance_intra = 0;
+	o->distance_inter = 0;
+	o->distance_external = 0;
+
 	if (argv_find(argv, argc, "intra-area", &idx))
 		o->distance_intra = atoi(argv[idx + 1]->arg);
 	idx = 0;
@@ -557,29 +663,29 @@ DEFUN (ospf6_interface_area,
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
 	struct interface *ifp;
-	u_int32_t area_id;
+	uint32_t area_id;
 
 	/* find/create ospf6 interface */
-	ifp = if_get_by_name(argv[idx_ifname]->arg, VRF_DEFAULT);
+	ifp = if_get_by_name(argv[idx_ifname]->arg, VRF_DEFAULT, 0);
 	oi = (struct ospf6_interface *)ifp->info;
 	if (oi == NULL)
 		oi = ospf6_interface_create(ifp);
 	if (oi->area) {
-		vty_out(vty, "%s already attached to Area %s%s",
-			oi->interface->name, oi->area->name, VNL);
+		vty_out(vty, "%s already attached to Area %s\n",
+			oi->interface->name, oi->area->name);
 		return CMD_SUCCESS;
 	}
 
 	/* parse Area-ID */
 	if (inet_pton(AF_INET, argv[idx_ipv4]->arg, &area_id) != 1) {
-		vty_out(vty, "Invalid Area-ID: %s%s", argv[idx_ipv4]->arg, VNL);
+		vty_out(vty, "Invalid Area-ID: %s\n", argv[idx_ipv4]->arg);
 		return CMD_SUCCESS;
 	}
 
 	/* find/create ospf6 area */
 	oa = ospf6_area_lookup(area_id, o);
 	if (oa == NULL)
-		oa = ospf6_area_create(area_id, o);
+		oa = ospf6_area_create(area_id, o, OSPF6_AREA_FMT_DOTTEDQUAD);
 
 	/* attach interface to area */
 	listnode_add(oa->if_list, oi); /* sort ?? */
@@ -616,36 +722,35 @@ DEFUN (no_ospf6_interface_area,
 	struct ospf6_interface *oi;
 	struct ospf6_area *oa;
 	struct interface *ifp;
-	u_int32_t area_id;
+	uint32_t area_id;
 
 	ifp = if_lookup_by_name(argv[idx_ifname]->arg, VRF_DEFAULT);
 	if (ifp == NULL) {
-		vty_out(vty, "No such interface %s%s", argv[idx_ifname]->arg,
-			VNL);
+		vty_out(vty, "No such interface %s\n", argv[idx_ifname]->arg);
 		return CMD_SUCCESS;
 	}
 
 	oi = (struct ospf6_interface *)ifp->info;
 	if (oi == NULL) {
-		vty_out(vty, "Interface %s not enabled%s", ifp->name, VNL);
+		vty_out(vty, "Interface %s not enabled\n", ifp->name);
 		return CMD_SUCCESS;
 	}
 
 	/* parse Area-ID */
 	if (inet_pton(AF_INET, argv[idx_ipv4]->arg, &area_id) != 1) {
-		vty_out(vty, "Invalid Area-ID: %s%s", argv[idx_ipv4]->arg, VNL);
+		vty_out(vty, "Invalid Area-ID: %s\n", argv[idx_ipv4]->arg);
 		return CMD_SUCCESS;
 	}
 
 	/* Verify Area */
 	if (oi->area == NULL) {
-		vty_out(vty, "No such Area-ID: %s%s", argv[idx_ipv4]->arg, VNL);
+		vty_out(vty, "No such Area-ID: %s\n", argv[idx_ipv4]->arg);
 		return CMD_SUCCESS;
 	}
 
 	if (oi->area->area_id != area_id) {
-		vty_out(vty, "Wrong Area-ID: %s is attached to area %s%s",
-			oi->interface->name, oi->area->name, VNL);
+		vty_out(vty, "Wrong Area-ID: %s is attached to area %s\n",
+			oi->interface->name, oi->area->name);
 		return CMD_SUCCESS;
 	}
 
@@ -668,7 +773,6 @@ DEFUN (ospf6_stub_router_admin,
        ospf6_stub_router_admin_cmd,
        "stub-router administrative",
        "Make router a stub router\n"
-       "Advertise inability to be a transit router\n"
        "Administratively applied, for an indefinite period\n")
 {
 	struct listnode *node;
@@ -691,7 +795,6 @@ DEFUN (no_ospf6_stub_router_admin,
        "no stub-router administrative",
        NO_STR
        "Make router a stub router\n"
-       "Advertise ability to be a transit router\n"
        "Administratively applied, for an indefinite period\n")
 {
 	struct listnode *node;
@@ -767,66 +870,62 @@ static void ospf6_show(struct vty *vty, struct ospf6 *o)
 
 	/* process id, router id */
 	inet_ntop(AF_INET, &o->router_id, router_id, sizeof(router_id));
-	vty_out(vty, " OSPFv3 Routing Process (0) with Router-ID %s%s",
-		router_id, VNL);
+	vty_out(vty, " OSPFv3 Routing Process (0) with Router-ID %s\n",
+		router_id);
 
 	/* running time */
 	monotime(&now);
 	timersub(&now, &o->starttime, &running);
 	timerstring(&running, duration, sizeof(duration));
-	vty_out(vty, " Running %s%s", duration, VNL);
+	vty_out(vty, " Running %s\n", duration);
 
 	/* Redistribute configuration */
 	/* XXX */
 
-	vty_out(vty, " LSA minimum arrival %d msecs%s", o->lsa_minarrival,
-		VTY_NEWLINE);
+	vty_out(vty, " LSA minimum arrival %d msecs\n", o->lsa_minarrival);
 
 	/* Show SPF parameters */
 	vty_out(vty,
-		" Initial SPF scheduling delay %d millisec(s)%s"
-		" Minimum hold time between consecutive SPFs %d millsecond(s)%s"
-		" Maximum hold time between consecutive SPFs %d millsecond(s)%s"
-		" Hold time multiplier is currently %d%s",
-		o->spf_delay, VNL, o->spf_holdtime, VNL, o->spf_max_holdtime,
-		VNL, o->spf_hold_multiplier, VNL);
+		" Initial SPF scheduling delay %d millisec(s)\n"
+		" Minimum hold time between consecutive SPFs %d millsecond(s)\n"
+		" Maximum hold time between consecutive SPFs %d millsecond(s)\n"
+		" Hold time multiplier is currently %d\n",
+		o->spf_delay, o->spf_holdtime, o->spf_max_holdtime,
+		o->spf_hold_multiplier);
 
 	vty_out(vty, " SPF algorithm ");
 	if (o->ts_spf.tv_sec || o->ts_spf.tv_usec) {
 		timersub(&now, &o->ts_spf, &result);
 		timerstring(&result, buf, sizeof(buf));
 		ospf6_spf_reason_string(o->last_spf_reason, rbuf, sizeof(rbuf));
-		vty_out(vty, "last executed %s ago, reason %s%s", buf, rbuf,
-			VNL);
-		vty_out(vty, " Last SPF duration %lld sec %lld usec%s",
+		vty_out(vty, "last executed %s ago, reason %s\n", buf, rbuf);
+		vty_out(vty, " Last SPF duration %lld sec %lld usec\n",
 			(long long)o->ts_spf_duration.tv_sec,
-			(long long)o->ts_spf_duration.tv_usec, VNL);
+			(long long)o->ts_spf_duration.tv_usec);
 	} else
-		vty_out(vty, "has not been run$%s", VNL);
+		vty_out(vty, "has not been run\n");
 	threadtimer_string(now, o->t_spf_calc, buf, sizeof(buf));
-	vty_out(vty, " SPF timer %s%s%s", (o->t_spf_calc ? "due in " : "is "),
-		buf, VNL);
+	vty_out(vty, " SPF timer %s%s\n", (o->t_spf_calc ? "due in " : "is "),
+		buf);
 
 	if (CHECK_FLAG(o->flag, OSPF6_STUB_ROUTER))
-		vty_out(vty, " Router Is Stub Router%s", VNL);
+		vty_out(vty, " Router Is Stub Router\n");
 
 	/* LSAs */
-	vty_out(vty, " Number of AS scoped LSAs is %u%s", o->lsdb->count, VNL);
+	vty_out(vty, " Number of AS scoped LSAs is %u\n", o->lsdb->count);
 
 	/* Areas */
-	vty_out(vty, " Number of areas in this router is %u%s",
-		listcount(o->area_list), VNL);
+	vty_out(vty, " Number of areas in this router is %u\n",
+		listcount(o->area_list));
 
 	if (CHECK_FLAG(o->config_flags, OSPF6_LOG_ADJACENCY_CHANGES)) {
 		if (CHECK_FLAG(o->config_flags, OSPF6_LOG_ADJACENCY_DETAIL))
-			vty_out(vty, " All adjacency changes are logged%s",
-				VTY_NEWLINE);
+			vty_out(vty, " All adjacency changes are logged\n");
 		else
-			vty_out(vty, " Adjacency changes are logged%s",
-				VTY_NEWLINE);
+			vty_out(vty, " Adjacency changes are logged\n");
 	}
 
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, n, oa))
 		ospf6_area_show(vty, oa);
@@ -927,7 +1026,7 @@ DEFUN (show_ipv6_ospf6_route_type_detail,
 static void ospf6_stub_router_config_write(struct vty *vty)
 {
 	if (CHECK_FLAG(ospf6->flag, OSPF6_STUB_ROUTER)) {
-		vty_out(vty, " stub-router administrative%s", VNL);
+		vty_out(vty, " stub-router administrative\n");
 	}
 	return;
 }
@@ -938,8 +1037,7 @@ static int ospf6_distance_config_write(struct vty *vty)
 	struct ospf6_distance *odistance;
 
 	if (ospf6->distance_all)
-		vty_out(vty, " distance %u%s", ospf6->distance_all,
-			VTY_NEWLINE);
+		vty_out(vty, " distance %u\n", ospf6->distance_all);
 
 	if (ospf6->distance_intra || ospf6->distance_inter
 	    || ospf6->distance_external) {
@@ -952,19 +1050,18 @@ static int ospf6_distance_config_write(struct vty *vty)
 		if (ospf6->distance_external)
 			vty_out(vty, " external %u", ospf6->distance_external);
 
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "\n");
 	}
 
 	for (rn = route_top(ospf6->distance_table); rn; rn = route_next(rn))
 		if ((odistance = rn->info) != NULL) {
 			char buf[PREFIX_STRLEN];
 
-			vty_out(vty, " distance %u %s %s%s",
+			vty_out(vty, " distance %u %s %s\n",
 				odistance->distance,
 				prefix2str(&rn->p, buf, sizeof(buf)),
 				odistance->access_list ? odistance->access_list
-						       : "",
-				VTY_NEWLINE);
+						       : "");
 		}
 	return 0;
 }
@@ -983,29 +1080,28 @@ static int config_write_ospf6(struct vty *vty)
 
 	inet_ntop(AF_INET, &ospf6->router_id_static, router_id,
 		  sizeof(router_id));
-	vty_out(vty, "router ospf6%s", VNL);
+	vty_out(vty, "router ospf6\n");
 	if (ospf6->router_id_static != 0)
-		vty_out(vty, " router-id %s%s", router_id, VNL);
+		vty_out(vty, " ospf6 router-id %s\n", router_id);
 
 	/* log-adjacency-changes flag print. */
 	if (CHECK_FLAG(ospf6->config_flags, OSPF6_LOG_ADJACENCY_CHANGES)) {
 		if (CHECK_FLAG(ospf6->config_flags, OSPF6_LOG_ADJACENCY_DETAIL))
-			vty_out(vty, " log-adjacency-changes detail%s",
-				VTY_NEWLINE);
+			vty_out(vty, " log-adjacency-changes detail\n");
 		else if (!DFLT_OSPF6_LOG_ADJACENCY_CHANGES)
-			vty_out(vty, " log-adjacency-changes%s", VTY_NEWLINE);
+			vty_out(vty, " log-adjacency-changes\n");
 	} else if (DFLT_OSPF6_LOG_ADJACENCY_CHANGES) {
-		vty_out(vty, " no log-adjacency-changes%s", VTY_NEWLINE);
+		vty_out(vty, " no log-adjacency-changes\n");
 	}
 
 	if (ospf6->ref_bandwidth != OSPF6_REFERENCE_BANDWIDTH)
-		vty_out(vty, " auto-cost reference-bandwidth %d%s",
-			ospf6->ref_bandwidth, VNL);
+		vty_out(vty, " auto-cost reference-bandwidth %d\n",
+			ospf6->ref_bandwidth);
 
 	/* LSA timers print. */
 	if (ospf6->lsa_minarrival != OSPF_MIN_LS_ARRIVAL)
-		vty_out(vty, " timers lsa min-arrival %d%s",
-			ospf6->lsa_minarrival, VTY_NEWLINE);
+		vty_out(vty, " timers lsa min-arrival %d\n",
+			ospf6->lsa_minarrival);
 
 	ospf6_stub_router_config_write(vty);
 	ospf6_redistribute_config_write(vty);
@@ -1015,10 +1111,10 @@ static int config_write_ospf6(struct vty *vty)
 
 	for (ALL_LIST_ELEMENTS_RO(ospf6->area_list, j, oa)) {
 		for (ALL_LIST_ELEMENTS_RO(oa->if_list, k, oi))
-			vty_out(vty, " interface %s area %s%s",
-				oi->interface->name, oa->name, VNL);
+			vty_out(vty, " interface %s area %s\n",
+				oi->interface->name, oa->name);
 	}
-	vty_out(vty, "!%s", VNL);
+	vty_out(vty, "!\n");
 	return 0;
 }
 
@@ -1044,6 +1140,9 @@ void ospf6_top_init(void)
 
 	install_default(OSPF6_NODE);
 	install_element(OSPF6_NODE, &ospf6_router_id_cmd);
+	install_element(OSPF6_NODE, &no_ospf6_router_id_cmd);
+	install_element(OSPF6_NODE, &ospf6_router_id_hdn_cmd);
+	install_element(OSPF6_NODE, &no_ospf6_router_id_hdn_cmd);
 	install_element(OSPF6_NODE, &ospf6_log_adjacency_changes_cmd);
 	install_element(OSPF6_NODE, &ospf6_log_adjacency_changes_detail_cmd);
 	install_element(OSPF6_NODE, &no_ospf6_log_adjacency_changes_cmd);
