@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -25,6 +24,7 @@
 #include "linklist.h"
 #include "vty.h"
 #include "command.h"
+#include "plist.h"
 
 #include "ospf6_proto.h"
 #include "ospf6_network.h"
@@ -45,8 +45,6 @@
 #include "ospf6d.h"
 #include "ospf6_bfd.h"
 
-char ospf6_daemon_version[] = OSPF6_DAEMON_VERSION;
-
 struct route_node *route_prev(struct route_node *node)
 {
 	struct route_node *end;
@@ -71,21 +69,6 @@ struct route_node *route_prev(struct route_node *node)
 	return prev;
 }
 
-
-/* show database functions */
-DEFUN (show_version_ospf6,
-       show_version_ospf6_cmd,
-       "show version ospf6",
-       SHOW_STR
-       "Display version\n"
-       "Display ospf6d version\n"
-      )
-{
-	vty_out(vty, "Zebra OSPF6d Version: %s%s", ospf6_daemon_version, VNL);
-
-	return CMD_SUCCESS;
-}
-
 static struct cmd_node debug_node = {
 	DEBUG_NODE, "", 1 /* VTYSH */
 };
@@ -103,15 +86,29 @@ static int config_write_ospf6_debug(struct vty *vty)
 	config_write_ospf6_debug_asbr(vty);
 	config_write_ospf6_debug_abr(vty);
 	config_write_ospf6_debug_flood(vty);
-	vty_out(vty, "!%s", VNL);
+
 	return 0;
 }
 
+DEFUN_NOSH (show_debugging_ospf6,
+	    show_debugging_ospf6_cmd,
+	    "show debugging [ospf6]",
+	    SHOW_STR
+	    DEBUG_STR
+	    OSPF6_STR)
+{
+	vty_out(vty, "OSPF6 debugging status:");
+
+	config_write_ospf6_debug(vty);
+
+	return CMD_SUCCESS;
+}
+
 #define AREA_LSDB_TITLE_FORMAT                                                 \
-	"%s        Area Scoped Link State Database (Area %s)%s%s"
+	"\n        Area Scoped Link State Database (Area %s)\n\n"
 #define IF_LSDB_TITLE_FORMAT                                                   \
-	"%s        I/F Scoped Link State Database (I/F %s in Area %s)%s%s"
-#define AS_LSDB_TITLE_FORMAT "%s        AS Scoped Link State Database%s%s"
+	"\n        I/F Scoped Link State Database (I/F %s in Area %s)\n\n"
+#define AS_LSDB_TITLE_FORMAT "\n        AS Scoped Link State Database\n\n"
 
 static int parse_show_level(int idx_level, int argc, struct cmd_token **argv)
 {
@@ -129,9 +126,9 @@ static int parse_show_level(int idx_level, int argc, struct cmd_token **argv)
 	return level;
 }
 
-static u_int16_t parse_type_spec(int idx_lsa, int argc, struct cmd_token **argv)
+static uint16_t parse_type_spec(int idx_lsa, int argc, struct cmd_token **argv)
 {
-	u_int16_t type = 0;
+	uint16_t type = 0;
 
 	if (argc > idx_lsa) {
 		if (strmatch(argv[idx_lsa]->text, "router"))
@@ -176,22 +173,22 @@ DEFUN (show_ipv6_ospf6_database,
 	level = parse_show_level(idx_level, argc, argv);
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-		vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL, VNL);
+		vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 		ospf6_lsdb_show(vty, level, NULL, NULL, NULL, oa->lsdb);
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 		for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-			vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-				oi->interface->name, oa->name, VNL, VNL);
+			vty_out(vty, IF_LSDB_TITLE_FORMAT, oi->interface->name,
+				oa->name);
 			ospf6_lsdb_show(vty, level, NULL, NULL, NULL, oi->lsdb);
 		}
 	}
 
-	vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+	vty_out(vty, AS_LSDB_TITLE_FORMAT);
 	ospf6_lsdb_show(vty, level, NULL, NULL, NULL, o->lsdb);
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -223,7 +220,7 @@ DEFUN (show_ipv6_ospf6_database_type,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int16_t type = 0;
+	uint16_t type = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 
@@ -233,8 +230,7 @@ DEFUN (show_ipv6_ospf6_database_type,
 	switch (OSPF6_LSA_SCOPE(type)) {
 	case OSPF6_SCOPE_AREA:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-			vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL,
-				VNL);
+			vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 			ospf6_lsdb_show(vty, level, &type, NULL, NULL,
 					oa->lsdb);
 		}
@@ -243,9 +239,8 @@ DEFUN (show_ipv6_ospf6_database_type,
 	case OSPF6_SCOPE_LINKLOCAL:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 			for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-				vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-					oi->interface->name, oa->name, VNL,
-					VNL);
+				vty_out(vty, IF_LSDB_TITLE_FORMAT,
+					oi->interface->name, oa->name);
 				ospf6_lsdb_show(vty, level, &type, NULL, NULL,
 						oi->lsdb);
 			}
@@ -253,7 +248,7 @@ DEFUN (show_ipv6_ospf6_database_type,
 		break;
 
 	case OSPF6_SCOPE_AS:
-		vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+		vty_out(vty, AS_LSDB_TITLE_FORMAT);
 		ospf6_lsdb_show(vty, level, &type, NULL, NULL, o->lsdb);
 		break;
 
@@ -262,7 +257,7 @@ DEFUN (show_ipv6_ospf6_database_type,
 		break;
 	}
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -287,7 +282,7 @@ DEFUN (show_ipv6_ospf6_database_id,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int32_t id = 0;
+	uint32_t id = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 
@@ -297,22 +292,22 @@ DEFUN (show_ipv6_ospf6_database_id,
 	level = parse_show_level(idx_level, argc, argv);
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-		vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL, VNL);
+		vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 		ospf6_lsdb_show(vty, level, NULL, &id, NULL, oa->lsdb);
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 		for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-			vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-				oi->interface->name, oa->name, VNL, VNL);
+			vty_out(vty, IF_LSDB_TITLE_FORMAT, oi->interface->name,
+				oa->name);
 			ospf6_lsdb_show(vty, level, NULL, &id, NULL, oi->lsdb);
 		}
 	}
 
-	vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+	vty_out(vty, AS_LSDB_TITLE_FORMAT);
 	ospf6_lsdb_show(vty, level, NULL, &id, NULL, o->lsdb);
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -338,30 +333,73 @@ DEFUN (show_ipv6_ospf6_database_router,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int32_t adv_router = 0;
+	uint32_t adv_router = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 	inet_pton(AF_INET, argv[idx_ipv4]->arg, &adv_router);
 	level = parse_show_level(idx_level, argc, argv);
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-		vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL, VNL);
+		vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 		ospf6_lsdb_show(vty, level, NULL, NULL, &adv_router, oa->lsdb);
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 		for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-			vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-				oi->interface->name, oa->name, VNL, VNL);
+			vty_out(vty, IF_LSDB_TITLE_FORMAT, oi->interface->name,
+				oa->name);
 			ospf6_lsdb_show(vty, level, NULL, NULL, &adv_router,
 					oi->lsdb);
 		}
 	}
 
-	vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+	vty_out(vty, AS_LSDB_TITLE_FORMAT);
 	ospf6_lsdb_show(vty, level, NULL, NULL, &adv_router, o->lsdb);
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
+	return CMD_SUCCESS;
+}
+
+DEFUN_HIDDEN (show_ipv6_ospf6_database_aggr_router,
+       show_ipv6_ospf6_database_aggr_router_cmd,
+       "show ipv6 ospf6 database aggr adv-router A.B.C.D",
+       SHOW_STR
+       IPV6_STR
+       OSPF6_STR
+       "Display Link state database\n"
+       "Aggregated Router LSA\n"
+       "Search by Advertising Router\n"
+       "Specify Advertising Router as IPv4 address notation\n")
+{
+	int level = OSPF6_LSDB_SHOW_LEVEL_DETAIL;
+	uint16_t type = htons(OSPF6_LSTYPE_ROUTER);
+	int idx_ipv4 = 6;
+	struct listnode *i;
+	struct ospf6 *o = ospf6;
+	struct ospf6_area *oa;
+	struct ospf6_lsdb *lsdb;
+	uint32_t adv_router = 0;
+
+	inet_pton(AF_INET, argv[idx_ipv4]->arg, &adv_router);
+
+	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
+		if (adv_router == o->router_id)
+			lsdb = oa->lsdb_self;
+		else
+			lsdb = oa->lsdb;
+		if (ospf6_create_single_router_lsa(oa, lsdb, adv_router)
+		    == NULL) {
+			vty_out(vty, "Adv router is not found in LSDB.");
+			return CMD_SUCCESS;
+		}
+		ospf6_lsdb_show(vty, level, &type, NULL, NULL,
+				oa->temp_router_lsa_lsdb);
+		/* Remove the temp cache */
+		ospf6_remove_temp_router_lsa(oa);
+	}
+
+	vty_out(vty, "\n");
+
 	return CMD_SUCCESS;
 }
 
@@ -396,8 +434,8 @@ DEFUN (show_ipv6_ospf6_database_type_id,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int16_t type = 0;
-	u_int32_t id = 0;
+	uint16_t type = 0;
+	uint32_t id = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 
@@ -408,8 +446,7 @@ DEFUN (show_ipv6_ospf6_database_type_id,
 	switch (OSPF6_LSA_SCOPE(type)) {
 	case OSPF6_SCOPE_AREA:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-			vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL,
-				VNL);
+			vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 			ospf6_lsdb_show(vty, level, &type, &id, NULL, oa->lsdb);
 		}
 		break;
@@ -417,9 +454,8 @@ DEFUN (show_ipv6_ospf6_database_type_id,
 	case OSPF6_SCOPE_LINKLOCAL:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 			for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-				vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-					oi->interface->name, oa->name, VNL,
-					VNL);
+				vty_out(vty, IF_LSDB_TITLE_FORMAT,
+					oi->interface->name, oa->name);
 				ospf6_lsdb_show(vty, level, &type, &id, NULL,
 						oi->lsdb);
 			}
@@ -427,7 +463,7 @@ DEFUN (show_ipv6_ospf6_database_type_id,
 		break;
 
 	case OSPF6_SCOPE_AS:
-		vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+		vty_out(vty, AS_LSDB_TITLE_FORMAT);
 		ospf6_lsdb_show(vty, level, &type, &id, NULL, o->lsdb);
 		break;
 
@@ -436,7 +472,7 @@ DEFUN (show_ipv6_ospf6_database_type_id,
 		break;
 	}
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -472,8 +508,8 @@ DEFUN (show_ipv6_ospf6_database_type_router,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int16_t type = 0;
-	u_int32_t adv_router = 0;
+	uint16_t type = 0;
+	uint32_t adv_router = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 
@@ -484,8 +520,7 @@ DEFUN (show_ipv6_ospf6_database_type_router,
 	switch (OSPF6_LSA_SCOPE(type)) {
 	case OSPF6_SCOPE_AREA:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-			vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL,
-				VNL);
+			vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 			ospf6_lsdb_show(vty, level, &type, NULL, &adv_router,
 					oa->lsdb);
 		}
@@ -494,9 +529,8 @@ DEFUN (show_ipv6_ospf6_database_type_router,
 	case OSPF6_SCOPE_LINKLOCAL:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 			for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-				vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-					oi->interface->name, oa->name, VNL,
-					VNL);
+				vty_out(vty, IF_LSDB_TITLE_FORMAT,
+					oi->interface->name, oa->name);
 				ospf6_lsdb_show(vty, level, &type, NULL,
 						&adv_router, oi->lsdb);
 			}
@@ -504,7 +538,7 @@ DEFUN (show_ipv6_ospf6_database_type_router,
 		break;
 
 	case OSPF6_SCOPE_AS:
-		vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+		vty_out(vty, AS_LSDB_TITLE_FORMAT);
 		ospf6_lsdb_show(vty, level, &type, NULL, &adv_router, o->lsdb);
 		break;
 
@@ -513,7 +547,7 @@ DEFUN (show_ipv6_ospf6_database_type_router,
 		break;
 	}
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -541,8 +575,8 @@ DEFUN (show_ipv6_ospf6_database_id_router,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int32_t id = 0;
-	u_int32_t adv_router = 0;
+	uint32_t id = 0;
+	uint32_t adv_router = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 	inet_pton(AF_INET, argv[idx_ls_id]->arg, &id);
@@ -550,23 +584,23 @@ DEFUN (show_ipv6_ospf6_database_id_router,
 	level = parse_show_level(idx_level, argc, argv);
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-		vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL, VNL);
+		vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 		ospf6_lsdb_show(vty, level, NULL, &id, &adv_router, oa->lsdb);
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 		for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-			vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-				oi->interface->name, oa->name, VNL, VNL);
+			vty_out(vty, IF_LSDB_TITLE_FORMAT, oi->interface->name,
+				oa->name);
 			ospf6_lsdb_show(vty, level, NULL, &id, &adv_router,
 					oi->lsdb);
 		}
 	}
 
-	vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+	vty_out(vty, AS_LSDB_TITLE_FORMAT);
 	ospf6_lsdb_show(vty, level, NULL, &id, &adv_router, o->lsdb);
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -594,8 +628,8 @@ DEFUN (show_ipv6_ospf6_database_adv_router_linkstate_id,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int32_t id = 0;
-	u_int32_t adv_router = 0;
+	uint32_t id = 0;
+	uint32_t adv_router = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 	inet_pton(AF_INET, argv[idx_adv_rtr]->arg, &adv_router);
@@ -603,23 +637,23 @@ DEFUN (show_ipv6_ospf6_database_adv_router_linkstate_id,
 	level = parse_show_level(idx_level, argc, argv);
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-		vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL, VNL);
+		vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 		ospf6_lsdb_show(vty, level, NULL, &id, &adv_router, oa->lsdb);
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 		for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-			vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-				oi->interface->name, oa->name, VNL, VNL);
+			vty_out(vty, IF_LSDB_TITLE_FORMAT, oi->interface->name,
+				oa->name);
 			ospf6_lsdb_show(vty, level, NULL, &id, &adv_router,
 					oi->lsdb);
 		}
 	}
 
-	vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+	vty_out(vty, AS_LSDB_TITLE_FORMAT);
 	ospf6_lsdb_show(vty, level, NULL, &id, &adv_router, o->lsdb);
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -653,9 +687,9 @@ DEFUN (show_ipv6_ospf6_database_type_id_router,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int16_t type = 0;
-	u_int32_t id = 0;
-	u_int32_t adv_router = 0;
+	uint16_t type = 0;
+	uint32_t id = 0;
+	uint32_t adv_router = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 
@@ -667,8 +701,7 @@ DEFUN (show_ipv6_ospf6_database_type_id_router,
 	switch (OSPF6_LSA_SCOPE(type)) {
 	case OSPF6_SCOPE_AREA:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-			vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL,
-				VNL);
+			vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 			ospf6_lsdb_show(vty, level, &type, &id, &adv_router,
 					oa->lsdb);
 		}
@@ -677,9 +710,8 @@ DEFUN (show_ipv6_ospf6_database_type_id_router,
 	case OSPF6_SCOPE_LINKLOCAL:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 			for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-				vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-					oi->interface->name, oa->name, VNL,
-					VNL);
+				vty_out(vty, IF_LSDB_TITLE_FORMAT,
+					oi->interface->name, oa->name);
 				ospf6_lsdb_show(vty, level, &type, &id,
 						&adv_router, oi->lsdb);
 			}
@@ -687,7 +719,7 @@ DEFUN (show_ipv6_ospf6_database_type_id_router,
 		break;
 
 	case OSPF6_SCOPE_AS:
-		vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+		vty_out(vty, AS_LSDB_TITLE_FORMAT);
 		ospf6_lsdb_show(vty, level, &type, &id, &adv_router, o->lsdb);
 		break;
 
@@ -696,7 +728,7 @@ DEFUN (show_ipv6_ospf6_database_type_id_router,
 		break;
 	}
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -733,9 +765,9 @@ DEFUN (show_ipv6_ospf6_database_type_adv_router_linkstate_id,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int16_t type = 0;
-	u_int32_t id = 0;
-	u_int32_t adv_router = 0;
+	uint16_t type = 0;
+	uint32_t id = 0;
+	uint32_t adv_router = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 
@@ -747,8 +779,7 @@ DEFUN (show_ipv6_ospf6_database_type_adv_router_linkstate_id,
 	switch (OSPF6_LSA_SCOPE(type)) {
 	case OSPF6_SCOPE_AREA:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-			vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL,
-				VNL);
+			vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 			ospf6_lsdb_show(vty, level, &type, &id, &adv_router,
 					oa->lsdb);
 		}
@@ -757,9 +788,8 @@ DEFUN (show_ipv6_ospf6_database_type_adv_router_linkstate_id,
 	case OSPF6_SCOPE_LINKLOCAL:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 			for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-				vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-					oi->interface->name, oa->name, VNL,
-					VNL);
+				vty_out(vty, IF_LSDB_TITLE_FORMAT,
+					oi->interface->name, oa->name);
 				ospf6_lsdb_show(vty, level, &type, &id,
 						&adv_router, oi->lsdb);
 			}
@@ -767,7 +797,7 @@ DEFUN (show_ipv6_ospf6_database_type_adv_router_linkstate_id,
 		break;
 
 	case OSPF6_SCOPE_AS:
-		vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+		vty_out(vty, AS_LSDB_TITLE_FORMAT);
 		ospf6_lsdb_show(vty, level, &type, &id, &adv_router, o->lsdb);
 		break;
 
@@ -776,7 +806,7 @@ DEFUN (show_ipv6_ospf6_database_type_adv_router_linkstate_id,
 		break;
 	}
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -798,30 +828,30 @@ DEFUN (show_ipv6_ospf6_database_self_originated,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int32_t adv_router = 0;
+	uint32_t adv_router = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 	level = parse_show_level(idx_level, argc, argv);
 	adv_router = o->router_id;
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-		vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL, VNL);
+		vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 		ospf6_lsdb_show(vty, level, NULL, NULL, &adv_router, oa->lsdb);
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 		for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-			vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-				oi->interface->name, oa->name, VNL, VNL);
+			vty_out(vty, IF_LSDB_TITLE_FORMAT, oi->interface->name,
+				oa->name);
 			ospf6_lsdb_show(vty, level, NULL, NULL, &adv_router,
 					oi->lsdb);
 		}
 	}
 
-	vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+	vty_out(vty, AS_LSDB_TITLE_FORMAT);
 	ospf6_lsdb_show(vty, level, NULL, NULL, &adv_router, o->lsdb);
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -854,8 +884,8 @@ DEFUN (show_ipv6_ospf6_database_type_self_originated,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int16_t type = 0;
-	u_int32_t adv_router = 0;
+	uint16_t type = 0;
+	uint32_t adv_router = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 
@@ -867,8 +897,7 @@ DEFUN (show_ipv6_ospf6_database_type_self_originated,
 	switch (OSPF6_LSA_SCOPE(type)) {
 	case OSPF6_SCOPE_AREA:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-			vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL,
-				VNL);
+			vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 			ospf6_lsdb_show(vty, level, &type, NULL, &adv_router,
 					oa->lsdb);
 		}
@@ -877,9 +906,8 @@ DEFUN (show_ipv6_ospf6_database_type_self_originated,
 	case OSPF6_SCOPE_LINKLOCAL:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 			for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-				vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-					oi->interface->name, oa->name, VNL,
-					VNL);
+				vty_out(vty, IF_LSDB_TITLE_FORMAT,
+					oi->interface->name, oa->name);
 				ospf6_lsdb_show(vty, level, &type, NULL,
 						&adv_router, oi->lsdb);
 			}
@@ -887,7 +915,7 @@ DEFUN (show_ipv6_ospf6_database_type_self_originated,
 		break;
 
 	case OSPF6_SCOPE_AS:
-		vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+		vty_out(vty, AS_LSDB_TITLE_FORMAT);
 		ospf6_lsdb_show(vty, level, &type, NULL, &adv_router, o->lsdb);
 		break;
 
@@ -896,7 +924,7 @@ DEFUN (show_ipv6_ospf6_database_type_self_originated,
 		break;
 	}
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -931,9 +959,9 @@ DEFUN (show_ipv6_ospf6_database_type_self_originated_linkstate_id,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int16_t type = 0;
-	u_int32_t adv_router = 0;
-	u_int32_t id = 0;
+	uint16_t type = 0;
+	uint32_t adv_router = 0;
+	uint32_t id = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 
@@ -945,8 +973,7 @@ DEFUN (show_ipv6_ospf6_database_type_self_originated_linkstate_id,
 	switch (OSPF6_LSA_SCOPE(type)) {
 	case OSPF6_SCOPE_AREA:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-			vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL,
-				VNL);
+			vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 			ospf6_lsdb_show(vty, level, &type, &id, &adv_router,
 					oa->lsdb);
 		}
@@ -955,9 +982,8 @@ DEFUN (show_ipv6_ospf6_database_type_self_originated_linkstate_id,
 	case OSPF6_SCOPE_LINKLOCAL:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 			for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-				vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-					oi->interface->name, oa->name, VNL,
-					VNL);
+				vty_out(vty, IF_LSDB_TITLE_FORMAT,
+					oi->interface->name, oa->name);
 				ospf6_lsdb_show(vty, level, &type, &id,
 						&adv_router, oi->lsdb);
 			}
@@ -965,7 +991,7 @@ DEFUN (show_ipv6_ospf6_database_type_self_originated_linkstate_id,
 		break;
 
 	case OSPF6_SCOPE_AS:
-		vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+		vty_out(vty, AS_LSDB_TITLE_FORMAT);
 		ospf6_lsdb_show(vty, level, &type, &id, &adv_router, o->lsdb);
 		break;
 
@@ -974,7 +1000,7 @@ DEFUN (show_ipv6_ospf6_database_type_self_originated_linkstate_id,
 		break;
 	}
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -1008,9 +1034,9 @@ DEFUN (show_ipv6_ospf6_database_type_id_self_originated,
 	struct ospf6 *o = ospf6;
 	struct ospf6_area *oa;
 	struct ospf6_interface *oi;
-	u_int16_t type = 0;
-	u_int32_t adv_router = 0;
-	u_int32_t id = 0;
+	uint16_t type = 0;
+	uint32_t adv_router = 0;
+	uint32_t id = 0;
 
 	OSPF6_CMD_CHECK_RUNNING();
 
@@ -1022,8 +1048,7 @@ DEFUN (show_ipv6_ospf6_database_type_id_self_originated,
 	switch (OSPF6_LSA_SCOPE(type)) {
 	case OSPF6_SCOPE_AREA:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
-			vty_out(vty, AREA_LSDB_TITLE_FORMAT, VNL, oa->name, VNL,
-				VNL);
+			vty_out(vty, AREA_LSDB_TITLE_FORMAT, oa->name);
 			ospf6_lsdb_show(vty, level, &type, &id, &adv_router,
 					oa->lsdb);
 		}
@@ -1032,9 +1057,8 @@ DEFUN (show_ipv6_ospf6_database_type_id_self_originated,
 	case OSPF6_SCOPE_LINKLOCAL:
 		for (ALL_LIST_ELEMENTS_RO(o->area_list, i, oa)) {
 			for (ALL_LIST_ELEMENTS_RO(oa->if_list, j, oi)) {
-				vty_out(vty, IF_LSDB_TITLE_FORMAT, VNL,
-					oi->interface->name, oa->name, VNL,
-					VNL);
+				vty_out(vty, IF_LSDB_TITLE_FORMAT,
+					oi->interface->name, oa->name);
 				ospf6_lsdb_show(vty, level, &type, &id,
 						&adv_router, oi->lsdb);
 			}
@@ -1042,7 +1066,7 @@ DEFUN (show_ipv6_ospf6_database_type_id_self_originated,
 		break;
 
 	case OSPF6_SCOPE_AS:
-		vty_out(vty, AS_LSDB_TITLE_FORMAT, VNL, VNL, VNL);
+		vty_out(vty, AS_LSDB_TITLE_FORMAT);
 		ospf6_lsdb_show(vty, level, &type, &id, &adv_router, o->lsdb);
 		break;
 
@@ -1051,7 +1075,7 @@ DEFUN (show_ipv6_ospf6_database_type_id_self_originated,
 		break;
 	}
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -1066,7 +1090,7 @@ DEFUN (show_ipv6_ospf6_border_routers,
        "Show detailed output\n")
 {
 	int idx_ipv4 = 4;
-	u_int32_t adv_router;
+	uint32_t adv_router;
 	struct ospf6_route *ro;
 	struct prefix prefix;
 
@@ -1084,8 +1108,8 @@ DEFUN (show_ipv6_ospf6_border_routers,
 			ro = ospf6_route_lookup(&prefix, ospf6->brouter_table);
 			if (!ro) {
 				vty_out(vty,
-					"No Route found for Router ID: %s%s",
-					argv[4]->arg, VNL);
+					"No Route found for Router ID: %s\n",
+					argv[4]->arg);
 				return CMD_SUCCESS;
 			}
 
@@ -1117,20 +1141,19 @@ DEFUN (show_ipv6_ospf6_linkstate,
        "Specify Router ID as IPv4 address notation\n"
        "Specify Link state ID as IPv4 address notation\n")
 {
-	int idx_ipv4 = 4;
+	int idx_ipv4 = 5;
 	struct listnode *node;
 	struct ospf6_area *oa;
 
 	OSPF6_CMD_CHECK_RUNNING();
 
 	for (ALL_LIST_ELEMENTS_RO(ospf6->area_list, node, oa)) {
-		vty_out(vty, "%s        SPF Result in Area %s%s%s", VNL,
-			oa->name, VNL, VNL);
+		vty_out(vty, "\n        SPF Result in Area %s\n\n", oa->name);
 		ospf6_linkstate_table_show(vty, idx_ipv4, argc, argv,
 					   oa->spf_table);
 	}
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
@@ -1151,16 +1174,29 @@ DEFUN (show_ipv6_ospf6_linkstate_detail,
 	OSPF6_CMD_CHECK_RUNNING();
 
 	for (ALL_LIST_ELEMENTS_RO(ospf6->area_list, node, oa)) {
-		vty_out(vty, "%s        SPF Result in Area %s%s%s", VNL,
-			oa->name, VNL, VNL);
+		vty_out(vty, "\n        SPF Result in Area %s\n\n", oa->name);
 		ospf6_linkstate_table_show(vty, idx_detail, argc, argv,
 					   oa->spf_table);
 	}
 
-	vty_out(vty, "%s", VNL);
+	vty_out(vty, "\n");
 	return CMD_SUCCESS;
 }
 
+static void ospf6_plist_add(struct prefix_list *plist)
+{
+	if (prefix_list_afi(plist) != AFI_IP6)
+		return;
+	ospf6_area_plist_update(plist, 1);
+}
+
+static void ospf6_plist_del(struct prefix_list *plist)
+{
+	if (prefix_list_afi(plist) != AFI_IP6)
+		return;
+	ospf6_area_plist_update(plist, 0);
+}
+
 /* Install ospf related commands. */
 void ospf6_init(void)
 {
@@ -1176,6 +1212,9 @@ void ospf6_init(void)
 	ospf6_asbr_init();
 	ospf6_abr_init();
 
+	prefix_list_add_hook(ospf6_plist_add);
+	prefix_list_delete_hook(ospf6_plist_del);
+
 	ospf6_bfd_init();
 	install_node(&debug_node, config_write_ospf6_debug);
 
@@ -1193,7 +1232,7 @@ void ospf6_init(void)
 
 	install_element_ospf6_clear_interface();
 
-	install_element(VIEW_NODE, &show_version_ospf6_cmd);
+	install_element(VIEW_NODE, &show_debugging_ospf6_cmd);
 
 	install_element(VIEW_NODE, &show_ipv6_ospf6_border_routers_cmd);
 
@@ -1223,10 +1262,11 @@ void ospf6_init(void)
 	install_element(
 		VIEW_NODE,
 		&show_ipv6_ospf6_database_type_self_originated_linkstate_id_cmd);
+	install_element(VIEW_NODE, &show_ipv6_ospf6_database_aggr_router_cmd);
 
 	/* Make ospf protocol socket. */
 	ospf6_serv_sock();
-	thread_add_read(master, ospf6_receive, NULL, ospf6_sock);
+	thread_add_read(master, ospf6_receive, NULL, ospf6_sock, NULL);
 }
 
 void ospf6_clean(void)
