@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -180,6 +179,7 @@ struct ospf_apiclient *ospf_apiclient_connect(char *host, int syncport)
 
 	fd1 = socket(AF_INET, SOCK_STREAM, 0);
 	if (fd1 < 0) {
+		close(async_server_sock);
 		fprintf(stderr,
 			"ospf_apiclient_connect: creating sync socket failed\n");
 		return NULL;
@@ -193,6 +193,7 @@ struct ospf_apiclient *ospf_apiclient_connect(char *host, int syncport)
 		fprintf(stderr,
 			"ospf_apiclient_connect: SO_REUSEADDR failed\n");
 		close(fd1);
+		close(async_server_sock);
 		return NULL;
 	}
 
@@ -203,6 +204,7 @@ struct ospf_apiclient *ospf_apiclient_connect(char *host, int syncport)
 		fprintf(stderr,
 			"ospf_apiclient_connect: SO_REUSEPORT failed\n");
 		close(fd1);
+		close(async_server_sock);
 		return NULL;
 	}
 #endif /* SO_REUSEPORT */
@@ -224,6 +226,7 @@ struct ospf_apiclient *ospf_apiclient_connect(char *host, int syncport)
 		fprintf(stderr,
 			"ospf_apiclient_connect: bind sync socket failed\n");
 		close(fd1);
+		close(async_server_sock);
 		return NULL;
 	}
 
@@ -257,6 +260,7 @@ struct ospf_apiclient *ospf_apiclient_connect(char *host, int syncport)
 			"ospf_apiclient_connect: accept async failed\n");
 		close(async_server_sock);
 		close(fd1);
+		close(fd2);
 		return NULL;
 	}
 
@@ -299,7 +303,7 @@ int ospf_apiclient_close(struct ospf_apiclient *oclient)
 static int ospf_apiclient_send_request(struct ospf_apiclient *oclient,
 				       struct msg *msg)
 {
-	u_int32_t reqseq;
+	uint32_t reqseq;
 	struct msg_reply *msgreply;
 	int rc;
 
@@ -337,10 +341,10 @@ static int ospf_apiclient_send_request(struct ospf_apiclient *oclient,
  * -----------------------------------------------------------
  */
 
-static u_int32_t ospf_apiclient_get_seqnr(void)
+static uint32_t ospf_apiclient_get_seqnr(void)
 {
-	static u_int32_t seqnr = MIN_SEQ;
-	u_int32_t tmp;
+	static uint32_t seqnr = MIN_SEQ;
+	uint32_t tmp;
 
 	tmp = seqnr;
 	/* Increment sequence number */
@@ -360,8 +364,8 @@ static u_int32_t ospf_apiclient_get_seqnr(void)
 /*
  * Synchronous request to register opaque type.
  */
-int ospf_apiclient_register_opaque_type(struct ospf_apiclient *cl, u_char ltype,
-					u_char otype)
+int ospf_apiclient_register_opaque_type(struct ospf_apiclient *cl,
+					uint8_t ltype, uint8_t otype)
 {
 	struct msg *msg;
 	int rc;
@@ -420,15 +424,15 @@ out:
 
 int ospf_apiclient_lsa_originate(struct ospf_apiclient *oclient,
 				 struct in_addr ifaddr, struct in_addr area_id,
-				 u_char lsa_type, u_char opaque_type,
-				 u_int32_t opaque_id, void *opaquedata,
+				 uint8_t lsa_type, uint8_t opaque_type,
+				 uint32_t opaque_id, void *opaquedata,
 				 int opaquelen)
 {
 	struct msg *msg;
 	int rc;
-	u_char buf[OSPF_MAX_LSA_SIZE];
+	uint8_t buf[OSPF_MAX_LSA_SIZE];
 	struct lsa_header *lsah;
-	u_int32_t tmp;
+	uint32_t tmp;
 
 
 	/* We can only originate opaque LSAs */
@@ -451,7 +455,7 @@ int ospf_apiclient_lsa_originate(struct ospf_apiclient *oclient,
 	lsah->checksum = 0;
 	lsah->length = htons(sizeof(struct lsa_header) + opaquelen);
 
-	memcpy(((u_char *)lsah) + sizeof(struct lsa_header), opaquedata,
+	memcpy(((uint8_t *)lsah) + sizeof(struct lsa_header), opaquedata,
 	       opaquelen);
 
 	msg = new_msg_originate_request(ospf_apiclient_get_seqnr(), ifaddr,
@@ -466,8 +470,8 @@ int ospf_apiclient_lsa_originate(struct ospf_apiclient *oclient,
 }
 
 int ospf_apiclient_lsa_delete(struct ospf_apiclient *oclient,
-			      struct in_addr area_id, u_char lsa_type,
-			      u_char opaque_type, u_int32_t opaque_id)
+			      struct in_addr area_id, uint8_t lsa_type,
+			      uint8_t opaque_type, uint32_t opaque_id)
 {
 	struct msg *msg;
 	int rc;
@@ -651,18 +655,18 @@ static void ospf_apiclient_msghandle(struct ospf_apiclient *oclient,
 
 void ospf_apiclient_register_callback(
 	struct ospf_apiclient *oclient,
-	void (*ready_notify)(u_char lsa_type, u_char opaque_type,
+	void (*ready_notify)(uint8_t lsa_type, uint8_t opaque_type,
 			     struct in_addr addr),
 	void (*new_if)(struct in_addr ifaddr, struct in_addr area_id),
 	void (*del_if)(struct in_addr ifaddr),
 	void (*ism_change)(struct in_addr ifaddr, struct in_addr area_id,
-			   u_char status),
+			   uint8_t status),
 	void (*nsm_change)(struct in_addr ifaddr, struct in_addr nbraddr,
-			   struct in_addr router_id, u_char status),
+			   struct in_addr router_id, uint8_t status),
 	void (*update_notify)(struct in_addr ifaddr, struct in_addr area_id,
-			      u_char self_origin, struct lsa_header *lsa),
+			      uint8_t self_origin, struct lsa_header *lsa),
 	void (*delete_notify)(struct in_addr ifaddr, struct in_addr area_id,
-			      u_char self_origin, struct lsa_header *lsa))
+			      uint8_t self_origin, struct lsa_header *lsa))
 {
 	assert(oclient);
 	assert(update_notify);
