@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -81,9 +80,10 @@ struct ospf_interface *ospf_apiserver_if_lookup_by_addr(struct in_addr address)
 {
 	struct listnode *node, *nnode;
 	struct ospf_interface *oi;
-	struct ospf *ospf;
+	struct ospf *ospf = NULL;
 
-	if (!(ospf = ospf_lookup()))
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+	if (!ospf)
 		return NULL;
 
 	for (ALL_LIST_ELEMENTS(ospf->oiflist, node, nnode, oi))
@@ -98,9 +98,10 @@ struct ospf_interface *ospf_apiserver_if_lookup_by_ifp(struct interface *ifp)
 {
 	struct listnode *node, *nnode;
 	struct ospf_interface *oi;
-	struct ospf *ospf;
+	struct ospf *ospf = NULL;
 
-	if (!(ospf = ospf_lookup()))
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+	if (!ospf)
 		return NULL;
 
 	for (ALL_LIST_ELEMENTS(ospf->oiflist, node, nnode, oi))
@@ -179,14 +180,14 @@ void ospf_apiserver_term(void)
 
 	/* Free client list itself */
 	if (apiserver_list)
-		list_delete(apiserver_list);
+		list_delete_and_null(&apiserver_list);
 
 	/* Free wildcard list */
 	/* XXX  */
 }
 
-static struct ospf_apiserver *lookup_apiserver(u_char lsa_type,
-					       u_char opaque_type)
+static struct ospf_apiserver *lookup_apiserver(uint8_t lsa_type,
+					       uint8_t opaque_type)
 {
 	struct listnode *n1, *n2;
 	struct registered_opaque_type *r;
@@ -281,30 +282,27 @@ void ospf_apiserver_event(enum event event, int fd,
 	switch (event) {
 	case OSPF_APISERVER_ACCEPT:
 		(void)thread_add_read(master, ospf_apiserver_accept, apiserv,
-				      fd);
+				      fd, NULL);
 		break;
 	case OSPF_APISERVER_SYNC_READ:
-		apiserv->t_sync_read = thread_add_read(
-			master, ospf_apiserver_read, apiserv, fd);
+		apiserv->t_sync_read = NULL;
+		thread_add_read(master, ospf_apiserver_read, apiserv, fd,
+				&apiserv->t_sync_read);
 		break;
 #ifdef USE_ASYNC_READ
 	case OSPF_APISERVER_ASYNC_READ:
-		apiserv->t_async_read = thread_add_read(
-			master, ospf_apiserver_read, apiserv, fd);
+		apiserv->t_async_read = NULL;
+		thread_add_read(master, ospf_apiserver_read, apiserv, fd,
+				&apiserv->t_async_read);
 		break;
 #endif /* USE_ASYNC_READ */
 	case OSPF_APISERVER_SYNC_WRITE:
-		if (!apiserv->t_sync_write) {
-			apiserv->t_sync_write = thread_add_write(
-				master, ospf_apiserver_sync_write, apiserv, fd);
-		}
+		thread_add_write(master, ospf_apiserver_sync_write, apiserv, fd,
+				 &apiserv->t_sync_write);
 		break;
 	case OSPF_APISERVER_ASYNC_WRITE:
-		if (!apiserv->t_async_write) {
-			apiserv->t_async_write = thread_add_write(
-				master, ospf_apiserver_async_write, apiserv,
-				fd);
-		}
+		thread_add_write(master, ospf_apiserver_async_write, apiserv,
+				 fd, &apiserv->t_async_write);
 		break;
 	}
 }
@@ -768,8 +766,8 @@ static int ospf_apiserver_send_msg(struct ospf_apiserver *apiserv,
 	return 0;
 }
 
-int ospf_apiserver_send_reply(struct ospf_apiserver *apiserv, u_int32_t seqnr,
-			      u_char rc)
+int ospf_apiserver_send_reply(struct ospf_apiserver *apiserv, uint32_t seqnr,
+			      uint8_t rc)
 {
 	struct msg *msg = new_msg_reply(seqnr, rc);
 	int ret;
@@ -833,7 +831,7 @@ int ospf_apiserver_handle_msg(struct ospf_apiserver *apiserv, struct msg *msg)
  */
 
 int ospf_apiserver_register_opaque_type(struct ospf_apiserver *apiserv,
-					u_char lsa_type, u_char opaque_type)
+					uint8_t lsa_type, uint8_t opaque_type)
 {
 	struct registered_opaque_type *regtype;
 	int (*originator_func)(void *arg);
@@ -897,7 +895,7 @@ int ospf_apiserver_register_opaque_type(struct ospf_apiserver *apiserv,
 }
 
 int ospf_apiserver_unregister_opaque_type(struct ospf_apiserver *apiserv,
-					  u_char lsa_type, u_char opaque_type)
+					  uint8_t lsa_type, uint8_t opaque_type)
 {
 	struct listnode *node, *nnode;
 	struct registered_opaque_type *regtype;
@@ -936,8 +934,8 @@ int ospf_apiserver_unregister_opaque_type(struct ospf_apiserver *apiserv,
 
 
 static int apiserver_is_opaque_type_registered(struct ospf_apiserver *apiserv,
-					       u_char lsa_type,
-					       u_char opaque_type)
+					       uint8_t lsa_type,
+					       uint8_t opaque_type)
 {
 	struct listnode *node, *nnode;
 	struct registered_opaque_type *regtype;
@@ -959,8 +957,8 @@ int ospf_apiserver_handle_register_opaque_type(struct ospf_apiserver *apiserv,
 					       struct msg *msg)
 {
 	struct msg_register_opaque_type *rmsg;
-	u_char lsa_type;
-	u_char opaque_type;
+	uint8_t lsa_type;
+	uint8_t opaque_type;
 	int rc = 0;
 
 	/* Extract parameters from register opaque type message */
@@ -1003,7 +1001,7 @@ void ospf_apiserver_notify_ready_type9(struct ospf_apiserver *apiserv)
 	struct ospf_interface *oi;
 	struct registered_opaque_type *r;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 	for (ALL_LIST_ELEMENTS(ospf->oiflist, node, nnode, oi)) {
 		/* Check if this interface is indeed ready for type 9 */
@@ -1051,7 +1049,7 @@ void ospf_apiserver_notify_ready_type10(struct ospf_apiserver *apiserv)
 	struct ospf *ospf;
 	struct ospf_area *area;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 	for (ALL_LIST_ELEMENTS(ospf->areas, node, nnode, area)) {
 		struct registered_opaque_type *r;
@@ -1098,7 +1096,7 @@ void ospf_apiserver_notify_ready_type11(struct ospf_apiserver *apiserv)
 	struct ospf *ospf;
 	struct registered_opaque_type *r;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 	/* Can type 11 be originated? */
 	if (!ospf_apiserver_is_ready_type11(ospf))
@@ -1137,8 +1135,8 @@ int ospf_apiserver_handle_unregister_opaque_type(struct ospf_apiserver *apiserv,
 						 struct msg *msg)
 {
 	struct msg_unregister_opaque_type *umsg;
-	u_char ltype;
-	u_char otype;
+	uint8_t ltype;
+	uint8_t otype;
 	int rc = 0;
 
 	/* Extract parameters from unregister opaque type message */
@@ -1165,7 +1163,7 @@ int ospf_apiserver_handle_register_event(struct ospf_apiserver *apiserv,
 {
 	struct msg_register_event *rmsg;
 	int rc;
-	u_int32_t seqnum;
+	uint32_t seqnum;
 
 	rmsg = (struct msg_register_event *)STREAM_DATA(msg->s);
 
@@ -1214,7 +1212,7 @@ static int apiserver_sync_callback(struct ospf_lsa *lsa, void *p_arg,
 
 	param = (struct param_t *)p_arg;
 	apiserv = param->apiserv;
-	seqnum = (u_int32_t)int_arg;
+	seqnum = (uint32_t)int_arg;
 
 	/* Check origin in filter. */
 	if ((param->filter->origin == ANY_ORIGIN)
@@ -1262,20 +1260,20 @@ int ospf_apiserver_handle_sync_lsdb(struct ospf_apiserver *apiserv,
 				    struct msg *msg)
 {
 	struct listnode *node, *nnode;
-	u_int32_t seqnum;
+	uint32_t seqnum;
 	int rc = 0;
 	struct msg_sync_lsdb *smsg;
 	struct ospf_apiserver_param_t {
 		struct ospf_apiserver *apiserv;
 		struct lsa_filter_type *filter;
 	} param;
-	u_int16_t mask;
+	uint16_t mask;
 	struct route_node *rn;
 	struct ospf_lsa *lsa;
 	struct ospf *ospf;
 	struct ospf_area *area;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 	/* Get request sequence number */
 	seqnum = msg_get_seq(msg);
@@ -1292,13 +1290,13 @@ int ospf_apiserver_handle_sync_lsdb(struct ospf_apiserver *apiserv,
 	/* Iterate over all areas. */
 	for (ALL_LIST_ELEMENTS(ospf->areas, node, nnode, area)) {
 		int i;
-		u_int32_t *area_id = NULL;
+		uint32_t *area_id = NULL;
 
 		/* Compare area_id with area_ids in sync request. */
 		if ((i = smsg->filter.num_areas) > 0) {
 			/* Let area_id point to the list of area IDs,
 			 * which is at the end of smsg->filter. */
-			area_id = (u_int32_t *)(&smsg->filter + 1);
+			area_id = (uint32_t *)(&smsg->filter + 1);
 			while (i) {
 				if (*area_id == area->area_id.s_addr) {
 					break;
@@ -1314,38 +1312,46 @@ int ospf_apiserver_handle_sync_lsdb(struct ospf_apiserver *apiserv,
 		if (i) {
 			/* Check msg type. */
 			if (mask & Power2[OSPF_ROUTER_LSA])
-				LSDB_LOOP(ROUTER_LSDB(area), rn, lsa)
-			apiserver_sync_callback(lsa, (void *)&param, seqnum);
+				LSDB_LOOP (ROUTER_LSDB(area), rn, lsa)
+					apiserver_sync_callback(
+						lsa, (void *)&param, seqnum);
 			if (mask & Power2[OSPF_NETWORK_LSA])
-				LSDB_LOOP(NETWORK_LSDB(area), rn, lsa)
-			apiserver_sync_callback(lsa, (void *)&param, seqnum);
+				LSDB_LOOP (NETWORK_LSDB(area), rn, lsa)
+					apiserver_sync_callback(
+						lsa, (void *)&param, seqnum);
 			if (mask & Power2[OSPF_SUMMARY_LSA])
-				LSDB_LOOP(SUMMARY_LSDB(area), rn, lsa)
-			apiserver_sync_callback(lsa, (void *)&param, seqnum);
+				LSDB_LOOP (SUMMARY_LSDB(area), rn, lsa)
+					apiserver_sync_callback(
+						lsa, (void *)&param, seqnum);
 			if (mask & Power2[OSPF_ASBR_SUMMARY_LSA])
-				LSDB_LOOP(ASBR_SUMMARY_LSDB(area), rn, lsa)
-			apiserver_sync_callback(lsa, (void *)&param, seqnum);
+				LSDB_LOOP (ASBR_SUMMARY_LSDB(area), rn, lsa)
+					apiserver_sync_callback(
+						lsa, (void *)&param, seqnum);
 			if (mask & Power2[OSPF_OPAQUE_LINK_LSA])
-				LSDB_LOOP(OPAQUE_LINK_LSDB(area), rn, lsa)
-			apiserver_sync_callback(lsa, (void *)&param, seqnum);
+				LSDB_LOOP (OPAQUE_LINK_LSDB(area), rn, lsa)
+					apiserver_sync_callback(
+						lsa, (void *)&param, seqnum);
 			if (mask & Power2[OSPF_OPAQUE_AREA_LSA])
-				LSDB_LOOP(OPAQUE_AREA_LSDB(area), rn, lsa)
-			apiserver_sync_callback(lsa, (void *)&param, seqnum);
+				LSDB_LOOP (OPAQUE_AREA_LSDB(area), rn, lsa)
+					apiserver_sync_callback(
+						lsa, (void *)&param, seqnum);
 		}
 	}
 
 	/* For AS-external LSAs */
 	if (ospf->lsdb) {
 		if (mask & Power2[OSPF_AS_EXTERNAL_LSA])
-			LSDB_LOOP(EXTERNAL_LSDB(ospf), rn, lsa)
-		apiserver_sync_callback(lsa, (void *)&param, seqnum);
+			LSDB_LOOP (EXTERNAL_LSDB(ospf), rn, lsa)
+				apiserver_sync_callback(lsa, (void *)&param,
+							seqnum);
 	}
 
 	/* For AS-external opaque LSAs */
 	if (ospf->lsdb) {
 		if (mask & Power2[OSPF_OPAQUE_AS_LSA])
-			LSDB_LOOP(OPAQUE_AS_LSDB(ospf), rn, lsa)
-		apiserver_sync_callback(lsa, (void *)&param, seqnum);
+			LSDB_LOOP (OPAQUE_AS_LSDB(ospf), rn, lsa)
+				apiserver_sync_callback(lsa, (void *)&param,
+							seqnum);
 	}
 
 	/* Send a reply back to client with return code */
@@ -1373,12 +1379,16 @@ struct ospf_lsa *ospf_apiserver_opaque_lsa_new(struct ospf_area *area,
 	struct stream *s;
 	struct lsa_header *newlsa;
 	struct ospf_lsa *new = NULL;
-	u_char options = 0x0;
-	u_int16_t length;
+	uint8_t options = 0x0;
+	uint16_t length;
 
 	struct ospf *ospf;
 
-	ospf = ospf_lookup();
+	if (oi && oi->ospf)
+		ospf = oi->ospf;
+	else
+		ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+
 	assert(ospf);
 
 	/* Create a stream for internal opaque LSA */
@@ -1409,7 +1419,7 @@ struct ospf_lsa *ospf_apiserver_opaque_lsa_new(struct ospf_area *area,
 		       ospf->router_id);
 
 	/* Set opaque-LSA body fields. */
-	stream_put(s, ((u_char *)protolsa) + sizeof(struct lsa_header),
+	stream_put(s, ((uint8_t *)protolsa) + sizeof(struct lsa_header),
 		   ntohs(protolsa->length) - sizeof(struct lsa_header));
 
 	/* Determine length of LSA. */
@@ -1433,6 +1443,7 @@ struct ospf_lsa *ospf_apiserver_opaque_lsa_new(struct ospf_area *area,
 
 	new->area = area;
 	new->oi = oi;
+	new->vrf_id = ospf->vrf_id;
 
 	SET_FLAG(new->flags, OSPF_LSA_SELF);
 	memcpy(new->data, newlsa, length);
@@ -1501,7 +1512,7 @@ int ospf_apiserver_handle_originate_request(struct ospf_apiserver *apiserv,
 	int ready = 0;
 	int rc = 0;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 	/* Extract opaque LSA data from message */
 	omsg = (struct msg_originate_request *)STREAM_DATA(msg->s);
@@ -1644,7 +1655,7 @@ void ospf_apiserver_flood_opaque_lsa(struct ospf_lsa *lsa)
 	case OSPF_OPAQUE_AS_LSA: {
 		struct ospf *ospf;
 
-		ospf = ospf_lookup();
+		ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 		assert(ospf);
 
 		/* Increment counters? XXX */
@@ -1660,7 +1671,7 @@ int ospf_apiserver_originate1(struct ospf_lsa *lsa)
 {
 	struct ospf *ospf;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	assert(ospf);
 
 	/* Install this LSA into LSDB. */
@@ -1730,7 +1741,7 @@ struct ospf_lsa *ospf_apiserver_lsa_refresher(struct ospf_lsa *lsa)
 	struct ospf_lsa *new = NULL;
 	struct ospf *ospf;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	assert(ospf);
 
 	apiserv = lookup_apiserver_by_lsa(lsa);
@@ -1814,7 +1825,7 @@ int ospf_apiserver_handle_delete_request(struct ospf_apiserver *apiserv,
 	int rc = 0;
 	struct ospf *ospf;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	assert(ospf);
 
 	/* Extract opaque LSA from message */
@@ -1866,7 +1877,7 @@ int ospf_apiserver_handle_delete_request(struct ospf_apiserver *apiserv,
 	 * the LSDB until it is finally handled by the maxage remover thread.
 	 * Therefore, the lookup function below may return non-NULL result.
 	 */
-	old = ospf_lsa_lookup(area, dmsg->lsa_type, id, ospf->router_id);
+	old = ospf_lsa_lookup(ospf, area, dmsg->lsa_type, id, ospf->router_id);
 	if (!old) {
 		zlog_warn(
 			"ospf_apiserver_lsa_delete: LSA[Type%d:%s] not in LSDB",
@@ -1893,8 +1904,8 @@ static int apiserver_flush_opaque_type_callback(struct ospf_lsa *lsa,
 {
 	struct param_t {
 		struct ospf_apiserver *apiserv;
-		u_char lsa_type;
-		u_char opaque_type;
+		uint8_t lsa_type;
+		uint8_t opaque_type;
 	} * param;
 
 	/* Sanity check */
@@ -1916,18 +1927,18 @@ static int apiserver_flush_opaque_type_callback(struct ospf_lsa *lsa,
    type or a connection to an application closes and all those opaque
    LSAs need to be flushed the LSDB. */
 void ospf_apiserver_flush_opaque_lsa(struct ospf_apiserver *apiserv,
-				     u_char lsa_type, u_char opaque_type)
+				     uint8_t lsa_type, uint8_t opaque_type)
 {
 	struct param_t {
 		struct ospf_apiserver *apiserv;
-		u_char lsa_type;
-		u_char opaque_type;
+		uint8_t lsa_type;
+		uint8_t opaque_type;
 	} param;
 	struct listnode *node, *nnode;
 	struct ospf *ospf;
 	struct ospf_area *area;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	assert(ospf);
 
 	/* Set parameter struct. */
@@ -1941,17 +1952,20 @@ void ospf_apiserver_flush_opaque_lsa(struct ospf_apiserver *apiserv,
 
 	case OSPF_OPAQUE_LINK_LSA:
 		for (ALL_LIST_ELEMENTS(ospf->areas, node, nnode, area))
-			LSDB_LOOP(OPAQUE_LINK_LSDB(area), rn, lsa)
-		apiserver_flush_opaque_type_callback(lsa, (void *)&param, 0);
+			LSDB_LOOP (OPAQUE_LINK_LSDB(area), rn, lsa)
+				apiserver_flush_opaque_type_callback(
+					lsa, (void *)&param, 0);
 		break;
 	case OSPF_OPAQUE_AREA_LSA:
 		for (ALL_LIST_ELEMENTS(ospf->areas, node, nnode, area))
-			LSDB_LOOP(OPAQUE_AREA_LSDB(area), rn, lsa)
-		apiserver_flush_opaque_type_callback(lsa, (void *)&param, 0);
+			LSDB_LOOP (OPAQUE_AREA_LSDB(area), rn, lsa)
+				apiserver_flush_opaque_type_callback(
+					lsa, (void *)&param, 0);
 		break;
 	case OSPF_OPAQUE_AS_LSA:
-		LSDB_LOOP(OPAQUE_LINK_LSDB(ospf), rn, lsa)
-		apiserver_flush_opaque_type_callback(lsa, (void *)&param, 0);
+		LSDB_LOOP (OPAQUE_LINK_LSDB(ospf), rn, lsa)
+			apiserver_flush_opaque_type_callback(lsa,
+							     (void *)&param, 0);
 		break;
 	default:
 		break;
@@ -2062,7 +2076,7 @@ void ospf_apiserver_show_info(struct vty *vty, struct ospf_lsa *lsa)
 {
 	struct opaque_lsa {
 		struct lsa_header header;
-		u_char data[1]; /* opaque data have variable length. This is
+		uint8_t data[1]; /* opaque data have variable length. This is
 				   start
 				   address */
 	};
@@ -2080,17 +2094,16 @@ void ospf_apiserver_show_info(struct vty *vty, struct ospf_lsa *lsa)
 	if (vty != NULL) {
 		int i;
 		vty_out(vty,
-			"  Added using OSPF API: %u octets of opaque data %s%s",
+			"  Added using OSPF API: %u octets of opaque data %s\n",
 			opaquelen,
 			VALID_OPAQUE_INFO_LEN(lsa->data) ? ""
-							 : "(Invalid length?)",
-			VTY_NEWLINE);
+							 : "(Invalid length?)");
 		vty_out(vty, "  Opaque data: ");
 
 		for (i = 0; i < opaquelen; i++) {
 			vty_out(vty, "0x%x ", olsa->data[i]);
 		}
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "\n");
 	} else {
 		int i;
 		zlog_debug(
@@ -2320,7 +2333,7 @@ void ospf_apiserver_clients_notify_nsm_change(struct ospf_neighbor *nbr)
 {
 	struct msg *msg;
 	struct in_addr ifaddr = {.s_addr = 0L};
-	struct in_addr nbraddr = {.s_addr = 0L};
+	struct in_addr nbraddr;
 
 	assert(nbr);
 
@@ -2342,7 +2355,7 @@ void ospf_apiserver_clients_notify_nsm_change(struct ospf_neighbor *nbr)
 	msg_free(msg);
 }
 
-static void apiserver_clients_lsa_change_notify(u_char msgtype,
+static void apiserver_clients_lsa_change_notify(uint8_t msgtype,
 						struct ospf_lsa *lsa)
 {
 	struct msg *msg;
@@ -2377,8 +2390,8 @@ static void apiserver_clients_lsa_change_notify(u_char msgtype,
 	/* Now send message to all clients with a matching filter */
 	for (ALL_LIST_ELEMENTS(apiserver_list, node, nnode, apiserv)) {
 		struct lsa_filter_type *filter;
-		u_int16_t mask;
-		u_int32_t *area;
+		uint16_t mask;
+		uint32_t *area;
 		int i;
 
 		/* Check filter for this client. */
@@ -2395,7 +2408,7 @@ static void apiserver_clients_lsa_change_notify(u_char msgtype,
 		}
 
 		if (i > 0) {
-			area = (u_int32_t *)(filter + 1);
+			area = (uint32_t *)(filter + 1);
 			while (i) {
 				if (*area == area_id.s_addr) {
 					break;
@@ -2431,7 +2444,7 @@ static void apiserver_clients_lsa_change_notify(u_char msgtype,
  */
 
 
-static int apiserver_notify_clients_lsa(u_char msgtype, struct ospf_lsa *lsa)
+static int apiserver_notify_clients_lsa(uint8_t msgtype, struct ospf_lsa *lsa)
 {
 	struct msg *msg;
 	/* default area for AS-External and Opaque11 LSAs */
