@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -78,7 +77,7 @@ struct ospf_route *ospf_find_asbr_route(struct ospf *ospf,
 
 	/* If none is found -- look through all. */
 	if (listcount(chosen) == 0) {
-		list_free(chosen);
+		list_delete_and_null(&chosen);
 		chosen = rn->info;
 	}
 
@@ -99,7 +98,7 @@ struct ospf_route *ospf_find_asbr_route(struct ospf *ospf,
 		}
 
 	if (chosen != rn->info)
-		list_delete(chosen);
+		list_delete_and_null(&chosen);
 
 	return best;
 }
@@ -242,7 +241,7 @@ ospf_ase_calculate_asbr_route (struct ospf *ospf,
 
 static struct ospf_route *
 ospf_ase_calculate_new_route(struct ospf_lsa *lsa,
-			     struct ospf_route *asbr_route, u_int32_t metric)
+			     struct ospf_route *asbr_route, uint32_t metric)
 {
 	struct as_external_lsa *al;
 	struct ospf_route *new;
@@ -283,7 +282,7 @@ ospf_ase_calculate_new_route(struct ospf_lsa *lsa,
 
 int ospf_ase_calculate_route(struct ospf *ospf, struct ospf_lsa *lsa)
 {
-	u_int32_t metric;
+	uint32_t metric;
 	struct as_external_lsa *al;
 	struct ospf_route *asbr_route;
 	struct prefix_ipv4 asbr, p;
@@ -599,7 +598,8 @@ static int ospf_ase_route_match_same(struct route_table *rt,
 	return 1;
 }
 
-static int ospf_ase_compare_tables(struct route_table *new_external_route,
+static int ospf_ase_compare_tables(struct ospf *ospf,
+				   struct route_table *new_external_route,
 				   struct route_table *old_external_route)
 {
 	struct route_node *rn, *new_rn;
@@ -610,8 +610,8 @@ static int ospf_ase_compare_tables(struct route_table *new_external_route,
 		if ((or = rn->info)) {
 			if (!(new_rn = route_node_lookup(new_external_route,
 							 &rn->p)))
-				ospf_zebra_delete((struct prefix_ipv4 *)&rn->p,
-						  or);
+				ospf_zebra_delete(
+					ospf, (struct prefix_ipv4 *)&rn->p, or);
 			else
 				route_unlock_node(new_rn);
 		}
@@ -622,8 +622,8 @@ static int ospf_ase_compare_tables(struct route_table *new_external_route,
 		if ((or = rn->info) != NULL)
 			if (!ospf_ase_route_match_same(old_external_route,
 						       &rn->p, or))
-				ospf_zebra_add((struct prefix_ipv4 *)&rn->p,
-					       or);
+				ospf_zebra_add(
+					ospf, (struct prefix_ipv4 *)&rn->p, or);
 
 	return 0;
 }
@@ -646,8 +646,8 @@ static int ospf_ase_calculate_timer(struct thread *t)
 		monotime(&start_time);
 
 		/* Calculate external route for each AS-external-LSA */
-		LSDB_LOOP(EXTERNAL_LSDB(ospf), rn, lsa)
-		ospf_ase_calculate_route(ospf, lsa);
+		LSDB_LOOP (EXTERNAL_LSDB(ospf), rn, lsa)
+			ospf_ase_calculate_route(ospf, lsa);
 
 		/*  This version simple adds to the table all NSSA areas  */
 		if (ospf->anyNSSA)
@@ -658,16 +658,17 @@ static int ospf_ase_calculate_timer(struct thread *t)
 						inet_ntoa(area->area_id));
 
 				if (area->external_routing == OSPF_AREA_NSSA)
-					LSDB_LOOP(NSSA_LSDB(area), rn, lsa)
-				ospf_ase_calculate_route(ospf, lsa);
+					LSDB_LOOP (NSSA_LSDB(area), rn, lsa)
+						ospf_ase_calculate_route(ospf,
+									 lsa);
 			}
 		/* kevinm: And add the NSSA routes in ospf_top */
-		LSDB_LOOP(NSSA_LSDB(ospf), rn, lsa)
-		ospf_ase_calculate_route(ospf, lsa);
+		LSDB_LOOP (NSSA_LSDB(ospf), rn, lsa)
+			ospf_ase_calculate_route(ospf, lsa);
 
 		/* Compare old and new external routing table and install the
 		   difference info zebra/kernel */
-		ospf_ase_compare_tables(ospf->new_external_route,
+		ospf_ase_compare_tables(ospf, ospf->new_external_route,
 					ospf->old_external_route);
 
 		/* Delete old external routing table */
@@ -677,9 +678,13 @@ static int ospf_ase_calculate_timer(struct thread *t)
 
 		monotime(&stop_time);
 
-		zlog_info("SPF Processing Time(usecs): External Routes: %lld\n",
-			  (stop_time.tv_sec - start_time.tv_sec) * 1000000LL
-				  + (stop_time.tv_usec - start_time.tv_usec));
+		if (IS_DEBUG_OSPF_EVENT)
+			zlog_info(
+				"SPF Processing Time(usecs): External Routes: %lld\n",
+				(stop_time.tv_sec - start_time.tv_sec)
+						* 1000000LL
+					+ (stop_time.tv_usec
+					   - start_time.tv_usec));
 	}
 	return 0;
 }
@@ -697,10 +702,8 @@ void ospf_ase_calculate_timer_add(struct ospf *ospf)
 	if (ospf == NULL)
 		return;
 
-	if (!ospf->t_ase_calc)
-		ospf->t_ase_calc =
-			thread_add_timer(master, ospf_ase_calculate_timer, ospf,
-					 OSPF_ASE_CALC_INTERVAL);
+	thread_add_timer(master, ospf_ase_calculate_timer, ospf,
+			 OSPF_ASE_CALC_INTERVAL, &ospf->t_ase_calc);
 }
 
 void ospf_ase_register_external_lsa(struct ospf_lsa *lsa, struct ospf *top)
@@ -761,7 +764,7 @@ void ospf_ase_external_lsas_finish(struct route_table *rt)
 		if ((lst = rn->info) != NULL) {
 			for (ALL_LIST_ELEMENTS(lst, node, nnode, lsa))
 				ospf_lsa_unlock(&lsa); /* external_lsas lst */
-			list_delete(lst);
+			list_delete_and_null(&lst);
 		}
 
 	route_table_finish(rt);
@@ -817,7 +820,7 @@ void ospf_ase_incremental_update(struct ospf *ospf, struct ospf_lsa *lsa)
 	}
 
 	/* install changes to zebra */
-	ospf_ase_compare_tables(ospf->new_external_route, tmp_old);
+	ospf_ase_compare_tables(ospf, ospf->new_external_route, tmp_old);
 
 	/* update ospf->old_external_route table */
 	if (rn && rn->info)
