@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -52,6 +51,8 @@ unsigned long conf_debug_ospf_lsa = 0;
 unsigned long conf_debug_ospf_zebra = 0;
 unsigned long conf_debug_ospf_nssa = 0;
 unsigned long conf_debug_ospf_te = 0;
+unsigned long conf_debug_ospf_ext = 0;
+unsigned long conf_debug_ospf_sr = 0;
 
 /* Enable debug option variables -- valid only session. */
 unsigned long term_debug_ospf_packet[5] = {0, 0, 0, 0, 0};
@@ -62,9 +63,10 @@ unsigned long term_debug_ospf_lsa = 0;
 unsigned long term_debug_ospf_zebra = 0;
 unsigned long term_debug_ospf_nssa = 0;
 unsigned long term_debug_ospf_te = 0;
+unsigned long term_debug_ospf_ext = 0;
+unsigned long term_debug_ospf_sr = 0;
 
-
-const char *ospf_redist_string(u_int route_type)
+const char *ospf_redist_string(unsigned int route_type)
 {
 	return (route_type == ZEBRA_ROUTE_MAX) ? "Default"
 					       : zebra_route_string(route_type);
@@ -74,7 +76,7 @@ const char *ospf_redist_string(u_int route_type)
 const char *ospf_area_name_string(struct ospf_area *area)
 {
 	static char buf[OSPF_AREA_STRING_MAXLEN] = "";
-	u_int32_t area_id;
+	uint32_t area_id;
 
 	if (!area)
 		return "-";
@@ -90,7 +92,7 @@ const char *ospf_area_name_string(struct ospf_area *area)
 const char *ospf_area_desc_string(struct ospf_area *area)
 {
 	static char buf[OSPF_AREA_DESC_STRING_MAXLEN] = "";
-	u_char type;
+	uint8_t type;
 
 	if (!area)
 		return "(incomplete)";
@@ -116,7 +118,7 @@ const char *ospf_area_desc_string(struct ospf_area *area)
 const char *ospf_if_name_string(struct ospf_interface *oi)
 {
 	static char buf[OSPF_IF_STRING_MAXLEN] = "";
-	u_int32_t ifaddr;
+	uint32_t ifaddr;
 
 	if (!oi || !oi->address)
 		return "inactive";
@@ -170,6 +172,7 @@ const char *ospf_timeval_dump(struct timeval *t, char *buf, size_t size)
 	if (us >= 1000) {
 		ms = us / 1000;
 		us %= 1000;
+		(void)us; /* unused */
 	}
 
 	if (ms >= 1000) {
@@ -225,12 +228,12 @@ const char *ospf_timer_dump(struct thread *t, char *buf, size_t size)
 	return ospf_timeval_dump(&result, buf, size);
 }
 
-static void ospf_packet_hello_dump(struct stream *s, u_int16_t length)
+static void ospf_packet_hello_dump(struct stream *s, uint16_t length)
 {
 	struct ospf_hello *hello;
 	int i;
 
-	hello = (struct ospf_hello *)STREAM_PNT(s);
+	hello = (struct ospf_hello *)stream_pnt(s);
 
 	zlog_debug("Hello");
 	zlog_debug("  NetworkMask %s", inet_ntoa(hello->network_mask));
@@ -239,7 +242,7 @@ static void ospf_packet_hello_dump(struct stream *s, u_int16_t length)
 		   ospf_options_dump(hello->options));
 	zlog_debug("  RtrPriority %d", hello->priority);
 	zlog_debug("  RtrDeadInterval %ld",
-		   (u_long)ntohl(hello->dead_interval));
+		   (unsigned long)ntohl(hello->dead_interval));
 	zlog_debug("  DRouter %s", inet_ntoa(hello->d_router));
 	zlog_debug("  BDRouter %s", inet_ntoa(hello->bd_router));
 
@@ -249,7 +252,7 @@ static void ospf_packet_hello_dump(struct stream *s, u_int16_t length)
 		zlog_debug("    Neighbor %s", inet_ntoa(hello->neighbors[i]));
 }
 
-static char *ospf_dd_flags_dump(u_char flags, char *buf, size_t size)
+static char *ospf_dd_flags_dump(uint8_t flags, char *buf, size_t size)
 {
 	memset(buf, 0, size);
 
@@ -260,7 +263,7 @@ static char *ospf_dd_flags_dump(u_char flags, char *buf, size_t size)
 	return buf;
 }
 
-static char *ospf_router_lsa_flags_dump(u_char flags, char *buf, size_t size)
+static char *ospf_router_lsa_flags_dump(uint8_t flags, char *buf, size_t size)
 {
 	memset(buf, 0, size);
 
@@ -272,13 +275,13 @@ static char *ospf_router_lsa_flags_dump(u_char flags, char *buf, size_t size)
 	return buf;
 }
 
-static void ospf_router_lsa_dump(struct stream *s, u_int16_t length)
+static void ospf_router_lsa_dump(struct stream *s, uint16_t length)
 {
 	char buf[BUFSIZ];
 	struct router_lsa *rl;
 	int i, len;
 
-	rl = (struct router_lsa *)STREAM_PNT(s);
+	rl = (struct router_lsa *)stream_pnt(s);
 
 	zlog_debug("  Router-LSA");
 	zlog_debug("    flags %s",
@@ -290,20 +293,20 @@ static void ospf_router_lsa_dump(struct stream *s, u_int16_t length)
 		zlog_debug("    Link ID %s", inet_ntoa(rl->link[i].link_id));
 		zlog_debug("    Link Data %s",
 			   inet_ntoa(rl->link[i].link_data));
-		zlog_debug("    Type %d", (u_char)rl->link[i].type);
-		zlog_debug("    TOS %d", (u_char)rl->link[i].tos);
+		zlog_debug("    Type %d", (uint8_t)rl->link[i].type);
+		zlog_debug("    TOS %d", (uint8_t)rl->link[i].tos);
 		zlog_debug("    metric %d", ntohs(rl->link[i].metric));
 
 		len -= 12;
 	}
 }
 
-static void ospf_network_lsa_dump(struct stream *s, u_int16_t length)
+static void ospf_network_lsa_dump(struct stream *s, uint16_t length)
 {
 	struct network_lsa *nl;
 	int i, cnt;
 
-	nl = (struct network_lsa *)STREAM_PNT(s);
+	nl = (struct network_lsa *)stream_pnt(s);
 	cnt = (ntohs(nl->header.length) - (OSPF_LSA_HEADER_SIZE + 4)) / 4;
 
 	zlog_debug("  Network-LSA");
@@ -319,13 +322,13 @@ static void ospf_network_lsa_dump(struct stream *s, u_int16_t length)
 			   inet_ntoa(nl->routers[i]));
 }
 
-static void ospf_summary_lsa_dump(struct stream *s, u_int16_t length)
+static void ospf_summary_lsa_dump(struct stream *s, uint16_t length)
 {
 	struct summary_lsa *sl;
 	int size;
 	int i;
 
-	sl = (struct summary_lsa *)STREAM_PNT(s);
+	sl = (struct summary_lsa *)stream_pnt(s);
 
 	zlog_debug("  Summary-LSA");
 	zlog_debug("    Network Mask %s", inet_ntoa(sl->mask));
@@ -336,13 +339,13 @@ static void ospf_summary_lsa_dump(struct stream *s, u_int16_t length)
 			   GET_METRIC(sl->metric));
 }
 
-static void ospf_as_external_lsa_dump(struct stream *s, u_int16_t length)
+static void ospf_as_external_lsa_dump(struct stream *s, uint16_t length)
 {
 	struct as_external_lsa *al;
 	int size;
 	int i;
 
-	al = (struct as_external_lsa *)STREAM_PNT(s);
+	al = (struct as_external_lsa *)stream_pnt(s);
 	zlog_debug("  %s", ospf_lsa_type_msg[al->header.type].str);
 	zlog_debug("    Network Mask %s", inet_ntoa(al->mask));
 
@@ -358,7 +361,7 @@ static void ospf_as_external_lsa_dump(struct stream *s, u_int16_t length)
 	}
 }
 
-static void ospf_lsa_header_list_dump(struct stream *s, u_int16_t length)
+static void ospf_lsa_header_list_dump(struct stream *s, uint16_t length)
 {
 	struct lsa_header *lsa;
 
@@ -366,7 +369,7 @@ static void ospf_lsa_header_list_dump(struct stream *s, u_int16_t length)
 
 	/* LSA Headers. */
 	while (length > 0) {
-		lsa = (struct lsa_header *)STREAM_PNT(s);
+		lsa = (struct lsa_header *)stream_pnt(s);
 		ospf_lsa_header_dump(lsa);
 
 		stream_forward_getp(s, OSPF_LSA_HEADER_SIZE);
@@ -374,15 +377,15 @@ static void ospf_lsa_header_list_dump(struct stream *s, u_int16_t length)
 	}
 }
 
-static void ospf_packet_db_desc_dump(struct stream *s, u_int16_t length)
+static void ospf_packet_db_desc_dump(struct stream *s, uint16_t length)
 {
 	struct ospf_db_desc *dd;
 	char dd_flags[8];
 
-	u_int32_t gp;
+	uint32_t gp;
 
 	gp = stream_get_getp(s);
-	dd = (struct ospf_db_desc *)STREAM_PNT(s);
+	dd = (struct ospf_db_desc *)stream_pnt(s);
 
 	zlog_debug("Database Description");
 	zlog_debug("  Interface MTU %d", ntohs(dd->mtu));
@@ -390,7 +393,8 @@ static void ospf_packet_db_desc_dump(struct stream *s, u_int16_t length)
 		   ospf_options_dump(dd->options));
 	zlog_debug("  Flags %d (%s)", dd->flags,
 		   ospf_dd_flags_dump(dd->flags, dd_flags, sizeof dd_flags));
-	zlog_debug("  Sequence Number 0x%08lx", (u_long)ntohl(dd->dd_seqnum));
+	zlog_debug("  Sequence Number 0x%08lx",
+		   (unsigned long)ntohl(dd->dd_seqnum));
 
 	length -= OSPF_HEADER_SIZE + OSPF_DB_DESC_MIN_SIZE;
 
@@ -401,10 +405,10 @@ static void ospf_packet_db_desc_dump(struct stream *s, u_int16_t length)
 	stream_set_getp(s, gp);
 }
 
-static void ospf_packet_ls_req_dump(struct stream *s, u_int16_t length)
+static void ospf_packet_ls_req_dump(struct stream *s, uint16_t length)
 {
-	u_int32_t sp;
-	u_int32_t ls_type;
+	uint32_t sp;
+	uint32_t ls_type;
 	struct in_addr ls_id;
 	struct in_addr adv_router;
 
@@ -428,12 +432,12 @@ static void ospf_packet_ls_req_dump(struct stream *s, u_int16_t length)
 	stream_set_getp(s, sp);
 }
 
-static void ospf_packet_ls_upd_dump(struct stream *s, u_int16_t length)
+static void ospf_packet_ls_upd_dump(struct stream *s, uint16_t length)
 {
-	u_int32_t sp;
+	uint32_t sp;
 	struct lsa_header *lsa;
 	int lsa_len;
-	u_int32_t count;
+	uint32_t count;
 
 	length -= OSPF_HEADER_SIZE;
 
@@ -452,7 +456,7 @@ static void ospf_packet_ls_upd_dump(struct stream *s, u_int16_t length)
 			break;
 		}
 
-		lsa = (struct lsa_header *)STREAM_PNT(s);
+		lsa = (struct lsa_header *)stream_pnt(s);
 		lsa_len = ntohs(lsa->length);
 		ospf_lsa_header_dump(lsa);
 
@@ -490,9 +494,9 @@ static void ospf_packet_ls_upd_dump(struct stream *s, u_int16_t length)
 	stream_set_getp(s, sp);
 }
 
-static void ospf_packet_ls_ack_dump(struct stream *s, u_int16_t length)
+static void ospf_packet_ls_ack_dump(struct stream *s, uint16_t length)
 {
-	u_int32_t sp;
+	uint32_t sp;
 
 	length -= OSPF_HEADER_SIZE;
 	sp = stream_get_getp(s);
@@ -511,11 +515,11 @@ void ospf_ip_header_dump(struct ip *iph)
 	zlog_debug("ip_hl %d", iph->ip_hl);
 	zlog_debug("ip_tos %d", iph->ip_tos);
 	zlog_debug("ip_len %d", iph->ip_len);
-	zlog_debug("ip_id %u", (u_int32_t)iph->ip_id);
-	zlog_debug("ip_off %u", (u_int32_t)iph->ip_off);
+	zlog_debug("ip_id %u", (uint32_t)iph->ip_id);
+	zlog_debug("ip_off %u", (uint32_t)iph->ip_off);
 	zlog_debug("ip_ttl %d", iph->ip_ttl);
 	zlog_debug("ip_p %d", iph->ip_p);
-	zlog_debug("ip_sum 0x%x", (u_int32_t)iph->ip_sum);
+	zlog_debug("ip_sum 0x%x", (uint32_t)iph->ip_sum);
 	zlog_debug("ip_src %s", inet_ntoa(iph->ip_src));
 	zlog_debug("ip_dst %s", inet_ntoa(iph->ip_dst));
 }
@@ -523,7 +527,7 @@ void ospf_ip_header_dump(struct ip *iph)
 static void ospf_header_dump(struct ospf_header *ospfh)
 {
 	char buf[9];
-	u_int16_t auth_type = ntohs(ospfh->auth_type);
+	uint16_t auth_type = ntohs(ospfh->auth_type);
 
 	zlog_debug("Header");
 	zlog_debug("  Version %d", ospfh->version);
@@ -549,7 +553,7 @@ static void ospf_header_dump(struct ospf_header *ospfh)
 		zlog_debug("  Key ID %d", ospfh->u.crypt.key_id);
 		zlog_debug("  Auth Data Len %d", ospfh->u.crypt.auth_data_len);
 		zlog_debug("  Sequence number %ld",
-			   (u_long)ntohl(ospfh->u.crypt.crypt_seqnum));
+			   (unsigned long)ntohl(ospfh->u.crypt.crypt_seqnum));
 		break;
 	default:
 		zlog_debug("* This is not supported authentication type");
@@ -566,7 +570,7 @@ void ospf_packet_dump(struct stream *s)
 	gp = stream_get_getp(s);
 
 	/* OSPF Header dump. */
-	ospfh = (struct ospf_header *)STREAM_PNT(s);
+	ospfh = (struct ospf_header *)stream_pnt(s);
 
 	/* Until detail flag is set, return. */
 	if (!(term_debug_ospf_packet[ospfh->type - 1] & OSPF_DEBUG_DETAIL))
@@ -627,7 +631,7 @@ DEFUN (debug_ospf_packet,
 	if (inst) // user passed instance ID
 	{
 		if (!ospf_lookup_instance(strtoul(argv[2]->arg, NULL, 10)))
-			return CMD_SUCCESS;
+			return CMD_NOT_MY_INSTANCE;
 	}
 
 	int type = 0;
@@ -703,7 +707,7 @@ DEFUN (no_debug_ospf_packet,
 	if (inst) // user passed instance ID
 	{
 		if (!ospf_lookup_instance(strtoul(argv[3]->arg, NULL, 10)))
-			return CMD_SUCCESS;
+			return CMD_NOT_MY_INSTANCE;
 	}
 
 	int type = 0;
@@ -774,7 +778,7 @@ DEFUN (debug_ospf_ism,
 	if (inst) // user passed instance ID
 	{
 		if (!ospf_lookup_instance(strtoul(argv[2]->arg, NULL, 10)))
-			return CMD_SUCCESS;
+			return CMD_NOT_MY_INSTANCE;
 	}
 
 	if (vty->node == CONFIG_NODE) {
@@ -825,7 +829,7 @@ DEFUN (no_debug_ospf_ism,
 	if (inst) // user passed instance ID
 	{
 		if (!ospf_lookup_instance(strtoul(argv[3]->arg, NULL, 10)))
-			return CMD_SUCCESS;
+			return CMD_NOT_MY_INSTANCE;
 	}
 
 	if (vty->node == CONFIG_NODE) {
@@ -916,9 +920,9 @@ DEFUN (debug_ospf_instance_nsm,
        "NSM Timer Information\n")
 {
 	int idx_number = 2;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if (!ospf_lookup_instance(instance))
 		return CMD_SUCCESS;
 
@@ -988,11 +992,11 @@ DEFUN (no_debug_ospf_instance_nsm,
        "NSM Timer Information\n")
 {
 	int idx_number = 3;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if (!ospf_lookup_instance(instance))
-		return CMD_SUCCESS;
+		return CMD_NOT_MY_INSTANCE;
 
 	return no_debug_ospf_nsm_common(vty, 5, argc, argv);
 }
@@ -1062,11 +1066,11 @@ DEFUN (debug_ospf_instance_lsa,
        "LSA Refresh\n")
 {
 	int idx_number = 2;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if (!ospf_lookup_instance(instance))
-		return CMD_SUCCESS;
+		return CMD_NOT_MY_INSTANCE;
 
 	return debug_ospf_lsa_common(vty, 4, argc, argv);
 }
@@ -1138,11 +1142,11 @@ DEFUN (no_debug_ospf_instance_lsa,
        "LSA Refres\n")
 {
 	int idx_number = 3;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if (!ospf_lookup_instance(instance))
-		return CMD_SUCCESS;
+		return CMD_NOT_MY_INSTANCE;
 
 	return no_debug_ospf_lsa_common(vty, 5, argc, argv);
 }
@@ -1182,7 +1186,7 @@ DEFUN (debug_ospf_zebra,
        "debug ospf zebra [<interface|redistribute>]",
        DEBUG_STR
        OSPF_STR
-       "OSPF Zebra information\n"
+       ZEBRA_STR
        "Zebra interface\n"
        "Zebra redistribute\n")
 {
@@ -1195,16 +1199,16 @@ DEFUN (debug_ospf_instance_zebra,
        DEBUG_STR
        OSPF_STR
        "Instance ID\n"
-       "OSPF Zebra information\n"
+       ZEBRA_STR
        "Zebra interface\n"
        "Zebra redistribute\n")
 {
 	int idx_number = 2;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if (!ospf_lookup_instance(instance))
-		return CMD_SUCCESS;
+		return CMD_NOT_MY_INSTANCE;
 
 	return debug_ospf_zebra_common(vty, 4, argc, argv);
 }
@@ -1245,7 +1249,7 @@ DEFUN (no_debug_ospf_zebra,
        NO_STR
        DEBUG_STR
        OSPF_STR
-       "OSPF Zebra information\n"
+       ZEBRA_STR
        "Zebra interface\n"
        "Zebra redistribute\n")
 {
@@ -1259,14 +1263,14 @@ DEFUN (no_debug_ospf_instance_zebra,
        DEBUG_STR
        OSPF_STR
        "Instance ID\n"
-       "OSPF Zebra information\n"
+       ZEBRA_STR
        "Zebra interface\n"
        "Zebra redistribute\n")
 {
 	int idx_number = 3;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if (!ospf_lookup_instance(instance))
 		return CMD_SUCCESS;
 
@@ -1310,9 +1314,9 @@ DEFUN (debug_ospf_instance_event,
        "OSPF event information\n")
 {
 	int idx_number = 2;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if (!ospf_lookup_instance(instance))
 		return CMD_SUCCESS;
 
@@ -1332,9 +1336,9 @@ DEFUN (no_debug_ospf_instance_event,
        "OSPF event information\n")
 {
 	int idx_number = 3;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if (!ospf_lookup_instance(instance))
 		return CMD_SUCCESS;
 
@@ -1380,9 +1384,9 @@ DEFUN (debug_ospf_instance_nssa,
        "OSPF nssa information\n")
 {
 	int idx_number = 2;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if (!ospf_lookup_instance(instance))
 		return CMD_SUCCESS;
 
@@ -1402,9 +1406,9 @@ DEFUN (no_debug_ospf_instance_nssa,
        "OSPF nssa information\n")
 {
 	int idx_number = 3;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if (!ospf_lookup_instance(instance))
 		return CMD_SUCCESS;
 
@@ -1441,6 +1445,33 @@ DEFUN (no_debug_ospf_te,
 	return CMD_SUCCESS;
 }
 
+DEFUN (debug_ospf_sr,
+       debug_ospf_sr_cmd,
+       "debug ospf sr",
+       DEBUG_STR
+       OSPF_STR
+       "OSPF-SR information\n")
+{
+	if (vty->node == CONFIG_NODE)
+		CONF_DEBUG_ON(sr, SR);
+	TERM_DEBUG_ON(sr, SR);
+	return CMD_SUCCESS;
+}
+
+DEFUN (no_debug_ospf_sr,
+       no_debug_ospf_sr_cmd,
+       "no debug ospf sr",
+       NO_STR
+       DEBUG_STR
+       OSPF_STR
+       "OSPF-SR information\n")
+{
+	if (vty->node == CONFIG_NODE)
+		CONF_DEBUG_OFF(sr, SR);
+	TERM_DEBUG_OFF(sr, SR);
+	return CMD_SUCCESS;
+}
+
 DEFUN (no_debug_ospf,
        no_debug_ospf_cmd,
        "no debug ospf",
@@ -1504,143 +1535,129 @@ static int show_debugging_ospf_common(struct vty *vty, struct ospf *ospf)
 	int i;
 
 	if (ospf->instance)
-		vty_out(vty, "%sOSPF Instance: %d%s%s", VTY_NEWLINE,
-			ospf->instance, VTY_NEWLINE, VTY_NEWLINE);
+		vty_out(vty, "\nOSPF Instance: %d\n\n", ospf->instance);
 
-	vty_out(vty, "OSPF debugging status:%s", VTY_NEWLINE);
+	vty_out(vty, "OSPF debugging status:\n");
 
 	/* Show debug status for events. */
 	if (IS_DEBUG_OSPF(event, EVENT))
-		vty_out(vty, "  OSPF event debugging is on%s", VTY_NEWLINE);
+		vty_out(vty, "  OSPF event debugging is on\n");
 
 	/* Show debug status for ISM. */
 	if (IS_DEBUG_OSPF(ism, ISM) == OSPF_DEBUG_ISM)
-		vty_out(vty, "  OSPF ISM debugging is on%s", VTY_NEWLINE);
+		vty_out(vty, "  OSPF ISM debugging is on\n");
 	else {
 		if (IS_DEBUG_OSPF(ism, ISM_STATUS))
-			vty_out(vty, "  OSPF ISM status debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty, "  OSPF ISM status debugging is on\n");
 		if (IS_DEBUG_OSPF(ism, ISM_EVENTS))
-			vty_out(vty, "  OSPF ISM event debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty, "  OSPF ISM event debugging is on\n");
 		if (IS_DEBUG_OSPF(ism, ISM_TIMERS))
-			vty_out(vty, "  OSPF ISM timer debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty, "  OSPF ISM timer debugging is on\n");
 	}
 
 	/* Show debug status for NSM. */
 	if (IS_DEBUG_OSPF(nsm, NSM) == OSPF_DEBUG_NSM)
-		vty_out(vty, "  OSPF NSM debugging is on%s", VTY_NEWLINE);
+		vty_out(vty, "  OSPF NSM debugging is on\n");
 	else {
 		if (IS_DEBUG_OSPF(nsm, NSM_STATUS))
-			vty_out(vty, "  OSPF NSM status debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty, "  OSPF NSM status debugging is on\n");
 		if (IS_DEBUG_OSPF(nsm, NSM_EVENTS))
-			vty_out(vty, "  OSPF NSM event debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty, "  OSPF NSM event debugging is on\n");
 		if (IS_DEBUG_OSPF(nsm, NSM_TIMERS))
-			vty_out(vty, "  OSPF NSM timer debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty, "  OSPF NSM timer debugging is on\n");
 	}
 
 	/* Show debug status for OSPF Packets. */
 	for (i = 0; i < 5; i++)
 		if (IS_DEBUG_OSPF_PACKET(i, SEND)
 		    && IS_DEBUG_OSPF_PACKET(i, RECV)) {
-			vty_out(vty, "  OSPF packet %s%s debugging is on%s",
+			vty_out(vty, "  OSPF packet %s%s debugging is on\n",
 				lookup_msg(ospf_packet_type_str, i + 1, NULL),
 				IS_DEBUG_OSPF_PACKET(i, DETAIL) ? " detail"
-								: "",
-				VTY_NEWLINE);
+								: "");
 		} else {
 			if (IS_DEBUG_OSPF_PACKET(i, SEND))
 				vty_out(vty,
-					"  OSPF packet %s send%s debugging is on%s",
+					"  OSPF packet %s send%s debugging is on\n",
 					lookup_msg(ospf_packet_type_str, i + 1,
 						   NULL),
 					IS_DEBUG_OSPF_PACKET(i, DETAIL)
 						? " detail"
-						: "",
-					VTY_NEWLINE);
+						: "");
 			if (IS_DEBUG_OSPF_PACKET(i, RECV))
 				vty_out(vty,
-					"  OSPF packet %s receive%s debugging is on%s",
+					"  OSPF packet %s receive%s debugging is on\n",
 					lookup_msg(ospf_packet_type_str, i + 1,
 						   NULL),
 					IS_DEBUG_OSPF_PACKET(i, DETAIL)
 						? " detail"
-						: "",
-					VTY_NEWLINE);
+						: "");
 		}
 
 	/* Show debug status for OSPF LSAs. */
 	if (IS_DEBUG_OSPF(lsa, LSA) == OSPF_DEBUG_LSA)
-		vty_out(vty, "  OSPF LSA debugging is on%s", VTY_NEWLINE);
+		vty_out(vty, "  OSPF LSA debugging is on\n");
 	else {
 		if (IS_DEBUG_OSPF(lsa, LSA_GENERATE))
-			vty_out(vty, "  OSPF LSA generation debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty, "  OSPF LSA generation debugging is on\n");
 		if (IS_DEBUG_OSPF(lsa, LSA_FLOODING))
-			vty_out(vty, "  OSPF LSA flooding debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty, "  OSPF LSA flooding debugging is on\n");
 		if (IS_DEBUG_OSPF(lsa, LSA_INSTALL))
-			vty_out(vty, "  OSPF LSA install debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty, "  OSPF LSA install debugging is on\n");
 		if (IS_DEBUG_OSPF(lsa, LSA_REFRESH))
-			vty_out(vty, "  OSPF LSA refresh debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty, "  OSPF LSA refresh debugging is on\n");
 	}
 
 	/* Show debug status for Zebra. */
 	if (IS_DEBUG_OSPF(zebra, ZEBRA) == OSPF_DEBUG_ZEBRA)
-		vty_out(vty, "  OSPF Zebra debugging is on%s", VTY_NEWLINE);
+		vty_out(vty, "  OSPF Zebra debugging is on\n");
 	else {
 		if (IS_DEBUG_OSPF(zebra, ZEBRA_INTERFACE))
-			vty_out(vty, "  OSPF Zebra interface debugging is on%s",
-				VTY_NEWLINE);
+			vty_out(vty,
+				"  OSPF Zebra interface debugging is on\n");
 		if (IS_DEBUG_OSPF(zebra, ZEBRA_REDISTRIBUTE))
 			vty_out(vty,
-				"  OSPF Zebra redistribute debugging is on%s",
-				VTY_NEWLINE);
+				"  OSPF Zebra redistribute debugging is on\n");
 	}
 
 	/* Show debug status for NSSA. */
 	if (IS_DEBUG_OSPF(nssa, NSSA) == OSPF_DEBUG_NSSA)
-		vty_out(vty, "  OSPF NSSA debugging is on%s", VTY_NEWLINE);
+		vty_out(vty, "  OSPF NSSA debugging is on\n");
 
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 
 	return CMD_SUCCESS;
 }
 
-DEFUN (show_debugging_ospf,
-       show_debugging_ospf_cmd,
-       "show debugging ospf",
-       SHOW_STR
-       DEBUG_STR
-       OSPF_STR)
+DEFUN_NOSH (show_debugging_ospf,
+	    show_debugging_ospf_cmd,
+	    "show debugging [ospf]",
+	    SHOW_STR
+	    DEBUG_STR
+	    OSPF_STR)
 {
-	struct ospf *ospf;
+	struct ospf *ospf = NULL;
 
-	if ((ospf = ospf_lookup()) == NULL)
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+	if (ospf == NULL)
 		return CMD_SUCCESS;
 
 	return show_debugging_ospf_common(vty, ospf);
 }
 
-DEFUN (show_debugging_ospf_instance,
-       show_debugging_ospf_instance_cmd,
-       "show debugging ospf (1-65535)",
-       SHOW_STR
-       DEBUG_STR
-       OSPF_STR
-       "Instance ID\n")
+DEFUN_NOSH (show_debugging_ospf_instance,
+	    show_debugging_ospf_instance_cmd,
+	    "show debugging ospf (1-65535)",
+	    SHOW_STR
+	    DEBUG_STR
+	    OSPF_STR
+	    "Instance ID\n")
 {
 	int idx_number = 3;
 	struct ospf *ospf;
-	u_short instance = 0;
+	unsigned short instance = 0;
 
-	VTY_GET_INTEGER("Instance", instance, argv[idx_number]->arg);
+	instance = strtoul(argv[idx_number]->arg, NULL, 10);
 	if ((ospf = ospf_lookup_instance(instance)) == NULL)
 		return CMD_SUCCESS;
 
@@ -1667,7 +1684,8 @@ static int config_write_debug(struct vty *vty)
 	char str[16];
 	memset(str, 0, 16);
 
-	if ((ospf = ospf_lookup()) == NULL)
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+	if (ospf == NULL)
 		return CMD_SUCCESS;
 
 	if (ospf->instance)
@@ -1675,77 +1693,65 @@ static int config_write_debug(struct vty *vty)
 
 	/* debug ospf ism (status|events|timers). */
 	if (IS_CONF_DEBUG_OSPF(ism, ISM) == OSPF_DEBUG_ISM)
-		vty_out(vty, "debug ospf%s ism%s", str, VTY_NEWLINE);
+		vty_out(vty, "debug ospf%s ism\n", str);
 	else {
 		if (IS_CONF_DEBUG_OSPF(ism, ISM_STATUS))
-			vty_out(vty, "debug ospf%s ism status%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s ism status\n", str);
 		if (IS_CONF_DEBUG_OSPF(ism, ISM_EVENTS))
-			vty_out(vty, "debug ospf%s ism event%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s ism event\n", str);
 		if (IS_CONF_DEBUG_OSPF(ism, ISM_TIMERS))
-			vty_out(vty, "debug ospf%s ism timer%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s ism timer\n", str);
 	}
 
 	/* debug ospf nsm (status|events|timers). */
 	if (IS_CONF_DEBUG_OSPF(nsm, NSM) == OSPF_DEBUG_NSM)
-		vty_out(vty, "debug ospf%s nsm%s", str, VTY_NEWLINE);
+		vty_out(vty, "debug ospf%s nsm\n", str);
 	else {
 		if (IS_CONF_DEBUG_OSPF(nsm, NSM_STATUS))
-			vty_out(vty, "debug ospf%s nsm status%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s nsm status\n", str);
 		if (IS_CONF_DEBUG_OSPF(nsm, NSM_EVENTS))
-			vty_out(vty, "debug ospf%s nsm event%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s nsm event\n", str);
 		if (IS_CONF_DEBUG_OSPF(nsm, NSM_TIMERS))
-			vty_out(vty, "debug ospf%s nsm timer%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s nsm timer\n", str);
 	}
 
 	/* debug ospf lsa (generate|flooding|install|refresh). */
 	if (IS_CONF_DEBUG_OSPF(lsa, LSA) == OSPF_DEBUG_LSA)
-		vty_out(vty, "debug ospf%s lsa%s", str, VTY_NEWLINE);
+		vty_out(vty, "debug ospf%s lsa\n", str);
 	else {
 		if (IS_CONF_DEBUG_OSPF(lsa, LSA_GENERATE))
-			vty_out(vty, "debug ospf%s lsa generate%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s lsa generate\n", str);
 		if (IS_CONF_DEBUG_OSPF(lsa, LSA_FLOODING))
-			vty_out(vty, "debug ospf%s lsa flooding%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s lsa flooding\n", str);
 		if (IS_CONF_DEBUG_OSPF(lsa, LSA_INSTALL))
-			vty_out(vty, "debug ospf%s lsa install%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s lsa install\n", str);
 		if (IS_CONF_DEBUG_OSPF(lsa, LSA_REFRESH))
-			vty_out(vty, "debug ospf%s lsa refresh%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s lsa refresh\n", str);
 
 		write = 1;
 	}
 
 	/* debug ospf zebra (interface|redistribute). */
 	if (IS_CONF_DEBUG_OSPF(zebra, ZEBRA) == OSPF_DEBUG_ZEBRA)
-		vty_out(vty, "debug ospf%s zebra%s", str, VTY_NEWLINE);
+		vty_out(vty, "debug ospf%s zebra\n", str);
 	else {
 		if (IS_CONF_DEBUG_OSPF(zebra, ZEBRA_INTERFACE))
-			vty_out(vty, "debug ospf%s zebra interface%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s zebra interface\n", str);
 		if (IS_CONF_DEBUG_OSPF(zebra, ZEBRA_REDISTRIBUTE))
-			vty_out(vty, "debug ospf%s zebra redistribute%s", str,
-				VTY_NEWLINE);
+			vty_out(vty, "debug ospf%s zebra redistribute\n", str);
 
 		write = 1;
 	}
 
 	/* debug ospf event. */
 	if (IS_CONF_DEBUG_OSPF(event, EVENT) == OSPF_DEBUG_EVENT) {
-		vty_out(vty, "debug ospf%s event%s", str, VTY_NEWLINE);
+		vty_out(vty, "debug ospf%s event\n", str);
 		write = 1;
 	}
 
 	/* debug ospf nssa. */
 	if (IS_CONF_DEBUG_OSPF(nssa, NSSA) == OSPF_DEBUG_NSSA) {
-		vty_out(vty, "debug ospf%s nssa%s", str, VTY_NEWLINE);
+		vty_out(vty, "debug ospf%s nssa\n", str);
 		write = 1;
 	}
 
@@ -1755,8 +1761,7 @@ static int config_write_debug(struct vty *vty)
 		r &= conf_debug_ospf_packet[i]
 		     & (OSPF_DEBUG_SEND_RECV | OSPF_DEBUG_DETAIL);
 	if (r == (OSPF_DEBUG_SEND_RECV | OSPF_DEBUG_DETAIL)) {
-		vty_out(vty, "debug ospf%s packet all detail%s", str,
-			VTY_NEWLINE);
+		vty_out(vty, "debug ospf%s packet all detail\n", str);
 		return 1;
 	}
 
@@ -1765,11 +1770,11 @@ static int config_write_debug(struct vty *vty)
 	for (i = 0; i < 5; i++)
 		r &= conf_debug_ospf_packet[i] & OSPF_DEBUG_SEND_RECV;
 	if (r == OSPF_DEBUG_SEND_RECV) {
-		vty_out(vty, "debug ospf%s packet all%s", str, VTY_NEWLINE);
+		vty_out(vty, "debug ospf%s packet all\n", str);
 		for (i = 0; i < 5; i++)
 			if (conf_debug_ospf_packet[i] & OSPF_DEBUG_DETAIL)
-				vty_out(vty, "debug ospf%s packet %s detail%s",
-					str, type_str[i], VTY_NEWLINE);
+				vty_out(vty, "debug ospf%s packet %s detail\n",
+					str, type_str[i]);
 		return 1;
 	}
 
@@ -1779,8 +1784,20 @@ static int config_write_debug(struct vty *vty)
 		if (conf_debug_ospf_packet[i] == 0)
 			continue;
 
-		vty_out(vty, "debug ospf%s packet %s%s%s", str, type_str[i],
-			detail_str[conf_debug_ospf_packet[i]], VTY_NEWLINE);
+		vty_out(vty, "debug ospf%s packet %s%s\n", str, type_str[i],
+			detail_str[conf_debug_ospf_packet[i]]);
+		write = 1;
+	}
+
+	/* debug ospf te */
+	if (IS_CONF_DEBUG_OSPF(te, TE) == OSPF_DEBUG_TE) {
+		vty_out(vty, "debug ospf%s te\n", str);
+		write = 1;
+	}
+
+	/* debug ospf sr */
+	if (IS_CONF_DEBUG_OSPF(sr, SR) == OSPF_DEBUG_SR) {
+		vty_out(vty, "debug ospf%s sr\n", str);
 		write = 1;
 	}
 
@@ -1800,6 +1817,7 @@ void debug_init()
 	install_element(ENABLE_NODE, &debug_ospf_event_cmd);
 	install_element(ENABLE_NODE, &debug_ospf_nssa_cmd);
 	install_element(ENABLE_NODE, &debug_ospf_te_cmd);
+	install_element(ENABLE_NODE, &debug_ospf_sr_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_ism_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_nsm_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_lsa_cmd);
@@ -1807,6 +1825,7 @@ void debug_init()
 	install_element(ENABLE_NODE, &no_debug_ospf_event_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_nssa_cmd);
 	install_element(ENABLE_NODE, &no_debug_ospf_te_cmd);
+	install_element(ENABLE_NODE, &no_debug_ospf_sr_cmd);
 
 	install_element(ENABLE_NODE, &show_debugging_ospf_instance_cmd);
 	install_element(ENABLE_NODE, &debug_ospf_packet_cmd);
@@ -1835,12 +1854,14 @@ void debug_init()
 	install_element(CONFIG_NODE, &debug_ospf_event_cmd);
 	install_element(CONFIG_NODE, &debug_ospf_nssa_cmd);
 	install_element(CONFIG_NODE, &debug_ospf_te_cmd);
+	install_element(CONFIG_NODE, &debug_ospf_sr_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_nsm_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_lsa_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_zebra_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_event_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_nssa_cmd);
 	install_element(CONFIG_NODE, &no_debug_ospf_te_cmd);
+	install_element(CONFIG_NODE, &no_debug_ospf_sr_cmd);
 
 	install_element(CONFIG_NODE, &debug_ospf_instance_nsm_cmd);
 	install_element(CONFIG_NODE, &debug_ospf_instance_lsa_cmd);
