@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -52,11 +51,33 @@ DEFINE_QOBJ_TYPE(ospf_interface)
 DEFINE_HOOK(ospf_vl_add, (struct ospf_vl_data * vd), (vd))
 DEFINE_HOOK(ospf_vl_delete, (struct ospf_vl_data * vd), (vd))
 
+int ospf_interface_neighbor_count(struct ospf_interface *oi)
+{
+	int count = 0;
+	struct route_node *rn;
+	struct ospf_neighbor *nbr = NULL;
+
+	for (rn = route_top(oi->nbrs); rn; rn = route_next(rn)) {
+		nbr = rn->info;
+		if (nbr) {
+			/* Do not show myself. */
+			if (nbr == oi->nbr_self)
+				continue;
+			/* Down state is not shown. */
+			if (nbr->state == NSM_Down)
+				continue;
+			count++;
+		}
+	}
+
+	return count;
+}
+
 int ospf_if_get_output_cost(struct ospf_interface *oi)
 {
 	/* If all else fails, use default OSPF cost */
-	u_int32_t cost;
-	u_int32_t bw, refbw;
+	uint32_t cost;
+	uint32_t bw, refbw;
 
 	/* ifp speed and bw can be 0 in some platforms, use ospf default bw
 	   if bw is configured under interface it would be used.
@@ -75,7 +96,7 @@ int ospf_if_get_output_cost(struct ospf_interface *oi)
 	/* See if a cost can be calculated from the zebra processes
 	   interface bandwidth field. */
 	else {
-		cost = (u_int32_t)((double)refbw / (double)bw + (double)0.5);
+		cost = (uint32_t)((double)refbw / (double)bw + (double)0.5);
 		if (cost < 1)
 			cost = 1;
 		else if (cost > 65535)
@@ -87,7 +108,7 @@ int ospf_if_get_output_cost(struct ospf_interface *oi)
 
 void ospf_if_recalculate_output_cost(struct interface *ifp)
 {
-	u_int32_t newcost;
+	uint32_t newcost;
 	struct route_node *rn;
 
 	for (rn = route_top(IF_OIFS(ifp)); rn; rn = route_next(rn)) {
@@ -172,6 +193,7 @@ static void ospf_add_to_if(struct interface *ifp, struct ospf_interface *oi)
 
 	p = *oi->address;
 	p.prefixlen = IPV4_MAX_PREFIXLEN;
+	apply_mask(&p);
 
 	rn = route_node_get(IF_OIFS(ifp), &p);
 	/* rn->info should either be NULL or equal to this oi
@@ -243,6 +265,11 @@ struct ospf_interface *ospf_if_new(struct ospf *ospf, struct interface *ifp,
 	oi->ospf = ospf;
 	QOBJ_REG(oi, ospf_interface);
 
+	if (IS_DEBUG_OSPF_EVENT)
+		zlog_debug("%s: ospf interface %s vrf %s id %u created",
+			   __PRETTY_FUNCTION__, ifp->name,
+			   ospf_vrf_id_to_name(ospf->vrf_id), ospf->vrf_id);
+
 	return oi;
 }
 
@@ -308,10 +335,16 @@ void ospf_if_free(struct ospf_interface *oi)
 	route_table_finish(oi->ls_upd_queue);
 
 	/* Free any lists that should be freed */
-	list_free(oi->nbr_nbma);
+	list_delete_and_null(&oi->nbr_nbma);
 
-	list_free(oi->ls_ack);
-	list_free(oi->ls_ack_direct.ls_ack);
+	list_delete_and_null(&oi->ls_ack);
+	list_delete_and_null(&oi->ls_ack_direct.ls_ack);
+
+	if (IS_DEBUG_OSPF_EVENT)
+		zlog_debug("%s: ospf interface %s vrf %s id %u deleted",
+			   __PRETTY_FUNCTION__, oi->ifp->name,
+			   ospf_vrf_id_to_name(oi->ifp->vrf_id),
+			   oi->ifp->vrf_id);
 
 	ospf_delete_from_if(oi->ifp, oi);
 
@@ -335,7 +368,11 @@ struct ospf_interface *ospf_if_exists(struct ospf_interface *oic)
 	struct ospf *ospf;
 	struct ospf_interface *oi;
 
-	if ((ospf = ospf_lookup()) == NULL)
+	if (!oic)
+		return NULL;
+
+	ospf = oic->ospf;
+	if (ospf == NULL)
 		return NULL;
 
 	for (ALL_LIST_ELEMENTS_RO(ospf->oiflist, node, oi))
@@ -422,7 +459,7 @@ struct ospf_interface *ospf_if_lookup_recv_if(struct ospf *ospf,
 		if (oi->type == OSPF_IFTYPE_VIRTUALLINK)
 			continue;
 
-		if (if_is_loopback(oi->ifp))
+		if (if_is_loopback(oi->ifp) || if_is_vrf(oi->ifp))
 			continue;
 
 		if (CHECK_FLAG(oi->connected->flags, ZEBRA_IFA_UNNUMBERED))
@@ -438,6 +475,15 @@ struct ospf_interface *ospf_if_lookup_recv_if(struct ospf *ospf,
 	return match;
 }
 
+static void ospf_if_reset_stats(struct ospf_interface *oi)
+{
+	oi->hello_in = oi->hello_out = 0;
+	oi->db_desc_in = oi->db_desc_out = 0;
+	oi->ls_req_in = oi->ls_req_out = 0;
+	oi->ls_upd_in = oi->ls_upd_out = 0;
+	oi->ls_ack_in = oi->ls_ack_out = 0;
+}
+
 void ospf_if_stream_set(struct ospf_interface *oi)
 {
 	/* set output fifo queue. */
@@ -453,6 +499,9 @@ void ospf_if_stream_unset(struct ospf_interface *oi)
 		ospf_fifo_free(oi->obuf);
 		oi->obuf = NULL;
 
+		/*reset protocol stats */
+		ospf_if_reset_stats(oi);
+
 		if (oi->on_write_q) {
 			listnode_delete(ospf->oi_write_q, oi);
 			if (list_isempty(ospf->oi_write_q))
@@ -494,7 +543,7 @@ static struct ospf_if_params *ospf_new_if_params(void)
 
 void ospf_del_if_params(struct ospf_if_params *oip)
 {
-	list_delete(oip->auth_crypt);
+	list_delete_and_null(&oip->auth_crypt);
 	bfd_info_free(&(oip->bfd_info));
 	XFREE(MTYPE_OSPF_IF_PARAMS, oip);
 }
@@ -563,6 +612,7 @@ struct ospf_if_params *ospf_get_if_params(struct interface *ifp,
 	p.family = AF_INET;
 	p.prefixlen = IPV4_MAX_PREFIXLEN;
 	p.prefix = addr;
+	apply_mask_ipv4(&p);
 
 	rn = route_node_get(IF_OIFS_PARAMS(ifp), (struct prefix *)&p);
 
@@ -653,7 +703,7 @@ static int ospf_if_delete_hook(struct interface *ifp)
 
 int ospf_if_is_enable(struct ospf_interface *oi)
 {
-	if (!if_is_loopback(oi->ifp))
+	if (!(if_is_loopback(oi->ifp) || if_is_vrf(oi->ifp)))
 		if (if_is_up(oi->ifp))
 			return 1;
 
@@ -775,14 +825,14 @@ void ospf_vl_data_free(struct ospf_vl_data *vl_data)
 	XFREE(MTYPE_OSPF_VL_DATA, vl_data);
 }
 
-u_int vlink_count = 0;
+unsigned int vlink_count = 0;
 
 struct ospf_interface *ospf_vl_new(struct ospf *ospf,
 				   struct ospf_vl_data *vl_data)
 {
 	struct ospf_interface *voi;
 	struct interface *vi;
-	char ifname[INTERFACE_NAMSIZ + 1];
+	char ifname[INTERFACE_NAMSIZ];
 	struct ospf_area *area;
 	struct in_addr area_id;
 	struct connected *co;
@@ -799,10 +849,12 @@ struct ospf_interface *ospf_vl_new(struct ospf *ospf,
 	}
 
 	if (IS_DEBUG_OSPF_EVENT)
-		zlog_debug("ospf_vl_new(): creating pseudo zebra interface");
+		zlog_debug(
+			"ospf_vl_new(): creating pseudo zebra interface vrf id %u",
+			ospf->vrf_id);
 
 	snprintf(ifname, sizeof(ifname), "VLINK%d", vlink_count);
-	vi = if_create(ifname, strnlen(ifname, sizeof(ifname)), VRF_DEFAULT);
+	vi = if_create(ifname, ospf->vrf_id);
 	/*
 	 * if_create sets ZEBRA_INTERFACE_LINKDETECTION
 	 * virtual links don't need this.
@@ -988,7 +1040,7 @@ static int ospf_vl_set_params(struct ospf_vl_data *vl_data, struct vertex *v)
 				if (IS_DEBUG_OSPF_EVENT)
 					zlog_debug(
 						"found back link through VL");
-				/* fallthru */
+			/* fallthru */
 			case LSA_LINK_TYPE_TRANSIT:
 			case LSA_LINK_TYPE_POINTOPOINT:
 				if (!IPV4_ADDR_SAME(&vl_data->peer_addr,
@@ -1122,7 +1174,7 @@ void ospf_crypt_key_add(struct list *crypt, struct crypt_key *ck)
 	listnode_add(crypt, ck);
 }
 
-struct crypt_key *ospf_crypt_key_lookup(struct list *auth_crypt, u_char key_id)
+struct crypt_key *ospf_crypt_key_lookup(struct list *auth_crypt, uint8_t key_id)
 {
 	struct listnode *node;
 	struct crypt_key *ck;
@@ -1134,7 +1186,7 @@ struct crypt_key *ospf_crypt_key_lookup(struct list *auth_crypt, u_char key_id)
 	return NULL;
 }
 
-int ospf_crypt_key_delete(struct list *auth_crypt, u_char key_id)
+int ospf_crypt_key_delete(struct list *auth_crypt, uint8_t key_id)
 {
 	struct listnode *node, *nnode;
 	struct crypt_key *ck;
@@ -1150,11 +1202,11 @@ int ospf_crypt_key_delete(struct list *auth_crypt, u_char key_id)
 	return 0;
 }
 
-u_char ospf_default_iftype(struct interface *ifp)
+uint8_t ospf_default_iftype(struct interface *ifp)
 {
 	if (if_is_pointopoint(ifp))
 		return OSPF_IFTYPE_POINTOPOINT;
-	else if (if_is_loopback(ifp))
+	else if (if_is_loopback(ifp) || if_is_vrf(ifp))
 		return OSPF_IFTYPE_LOOPBACK;
 	else
 		return OSPF_IFTYPE_BROADCAST;
@@ -1163,7 +1215,6 @@ u_char ospf_default_iftype(struct interface *ifp)
 void ospf_if_init()
 {
 	/* Initialize Zebra interface data structure. */
-	om->iflist = vrf_iflist(VRF_DEFAULT);
-	if_add_hook(IF_NEW_HOOK, ospf_if_new_hook);
-	if_add_hook(IF_DELETE_HOOK, ospf_if_delete_hook);
+	hook_register_prio(if_add, 0, ospf_if_new_hook);
+	hook_register_prio(if_del, 0, ospf_if_delete_hook);
 }
