@@ -14,10 +14,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -50,11 +49,12 @@
 #include "ospfd/ospf_route.h"
 #include "ospfd/ospf_ase.h"
 #include "ospfd/ospf_zebra.h"
+#include "ospfd/ospf_abr.h"
 
 
-u_int32_t get_metric(u_char *metric)
+uint32_t get_metric(uint8_t *metric)
 {
-	u_int32_t m;
+	uint32_t m;
 	m = metric[0];
 	m = (m << 8) + metric[1];
 	m = (m << 8) + metric[2];
@@ -121,28 +121,28 @@ int get_age(struct ospf_lsa *lsa)
 
 /* All the offsets are zero-based. The offsets in the RFC1008 are
    one-based. */
-u_int16_t ospf_lsa_checksum(struct lsa_header *lsa)
+uint16_t ospf_lsa_checksum(struct lsa_header *lsa)
 {
-	u_char *buffer = (u_char *)&lsa->options;
-	int options_offset = buffer - (u_char *)&lsa->ls_age; /* should be 2 */
+	uint8_t *buffer = (uint8_t *)&lsa->options;
+	int options_offset = buffer - (uint8_t *)&lsa->ls_age; /* should be 2 */
 
 	/* Skip the AGE field */
-	u_int16_t len = ntohs(lsa->length) - options_offset;
+	uint16_t len = ntohs(lsa->length) - options_offset;
 
 	/* Checksum offset starts from "options" field, not the beginning of the
 	   lsa_header struct. The offset is 14, rather than 16. */
-	int checksum_offset = (u_char *)&lsa->checksum - buffer;
+	int checksum_offset = (uint8_t *)&lsa->checksum - buffer;
 
 	return fletcher_checksum(buffer, len, checksum_offset);
 }
 
 int ospf_lsa_checksum_valid(struct lsa_header *lsa)
 {
-	u_char *buffer = (u_char *)&lsa->options;
-	int options_offset = buffer - (u_char *)&lsa->ls_age; /* should be 2 */
+	uint8_t *buffer = (uint8_t *)&lsa->options;
+	int options_offset = buffer - (uint8_t *)&lsa->ls_age; /* should be 2 */
 
 	/* Skip the AGE field */
-	u_int16_t len = ntohs(lsa->length) - options_offset;
+	uint16_t len = ntohs(lsa->length) - options_offset;
 
 	return (fletcher_checksum(buffer, len, FLETCHER_CHECKSUM_VALIDATE)
 		== 0);
@@ -162,6 +162,7 @@ struct ospf_lsa *ospf_lsa_new()
 	monotime(&new->tv_recv);
 	new->tv_orig = new->tv_recv;
 	new->refresh_list = -1;
+	new->vrf_id = VRF_DEFAULT;
 
 	return new;
 }
@@ -284,26 +285,26 @@ const char *dump_lsa_key(struct ospf_lsa *lsa)
 
 	if (lsa != NULL && (lsah = lsa->data) != NULL) {
 		char id[INET_ADDRSTRLEN], ar[INET_ADDRSTRLEN];
-		strcpy(id, inet_ntoa(lsah->id));
-		strcpy(ar, inet_ntoa(lsah->adv_router));
+		strlcpy(id, inet_ntoa(lsah->id), sizeof(id));
+		strlcpy(ar, inet_ntoa(lsah->adv_router), sizeof(ar));
 
 		sprintf(buf, "Type%d,id(%s),ar(%s)", lsah->type, id, ar);
 	} else
-		strcpy(buf, "NULL");
+		strlcpy(buf, "NULL", sizeof(buf));
 
 	return buf;
 }
 
-u_int32_t lsa_seqnum_increment(struct ospf_lsa *lsa)
+uint32_t lsa_seqnum_increment(struct ospf_lsa *lsa)
 {
-	u_int32_t seqnum;
+	uint32_t seqnum;
 
 	seqnum = ntohl(lsa->data->ls_seqnum) + 1;
 
 	return htonl(seqnum);
 }
 
-void lsa_header_set(struct stream *s, u_char options, u_char type,
+void lsa_header_set(struct stream *s, uint8_t options, uint8_t type,
 		    struct in_addr id, struct in_addr router_id)
 {
 	struct lsa_header *lsah;
@@ -323,9 +324,9 @@ void lsa_header_set(struct stream *s, u_char options, u_char type,
 
 /* router-LSA related functions. */
 /* Get router-LSA flags. */
-static u_char router_lsa_flags(struct ospf_area *area)
+static uint8_t router_lsa_flags(struct ospf_area *area)
 {
-	u_char flags;
+	uint8_t flags;
 
 	flags = area->ospf->flags;
 
@@ -395,7 +396,7 @@ struct ospf_neighbor *ospf_nbr_lookup_ptop(struct ospf_interface *oi)
 /* Determine cost of link, taking RFC3137 stub-router support into
  * consideration
  */
-static u_int16_t ospf_link_cost(struct ospf_interface *oi)
+static uint16_t ospf_link_cost(struct ospf_interface *oi)
 {
 	/* RFC3137 stub router support */
 	if (!CHECK_FLAG(oi->area->stub_router_state, OSPF_AREA_IS_STUB_ROUTED))
@@ -406,8 +407,8 @@ static u_int16_t ospf_link_cost(struct ospf_interface *oi)
 
 /* Set a link information. */
 static char link_info_set(struct stream *s, struct in_addr id,
-			  struct in_addr data, u_char type, u_char tos,
-			  u_int16_t cost)
+			  struct in_addr data, uint8_t type, uint8_t tos,
+			  uint16_t cost)
 {
 	/* LSA stream is initially allocated to OSPF_MAX_LSA_SIZE, suits
 	 * vast majority of cases. Some rare routers with lots of links need
@@ -437,7 +438,7 @@ static char link_info_set(struct stream *s, struct in_addr id,
 		if (ret == OSPF_MAX_LSA_SIZE) {
 			zlog_warn(
 				"%s: Out of space in LSA stream, left %zd, size %zd",
-				__func__, STREAM_REMAIN(s), STREAM_SIZE(s));
+				__func__, STREAM_WRITEABLE(s), STREAM_SIZE(s));
 			return 0;
 		}
 	}
@@ -458,7 +459,7 @@ static int lsa_link_ptop_set(struct stream *s, struct ospf_interface *oi)
 	int links = 0;
 	struct ospf_neighbor *nbr;
 	struct in_addr id, mask, data;
-	u_int16_t cost = ospf_link_cost(oi);
+	uint16_t cost = ospf_link_cost(oi);
 
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE))
 		zlog_debug("LSA[Type1]: Set link Point-to-Point");
@@ -503,7 +504,7 @@ static int lsa_link_broadcast_set(struct stream *s, struct ospf_interface *oi)
 {
 	struct ospf_neighbor *dr;
 	struct in_addr id, mask;
-	u_int16_t cost = ospf_link_cost(oi);
+	uint16_t cost = ospf_link_cost(oi);
 
 	/* Describe Type 3 Link. */
 	if (oi->state == ISM_Waiting) {
@@ -562,7 +563,7 @@ static int lsa_link_loopback_set(struct stream *s, struct ospf_interface *oi)
 static int lsa_link_virtuallink_set(struct stream *s, struct ospf_interface *oi)
 {
 	struct ospf_neighbor *nbr;
-	u_int16_t cost = ospf_link_cost(oi);
+	uint16_t cost = ospf_link_cost(oi);
 
 	if (oi->state == ISM_PointToPoint)
 		if ((nbr = ospf_nbr_lookup_ptop(oi)))
@@ -588,7 +589,7 @@ static int lsa_link_ptomp_set(struct stream *s, struct ospf_interface *oi)
 	struct route_node *rn;
 	struct ospf_neighbor *nbr = NULL;
 	struct in_addr id, mask;
-	u_int16_t cost = ospf_link_cost(oi);
+	uint16_t cost = ospf_link_cost(oi);
 
 	mask.s_addr = 0xffffffff;
 	id.s_addr = oi->address->u.prefix4.s_addr;
@@ -669,7 +670,7 @@ static int router_lsa_link_set(struct stream *s, struct ospf_area *area)
 static void ospf_router_lsa_body_set(struct stream *s, struct ospf_area *area)
 {
 	unsigned long putp;
-	u_int16_t cnt;
+	uint16_t cnt;
 
 	/* Set flags. */
 	stream_putc(s, router_lsa_flags(area));
@@ -787,6 +788,7 @@ static struct ospf_lsa *ospf_router_lsa_new(struct ospf_area *area)
 
 	new->area = area;
 	SET_FLAG(new->flags, OSPF_LSA_SELF | OSPF_LSA_SELF_CHECKED);
+	new->vrf_id = area->ospf->vrf_id;
 
 	/* Copy LSA data to store, discard stream. */
 	new->data = ospf_lsa_data_new(length);
@@ -1002,6 +1004,7 @@ static struct ospf_lsa *ospf_network_lsa_new(struct ospf_interface *oi)
 
 	new->area = oi->area;
 	SET_FLAG(new->flags, OSPF_LSA_SELF | OSPF_LSA_SELF_CHECKED);
+	new->vrf_id = oi->ospf->vrf_id;
 
 	/* Copy LSA to store. */
 	new->data = ospf_lsa_data_new(length);
@@ -1112,9 +1115,9 @@ static struct ospf_lsa *ospf_network_lsa_refresh(struct ospf_lsa *lsa)
 	return new;
 }
 
-static void stream_put_ospf_metric(struct stream *s, u_int32_t metric_value)
+static void stream_put_ospf_metric(struct stream *s, uint32_t metric_value)
 {
-	u_int32_t metric;
+	uint32_t metric;
 	char *mp;
 
 	/* Put 0 metric. TOS metric is not supported. */
@@ -1126,7 +1129,7 @@ static void stream_put_ospf_metric(struct stream *s, u_int32_t metric_value)
 
 /* summary-LSA related functions. */
 static void ospf_summary_lsa_body_set(struct stream *s, struct prefix *p,
-				      u_int32_t metric)
+				      uint32_t metric)
 {
 	struct in_addr mask;
 
@@ -1136,14 +1139,14 @@ static void ospf_summary_lsa_body_set(struct stream *s, struct prefix *p,
 	stream_put_ipv4(s, mask.s_addr);
 
 	/* Set # TOS. */
-	stream_putc(s, (u_char)0);
+	stream_putc(s, (uint8_t)0);
 
 	/* Set metric. */
 	stream_put_ospf_metric(s, metric);
 }
 
 static struct ospf_lsa *ospf_summary_lsa_new(struct ospf_area *area,
-					     struct prefix *p, u_int32_t metric,
+					     struct prefix *p, uint32_t metric,
 					     struct in_addr id)
 {
 	struct stream *s;
@@ -1181,6 +1184,7 @@ static struct ospf_lsa *ospf_summary_lsa_new(struct ospf_area *area,
 	new = ospf_lsa_new();
 	new->area = area;
 	SET_FLAG(new->flags, OSPF_LSA_SELF | OSPF_LSA_SELF_CHECKED);
+	new->vrf_id = area->ospf->vrf_id;
 
 	/* Copy LSA to store. */
 	new->data = ospf_lsa_data_new(length);
@@ -1192,7 +1196,7 @@ static struct ospf_lsa *ospf_summary_lsa_new(struct ospf_area *area,
 
 /* Originate Summary-LSA. */
 struct ospf_lsa *ospf_summary_lsa_originate(struct prefix_ipv4 *p,
-					    u_int32_t metric,
+					    uint32_t metric,
 					    struct ospf_area *area)
 {
 	struct ospf_lsa *new;
@@ -1270,13 +1274,13 @@ static struct ospf_lsa *ospf_summary_lsa_refresh(struct ospf *ospf,
 
 /* summary-ASBR-LSA related functions. */
 static void ospf_summary_asbr_lsa_body_set(struct stream *s, struct prefix *p,
-					   u_int32_t metric)
+					   uint32_t metric)
 {
 	/* Put Network Mask. */
-	stream_put_ipv4(s, (u_int32_t)0);
+	stream_put_ipv4(s, (uint32_t)0);
 
 	/* Set # TOS. */
-	stream_putc(s, (u_char)0);
+	stream_putc(s, (uint8_t)0);
 
 	/* Set metric. */
 	stream_put_ospf_metric(s, metric);
@@ -1284,7 +1288,7 @@ static void ospf_summary_asbr_lsa_body_set(struct stream *s, struct prefix *p,
 
 static struct ospf_lsa *ospf_summary_asbr_lsa_new(struct ospf_area *area,
 						  struct prefix *p,
-						  u_int32_t metric,
+						  uint32_t metric,
 						  struct in_addr id)
 {
 	struct stream *s;
@@ -1322,6 +1326,7 @@ static struct ospf_lsa *ospf_summary_asbr_lsa_new(struct ospf_area *area,
 	new = ospf_lsa_new();
 	new->area = area;
 	SET_FLAG(new->flags, OSPF_LSA_SELF | OSPF_LSA_SELF_CHECKED);
+	new->vrf_id = area->ospf->vrf_id;
 
 	/* Copy LSA to store. */
 	new->data = ospf_lsa_data_new(length);
@@ -1333,7 +1338,7 @@ static struct ospf_lsa *ospf_summary_asbr_lsa_new(struct ospf_area *area,
 
 /* Originate summary-ASBR-LSA. */
 struct ospf_lsa *ospf_summary_asbr_lsa_originate(struct prefix_ipv4 *p,
-						 u_int32_t metric,
+						 uint32_t metric,
 						 struct ospf_area *area)
 {
 	struct ospf_lsa *new;
@@ -1496,7 +1501,7 @@ struct in_addr ospf_get_nssa_ip(struct ospf_area *area)
 
 #define DEFAULT_METRIC_TYPE		     EXTERNAL_METRIC_TYPE_2
 
-int metric_type(struct ospf *ospf, u_char src, u_short instance)
+int metric_type(struct ospf *ospf, uint8_t src, unsigned short instance)
 {
 	struct ospf_redist *red;
 
@@ -1506,7 +1511,7 @@ int metric_type(struct ospf *ospf, u_char src, u_short instance)
 						: red->dmetric.type);
 }
 
-int metric_value(struct ospf *ospf, u_char src, u_short instance)
+int metric_value(struct ospf *ospf, uint8_t src, unsigned short instance)
 {
 	struct ospf_redist *red;
 
@@ -1533,10 +1538,10 @@ static void ospf_external_lsa_body_set(struct stream *s,
 {
 	struct prefix_ipv4 *p = &ei->p;
 	struct in_addr mask, fwd_addr;
-	u_int32_t mvalue;
+	uint32_t mvalue;
 	int mtype;
 	int type;
-	u_short instance;
+	unsigned short instance;
 
 	/* Put Network Mask. */
 	masklen2ip(p->prefixlen, &mask);
@@ -1628,6 +1633,7 @@ static struct ospf_lsa *ospf_external_lsa_new(struct ospf *ospf,
 	new->area = NULL;
 	SET_FLAG(new->flags,
 		 OSPF_LSA_SELF | OSPF_LSA_APPROVED | OSPF_LSA_SELF_CHECKED);
+	new->vrf_id = ospf->vrf_id;
 
 	/* Copy LSA data to store, discard stream. */
 	new->data = ospf_lsa_data_new(length);
@@ -1697,7 +1703,7 @@ static void ospf_install_flood_nssa(struct ospf *ospf, struct ospf_lsa *lsa,
 			not adversited into OSPF as an internal OSPF route and
 			the
 			type-7 LSA's P-bit is set a forwarding address should be
-			selected from one of the router's active OSPF inteface
+			selected from one of the router's active OSPF interface
 			addresses
 			which belong to the NSSA.  If no such addresses exist,
 			then
@@ -1750,6 +1756,7 @@ static struct ospf_lsa *ospf_lsa_translated_nssa_new(struct ospf *ospf,
 	ei.route_map_set.metric = -1;
 	ei.route_map_set.metric_type = -1;
 	ei.tag = 0;
+	ei.instance = 0;
 
 	if ((new = ospf_external_lsa_new(ospf, &ei, &type7->data->id))
 	    == NULL) {
@@ -1866,8 +1873,7 @@ struct ospf_lsa *ospf_translated_nssa_refresh(struct ospf *ospf,
 			if (area->external_routing != OSPF_AREA_NSSA && !type7)
 				continue;
 
-			LSDB_LOOP(NSSA_LSDB(area), rn, lsa)
-			{
+			LSDB_LOOP (NSSA_LSDB(area), rn, lsa) {
 				if (lsa->data->id.s_addr
 				    == type5->data->id.s_addr) {
 					type7 = lsa;
@@ -2032,7 +2038,7 @@ int ospf_external_lsa_originate_timer(struct thread *thread)
 
 	ospf->t_external_lsa = NULL;
 
-	ext_list = om->external[type];
+	ext_list = ospf->external[type];
 	if (!ext_list)
 		return 0;
 
@@ -2072,7 +2078,7 @@ static struct external_info *ospf_default_external_info(struct ospf *ospf)
 		if (type == ZEBRA_ROUTE_OSPF)
 			continue;
 
-		ext_list = om->external[type];
+		ext_list = ospf->external[type];
 		if (!ext_list)
 			continue;
 
@@ -2109,7 +2115,8 @@ int ospf_default_originate_timer(struct thread *thread)
 		/* If there is no default route via redistribute,
 		   then originate AS-external-LSA with nexthop 0 (self). */
 		nexthop.s_addr = 0;
-		ospf_external_info_add(DEFAULT_ROUTE, 0, p, 0, nexthop, 0);
+		ospf_external_info_add(ospf, DEFAULT_ROUTE, 0, p, 0, nexthop,
+				       0);
 	}
 
 	if ((ei = ospf_default_external_info(ospf)))
@@ -2122,14 +2129,14 @@ int ospf_default_originate_timer(struct thread *thread)
 void ospf_nssa_lsa_flush(struct ospf *ospf, struct prefix_ipv4 *p)
 {
 	struct listnode *node, *nnode;
-	struct ospf_lsa *lsa;
+	struct ospf_lsa *lsa = NULL;
 	struct ospf_area *area;
 
 	for (ALL_LIST_ELEMENTS(ospf->areas, node, nnode, area)) {
 		if (area->external_routing == OSPF_AREA_NSSA) {
-			if (!(lsa = ospf_lsa_lookup(area, OSPF_AS_NSSA_LSA,
-						    p->prefix,
-						    ospf->router_id))) {
+			lsa = ospf_lsa_lookup(ospf, area, OSPF_AS_NSSA_LSA,
+					      p->prefix, ospf->router_id);
+			if (!lsa) {
 				if (IS_DEBUG_OSPF(lsa, LSA_FLOODING))
 					zlog_debug(
 						"LSA: There is no such AS-NSSA-LSA %s/%d in LSDB",
@@ -2147,7 +2154,7 @@ void ospf_nssa_lsa_flush(struct ospf *ospf, struct prefix_ipv4 *p)
 }
 
 /* Flush an AS-external-LSA from LSDB and routing domain. */
-void ospf_external_lsa_flush(struct ospf *ospf, u_char type,
+void ospf_external_lsa_flush(struct ospf *ospf, uint8_t type,
 			     struct prefix_ipv4 *p,
 			     ifindex_t ifindex /*, struct in_addr nexthop */)
 {
@@ -2232,32 +2239,39 @@ void ospf_external_lsa_refresh_default(struct ospf *ospf)
 	}
 }
 
-void ospf_external_lsa_refresh_type(struct ospf *ospf, u_char type,
-				    u_short instance, int force)
+void ospf_external_lsa_refresh_type(struct ospf *ospf, uint8_t type,
+				    unsigned short instance, int force)
 {
 	struct route_node *rn;
 	struct external_info *ei;
 	struct ospf_external *ext;
 
-	if (type != DEFAULT_ROUTE)
-		if ((ext = ospf_external_lookup(type, instance))
-		    && EXTERNAL_INFO(ext))
-			/* Refresh each redistributed AS-external-LSAs. */
-			for (rn = route_top(EXTERNAL_INFO(ext)); rn;
-			     rn = route_next(rn))
-				if ((ei = rn->info))
-					if (!is_prefix_default(&ei->p)) {
-						struct ospf_lsa *lsa;
+	if (type == DEFAULT_ROUTE)
+		return;
 
-						if ((lsa = ospf_external_info_find_lsa(
-							     ospf, &ei->p)))
-							ospf_external_lsa_refresh(
-								ospf, lsa, ei,
-								force);
-						else
-							ospf_external_lsa_originate(
-								ospf, ei);
-					}
+	ext = ospf_external_lookup(ospf, type, instance);
+
+	if (ext && EXTERNAL_INFO(ext)) {
+		/* Refresh each redistributed AS-external-LSAs. */
+		for (rn = route_top(EXTERNAL_INFO(ext)); rn;
+		     rn = route_next(rn)) {
+			ei = rn->info;
+			if (ei) {
+				if (!is_prefix_default(&ei->p)) {
+					struct ospf_lsa *lsa;
+
+					lsa = ospf_external_info_find_lsa(
+						ospf, &ei->p);
+					if (lsa)
+						ospf_external_lsa_refresh(
+							ospf, lsa, ei, force);
+					else
+						ospf_external_lsa_originate(
+							ospf, ei);
+				}
+			}
+		}
+	}
 }
 
 /* Refresh AS-external-LSA. */
@@ -2417,7 +2431,7 @@ ospf_summary_lsa_install(struct ospf *ospf, struct ospf_lsa *new, int rt_recalc)
 #if 0
       /* This doesn't exist yet... */
       ospf_summary_incremental_update(new); */
-#else  /* #if 0 */
+#else /* #if 0 */
 		ospf_spf_calculate_schedule(ospf, SPF_FLAG_SUMMARY_LSA_INSTALL);
 #endif /* #if 0 */
 	}
@@ -2489,6 +2503,7 @@ static struct ospf_lsa *ospf_external_lsa_install(struct ospf *ospf,
 			 * abr_task.
 			 */
 			ospf_translated_nssa_refresh(ospf, new, NULL);
+			ospf_schedule_abr_task(ospf);
 		}
 	}
 
@@ -2707,7 +2722,8 @@ struct ospf_lsa *ospf_lsa_install(struct ospf *ospf, struct ospf_interface *oi,
 					      new->data->type, NULL));
 			break;
 		default:
-			strcpy(area_str, inet_ntoa(new->area->area_id));
+			strlcpy(area_str, inet_ntoa(new->area->area_id),
+				sizeof(area_str));
 			zlog_debug("LSA[%s]: Install %s to Area %s",
 				   dump_lsa_key(new),
 				   lookup_msg(ospf_lsa_type_msg,
@@ -2803,7 +2819,7 @@ static int ospf_maxage_lsa_remover(struct thread *thread)
 						"LSA[Type%d:%s]: LSA 0x%lx is self-originated: ",
 						lsa->data->type,
 						inet_ntoa(lsa->data->id),
-						(u_long)lsa);
+						(unsigned long)lsa);
 
 			if (IS_DEBUG_OSPF(lsa, LSA_FLOODING))
 				zlog_debug(
@@ -2845,11 +2861,12 @@ static int ospf_maxage_lsa_remover(struct thread *thread)
 void ospf_lsa_maxage_delete(struct ospf *ospf, struct ospf_lsa *lsa)
 {
 	struct route_node *rn;
-	struct prefix_ptr lsa_prefix;
+	struct prefix lsa_prefix;
 
+	memset(&lsa_prefix, 0, sizeof(struct prefix));
 	lsa_prefix.family = 0;
-	lsa_prefix.prefixlen = sizeof(lsa_prefix.prefix) * CHAR_BIT;
-	lsa_prefix.prefix = (uintptr_t)lsa;
+	lsa_prefix.prefixlen = sizeof(lsa_prefix.u.ptr) * CHAR_BIT;
+	lsa_prefix.u.ptr = (uintptr_t)lsa;
 
 	if ((rn = route_node_lookup(ospf->maxage_lsa,
 				    (struct prefix *)&lsa_prefix))) {
@@ -2861,6 +2878,10 @@ void ospf_lsa_maxage_delete(struct ospf *ospf, struct ospf_lsa *lsa)
 				rn); /* unlock node because lsa is deleted */
 		}
 		route_unlock_node(rn); /* route_node_lookup */
+	} else {
+		if (IS_DEBUG_OSPF_EVENT)
+			zlog_debug("%s: lsa %s is not found in maxage db.",
+				   __PRETTY_FUNCTION__, dump_lsa_key(lsa));
 	}
 }
 
@@ -2871,7 +2892,7 @@ void ospf_lsa_maxage_delete(struct ospf *ospf, struct ospf_lsa *lsa)
  */
 void ospf_lsa_maxage(struct ospf *ospf, struct ospf_lsa *lsa)
 {
-	struct prefix_ptr lsa_prefix;
+	struct prefix lsa_prefix;
 	struct route_node *rn;
 
 	/* When we saw a MaxAge LSA flooded to us, we put it on the list
@@ -2885,9 +2906,10 @@ void ospf_lsa_maxage(struct ospf *ospf, struct ospf_lsa *lsa)
 		return;
 	}
 
+	memset(&lsa_prefix, 0, sizeof(struct prefix));
 	lsa_prefix.family = 0;
-	lsa_prefix.prefixlen = sizeof(lsa_prefix.prefix) * CHAR_BIT;
-	lsa_prefix.prefix = (uintptr_t)lsa;
+	lsa_prefix.prefixlen = sizeof(lsa_prefix.u.ptr) * CHAR_BIT;
+	lsa_prefix.u.ptr = (uintptr_t)lsa;
 
 	if ((rn = route_node_get(ospf->maxage_lsa,
 				 (struct prefix *)&lsa_prefix))
@@ -2904,7 +2926,8 @@ void ospf_lsa_maxage(struct ospf *ospf, struct ospf_lsa *lsa)
 			SET_FLAG(lsa->flags, OSPF_LSA_IN_MAXAGE);
 		}
 	} else {
-		zlog_err("Unable to allocate memory for maxage lsa\n");
+		zlog_err("Unable to allocate memory for maxage lsa %s\n",
+			 dump_lsa_key(lsa));
 		assert(0);
 	}
 
@@ -2982,28 +3005,28 @@ int ospf_lsa_maxage_walker(struct thread *thread)
 	ospf->t_maxage_walker = NULL;
 
 	for (ALL_LIST_ELEMENTS(ospf->areas, node, nnode, area)) {
-		LSDB_LOOP(ROUTER_LSDB(area), rn, lsa)
-		ospf_lsa_maxage_walker_remover(ospf, lsa);
-		LSDB_LOOP(NETWORK_LSDB(area), rn, lsa)
-		ospf_lsa_maxage_walker_remover(ospf, lsa);
-		LSDB_LOOP(SUMMARY_LSDB(area), rn, lsa)
-		ospf_lsa_maxage_walker_remover(ospf, lsa);
-		LSDB_LOOP(ASBR_SUMMARY_LSDB(area), rn, lsa)
-		ospf_lsa_maxage_walker_remover(ospf, lsa);
-		LSDB_LOOP(OPAQUE_AREA_LSDB(area), rn, lsa)
-		ospf_lsa_maxage_walker_remover(ospf, lsa);
-		LSDB_LOOP(OPAQUE_LINK_LSDB(area), rn, lsa)
-		ospf_lsa_maxage_walker_remover(ospf, lsa);
-		LSDB_LOOP(NSSA_LSDB(area), rn, lsa)
-		ospf_lsa_maxage_walker_remover(ospf, lsa);
+		LSDB_LOOP (ROUTER_LSDB(area), rn, lsa)
+			ospf_lsa_maxage_walker_remover(ospf, lsa);
+		LSDB_LOOP (NETWORK_LSDB(area), rn, lsa)
+			ospf_lsa_maxage_walker_remover(ospf, lsa);
+		LSDB_LOOP (SUMMARY_LSDB(area), rn, lsa)
+			ospf_lsa_maxage_walker_remover(ospf, lsa);
+		LSDB_LOOP (ASBR_SUMMARY_LSDB(area), rn, lsa)
+			ospf_lsa_maxage_walker_remover(ospf, lsa);
+		LSDB_LOOP (OPAQUE_AREA_LSDB(area), rn, lsa)
+			ospf_lsa_maxage_walker_remover(ospf, lsa);
+		LSDB_LOOP (OPAQUE_LINK_LSDB(area), rn, lsa)
+			ospf_lsa_maxage_walker_remover(ospf, lsa);
+		LSDB_LOOP (NSSA_LSDB(area), rn, lsa)
+			ospf_lsa_maxage_walker_remover(ospf, lsa);
 	}
 
 	/* for AS-external-LSAs. */
 	if (ospf->lsdb) {
-		LSDB_LOOP(EXTERNAL_LSDB(ospf), rn, lsa)
-		ospf_lsa_maxage_walker_remover(ospf, lsa);
-		LSDB_LOOP(OPAQUE_AS_LSDB(ospf), rn, lsa)
-		ospf_lsa_maxage_walker_remover(ospf, lsa);
+		LSDB_LOOP (EXTERNAL_LSDB(ospf), rn, lsa)
+			ospf_lsa_maxage_walker_remover(ospf, lsa);
+		LSDB_LOOP (OPAQUE_AS_LSDB(ospf), rn, lsa)
+			ospf_lsa_maxage_walker_remover(ospf, lsa);
 	}
 
 	OSPF_TIMER_ON(ospf->t_maxage_walker, ospf_lsa_maxage_walker,
@@ -3011,7 +3034,7 @@ int ospf_lsa_maxage_walker(struct thread *thread)
 	return 0;
 }
 
-struct ospf_lsa *ospf_lsa_lookup_by_prefix(struct ospf_lsdb *lsdb, u_char type,
+struct ospf_lsa *ospf_lsa_lookup_by_prefix(struct ospf_lsdb *lsdb, uint8_t type,
 					   struct prefix_ipv4 *p,
 					   struct in_addr router_id)
 {
@@ -3040,11 +3063,12 @@ struct ospf_lsa *ospf_lsa_lookup_by_prefix(struct ospf_lsdb *lsdb, u_char type,
 	return lsa;
 }
 
-struct ospf_lsa *ospf_lsa_lookup(struct ospf_area *area, u_int32_t type,
-				 struct in_addr id, struct in_addr adv_router)
+struct ospf_lsa *ospf_lsa_lookup(struct ospf *ospf, struct ospf_area *area,
+				 uint32_t type, struct in_addr id,
+				 struct in_addr adv_router)
 {
-	struct ospf *ospf = ospf_lookup();
-	assert(ospf);
+	if (!ospf)
+		return NULL;
 
 	switch (type) {
 	case OSPF_ROUTER_LSA:
@@ -3065,7 +3089,7 @@ struct ospf_lsa *ospf_lsa_lookup(struct ospf_area *area, u_int32_t type,
 	return NULL;
 }
 
-struct ospf_lsa *ospf_lsa_lookup_by_id(struct ospf_area *area, u_int32_t type,
+struct ospf_lsa *ospf_lsa_lookup_by_id(struct ospf_area *area, uint32_t type,
 				       struct in_addr id)
 {
 	struct ospf_lsa *lsa;
@@ -3114,7 +3138,8 @@ struct ospf_lsa *ospf_lsa_lookup_by_header(struct ospf_area *area,
 	 * they two were forming a unique LSA-ID.
 	 */
 
-	match = ospf_lsa_lookup(area, lsah->type, lsah->id, lsah->adv_router);
+	match = ospf_lsa_lookup(area->ospf, area, lsah->type, lsah->id,
+				lsah->adv_router);
 
 	if (match == NULL)
 		if (IS_DEBUG_OSPF(lsa, LSA) == OSPF_DEBUG_LSA)
@@ -3212,7 +3237,7 @@ int ospf_lsa_different(struct ospf_lsa *l1, struct ospf_lsa *l2)
 void ospf_lsa_flush_self_originated(struct ospf_neighbor *nbr,
 				    struct ospf_lsa *self, struct ospf_lsa *new)
 {
-	u_int32_t seqnum;
+	uint32_t seqnum;
 
 	/* Adjust LS Sequence Number. */
 	seqnum = ntohl(new->data->ls_seqnum) + 1;
@@ -3286,6 +3311,8 @@ void ospf_flush_self_originated_lsas_now(struct ospf *ospf)
 	struct route_node *rn;
 	int need_to_flush_ase = 0;
 
+	ospf->inst_shutdown = 1;
+
 	for (ALL_LIST_ELEMENTS(ospf->areas, node, nnode, area)) {
 		if ((lsa = area->router_lsa_self) != NULL) {
 			if (IS_DEBUG_OSPF_EVENT)
@@ -3321,21 +3348,21 @@ void ospf_flush_self_originated_lsas_now(struct ospf *ospf)
 				need_to_flush_ase = 1;
 		}
 
-		LSDB_LOOP(SUMMARY_LSDB(area), rn, lsa)
-		ospf_lsa_flush_schedule(ospf, lsa);
-		LSDB_LOOP(ASBR_SUMMARY_LSDB(area), rn, lsa)
-		ospf_lsa_flush_schedule(ospf, lsa);
-		LSDB_LOOP(OPAQUE_LINK_LSDB(area), rn, lsa)
-		ospf_lsa_flush_schedule(ospf, lsa);
-		LSDB_LOOP(OPAQUE_AREA_LSDB(area), rn, lsa)
-		ospf_lsa_flush_schedule(ospf, lsa);
+		LSDB_LOOP (SUMMARY_LSDB(area), rn, lsa)
+			ospf_lsa_flush_schedule(ospf, lsa);
+		LSDB_LOOP (ASBR_SUMMARY_LSDB(area), rn, lsa)
+			ospf_lsa_flush_schedule(ospf, lsa);
+		LSDB_LOOP (OPAQUE_LINK_LSDB(area), rn, lsa)
+			ospf_lsa_flush_schedule(ospf, lsa);
+		LSDB_LOOP (OPAQUE_AREA_LSDB(area), rn, lsa)
+			ospf_lsa_flush_schedule(ospf, lsa);
 	}
 
 	if (need_to_flush_ase) {
-		LSDB_LOOP(EXTERNAL_LSDB(ospf), rn, lsa)
-		ospf_lsa_flush_schedule(ospf, lsa);
-		LSDB_LOOP(OPAQUE_AS_LSDB(ospf), rn, lsa)
-		ospf_lsa_flush_schedule(ospf, lsa);
+		LSDB_LOOP (EXTERNAL_LSDB(ospf), rn, lsa)
+			ospf_lsa_flush_schedule(ospf, lsa);
+		LSDB_LOOP (OPAQUE_AS_LSDB(ospf), rn, lsa)
+			ospf_lsa_flush_schedule(ospf, lsa);
 	}
 
 	/*
@@ -3395,7 +3422,7 @@ int ospf_lsa_is_self_originated(struct ospf *ospf, struct ospf_lsa *lsa)
 
 /* Get unique Link State ID. */
 struct in_addr ospf_lsa_unique_id(struct ospf *ospf, struct ospf_lsdb *lsdb,
-				  u_char type, struct prefix_ipv4 *p)
+				  uint8_t type, struct prefix_ipv4 *p)
 {
 	struct ospf_lsa *lsa;
 	struct in_addr mask, id;
@@ -3447,7 +3474,7 @@ struct in_addr ospf_lsa_unique_id(struct ospf *ospf, struct ospf_lsdb *lsdb,
 #define LSA_ACTION_FLUSH_AREA 2
 
 struct lsa_action {
-	u_char action;
+	uint8_t action;
 	struct ospf_area *area;
 	struct ospf_lsa *lsa;
 };
@@ -3485,7 +3512,7 @@ void ospf_schedule_lsa_flood_area(struct ospf_area *area, struct ospf_lsa *lsa)
 	data->area = area;
 	data->lsa = ospf_lsa_lock(lsa); /* Message / Flood area */
 
-	thread_add_event(master, ospf_lsa_action, data, 0);
+	thread_add_event(master, ospf_lsa_action, data, 0, NULL);
 }
 
 void ospf_schedule_lsa_flush_area(struct ospf_area *area, struct ospf_lsa *lsa)
@@ -3497,7 +3524,7 @@ void ospf_schedule_lsa_flush_area(struct ospf_area *area, struct ospf_lsa *lsa)
 	data->area = area;
 	data->lsa = ospf_lsa_lock(lsa); /* Message / Flush area */
 
-	thread_add_event(master, ospf_lsa_action, data, 0);
+	thread_add_event(master, ospf_lsa_action, data, 0, NULL);
 }
 
 
@@ -3530,7 +3557,7 @@ struct ospf_lsa *ospf_lsa_refresh(struct ospf *ospf, struct ospf_lsa *lsa)
 		 */
 		if (CHECK_FLAG(lsa->flags, OSPF_LSA_LOCAL_XLT))
 			break;
-		ei = ospf_external_info_check(lsa);
+		ei = ospf_external_info_check(ospf, lsa);
 		if (ei)
 			new = ospf_external_lsa_refresh(ospf, lsa, ei,
 							LSA_REFRESH_FORCE);
@@ -3550,7 +3577,7 @@ struct ospf_lsa *ospf_lsa_refresh(struct ospf *ospf, struct ospf_lsa *lsa)
 
 void ospf_refresher_register_lsa(struct ospf *ospf, struct ospf_lsa *lsa)
 {
-	u_int16_t index, current_index;
+	uint16_t index, current_index;
 
 	assert(lsa->lock > 0);
 	assert(IS_LSA_SELF(lsa));
@@ -3607,7 +3634,7 @@ void ospf_refresher_unregister_lsa(struct ospf *ospf, struct ospf_lsa *lsa)
 			ospf->lsa_refresh_queue.qs[lsa->refresh_list];
 		listnode_delete(refresh_list, lsa);
 		if (!listcount(refresh_list)) {
-			list_free(refresh_list);
+			list_delete_and_null(&refresh_list);
 			ospf->lsa_refresh_queue.qs[lsa->refresh_list] = NULL;
 		}
 		ospf_lsa_unlock(&lsa); /* lsa_refresh_queue */
@@ -3676,13 +3703,13 @@ int ospf_lsa_refresh_walker(struct thread *t)
 				lsa->refresh_list = -1;
 				listnode_add(lsa_to_refresh, lsa);
 			}
-			list_free(refresh_list);
+			list_delete_and_null(&refresh_list);
 		}
 	}
 
-	ospf->t_lsa_refresher =
-		thread_add_timer(master, ospf_lsa_refresh_walker, ospf,
-				 ospf->lsa_refresh_interval);
+	ospf->t_lsa_refresher = NULL;
+	thread_add_timer(master, ospf_lsa_refresh_walker, ospf,
+			 ospf->lsa_refresh_interval, &ospf->t_lsa_refresher);
 	ospf->lsa_refresher_started = monotime(NULL);
 
 	for (ALL_LIST_ELEMENTS(lsa_to_refresh, node, nnode, lsa)) {
@@ -3692,7 +3719,7 @@ int ospf_lsa_refresh_walker(struct thread *t)
 			&lsa); /* lsa_refresh_queue & temp for lsa_to_refresh*/
 	}
 
-	list_delete(lsa_to_refresh);
+	list_delete_and_null(&lsa_to_refresh);
 
 	if (IS_DEBUG_OSPF(lsa, LSA_REFRESH))
 		zlog_debug("LSA[Refresh]: ospf_lsa_refresh_walker(): end");
