@@ -15,10 +15,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -110,7 +109,7 @@ static void nsm_timer_set(struct ospf_neighbor *nbr)
 	case NSM_Down:
 		OSPF_NSM_TIMER_OFF(nbr->t_inactivity);
 		OSPF_NSM_TIMER_OFF(nbr->t_hello_reply);
-		/* fallthru */
+	/* fallthru */
 	case NSM_Attempt:
 	case NSM_Init:
 	case NSM_TwoWay:
@@ -280,38 +279,38 @@ static int nsm_negotiation_done(struct ospf_neighbor *nbr)
 	/* Send proactive ARP requests */
 	ospf_proactively_arp(nbr);
 
-	LSDB_LOOP(ROUTER_LSDB(area), rn, lsa)
-	ospf_db_summary_add(nbr, lsa);
-	LSDB_LOOP(NETWORK_LSDB(area), rn, lsa)
-	ospf_db_summary_add(nbr, lsa);
-	LSDB_LOOP(SUMMARY_LSDB(area), rn, lsa)
-	ospf_db_summary_add(nbr, lsa);
-	LSDB_LOOP(ASBR_SUMMARY_LSDB(area), rn, lsa)
-	ospf_db_summary_add(nbr, lsa);
+	LSDB_LOOP (ROUTER_LSDB(area), rn, lsa)
+		ospf_db_summary_add(nbr, lsa);
+	LSDB_LOOP (NETWORK_LSDB(area), rn, lsa)
+		ospf_db_summary_add(nbr, lsa);
+	LSDB_LOOP (SUMMARY_LSDB(area), rn, lsa)
+		ospf_db_summary_add(nbr, lsa);
+	LSDB_LOOP (ASBR_SUMMARY_LSDB(area), rn, lsa)
+		ospf_db_summary_add(nbr, lsa);
 
 	/* Process only if the neighbor is opaque capable. */
 	if (CHECK_FLAG(nbr->options, OSPF_OPTION_O)) {
-		LSDB_LOOP(OPAQUE_LINK_LSDB(area), rn, lsa)
-		ospf_db_summary_add(nbr, lsa);
-		LSDB_LOOP(OPAQUE_AREA_LSDB(area), rn, lsa)
-		ospf_db_summary_add(nbr, lsa);
+		LSDB_LOOP (OPAQUE_LINK_LSDB(area), rn, lsa)
+			ospf_db_summary_add(nbr, lsa);
+		LSDB_LOOP (OPAQUE_AREA_LSDB(area), rn, lsa)
+			ospf_db_summary_add(nbr, lsa);
 	}
 
 	if (CHECK_FLAG(nbr->options, OSPF_OPTION_NP)) {
-		LSDB_LOOP(NSSA_LSDB(area), rn, lsa)
-		ospf_db_summary_add(nbr, lsa);
+		LSDB_LOOP (NSSA_LSDB(area), rn, lsa)
+			ospf_db_summary_add(nbr, lsa);
 	}
 
 	if (nbr->oi->type != OSPF_IFTYPE_VIRTUALLINK
 	    && area->external_routing == OSPF_AREA_DEFAULT)
-		LSDB_LOOP(EXTERNAL_LSDB(nbr->oi->ospf), rn, lsa)
-	ospf_db_summary_add(nbr, lsa);
+		LSDB_LOOP (EXTERNAL_LSDB(nbr->oi->ospf), rn, lsa)
+			ospf_db_summary_add(nbr, lsa);
 
 	if (CHECK_FLAG(nbr->options, OSPF_OPTION_O)
 	    && (nbr->oi->type != OSPF_IFTYPE_VIRTUALLINK
 		&& area->external_routing == OSPF_AREA_DEFAULT))
-		LSDB_LOOP(OPAQUE_AS_LSDB(nbr->oi->ospf), rn, lsa)
-	ospf_db_summary_add(nbr, lsa);
+		LSDB_LOOP (OPAQUE_AS_LSDB(nbr->oi->ospf), rn, lsa)
+			ospf_db_summary_add(nbr, lsa);
 
 	return 0;
 }
@@ -615,7 +614,7 @@ static void nsm_change_state(struct ospf_neighbor *nbr, int state)
 {
 	struct ospf_interface *oi = nbr->oi;
 	struct ospf_area *vl_area = NULL;
-	u_char old_state;
+	uint8_t old_state;
 	int x;
 	int force = 1;
 
@@ -703,12 +702,13 @@ static void nsm_change_state(struct ospf_neighbor *nbr, int state)
 							oi->ospf);
 		}
 
-		zlog_info(
-			"nsm_change_state(%s, %s -> %s): "
-			"scheduling new router-LSA origination",
-			inet_ntoa(nbr->router_id),
-			lookup_msg(ospf_nsm_state_msg, old_state, NULL),
-			lookup_msg(ospf_nsm_state_msg, state, NULL));
+		if (CHECK_FLAG(oi->ospf->config, OSPF_LOG_ADJACENCY_DETAIL))
+			zlog_info(
+				"%s:(%s, %s -> %s): "
+				"scheduling new router-LSA origination",
+				__PRETTY_FUNCTION__, inet_ntoa(nbr->router_id),
+				lookup_msg(ospf_nsm_state_msg, old_state, NULL),
+				lookup_msg(ospf_nsm_state_msg, state, NULL));
 
 		ospf_router_lsa_update_area(oi->area);
 
