@@ -16,10 +16,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -221,32 +220,32 @@ static oid ospf_trap_oid[] = {OSPF2MIB, 16, 2}; /* Not reverse mappable! */
 static struct in_addr ospf_empty_addr = {.s_addr = 0};
 
 /* Hook functions. */
-static u_char *ospfGeneralGroup(struct variable *, oid *, size_t *, int,
-				size_t *, WriteMethod **);
-static u_char *ospfAreaEntry(struct variable *, oid *, size_t *, int, size_t *,
-			     WriteMethod **);
-static u_char *ospfStubAreaEntry(struct variable *, oid *, size_t *, int,
+static uint8_t *ospfGeneralGroup(struct variable *, oid *, size_t *, int,
 				 size_t *, WriteMethod **);
-static u_char *ospfLsdbEntry(struct variable *, oid *, size_t *, int, size_t *,
-			     WriteMethod **);
-static u_char *ospfAreaRangeEntry(struct variable *, oid *, size_t *, int,
+static uint8_t *ospfAreaEntry(struct variable *, oid *, size_t *, int, size_t *,
+			      WriteMethod **);
+static uint8_t *ospfStubAreaEntry(struct variable *, oid *, size_t *, int,
 				  size_t *, WriteMethod **);
-static u_char *ospfHostEntry(struct variable *, oid *, size_t *, int, size_t *,
-			     WriteMethod **);
-static u_char *ospfIfEntry(struct variable *, oid *, size_t *, int, size_t *,
-			   WriteMethod **);
-static u_char *ospfIfMetricEntry(struct variable *, oid *, size_t *, int,
-				 size_t *, WriteMethod **);
-static u_char *ospfVirtIfEntry(struct variable *, oid *, size_t *, int,
-			       size_t *, WriteMethod **);
-static u_char *ospfNbrEntry(struct variable *, oid *, size_t *, int, size_t *,
+static uint8_t *ospfLsdbEntry(struct variable *, oid *, size_t *, int, size_t *,
+			      WriteMethod **);
+static uint8_t *ospfAreaRangeEntry(struct variable *, oid *, size_t *, int,
+				   size_t *, WriteMethod **);
+static uint8_t *ospfHostEntry(struct variable *, oid *, size_t *, int, size_t *,
+			      WriteMethod **);
+static uint8_t *ospfIfEntry(struct variable *, oid *, size_t *, int, size_t *,
 			    WriteMethod **);
-static u_char *ospfVirtNbrEntry(struct variable *, oid *, size_t *, int,
+static uint8_t *ospfIfMetricEntry(struct variable *, oid *, size_t *, int,
+				  size_t *, WriteMethod **);
+static uint8_t *ospfVirtIfEntry(struct variable *, oid *, size_t *, int,
 				size_t *, WriteMethod **);
-static u_char *ospfExtLsdbEntry(struct variable *, oid *, size_t *, int,
-				size_t *, WriteMethod **);
-static u_char *ospfAreaAggregateEntry(struct variable *, oid *, size_t *, int,
-				      size_t *, WriteMethod **);
+static uint8_t *ospfNbrEntry(struct variable *, oid *, size_t *, int, size_t *,
+			     WriteMethod **);
+static uint8_t *ospfVirtNbrEntry(struct variable *, oid *, size_t *, int,
+				 size_t *, WriteMethod **);
+static uint8_t *ospfExtLsdbEntry(struct variable *, oid *, size_t *, int,
+				 size_t *, WriteMethod **);
+static uint8_t *ospfAreaAggregateEntry(struct variable *, oid *, size_t *, int,
+				       size_t *, WriteMethod **);
 
 static struct variable ospf_variables[] = {
 	/* OSPF general variables */
@@ -527,13 +526,13 @@ static int ospf_admin_stat(struct ospf *ospf)
 	return 0;
 }
 
-static u_char *ospfGeneralGroup(struct variable *v, oid *name, size_t *length,
-				int exact, size_t *var_len,
-				WriteMethod **write_method)
+static uint8_t *ospfGeneralGroup(struct variable *v, oid *name, size_t *length,
+				 int exact, size_t *var_len,
+				 WriteMethod **write_method)
 {
 	struct ospf *ospf;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 	/* Check whether the instance identifier is valid */
 	if (smux_header_generic(v, name, length, exact, var_len, write_method)
@@ -662,7 +661,7 @@ static struct ospf_area *ospfAreaLookup(struct variable *v, oid name[],
 	struct ospf_area *area;
 	int len;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -696,9 +695,9 @@ static struct ospf_area *ospfAreaLookup(struct variable *v, oid name[],
 	return NULL;
 }
 
-static u_char *ospfAreaEntry(struct variable *v, oid *name, size_t *length,
-			     int exact, size_t *var_len,
-			     WriteMethod **write_method)
+static uint8_t *ospfAreaEntry(struct variable *v, oid *name, size_t *length,
+			      int exact, size_t *var_len,
+			      WriteMethod **write_method)
 {
 	struct ospf_area *area;
 	struct in_addr addr;
@@ -740,8 +739,6 @@ static u_char *ospfAreaEntry(struct variable *v, oid *name, size_t *length,
 		return SNMP_INTEGER(0);
 		break;
 	case OSPFAREASUMMARY: /* 9 */
-			      /* $FRR indent$ */
-			      /* clang-format off */
 #define OSPF_noAreaSummary   1
 #define OSPF_sendAreaSummary 2
 		if (area->no_summary)
@@ -766,7 +763,7 @@ static struct ospf_area *ospf_stub_area_lookup_next(struct in_addr *area_id,
 	struct listnode *node;
 	struct ospf *ospf;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -793,7 +790,7 @@ static struct ospf_area *ospfStubAreaLookup(struct variable *v, oid name[],
 	struct ospf_area *area;
 	int len;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -837,9 +834,9 @@ static struct ospf_area *ospfStubAreaLookup(struct variable *v, oid name[],
 	return NULL;
 }
 
-static u_char *ospfStubAreaEntry(struct variable *v, oid *name, size_t *length,
-				 int exact, size_t *var_len,
-				 WriteMethod **write_method)
+static uint8_t *ospfStubAreaEntry(struct variable *v, oid *name, size_t *length,
+				  int exact, size_t *var_len,
+				  WriteMethod **write_method)
 {
 	struct ospf_area *area;
 	struct in_addr addr;
@@ -873,7 +870,7 @@ static u_char *ospfStubAreaEntry(struct variable *v, oid *name, size_t *length,
 		return SNMP_INTEGER(SNMP_VALID);
 		break;
 	case OSPFSTUBMETRICTYPE: /* 5 */
-/* OSPF Metric type. */
+				 /* OSPF Metric type. */
 #define OSPF_ospfMetric     1
 #define OSPF_comparableCost 2
 #define OSPF_nonComparable  3
@@ -886,7 +883,7 @@ static u_char *ospfStubAreaEntry(struct variable *v, oid *name, size_t *length,
 	return NULL;
 }
 
-static struct ospf_lsa *lsdb_lookup_next(struct ospf_area *area, u_char *type,
+static struct ospf_lsa *lsdb_lookup_next(struct ospf_area *area, uint8_t *type,
 					 int type_next, struct in_addr *ls_id,
 					 int ls_id_next,
 					 struct in_addr *router_id,
@@ -922,7 +919,7 @@ static struct ospf_lsa *lsdb_lookup_next(struct ospf_area *area, u_char *type,
 
 static struct ospf_lsa *ospfLsdbLookup(struct variable *v, oid *name,
 				       size_t *length, struct in_addr *area_id,
-				       u_char *type, struct in_addr *ls_id,
+				       uint8_t *type, struct in_addr *ls_id,
 				       struct in_addr *router_id, int exact)
 {
 	struct ospf *ospf;
@@ -935,7 +932,7 @@ static struct ospf_lsa *ospfLsdbLookup(struct variable *v, oid *name,
 	oid *offset;
 	int offsetlen;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 #define OSPF_LSDB_ENTRY_OFFSET (IN_ADDR_SIZE + 1 + IN_ADDR_SIZE + IN_ADDR_SIZE)
 
@@ -1060,14 +1057,14 @@ static struct ospf_lsa *ospfLsdbLookup(struct variable *v, oid *name,
 	return NULL;
 }
 
-static u_char *ospfLsdbEntry(struct variable *v, oid *name, size_t *length,
-			     int exact, size_t *var_len,
-			     WriteMethod **write_method)
+static uint8_t *ospfLsdbEntry(struct variable *v, oid *name, size_t *length,
+			      int exact, size_t *var_len,
+			      WriteMethod **write_method)
 {
 	struct ospf_lsa *lsa;
 	struct lsa_header *lsah;
 	struct in_addr area_id;
-	u_char type;
+	uint8_t type;
 	struct in_addr ls_id;
 	struct in_addr router_id;
 	struct ospf *ospf;
@@ -1085,7 +1082,7 @@ static u_char *ospfLsdbEntry(struct variable *v, oid *name, size_t *length,
 	memset(&router_id, 0, sizeof(struct in_addr));
 
 	/* Check OSPF instance. */
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -1121,7 +1118,7 @@ static u_char *ospfLsdbEntry(struct variable *v, oid *name, size_t *length,
 		break;
 	case OSPFLSDBADVERTISEMENT: /* 8 */
 		*var_len = ntohs(lsah->length);
-		return (u_char *)lsah;
+		return (uint8_t *)lsah;
 		break;
 	default:
 		return NULL;
@@ -1146,7 +1143,7 @@ static struct ospf_area_range *ospfAreaRangeLookup(struct variable *v,
 	p.family = AF_INET;
 	p.prefixlen = IPV4_MAX_BITLEN;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 	if (exact) {
 		/* Area ID + Range Network. */
@@ -1225,9 +1222,9 @@ static struct ospf_area_range *ospfAreaRangeLookup(struct variable *v,
 	return NULL;
 }
 
-static u_char *ospfAreaRangeEntry(struct variable *v, oid *name, size_t *length,
-				  int exact, size_t *var_len,
-				  WriteMethod **write_method)
+static uint8_t *ospfAreaRangeEntry(struct variable *v, oid *name,
+				   size_t *length, int exact, size_t *var_len,
+				   WriteMethod **write_method)
 {
 	struct ospf_area_range *range;
 	struct in_addr area_id;
@@ -1240,7 +1237,7 @@ static u_char *ospfAreaRangeEntry(struct variable *v, oid *name, size_t *length,
 		return NULL;
 
 	/* Check OSPF instance. */
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -1270,8 +1267,6 @@ static u_char *ospfAreaRangeEntry(struct variable *v, oid *name, size_t *length,
 		return SNMP_INTEGER(SNMP_VALID);
 		break;
 	case OSPFAREARANGEEFFECT: /* 5 */
-				  /* $FRR indent$ */
-				  /* clang-format off */
 #define OSPF_advertiseMatching      1
 #define OSPF_doNotAdvertiseMatching 2
 		return SNMP_INTEGER(OSPF_advertiseMatching);
@@ -1291,7 +1286,7 @@ static struct ospf_nbr_nbma *ospfHostLookup(struct variable *v, oid *name,
 	struct ospf_nbr_nbma *nbr_nbma;
 	struct ospf *ospf;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -1334,9 +1329,9 @@ static struct ospf_nbr_nbma *ospfHostLookup(struct variable *v, oid *name,
 	return NULL;
 }
 
-static u_char *ospfHostEntry(struct variable *v, oid *name, size_t *length,
-			     int exact, size_t *var_len,
-			     WriteMethod **write_method)
+static uint8_t *ospfHostEntry(struct variable *v, oid *name, size_t *length,
+			      int exact, size_t *var_len,
+			      WriteMethod **write_method)
 {
 	struct ospf_nbr_nbma *nbr_nbma;
 	struct ospf_interface *oi;
@@ -1348,7 +1343,7 @@ static u_char *ospfHostEntry(struct variable *v, oid *name, size_t *length,
 		return NULL;
 
 	/* Check OSPF instance. */
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -1505,7 +1500,7 @@ static struct ospf_interface *ospf_snmp_if_lookup(struct in_addr *ifaddr,
 	struct listnode *node;
 	struct ospf_snmp_if *osif;
 	struct ospf_interface *oi = NULL;
-	struct ospf *ospf = ospf_lookup();
+	struct ospf *ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 	for (ALL_LIST_ELEMENTS_RO(ospf_snmp_iflist, node, osif)) {
 		if (ifaddr->s_addr) {
@@ -1528,7 +1523,7 @@ static struct ospf_interface *ospf_snmp_if_lookup_next(struct in_addr *ifaddr,
 {
 	struct ospf_snmp_if *osif;
 	struct listnode *nn;
-	struct ospf *ospf = ospf_lookup();
+	struct ospf *ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	struct ospf_interface *oi = NULL;
 
 	if (ospf == NULL)
@@ -1659,9 +1654,9 @@ static struct ospf_interface *ospfIfLookup(struct variable *v, oid *name,
 	return NULL;
 }
 
-static u_char *ospfIfEntry(struct variable *v, oid *name, size_t *length,
-			   int exact, size_t *var_len,
-			   WriteMethod **write_method)
+static uint8_t *ospfIfEntry(struct variable *v, oid *name, size_t *length,
+			    int exact, size_t *var_len,
+			    WriteMethod **write_method)
 {
 	ifindex_t ifindex;
 	struct in_addr ifaddr;
@@ -1676,7 +1671,7 @@ static u_char *ospfIfEntry(struct variable *v, oid *name, size_t *length,
 	memset(&ifaddr, 0, sizeof(struct in_addr));
 
 	/* Check OSPF instance. */
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -1739,14 +1734,12 @@ static u_char *ospfIfEntry(struct variable *v, oid *name, size_t *length,
 		break;
 	case OSPFIFAUTHKEY: /* 16 */
 		*var_len = 0;
-		return (u_char *)OSPF_IF_PARAM(oi, auth_simple);
+		return (uint8_t *)OSPF_IF_PARAM(oi, auth_simple);
 		break;
 	case OSPFIFSTATUS: /* 17 */
 		return SNMP_INTEGER(SNMP_VALID);
 		break;
 	case OSPFIFMULTICASTFORWARDING: /* 18 */
-					/* $FRR indent$ */
-					/* clang-format off */
 #define ospf_snmp_multiforward_blocked    1
 #define ospf_snmp_multiforward_multicast  2
 #define ospf_snmp_multiforward_unicast    3
@@ -1828,9 +1821,9 @@ static struct ospf_interface *ospfIfMetricLookup(struct variable *v, oid *name,
 	return NULL;
 }
 
-static u_char *ospfIfMetricEntry(struct variable *v, oid *name, size_t *length,
-				 int exact, size_t *var_len,
-				 WriteMethod **write_method)
+static uint8_t *ospfIfMetricEntry(struct variable *v, oid *name, size_t *length,
+				  int exact, size_t *var_len,
+				  WriteMethod **write_method)
 {
 	/* Currently we support metric 1 only. */
 	ifindex_t ifindex;
@@ -1846,7 +1839,7 @@ static u_char *ospfIfMetricEntry(struct variable *v, oid *name, size_t *length,
 	memset(&ifaddr, 0, sizeof(struct in_addr));
 
 	/* Check OSPF instance. */
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -2021,9 +2014,9 @@ ospfVirtIfLookup(struct variable *v, oid *name, size_t *length,
 	return NULL;
 }
 
-static u_char *ospfVirtIfEntry(struct variable *v, oid *name, size_t *length,
-			       int exact, size_t *var_len,
-			       WriteMethod **write_method)
+static uint8_t *ospfVirtIfEntry(struct variable *v, oid *name, size_t *length,
+				int exact, size_t *var_len,
+				WriteMethod **write_method)
 {
 	struct ospf_vl_data *vl_data;
 	struct ospf_interface *oi;
@@ -2072,7 +2065,7 @@ static u_char *ospfVirtIfEntry(struct variable *v, oid *name, size_t *length,
 		break;
 	case OSPFVIRTIFAUTHKEY:
 		*var_len = 0;
-		return (u_char *)OSPF_IF_PARAM(oi, auth_simple);
+		return (uint8_t *)OSPF_IF_PARAM(oi, auth_simple);
 		break;
 	case OSPFVIRTIFSTATUS:
 		return SNMP_INTEGER(SNMP_VALID);
@@ -2124,9 +2117,9 @@ static struct ospf_neighbor *ospf_snmp_nbr_lookup_next(struct in_addr *nbr_addr,
 	struct ospf_neighbor *nbr;
 	struct route_node *rn;
 	struct ospf_neighbor *min = NULL;
-	struct ospf *ospf = ospf;
+	struct ospf *ospf;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 	for (ALL_LIST_ELEMENTS_RO(ospf->oiflist, nn, oi)) {
 		for (rn = route_top(oi->nbrs); rn; rn = route_next(rn))
@@ -2166,7 +2159,7 @@ static struct ospf_neighbor *ospfNbrLookup(struct variable *v, oid *name,
 	struct ospf_neighbor *nbr;
 	struct ospf *ospf;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
 	if (!ospf)
 		return NULL;
@@ -2222,7 +2215,7 @@ ospfNbrState OBJECT-TYPE
 		    full (8)
 		  }
 */
-static int32_t ospf_snmp_neighbor_state(u_char nst)
+static int32_t ospf_snmp_neighbor_state(uint8_t nst)
 {
 	switch (nst) {
 	case NSM_Attempt:
@@ -2244,9 +2237,9 @@ static int32_t ospf_snmp_neighbor_state(u_char nst)
 	}
 }
 
-static u_char *ospfNbrEntry(struct variable *v, oid *name, size_t *length,
-			    int exact, size_t *var_len,
-			    WriteMethod **write_method)
+static uint8_t *ospfNbrEntry(struct variable *v, oid *name, size_t *length,
+			     int exact, size_t *var_len,
+			     WriteMethod **write_method)
 {
 	struct in_addr nbr_addr;
 	ifindex_t ifindex;
@@ -2309,9 +2302,9 @@ static u_char *ospfNbrEntry(struct variable *v, oid *name, size_t *length,
 	return NULL;
 }
 
-static u_char *ospfVirtNbrEntry(struct variable *v, oid *name, size_t *length,
-				int exact, size_t *var_len,
-				WriteMethod **write_method)
+static uint8_t *ospfVirtNbrEntry(struct variable *v, oid *name, size_t *length,
+				 int exact, size_t *var_len,
+				 WriteMethod **write_method)
 {
 	struct ospf_vl_data *vl_data;
 	struct in_addr area_id;
@@ -2326,7 +2319,7 @@ static u_char *ospfVirtNbrEntry(struct variable *v, oid *name, size_t *length,
 	memset(&neighbor, 0, sizeof(struct in_addr));
 
 	/* Check OSPF instance. */
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -2337,28 +2330,28 @@ static u_char *ospfVirtNbrEntry(struct variable *v, oid *name, size_t *length,
 	/* Return the current value of the variable */
 	switch (v->magic) {
 	case OSPFVIRTNBRAREA:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFVIRTNBRRTRID:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFVIRTNBRIPADDR:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFVIRTNBROPTIONS:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFVIRTNBRSTATE:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFVIRTNBREVENTS:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFVIRTNBRLSRETRANSQLEN:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFVIRTNBRHELLOSUPPRESSED:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	default:
 		return NULL;
@@ -2368,19 +2361,19 @@ static u_char *ospfVirtNbrEntry(struct variable *v, oid *name, size_t *length,
 }
 
 static struct ospf_lsa *ospfExtLsdbLookup(struct variable *v, oid *name,
-					  size_t *length, u_char *type,
+					  size_t *length, uint8_t *type,
 					  struct in_addr *ls_id,
 					  struct in_addr *router_id, int exact)
 {
 	int first;
 	oid *offset;
 	int offsetlen;
-	u_char lsa_type;
+	uint8_t lsa_type;
 	unsigned int len;
 	struct ospf_lsa *lsa;
 	struct ospf *ospf;
 
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (exact) {
 		if (*length != v->namelen + 1 + IN_ADDR_SIZE + IN_ADDR_SIZE)
 			return NULL;
@@ -2457,13 +2450,13 @@ static struct ospf_lsa *ospfExtLsdbLookup(struct variable *v, oid *name,
 	return NULL;
 }
 
-static u_char *ospfExtLsdbEntry(struct variable *v, oid *name, size_t *length,
-				int exact, size_t *var_len,
-				WriteMethod **write_method)
+static uint8_t *ospfExtLsdbEntry(struct variable *v, oid *name, size_t *length,
+				 int exact, size_t *var_len,
+				 WriteMethod **write_method)
 {
 	struct ospf_lsa *lsa;
 	struct lsa_header *lsah;
-	u_char type;
+	uint8_t type;
 	struct in_addr ls_id;
 	struct in_addr router_id;
 	struct ospf *ospf;
@@ -2477,7 +2470,7 @@ static u_char *ospfExtLsdbEntry(struct variable *v, oid *name, size_t *length,
 	memset(&router_id, 0, sizeof(struct in_addr));
 
 	/* Check OSPF instance. */
-	ospf = ospf_lookup();
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	if (ospf == NULL)
 		return NULL;
 
@@ -2510,7 +2503,7 @@ static u_char *ospfExtLsdbEntry(struct variable *v, oid *name, size_t *length,
 		break;
 	case OSPFEXTLSDBADVERTISEMENT:
 		*var_len = ntohs(lsah->length);
-		return (u_char *)lsah;
+		return (uint8_t *)lsah;
 		break;
 	default:
 		return NULL;
@@ -2519,10 +2512,10 @@ static u_char *ospfExtLsdbEntry(struct variable *v, oid *name, size_t *length,
 	return NULL;
 }
 
-static u_char *ospfAreaAggregateEntry(struct variable *v, oid *name,
-				      size_t *length, int exact,
-				      size_t *var_len,
-				      WriteMethod **write_method)
+static uint8_t *ospfAreaAggregateEntry(struct variable *v, oid *name,
+				       size_t *length, int exact,
+				       size_t *var_len,
+				       WriteMethod **write_method)
 {
 	if (smux_header_table(v, name, length, exact, var_len, write_method)
 	    == MATCH_FAILED)
@@ -2531,22 +2524,22 @@ static u_char *ospfAreaAggregateEntry(struct variable *v, oid *name,
 	/* Return the current value of the variable */
 	switch (v->magic) {
 	case OSPFAREAAGGREGATEAREAID:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFAREAAGGREGATELSDBTYPE:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFAREAAGGREGATENET:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFAREAAGGREGATEMASK:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFAREAAGGREGATESTATUS:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	case OSPFAREAAGGREGATEEFFECT:
-		return (u_char *)NULL;
+		return (uint8_t *)NULL;
 		break;
 	default:
 		return NULL;
@@ -2590,8 +2583,9 @@ static void ospfTrapNbrStateChange(struct ospf_neighbor *on)
 	char msgbuf[16];
 
 	ospf_nbr_state_message(on, msgbuf, sizeof(msgbuf));
-	zlog_info("ospfTrapNbrStateChange trap sent: %s now %s",
-		  inet_ntoa(on->address.u.prefix4), msgbuf);
+	if (IS_DEBUG_OSPF_EVENT)
+		zlog_info("%s: trap sent: %s now %s", __PRETTY_FUNCTION__,
+			  inet_ntoa(on->address.u.prefix4), msgbuf);
 
 	oid_copy_addr(index, &(on->address.u.prefix4), IN_ADDR_SIZE);
 	index[IN_ADDR_SIZE] = 0;
@@ -2647,9 +2641,10 @@ static void ospfTrapIfStateChange(struct ospf_interface *oi)
 {
 	oid index[sizeof(oid) * (IN_ADDR_SIZE + 1)];
 
-	zlog_info("ospfTrapIfStateChange trap sent: %s now %s",
-		  inet_ntoa(oi->address->u.prefix4),
-		  lookup_msg(ospf_ism_state_msg, oi->state, NULL));
+	if (IS_DEBUG_OSPF_EVENT)
+		zlog_info("%s: trap sent: %s now %s", __PRETTY_FUNCTION__,
+			  inet_ntoa(oi->address->u.prefix4),
+			  lookup_msg(ospf_ism_state_msg, oi->state, NULL));
 
 	oid_copy_addr(index, &(oi->address->u.prefix4), IN_ADDR_SIZE);
 	index[IN_ADDR_SIZE] = 0;
