@@ -1,21 +1,20 @@
 /*
-  PIM for Quagga
-  Copyright (C) 2008  Everton da Silva Marques
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING; if not, write to the
-  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-  MA 02110-1301 USA
-*/
+ * PIM for Quagga
+ * Copyright (C) 2008  Everton da Silva Marques
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -28,8 +27,10 @@
 #include "linklist.h"
 #include "plist.h"
 #include "hash.h"
+#include "ferr.h"
 
 #include "pimd.h"
+#include "pim_instance.h"
 #include "pim_zebra.h"
 #include "pim_iface.h"
 #include "pim_igmp.h"
@@ -44,65 +45,91 @@
 #include "pim_ssmpingd.h"
 #include "pim_rp.h"
 #include "pim_nht.h"
-
-struct interface *pim_regiface = NULL;
-struct list *pim_ifchannel_list = NULL;
-static int pim_iface_vif_index[MAXVIFS];
+#include "pim_jp_agg.h"
 
 static void pim_if_igmp_join_del_all(struct interface *ifp);
 static int igmp_join_sock(const char *ifname, ifindex_t ifindex,
 			  struct in_addr group_addr,
 			  struct in_addr source_addr);
 
-void pim_if_init(void)
+void pim_if_init(struct pim_instance *pim)
 {
 	int i;
 
 	for (i = 0; i < MAXVIFS; i++)
-		pim_iface_vif_index[i] = 0;
-
-	vrf_iflist_create(VRF_DEFAULT);
-	pim_ifchannel_list = list_new();
-	pim_ifchannel_list->cmp =
-		(int (*)(void *, void *))pim_ifchannel_compare;
+		pim->iface_vif_index[i] = 0;
 }
 
-void pim_if_terminate(void)
+void pim_if_terminate(struct pim_instance *pim)
 {
-	if (pim_ifchannel_list)
-		list_free(pim_ifchannel_list);
+	// Nothing to do at this moment
+	return;
 }
 
 static void *if_list_clean(struct pim_interface *pim_ifp)
 {
-	if (pim_ifp->igmp_join_list) {
-		list_delete(pim_ifp->igmp_join_list);
-	}
+	struct pim_ifchannel *ch;
 
-	if (pim_ifp->igmp_socket_list) {
-		list_delete(pim_ifp->igmp_socket_list);
-	}
+	if (pim_ifp->igmp_join_list)
+		list_delete_and_null(&pim_ifp->igmp_join_list);
 
-	if (pim_ifp->pim_neighbor_list) {
-		list_delete(pim_ifp->pim_neighbor_list);
-	}
+	if (pim_ifp->igmp_socket_list)
+		list_delete_and_null(&pim_ifp->igmp_socket_list);
+
+	if (pim_ifp->pim_neighbor_list)
+		list_delete_and_null(&pim_ifp->pim_neighbor_list);
 
 	if (pim_ifp->upstream_switch_list)
-		list_delete(pim_ifp->upstream_switch_list);
+		list_delete_and_null(&pim_ifp->upstream_switch_list);
 
-	if (pim_ifp->pim_ifchannel_list) {
-		list_delete(pim_ifp->pim_ifchannel_list);
+	if (pim_ifp->sec_addr_list)
+		list_delete_and_null(&pim_ifp->sec_addr_list);
+
+	while (!RB_EMPTY(pim_ifchannel_rb, &pim_ifp->ifchannel_rb)) {
+		ch = RB_ROOT(pim_ifchannel_rb, &pim_ifp->ifchannel_rb);
+
+		pim_ifchannel_delete(ch);
 	}
 
-	if (pim_ifp->pim_ifchannel_hash)
-		hash_free(pim_ifp->pim_ifchannel_hash);
-
 	XFREE(MTYPE_PIM_INTERFACE, pim_ifp);
 
 	return 0;
 }
 
-struct pim_interface *pim_if_new(struct interface *ifp, int igmp, int pim)
+static void pim_sec_addr_free(struct pim_secondary_addr *sec_addr)
+{
+	XFREE(MTYPE_PIM_SEC_ADDR, sec_addr);
+}
+
+static int pim_sec_addr_comp(const void *p1, const void *p2)
+{
+	const struct pim_secondary_addr *sec1 = p1;
+	const struct pim_secondary_addr *sec2 = p2;
+
+	if (sec1->addr.family == AF_INET && sec2->addr.family == AF_INET6)
+		return -1;
+
+	if (sec1->addr.family == AF_INET6 && sec2->addr.family == AF_INET)
+		return 1;
+
+	if (sec1->addr.family == AF_INET) {
+		if (ntohl(sec1->addr.u.prefix4.s_addr)
+		    < ntohl(sec2->addr.u.prefix4.s_addr))
+			return -1;
+
+		if (ntohl(sec1->addr.u.prefix4.s_addr)
+		    > ntohl(sec2->addr.u.prefix4.s_addr))
+			return 1;
+	} else {
+		return memcmp(&sec1->addr.u.prefix6, &sec2->addr.u.prefix6,
+			      sizeof(struct in6_addr));
+	}
+
+	return 0;
+}
+
+struct pim_interface *pim_if_new(struct interface *ifp, int igmp, int pim,
+				 bool ispimreg)
 {
 	struct pim_interface *pim_ifp;
 
@@ -116,6 +143,7 @@ struct pim_interface *pim_if_new(struct interface *ifp, int igmp, int pim)
 	}
 
 	pim_ifp->options = 0;
+	pim_ifp->pim = pim_get_pim_instance(ifp->vrf_id);
 	pim_ifp->mroute_vif_index = -1;
 
 	pim_ifp->igmp_version = IGMP_DEFAULT_VERSION;
@@ -146,15 +174,13 @@ struct pim_interface *pim_if_new(struct interface *ifp, int igmp, int pim)
 	pim_ifp->igmp_socket_list = NULL;
 	pim_ifp->pim_neighbor_list = NULL;
 	pim_ifp->upstream_switch_list = NULL;
-	pim_ifp->pim_ifchannel_list = NULL;
-	pim_ifp->pim_ifchannel_hash = NULL;
 	pim_ifp->pim_generation_id = 0;
 
 	/* list of struct igmp_sock */
 	pim_ifp->igmp_socket_list = list_new();
 	if (!pim_ifp->igmp_socket_list) {
-		zlog_err("%s %s: failure: igmp_socket_list=list_new()",
-			 __FILE__, __PRETTY_FUNCTION__);
+		zlog_err("%s: failure: igmp_socket_list=list_new()",
+			 __PRETTY_FUNCTION__);
 		return if_list_clean(pim_ifp);
 	}
 	pim_ifp->igmp_socket_list->del = (void (*)(void *))igmp_sock_free;
@@ -162,38 +188,39 @@ struct pim_interface *pim_if_new(struct interface *ifp, int igmp, int pim)
 	/* list of struct pim_neighbor */
 	pim_ifp->pim_neighbor_list = list_new();
 	if (!pim_ifp->pim_neighbor_list) {
-		zlog_err("%s %s: failure: pim_neighbor_list=list_new()",
-			 __FILE__, __PRETTY_FUNCTION__);
+		zlog_err("%s: failure: pim_neighbor_list=list_new()",
+			 __PRETTY_FUNCTION__);
 		return if_list_clean(pim_ifp);
 	}
 	pim_ifp->pim_neighbor_list->del = (void (*)(void *))pim_neighbor_free;
 
 	pim_ifp->upstream_switch_list = list_new();
 	if (!pim_ifp->upstream_switch_list) {
-		zlog_err("%s %s: failure: upstream_switch_list=list_new()",
-			 __FILE__, __PRETTY_FUNCTION__);
+		zlog_err("%s: failure: upstream_switch_list=list_new()",
+			 __PRETTY_FUNCTION__);
 		return if_list_clean(pim_ifp);
 	}
+	pim_ifp->upstream_switch_list->del =
+		(void (*)(void *))pim_jp_agg_group_list_free;
+	pim_ifp->upstream_switch_list->cmp = pim_jp_agg_group_list_cmp;
 
-	/* list of struct pim_ifchannel */
-	pim_ifp->pim_ifchannel_list = list_new();
-	if (!pim_ifp->pim_ifchannel_list) {
-		zlog_err("%s %s: failure: pim_ifchannel_list=list_new()",
-			 __FILE__, __PRETTY_FUNCTION__);
+	pim_ifp->sec_addr_list = list_new();
+	if (!pim_ifp->sec_addr_list) {
+		zlog_err("%s: failure: secondary addresslist",
+			 __PRETTY_FUNCTION__);
 		return if_list_clean(pim_ifp);
 	}
-	pim_ifp->pim_ifchannel_list->del = (void (*)(void *))pim_ifchannel_free;
-	pim_ifp->pim_ifchannel_list->cmp =
-		(int (*)(void *, void *))pim_ifchannel_compare;
+	pim_ifp->sec_addr_list->del = (void (*)(void *))pim_sec_addr_free;
+	pim_ifp->sec_addr_list->cmp =
+		(int (*)(void *, void *))pim_sec_addr_comp;
 
-	pim_ifp->pim_ifchannel_hash =
-		hash_create(pim_ifchannel_hash_key, pim_ifchannel_equal);
+	RB_INIT(pim_ifchannel_rb, &pim_ifp->ifchannel_rb);
 
 	ifp->info = pim_ifp;
 
 	pim_sock_reset(ifp);
 
-	pim_if_add_vif(ifp);
+	pim_if_add_vif(ifp, ispimreg);
 
 	return pim_ifp;
 }
@@ -201,6 +228,7 @@ struct pim_interface *pim_if_new(struct interface *ifp, int igmp, int pim)
 void pim_if_delete(struct interface *ifp)
 {
 	struct pim_interface *pim_ifp;
+	struct pim_ifchannel *ch;
 
 	zassert(ifp);
 	pim_ifp = ifp->info;
@@ -217,12 +245,19 @@ void pim_if_delete(struct interface *ifp)
 
 	pim_if_del_vif(ifp);
 
-	list_delete(pim_ifp->igmp_socket_list);
-	list_delete(pim_ifp->pim_neighbor_list);
-	list_delete(pim_ifp->upstream_switch_list);
-	list_delete(pim_ifp->pim_ifchannel_list);
+	list_delete_and_null(&pim_ifp->igmp_socket_list);
+	list_delete_and_null(&pim_ifp->pim_neighbor_list);
+	list_delete_and_null(&pim_ifp->upstream_switch_list);
+	list_delete_and_null(&pim_ifp->sec_addr_list);
 
-	hash_free(pim_ifp->pim_ifchannel_hash);
+	if (pim_ifp->boundary_oil_plist)
+		XFREE(MTYPE_PIM_INTERFACE, pim_ifp->boundary_oil_plist);
+
+	while (!RB_EMPTY(pim_ifchannel_rb, &pim_ifp->ifchannel_rb)) {
+		ch = RB_ROOT(pim_ifchannel_rb, &pim_ifp->ifchannel_rb);
+
+		pim_ifchannel_delete(ch);
+	}
 
 	XFREE(MTYPE_PIM_INTERFACE, pim_ifp);
 
@@ -232,15 +267,12 @@ void pim_if_delete(struct interface *ifp)
 void pim_if_update_could_assert(struct interface *ifp)
 {
 	struct pim_interface *pim_ifp;
-	struct listnode *node;
-	struct listnode *next_node;
 	struct pim_ifchannel *ch;
 
 	pim_ifp = ifp->info;
 	zassert(pim_ifp);
 
-	for (ALL_LIST_ELEMENTS(pim_ifp->pim_ifchannel_list, node, next_node,
-			       ch)) {
+	RB_FOREACH (ch, pim_ifchannel_rb, &pim_ifp->ifchannel_rb) {
 		pim_ifchannel_update_could_assert(ch);
 	}
 }
@@ -248,15 +280,12 @@ void pim_if_update_could_assert(struct interface *ifp)
 static void pim_if_update_my_assert_metric(struct interface *ifp)
 {
 	struct pim_interface *pim_ifp;
-	struct listnode *node;
-	struct listnode *next_node;
 	struct pim_ifchannel *ch;
 
 	pim_ifp = ifp->info;
 	zassert(pim_ifp);
 
-	for (ALL_LIST_ELEMENTS(pim_ifp->pim_ifchannel_list, node, next_node,
-			       ch)) {
+	RB_FOREACH (ch, pim_ifchannel_rb, &pim_ifp->ifchannel_rb) {
 		pim_ifchannel_update_my_assert_metric(ch);
 	}
 }
@@ -308,7 +337,7 @@ static int detect_primary_address_change(struct interface *ifp,
 	int changed;
 
 	if (force_prim_as_any)
-		new_prim_addr = qpim_inaddr_any;
+		new_prim_addr.s_addr = INADDR_ANY;
 	else
 		new_prim_addr = pim_find_primary_addr(ifp);
 
@@ -333,37 +362,14 @@ static int detect_primary_address_change(struct interface *ifp,
 	return changed;
 }
 
-static int pim_sec_addr_comp(const void *p1, const void *p2)
-{
-	const struct pim_secondary_addr *sec1 = p1;
-	const struct pim_secondary_addr *sec2 = p2;
-
-	if (ntohl(sec1->addr.s_addr) < ntohl(sec2->addr.s_addr))
-		return -1;
-
-	if (ntohl(sec1->addr.s_addr) > ntohl(sec2->addr.s_addr))
-		return 1;
-
-	return 0;
-}
-
-static void pim_sec_addr_free(struct pim_secondary_addr *sec_addr)
-{
-	XFREE(MTYPE_PIM_SEC_ADDR, sec_addr);
-}
-
 static struct pim_secondary_addr *
-pim_sec_addr_find(struct pim_interface *pim_ifp, struct in_addr addr)
+pim_sec_addr_find(struct pim_interface *pim_ifp, struct prefix *addr)
 {
 	struct pim_secondary_addr *sec_addr;
 	struct listnode *node;
 
-	if (!pim_ifp->sec_addr_list) {
-		return NULL;
-	}
-
 	for (ALL_LIST_ELEMENTS_RO(pim_ifp->sec_addr_list, node, sec_addr)) {
-		if (sec_addr->addr.s_addr == addr.s_addr) {
+		if (prefix_cmp(&sec_addr->addr, addr)) {
 			return sec_addr;
 		}
 	}
@@ -378,7 +384,7 @@ static void pim_sec_addr_del(struct pim_interface *pim_ifp,
 	pim_sec_addr_free(sec_addr);
 }
 
-static int pim_sec_addr_add(struct pim_interface *pim_ifp, struct in_addr addr)
+static int pim_sec_addr_add(struct pim_interface *pim_ifp, struct prefix *addr)
 {
 	int changed = 0;
 	struct pim_secondary_addr *sec_addr;
@@ -389,25 +395,12 @@ static int pim_sec_addr_add(struct pim_interface *pim_ifp, struct in_addr addr)
 		return changed;
 	}
 
-	if (!pim_ifp->sec_addr_list) {
-		pim_ifp->sec_addr_list = list_new();
-		pim_ifp->sec_addr_list->del =
-			(void (*)(void *))pim_sec_addr_free;
-		pim_ifp->sec_addr_list->cmp =
-			(int (*)(void *, void *))pim_sec_addr_comp;
-	}
-
 	sec_addr = XCALLOC(MTYPE_PIM_SEC_ADDR, sizeof(*sec_addr));
-	if (!sec_addr) {
-		if (list_isempty(pim_ifp->sec_addr_list)) {
-			list_free(pim_ifp->sec_addr_list);
-			pim_ifp->sec_addr_list = NULL;
-		}
+	if (!sec_addr)
 		return changed;
-	}
 
 	changed = 1;
-	sec_addr->addr = addr;
+	sec_addr->addr = *addr;
 	listnode_add_sort(pim_ifp->sec_addr_list, sec_addr);
 
 	return changed;
@@ -417,15 +410,10 @@ static int pim_sec_addr_del_all(struct pim_interface *pim_ifp)
 {
 	int changed = 0;
 
-	if (!pim_ifp->sec_addr_list) {
-		return changed;
-	}
 	if (!list_isempty(pim_ifp->sec_addr_list)) {
 		changed = 1;
 		/* remove all nodes and free up the list itself */
 		list_delete_all_node(pim_ifp->sec_addr_list);
-		list_free(pim_ifp->sec_addr_list);
-		pim_ifp->sec_addr_list = NULL;
 	}
 
 	return changed;
@@ -440,20 +428,13 @@ static int pim_sec_addr_update(struct interface *ifp)
 	struct pim_secondary_addr *sec_addr;
 	int changed = 0;
 
-	if (pim_ifp->sec_addr_list) {
-		for (ALL_LIST_ELEMENTS_RO(pim_ifp->sec_addr_list, node,
-					  sec_addr)) {
-			sec_addr->flags |= PIM_SEC_ADDRF_STALE;
-		}
+	for (ALL_LIST_ELEMENTS_RO(pim_ifp->sec_addr_list, node, sec_addr)) {
+		sec_addr->flags |= PIM_SEC_ADDRF_STALE;
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(ifp->connected, node, ifc)) {
 		struct prefix *p = ifc->address;
 
-		if (p->family != AF_INET) {
-			continue;
-		}
-
 		if (PIM_INADDR_IS_ANY(p->u.prefix4)) {
 			continue;
 		}
@@ -464,25 +445,17 @@ static int pim_sec_addr_update(struct interface *ifp)
 			continue;
 		}
 
-		if (pim_sec_addr_add(pim_ifp, p->u.prefix4)) {
+		if (pim_sec_addr_add(pim_ifp, p)) {
 			changed = 1;
 		}
 	}
 
-	if (pim_ifp->sec_addr_list) {
-		/* Drop stale entries */
-		for (ALL_LIST_ELEMENTS(pim_ifp->sec_addr_list, node, nextnode,
-				       sec_addr)) {
-			if (sec_addr->flags & PIM_SEC_ADDRF_STALE) {
-				pim_sec_addr_del(pim_ifp, sec_addr);
-				changed = 1;
-			}
-		}
-
-		/* If the list went empty free it up */
-		if (list_isempty(pim_ifp->sec_addr_list)) {
-			list_free(pim_ifp->sec_addr_list);
-			pim_ifp->sec_addr_list = NULL;
+	/* Drop stale entries */
+	for (ALL_LIST_ELEMENTS(pim_ifp->sec_addr_list, node, nextnode,
+			       sec_addr)) {
+		if (sec_addr->flags & PIM_SEC_ADDRF_STALE) {
+			pim_sec_addr_del(pim_ifp, sec_addr);
+			changed = 1;
 		}
 	}
 
@@ -589,6 +562,9 @@ void pim_if_addr_add(struct connected *ifc)
 
 	detect_address_change(ifp, 0, __PRETTY_FUNCTION__);
 
+	// if (ifc->address->family != AF_INET)
+	//  return;
+
 	if (PIM_IF_TEST_IGMP(pim_ifp->options)) {
 		struct igmp_sock *igmp;
 
@@ -597,8 +573,13 @@ void pim_if_addr_add(struct connected *ifc)
 						   ifaddr);
 		if (!igmp) {
 			/* if addr new, add IGMP socket */
+			if (ifc->address->family == AF_INET)
+				pim_igmp_sock_add(pim_ifp->igmp_socket_list,
+						  ifaddr, ifp, false);
+		} else if (igmp->mtrace_only) {
+			igmp_sock_delete(igmp);
 			pim_igmp_sock_add(pim_ifp->igmp_socket_list, ifaddr,
-					  ifp);
+					  ifp, false);
 		}
 
 		/* Replay Static IGMP groups */
@@ -635,6 +616,20 @@ void pim_if_addr_add(struct connected *ifc)
 			}
 		}
 	} /* igmp */
+	else {
+		struct igmp_sock *igmp;
+
+		/* lookup IGMP socket */
+		igmp = pim_igmp_sock_lookup_ifaddr(pim_ifp->igmp_socket_list,
+						   ifaddr);
+		if (ifc->address->family == AF_INET) {
+			if (igmp)
+				igmp_sock_delete(igmp);
+			/* if addr new, add IGMP socket */
+			pim_igmp_sock_add(pim_ifp->igmp_socket_list, ifaddr,
+					  ifp, true);
+		}
+	} /* igmp mtrace only */
 
 	if (PIM_IF_TEST_PIM(pim_ifp->options)) {
 
@@ -665,9 +660,10 @@ void pim_if_addr_add(struct connected *ifc)
 			rpf.rpf_addr.family = AF_INET;
 			rpf.rpf_addr.prefixlen = IPV4_MAX_BITLEN;
 			rpf.rpf_addr.u.prefix4 = ifc->address->u.prefix4;
-			pnc = pim_nexthop_cache_find(&rpf);
+			pnc = pim_nexthop_cache_find(pim_ifp->pim, &rpf);
 			if (pnc)
-				pim_sendmsg_zebra_rnh(zclient, pnc,
+				pim_sendmsg_zebra_rnh(pim_ifp->pim, zclient,
+						      pnc,
 						      ZEBRA_NEXTHOP_REGISTER);
 		}
 	} /* pim */
@@ -677,7 +673,7 @@ void pim_if_addr_add(struct connected *ifc)
 	  address assigned, then try to create a vif_index.
 	*/
 	if (pim_ifp->mroute_vif_index < 0) {
-		pim_if_add_vif(ifp);
+		pim_if_add_vif(ifp, false);
 	}
 	pim_ifchannel_scan_forward_start(ifp);
 }
@@ -781,12 +777,10 @@ void pim_if_addr_add_all(struct interface *ifp)
 	for (ALL_LIST_ELEMENTS(ifp->connected, node, nextnode, ifc)) {
 		struct prefix *p = ifc->address;
 
-		if (p->family != AF_INET) {
+		if (p->family != AF_INET)
 			v6_addrs++;
-			continue;
-		}
-
-		v4_addrs++;
+		else
+			v4_addrs++;
 		pim_if_addr_add(ifc);
 	}
 
@@ -812,11 +806,11 @@ void pim_if_addr_add_all(struct interface *ifp)
 	 * address assigned, then try to create a vif_index.
 	 */
 	if (pim_ifp->mroute_vif_index < 0) {
-		pim_if_add_vif(ifp);
+		pim_if_add_vif(ifp, false);
 	}
 	pim_ifchannel_scan_forward_start(ifp);
 
-	pim_rp_setup();
+	pim_rp_setup(pim_ifp->pim);
 	pim_rp_check_on_if_add(pim_ifp);
 }
 
@@ -825,6 +819,12 @@ void pim_if_addr_del_all(struct interface *ifp)
 	struct connected *ifc;
 	struct listnode *node;
 	struct listnode *nextnode;
+	struct vrf *vrf = vrf_lookup_by_id(ifp->vrf_id);
+	struct pim_instance *pim;
+
+	if (!vrf)
+		return;
+	pim = vrf->info;
 
 	/* PIM/IGMP enabled ? */
 	if (!ifp->info)
@@ -839,8 +839,8 @@ void pim_if_addr_del_all(struct interface *ifp)
 		pim_if_addr_del(ifc, 1 /* force_prim_as_any=true */);
 	}
 
-	pim_rp_setup();
-	pim_i_am_rp_re_evaluate();
+	pim_rp_setup(pim);
+	pim_i_am_rp_re_evaluate(pim);
 }
 
 void pim_if_addr_del_all_igmp(struct interface *ifp)
@@ -887,10 +887,14 @@ struct in_addr pim_find_primary_addr(struct interface *ifp)
 {
 	struct connected *ifc;
 	struct listnode *node;
-	struct in_addr addr;
+	struct in_addr addr = {0};
 	int v4_addrs = 0;
 	int v6_addrs = 0;
 	struct pim_interface *pim_ifp = ifp->info;
+	struct vrf *vrf = vrf_lookup_by_id(ifp->vrf_id);
+
+	if (!vrf)
+		return addr;
 
 	if (pim_ifp && PIM_INADDR_ISNOT_ANY(pim_ifp->update_source)) {
 		return pim_ifp->update_source;
@@ -927,7 +931,12 @@ struct in_addr pim_find_primary_addr(struct interface *ifp)
 	 */
 	if (!v4_addrs && v6_addrs && !if_is_loopback(ifp)) {
 		struct interface *lo_ifp;
-		lo_ifp = if_lookup_by_name("lo", VRF_DEFAULT);
+		// DBS - Come back and check here
+		if (ifp->vrf_id == VRF_DEFAULT)
+			lo_ifp = if_lookup_by_name("lo", vrf->vrf_id);
+		else
+			lo_ifp = if_lookup_by_name(vrf->name, vrf->vrf_id);
+
 		if (lo_ifp)
 			return pim_find_primary_addr(lo_ifp);
 	}
@@ -939,7 +948,10 @@ struct in_addr pim_find_primary_addr(struct interface *ifp)
 
 static int pim_iface_next_vif_index(struct interface *ifp)
 {
+	struct pim_interface *pim_ifp = ifp->info;
+	struct pim_instance *pim = pim_ifp->pim;
 	int i;
+
 	/*
 	 * The pimreg vif is always going to be in index 0
 	 * of the table.
@@ -948,7 +960,7 @@ static int pim_iface_next_vif_index(struct interface *ifp)
 		return 0;
 
 	for (i = 1; i < MAXVIFS; i++) {
-		if (pim_iface_vif_index[i] == 0)
+		if (pim->iface_vif_index[i] == 0)
 			return i;
 	}
 	return MAXVIFS;
@@ -959,7 +971,7 @@ static int pim_iface_next_vif_index(struct interface *ifp)
 
   see also pim_if_find_vifindex_by_ifindex()
  */
-int pim_if_add_vif(struct interface *ifp)
+int pim_if_add_vif(struct interface *ifp, bool ispimreg)
 {
 	struct pim_interface *pim_ifp = ifp->info;
 	struct in_addr ifaddr;
@@ -981,8 +993,7 @@ int pim_if_add_vif(struct interface *ifp)
 	}
 
 	ifaddr = pim_ifp->primary_address;
-	if (ifp->ifindex != PIM_OIF_PIM_REGISTER_VIF
-	    && PIM_INADDR_IS_ANY(ifaddr)) {
+	if (!ispimreg && PIM_INADDR_IS_ANY(ifaddr)) {
 		zlog_warn(
 			"%s: could not get address for interface %s ifindex=%d",
 			__PRETTY_FUNCTION__, ifp->name, ifp->ifindex);
@@ -1010,7 +1021,7 @@ int pim_if_add_vif(struct interface *ifp)
 		return -5;
 	}
 
-	pim_iface_vif_index[pim_ifp->mroute_vif_index] = 1;
+	pim_ifp->pim->iface_vif_index[pim_ifp->mroute_vif_index] = 1;
 	return 0;
 }
 
@@ -1025,57 +1036,25 @@ int pim_if_del_vif(struct interface *ifp)
 		return -1;
 	}
 
-	pim_mroute_del_vif(pim_ifp->mroute_vif_index);
+	pim_mroute_del_vif(ifp);
 
 	/*
 	  Update vif_index
 	 */
-	pim_iface_vif_index[pim_ifp->mroute_vif_index] = 0;
+	pim_ifp->pim->iface_vif_index[pim_ifp->mroute_vif_index] = 0;
 
 	pim_ifp->mroute_vif_index = -1;
 
 	return 0;
 }
 
-void pim_if_add_vif_all()
+// DBS - VRF Revist
+struct interface *pim_if_find_by_vif_index(struct pim_instance *pim,
+					   ifindex_t vif_index)
 {
-	struct listnode *ifnode;
-	struct listnode *ifnextnode;
 	struct interface *ifp;
 
-	for (ALL_LIST_ELEMENTS(vrf_iflist(VRF_DEFAULT), ifnode, ifnextnode,
-			       ifp)) {
-		if (!ifp->info)
-			continue;
-
-		pim_if_add_vif(ifp);
-	}
-}
-
-void pim_if_del_vif_all()
-{
-	struct listnode *ifnode;
-	struct listnode *ifnextnode;
-	struct interface *ifp;
-
-	for (ALL_LIST_ELEMENTS(vrf_iflist(VRF_DEFAULT), ifnode, ifnextnode,
-			       ifp)) {
-		if (!ifp->info)
-			continue;
-
-		pim_if_del_vif(ifp);
-	}
-}
-
-struct interface *pim_if_find_by_vif_index(ifindex_t vif_index)
-{
-	struct listnode *ifnode;
-	struct interface *ifp;
-
-	if (vif_index == 0)
-		return if_lookup_by_name("pimreg", VRF_DEFAULT);
-
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), ifnode, ifp)) {
+	FOR_ALL_INTERFACES (pim->vrf, ifp) {
 		if (ifp->info) {
 			struct pim_interface *pim_ifp;
 			pim_ifp = ifp->info;
@@ -1091,12 +1070,12 @@ struct interface *pim_if_find_by_vif_index(ifindex_t vif_index)
 /*
   pim_if_add_vif() uses ifindex as vif_index
  */
-int pim_if_find_vifindex_by_ifindex(ifindex_t ifindex)
+int pim_if_find_vifindex_by_ifindex(struct pim_instance *pim, ifindex_t ifindex)
 {
 	struct pim_interface *pim_ifp;
 	struct interface *ifp;
 
-	ifp = if_lookup_by_index(ifindex, VRF_DEFAULT);
+	ifp = if_lookup_by_index(ifindex, pim->vrf_id);
 	if (!ifp || !ifp->info)
 		return -1;
 	pim_ifp = ifp->info;
@@ -1171,6 +1150,7 @@ struct pim_neighbor *pim_if_find_neighbor(struct interface *ifp,
 	struct listnode *neighnode;
 	struct pim_neighbor *neigh;
 	struct pim_interface *pim_ifp;
+	struct prefix p;
 
 	zassert(ifp);
 
@@ -1181,6 +1161,10 @@ struct pim_neighbor *pim_if_find_neighbor(struct interface *ifp,
 		return 0;
 	}
 
+	p.family = AF_INET;
+	p.u.prefix4 = addr;
+	p.prefixlen = IPV4_MAX_PREFIXLEN;
+
 	for (ALL_LIST_ELEMENTS_RO(pim_ifp->pim_neighbor_list, neighnode,
 				  neigh)) {
 
@@ -1189,7 +1173,7 @@ struct pim_neighbor *pim_if_find_neighbor(struct interface *ifp,
 			return neigh;
 
 		/* secondary address ? */
-		if (pim_neighbor_find_secondary(neigh, addr))
+		if (pim_neighbor_find_secondary(neigh, &p))
 			return neigh;
 	}
 
@@ -1318,55 +1302,38 @@ static struct igmp_join *igmp_join_new(struct interface *ifp,
 	return ij;
 }
 
-int pim_if_igmp_join_add(struct interface *ifp, struct in_addr group_addr,
-			 struct in_addr source_addr)
+ferr_r pim_if_igmp_join_add(struct interface *ifp, struct in_addr group_addr,
+			    struct in_addr source_addr)
 {
 	struct pim_interface *pim_ifp;
 	struct igmp_join *ij;
 
 	pim_ifp = ifp->info;
 	if (!pim_ifp) {
-		zlog_warn("%s: multicast not enabled on interface %s",
-			  __PRETTY_FUNCTION__, ifp->name);
-		return -1;
+		return ferr_cfg_invalid("multicast not enabled on interface %s",
+					ifp->name);
 	}
 
 	if (!pim_ifp->igmp_join_list) {
 		pim_ifp->igmp_join_list = list_new();
 		if (!pim_ifp->igmp_join_list) {
-			zlog_err("%s %s: failure: igmp_join_list=list_new()",
-				 __FILE__, __PRETTY_FUNCTION__);
-			return -2;
+			return ferr_cfg_invalid("Insufficient memory");
 		}
 		pim_ifp->igmp_join_list->del = (void (*)(void *))igmp_join_free;
 	}
 
 	ij = igmp_join_find(pim_ifp->igmp_join_list, group_addr, source_addr);
+
+	/* This interface has already been configured to join this IGMP group
+	 */
 	if (ij) {
-		char group_str[INET_ADDRSTRLEN];
-		char source_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<grp?>", group_addr, group_str,
-			       sizeof(group_str));
-		pim_inet4_dump("<src?>", source_addr, source_str,
-			       sizeof(source_str));
-		zlog_warn(
-			"%s: can't re-join existing IGMP group %s source %s on interface %s",
-			__PRETTY_FUNCTION__, group_str, source_str, ifp->name);
-		return -3;
+		return ferr_ok();
 	}
 
 	ij = igmp_join_new(ifp, group_addr, source_addr);
 	if (!ij) {
-		char group_str[INET_ADDRSTRLEN];
-		char source_str[INET_ADDRSTRLEN];
-		pim_inet4_dump("<grp?>", group_addr, group_str,
-			       sizeof(group_str));
-		pim_inet4_dump("<src?>", source_addr, source_str,
-			       sizeof(source_str));
-		zlog_warn(
-			"%s: igmp_join_new() failure for IGMP group %s source %s on interface %s",
-			__PRETTY_FUNCTION__, group_str, source_str, ifp->name);
-		return -4;
+		return ferr_cfg_invalid(
+			"Failure to create new join data structure, see log file for more information");
 	}
 
 	if (PIM_DEBUG_IGMP_EVENTS) {
@@ -1381,7 +1348,7 @@ int pim_if_igmp_join_add(struct interface *ifp, struct in_addr group_addr,
 			__PRETTY_FUNCTION__, source_str, group_str, ifp->name);
 	}
 
-	return 0;
+	return ferr_ok();
 }
 
 
@@ -1434,7 +1401,7 @@ int pim_if_igmp_join_del(struct interface *ifp, struct in_addr group_addr,
 	listnode_delete(pim_ifp->igmp_join_list, ij);
 	igmp_join_free(ij);
 	if (listcount(pim_ifp->igmp_join_list) < 1) {
-		list_delete(pim_ifp->igmp_join_list);
+		list_delete_and_null(&pim_ifp->igmp_join_list);
 		pim_ifp->igmp_join_list = 0;
 	}
 
@@ -1480,15 +1447,12 @@ void pim_if_assert_on_neighbor_down(struct interface *ifp,
 				    struct in_addr neigh_addr)
 {
 	struct pim_interface *pim_ifp;
-	struct listnode *node;
-	struct listnode *next_node;
 	struct pim_ifchannel *ch;
 
 	pim_ifp = ifp->info;
 	zassert(pim_ifp);
 
-	for (ALL_LIST_ELEMENTS(pim_ifp->pim_ifchannel_list, node, next_node,
-			       ch)) {
+	RB_FOREACH (ch, pim_ifchannel_rb, &pim_ifp->ifchannel_rb) {
 		/* Is (S,G,I) assert loser ? */
 		if (ch->ifassert_state != PIM_IFASSERT_I_AM_LOSER)
 			continue;
@@ -1502,24 +1466,23 @@ void pim_if_assert_on_neighbor_down(struct interface *ifp,
 
 void pim_if_update_join_desired(struct pim_interface *pim_ifp)
 {
-	struct listnode *ch_node;
 	struct pim_ifchannel *ch;
 
 	/* clear off flag from interface's upstreams */
-	for (ALL_LIST_ELEMENTS_RO(pim_ifp->pim_ifchannel_list, ch_node, ch)) {
+	RB_FOREACH (ch, pim_ifchannel_rb, &pim_ifp->ifchannel_rb) {
 		PIM_UPSTREAM_FLAG_UNSET_DR_JOIN_DESIRED_UPDATED(
 			ch->upstream->flags);
 	}
 
 	/* scan per-interface (S,G,I) state on this I interface */
-	for (ALL_LIST_ELEMENTS_RO(pim_ifp->pim_ifchannel_list, ch_node, ch)) {
+	RB_FOREACH (ch, pim_ifchannel_rb, &pim_ifp->ifchannel_rb) {
 		struct pim_upstream *up = ch->upstream;
 
 		if (PIM_UPSTREAM_FLAG_TEST_DR_JOIN_DESIRED_UPDATED(up->flags))
 			continue;
 
 		/* update join_desired for the global (S,G) state */
-		pim_upstream_update_join_desired(up);
+		pim_upstream_update_join_desired(pim_ifp->pim, up);
 		PIM_UPSTREAM_FLAG_SET_DR_JOIN_DESIRED_UPDATED(up->flags);
 	}
 }
@@ -1527,16 +1490,13 @@ void pim_if_update_join_desired(struct pim_interface *pim_ifp)
 void pim_if_update_assert_tracking_desired(struct interface *ifp)
 {
 	struct pim_interface *pim_ifp;
-	struct listnode *node;
-	struct listnode *next_node;
 	struct pim_ifchannel *ch;
 
 	pim_ifp = ifp->info;
 	if (!pim_ifp)
 		return;
 
-	for (ALL_LIST_ELEMENTS(pim_ifp->pim_ifchannel_list, node, next_node,
-			       ch)) {
+	RB_FOREACH (ch, pim_ifchannel_rb, &pim_ifp->ifchannel_rb) {
 		pim_ifchannel_update_assert_tracking_desired(ch);
 	}
 }
@@ -1546,14 +1506,21 @@ void pim_if_update_assert_tracking_desired(struct interface *ifp)
  * The pimreg is a special interface that we have that is not
  * quite an inteface but a VIF is created for it.
  */
-void pim_if_create_pimreg(void)
+void pim_if_create_pimreg(struct pim_instance *pim)
 {
-	if (!pim_regiface) {
-		pim_regiface =
-			if_create("pimreg", strlen("pimreg"), VRF_DEFAULT);
-		pim_regiface->ifindex = PIM_OIF_PIM_REGISTER_VIF;
+	char pimreg_name[INTERFACE_NAMSIZ];
 
-		pim_if_new(pim_regiface, 0, 0);
+	if (!pim->regiface) {
+		if (pim->vrf_id == VRF_DEFAULT)
+			strlcpy(pimreg_name, "pimreg", sizeof(pimreg_name));
+		else
+			snprintf(pimreg_name, sizeof(pimreg_name), "pimreg%u",
+				 pim->vrf->data.l.table_id);
+
+		pim->regiface = if_create(pimreg_name, pim->vrf_id);
+		pim->regiface->ifindex = PIM_OIF_PIM_REGISTER_VIF;
+
+		pim_if_new(pim->regiface, 0, 0, true);
 	}
 }
 
@@ -1580,14 +1547,37 @@ int pim_if_connected_to_source(struct interface *ifp, struct in_addr src)
 	return 0;
 }
 
-struct interface *pim_if_lookup_address_vrf(struct in_addr src, vrf_id_t vrf_id)
+int pim_if_is_loopback(struct pim_instance *pim, struct interface *ifp)
 {
-	struct listnode *ifnode;
-	struct interface *ifp;
+	if (if_is_loopback(ifp))
+		return 1;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(vrf_id), ifnode, ifp)) {
-		if (pim_if_connected_to_source(ifp, src) && ifp->info)
-			return ifp;
+	if (strcmp(ifp->name, pim->vrf->name) == 0)
+		return 1;
+
+	return 0;
+}
+
+int pim_if_is_vrf_device(struct interface *ifp)
+{
+	struct vrf *vrf;
+
+	RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
+		if (strncmp(ifp->name, vrf->name, strlen(ifp->name)) == 0)
+			return 1;
 	}
-	return NULL;
+
+	return 0;
+}
+
+int pim_if_ifchannel_count(struct pim_interface *pim_ifp)
+{
+	struct pim_ifchannel *ch;
+	int count = 0;
+
+	RB_FOREACH (ch, pim_ifchannel_rb, &pim_ifp->ifchannel_rb) {
+		count++;
+	}
+
+	return count;
 }
