@@ -1,22 +1,21 @@
 /*
-  PIM for Quagga
-  Copyright (C) 2008  Everton da Silva Marques
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING; if not, write to the
-  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-  MA 02110-1301 USA
-*/
+ * PIM for Quagga
+ * Copyright (C) 2008  Everton da Silva Marques
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -30,6 +29,7 @@
 #include "pim_igmp.h"
 #include "pim_igmpv2.h"
 #include "pim_igmpv3.h"
+#include "pim_igmp_mtrace.h"
 #include "pim_iface.h"
 #include "pim_sock.h"
 #include "pim_mroute.h"
@@ -158,7 +158,6 @@ static int pim_igmp_other_querier_expire(struct thread *t)
 
 	igmp = THREAD_ARG(t);
 
-	zassert(igmp->t_other_querier_timer);
 	zassert(!igmp->t_igmp_query_timer);
 
 	if (PIM_DEBUG_IGMP_TRACE) {
@@ -169,8 +168,6 @@ static int pim_igmp_other_querier_expire(struct thread *t)
 			   ifaddr_str);
 	}
 
-	igmp->t_other_querier_timer = NULL;
-
 	/*
 	  We are the current querier, then
 	  re-start sending general queries.
@@ -208,9 +205,7 @@ void pim_igmp_other_querier_timer_on(struct igmp_sock *igmp)
 				"Querier %s resetting TIMER event for Other-Querier-Present",
 				ifaddr_str);
 		}
-
 		THREAD_OFF(igmp->t_other_querier_timer);
-		zassert(!igmp->t_other_querier_timer);
 	} else {
 		/*
 		  We are the current querier, then stop sending general queries:
@@ -253,9 +248,9 @@ void pim_igmp_other_querier_timer_on(struct igmp_sock *igmp)
 			other_querier_present_interval_msec % 1000);
 	}
 
-	THREAD_TIMER_MSEC_ON(master, igmp->t_other_querier_timer,
-			     pim_igmp_other_querier_expire, igmp,
-			     other_querier_present_interval_msec);
+	thread_add_timer_msec(master, pim_igmp_other_querier_expire, igmp,
+			      other_querier_present_interval_msec,
+			      &igmp->t_other_querier_timer);
 }
 
 void pim_igmp_other_querier_timer_off(struct igmp_sock *igmp)
@@ -273,7 +268,6 @@ void pim_igmp_other_querier_timer_off(struct igmp_sock *igmp)
 		}
 	}
 	THREAD_OFF(igmp->t_other_querier_timer);
-	zassert(!igmp->t_other_querier_timer);
 }
 
 static int igmp_recv_query(struct igmp_sock *igmp, int query_version,
@@ -287,6 +281,9 @@ static int igmp_recv_query(struct igmp_sock *igmp, int query_version,
 	uint16_t recv_checksum;
 	uint16_t checksum;
 
+	if (igmp->mtrace_only)
+		return 0;
+
 	memcpy(&group_addr, igmp_msg + 4, sizeof(struct in_addr));
 
 	ifp = igmp->interface;
@@ -306,22 +303,19 @@ static int igmp_recv_query(struct igmp_sock *igmp, int query_version,
 		return -1;
 	}
 
-	/* RFC 3376 defines some guidelines on operating in backwards
-	 * compatibility
-	 * with older versions of IGMP but there are some gaps in the logic:
+	/*
+	 * RFC 3376 defines some guidelines on operating in backwards
+	 * compatibility with older versions of IGMP but there are some gaps in
+	 * the logic:
 	 *
 	 * - once we drop from say version 3 to version 2 we will never go back
-	 * to
-	 *   version 3 even if the node that TXed an IGMP v2 query upgrades to
-	 * v3
+	 *   to version 3 even if the node that TXed an IGMP v2 query upgrades
+	 *   to v3
 	 *
 	 * - The node with the lowest IP is the querier so we will only know to
-	 * drop
-	 *   from v3 to v2 if the node that is the querier is also the one that
-	 * is
-	 *   running igmp v2.  If a non-querier only supports igmp v2 we will
-	 * have
-	 *   no way of knowing.
+	 *   drop from v3 to v2 if the node that is the querier is also the one
+	 *   that is running igmp v2.  If a non-querier only supports igmp v2
+	 *   we will have no way of knowing.
 	 *
 	 * For now we will simplify things and inform the user that they need to
 	 * configure all PIM routers to use the same version of IGMP.
@@ -396,6 +390,9 @@ static int igmp_v1_recv_report(struct igmp_sock *igmp, struct in_addr from,
 
 	on_trace(__PRETTY_FUNCTION__, igmp->interface, from);
 
+	if (igmp->mtrace_only)
+		return 0;
+
 	if (igmp_msg_len != IGMP_V12_MSG_SIZE) {
 		zlog_warn(
 			"Recv IGMP report v1 from %s on %s: size=%d other than correct=%d",
@@ -410,6 +407,9 @@ static int igmp_v1_recv_report(struct igmp_sock *igmp, struct in_addr from,
 
 	memcpy(&group_addr, igmp_msg + 4, sizeof(struct in_addr));
 
+	if (pim_is_group_filtered(ifp->info, &group_addr))
+		return -1;
+
 	/* non-existant group is created as INCLUDE {empty} */
 	group = igmp_add_group_by_addr(igmp, group_addr);
 	if (!group) {
@@ -511,6 +511,15 @@ int pim_igmp_packet(struct igmp_sock *igmp, char *buf, size_t len)
 	case PIM_IGMP_V2_LEAVE_GROUP:
 		return igmp_v2_recv_leave(igmp, ip_hdr->ip_src, from_str,
 					  igmp_msg, igmp_msg_len);
+
+	case PIM_IGMP_MTRACE_RESPONSE:
+		return igmp_mtrace_recv_response(igmp, ip_hdr, ip_hdr->ip_src,
+						 from_str, igmp_msg,
+						 igmp_msg_len);
+	case PIM_IGMP_MTRACE_QUERY_REQUEST:
+		return igmp_mtrace_recv_qry_req(igmp, ip_hdr, ip_hdr->ip_src,
+						from_str, igmp_msg,
+						igmp_msg_len);
 	}
 
 	zlog_warn("Ignoring unsupported IGMP message type: %d", msg_type);
@@ -571,8 +580,8 @@ void pim_igmp_general_query_on(struct igmp_sock *igmp)
 			startup_mode ? "startup" : "non-startup", igmp->fd);
 	}
 	igmp->t_igmp_query_timer = NULL;
-	THREAD_TIMER_ON(master, igmp->t_igmp_query_timer,
-			pim_igmp_general_query, igmp, query_interval);
+	thread_add_timer(master, pim_igmp_general_query, igmp, query_interval,
+			 &igmp->t_igmp_query_timer);
 }
 
 void pim_igmp_general_query_off(struct igmp_sock *igmp)
@@ -703,7 +712,7 @@ void igmp_startup_mode_on(struct igmp_sock *igmp)
 
 static void igmp_group_free(struct igmp_group *group)
 {
-	list_free(group->group_source_list);
+	list_delete_and_null(&group->group_source_list);
 
 	XFREE(MTYPE_PIM_IGMP_GROUP, group);
 }
@@ -755,7 +764,7 @@ void igmp_sock_free(struct igmp_sock *igmp)
 	zassert(igmp->igmp_group_list);
 	zassert(!listcount(igmp->igmp_group_list));
 
-	list_free(igmp->igmp_group_list);
+	list_delete_and_null(&igmp->igmp_group_list);
 	hash_free(igmp->igmp_group_hash);
 
 	XFREE(MTYPE_PIM_IGMP_SOCKET, igmp);
@@ -815,10 +824,11 @@ static int igmp_group_hash_equal(const void *arg1, const void *arg2)
 }
 
 static struct igmp_sock *igmp_sock_new(int fd, struct in_addr ifaddr,
-				       struct interface *ifp)
+				       struct interface *ifp, int mtrace_only)
 {
 	struct pim_interface *pim_ifp;
 	struct igmp_sock *igmp;
+	char hash_name[64];
 
 	pim_ifp = ifp->info;
 
@@ -843,8 +853,9 @@ static struct igmp_sock *igmp_sock_new(int fd, struct in_addr ifaddr,
 	}
 	igmp->igmp_group_list->del = (void (*)(void *))igmp_group_free;
 
-	igmp->igmp_group_hash =
-		hash_create(igmp_group_hash_key, igmp_group_hash_equal);
+	snprintf(hash_name, 64, "IGMP %s hash", ifp->name);
+	igmp->igmp_group_hash = hash_create(igmp_group_hash_key,
+					    igmp_group_hash_equal, hash_name);
 
 	igmp->fd = fd;
 	igmp->interface = ifp;
@@ -856,6 +867,13 @@ static struct igmp_sock *igmp_sock_new(int fd, struct in_addr ifaddr,
 		pim_ifp->igmp_default_robustness_variable;
 	igmp->sock_creation = pim_time_monotonic_sec();
 
+	if (mtrace_only) {
+		igmp->mtrace_only = mtrace_only;
+		return igmp;
+	}
+
+	igmp->mtrace_only = false;
+
 	/*
 	  igmp_startup_mode_on() will reset QQI:
 
@@ -907,13 +925,14 @@ static void igmp_read_on(struct igmp_sock *igmp)
 			   igmp->fd);
 	}
 	igmp->t_igmp_read = NULL;
-	THREAD_READ_ON(master, igmp->t_igmp_read, pim_igmp_read, igmp,
-		       igmp->fd);
+	thread_add_read(master, pim_igmp_read, igmp, igmp->fd,
+			&igmp->t_igmp_read);
 }
 
 struct igmp_sock *pim_igmp_sock_add(struct list *igmp_sock_list,
 				    struct in_addr ifaddr,
-				    struct interface *ifp)
+				    struct interface *ifp,
+				    bool mtrace_only)
 {
 	struct pim_interface *pim_ifp;
 	struct igmp_sock *igmp;
@@ -928,7 +947,7 @@ struct igmp_sock *pim_igmp_sock_add(struct list *igmp_sock_list,
 		return 0;
 	}
 
-	igmp = igmp_sock_new(fd, ifaddr, ifp);
+	igmp = igmp_sock_new(fd, ifaddr, ifp, mtrace_only);
 	if (!igmp) {
 		zlog_err("%s %s: igmp_sock_new() failure", __FILE__,
 			 __PRETTY_FUNCTION__);
@@ -978,7 +997,6 @@ static int igmp_group_timer(struct thread *t)
 
 	zassert(group->group_filtermode_isexcl);
 
-	group->t_group_timer = NULL;
 	group->group_filtermode_isexcl = 0;
 
 	/* Any source (*,G) is forwarded only if mode is EXCLUDE {empty} */
@@ -986,7 +1004,6 @@ static int igmp_group_timer(struct thread *t)
 
 	igmp_source_delete_expired(group->group_source_list);
 
-	zassert(!group->t_group_timer);
 	zassert(!group->group_filtermode_isexcl);
 
 	/*
@@ -1014,9 +1031,7 @@ static void group_timer_off(struct igmp_group *group)
 		zlog_debug("Cancelling TIMER event for group %s on %s",
 			   group_str, group->group_igmp_sock->interface->name);
 	}
-
 	THREAD_OFF(group->t_group_timer);
-	zassert(!group->t_group_timer);
 }
 
 void igmp_group_timer_on(struct igmp_group *group, long interval_msec,
@@ -1043,8 +1058,8 @@ void igmp_group_timer_on(struct igmp_group *group, long interval_msec,
 	*/
 	zassert(group->group_filtermode_isexcl);
 
-	THREAD_TIMER_MSEC_ON(master, group->t_group_timer, igmp_group_timer,
-			     group, interval_msec);
+	thread_add_timer_msec(master, igmp_group_timer, group, interval_msec,
+			      &group->t_group_timer);
 }
 
 struct igmp_group *find_group_by_addr(struct igmp_sock *igmp,
