@@ -1,22 +1,21 @@
 /*
-  PIM for Quagga
-  Copyright (C) 2008  Everton da Silva Marques
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING; if not, write to the
-  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-  MA 02110-1301 USA
-*/
+ * PIM for Quagga
+ * Copyright (C) 2008  Everton da Silva Marques
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 #include "log.h"
@@ -130,9 +129,6 @@ static int igmp_source_timer(struct thread *t)
 			group->group_igmp_sock->interface->name);
 	}
 
-	zassert(source->t_source_timer);
-	source->t_source_timer = NULL;
-
 	/*
 	  RFC 3376: 6.3. IGMPv3 Source-Specific Forwarding Rules
 
@@ -153,8 +149,6 @@ static int igmp_source_timer(struct thread *t)
 	  Source timer switched from (T > 0) to (T == 0): disable forwarding.
 	 */
 
-	zassert(!source->t_source_timer);
-
 	if (group->group_filtermode_isexcl) {
 		/* EXCLUDE mode */
 
@@ -198,13 +192,13 @@ static void source_timer_off(struct igmp_group *group,
 	}
 
 	THREAD_OFF(source->t_source_timer);
-	zassert(!source->t_source_timer);
 }
 
 static void igmp_source_timer_on(struct igmp_group *group,
 				 struct igmp_source *source, long interval_msec)
 {
 	source_timer_off(group, source);
+	struct pim_interface *pim_ifp = group->group_igmp_sock->interface->info;
 
 	if (PIM_DEBUG_IGMP_EVENTS) {
 		char group_str[INET_ADDRSTRLEN];
@@ -219,16 +213,15 @@ static void igmp_source_timer_on(struct igmp_group *group,
 			source_str, group->group_igmp_sock->interface->name);
 	}
 
-	THREAD_TIMER_MSEC_ON(master, source->t_source_timer, igmp_source_timer,
-			     source, interval_msec);
-	zassert(source->t_source_timer);
+	thread_add_timer_msec(master, igmp_source_timer, source, interval_msec,
+			      &source->t_source_timer);
 
 	/*
 	  RFC 3376: 6.3. IGMPv3 Source-Specific Forwarding Rules
 
 	  Source timer switched from (T == 0) to (T > 0): enable forwarding.
 	*/
-	igmp_source_forward_start(source);
+	igmp_source_forward_start(pim_ifp->pim, source);
 }
 
 void igmp_source_reset_gmi(struct igmp_sock *igmp, struct igmp_group *group,
@@ -318,10 +311,12 @@ static void source_clear_send_flag(struct list *source_list)
 */
 static void group_exclude_fwd_anysrc_ifempty(struct igmp_group *group)
 {
+	struct pim_interface *pim_ifp = group->group_igmp_sock->interface->info;
+
 	zassert(group->group_filtermode_isexcl);
 
 	if (listcount(group->group_source_list) < 1) {
-		igmp_anysource_forward_start(group);
+		igmp_anysource_forward_start(pim_ifp->pim, group);
 	}
 }
 
@@ -478,8 +473,6 @@ struct igmp_source *source_new(struct igmp_group *group,
 
 	listnode_add(group->group_source_list, src);
 
-	zassert(!src->t_source_timer); /* source timer == 0 */
-
 	/* Any source (*,G) is forwarded only if mode is EXCLUDE {empty} */
 	igmp_anysource_forward_stop(group);
 
@@ -678,6 +671,9 @@ void igmpv3_report_isex(struct igmp_sock *igmp, struct in_addr from,
 	on_trace(__PRETTY_FUNCTION__, ifp, from, group_addr, num_sources,
 		 sources);
 
+	if (pim_is_group_filtered(ifp->info, &group_addr))
+		return;
+
 	/* non-existant group is created as INCLUDE {empty} */
 	group = igmp_add_group_by_addr(igmp, group_addr);
 	if (!group) {
@@ -1286,8 +1282,6 @@ static int igmp_group_retransmit(struct thread *t)
 	num_retransmit_sources_left =
 		group_retransmit_sources(group, send_with_sflag_set);
 
-	group->t_group_query_retransmit_timer = NULL;
-
 	/*
 	  Keep group retransmit timer running if there is any retransmit
 	  counter pending
@@ -1331,8 +1325,8 @@ static void group_retransmit_timer_on(struct igmp_group *group)
 			igmp->interface->name);
 	}
 
-	THREAD_TIMER_MSEC_ON(master, group->t_group_query_retransmit_timer,
-			     igmp_group_retransmit, group, lmqi_msec);
+	thread_add_timer_msec(master, igmp_group_retransmit, group, lmqi_msec,
+			      &group->t_group_query_retransmit_timer);
 }
 
 static long igmp_group_timer_remain_msec(struct igmp_group *group)
@@ -1878,6 +1872,12 @@ int igmp_v3_recv_report(struct igmp_sock *igmp, struct in_addr from,
 	struct interface *ifp = igmp->interface;
 	int i;
 	int local_ncb = 0;
+	struct pim_interface *pim_ifp;
+
+	if (igmp->mtrace_only)
+		return 0;
+
+	pim_ifp = igmp->interface->info;
 
 	if (igmp_msg_len < IGMP_V3_MSG_MIN_SIZE) {
 		zlog_warn(
@@ -1929,6 +1929,7 @@ int igmp_v3_recv_report(struct igmp_sock *igmp, struct in_addr from,
 		int j;
 		struct prefix lncb;
 		struct prefix g;
+		bool filtered = false;
 
 		if ((group_record + IGMP_V3_GROUP_RECORD_MIN_SIZE)
 		    > report_pastend) {
@@ -1992,6 +1993,16 @@ int igmp_v3_recv_report(struct igmp_sock *igmp, struct in_addr from,
 		g.family = AF_INET;
 		g.u.prefix4 = rec_group;
 		g.prefixlen = 32;
+
+		/* determine filtering status for group */
+		filtered = pim_is_group_filtered(ifp->info, &rec_group);
+
+		if (PIM_DEBUG_IGMP_PACKETS && filtered)
+			zlog_debug(
+				"Filtering IGMPv3 group record %s from %s on %s per prefix-list %s",
+				inet_ntoa(rec_group), from_str, ifp->name,
+				pim_ifp->boundary_oil_plist);
+
 		/*
 		 * If we receive a igmp report with the group in 224.0.0.0/24
 		 * then we should ignore it
@@ -1999,7 +2010,7 @@ int igmp_v3_recv_report(struct igmp_sock *igmp, struct in_addr from,
 		if (prefix_match(&lncb, &g))
 			local_ncb = 1;
 
-		if (!local_ncb)
+		if (!local_ncb && !filtered)
 			switch (rec_type) {
 			case IGMP_GRP_REC_TYPE_MODE_IS_INCLUDE:
 				igmpv3_report_isin(igmp, from, rec_group,
