@@ -1,22 +1,21 @@
 /*
-  PIM for Quagga
-  Copyright (C) 2008  Everton da Silva Marques
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING; if not, write to the
-  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-  MA 02110-1301 USA
-*/
+ * PIM for Quagga
+ * Copyright (C) 2008  Everton da Silva Marques
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -39,6 +38,7 @@
 #include "pim_zebra.h"
 #include "pim_join.h"
 #include "pim_jp_agg.h"
+#include "pim_bfd.h"
 
 static void dr_election_by_addr(struct interface *ifp)
 {
@@ -217,8 +217,6 @@ static int on_neighbor_timer(struct thread *t)
 			neigh->holdtime, src_str, ifp->name);
 	}
 
-	neigh->t_expire_timer = NULL;
-
 	snprintf(msg, sizeof(msg), "%d-sec holdtime expired", neigh->holdtime);
 	pim_neighbor_delete(ifp, neigh, msg);
 
@@ -256,8 +254,8 @@ void pim_neighbor_timer_reset(struct pim_neighbor *neigh, uint16_t holdtime)
 			   neigh->interface->name);
 	}
 
-	THREAD_TIMER_ON(master, neigh->t_expire_timer, on_neighbor_timer, neigh,
-			neigh->holdtime);
+	thread_add_timer(master, on_neighbor_timer, neigh, neigh->holdtime,
+			 &neigh->t_expire_timer);
 }
 
 static int on_neighbor_jp_timer(struct thread *t)
@@ -273,14 +271,13 @@ static int on_neighbor_jp_timer(struct thread *t)
 			   __PRETTY_FUNCTION__, src_str, neigh->interface->name,
 			   neigh->upstream_jp_agg->count);
 	}
-	neigh->jp_timer = NULL;
 
 	rpf.source_nexthop.interface = neigh->interface;
 	rpf.rpf_addr.u.prefix4 = neigh->source_addr;
 	pim_joinprune_send(&rpf, neigh->upstream_jp_agg);
 
-	THREAD_TIMER_ON(master, neigh->jp_timer, on_neighbor_jp_timer, neigh,
-			qpim_t_periodic);
+	thread_add_timer(master, on_neighbor_jp_timer, neigh, qpim_t_periodic,
+			 &neigh->jp_timer);
 
 	return 0;
 }
@@ -288,8 +285,8 @@ static int on_neighbor_jp_timer(struct thread *t)
 static void pim_neighbor_start_jp_timer(struct pim_neighbor *neigh)
 {
 	THREAD_TIMER_OFF(neigh->jp_timer);
-	THREAD_TIMER_ON(master, neigh->jp_timer, on_neighbor_jp_timer, neigh,
-			qpim_t_periodic);
+	thread_add_timer(master, on_neighbor_jp_timer, neigh, qpim_t_periodic,
+			 &neigh->jp_timer);
 }
 
 static struct pim_neighbor *
@@ -374,6 +371,9 @@ pim_neighbor_new(struct interface *ifp, struct in_addr source_addr,
 		++pim_ifp->pim_dr_num_nondrpri_neighbors;
 	}
 
+	// Register PIM Neighbor with BFD
+	pim_bfd_trigger_event(pim_ifp, neigh, 1);
+
 	return neigh;
 }
 
@@ -401,8 +401,7 @@ static void delete_prefix_list(struct pim_neighbor *neigh)
 		}
 #endif
 
-		list_delete(neigh->prefix_list);
-		neigh->prefix_list = 0;
+		list_delete_and_null(&neigh->prefix_list);
 	}
 }
 
@@ -412,12 +411,34 @@ void pim_neighbor_free(struct pim_neighbor *neigh)
 
 	delete_prefix_list(neigh);
 
-	list_delete(neigh->upstream_jp_agg);
+	list_delete_and_null(&neigh->upstream_jp_agg);
 	THREAD_OFF(neigh->jp_timer);
 
 	XFREE(MTYPE_PIM_NEIGHBOR, neigh);
 }
 
+struct pim_neighbor *pim_neighbor_find_by_secondary(struct interface *ifp,
+						    struct prefix *src)
+{
+	struct pim_interface *pim_ifp;
+	struct listnode *node, *pnode;
+	struct pim_neighbor *neigh;
+	struct prefix *p;
+
+	pim_ifp = ifp->info;
+	if (!pim_ifp)
+		return NULL;
+
+	for (ALL_LIST_ELEMENTS_RO(pim_ifp->pim_neighbor_list, node, neigh)) {
+		for (ALL_LIST_ELEMENTS_RO(neigh->prefix_list, pnode, p)) {
+			if (prefix_same(p, src))
+				return neigh;
+		}
+	}
+
+	return NULL;
+}
+
 struct pim_neighbor *pim_neighbor_find(struct interface *ifp,
 				       struct in_addr source_addr)
 {
@@ -456,17 +477,6 @@ struct pim_neighbor *pim_neighbor_find_if(struct interface *ifp)
 	return listnode_head(pim_ifp->pim_neighbor_list);
 }
 
-/* rpf info associated with an upstream entry needs to be re-evaluated
- * when an RPF neighbor comes or goes */
-static void pim_neighbor_rpf_update(void)
-{
-	/* XXX: for the time being piggyback on the timer used on rib changes
-	 * to scan and update the rpf nexthop. This is expensive processing
-	 * and we should be able to optimize neighbor changes differently than
-	 * nexthop changes. */
-	sched_rpf_cache_refresh();
-}
-
 struct pim_neighbor *
 pim_neighbor_add(struct interface *ifp, struct in_addr source_addr,
 		 pim_hello_options hello_options, uint16_t holdtime,
@@ -523,7 +533,7 @@ pim_neighbor_add(struct interface *ifp, struct in_addr source_addr,
 	else
 		pim_hello_restart_triggered(neigh->interface);
 
-	pim_upstream_find_new_rpf();
+	pim_upstream_find_new_rpf(pim_ifp->pim);
 
 	/* RNH can send nexthop update prior to PIM neibhor UP
 	   in that case nexthop cache would not consider this neighbor
@@ -531,11 +541,11 @@ pim_neighbor_add(struct interface *ifp, struct in_addr source_addr,
 	   Upon PIM neighbor UP, iterate all RPs and update
 	   nexthop cache with this neighbor.
 	 */
-	pim_resolve_rp_nh();
+	pim_resolve_rp_nh(pim_ifp->pim);
 
-	pim_rp_setup();
+	pim_rp_setup(pim_ifp->pim);
 
-	pim_neighbor_rpf_update();
+	sched_rpf_cache_refresh(pim_ifp->pim);
 	return neigh;
 }
 
@@ -650,11 +660,14 @@ void pim_neighbor_delete(struct interface *ifp, struct pim_neighbor *neigh,
 			   __PRETTY_FUNCTION__, src_str, ifp->name);
 	}
 
+	// De-Register PIM Neighbor with BFD
+	pim_bfd_trigger_event(pim_ifp, neigh, 0);
+
 	listnode_delete(pim_ifp->pim_neighbor_list, neigh);
 
 	pim_neighbor_free(neigh);
 
-	pim_neighbor_rpf_update();
+	sched_rpf_cache_refresh(pim_ifp->pim);
 }
 
 void pim_neighbor_delete_all(struct interface *ifp, const char *delete_message)
@@ -674,7 +687,7 @@ void pim_neighbor_delete_all(struct interface *ifp, const char *delete_message)
 }
 
 struct prefix *pim_neighbor_find_secondary(struct pim_neighbor *neigh,
-					   struct in_addr addr)
+					   struct prefix *addr)
 {
 	struct listnode *node;
 	struct prefix *p;
@@ -683,14 +696,11 @@ struct prefix *pim_neighbor_find_secondary(struct pim_neighbor *neigh,
 		return 0;
 
 	for (ALL_LIST_ELEMENTS_RO(neigh->prefix_list, node, p)) {
-		if (p->family == AF_INET) {
-			if (addr.s_addr == p->u.prefix4.s_addr) {
-				return p;
-			}
-		}
+		if (prefix_same(p, addr))
+			return p;
 	}
 
-	return 0;
+	return NULL;
 }
 
 /*
@@ -734,7 +744,7 @@ static void delete_from_neigh_addr(struct interface *ifp,
 					  neigh_node, neigh)) {
 			{
 				struct prefix *p = pim_neighbor_find_secondary(
-					neigh, addr->u.prefix4);
+					neigh, addr);
 				if (p) {
 					char addr_str[INET_ADDRSTRLEN];
 					char this_neigh_str[INET_ADDRSTRLEN];
