@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
- * MA 02110-1301 USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <zebra.h>
 
@@ -31,6 +30,7 @@
 #include "vrf.h"
 #include "plist.h"
 #include "nexthop.h"
+#include "table.h"
 
 #include "pimd.h"
 #include "pim_vty.h"
@@ -46,8 +46,19 @@
 #include "pim_nht.h"
 
 
-static struct list *qpim_rp_list = NULL;
-static struct rp_info *tail = NULL;
+/* Cleanup pim->rpf_hash each node data */
+void pim_rp_list_hash_clean(void *data)
+{
+	struct pim_nexthop_cache *pnc = (struct pim_nexthop_cache *)data;
+
+	list_delete_and_null(&pnc->rp_list);
+
+	hash_clean(pnc->upstream_hash, NULL);
+	hash_free(pnc->upstream_hash);
+	pnc->upstream_hash = NULL;
+
+	XFREE(MTYPE_PIM_NEXTHOP_CACHE, pnc);
+}
 
 static void pim_rp_info_free(struct rp_info *rp_info)
 {
@@ -82,45 +93,82 @@ int pim_rp_list_cmp(void *v1, void *v2)
 	return 0;
 }
 
-void pim_rp_init(void)
+void pim_rp_init(struct pim_instance *pim)
 {
 	struct rp_info *rp_info;
+	struct route_node *rn;
 
-	qpim_rp_list = list_new();
-	qpim_rp_list->del = (void (*)(void *))pim_rp_info_free;
-	qpim_rp_list->cmp = pim_rp_list_cmp;
+	pim->rp_list = list_new();
+	if (!pim->rp_list) {
+		zlog_err("Unable to alloc rp_list");
+		return;
+	}
+	pim->rp_list->del = (void (*)(void *))pim_rp_info_free;
+	pim->rp_list->cmp = pim_rp_list_cmp;
+
+	pim->rp_table = route_table_init();
+	if (!pim->rp_table) {
+		zlog_err("Unable to alloc rp_table");
+		list_delete_and_null(&pim->rp_list);
+		return;
+	}
 
 	rp_info = XCALLOC(MTYPE_PIM_RP, sizeof(*rp_info));
 
-	if (!rp_info)
+	if (!rp_info) {
+		zlog_err("Unable to alloc rp_info");
+		route_table_finish(pim->rp_table);
+		list_delete_and_null(&pim->rp_list);
 		return;
+	}
 
-	str2prefix("224.0.0.0/4", &rp_info->group);
+	if (!str2prefix("224.0.0.0/4", &rp_info->group)) {
+		zlog_err("Unable to convert 224.0.0.0/4 to prefix");
+		list_delete_and_null(&pim->rp_list);
+		route_table_finish(pim->rp_table);
+		XFREE(MTYPE_PIM_RP, rp_info);
+		return;
+	}
 	rp_info->group.family = AF_INET;
 	rp_info->rp.rpf_addr.family = AF_INET;
+	rp_info->rp.rpf_addr.prefixlen = IPV4_MAX_PREFIXLEN;
 	rp_info->rp.rpf_addr.u.prefix4.s_addr = INADDR_NONE;
-	tail = rp_info;
 
-	listnode_add(qpim_rp_list, rp_info);
+	listnode_add(pim->rp_list, rp_info);
+
+	rn = route_node_get(pim->rp_table, &rp_info->group);
+	if (!rn) {
+		zlog_err("Failure to get route node for pim->rp_table");
+		list_delete_and_null(&pim->rp_list);
+		route_table_finish(pim->rp_table);
+		XFREE(MTYPE_PIM_RP, rp_info);
+		return;
+	}
+
+	rn->info = rp_info;
+	if (PIM_DEBUG_TRACE)
+		zlog_debug(
+			"Allocated: %p for rp_info: %p(224.0.0.0/4) Lock: %d",
+			rn, rp_info, rn->lock);
 }
 
-void pim_rp_free(void)
+void pim_rp_free(struct pim_instance *pim)
 {
-	if (qpim_rp_list)
-		list_delete(qpim_rp_list);
-	qpim_rp_list = NULL;
+	if (pim->rp_list)
+		list_delete_and_null(&pim->rp_list);
 }
 
 /*
  * Given an RP's prefix-list, return the RP's rp_info for that prefix-list
  */
-static struct rp_info *pim_rp_find_prefix_list(struct in_addr rp,
+static struct rp_info *pim_rp_find_prefix_list(struct pim_instance *pim,
+					       struct in_addr rp,
 					       const char *plist)
 {
 	struct listnode *node;
 	struct rp_info *rp_info;
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (rp.s_addr == rp_info->rp.rpf_addr.u.prefix4.s_addr
 		    && rp_info->plist && strcmp(rp_info->plist, plist) == 0) {
 			return rp_info;
@@ -133,12 +181,12 @@ static struct rp_info *pim_rp_find_prefix_list(struct in_addr rp,
 /*
  * Return true if plist is used by any rp_info
  */
-static int pim_rp_prefix_list_used(const char *plist)
+static int pim_rp_prefix_list_used(struct pim_instance *pim, const char *plist)
 {
 	struct listnode *node;
 	struct rp_info *rp_info;
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (rp_info->plist && strcmp(rp_info->plist, plist) == 0) {
 			return 1;
 		}
@@ -151,13 +199,14 @@ static int pim_rp_prefix_list_used(const char *plist)
  * Given an RP's address, return the RP's rp_info that is an exact match for
  * 'group'
  */
-static struct rp_info *pim_rp_find_exact(struct in_addr rp,
+static struct rp_info *pim_rp_find_exact(struct pim_instance *pim,
+					 struct in_addr rp,
 					 struct prefix *group)
 {
 	struct listnode *node;
 	struct rp_info *rp_info;
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (rp.s_addr == rp_info->rp.rpf_addr.u.prefix4.s_addr
 		    && prefix_same(&rp_info->group, group))
 			return rp_info;
@@ -169,26 +218,64 @@ static struct rp_info *pim_rp_find_exact(struct in_addr rp,
 /*
  * Given a group, return the rp_info for that group
  */
-static struct rp_info *pim_rp_find_match_group(struct prefix *group)
+static struct rp_info *pim_rp_find_match_group(struct pim_instance *pim,
+					       struct prefix *group)
 {
 	struct listnode *node;
+	struct rp_info *best = NULL;
 	struct rp_info *rp_info;
 	struct prefix_list *plist;
+	struct prefix *p, *bp;
+	struct route_node *rn;
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	bp = NULL;
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (rp_info->plist) {
 			plist = prefix_list_lookup(AFI_IP, rp_info->plist);
 
-			if (plist
-			    && prefix_list_apply(plist, group) == PREFIX_PERMIT)
-				return rp_info;
-		} else {
-			if (prefix_match(&rp_info->group, group))
-				return rp_info;
+			if (prefix_list_apply_which_prefix(plist, &p, group)
+			    == PREFIX_DENY)
+				continue;
+
+			if (!best) {
+				best = rp_info;
+				bp = p;
+				continue;
+			}
+
+			if (bp && bp->prefixlen < p->prefixlen) {
+				best = rp_info;
+				bp = p;
+			}
 		}
 	}
 
-	return NULL;
+	rn = route_node_match(pim->rp_table, group);
+	if (!rn) {
+		zlog_err(
+			"%s: BUG We should have found default group information\n",
+			__PRETTY_FUNCTION__);
+		return best;
+	}
+
+	rp_info = rn->info;
+	if (PIM_DEBUG_TRACE) {
+		char buf[PREFIX_STRLEN];
+
+		route_unlock_node(rn);
+		zlog_debug("Lookedup: %p for rp_info: %p(%s) Lock: %d", rn,
+			   rp_info,
+			   prefix2str(&rp_info->group, buf, sizeof(buf)),
+			   rn->lock);
+	}
+
+	if (!best)
+		return rp_info;
+
+	if (rp_info->group.prefixlen < best->group.prefixlen)
+		best = rp_info;
+
+	return best;
 }
 
 /*
@@ -198,18 +285,19 @@ static struct rp_info *pim_rp_find_match_group(struct prefix *group)
  *
  * This is a placeholder function for now.
  */
-static void pim_rp_refresh_group_to_rp_mapping()
+static void pim_rp_refresh_group_to_rp_mapping(struct pim_instance *pim)
 {
-	pim_msdp_i_am_rp_changed();
+	pim_msdp_i_am_rp_changed(pim);
 }
 
-void pim_rp_prefix_list_update(struct prefix_list *plist)
+void pim_rp_prefix_list_update(struct pim_instance *pim,
+			       struct prefix_list *plist)
 {
 	struct listnode *node;
 	struct rp_info *rp_info;
 	int refresh_needed = 0;
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (rp_info->plist
 		    && strcmp(rp_info->plist, prefix_list_name(plist)) == 0) {
 			refresh_needed = 1;
@@ -218,7 +306,7 @@ void pim_rp_prefix_list_update(struct prefix_list *plist)
 	}
 
 	if (refresh_needed)
-		pim_rp_refresh_group_to_rp_mapping();
+		pim_rp_refresh_group_to_rp_mapping(pim);
 }
 
 static int pim_rp_check_interface_addrs(struct rp_info *rp_info,
@@ -236,8 +324,7 @@ static int pim_rp_check_interface_addrs(struct rp_info *rp_info,
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(pim_ifp->sec_addr_list, node, sec_addr)) {
-		if (sec_addr->addr.s_addr
-		    == rp_info->rp.rpf_addr.u.prefix4.s_addr) {
+		if (prefix_same(&sec_addr->addr, &rp_info->rp.rpf_addr)) {
 			return 1;
 		}
 	}
@@ -245,13 +332,13 @@ static int pim_rp_check_interface_addrs(struct rp_info *rp_info,
 	return 0;
 }
 
-static void pim_rp_check_interfaces(struct rp_info *rp_info)
+static void pim_rp_check_interfaces(struct pim_instance *pim,
+				    struct rp_info *rp_info)
 {
-	struct listnode *node;
 	struct interface *ifp;
 
 	rp_info->i_am_rp = 0;
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp)) {
+	FOR_ALL_INTERFACES (pim->vrf, ifp) {
 		struct pim_interface *pim_ifp = ifp->info;
 
 		if (!pim_ifp)
@@ -263,7 +350,8 @@ static void pim_rp_check_interfaces(struct rp_info *rp_info)
 	}
 }
 
-int pim_rp_new(const char *rp, const char *group_range, const char *plist)
+int pim_rp_new(struct pim_instance *pim, const char *rp,
+	       const char *group_range, const char *plist)
 {
 	int result = 0;
 	struct rp_info *rp_info;
@@ -274,6 +362,7 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 	char buffer[BUFSIZ];
 	struct prefix nht_p;
 	struct pim_nexthop_cache pnc;
+	struct route_node *rn;
 
 	rp_info = XCALLOC(MTYPE_PIM_RP, sizeof(*rp_info));
 	if (!rp_info)
@@ -290,6 +379,7 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 	}
 
 	rp_info->rp.rpf_addr.family = AF_INET;
+	rp_info->rp.rpf_addr.prefixlen = IPV4_MAX_PREFIXLEN;
 	result = inet_pton(rp_info->rp.rpf_addr.family, rp,
 			   &rp_info->rp.rpf_addr.u.prefix4);
 
@@ -302,7 +392,7 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 		/*
 		 * Return if the prefix-list is already configured for this RP
 		 */
-		if (pim_rp_find_prefix_list(rp_info->rp.rpf_addr.u.prefix4,
+		if (pim_rp_find_prefix_list(pim, rp_info->rp.rpf_addr.u.prefix4,
 					    plist)) {
 			XFREE(MTYPE_PIM_RP, rp_info);
 			return PIM_SUCCESS;
@@ -311,7 +401,7 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 		/*
 		 * Barf if the prefix-list is already configured for an RP
 		 */
-		if (pim_rp_prefix_list_used(plist)) {
+		if (pim_rp_prefix_list_used(pim, plist)) {
 			XFREE(MTYPE_PIM_RP, rp_info);
 			return PIM_RP_PFXLIST_IN_USE;
 		}
@@ -319,16 +409,16 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 		/*
 		 * Free any existing rp_info entries for this RP
 		 */
-		for (ALL_LIST_ELEMENTS(qpim_rp_list, node, nnode,
+		for (ALL_LIST_ELEMENTS(pim->rp_list, node, nnode,
 				       tmp_rp_info)) {
 			if (rp_info->rp.rpf_addr.u.prefix4.s_addr
 			    == tmp_rp_info->rp.rpf_addr.u.prefix4.s_addr) {
 				if (tmp_rp_info->plist)
-					pim_rp_del(rp, NULL,
+					pim_rp_del(pim, rp, NULL,
 						   tmp_rp_info->plist);
 				else
 					pim_rp_del(
-						rp,
+						pim, rp,
 						prefix2str(&tmp_rp_info->group,
 							   buffer, BUFSIZ),
 						NULL);
@@ -337,8 +427,12 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 
 		rp_info->plist = XSTRDUP(MTYPE_PIM_FILTER_NAME, plist);
 	} else {
-		str2prefix("224.0.0.0/4", &group_all);
-		rp_all = pim_rp_find_match_group(&group_all);
+
+		if (!str2prefix("224.0.0.0/4", &group_all)) {
+			XFREE(MTYPE_PIM_RP, rp_info);
+			return PIM_GROUP_BAD_ADDRESS;
+		}
+		rp_all = pim_rp_find_match_group(pim, &group_all);
 
 		/*
 		 * Barf if group is a non-multicast subnet
@@ -351,13 +445,13 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 		/*
 		 * Remove any prefix-list rp_info entries for this RP
 		 */
-		for (ALL_LIST_ELEMENTS(qpim_rp_list, node, nnode,
+		for (ALL_LIST_ELEMENTS(pim->rp_list, node, nnode,
 				       tmp_rp_info)) {
 			if (tmp_rp_info->plist
 			    && rp_info->rp.rpf_addr.u.prefix4.s_addr
 				       == tmp_rp_info->rp.rpf_addr.u.prefix4
 						  .s_addr) {
-				pim_rp_del(rp, NULL, tmp_rp_info->plist);
+				pim_rp_del(pim, rp, NULL, tmp_rp_info->plist);
 			}
 		}
 
@@ -374,7 +468,7 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 			nht_p.prefixlen = IPV4_MAX_BITLEN;
 			nht_p.u.prefix4 =
 				rp_all->rp.rpf_addr.u.prefix4; // RP address
-			if (PIM_DEBUG_PIM_TRACE) {
+			if (PIM_DEBUG_PIM_NHT_RP) {
 				char buf[PREFIX2STR_BUFFER];
 				char buf1[PREFIX2STR_BUFFER];
 				prefix2str(&nht_p, buf, sizeof(buf));
@@ -384,31 +478,29 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 					__PRETTY_FUNCTION__, buf, buf1);
 			}
 			memset(&pnc, 0, sizeof(struct pim_nexthop_cache));
-			if ((pim_find_or_track_nexthop(&nht_p, NULL, rp_all,
-						       &pnc))
-			    == 1) {
-				// Compute PIM RPF using Cached nexthop
-				if ((pim_ecmp_nexthop_search(
-					    &pnc, &rp_all->rp.source_nexthop,
-					    &nht_p, &rp_all->group, 1))
-				    != 0)
+			if (pim_find_or_track_nexthop(pim, &nht_p, NULL, rp_all,
+						      &pnc)) {
+				if (!pim_ecmp_nexthop_search(
+					    pim, &pnc,
+					    &rp_all->rp.source_nexthop, &nht_p,
+					    &rp_all->group, 1))
 					return PIM_RP_NO_PATH;
 			} else {
 				if (pim_nexthop_lookup(
-					    &rp_all->rp.source_nexthop,
+					    pim, &rp_all->rp.source_nexthop,
 					    rp_all->rp.rpf_addr.u.prefix4, 1)
 				    != 0)
 					return PIM_RP_NO_PATH;
 			}
-			pim_rp_check_interfaces(rp_all);
-			pim_rp_refresh_group_to_rp_mapping();
+			pim_rp_check_interfaces(pim, rp_all);
+			pim_rp_refresh_group_to_rp_mapping(pim);
 			return PIM_SUCCESS;
 		}
 
 		/*
 		 * Return if the group is already configured for this RP
 		 */
-		if (pim_rp_find_exact(rp_info->rp.rpf_addr.u.prefix4,
+		if (pim_rp_find_exact(pim, rp_info->rp.rpf_addr.u.prefix4,
 				      &rp_info->group)) {
 			XFREE(MTYPE_PIM_RP, rp_info);
 			return PIM_SUCCESS;
@@ -417,7 +509,7 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 		/*
 		 * Barf if this group is already covered by some other RP
 		 */
-		tmp_rp_info = pim_rp_find_match_group(&rp_info->group);
+		tmp_rp_info = pim_rp_find_match_group(pim, &rp_info->group);
 
 		if (tmp_rp_info) {
 			if (tmp_rp_info->plist) {
@@ -431,8 +523,8 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 				 * For all others
 				 * though we must return PIM_GROUP_OVERLAP
 				 */
-				if (!prefix_same(&group_all,
-						 &tmp_rp_info->group)) {
+				if (prefix_same(&rp_info->group,
+						&tmp_rp_info->group)) {
 					XFREE(MTYPE_PIM_RP, rp_info);
 					return PIM_GROUP_OVERLAP;
 				}
@@ -440,13 +532,30 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 		}
 	}
 
-	listnode_add_sort(qpim_rp_list, rp_info);
+	listnode_add_sort(pim->rp_list, rp_info);
+	rn = route_node_get(pim->rp_table, &rp_info->group);
+	if (!rn) {
+		char buf[PREFIX_STRLEN];
+		zlog_err("Failure to get route node for pim->rp_table: %s",
+			 prefix2str(&rp_info->group, buf, sizeof(buf)));
+		return PIM_MALLOC_FAIL;
+	}
+	rn->info = rp_info;
+
+	if (PIM_DEBUG_TRACE) {
+		char buf[PREFIX_STRLEN];
+
+		zlog_debug("Allocated: %p for rp_info: %p(%s) Lock: %d", rn,
+			   rp_info,
+			   prefix2str(&rp_info->group, buf, sizeof(buf)),
+			   rn->lock);
+	}
 
 	/* Register addr with Zebra NHT */
 	nht_p.family = AF_INET;
 	nht_p.prefixlen = IPV4_MAX_BITLEN;
 	nht_p.u.prefix4 = rp_info->rp.rpf_addr.u.prefix4;
-	if (PIM_DEBUG_PIM_TRACE) {
+	if (PIM_DEBUG_PIM_NHT_RP) {
 		char buf[PREFIX2STR_BUFFER];
 		char buf1[PREFIX2STR_BUFFER];
 		prefix2str(&nht_p, buf, sizeof(buf));
@@ -456,25 +565,25 @@ int pim_rp_new(const char *rp, const char *group_range, const char *plist)
 	}
 
 	memset(&pnc, 0, sizeof(struct pim_nexthop_cache));
-	if ((pim_find_or_track_nexthop(&nht_p, NULL, rp_info, &pnc)) == 1) {
-		// Compute PIM RPF using Cached nexthop
-		if (pim_ecmp_nexthop_search(&pnc, &rp_info->rp.source_nexthop,
-					    &nht_p, &rp_info->group, 1)
-		    != 0)
+	if (pim_find_or_track_nexthop(pim, &nht_p, NULL, rp_info, &pnc)) {
+		if (!pim_ecmp_nexthop_search(pim, &pnc,
+					     &rp_info->rp.source_nexthop,
+					     &nht_p, &rp_info->group, 1))
 			return PIM_RP_NO_PATH;
 	} else {
-		if (pim_nexthop_lookup(&rp_info->rp.source_nexthop,
+		if (pim_nexthop_lookup(pim, &rp_info->rp.source_nexthop,
 				       rp_info->rp.rpf_addr.u.prefix4, 1)
 		    != 0)
 			return PIM_RP_NO_PATH;
 	}
 
-	pim_rp_check_interfaces(rp_info);
-	pim_rp_refresh_group_to_rp_mapping();
+	pim_rp_check_interfaces(pim, rp_info);
+	pim_rp_refresh_group_to_rp_mapping(pim);
 	return PIM_SUCCESS;
 }
 
-int pim_rp_del(const char *rp, const char *group_range, const char *plist)
+int pim_rp_del(struct pim_instance *pim, const char *rp,
+	       const char *group_range, const char *plist)
 {
 	struct prefix group;
 	struct in_addr rp_addr;
@@ -483,6 +592,8 @@ int pim_rp_del(const char *rp, const char *group_range, const char *plist)
 	struct rp_info *rp_all;
 	int result;
 	struct prefix nht_p;
+	struct route_node *rn;
+	bool was_plist = false;
 
 	if (group_range == NULL)
 		result = str2prefix("224.0.0.0/4", &group);
@@ -497,9 +608,9 @@ int pim_rp_del(const char *rp, const char *group_range, const char *plist)
 		return PIM_RP_BAD_ADDRESS;
 
 	if (plist)
-		rp_info = pim_rp_find_prefix_list(rp_addr, plist);
+		rp_info = pim_rp_find_prefix_list(pim, rp_addr, plist);
 	else
-		rp_info = pim_rp_find_exact(rp_addr, &group);
+		rp_info = pim_rp_find_exact(pim, rp_addr, &group);
 
 	if (!rp_info)
 		return PIM_RP_NOT_FOUND;
@@ -507,22 +618,25 @@ int pim_rp_del(const char *rp, const char *group_range, const char *plist)
 	if (rp_info->plist) {
 		XFREE(MTYPE_PIM_FILTER_NAME, rp_info->plist);
 		rp_info->plist = NULL;
+		was_plist = true;
 	}
 
 	/* Deregister addr with Zebra NHT */
 	nht_p.family = AF_INET;
 	nht_p.prefixlen = IPV4_MAX_BITLEN;
 	nht_p.u.prefix4 = rp_info->rp.rpf_addr.u.prefix4;
-	if (PIM_DEBUG_PIM_TRACE) {
+	if (PIM_DEBUG_PIM_NHT_RP) {
 		char buf[PREFIX2STR_BUFFER];
 		prefix2str(&nht_p, buf, sizeof(buf));
 		zlog_debug("%s: Deregister RP addr %s with Zebra ",
 			   __PRETTY_FUNCTION__, buf);
 	}
-	pim_delete_tracked_nexthop(&nht_p, NULL, rp_info);
+	pim_delete_tracked_nexthop(pim, &nht_p, NULL, rp_info);
 
-	str2prefix("224.0.0.0/4", &g_all);
-	rp_all = pim_rp_find_match_group(&g_all);
+	if (!str2prefix("224.0.0.0/4", &g_all))
+		return PIM_RP_BAD_ADDRESS;
+
+	rp_all = pim_rp_find_match_group(pim, &g_all);
 
 	if (rp_all == rp_info) {
 		rp_all->rp.rpf_addr.family = AF_INET;
@@ -531,20 +645,44 @@ int pim_rp_del(const char *rp, const char *group_range, const char *plist)
 		return PIM_SUCCESS;
 	}
 
-	listnode_delete(qpim_rp_list, rp_info);
-	pim_rp_refresh_group_to_rp_mapping();
+	listnode_delete(pim->rp_list, rp_info);
+
+	if (!was_plist) {
+		rn = route_node_get(pim->rp_table, &rp_info->group);
+		if (rn) {
+			if (rn->info != rp_info)
+				zlog_err("WTF matey");
+
+			if (PIM_DEBUG_TRACE) {
+				char buf[PREFIX_STRLEN];
+
+				zlog_debug(
+					"%s:Found for Freeing: %p for rp_info: %p(%s) Lock: %d",
+					__PRETTY_FUNCTION__, rn, rp_info,
+					prefix2str(&rp_info->group, buf,
+						   sizeof(buf)),
+					rn->lock);
+			}
+			rn->info = NULL;
+			route_unlock_node(rn);
+			route_unlock_node(rn);
+		}
+	}
+
+	pim_rp_refresh_group_to_rp_mapping(pim);
+
+	XFREE(MTYPE_PIM_RP, rp_info);
 	return PIM_SUCCESS;
 }
 
-int pim_rp_setup(void)
+void pim_rp_setup(struct pim_instance *pim)
 {
 	struct listnode *node;
 	struct rp_info *rp_info;
-	int ret = 0;
 	struct prefix nht_p;
 	struct pim_nexthop_cache pnc;
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (rp_info->rp.rpf_addr.u.prefix4.s_addr == INADDR_NONE)
 			continue;
 
@@ -552,38 +690,26 @@ int pim_rp_setup(void)
 		nht_p.prefixlen = IPV4_MAX_BITLEN;
 		nht_p.u.prefix4 = rp_info->rp.rpf_addr.u.prefix4;
 		memset(&pnc, 0, sizeof(struct pim_nexthop_cache));
-		if ((pim_find_or_track_nexthop(&nht_p, NULL, rp_info, &pnc))
-		    == 1) {
-			// Compute PIM RPF using Cached nexthop
-			if ((pim_ecmp_nexthop_search(
-				    &pnc, &rp_info->rp.source_nexthop, &nht_p,
-				    &rp_info->group, 1))
-			    != 0)
-				ret++;
-		} else {
-			if (PIM_DEBUG_ZEBRA) {
+		if (pim_find_or_track_nexthop(pim, &nht_p, NULL, rp_info, &pnc))
+			pim_ecmp_nexthop_search(pim, &pnc,
+						&rp_info->rp.source_nexthop,
+						&nht_p, &rp_info->group, 1);
+		else {
+			if (PIM_DEBUG_PIM_NHT_RP) {
 				char buf[PREFIX2STR_BUFFER];
 				prefix2str(&nht_p, buf, sizeof(buf));
 				zlog_debug(
 					"%s: NHT Local Nexthop not found for RP %s ",
 					__PRETTY_FUNCTION__, buf);
 			}
-			if (pim_nexthop_lookup(&rp_info->rp.source_nexthop,
-					       rp_info->rp.rpf_addr.u.prefix4,
-					       1)
-			    != 0) {
-				if (PIM_DEBUG_PIM_TRACE)
+			if (!pim_nexthop_lookup(
+				    pim, &rp_info->rp.source_nexthop,
+				    rp_info->rp.rpf_addr.u.prefix4, 1))
+				if (PIM_DEBUG_PIM_NHT_RP)
 					zlog_debug(
 						"Unable to lookup nexthop for rp specified");
-				ret++;
-			}
 		}
 	}
-
-	if (ret)
-		return 0;
-
-	return 1;
 }
 
 /*
@@ -595,11 +721,12 @@ void pim_rp_check_on_if_add(struct pim_interface *pim_ifp)
 	struct listnode *node;
 	struct rp_info *rp_info;
 	bool i_am_rp_changed = false;
+	struct pim_instance *pim = pim_ifp->pim;
 
-	if (qpim_rp_list == NULL)
+	if (pim->rp_list == NULL)
 		return;
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (pim_rpf_addr_is_inaddr_none(&rp_info->rp))
 			continue;
 
@@ -613,7 +740,7 @@ void pim_rp_check_on_if_add(struct pim_interface *pim_ifp)
 		if (pim_rp_check_interface_addrs(rp_info, pim_ifp)) {
 			i_am_rp_changed = true;
 			rp_info->i_am_rp = 1;
-			if (PIM_DEBUG_ZEBRA) {
+			if (PIM_DEBUG_PIM_NHT_RP) {
 				char rp[PREFIX_STRLEN];
 				pim_addr_dump("<rp?>", &rp_info->rp.rpf_addr,
 					      rp, sizeof(rp));
@@ -623,33 +750,33 @@ void pim_rp_check_on_if_add(struct pim_interface *pim_ifp)
 	}
 
 	if (i_am_rp_changed) {
-		pim_msdp_i_am_rp_changed();
+		pim_msdp_i_am_rp_changed(pim);
 	}
 }
 
 /* up-optimized re-evaluation of "i_am_rp". this is used when ifaddresses
  * are removed. Removing numbers is an uncommon event in an active network
  * so I have made no attempt to optimize it. */
-void pim_i_am_rp_re_evaluate(void)
+void pim_i_am_rp_re_evaluate(struct pim_instance *pim)
 {
 	struct listnode *node;
 	struct rp_info *rp_info;
 	bool i_am_rp_changed = false;
 	int old_i_am_rp;
 
-	if (qpim_rp_list == NULL)
+	if (pim->rp_list == NULL)
 		return;
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (pim_rpf_addr_is_inaddr_none(&rp_info->rp))
 			continue;
 
 		old_i_am_rp = rp_info->i_am_rp;
-		pim_rp_check_interfaces(rp_info);
+		pim_rp_check_interfaces(pim, rp_info);
 
 		if (old_i_am_rp != rp_info->i_am_rp) {
 			i_am_rp_changed = true;
-			if (PIM_DEBUG_ZEBRA) {
+			if (PIM_DEBUG_PIM_NHT_RP) {
 				char rp[PREFIX_STRLEN];
 				pim_addr_dump("<rp?>", &rp_info->rp.rpf_addr,
 					      rp, sizeof(rp));
@@ -665,7 +792,7 @@ void pim_i_am_rp_re_evaluate(void)
 	}
 
 	if (i_am_rp_changed) {
-		pim_msdp_i_am_rp_changed();
+		pim_msdp_i_am_rp_changed(pim);
 	}
 }
 
@@ -675,7 +802,7 @@ void pim_i_am_rp_re_evaluate(void)
  *
  * Since we only have static RP, all groups are part of this RP
  */
-int pim_rp_i_am_rp(struct in_addr group)
+int pim_rp_i_am_rp(struct pim_instance *pim, struct in_addr group)
 {
 	struct prefix g;
 	struct rp_info *rp_info;
@@ -685,7 +812,7 @@ int pim_rp_i_am_rp(struct in_addr group)
 	g.prefixlen = 32;
 	g.u.prefix4 = group;
 
-	rp_info = pim_rp_find_match_group(&g);
+	rp_info = pim_rp_find_match_group(pim, &g);
 
 	if (rp_info)
 		return rp_info->i_am_rp;
@@ -698,7 +825,7 @@ int pim_rp_i_am_rp(struct in_addr group)
  *
  * Return the RP that the Group belongs too.
  */
-struct pim_rpf *pim_rp_g(struct in_addr group)
+struct pim_rpf *pim_rp_g(struct pim_instance *pim, struct in_addr group)
 {
 	struct prefix g;
 	struct rp_info *rp_info;
@@ -708,7 +835,7 @@ struct pim_rpf *pim_rp_g(struct in_addr group)
 	g.prefixlen = 32;
 	g.u.prefix4 = group;
 
-	rp_info = pim_rp_find_match_group(&g);
+	rp_info = pim_rp_find_match_group(pim, &g);
 
 	if (rp_info) {
 		struct prefix nht_p;
@@ -717,7 +844,7 @@ struct pim_rpf *pim_rp_g(struct in_addr group)
 		nht_p.family = AF_INET;
 		nht_p.prefixlen = IPV4_MAX_BITLEN;
 		nht_p.u.prefix4 = rp_info->rp.rpf_addr.u.prefix4;
-		if (PIM_DEBUG_PIM_TRACE) {
+		if (PIM_DEBUG_PIM_NHT_RP) {
 			char buf[PREFIX2STR_BUFFER];
 			char buf1[PREFIX2STR_BUFFER];
 			prefix2str(&nht_p, buf, sizeof(buf));
@@ -727,14 +854,12 @@ struct pim_rpf *pim_rp_g(struct in_addr group)
 				__PRETTY_FUNCTION__, buf, buf1);
 		}
 		memset(&pnc, 0, sizeof(struct pim_nexthop_cache));
-		if ((pim_find_or_track_nexthop(&nht_p, NULL, rp_info, &pnc))
-		    == 1) {
-			// Compute PIM RPF using Cached nexthop
-			pim_ecmp_nexthop_search(&pnc,
+		if (pim_find_or_track_nexthop(pim, &nht_p, NULL, rp_info, &pnc))
+			pim_ecmp_nexthop_search(pim, &pnc,
 						&rp_info->rp.source_nexthop,
 						&nht_p, &rp_info->group, 1);
-		} else {
-			if (PIM_DEBUG_ZEBRA) {
+		else {
+			if (PIM_DEBUG_PIM_NHT_RP) {
 				char buf[PREFIX2STR_BUFFER];
 				char buf1[PREFIX2STR_BUFFER];
 				prefix2str(&nht_p, buf, sizeof(buf));
@@ -743,8 +868,8 @@ struct pim_rpf *pim_rp_g(struct in_addr group)
 					"%s: Nexthop cache not found for RP %s grp %s register with Zebra",
 					__PRETTY_FUNCTION__, buf, buf1);
 			}
-			pim_rpf_set_refresh_time();
-			pim_nexthop_lookup(&rp_info->rp.source_nexthop,
+			pim_rpf_set_refresh_time(pim);
+			pim_nexthop_lookup(pim, &rp_info->rp.source_nexthop,
 					   rp_info->rp.rpf_addr.u.prefix4, 1);
 		}
 		return (&rp_info->rp);
@@ -762,8 +887,8 @@ struct pim_rpf *pim_rp_g(struct in_addr group)
  * then return failure.
  *
  */
-int pim_rp_set_upstream_addr(struct in_addr *up, struct in_addr source,
-			     struct in_addr group)
+int pim_rp_set_upstream_addr(struct pim_instance *pim, struct in_addr *up,
+			     struct in_addr source, struct in_addr group)
 {
 	struct rp_info *rp_info;
 	struct prefix g;
@@ -773,11 +898,11 @@ int pim_rp_set_upstream_addr(struct in_addr *up, struct in_addr source,
 	g.prefixlen = 32;
 	g.u.prefix4 = group;
 
-	rp_info = pim_rp_find_match_group(&g);
+	rp_info = pim_rp_find_match_group(pim, &g);
 
 	if ((pim_rpf_addr_is_inaddr_none(&rp_info->rp))
 	    && (source.s_addr == INADDR_ANY)) {
-		if (PIM_DEBUG_PIM_TRACE)
+		if (PIM_DEBUG_PIM_NHT_RP)
 			zlog_debug("%s: Received a (*,G) with no RP configured",
 				   __PRETTY_FUNCTION__);
 		return 0;
@@ -789,7 +914,8 @@ int pim_rp_set_upstream_addr(struct in_addr *up, struct in_addr source,
 	return 1;
 }
 
-int pim_rp_config_write(struct vty *vty)
+int pim_rp_config_write(struct pim_instance *pim, struct vty *vty,
+			const char *spaces)
 {
 	struct listnode *node;
 	struct rp_info *rp_info;
@@ -797,30 +923,30 @@ int pim_rp_config_write(struct vty *vty)
 	char group_buffer[32];
 	int count = 0;
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (pim_rpf_addr_is_inaddr_none(&rp_info->rp))
 			continue;
 
 		if (rp_info->plist)
-			vty_out(vty, "ip pim rp %s prefix-list %s%s",
+			vty_out(vty, "%sip pim rp %s prefix-list %s\n", spaces,
 				inet_ntop(AF_INET,
 					  &rp_info->rp.rpf_addr.u.prefix4,
 					  rp_buffer, 32),
-				rp_info->plist, VTY_NEWLINE);
+				rp_info->plist);
 		else
-			vty_out(vty, "ip pim rp %s %s%s",
+			vty_out(vty, "%sip pim rp %s %s\n", spaces,
 				inet_ntop(AF_INET,
 					  &rp_info->rp.rpf_addr.u.prefix4,
 					  rp_buffer, 32),
-				prefix2str(&rp_info->group, group_buffer, 32),
-				VTY_NEWLINE);
+				prefix2str(&rp_info->group, group_buffer, 32));
 		count++;
 	}
 
 	return count;
 }
 
-int pim_rp_check_is_my_ip_address(struct in_addr group,
+int pim_rp_check_is_my_ip_address(struct pim_instance *pim,
+				  struct in_addr group,
 				  struct in_addr dest_addr)
 {
 	struct rp_info *rp_info;
@@ -831,25 +957,26 @@ int pim_rp_check_is_my_ip_address(struct in_addr group,
 	g.prefixlen = 32;
 	g.u.prefix4 = group;
 
-	rp_info = pim_rp_find_match_group(&g);
+	rp_info = pim_rp_find_match_group(pim, &g);
 	/*
 	 * See if we can short-cut some?
 	 * This might not make sense if we ever leave a static RP
 	 * type of configuration.
 	 * Note - Premature optimization might bite our patooeys' here.
 	 */
-	if (I_am_RP(group)) {
+	if (I_am_RP(pim, group)) {
 		if (dest_addr.s_addr == rp_info->rp.rpf_addr.u.prefix4.s_addr)
 			return 1;
 	}
 
-	if (if_lookup_exact_address(&dest_addr, AF_INET, VRF_DEFAULT))
+	if (if_lookup_exact_address(&dest_addr, AF_INET, pim->vrf_id))
 		return 1;
 
 	return 0;
 }
 
-void pim_rp_show_information(struct vty *vty, u_char uj)
+void pim_rp_show_information(struct pim_instance *pim, struct vty *vty,
+			     uint8_t uj)
 {
 	struct rp_info *rp_info;
 	struct rp_info *prev_rp_info = NULL;
@@ -863,10 +990,9 @@ void pim_rp_show_information(struct vty *vty, u_char uj)
 		json = json_object_new_object();
 	else
 		vty_out(vty,
-			"RP address       group/prefix-list   OIF         I am RP%s",
-			VTY_NEWLINE);
+			"RP address       group/prefix-list   OIF         I am RP\n");
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (!pim_rpf_addr_is_inaddr_none(&rp_info->rp)) {
 			char buf[48];
 
@@ -934,9 +1060,9 @@ void pim_rp_show_information(struct vty *vty, u_char uj)
 					vty_out(vty, "%-10s  ", "(Unknown)");
 
 				if (rp_info->i_am_rp)
-					vty_out(vty, "yes%s", VTY_NEWLINE);
+					vty_out(vty, "yes\n");
 				else
-					vty_out(vty, "no%s", VTY_NEWLINE);
+					vty_out(vty, "no\n");
 			}
 
 			prev_rp_info = rp_info;
@@ -950,14 +1076,13 @@ void pim_rp_show_information(struct vty *vty, u_char uj)
 				inet_ntoa(prev_rp_info->rp.rpf_addr.u.prefix4),
 				json_rp_rows);
 
-		vty_out(vty, "%s%s", json_object_to_json_string_ext(
-					     json, JSON_C_TO_STRING_PRETTY),
-			VTY_NEWLINE);
+		vty_out(vty, "%s\n", json_object_to_json_string_ext(
+					     json, JSON_C_TO_STRING_PRETTY));
 		json_object_free(json);
 	}
 }
 
-void pim_resolve_rp_nh(void)
+void pim_resolve_rp_nh(struct pim_instance *pim)
 {
 	struct listnode *node = NULL;
 	struct rp_info *rp_info = NULL;
@@ -966,7 +1091,7 @@ void pim_resolve_rp_nh(void)
 	struct pim_nexthop_cache pnc;
 	struct pim_neighbor *nbr = NULL;
 
-	for (ALL_LIST_ELEMENTS_RO(qpim_rp_list, node, rp_info)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->rp_list, node, rp_info)) {
 		if (rp_info->rp.rpf_addr.u.prefix4.s_addr == INADDR_NONE)
 			continue;
 
@@ -974,43 +1099,32 @@ void pim_resolve_rp_nh(void)
 		nht_p.prefixlen = IPV4_MAX_BITLEN;
 		nht_p.u.prefix4 = rp_info->rp.rpf_addr.u.prefix4;
 		memset(&pnc, 0, sizeof(struct pim_nexthop_cache));
-		if ((pim_find_or_track_nexthop(&nht_p, NULL, rp_info, &pnc))
-		    == 1) {
-			for (nh_node = pnc.nexthop; nh_node;
-			     nh_node = nh_node->next) {
-				if (nh_node->gate.ipv4.s_addr == 0) {
-					nbr = pim_neighbor_find_if(
-						if_lookup_by_index(
-							nh_node->ifindex,
-							VRF_DEFAULT));
-					if (nbr) {
-						nh_node->gate.ipv4 =
-							nbr->source_addr;
-						if (PIM_DEBUG_TRACE) {
-							char str[PREFIX_STRLEN];
-							char str1
-								[INET_ADDRSTRLEN];
-							struct interface *ifp1 =
-								if_lookup_by_index(
-									nh_node->ifindex,
-									VRF_DEFAULT);
-							pim_inet4_dump(
-								"<nht_nbr?>",
-								nbr->source_addr,
-								str1,
-								sizeof(str1));
-							pim_addr_dump(
-								"<nht_addr?>",
-								&nht_p, str,
-								sizeof(str));
-							zlog_debug(
-								"%s: addr %s new nexthop addr %s interface %s",
-								__PRETTY_FUNCTION__,
-								str, str1,
-								ifp1->name);
-						}
-					}
-				}
+		if (!pim_find_or_track_nexthop(pim, &nht_p, NULL, rp_info,
+					       &pnc))
+			continue;
+
+		for (nh_node = pnc.nexthop; nh_node; nh_node = nh_node->next) {
+			if (nh_node->gate.ipv4.s_addr != 0)
+				continue;
+
+			struct interface *ifp1 = if_lookup_by_index(
+				nh_node->ifindex, pim->vrf_id);
+			nbr = pim_neighbor_find_if(ifp1);
+			if (!nbr)
+				continue;
+
+			nh_node->gate.ipv4 = nbr->source_addr;
+			if (PIM_DEBUG_PIM_NHT_RP) {
+				char str[PREFIX_STRLEN];
+				char str1[INET_ADDRSTRLEN];
+				pim_inet4_dump("<nht_nbr?>", nbr->source_addr,
+					       str1, sizeof(str1));
+				pim_addr_dump("<nht_addr?>", &nht_p, str,
+					      sizeof(str));
+				zlog_debug(
+					"%s: addr %s new nexthop addr %s interface %s",
+					__PRETTY_FUNCTION__, str, str1,
+					ifp1->name);
 			}
 		}
 	}
