@@ -1,22 +1,21 @@
 /*
-  PIM for Quagga
-  Copyright (C) 2008  Everton da Silva Marques
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING; if not, write to the
-  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-  MA 02110-1301 USA
-*/
+ * PIM for Quagga
+ * Copyright (C) 2008  Everton da Silva Marques
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -88,6 +87,7 @@ uint8_t *pim_tlv_append_uint32(uint8_t *buf, const uint8_t *buf_pastend,
 }
 
 #define ucast_ipv4_encoding_len (2 + sizeof(struct in_addr))
+#define ucast_ipv6_encoding_len (2 + sizeof(struct in6_addr))
 
 /*
  * An Encoded-Unicast address takes the following format:
@@ -132,6 +132,14 @@ int pim_encode_addr_ucast(uint8_t *buf, struct prefix *p)
 		memcpy(buf, &p->u.prefix4, sizeof(struct in_addr));
 		return ucast_ipv4_encoding_len;
 		break;
+	case AF_INET6:
+		*(uint8_t *)buf = PIM_MSG_ADDRESS_FAMILY_IPV6;
+		++buf;
+		*(uint8_t *)buf = 0;
+		++buf;
+		memcpy(buf, &p->u.prefix6, sizeof(struct in6_addr));
+		return ucast_ipv6_encoding_len;
+		break;
 	default:
 		return 0;
 		break;
@@ -211,12 +219,12 @@ int pim_encode_addr_group(uint8_t *buf, afi_t afi, int bidir, int scope,
 }
 
 uint8_t *pim_tlv_append_addrlist_ucast(uint8_t *buf, const uint8_t *buf_pastend,
-				       struct list *ifconnected)
+				       struct list *ifconnected, int family)
 {
 	struct listnode *node;
 	uint16_t option_len = 0;
-
 	uint8_t *curr;
+	size_t uel;
 
 	node = listhead(ifconnected);
 
@@ -225,8 +233,10 @@ uint8_t *pim_tlv_append_addrlist_ucast(uint8_t *buf, const uint8_t *buf_pastend,
 		return buf;
 	}
 
-	/* Skip first address (primary) */
-	node = listnextnode(node);
+	if (family == AF_INET)
+		uel = ucast_ipv4_encoding_len;
+	else
+		uel = ucast_ipv6_encoding_len;
 
 	/* Scan secondary address list */
 	curr = buf + 4; /* skip T and L */
@@ -235,9 +245,15 @@ uint8_t *pim_tlv_append_addrlist_ucast(uint8_t *buf, const uint8_t *buf_pastend,
 		struct prefix *p = ifc->address;
 		int l_encode;
 
-		if ((curr + ucast_ipv4_encoding_len) > buf_pastend)
+		if (!CHECK_FLAG(ifc->flags, ZEBRA_IFA_SECONDARY))
+			continue;
+
+		if ((curr + uel) > buf_pastend)
 			return 0;
 
+		if (p->family != family)
+			continue;
+
 		l_encode = pim_encode_addr_ucast(curr, p);
 		curr += l_encode;
 		option_len += l_encode;
@@ -246,8 +262,7 @@ uint8_t *pim_tlv_append_addrlist_ucast(uint8_t *buf, const uint8_t *buf_pastend,
 	if (PIM_DEBUG_PIM_TRACE_DETAIL) {
 		zlog_debug(
 			"%s: number of encoded secondary unicast IPv4 addresses: %zu",
-			__PRETTY_FUNCTION__,
-			option_len / ucast_ipv4_encoding_len);
+			__PRETTY_FUNCTION__, option_len / uel);
 	}
 
 	if (option_len < 1) {
@@ -472,9 +487,24 @@ int pim_parse_addr_ucast(struct prefix *p, const uint8_t *buf, int buf_size)
 		p->family = AF_INET; /* notice: AF_INET !=
 					PIM_MSG_ADDRESS_FAMILY_IPV4 */
 		memcpy(&p->u.prefix4, addr, sizeof(struct in_addr));
-
+		p->prefixlen = IPV4_MAX_PREFIXLEN;
 		addr += sizeof(struct in_addr);
 
+		break;
+	case PIM_MSG_ADDRESS_FAMILY_IPV6:
+		if ((addr + sizeof(struct in6_addr)) > pastend) {
+			zlog_warn(
+				"%s: IPv6 unicast address overflow: left=%zd needed %zu",
+				__PRETTY_FUNCTION__, pastend - addr,
+				sizeof(struct in6_addr));
+			return -3;
+		}
+
+		p->family = AF_INET6;
+		p->prefixlen = IPV6_MAX_PREFIXLEN;
+		memcpy(&p->u.prefix6, addr, sizeof(struct in6_addr));
+		addr += sizeof(struct in6_addr);
+
 		break;
 	default: {
 		zlog_warn("%s: unknown unicast address encoding family=%d from",
@@ -627,7 +657,7 @@ int pim_parse_addr_source(struct prefix_sg *sg, uint8_t *flags,
 #define FREE_ADDR_LIST(hello_option_addr_list)                                 \
 	{                                                                      \
 		if (hello_option_addr_list) {                                  \
-			list_delete(hello_option_addr_list);                   \
+			list_delete_and_null(&hello_option_addr_list);         \
 			hello_option_addr_list = 0;                            \
 		}                                                              \
 	}
@@ -688,6 +718,8 @@ int pim_tlv_parse_addr_list(const char *ifname, struct in_addr src_addr,
 						: -1,
 					addr_str, src_str, ifname);
 			} break;
+			case AF_INET6:
+				break;
 			default: {
 				char src_str[INET_ADDRSTRLEN];
 				pim_inet4_dump("<src?>", src_addr, src_str,
@@ -747,8 +779,7 @@ int pim_tlv_parse_addr_list(const char *ifname, struct in_addr src_addr,
 				FREE_ADDR_LIST(*hello_option_addr_list);
 				return -3;
 			}
-			p->family = tmp.family;
-			p->u.prefix4 = tmp.u.prefix4;
+			prefix_copy(p, &tmp);
 			listnode_add(*hello_option_addr_list, p);
 		}
 
