@@ -1,22 +1,21 @@
 /*
-  PIM for Quagga
-  Copyright (C) 2008  Everton da Silva Marques
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING; if not, write to the
-  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-  MA 02110-1301 USA
-*/
+ * PIM for Quagga
+ * Copyright (C) 2008  Everton da Silva Marques
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -55,10 +54,6 @@
 #include "pim_nht.h"
 #include "pim_ssm.h"
 
-struct hash *pim_upstream_hash = NULL;
-struct list *pim_upstream_list = NULL;
-struct timer_wheel *pim_upstream_sg_wheel = NULL;
-
 static void join_timer_stop(struct pim_upstream *up);
 static void
 pim_upstream_update_assert_tracking_desired(struct pim_upstream *up);
@@ -68,7 +63,8 @@ pim_upstream_update_assert_tracking_desired(struct pim_upstream *up);
  * remove the parent pointer from
  * those pointing at us
  */
-static void pim_upstream_remove_children(struct pim_upstream *up)
+static void pim_upstream_remove_children(struct pim_instance *pim,
+					 struct pim_upstream *up)
 {
 	struct pim_upstream *child;
 
@@ -80,13 +76,13 @@ static void pim_upstream_remove_children(struct pim_upstream *up)
 		listnode_delete(up->sources, child);
 		if (PIM_UPSTREAM_FLAG_TEST_SRC_LHR(child->flags)) {
 			PIM_UPSTREAM_FLAG_UNSET_SRC_LHR(child->flags);
-			child = pim_upstream_del(child, __PRETTY_FUNCTION__);
+			child = pim_upstream_del(pim, child,
+						 __PRETTY_FUNCTION__);
 		}
 		if (child)
 			child->parent = NULL;
 	}
-	list_delete(up->sources);
-	up->sources = NULL;
+	list_delete_and_null(&up->sources);
 }
 
 /*
@@ -94,7 +90,8 @@ static void pim_upstream_remove_children(struct pim_upstream *up)
  * Find the children that would point
  * at us.
  */
-static void pim_upstream_find_new_children(struct pim_upstream *up)
+static void pim_upstream_find_new_children(struct pim_instance *pim,
+					   struct pim_upstream *up)
 {
 	struct pim_upstream *child;
 	struct listnode *ch_node;
@@ -107,7 +104,7 @@ static void pim_upstream_find_new_children(struct pim_upstream *up)
 	    && (up->sg.grp.s_addr == INADDR_ANY))
 		return;
 
-	for (ALL_LIST_ELEMENTS_RO(pim_upstream_list, ch_node, child)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->upstream_list, ch_node, child)) {
 		if ((up->sg.grp.s_addr != INADDR_ANY)
 		    && (child->sg.grp.s_addr == up->sg.grp.s_addr)
 		    && (child != up)) {
@@ -122,7 +119,8 @@ static void pim_upstream_find_new_children(struct pim_upstream *up)
  * If we have a (S,G), find the (*,G)
  * If we have a (*,G), find the (*,*)
  */
-static struct pim_upstream *pim_upstream_find_parent(struct pim_upstream *child)
+static struct pim_upstream *pim_upstream_find_parent(struct pim_instance *pim,
+						     struct pim_upstream *child)
 {
 	struct prefix_sg any = child->sg;
 	struct pim_upstream *up = NULL;
@@ -131,7 +129,7 @@ static struct pim_upstream *pim_upstream_find_parent(struct pim_upstream *child)
 	if ((child->sg.src.s_addr != INADDR_ANY)
 	    && (child->sg.grp.s_addr != INADDR_ANY)) {
 		any.src.s_addr = INADDR_ANY;
-		up = pim_upstream_find(&any);
+		up = pim_upstream_find(pim, &any);
 
 		if (up)
 			listnode_add(up->sources, child);
@@ -154,21 +152,24 @@ static void upstream_channel_oil_detach(struct pim_upstream *up)
 		/* Detaching from channel_oil, channel_oil may exist post del,
 		   but upstream would not keep reference of it
 		 */
+		up->channel_oil->up = NULL;
 		pim_channel_oil_del(up->channel_oil);
 		up->channel_oil = NULL;
 	}
 }
 
-struct pim_upstream *pim_upstream_del(struct pim_upstream *up, const char *name)
+struct pim_upstream *pim_upstream_del(struct pim_instance *pim,
+				      struct pim_upstream *up, const char *name)
 {
 	bool notify_msdp = false;
 	struct prefix nht_p;
 
 	if (PIM_DEBUG_TRACE)
 		zlog_debug(
-			"%s(%s): Delete %s ref count: %d , flags: %d c_oil ref count %d (Pre decrement)",
-			__PRETTY_FUNCTION__, name, up->sg_str, up->ref_count,
-			up->flags, up->channel_oil->oil_ref_count);
+			"%s(%s): Delete %s[%s] ref count: %d , flags: %d c_oil ref count %d (Pre decrement)",
+			__PRETTY_FUNCTION__, name, up->sg_str, pim->vrf->name,
+			up->ref_count, up->flags,
+			up->channel_oil->oil_ref_count);
 
 	--up->ref_count;
 
@@ -195,19 +196,18 @@ struct pim_upstream *pim_upstream_del(struct pim_upstream *up, const char *name)
 	up->rpf.source_nexthop.interface = NULL;
 
 	if (up->sg.src.s_addr != INADDR_ANY) {
-		wheel_remove_item(pim_upstream_sg_wheel, up);
+		wheel_remove_item(pim->upstream_sg_wheel, up);
 		notify_msdp = true;
 	}
 
-	pim_upstream_remove_children(up);
+	pim_upstream_remove_children(pim, up);
 	if (up->sources)
-		list_delete(up->sources);
-	up->sources = NULL;
+		list_delete_and_null(&up->sources);
+
 	pim_mroute_del(up->channel_oil, __PRETTY_FUNCTION__);
 	upstream_channel_oil_detach(up);
 
-	list_delete(up->ifchannels);
-	up->ifchannels = NULL;
+	list_delete_and_null(&up->ifchannels);
 
 	/*
 	  notice that listnode_delete() can't be moved
@@ -218,11 +218,11 @@ struct pim_upstream *pim_upstream_del(struct pim_upstream *up, const char *name)
 		listnode_delete(up->parent->sources, up);
 	up->parent = NULL;
 
-	listnode_delete(pim_upstream_list, up);
-	hash_release(pim_upstream_hash, up);
+	listnode_delete(pim->upstream_list, up);
+	hash_release(pim->upstream_hash, up);
 
 	if (notify_msdp) {
-		pim_msdp_up_del(&up->sg);
+		pim_msdp_up_del(pim, &up->sg);
 	}
 
 	/* Deregister addr with Zebra NHT */
@@ -235,7 +235,7 @@ struct pim_upstream *pim_upstream_del(struct pim_upstream *up, const char *name)
 		zlog_debug("%s: Deregister upstream %s addr %s with Zebra NHT",
 			   __PRETTY_FUNCTION__, up->sg_str, buf);
 	}
-	pim_delete_tracked_nexthop(&nht_p, up, NULL);
+	pim_delete_tracked_nexthop(pim, &nht_p, up, NULL);
 
 	pim_upstream_free(up);
 
@@ -269,8 +269,6 @@ static int on_join_timer(struct thread *t)
 
 	up = THREAD_ARG(t);
 
-	up->t_join_timer = NULL;
-
 	/*
 	 * In the case of a HFR we will not ahve anyone to send this to.
 	 */
@@ -325,8 +323,8 @@ void join_timer_start(struct pim_upstream *up)
 		pim_jp_agg_add_group(nbr->upstream_jp_agg, up, 1);
 	else {
 		THREAD_OFF(up->t_join_timer);
-		THREAD_TIMER_ON(master, up->t_join_timer, on_join_timer, up,
-				qpim_t_periodic);
+		thread_add_timer(master, on_join_timer, up, qpim_t_periodic,
+				 &up->t_join_timer);
 	}
 	pim_jp_agg_upstream_verification(up, true);
 }
@@ -354,8 +352,8 @@ static void pim_upstream_join_timer_restart_msec(struct pim_upstream *up,
 	}
 
 	THREAD_OFF(up->t_join_timer);
-	THREAD_TIMER_MSEC_ON(master, up->t_join_timer, on_join_timer, up,
-			     interval_msec);
+	thread_add_timer_msec(master, on_join_timer, up, interval_msec,
+			      &up->t_join_timer);
 }
 
 void pim_upstream_join_suppress(struct pim_upstream *up,
@@ -445,7 +443,7 @@ static void forward_off(struct pim_upstream *up)
 	/* scan per-interface (S,G) state */
 	for (ALL_LIST_ELEMENTS(up->ifchannels, chnode, chnextnode, ch)) {
 
-		pim_forward_stop(ch);
+		pim_forward_stop(ch, false);
 
 	} /* scan iface channel list */
 }
@@ -472,12 +470,12 @@ static int pim_upstream_could_register(struct pim_upstream *up)
 
 /* Source registration is supressed for SSM groups. When the SSM range changes
  * we re-revaluate register setup for existing upstream entries */
-void pim_upstream_register_reevaluate(void)
+void pim_upstream_register_reevaluate(struct pim_instance *pim)
 {
 	struct listnode *upnode;
 	struct pim_upstream *up;
 
-	for (ALL_LIST_ELEMENTS_RO(pim_upstream_list, upnode, up)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->upstream_list, upnode, up)) {
 		/* If FHR is set CouldRegister is True. Also check if the flow
 		 * is actually active; if it is not kat setup will trigger
 		 * source
@@ -485,7 +483,7 @@ void pim_upstream_register_reevaluate(void)
 		if (!PIM_UPSTREAM_FLAG_TEST_FHR(up->flags) || !up->t_ka_timer)
 			continue;
 
-		if (pim_is_grp_ssm(up->sg.grp)) {
+		if (pim_is_grp_ssm(pim, up->sg.grp)) {
 			/* clear the register state  for SSM groups */
 			if (up->reg_state != PIM_REG_NOINFO) {
 				if (PIM_DEBUG_PIM_EVENTS)
@@ -494,7 +492,7 @@ void pim_upstream_register_reevaluate(void)
 						up->sg_str);
 				/* remove regiface from the OIL if it is there*/
 				pim_channel_del_oif(up->channel_oil,
-						    pim_regiface,
+						    pim->regiface,
 						    PIM_OIF_FLAG_PROTO_PIM);
 				up->reg_state = PIM_REG_NOINFO;
 			}
@@ -506,7 +504,7 @@ void pim_upstream_register_reevaluate(void)
 						"Register %s as G is now ASM",
 						up->sg_str);
 				pim_channel_add_oif(up->channel_oil,
-						    pim_regiface,
+						    pim->regiface,
 						    PIM_OIF_FLAG_PROTO_PIM);
 				up->reg_state = PIM_REG_JOIN;
 			}
@@ -514,7 +512,7 @@ void pim_upstream_register_reevaluate(void)
 	}
 }
 
-void pim_upstream_switch(struct pim_upstream *up,
+void pim_upstream_switch(struct pim_instance *pim, struct pim_upstream *up,
 			 enum pim_upstream_state new_state)
 {
 	enum pim_upstream_state old_state = up->join_state;
@@ -536,14 +534,14 @@ void pim_upstream_switch(struct pim_upstream *up,
 		if (old_state != PIM_UPSTREAM_JOINED) {
 			int old_fhr = PIM_UPSTREAM_FLAG_TEST_FHR(up->flags);
 			forward_on(up);
-			pim_msdp_up_join_state_changed(up);
+			pim_msdp_up_join_state_changed(pim, up);
 			if (pim_upstream_could_register(up)) {
 				PIM_UPSTREAM_FLAG_SET_FHR(up->flags);
 				if (!old_fhr
 				    && PIM_UPSTREAM_FLAG_TEST_SRC_STREAM(
 					       up->flags)) {
 					pim_upstream_keep_alive_timer_start(
-						up, qpim_keep_alive_time);
+						up, pim->keep_alive_time);
 					pim_register_join(up);
 				}
 			} else {
@@ -557,13 +555,13 @@ void pim_upstream_switch(struct pim_upstream *up,
 
 		forward_off(up);
 		if (old_state == PIM_UPSTREAM_JOINED)
-			pim_msdp_up_join_state_changed(up);
+			pim_msdp_up_join_state_changed(pim, up);
 
 		/* IHR, Trigger SGRpt on *,G IIF to prune S,G from RPT towards
 		   RP.
 		   If I am RP for G then send S,G prune to its IIF. */
 		if (pim_upstream_is_sg_rpt(up) && up->parent
-		    && !I_am_RP(up->sg.grp)) {
+		    && !I_am_RP(pim, up->sg.grp)) {
 			if (PIM_DEBUG_PIM_TRACE_DETAIL)
 				zlog_debug(
 					"%s: *,G IIF %s S,G IIF %s ",
@@ -601,8 +599,11 @@ int pim_upstream_compare(void *arg1, void *arg2)
 	return 0;
 }
 
-static struct pim_upstream *
-pim_upstream_new(struct prefix_sg *sg, struct interface *incoming, int flags)
+static struct pim_upstream *pim_upstream_new(struct pim_instance *pim,
+					     struct prefix_sg *sg,
+					     struct interface *incoming,
+					     int flags,
+					     struct pim_ifchannel *ch)
 {
 	enum pim_rpf_result rpf_result;
 	struct pim_interface *pim_ifp;
@@ -617,25 +618,29 @@ pim_upstream_new(struct prefix_sg *sg, struct interface *incoming, int flags)
 
 	up->sg = *sg;
 	pim_str_sg_set(sg, up->sg_str);
-	up = hash_get(pim_upstream_hash, up, hash_alloc_intern);
-	if (!pim_rp_set_upstream_addr(&up->upstream_addr, sg->src, sg->grp)) {
+	if (ch)
+		ch->upstream = up;
+
+	up = hash_get(pim->upstream_hash, up, hash_alloc_intern);
+	if (!pim_rp_set_upstream_addr(pim, &up->upstream_addr, sg->src,
+				      sg->grp)) {
 		if (PIM_DEBUG_TRACE)
 			zlog_debug("%s: Received a (*,G) with no RP configured",
 				   __PRETTY_FUNCTION__);
 
-		hash_release(pim_upstream_hash, up);
+		hash_release(pim->upstream_hash, up);
 		XFREE(MTYPE_PIM_UPSTREAM, up);
 		return NULL;
 	}
 
-	up->parent = pim_upstream_find_parent(up);
+	up->parent = pim_upstream_find_parent(pim, up);
 	if (up->sg.src.s_addr == INADDR_ANY) {
 		up->sources = list_new();
 		up->sources->cmp = pim_upstream_compare;
 	} else
 		up->sources = NULL;
 
-	pim_upstream_find_new_children(up);
+	pim_upstream_find_new_children(pim, up);
 	up->flags = flags;
 	up->ref_count = 1;
 	up->t_join_timer = NULL;
@@ -663,9 +668,9 @@ pim_upstream_new(struct prefix_sg *sg, struct interface *incoming, int flags)
 	up->ifchannels->cmp = (int (*)(void *, void *))pim_ifchannel_compare;
 
 	if (up->sg.src.s_addr != INADDR_ANY)
-		wheel_add_item(pim_upstream_sg_wheel, up);
+		wheel_add_item(pim->upstream_sg_wheel, up);
 
-	rpf_result = pim_rpf_update(up, NULL, 1);
+	rpf_result = pim_rpf_update(pim, up, NULL, 1);
 	if (rpf_result == PIM_RPF_FAILURE) {
 		struct prefix nht_p;
 
@@ -677,7 +682,7 @@ pim_upstream_new(struct prefix_sg *sg, struct interface *incoming, int flags)
 		nht_p.family = AF_INET;
 		nht_p.prefixlen = IPV4_MAX_BITLEN;
 		nht_p.u.prefix4 = up->upstream_addr;
-		pim_delete_tracked_nexthop(&nht_p, up, NULL);
+		pim_delete_tracked_nexthop(pim, &nht_p, up, NULL);
 
 		if (up->parent) {
 			listnode_delete(up->parent->sources, up);
@@ -685,13 +690,15 @@ pim_upstream_new(struct prefix_sg *sg, struct interface *incoming, int flags)
 		}
 
 		if (up->sg.src.s_addr != INADDR_ANY)
-			wheel_remove_item(pim_upstream_sg_wheel, up);
+			wheel_remove_item(pim->upstream_sg_wheel, up);
 
-		pim_upstream_remove_children(up);
+		pim_upstream_remove_children(pim, up);
 		if (up->sources)
-			list_delete(up->sources);
+			list_delete_and_null(&up->sources);
 
-		hash_release(pim_upstream_hash, up);
+		list_delete_and_null(&up->ifchannels);
+
+		hash_release(pim->upstream_hash, up);
 		XFREE(MTYPE_PIM_UPSTREAM, up);
 		return NULL;
 	}
@@ -700,9 +707,9 @@ pim_upstream_new(struct prefix_sg *sg, struct interface *incoming, int flags)
 		pim_ifp = up->rpf.source_nexthop.interface->info;
 		if (pim_ifp)
 			up->channel_oil = pim_channel_oil_add(
-				&up->sg, pim_ifp->mroute_vif_index);
+				pim, &up->sg, pim_ifp->mroute_vif_index);
 	}
-	listnode_add_sort(pim_upstream_list, up);
+	listnode_add_sort(pim->upstream_list, up);
 
 	if (PIM_DEBUG_TRACE) {
 		zlog_debug(
@@ -714,13 +721,14 @@ pim_upstream_new(struct prefix_sg *sg, struct interface *incoming, int flags)
 	return up;
 }
 
-struct pim_upstream *pim_upstream_find(struct prefix_sg *sg)
+struct pim_upstream *pim_upstream_find(struct pim_instance *pim,
+				       struct prefix_sg *sg)
 {
 	struct pim_upstream lookup;
 	struct pim_upstream *up = NULL;
 
 	lookup.sg = *sg;
-	up = hash_lookup(pim_upstream_hash, &lookup);
+	up = hash_lookup(pim->upstream_hash, &lookup);
 	return up;
 }
 
@@ -729,8 +737,11 @@ struct pim_upstream *pim_upstream_find_or_add(struct prefix_sg *sg,
 					      int flags, const char *name)
 {
 	struct pim_upstream *up;
+	struct pim_interface *pim_ifp;
 
-	up = pim_upstream_find(sg);
+	pim_ifp = incoming->info;
+
+	up = pim_upstream_find(pim_ifp->pim, sg);
 
 	if (up) {
 		if (!(up->flags & flags)) {
@@ -743,7 +754,8 @@ struct pim_upstream *pim_upstream_find_or_add(struct prefix_sg *sg,
 					up->ref_count);
 		}
 	} else
-		up = pim_upstream_add(sg, incoming, flags, name);
+		up = pim_upstream_add(pim_ifp->pim, sg, incoming, flags, name,
+				      NULL);
 
 	return up;
 }
@@ -758,18 +770,21 @@ void pim_upstream_ref(struct pim_upstream *up, int flags, const char *name)
 			   up->ref_count);
 }
 
-struct pim_upstream *pim_upstream_add(struct prefix_sg *sg,
+struct pim_upstream *pim_upstream_add(struct pim_instance *pim,
+				      struct prefix_sg *sg,
 				      struct interface *incoming, int flags,
-				      const char *name)
+				      const char *name,
+				      struct pim_ifchannel *ch)
 {
 	struct pim_upstream *up = NULL;
 	int found = 0;
-	up = pim_upstream_find(sg);
+
+	up = pim_upstream_find(pim, sg);
 	if (up) {
 		pim_upstream_ref(up, flags, name);
 		found = 1;
 	} else {
-		up = pim_upstream_new(sg, incoming, flags);
+		up = pim_upstream_new(pim, sg, incoming, flags, ch);
 	}
 
 	if (PIM_DEBUG_TRACE) {
@@ -845,15 +860,15 @@ int pim_upstream_evaluate_join_desired_interface(struct pim_upstream *up,
 
   See also pim_upstream_update_join_desired() below.
  */
-int pim_upstream_evaluate_join_desired(struct pim_upstream *up)
+int pim_upstream_evaluate_join_desired(struct pim_instance *pim,
+				       struct pim_upstream *up)
 {
 	struct interface *ifp;
-	struct listnode *node;
 	struct pim_ifchannel *ch, *starch;
 	struct pim_upstream *starup = up->parent;
 	int ret = 0;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp)) {
+	FOR_ALL_INTERFACES (pim->vrf, ifp) {
 		if (!ifp->info)
 			continue;
 
@@ -877,14 +892,15 @@ int pim_upstream_evaluate_join_desired(struct pim_upstream *up)
 /*
   See also pim_upstream_evaluate_join_desired() above.
 */
-void pim_upstream_update_join_desired(struct pim_upstream *up)
+void pim_upstream_update_join_desired(struct pim_instance *pim,
+				      struct pim_upstream *up)
 {
 	int was_join_desired; /* boolean */
 	int is_join_desired;  /* boolean */
 
 	was_join_desired = PIM_UPSTREAM_FLAG_TEST_DR_JOIN_DESIRED(up->flags);
 
-	is_join_desired = pim_upstream_evaluate_join_desired(up);
+	is_join_desired = pim_upstream_evaluate_join_desired(pim, up);
 	if (is_join_desired)
 		PIM_UPSTREAM_FLAG_SET_DR_JOIN_DESIRED(up->flags);
 	else
@@ -892,13 +908,13 @@ void pim_upstream_update_join_desired(struct pim_upstream *up)
 
 	/* switched from false to true */
 	if (is_join_desired && !was_join_desired) {
-		pim_upstream_switch(up, PIM_UPSTREAM_JOINED);
+		pim_upstream_switch(pim, up, PIM_UPSTREAM_JOINED);
 		return;
 	}
 
 	/* switched from true to false */
 	if (!is_join_desired && was_join_desired) {
-		pim_upstream_switch(up, PIM_UPSTREAM_NOTJOINED);
+		pim_upstream_switch(pim, up, PIM_UPSTREAM_NOTJOINED);
 		return;
 	}
 }
@@ -912,7 +928,8 @@ void pim_upstream_update_join_desired(struct pim_upstream *up)
   Join Timer is set to expire in more than t_override seconds, reset
   it so that it expires after t_override seconds.
 */
-void pim_upstream_rpf_genid_changed(struct in_addr neigh_addr)
+void pim_upstream_rpf_genid_changed(struct pim_instance *pim,
+				    struct in_addr neigh_addr)
 {
 	struct listnode *up_node;
 	struct listnode *up_nextnode;
@@ -921,7 +938,7 @@ void pim_upstream_rpf_genid_changed(struct in_addr neigh_addr)
 	/*
 	 * Scan all (S,G) upstreams searching for RPF'(S,G)=neigh_addr
 	 */
-	for (ALL_LIST_ELEMENTS(pim_upstream_list, up_node, up_nextnode, up)) {
+	for (ALL_LIST_ELEMENTS(pim->upstream_list, up_node, up_nextnode, up)) {
 
 		if (PIM_DEBUG_TRACE) {
 			char neigh_str[INET_ADDRSTRLEN];
@@ -931,8 +948,9 @@ void pim_upstream_rpf_genid_changed(struct in_addr neigh_addr)
 			pim_addr_dump("<rpf?>", &up->rpf.rpf_addr, rpf_addr_str,
 				      sizeof(rpf_addr_str));
 			zlog_debug(
-				"%s: matching neigh=%s against upstream (S,G)=%s joined=%d rpf_addr=%s",
+				"%s: matching neigh=%s against upstream (S,G)=%s[%s] joined=%d rpf_addr=%s",
 				__PRETTY_FUNCTION__, neigh_str, up->sg_str,
+				pim->vrf->name,
 				up->join_state == PIM_UPSTREAM_JOINED,
 				rpf_addr_str);
 		}
@@ -1023,7 +1041,8 @@ static void pim_upstream_update_assert_tracking_desired(struct pim_upstream *up)
 /* When kat is stopped CouldRegister goes to false so we need to
  * transition  the (S, G) on FHR to NI state and remove reg tunnel
  * from the OIL */
-static void pim_upstream_fhr_kat_expiry(struct pim_upstream *up)
+static void pim_upstream_fhr_kat_expiry(struct pim_instance *pim,
+					struct pim_upstream *up)
 {
 	if (!PIM_UPSTREAM_FLAG_TEST_FHR(up->flags))
 		return;
@@ -1035,7 +1054,7 @@ static void pim_upstream_fhr_kat_expiry(struct pim_upstream *up)
 	/* stop reg-stop timer */
 	THREAD_OFF(up->t_rs_timer);
 	/* remove regiface from the OIL if it is there*/
-	pim_channel_del_oif(up->channel_oil, pim_regiface,
+	pim_channel_del_oif(up->channel_oil, pim->regiface,
 			    PIM_OIF_FLAG_PROTO_PIM);
 	/* clear the register state */
 	up->reg_state = PIM_REG_NOINFO;
@@ -1068,11 +1087,12 @@ static void pim_upstream_fhr_kat_start(struct pim_upstream *up)
 static int pim_upstream_keep_alive_timer(struct thread *t)
 {
 	struct pim_upstream *up;
+	struct pim_instance *pim;
 
 	up = THREAD_ARG(t);
-	up->t_ka_timer = NULL;
+	pim = up->channel_oil->pim;
 
-	if (I_am_RP(up->sg.grp)) {
+	if (I_am_RP(pim, up->sg.grp)) {
 		pim_br_clear_pmbr(&up->sg);
 		/*
 		 * We need to do more here :)
@@ -1081,19 +1101,27 @@ static int pim_upstream_keep_alive_timer(struct thread *t)
 	}
 
 	/* source is no longer active - pull the SA from MSDP's cache */
-	pim_msdp_sa_local_del(&up->sg);
+	pim_msdp_sa_local_del(pim, &up->sg);
 
 	/* if entry was created because of activity we need to deref it */
 	if (PIM_UPSTREAM_FLAG_TEST_SRC_STREAM(up->flags)) {
-		pim_upstream_fhr_kat_expiry(up);
+		pim_upstream_fhr_kat_expiry(pim, up);
 		if (PIM_DEBUG_TRACE)
-			zlog_debug("kat expired on %s; remove stream reference",
-				   up->sg_str);
+			zlog_debug(
+				"kat expired on %s[%s]; remove stream reference",
+				up->sg_str, pim->vrf->name);
 		PIM_UPSTREAM_FLAG_UNSET_SRC_STREAM(up->flags);
-		pim_upstream_del(up, __PRETTY_FUNCTION__);
+		pim_upstream_del(pim, up, __PRETTY_FUNCTION__);
 	} else if (PIM_UPSTREAM_FLAG_TEST_SRC_LHR(up->flags)) {
+		struct pim_upstream *parent = up->parent;
+
 		PIM_UPSTREAM_FLAG_UNSET_SRC_LHR(up->flags);
-		pim_upstream_del(up, __PRETTY_FUNCTION__);
+		pim_upstream_del(pim, up, __PRETTY_FUNCTION__);
+
+		if (parent) {
+			pim_jp_agg_single_upstream_send(&parent->rpf, parent,
+							true);
+		}
 	}
 
 	return 0;
@@ -1107,8 +1135,8 @@ void pim_upstream_keep_alive_timer_start(struct pim_upstream *up, uint32_t time)
 				   up->sg_str);
 	}
 	THREAD_OFF(up->t_ka_timer);
-	THREAD_TIMER_ON(master, up->t_ka_timer, pim_upstream_keep_alive_timer,
-			up, time);
+	thread_add_timer(master, pim_upstream_keep_alive_timer, up, time,
+			 &up->t_ka_timer);
 
 	/* any time keepalive is started against a SG we will have to
 	 * re-evaluate our active source database */
@@ -1118,21 +1146,18 @@ void pim_upstream_keep_alive_timer_start(struct pim_upstream *up, uint32_t time)
 /* MSDP on RP needs to know if a source is registerable to this RP */
 static int pim_upstream_msdp_reg_timer(struct thread *t)
 {
-	struct pim_upstream *up;
-
-	up = THREAD_ARG(t);
-	up->t_msdp_reg_timer = NULL;
+	struct pim_upstream *up = THREAD_ARG(t);
+	struct pim_instance *pim = up->channel_oil->pim;
 
 	/* source is no longer active - pull the SA from MSDP's cache */
-	pim_msdp_sa_local_del(&up->sg);
+	pim_msdp_sa_local_del(pim, &up->sg);
 	return 1;
 }
 void pim_upstream_msdp_reg_timer_start(struct pim_upstream *up)
 {
 	THREAD_OFF(up->t_msdp_reg_timer);
-	THREAD_TIMER_ON(master, up->t_msdp_reg_timer,
-			pim_upstream_msdp_reg_timer, up,
-			PIM_MSDP_REG_RXED_PERIOD);
+	thread_add_timer(master, pim_upstream_msdp_reg_timer, up,
+			 PIM_MSDP_REG_RXED_PERIOD, &up->t_msdp_reg_timer);
 
 	pim_msdp_sa_local_update(up);
 }
@@ -1165,9 +1190,10 @@ void pim_upstream_msdp_reg_timer_start(struct pim_upstream *up)
  *  SwitchToSptDesired(S,G) return true once a single packet has been
  *  received for the source and group.
  */
-int pim_upstream_switch_to_spt_desired(struct prefix_sg *sg)
+int pim_upstream_switch_to_spt_desired(struct pim_instance *pim,
+				       struct prefix_sg *sg)
 {
-	if (I_am_RP(sg->grp))
+	if (I_am_RP(pim, sg->grp))
 		return 1;
 
 	return 0;
@@ -1301,24 +1327,24 @@ const char *pim_reg_state2str(enum pim_reg_state reg_state, char *state_str)
 static int pim_upstream_register_stop_timer(struct thread *t)
 {
 	struct pim_interface *pim_ifp;
+	struct pim_instance *pim;
 	struct pim_upstream *up;
 	struct pim_rpf *rpg;
 	struct ip ip_hdr;
 	up = THREAD_ARG(t);
-
-	up->t_rs_timer = NULL;
+	pim = up->channel_oil->pim;
 
 	if (PIM_DEBUG_TRACE) {
 		char state_str[PIM_REG_STATE_STR_LEN];
-		zlog_debug("%s: (S,G)=%s upstream register stop timer %s",
-			   __PRETTY_FUNCTION__, up->sg_str,
+		zlog_debug("%s: (S,G)=%s[%s] upstream register stop timer %s",
+			   __PRETTY_FUNCTION__, up->sg_str, pim->vrf->name,
 			   pim_reg_state2str(up->reg_state, state_str));
 	}
 
 	switch (up->reg_state) {
 	case PIM_REG_JOIN_PENDING:
 		up->reg_state = PIM_REG_JOIN;
-		pim_channel_add_oif(up->channel_oil, pim_regiface,
+		pim_channel_add_oif(up->channel_oil, pim->regiface,
 				    PIM_OIF_FLAG_PROTO_PIM);
 		break;
 	case PIM_REG_JOIN:
@@ -1337,15 +1363,22 @@ static int pim_upstream_register_stop_timer(struct thread *t)
 		pim_upstream_start_register_stop_timer(up, 1);
 
 		if (((up->channel_oil->cc.lastused / 100)
-		     > PIM_KEEPALIVE_PERIOD)
-		    && (I_am_RP(up->sg.grp))) {
+		     > pim->keep_alive_time)
+		    && (I_am_RP(pim_ifp->pim, up->sg.grp))) {
 			if (PIM_DEBUG_TRACE)
 				zlog_debug(
 					"%s: Stop sending the register, because I am the RP and we haven't seen a packet in a while",
 					__PRETTY_FUNCTION__);
 			return 0;
 		}
-		rpg = RP(up->sg.grp);
+		rpg = RP(pim_ifp->pim, up->sg.grp);
+		if (!rpg) {
+			if (PIM_DEBUG_TRACE)
+				zlog_debug(
+					"%s: Cannot send register for %s no RPF to the RP",
+					__PRETTY_FUNCTION__, up->sg_str);
+			return 0;
+		}
 		memset(&ip_hdr, 0, sizeof(struct ip));
 		ip_hdr.ip_p = PIM_IP_PROTO_PIM;
 		ip_hdr.ip_hl = 5;
@@ -1369,10 +1402,7 @@ void pim_upstream_start_register_stop_timer(struct pim_upstream *up,
 {
 	uint32_t time;
 
-	if (up->t_rs_timer) {
-		THREAD_TIMER_OFF(up->t_rs_timer);
-		up->t_rs_timer = NULL;
-	}
+	THREAD_TIMER_OFF(up->t_rs_timer);
 
 	if (!null_register) {
 		uint32_t lower = (0.5 * PIM_REGISTER_SUPPRESSION_PERIOD);
@@ -1387,16 +1417,16 @@ void pim_upstream_start_register_stop_timer(struct pim_upstream *up,
 			"%s: (S,G)=%s Starting upstream register stop timer %d",
 			__PRETTY_FUNCTION__, up->sg_str, time);
 	}
-	THREAD_TIMER_ON(master, up->t_rs_timer,
-			pim_upstream_register_stop_timer, up, time);
+	thread_add_timer(master, pim_upstream_register_stop_timer, up, time,
+			 &up->t_rs_timer);
 }
 
-int pim_upstream_inherited_olist_decide(struct pim_upstream *up)
+int pim_upstream_inherited_olist_decide(struct pim_instance *pim,
+					struct pim_upstream *up)
 {
 	struct interface *ifp;
 	struct pim_interface *pim_ifp = NULL;
 	struct pim_ifchannel *ch, *starch;
-	struct listnode *node;
 	struct pim_upstream *starup = up->parent;
 	int output_intf = 0;
 
@@ -1408,10 +1438,10 @@ int pim_upstream_inherited_olist_decide(struct pim_upstream *up)
 				   __PRETTY_FUNCTION__, up->sg_str);
 	}
 	if (pim_ifp && !up->channel_oil)
-		up->channel_oil =
-			pim_channel_oil_add(&up->sg, pim_ifp->mroute_vif_index);
+		up->channel_oil = pim_channel_oil_add(
+			pim, &up->sg, pim_ifp->mroute_vif_index);
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp)) {
+	FOR_ALL_INTERFACES (pim->vrf, ifp) {
 		if (!ifp->info)
 			continue;
 
@@ -1456,9 +1486,10 @@ int pim_upstream_inherited_olist_decide(struct pim_upstream *up)
  * return 1 if there are any output interfaces
  * return 0 if there are not any output interfaces
  */
-int pim_upstream_inherited_olist(struct pim_upstream *up)
+int pim_upstream_inherited_olist(struct pim_instance *pim,
+				 struct pim_upstream *up)
 {
-	int output_intf = pim_upstream_inherited_olist_decide(up);
+	int output_intf = pim_upstream_inherited_olist_decide(pim, up);
 
 	/*
 	 * If we have output_intf switch state to Join and work like normal
@@ -1467,7 +1498,7 @@ int pim_upstream_inherited_olist(struct pim_upstream *up)
 	 * incoming packets so we don't bother the other stuff!
 	 */
 	if (output_intf)
-		pim_upstream_switch(up, PIM_UPSTREAM_JOINED);
+		pim_upstream_switch(pim, up, PIM_UPSTREAM_JOINED);
 	else
 		forward_on(up);
 
@@ -1485,7 +1516,7 @@ int pim_upstream_empty_inherited_olist(struct pim_upstream *up)
  * set and see if the new neighbor allows
  * the join to be sent
  */
-void pim_upstream_find_new_rpf(void)
+void pim_upstream_find_new_rpf(struct pim_instance *pim)
 {
 	struct listnode *up_node;
 	struct listnode *up_nextnode;
@@ -1494,36 +1525,35 @@ void pim_upstream_find_new_rpf(void)
 	/*
 	 * Scan all (S,G) upstreams searching for RPF'(S,G)=neigh_addr
 	 */
-	for (ALL_LIST_ELEMENTS(pim_upstream_list, up_node, up_nextnode, up)) {
+	for (ALL_LIST_ELEMENTS(pim->upstream_list, up_node, up_nextnode, up)) {
 		if (pim_rpf_addr_is_inaddr_any(&up->rpf)) {
 			if (PIM_DEBUG_TRACE)
 				zlog_debug(
 					"Upstream %s without a path to send join, checking",
 					up->sg_str);
-			pim_rpf_update(up, NULL, 1);
+			pim_rpf_update(pim, up, NULL, 1);
 		}
 	}
 }
 
-static unsigned int pim_upstream_hash_key(void *arg)
+unsigned int pim_upstream_hash_key(void *arg)
 {
 	struct pim_upstream *up = (struct pim_upstream *)arg;
 
 	return jhash_2words(up->sg.src.s_addr, up->sg.grp.s_addr, 0);
 }
 
-void pim_upstream_terminate(void)
+void pim_upstream_terminate(struct pim_instance *pim)
 {
-	if (pim_upstream_list)
-		list_delete(pim_upstream_list);
-	pim_upstream_list = NULL;
+	if (pim->upstream_list)
+		list_delete_and_null(&pim->upstream_list);
 
-	if (pim_upstream_hash)
-		hash_free(pim_upstream_hash);
-	pim_upstream_hash = NULL;
+	if (pim->upstream_hash)
+		hash_free(pim->upstream_hash);
+	pim->upstream_hash = NULL;
 }
 
-static int pim_upstream_equal(const void *arg1, const void *arg2)
+int pim_upstream_equal(const void *arg1, const void *arg2)
 {
 	const struct pim_upstream *up1 = (const struct pim_upstream *)arg1;
 	const struct pim_upstream *up2 = (const struct pim_upstream *)arg2;
@@ -1551,6 +1581,8 @@ static int pim_upstream_equal(const void *arg1, const void *arg2)
  */
 static bool pim_upstream_kat_start_ok(struct pim_upstream *up)
 {
+	struct pim_instance *pim = up->channel_oil->pim;
+
 	/* "iif == RPF_interface(S)" check has to be done by the kernel or hw
 	 * so we will skip that here */
 	if (pim_if_connected_to_source(up->rpf.source_nexthop.interface,
@@ -1570,7 +1602,7 @@ static bool pim_upstream_kat_start_ok(struct pim_upstream *up)
 		 * MUST be
 		 * removed to handle spt turn-arounds correctly in a 3-tier clos
 		 */
-		if (I_am_RP(up->sg.grp))
+		if (I_am_RP(pim, up->sg.grp))
 			return true;
 	}
 
@@ -1584,12 +1616,14 @@ static bool pim_upstream_kat_start_ok(struct pim_upstream *up)
 static void pim_upstream_sg_running(void *arg)
 {
 	struct pim_upstream *up = (struct pim_upstream *)arg;
+	struct pim_instance *pim = up->channel_oil->pim;
 
 	// No packet can have arrived here if this is the case
-	if (!up->channel_oil || !up->channel_oil->installed) {
+	if (!up->channel_oil->installed) {
 		if (PIM_DEBUG_TRACE)
-			zlog_debug("%s: %s is not installed in mroute",
-				   __PRETTY_FUNCTION__, up->sg_str);
+			zlog_debug("%s: %s[%s] is not installed in mroute",
+				   __PRETTY_FUNCTION__, up->sg_str,
+				   pim->vrf->name);
 		return;
 	}
 
@@ -1604,9 +1638,10 @@ static void pim_upstream_sg_running(void *arg)
 	if (up->channel_oil->oil_inherited_rescan) {
 		if (PIM_DEBUG_TRACE)
 			zlog_debug(
-				"%s: Handling unscanned inherited_olist for %s",
-				__PRETTY_FUNCTION__, up->sg_str);
-		pim_upstream_inherited_olist_decide(up);
+				"%s: Handling unscanned inherited_olist for %s[%s]",
+				__PRETTY_FUNCTION__, up->sg_str,
+				pim->vrf->name);
+		pim_upstream_inherited_olist_decide(pim, up);
 		up->channel_oil->oil_inherited_rescan = 0;
 	}
 	pim_mroute_update_counters(up->channel_oil);
@@ -1616,8 +1651,8 @@ static void pim_upstream_sg_running(void *arg)
 	    && (up->channel_oil->cc.lastused / 100 > 30)) {
 		if (PIM_DEBUG_TRACE) {
 			zlog_debug(
-				"%s: %s old packet count is equal or lastused is greater than 30, (%ld,%ld,%lld)",
-				__PRETTY_FUNCTION__, up->sg_str,
+				"%s[%s]: %s old packet count is equal or lastused is greater than 30, (%ld,%ld,%lld)",
+				__PRETTY_FUNCTION__, up->sg_str, pim->vrf->name,
 				up->channel_oil->cc.oldpktcnt,
 				up->channel_oil->cc.pktcnt,
 				up->channel_oil->cc.lastused / 100);
@@ -1631,17 +1666,17 @@ static void pim_upstream_sg_running(void *arg)
 		if (!PIM_UPSTREAM_FLAG_TEST_SRC_STREAM(up->flags)) {
 			if (PIM_DEBUG_TRACE)
 				zlog_debug(
-					"source reference created on kat restart %s",
-					up->sg_str);
+					"source reference created on kat restart %s[%s]",
+					up->sg_str, pim->vrf->name);
 
 			pim_upstream_ref(up, PIM_UPSTREAM_FLAG_MASK_SRC_STREAM,
 					 __PRETTY_FUNCTION__);
 			PIM_UPSTREAM_FLAG_SET_SRC_STREAM(up->flags);
 			pim_upstream_fhr_kat_start(up);
 		}
-		pim_upstream_keep_alive_timer_start(up, qpim_keep_alive_time);
+		pim_upstream_keep_alive_timer_start(up, pim->keep_alive_time);
 	} else if (PIM_UPSTREAM_FLAG_TEST_SRC_LHR(up->flags))
-		pim_upstream_keep_alive_timer_start(up, qpim_keep_alive_time);
+		pim_upstream_keep_alive_timer_start(up, pim->keep_alive_time);
 
 	if (up->sptbit != PIM_UPSTREAM_SPTBIT_TRUE) {
 		pim_upstream_set_sptbit(up, up->rpf.source_nexthop.interface);
@@ -1649,29 +1684,30 @@ static void pim_upstream_sg_running(void *arg)
 	return;
 }
 
-void pim_upstream_add_lhr_star_pimreg(void)
+void pim_upstream_add_lhr_star_pimreg(struct pim_instance *pim)
 {
 	struct pim_upstream *up;
 	struct listnode *node;
 
-	for (ALL_LIST_ELEMENTS_RO(pim_upstream_list, node, up)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->upstream_list, node, up)) {
 		if (up->sg.src.s_addr != INADDR_ANY)
 			continue;
 
 		if (!PIM_UPSTREAM_FLAG_TEST_SRC_IGMP(up->flags))
 			continue;
 
-		pim_channel_add_oif(up->channel_oil, pim_regiface,
+		pim_channel_add_oif(up->channel_oil, pim->regiface,
 				    PIM_OIF_FLAG_PROTO_IGMP);
 	}
 }
 
-void pim_upstream_spt_prefix_list_update(struct prefix_list *pl)
+void pim_upstream_spt_prefix_list_update(struct pim_instance *pim,
+					 struct prefix_list *pl)
 {
 	const char *pname = prefix_list_name(pl);
 
-	if (pimg->spt.plist && strcmp(pimg->spt.plist, pname) == 0) {
-		pim_upstream_remove_lhr_star_pimreg(pname);
+	if (pim->spt.plist && strcmp(pim->spt.plist, pname) == 0) {
+		pim_upstream_remove_lhr_star_pimreg(pim, pname);
 	}
 }
 
@@ -1687,7 +1723,8 @@ void pim_upstream_spt_prefix_list_update(struct prefix_list *pl)
  * the interface
  *
  */
-void pim_upstream_remove_lhr_star_pimreg(const char *nlist)
+void pim_upstream_remove_lhr_star_pimreg(struct pim_instance *pim,
+					 const char *nlist)
 {
 	struct pim_upstream *up;
 	struct listnode *node;
@@ -1700,7 +1737,7 @@ void pim_upstream_remove_lhr_star_pimreg(const char *nlist)
 	g.family = AF_INET;
 	g.prefixlen = IPV4_MAX_PREFIXLEN;
 
-	for (ALL_LIST_ELEMENTS_RO(pim_upstream_list, node, up)) {
+	for (ALL_LIST_ELEMENTS_RO(pim->upstream_list, node, up)) {
 		if (up->sg.src.s_addr != INADDR_ANY)
 			continue;
 
@@ -1708,30 +1745,34 @@ void pim_upstream_remove_lhr_star_pimreg(const char *nlist)
 			continue;
 
 		if (!nlist) {
-			pim_channel_del_oif(up->channel_oil, pim_regiface,
+			pim_channel_del_oif(up->channel_oil, pim->regiface,
 					    PIM_OIF_FLAG_PROTO_IGMP);
 			continue;
 		}
 		g.u.prefix4 = up->sg.grp;
 		apply_new = prefix_list_apply(np, &g);
 		if (apply_new == PREFIX_DENY)
-			pim_channel_add_oif(up->channel_oil, pim_regiface,
+			pim_channel_add_oif(up->channel_oil, pim->regiface,
 					    PIM_OIF_FLAG_PROTO_IGMP);
 		else
-			pim_channel_del_oif(up->channel_oil, pim_regiface,
+			pim_channel_del_oif(up->channel_oil, pim->regiface,
 					    PIM_OIF_FLAG_PROTO_IGMP);
 	}
 }
 
-void pim_upstream_init(void)
+void pim_upstream_init(struct pim_instance *pim)
 {
-	pim_upstream_sg_wheel =
+	char hash_name[64];
+
+	pim->upstream_sg_wheel =
 		wheel_init(master, 31000, 100, pim_upstream_hash_key,
 			   pim_upstream_sg_running);
-	pim_upstream_hash = hash_create_size(8192, pim_upstream_hash_key,
-					     pim_upstream_equal);
 
-	pim_upstream_list = list_new();
-	pim_upstream_list->del = (void (*)(void *))pim_upstream_free;
-	pim_upstream_list->cmp = pim_upstream_compare;
+	snprintf(hash_name, 64, "PIM %s Upstream Hash", pim->vrf->name);
+	pim->upstream_hash = hash_create_size(8192, pim_upstream_hash_key,
+					      pim_upstream_equal, hash_name);
+
+	pim->upstream_list = list_new();
+	pim->upstream_list->del = (void (*)(void *))pim_upstream_free;
+	pim->upstream_list->cmp = pim_upstream_compare;
 }
