@@ -1,22 +1,21 @@
 /*
-  PIM for Quagga
-  Copyright (C) 2008  Everton da Silva Marques
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; see the file COPYING; if not, write to the
-  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-  MA 02110-1301 USA
-*/
+ * PIM for Quagga
+ * Copyright (C) 2008  Everton da Silva Marques
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
 
 #include <zebra.h>
 
@@ -30,6 +29,7 @@
 #include "network.h"
 #include "vty.h"
 #include "plist.h"
+#include "lib/bfd.h"
 
 #include "pimd.h"
 #include "pim_pim.h"
@@ -80,8 +80,8 @@ static int pim_zebra_if_add(int command, struct zclient *zclient,
 
 	if (PIM_DEBUG_ZEBRA) {
 		zlog_debug(
-			"%s: %s index %d flags %ld metric %d mtu %d operative %d",
-			__PRETTY_FUNCTION__, ifp->name, ifp->ifindex,
+			"%s: %s index %d(%u) flags %ld metric %d mtu %d operative %d",
+			__PRETTY_FUNCTION__, ifp->name, ifp->ifindex, vrf_id,
 			(long)ifp->flags, ifp->metric, ifp->mtu,
 			if_is_operative(ifp));
 	}
@@ -89,6 +89,23 @@ static int pim_zebra_if_add(int command, struct zclient *zclient,
 	if (if_is_operative(ifp))
 		pim_if_addr_add_all(ifp);
 
+	/*
+	 * If we are a vrf device that is up, open up the pim_socket for
+	 * listening
+	 * to incoming pim messages irrelevant if the user has configured us
+	 * for pim or not.
+	 */
+	if (pim_if_is_vrf_device(ifp)) {
+		struct pim_interface *pim_ifp;
+
+		if (!ifp->info) {
+			pim_ifp = pim_if_new(ifp, 0, 0, false);
+			ifp->info = pim_ifp;
+		}
+
+		pim_sock_add(ifp);
+	}
+
 	return 0;
 }
 
@@ -113,8 +130,8 @@ static int pim_zebra_if_del(int command, struct zclient *zclient,
 
 	if (PIM_DEBUG_ZEBRA) {
 		zlog_debug(
-			"%s: %s index %d flags %ld metric %d mtu %d operative %d",
-			__PRETTY_FUNCTION__, ifp->name, ifp->ifindex,
+			"%s: %s index %d(%u) flags %ld metric %d mtu %d operative %d",
+			__PRETTY_FUNCTION__, ifp->name, ifp->ifindex, vrf_id,
 			(long)ifp->flags, ifp->metric, ifp->mtu,
 			if_is_operative(ifp));
 	}
@@ -129,6 +146,7 @@ static int pim_zebra_if_state_up(int command, struct zclient *zclient,
 				 zebra_size_t length, vrf_id_t vrf_id)
 {
 	struct interface *ifp;
+	uint32_t table_id;
 
 	/*
 	  zebra api notifies interface up/down events by using the same call
@@ -140,8 +158,8 @@ static int pim_zebra_if_state_up(int command, struct zclient *zclient,
 
 	if (PIM_DEBUG_ZEBRA) {
 		zlog_debug(
-			"%s: %s index %d flags %ld metric %d mtu %d operative %d",
-			__PRETTY_FUNCTION__, ifp->name, ifp->ifindex,
+			"%s: %s index %d(%u) flags %ld metric %d mtu %d operative %d",
+			__PRETTY_FUNCTION__, ifp->name, ifp->ifindex, vrf_id,
 			(long)ifp->flags, ifp->metric, ifp->mtu,
 			if_is_operative(ifp));
 	}
@@ -154,6 +172,29 @@ static int pim_zebra_if_state_up(int command, struct zclient *zclient,
 		pim_if_addr_add_all(ifp);
 	}
 
+	/*
+	 * If we have a pimreg device callback and it's for a specific
+	 * table set the master appropriately
+	 */
+	if (sscanf(ifp->name, "pimreg%d", &table_id) == 1) {
+		struct vrf *vrf;
+		RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name) {
+			if ((table_id == vrf->data.l.table_id)
+			    && (ifp->vrf_id != vrf->vrf_id)) {
+				struct interface *master = if_lookup_by_name(
+					vrf->name, vrf->vrf_id);
+
+				if (!master) {
+					zlog_debug(
+						"%s: Unable to find Master interface for %s",
+						__PRETTY_FUNCTION__, vrf->name);
+					return 0;
+				}
+				zclient_interface_set_master(zclient, master,
+							     ifp);
+			}
+		}
+	}
 	return 0;
 }
 
@@ -172,8 +213,8 @@ static int pim_zebra_if_state_down(int command, struct zclient *zclient,
 
 	if (PIM_DEBUG_ZEBRA) {
 		zlog_debug(
-			"%s: %s index %d flags %ld metric %d mtu %d operative %d",
-			__PRETTY_FUNCTION__, ifp->name, ifp->ifindex,
+			"%s: %s index %d(%u) flags %ld metric %d mtu %d operative %d",
+			__PRETTY_FUNCTION__, ifp->name, ifp->ifindex, vrf_id,
 			(long)ifp->flags, ifp->metric, ifp->mtu,
 			if_is_operative(ifp));
 	}
@@ -252,8 +293,9 @@ static int pim_zebra_if_address_add(int command, struct zclient *zclient,
 	if (PIM_DEBUG_ZEBRA) {
 		char buf[BUFSIZ];
 		prefix2str(p, buf, BUFSIZ);
-		zlog_debug("%s: %s connected IP address %s flags %u %s",
-			   __PRETTY_FUNCTION__, c->ifp->name, buf, c->flags,
+		zlog_debug("%s: %s(%u) connected IP address %s flags %u %s",
+			   __PRETTY_FUNCTION__, c->ifp->name, vrf_id, buf,
+			   c->flags,
 			   CHECK_FLAG(c->flags, ZEBRA_IFA_SECONDARY)
 				   ? "secondary"
 				   : "primary");
@@ -263,29 +305,12 @@ static int pim_zebra_if_address_add(int command, struct zclient *zclient,
 #endif
 	}
 
-	if (p->family != AF_INET) {
-		struct listnode *cnode;
-		struct connected *conn;
-		int v4addrs = 0;
-
-		for (ALL_LIST_ELEMENTS_RO(c->ifp->connected, cnode, conn)) {
-			if (conn->address->family == AF_INET)
-				v4addrs++;
-		}
-		if (!v4addrs && pim_ifp) {
-			pim_ifp->primary_address =
-				pim_find_primary_addr(c->ifp);
-			pim_if_addr_add_all(c->ifp);
-			pim_if_add_vif(c->ifp);
-		}
-		return 0;
-	}
-
 	if (!CHECK_FLAG(c->flags, ZEBRA_IFA_SECONDARY)) {
 		/* trying to add primary address */
 
 		struct in_addr primary_addr = pim_find_primary_addr(c->ifp);
-		if (primary_addr.s_addr != p->u.prefix4.s_addr) {
+		if (p->family != AF_INET
+		    || primary_addr.s_addr != p->u.prefix4.s_addr) {
 			if (PIM_DEBUG_ZEBRA) {
 				/* but we had a primary address already */
 
@@ -306,11 +331,10 @@ static int pim_zebra_if_address_add(int command, struct zclient *zclient,
 		pim_rp_check_on_if_add(pim_ifp);
 
 	if (if_is_loopback(c->ifp)) {
-		struct listnode *ifnode;
+		struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 		struct interface *ifp;
 
-		for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), ifnode,
-					  ifp)) {
+		FOR_ALL_INTERFACES (vrf, ifp) {
 			if (!if_is_loopback(ifp) && if_is_operative(ifp))
 				pim_if_addr_add_all(ifp);
 		}
@@ -324,6 +348,12 @@ static int pim_zebra_if_address_del(int command, struct zclient *client,
 {
 	struct connected *c;
 	struct prefix *p;
+	struct vrf *vrf = vrf_lookup_by_id(vrf_id);
+	struct pim_instance *pim;
+
+	if (!vrf)
+		return 0;
+	pim = vrf->info;
 
 	/*
 	  zebra api notifies address adds/dels events by using the same call
@@ -343,8 +373,8 @@ static int pim_zebra_if_address_del(int command, struct zclient *client,
 			char buf[BUFSIZ];
 			prefix2str(p, buf, BUFSIZ);
 			zlog_debug(
-				"%s: %s disconnected IP address %s flags %u %s",
-				__PRETTY_FUNCTION__, c->ifp->name, buf,
+				"%s: %s(%u) disconnected IP address %s flags %u %s",
+				__PRETTY_FUNCTION__, c->ifp->name, vrf_id, buf,
 				c->flags,
 				CHECK_FLAG(c->flags, ZEBRA_IFA_SECONDARY)
 					? "secondary"
@@ -356,24 +386,23 @@ static int pim_zebra_if_address_del(int command, struct zclient *client,
 		}
 
 		pim_if_addr_del(c, 0);
-		pim_rp_setup();
-		pim_i_am_rp_re_evaluate();
+		pim_rp_setup(pim);
+		pim_i_am_rp_re_evaluate(pim);
 	}
 
 	connected_free(c);
 	return 0;
 }
 
-static void scan_upstream_rpf_cache()
+static void scan_upstream_rpf_cache(struct pim_instance *pim)
 {
 	struct listnode *up_node;
-	struct listnode *ifnode;
 	struct listnode *up_nextnode;
 	struct listnode *node;
 	struct pim_upstream *up;
 	struct interface *ifp;
 
-	for (ALL_LIST_ELEMENTS(pim_upstream_list, up_node, up_nextnode, up)) {
+	for (ALL_LIST_ELEMENTS(pim->upstream_list, up_node, up_nextnode, up)) {
 		enum pim_rpf_result rpf_result;
 		struct pim_rpf old;
 		struct prefix nht_p;
@@ -381,11 +410,11 @@ static void scan_upstream_rpf_cache()
 		nht_p.family = AF_INET;
 		nht_p.prefixlen = IPV4_MAX_BITLEN;
 		nht_p.u.prefix4.s_addr = up->upstream_addr.s_addr;
-		pim_resolve_upstream_nh(&nht_p);
+		pim_resolve_upstream_nh(pim, &nht_p);
 
 		old.source_nexthop.interface = up->rpf.source_nexthop.interface;
 		old.source_nexthop.nbr = up->rpf.source_nexthop.nbr;
-		rpf_result = pim_rpf_update(up, &old, 0);
+		rpf_result = pim_rpf_update(pim, up, &old, 0);
 
 		if (rpf_result == PIM_RPF_FAILURE)
 			continue;
@@ -400,11 +429,12 @@ static void scan_upstream_rpf_cache()
 							up);
 
 			/*
-			 * We have detected a case where we might need to rescan
+			 * We have detected a case where we might need
+			 * to rescan
 			 * the inherited o_list so do it.
 			 */
 			if (up->channel_oil->oil_inherited_rescan) {
-				pim_upstream_inherited_olist_decide(up);
+				pim_upstream_inherited_olist_decide(pim, up);
 				up->channel_oil->oil_inherited_rescan = 0;
 			}
 
@@ -419,23 +449,20 @@ static void scan_upstream_rpf_cache()
 						       __PRETTY_FUNCTION__);
 
 				/*
-				 * RFC 4601: 4.5.7.  Sending (S,G) Join/Prune
-				 * Messages
+				 * RFC 4601: 4.5.7.  Sending (S,G)
+				 * Join/Prune Messages
 				 *
 				 * Transitions from Joined State
 				 *
 				 * RPF'(S,G) changes not due to an Assert
 				 *
-				 * The upstream (S,G) state machine remains in
-				 * Joined
-				 * state. Send Join(S,G) to the new upstream
-				 * neighbor, which is
-				 * the new value of RPF'(S,G).  Send Prune(S,G)
-				 * to the old
-				 * upstream neighbor, which is the old value of
-				 * RPF'(S,G).  Set
-				 * the Join Timer (JT) to expire after
-				 * t_periodic seconds.
+				 * The upstream (S,G) state machine remains
+				 * in Joined state. Send Join(S,G) to the new
+				 * upstream neighbor, which is the new value
+				 * of RPF'(S,G).  Send Prune(S,G) to the old
+				 * upstream neighbor, which is the old value
+				 * of RPF'(S,G).  Set the Join Timer (JT) to
+				 * expire after t_periodic seconds.
 				 */
 				pim_jp_agg_switch_interface(&old, &up->rpf, up);
 
@@ -445,13 +472,13 @@ static void scan_upstream_rpf_cache()
 			/* FIXME can join_desired actually be changed by
 			   pim_rpf_update()
 			   returning PIM_RPF_CHANGED ? */
-			pim_upstream_update_join_desired(up);
+			pim_upstream_update_join_desired(pim, up);
 
 		} /* PIM_RPF_CHANGED */
 
 	} /* for (qpim_upstream_list) */
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), ifnode, ifp))
+	FOR_ALL_INTERFACES (pim->vrf, ifp)
 		if (ifp->info) {
 			struct pim_interface *pim_ifp = ifp->info;
 			struct pim_iface_upstream_switch *us;
@@ -459,6 +486,7 @@ static void scan_upstream_rpf_cache()
 			for (ALL_LIST_ELEMENTS_RO(pim_ifp->upstream_switch_list,
 						  node, us)) {
 				struct pim_rpf rpf;
+
 				rpf.source_nexthop.interface = ifp;
 				rpf.rpf_addr.u.prefix4 = us->address;
 				pim_joinprune_send(&rpf, us->us);
@@ -473,7 +501,8 @@ void pim_scan_individual_oil(struct channel_oil *c_oil, int in_vif_index)
 	int input_iface_vif_index;
 	int old_vif_index;
 
-	if (!pim_rp_set_upstream_addr(&vif_source, c_oil->oil.mfcc_origin,
+	if (!pim_rp_set_upstream_addr(c_oil->pim, &vif_source,
+				      c_oil->oil.mfcc_origin,
 				      c_oil->oil.mfcc_mcastgrp))
 		return;
 
@@ -497,11 +526,11 @@ void pim_scan_individual_oil(struct channel_oil *c_oil, int in_vif_index)
 			pim_inet4_dump("<group?>", c_oil->oil.mfcc_mcastgrp,
 				       group_str, sizeof(group_str));
 			zlog_debug(
-				"%s: channel_oil (%s, %s) upstream info is not present.",
+				"%s: channel_oil (%s,%s) upstream info is not present.",
 				__PRETTY_FUNCTION__, source_str, group_str);
 		}
 		input_iface_vif_index = pim_ecmp_fib_lookup_if_vif_index(
-			vif_source, &src, &grp);
+			c_oil->pim, vif_source, &src, &grp);
 	}
 
 	if (input_iface_vif_index < 1) {
@@ -530,10 +559,10 @@ void pim_scan_individual_oil(struct channel_oil *c_oil, int in_vif_index)
 	}
 
 	if (PIM_DEBUG_ZEBRA) {
-		struct interface *old_iif =
-			pim_if_find_by_vif_index(c_oil->oil.mfcc_parent);
-		struct interface *new_iif =
-			pim_if_find_by_vif_index(input_iface_vif_index);
+		struct interface *old_iif = pim_if_find_by_vif_index(
+			c_oil->pim, c_oil->oil.mfcc_parent);
+		struct interface *new_iif = pim_if_find_by_vif_index(
+			c_oil->pim, input_iface_vif_index);
 		char source_str[INET_ADDRSTRLEN];
 		char group_str[INET_ADDRSTRLEN];
 		pim_inet4_dump("<source?>", c_oil->oil.mfcc_origin, source_str,
@@ -543,14 +572,16 @@ void pim_scan_individual_oil(struct channel_oil *c_oil, int in_vif_index)
 		zlog_debug(
 			"%s %s: (S,G)=(%s,%s) input interface changed from %s vif_index=%d to %s vif_index=%d",
 			__FILE__, __PRETTY_FUNCTION__, source_str, group_str,
-			old_iif->name, c_oil->oil.mfcc_parent, new_iif->name,
+			(old_iif) ? old_iif->name : "<old_iif?>",
+			c_oil->oil.mfcc_parent,
+			(new_iif) ? new_iif->name : "<new_iif?>",
 			input_iface_vif_index);
 	}
 
 	/* new iif loops to existing oif ? */
 	if (c_oil->oil.mfcc_ttls[input_iface_vif_index]) {
-		struct interface *new_iif =
-			pim_if_find_by_vif_index(input_iface_vif_index);
+		struct interface *new_iif = pim_if_find_by_vif_index(
+			c_oil->pim, input_iface_vif_index);
 
 		if (PIM_DEBUG_ZEBRA) {
 			char source_str[INET_ADDRSTRLEN];
@@ -562,7 +593,8 @@ void pim_scan_individual_oil(struct channel_oil *c_oil, int in_vif_index)
 			zlog_debug(
 				"%s %s: (S,G)=(%s,%s) new iif loops to existing oif: %s vif_index=%d",
 				__FILE__, __PRETTY_FUNCTION__, source_str,
-				group_str, new_iif->name,
+				group_str,
+				(new_iif) ? new_iif->name : "<new_iif?>",
 				input_iface_vif_index);
 		}
 	}
@@ -575,10 +607,10 @@ void pim_scan_individual_oil(struct channel_oil *c_oil, int in_vif_index)
 	if (pim_mroute_add(c_oil, __PRETTY_FUNCTION__)) {
 		if (PIM_DEBUG_MROUTE) {
 			/* just log warning */
-			struct interface *old_iif =
-				pim_if_find_by_vif_index(old_vif_index);
-			struct interface *new_iif =
-				pim_if_find_by_vif_index(input_iface_vif_index);
+			struct interface *old_iif = pim_if_find_by_vif_index(
+				c_oil->pim, old_vif_index);
+			struct interface *new_iif = pim_if_find_by_vif_index(
+				c_oil->pim, input_iface_vif_index);
 			char source_str[INET_ADDRSTRLEN];
 			char group_str[INET_ADDRSTRLEN];
 			pim_inet4_dump("<source?>", c_oil->oil.mfcc_origin,
@@ -597,7 +629,7 @@ void pim_scan_individual_oil(struct channel_oil *c_oil, int in_vif_index)
 	}
 }
 
-void pim_scan_oil()
+void pim_scan_oil(struct pim_instance *pim)
 {
 	struct listnode *node;
 	struct listnode *nextnode;
@@ -605,16 +637,17 @@ void pim_scan_oil()
 	ifindex_t ifindex;
 	int vif_index = 0;
 
-	qpim_scan_oil_last = pim_time_monotonic_sec();
-	++qpim_scan_oil_events;
+	pim->scan_oil_last = pim_time_monotonic_sec();
+	++pim->scan_oil_events;
 
-	for (ALL_LIST_ELEMENTS(pim_channel_oil_list, node, nextnode, c_oil)) {
+	for (ALL_LIST_ELEMENTS(pim->channel_oil_list, node, nextnode, c_oil)) {
 		if (c_oil->up && c_oil->up->rpf.source_nexthop.interface) {
 			ifindex = c_oil->up->rpf.source_nexthop
 					  .interface->ifindex;
-			vif_index = pim_if_find_vifindex_by_ifindex(ifindex);
-			/* Pass Current selected NH vif index to mroute download
-			 */
+			vif_index =
+				pim_if_find_vifindex_by_ifindex(pim, ifindex);
+			/* Pass Current selected NH vif index to mroute
+			 * download */
 			if (vif_index)
 				pim_scan_individual_oil(c_oil, vif_index);
 		} else
@@ -624,31 +657,29 @@ void pim_scan_oil()
 
 static int on_rpf_cache_refresh(struct thread *t)
 {
-	zassert(qpim_rpf_cache_refresher);
-
-	qpim_rpf_cache_refresher = 0;
+	struct pim_instance *pim = THREAD_ARG(t);
 
 	/* update PIM protocol state */
-	scan_upstream_rpf_cache();
+	scan_upstream_rpf_cache(pim);
 
 	/* update kernel multicast forwarding cache (MFC) */
-	pim_scan_oil();
+	pim_scan_oil(pim);
 
-	qpim_rpf_cache_refresh_last = pim_time_monotonic_sec();
-	++qpim_rpf_cache_refresh_events;
+	pim->rpf_cache_refresh_last = pim_time_monotonic_sec();
+	++pim->rpf_cache_refresh_events;
 
 	// It is called as part of pim_neighbor_add
 	// pim_rp_setup ();
 	return 0;
 }
 
-void sched_rpf_cache_refresh(void)
+void sched_rpf_cache_refresh(struct pim_instance *pim)
 {
-	++qpim_rpf_cache_refresh_requests;
+	++pim->rpf_cache_refresh_requests;
 
-	pim_rpf_set_refresh_time();
+	pim_rpf_set_refresh_time(pim);
 
-	if (qpim_rpf_cache_refresher) {
+	if (pim->rpf_cache_refresher) {
 		/* Refresh timer is already running */
 		return;
 	}
@@ -660,31 +691,25 @@ void sched_rpf_cache_refresh(void)
 			   qpim_rpf_cache_refresh_delay_msec);
 	}
 
-	THREAD_TIMER_MSEC_ON(master, qpim_rpf_cache_refresher,
-			     on_rpf_cache_refresh, 0,
-			     qpim_rpf_cache_refresh_delay_msec);
+	thread_add_timer_msec(master, on_rpf_cache_refresh, pim,
+			      qpim_rpf_cache_refresh_delay_msec,
+			      &pim->rpf_cache_refresher);
 }
 
 static void pim_zebra_connected(struct zclient *zclient)
 {
-	zclient_send_reg_requests(zclient, VRF_DEFAULT);
+	/* Send the client registration */
+	bfd_client_sendmsg(zclient, ZEBRA_BFD_CLIENT_REGISTER);
+
+	zclient_send_reg_requests(zclient, pimg->vrf_id);
 }
 
 void pim_zebra_init(void)
 {
 	int i;
 
-#ifdef HAVE_TCP_ZEBRA
-	zlog_notice(
-		"zclient update contacting ZEBRA daemon at socket TCP %s,%d",
-		"127.0.0.1", ZEBRA_PORT);
-#else
-	zlog_notice("zclient update contacting ZEBRA daemon at socket UNIX %s",
-		    zclient_serv_path_get());
-#endif
-
 	/* Socket for receiving updates from Zebra daemon */
-	zclient = zclient_new(master);
+	zclient = zclient_new_notify(master, &zclient_options_default);
 
 	zclient->zebra_connected = pim_zebra_connected;
 	zclient->router_id_update = pim_router_id_update_zebra;
@@ -696,18 +721,16 @@ void pim_zebra_init(void)
 	zclient->interface_address_delete = pim_zebra_if_address_del;
 	zclient->nexthop_update = pim_parse_nexthop_update;
 
-	zclient_init(zclient, ZEBRA_ROUTE_PIM, 0);
+	zclient_init(zclient, ZEBRA_ROUTE_PIM, 0, &pimd_privs);
 	if (PIM_DEBUG_PIM_TRACE) {
 		zlog_info("zclient_init cleared redistribution request");
 	}
 
-	zassert(zclient->redist_default == ZEBRA_ROUTE_PIM);
-
 	/* Request all redistribution */
 	for (i = 0; i < ZEBRA_ROUTE_MAX; i++) {
 		if (i == zclient->redist_default)
 			continue;
-		vrf_bitmap_set(zclient->redist[AFI_IP][i], VRF_DEFAULT);
+		vrf_bitmap_set(zclient->redist[AFI_IP][i], pimg->vrf_id);
 		;
 		if (PIM_DEBUG_PIM_TRACE) {
 			zlog_debug("%s: requesting redistribution for %s (%i)",
@@ -718,7 +741,7 @@ void pim_zebra_init(void)
 
 	/* Request default information */
 	zclient_redistribute_default(ZEBRA_REDISTRIBUTE_DEFAULT_ADD, zclient,
-				     VRF_DEFAULT);
+				     pimg->vrf_id);
 
 	if (PIM_DEBUG_PIM_TRACE) {
 		zlog_info("%s: requesting default information redistribution",
@@ -731,7 +754,8 @@ void pim_zebra_init(void)
 	zclient_lookup_new();
 }
 
-void igmp_anysource_forward_start(struct igmp_group *group)
+void igmp_anysource_forward_start(struct pim_instance *pim,
+				  struct igmp_group *group)
 {
 	struct igmp_source *source;
 	struct in_addr src_addr = {.s_addr = 0};
@@ -746,7 +770,7 @@ void igmp_anysource_forward_start(struct igmp_group *group)
 		return;
 	}
 
-	igmp_source_forward_start(source);
+	igmp_source_forward_start(pim, source);
 }
 
 void igmp_anysource_forward_stop(struct igmp_group *group)
@@ -759,7 +783,8 @@ void igmp_anysource_forward_stop(struct igmp_group *group)
 		igmp_source_forward_stop(source);
 }
 
-static void igmp_source_forward_reevaluate_one(struct igmp_source *source)
+static void igmp_source_forward_reevaluate_one(struct pim_instance *pim,
+					       struct igmp_source *source)
 {
 	struct prefix_sg sg;
 	struct igmp_group *group = source->source_group;
@@ -774,7 +799,7 @@ static void igmp_source_forward_reevaluate_one(struct igmp_source *source)
 	sg.grp = group->group_addr;
 
 	ch = pim_ifchannel_find(group->group_igmp_sock->interface, &sg);
-	if (pim_is_grp_ssm(group->group_addr)) {
+	if (pim_is_grp_ssm(pim, group->group_addr)) {
 		/* If SSM group withdraw local membership */
 		if (ch
 		    && (ch->local_ifmembership == PIM_IFMEMBERSHIP_INCLUDE)) {
@@ -799,12 +824,11 @@ static void igmp_source_forward_reevaluate_one(struct igmp_source *source)
 	}
 }
 
-void igmp_source_forward_reevaluate_all(void)
+void igmp_source_forward_reevaluate_all(struct pim_instance *pim)
 {
-	struct listnode *ifnode;
 	struct interface *ifp;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), ifnode, ifp)) {
+	FOR_ALL_INTERFACES (pim->vrf, ifp) {
 		struct pim_interface *pim_ifp = ifp->info;
 		struct listnode *sock_node;
 		struct igmp_sock *igmp;
@@ -828,14 +852,16 @@ void igmp_source_forward_reevaluate_all(void)
 				for (ALL_LIST_ELEMENTS_RO(
 					     grp->group_source_list, srcnode,
 					     src)) {
-					igmp_source_forward_reevaluate_one(src);
+					igmp_source_forward_reevaluate_one(pim,
+									   src);
 				} /* scan group sources */
 			}	 /* scan igmp groups */
 		}		  /* scan igmp sockets */
 	}			  /* scan interfaces */
 }
 
-void igmp_source_forward_start(struct igmp_source *source)
+void igmp_source_forward_start(struct pim_instance *pim,
+			       struct igmp_source *source)
 {
 	struct igmp_group *group;
 	struct prefix_sg sg;
@@ -867,13 +893,12 @@ void igmp_source_forward_start(struct igmp_source *source)
 		struct in_addr vif_source;
 		struct pim_interface *pim_oif;
 		struct prefix nht_p, src, grp;
-		int ret = 0;
 		struct pim_nexthop_cache out_pnc;
 		struct pim_nexthop nexthop;
 		struct pim_upstream *up = NULL;
 
-		if (!pim_rp_set_upstream_addr(&vif_source, source->source_addr,
-					      sg.grp))
+		if (!pim_rp_set_upstream_addr(pim, &vif_source,
+					      source->source_addr, sg.grp))
 			return;
 
 		/* Register addr with Zebra NHT */
@@ -889,22 +914,21 @@ void igmp_source_forward_start(struct igmp_source *source)
 		grp.prefixlen = IPV4_MAX_BITLEN;
 		grp.u.prefix4 = sg.grp;
 
-		if ((ret = pim_find_or_track_nexthop(&nht_p, NULL, NULL,
-						     &out_pnc))
-		    == 1) {
+		if (pim_find_or_track_nexthop(pim, &nht_p, NULL, NULL,
+					      &out_pnc)) {
 			if (out_pnc.nexthop_num) {
-				up = pim_upstream_find(&sg);
-				memset(&nexthop, 0, sizeof(struct pim_nexthop));
+				up = pim_upstream_find(pim, &sg);
+				memset(&nexthop, 0, sizeof(nexthop));
 				if (up)
 					memcpy(&nexthop,
 					       &up->rpf.source_nexthop,
 					       sizeof(struct pim_nexthop));
-				// Compute PIM RPF using Cached nexthop
-				pim_ecmp_nexthop_search(&out_pnc, &nexthop,
+				pim_ecmp_nexthop_search(pim, &out_pnc, &nexthop,
 							&src, &grp, 0);
 				if (nexthop.interface)
 					input_iface_vif_index =
 						pim_if_find_vifindex_by_ifindex(
+							pim,
 							nexthop.interface->ifindex);
 			} else {
 				if (PIM_DEBUG_ZEBRA) {
@@ -924,8 +948,8 @@ void igmp_source_forward_start(struct igmp_source *source)
 			}
 		} else
 			input_iface_vif_index =
-				pim_ecmp_fib_lookup_if_vif_index(vif_source,
-								 &src, &grp);
+				pim_ecmp_fib_lookup_if_vif_index(
+					pim, vif_source, &src, &grp);
 
 		if (PIM_DEBUG_ZEBRA) {
 			char buf2[INET_ADDRSTRLEN];
@@ -984,7 +1008,7 @@ void igmp_source_forward_start(struct igmp_source *source)
 		}
 
 		source->source_channel_oil =
-			pim_channel_oil_add(&sg, input_iface_vif_index);
+			pim_channel_oil_add(pim, &sg, input_iface_vif_index);
 		if (!source->source_channel_oil) {
 			if (PIM_DEBUG_IGMP_TRACE) {
 				zlog_debug(
@@ -1090,6 +1114,11 @@ void pim_forward_start(struct pim_ifchannel *ch)
 	struct pim_upstream *up = ch->upstream;
 	uint32_t mask = PIM_OIF_FLAG_PROTO_PIM;
 	int input_iface_vif_index = 0;
+	struct pim_instance *pim;
+	struct pim_interface *pim_ifp;
+
+	pim_ifp = ch->interface->info;
+	pim = pim_ifp->pim;
 
 	if (PIM_DEBUG_PIM_TRACE) {
 		char source_str[INET_ADDRSTRLEN];
@@ -1114,7 +1143,6 @@ void pim_forward_start(struct pim_ifchannel *ch)
 	    || (up->channel_oil
 		&& up->channel_oil->oil.mfcc_parent >= MAXVIFS)) {
 		struct prefix nht_p, src, grp;
-		int ret = 0;
 		struct pim_nexthop_cache out_pnc;
 
 		/* Register addr with Zebra NHT */
@@ -1126,9 +1154,8 @@ void pim_forward_start(struct pim_ifchannel *ch)
 		grp.u.prefix4 = up->sg.grp;
 		memset(&out_pnc, 0, sizeof(struct pim_nexthop_cache));
 
-		if ((ret = pim_find_or_track_nexthop(&nht_p, NULL, NULL,
-						     &out_pnc))
-		    == 1) {
+		if (pim_find_or_track_nexthop(pim, &nht_p, NULL, NULL,
+					      &out_pnc)) {
 			if (out_pnc.nexthop_num) {
 				src.family = AF_INET;
 				src.prefixlen = IPV4_MAX_BITLEN;
@@ -1139,11 +1166,12 @@ void pim_forward_start(struct pim_ifchannel *ch)
 				grp.u.prefix4 = up->sg.grp;
 				// Compute PIM RPF using Cached nexthop
 				if (pim_ecmp_nexthop_search(
-					    &out_pnc, &up->rpf.source_nexthop,
-					    &src, &grp, 0)
-				    == 0)
+					    pim, &out_pnc,
+					    &up->rpf.source_nexthop, &src, &grp,
+					    0))
 					input_iface_vif_index =
 						pim_if_find_vifindex_by_ifindex(
+							pim,
 							up->rpf.source_nexthop
 								.interface->ifindex);
 				else {
@@ -1172,7 +1200,7 @@ void pim_forward_start(struct pim_ifchannel *ch)
 		} else
 			input_iface_vif_index =
 				pim_ecmp_fib_lookup_if_vif_index(
-					up->upstream_addr, &src, &grp);
+					pim, up->upstream_addr, &src, &grp);
 
 		if (input_iface_vif_index < 1) {
 			if (PIM_DEBUG_PIM_TRACE) {
@@ -1187,16 +1215,16 @@ void pim_forward_start(struct pim_ifchannel *ch)
 			return;
 		}
 		if (PIM_DEBUG_TRACE) {
-			struct interface *in_intf =
-				pim_if_find_by_vif_index(input_iface_vif_index);
+			struct interface *in_intf = pim_if_find_by_vif_index(
+				pim, input_iface_vif_index);
 			zlog_debug(
 				"%s: Update channel_oil IIF %s VIFI %d entry %s ",
 				__PRETTY_FUNCTION__,
 				in_intf ? in_intf->name : "NIL",
 				input_iface_vif_index, up->sg_str);
 		}
-		up->channel_oil =
-			pim_channel_oil_add(&up->sg, input_iface_vif_index);
+		up->channel_oil = pim_channel_oil_add(pim, &up->sg,
+						      input_iface_vif_index);
 		if (!up->channel_oil) {
 			if (PIM_DEBUG_PIM_TRACE)
 				zlog_debug(
@@ -1213,17 +1241,21 @@ void pim_forward_start(struct pim_ifchannel *ch)
 	pim_channel_add_oif(up->channel_oil, ch->interface, mask);
 }
 
-void pim_forward_stop(struct pim_ifchannel *ch)
+void pim_forward_stop(struct pim_ifchannel *ch, bool install_it)
 {
 	struct pim_upstream *up = ch->upstream;
 
 	if (PIM_DEBUG_PIM_TRACE) {
-		zlog_debug("%s: (S,G)=%s oif=%s", __PRETTY_FUNCTION__,
-			   ch->sg_str, ch->interface->name);
+		zlog_debug("%s: (S,G)=%s oif=%s install_it: %d installed: %d",
+			   __PRETTY_FUNCTION__, ch->sg_str, ch->interface->name,
+			   install_it, up->channel_oil->installed);
 	}
 
 	pim_channel_del_oif(up->channel_oil, ch->interface,
 			    PIM_OIF_FLAG_PROTO_PIM);
+
+	if (install_it && !up->channel_oil->installed)
+		pim_mroute_add(up->channel_oil, __PRETTY_FUNCTION__);
 }
 
 void pim_zebra_zclient_update(struct vty *vty)
@@ -1231,10 +1263,9 @@ void pim_zebra_zclient_update(struct vty *vty)
 	vty_out(vty, "Zclient update socket: ");
 
 	if (zclient) {
-		vty_out(vty, "%d failures=%d%s", zclient->sock, zclient->fail,
-			VTY_NEWLINE);
+		vty_out(vty, "%d failures=%d\n", zclient->sock, zclient->fail);
 	} else {
-		vty_out(vty, "<null zclient>%s", VTY_NEWLINE);
+		vty_out(vty, "<null zclient>\n");
 	}
 }
 
