@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -59,8 +58,6 @@ const struct message ri_version_msg[] = {{RI_RIP_VERSION_1, "1"},
 					 {RI_RIP_VERSION_NONE, "none"},
 					 {0}};
 
-extern struct zebra_privs_t ripd_privs;
-
 /* RIP enabled network vector. */
 vector rip_enable_interface;
 
@@ -136,7 +133,7 @@ void rip_interface_multicast_set(int sock, struct connected *connected)
 }
 
 /* Send RIP request packet to specified interface. */
-static void rip_request_interface_send(struct interface *ifp, u_char version)
+static void rip_request_interface_send(struct interface *ifp, uint8_t version)
 {
 	struct sockaddr_in to;
 
@@ -337,10 +334,10 @@ static int rip_if_ipv4_address_check(struct interface *ifp)
 /* Does this address belongs to me ? */
 int if_check_address(struct in_addr addr)
 {
-	struct listnode *node;
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct interface *ifp;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp)) {
+	FOR_ALL_INTERFACES (vrf, ifp) {
 		struct listnode *cnode;
 		struct connected *connected;
 
@@ -472,7 +469,7 @@ int rip_interface_delete(int command, struct zclient *zclient,
 
 	/* To support pseudo interface do not free interface structure.  */
 	/* if_delete(ifp); */
-	ifp->ifindex = IFINDEX_DELETED;
+	if_set_index(ifp, IFINDEX_INTERNAL);
 
 	return 0;
 }
@@ -491,10 +488,10 @@ static void rip_interface_clean(struct rip_interface *ri)
 
 void rip_interfaces_clean(void)
 {
-	struct listnode *node;
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct interface *ifp;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp))
+	FOR_ALL_INTERFACES (vrf, ifp)
 		rip_interface_clean(ifp->info);
 }
 
@@ -543,10 +540,10 @@ static void rip_interface_reset(struct rip_interface *ri)
 
 void rip_interfaces_reset(void)
 {
-	struct listnode *node;
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct interface *ifp;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp))
+	FOR_ALL_INTERFACES (vrf, ifp)
 		rip_interface_reset(ifp->info);
 }
 
@@ -562,7 +559,7 @@ int rip_if_down(struct interface *ifp)
 			if ((list = rp->info) != NULL)
 				for (ALL_LIST_ELEMENTS(list, listnode, nextnode,
 						       rinfo))
-					if (rinfo->ifindex == ifp->ifindex)
+					if (rinfo->nh.ifindex == ifp->ifindex)
 						rip_ecmp_delete(rinfo);
 
 		ri = ifp->info;
@@ -584,16 +581,17 @@ int rip_if_down(struct interface *ifp)
 /* Needed for stop RIP process. */
 void rip_if_down_all()
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct interface *ifp;
-	struct listnode *node, *nnode;
 
-	for (ALL_LIST_ELEMENTS(vrf_iflist(VRF_DEFAULT), node, nnode, ifp))
+	FOR_ALL_INTERFACES (vrf, ifp)
 		rip_if_down(ifp);
 }
 
 static void rip_apply_address_add(struct connected *ifc)
 {
 	struct prefix_ipv4 address;
+	struct nexthop nh;
 	struct prefix *p;
 
 	if (!rip)
@@ -605,18 +603,22 @@ static void rip_apply_address_add(struct connected *ifc)
 	p = ifc->address;
 
 	memset(&address, 0, sizeof(address));
+	memset(&nh, 0, sizeof(nh));
+
 	address.family = p->family;
 	address.prefix = p->u.prefix4;
 	address.prefixlen = p->prefixlen;
 	apply_mask_ipv4(&address);
 
+	nh.ifindex = ifc->ifp->ifindex;
+	nh.type = NEXTHOP_TYPE_IFINDEX;
+
 	/* Check if this interface is RIP enabled or not
 	   or  Check if this address's prefix is RIP enabled */
 	if ((rip_enable_if_lookup(ifc->ifp->name) >= 0)
 	    || (rip_enable_network_lookup2(ifc) >= 0))
 		rip_redistribute_add(ZEBRA_ROUTE_CONNECT, RIP_ROUTE_INTERFACE,
-				     &address, ifc->ifp->ifindex, NULL, 0, 0,
-				     0);
+				     &address, &nh, 0, 0, 0);
 }
 
 int rip_interface_address_add(int command, struct zclient *zclient,
@@ -882,6 +884,9 @@ static void rip_connect_set(struct interface *ifp, int set)
 	struct listnode *node, *nnode;
 	struct connected *connected;
 	struct prefix_ipv4 address;
+	struct nexthop nh;
+
+	memset(&nh, 0, sizeof(nh));
 
 	for (ALL_LIST_ELEMENTS(ifp->connected, node, nnode, connected)) {
 		struct prefix *p;
@@ -895,24 +900,24 @@ static void rip_connect_set(struct interface *ifp, int set)
 		address.prefixlen = p->prefixlen;
 		apply_mask_ipv4(&address);
 
+		nh.ifindex = connected->ifp->ifindex;
+		nh.type = NEXTHOP_TYPE_IFINDEX;
 		if (set) {
 			/* Check once more wether this prefix is within a
 			 * "network IF_OR_PREF" one */
 			if ((rip_enable_if_lookup(connected->ifp->name) >= 0)
 			    || (rip_enable_network_lookup2(connected) >= 0))
-				rip_redistribute_add(
-					ZEBRA_ROUTE_CONNECT,
-					RIP_ROUTE_INTERFACE, &address,
-					connected->ifp->ifindex, NULL, 0, 0, 0);
+				rip_redistribute_add(ZEBRA_ROUTE_CONNECT,
+						     RIP_ROUTE_INTERFACE,
+						     &address, &nh, 0, 0, 0);
 		} else {
 			rip_redistribute_delete(ZEBRA_ROUTE_CONNECT,
 						RIP_ROUTE_INTERFACE, &address,
 						connected->ifp->ifindex);
 			if (rip_redistribute_check(ZEBRA_ROUTE_CONNECT))
-				rip_redistribute_add(
-					ZEBRA_ROUTE_CONNECT,
-					RIP_ROUTE_REDISTRIBUTE, &address,
-					connected->ifp->ifindex, NULL, 0, 0, 0);
+				rip_redistribute_add(ZEBRA_ROUTE_CONNECT,
+						     RIP_ROUTE_REDISTRIBUTE,
+						     &address, &nh, 0, 0, 0);
 		}
 	}
 }
@@ -958,9 +963,8 @@ void rip_enable_apply(struct interface *ifp)
 				zlog_debug("turn on %s", ifp->name);
 
 			/* Add interface wake up thread. */
-			if (!ri->t_wakeup)
-				ri->t_wakeup = thread_add_timer(
-					master, rip_interface_wakeup, ifp, 1);
+			thread_add_timer(master, rip_interface_wakeup, ifp, 1,
+					 &ri->t_wakeup);
 			rip_connect_set(ifp, 1);
 		}
 	} else {
@@ -979,11 +983,11 @@ void rip_enable_apply(struct interface *ifp)
 /* Apply network configuration to all interface. */
 void rip_enable_apply_all()
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct interface *ifp;
-	struct listnode *node, *nnode;
 
 	/* Check each interface. */
-	for (ALL_LIST_ELEMENTS(vrf_iflist(VRF_DEFAULT), node, nnode, ifp))
+	FOR_ALL_INTERFACES (vrf, ifp)
 		rip_enable_apply(ifp);
 }
 
@@ -1093,10 +1097,10 @@ void rip_passive_interface_apply(struct interface *ifp)
 
 static void rip_passive_interface_apply_all(void)
 {
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct interface *ifp;
-	struct listnode *node, *nnode;
 
-	for (ALL_LIST_ELEMENTS(vrf_iflist(VRF_DEFAULT), node, nnode, ifp))
+	FOR_ALL_INTERFACES (vrf, ifp)
 		rip_passive_interface_apply(ifp);
 }
 
@@ -1104,7 +1108,7 @@ static void rip_passive_interface_apply_all(void)
 static int rip_passive_nondefault_set(struct vty *vty, const char *ifname)
 {
 	if (rip_passive_nondefault_lookup(ifname) >= 0)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	vector_set(Vrip_passive_nondefault, strdup(ifname));
 
@@ -1120,7 +1124,7 @@ static int rip_passive_nondefault_unset(struct vty *vty, const char *ifname)
 
 	i = rip_passive_nondefault_lookup(ifname);
 	if (i < 0)
-		return CMD_WARNING;
+		return CMD_WARNING_CONFIG_FAILED;
 
 	str = vector_slot(Vrip_passive_nondefault, i);
 	free(str);
@@ -1165,9 +1169,9 @@ DEFUN (rip_network,
 		ret = rip_enable_if_add(argv[idx_ipv4_word]->arg);
 
 	if (ret < 0) {
-		vty_out(vty, "There is a same network configuration %s%s",
-			argv[idx_ipv4_word]->arg, VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "There is a same network configuration %s\n",
+			argv[idx_ipv4_word]->arg);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	return CMD_SUCCESS;
@@ -1194,9 +1198,9 @@ DEFUN (no_rip_network,
 		ret = rip_enable_if_delete(argv[idx_ipv4_word]->arg);
 
 	if (ret < 0) {
-		vty_out(vty, "Can't find network configuration %s%s",
-			argv[idx_ipv4_word]->arg, VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Can't find network configuration %s\n",
+			argv[idx_ipv4_word]->arg);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	return CMD_SUCCESS;
@@ -1216,9 +1220,8 @@ DEFUN (rip_neighbor,
 	ret = str2prefix_ipv4(argv[idx_ipv4]->arg, &p);
 
 	if (ret <= 0) {
-		vty_out(vty, "Please specify address by A.B.C.D%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Please specify address by A.B.C.D\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	rip_neighbor_add(&p);
@@ -1241,9 +1244,8 @@ DEFUN (no_rip_neighbor,
 	ret = str2prefix_ipv4(argv[idx_ipv4]->arg, &p);
 
 	if (ret <= 0) {
-		vty_out(vty, "Please specify address by A.B.C.D%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Please specify address by A.B.C.D\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	rip_neighbor_delete(&p);
@@ -1258,8 +1260,7 @@ DEFUN (ip_rip_receive_version,
        "Routing Information Protocol\n"
        "Advertisement reception\n"
        "Version control\n"
-       "RIP version 1\n"
-       "RIP version 2\n"
+       "RIP version\n"
        "None\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
@@ -1282,36 +1283,18 @@ DEFUN (ip_rip_receive_version,
 		break;
 	}
 
-	return CMD_WARNING;
+	return CMD_WARNING_CONFIG_FAILED;
 }
 
 DEFUN (ip_rip_receive_version_1,
        ip_rip_receive_version_1_cmd,
-       "ip rip receive version (1-1) (2-2)",
+       "ip rip receive version <1 2|2 1>",
        IP_STR
        "Routing Information Protocol\n"
        "Advertisement reception\n"
        "Version control\n"
        "RIP version 1\n"
-       "RIP version 2\n")
-{
-	VTY_DECLVAR_CONTEXT(interface, ifp);
-	struct rip_interface *ri;
-
-	ri = ifp->info;
-
-	/* Version 1 and 2. */
-	ri->ri_receive = RI_RIP_VERSION_1_AND_2;
-	return CMD_SUCCESS;
-}
-
-DEFUN (ip_rip_receive_version_2,
-       ip_rip_receive_version_2_cmd,
-       "ip rip receive version (2-2) (1-1)",
-       IP_STR
-       "Routing Information Protocol\n"
-       "Advertisement reception\n"
-       "Version control\n"
+       "RIP version 2\n"
        "RIP version 2\n"
        "RIP version 1\n")
 {
@@ -1333,8 +1316,7 @@ DEFUN (no_ip_rip_receive_version,
        "Routing Information Protocol\n"
        "Advertisement reception\n"
        "Version control\n"
-       "Version 1\n"
-       "Version 2\n")
+       "RIP version\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct rip_interface *ri;
@@ -1353,8 +1335,7 @@ DEFUN (ip_rip_send_version,
        "Routing Information Protocol\n"
        "Advertisement transmission\n"
        "Version control\n"
-       "RIP version 1\n"
-       "RIP version 2\n")
+       "RIP version\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	int idx_type = 4;
@@ -1371,36 +1352,18 @@ DEFUN (ip_rip_send_version,
 		ri->ri_send = RI_RIP_VERSION_2;
 		return CMD_SUCCESS;
 	}
-	return CMD_WARNING;
+	return CMD_WARNING_CONFIG_FAILED;
 }
 
 DEFUN (ip_rip_send_version_1,
        ip_rip_send_version_1_cmd,
-       "ip rip send version (1-1) (2-2)",
+       "ip rip send version <1 2|2 1>",
        IP_STR
        "Routing Information Protocol\n"
        "Advertisement transmission\n"
        "Version control\n"
        "RIP version 1\n"
-       "RIP version 2\n")
-{
-	VTY_DECLVAR_CONTEXT(interface, ifp);
-	struct rip_interface *ri;
-
-	ri = ifp->info;
-
-	/* Version 1 and 2. */
-	ri->ri_send = RI_RIP_VERSION_1_AND_2;
-	return CMD_SUCCESS;
-}
-
-DEFUN (ip_rip_send_version_2,
-       ip_rip_send_version_2_cmd,
-       "ip rip send version (2-2) (1-1)",
-       IP_STR
-       "Routing Information Protocol\n"
-       "Advertisement transmission\n"
-       "Version control\n"
+       "RIP version 2\n"
        "RIP version 2\n"
        "RIP version 1\n")
 {
@@ -1422,8 +1385,7 @@ DEFUN (no_ip_rip_send_version,
        "Routing Information Protocol\n"
        "Advertisement transmission\n"
        "Version control\n"
-       "Version 1\n"
-       "Version 2\n")
+       "RIP version\n")
 {
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct rip_interface *ri;
@@ -1501,9 +1463,8 @@ DEFUN (ip_rip_authentication_mode,
 	if (argc > 5) {
 		if (auth_type != RIP_AUTH_MD5) {
 			vty_out(vty,
-				"auth length argument only valid for md5%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
+				"auth length argument only valid for md5\n");
+			return CMD_WARNING_CONFIG_FAILED;
 		}
 		if (strmatch("rfc", authlen))
 			ri->md5_auth_len = RIP_AUTH_MD5_SIZE;
@@ -1558,15 +1519,13 @@ DEFUN (ip_rip_authentication_string,
 
 	if (strlen(argv[idx_line]->arg) > 16) {
 		vty_out(vty,
-			"%% RIPv2 authentication string must be shorter than 16%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+			"%% RIPv2 authentication string must be shorter than 16\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (ri->key_chain) {
-		vty_out(vty, "%% key-chain configuration exists%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% key-chain configuration exists\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (ri->auth_str)
@@ -1617,9 +1576,8 @@ DEFUN (ip_rip_authentication_key_chain,
 	ri = ifp->info;
 
 	if (ri->auth_str) {
-		vty_out(vty, "%% authentication string configuration exists%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% authentication string configuration exists\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (ri->key_chain)
@@ -1776,15 +1734,12 @@ DEFUN (no_rip_passive_interface,
 /* Write rip configuration of each interface. */
 static int rip_interface_config_write(struct vty *vty)
 {
-	struct listnode *node;
+	struct vrf *vrf = vrf_lookup_by_id(VRF_DEFAULT);
 	struct interface *ifp;
 
-	for (ALL_LIST_ELEMENTS_RO(vrf_iflist(VRF_DEFAULT), node, ifp)) {
+	FOR_ALL_INTERFACES (vrf, ifp) {
 		struct rip_interface *ri;
 
-		if (ifp->ifindex == IFINDEX_DELETED)
-			continue;
-
 		ri = ifp->info;
 
 		/* Do not display the interface if there is no
@@ -1799,51 +1754,44 @@ static int rip_interface_config_write(struct vty *vty)
 		    && (!ri->auth_str) && (!ri->key_chain))
 			continue;
 
-		vty_out(vty, "interface %s%s", ifp->name, VTY_NEWLINE);
+		vty_frame(vty, "interface %s\n", ifp->name);
 
 		if (ifp->desc)
-			vty_out(vty, " description %s%s", ifp->desc,
-				VTY_NEWLINE);
+			vty_out(vty, " description %s\n", ifp->desc);
 
 		/* Split horizon. */
 		if (ri->split_horizon != ri->split_horizon_default) {
 			switch (ri->split_horizon) {
 			case RIP_SPLIT_HORIZON:
-				vty_out(vty, " ip rip split-horizon%s",
-					VTY_NEWLINE);
+				vty_out(vty, " ip rip split-horizon\n");
 				break;
 			case RIP_SPLIT_HORIZON_POISONED_REVERSE:
 				vty_out(vty,
-					" ip rip split-horizon poisoned-reverse%s",
-					VTY_NEWLINE);
+					" ip rip split-horizon poisoned-reverse\n");
 				break;
 			case RIP_NO_SPLIT_HORIZON:
 			default:
-				vty_out(vty, " no ip rip split-horizon%s",
-					VTY_NEWLINE);
+				vty_out(vty, " no ip rip split-horizon\n");
 				break;
 			}
 		}
 
 		/* RIP version setting. */
 		if (ri->ri_send != RI_RIP_UNSPEC)
-			vty_out(vty, " ip rip send version %s%s",
-				lookup_msg(ri_version_msg, ri->ri_send, NULL),
-				VTY_NEWLINE);
+			vty_out(vty, " ip rip send version %s\n",
+				lookup_msg(ri_version_msg, ri->ri_send, NULL));
 
 		if (ri->ri_receive != RI_RIP_UNSPEC)
-			vty_out(vty, " ip rip receive version %s%s",
+			vty_out(vty, " ip rip receive version %s \n",
 				lookup_msg(ri_version_msg, ri->ri_receive,
-					   NULL),
-				VTY_NEWLINE);
+					   NULL));
 
 		if (ri->v2_broadcast)
-			vty_out(vty, " ip rip v2-broadcast%s", VTY_NEWLINE);
+			vty_out(vty, " ip rip v2-broadcast\n");
 
 		/* RIP authentication. */
 		if (ri->auth_type == RIP_AUTH_SIMPLE_PASSWORD)
-			vty_out(vty, " ip rip authentication mode text%s",
-				VTY_NEWLINE);
+			vty_out(vty, " ip rip authentication mode text\n");
 
 		if (ri->auth_type == RIP_AUTH_MD5) {
 			vty_out(vty, " ip rip authentication mode md5");
@@ -1851,18 +1799,18 @@ static int rip_interface_config_write(struct vty *vty)
 				vty_out(vty, " auth-length old-ripd");
 			else
 				vty_out(vty, " auth-length rfc");
-			vty_out(vty, "%s", VTY_NEWLINE);
+			vty_out(vty, "\n");
 		}
 
 		if (ri->auth_str)
-			vty_out(vty, " ip rip authentication string %s%s",
-				ri->auth_str, VTY_NEWLINE);
+			vty_out(vty, " ip rip authentication string %s\n",
+				ri->auth_str);
 
 		if (ri->key_chain)
-			vty_out(vty, " ip rip authentication key-chain %s%s",
-				ri->key_chain, VTY_NEWLINE);
+			vty_out(vty, " ip rip authentication key-chain %s\n",
+				ri->key_chain);
 
-		vty_out(vty, "!%s", VTY_NEWLINE);
+		vty_endframe(vty, "!\n");
 	}
 	return 0;
 }
@@ -1877,36 +1825,33 @@ int config_write_rip_network(struct vty *vty, int config_mode)
 	for (node = route_top(rip_enable_network); node;
 	     node = route_next(node))
 		if (node->info)
-			vty_out(vty, "%s%s/%d%s",
+			vty_out(vty, "%s%s/%d\n",
 				config_mode ? " network " : "    ",
-				inet_ntoa(node->p.u.prefix4), node->p.prefixlen,
-				VTY_NEWLINE);
+				inet_ntoa(node->p.u.prefix4),
+				node->p.prefixlen);
 
 	/* Interface name RIP enable statement. */
 	for (i = 0; i < vector_active(rip_enable_interface); i++)
 		if ((ifname = vector_slot(rip_enable_interface, i)) != NULL)
-			vty_out(vty, "%s%s%s",
-				config_mode ? " network " : "    ", ifname,
-				VTY_NEWLINE);
+			vty_out(vty, "%s%s\n",
+				config_mode ? " network " : "    ", ifname);
 
 	/* RIP neighbors listing. */
 	for (node = route_top(rip->neighbor); node; node = route_next(node))
 		if (node->info)
-			vty_out(vty, "%s%s%s",
+			vty_out(vty, "%s%s\n",
 				config_mode ? " neighbor " : "    ",
-				inet_ntoa(node->p.u.prefix4), VTY_NEWLINE);
+				inet_ntoa(node->p.u.prefix4));
 
 	/* RIP passive interface listing. */
 	if (config_mode) {
 		if (passive_default)
-			vty_out(vty, " passive-interface default%s",
-				VTY_NEWLINE);
+			vty_out(vty, " passive-interface default\n");
 		for (i = 0; i < vector_active(Vrip_passive_nondefault); i++)
 			if ((ifname = vector_slot(Vrip_passive_nondefault, i))
 			    != NULL)
-				vty_out(vty, " %spassive-interface %s%s",
-					(passive_default ? "no " : ""), ifname,
-					VTY_NEWLINE);
+				vty_out(vty, " %spassive-interface %s\n",
+					(passive_default ? "no " : ""), ifname);
 	}
 
 	return 0;
@@ -1935,8 +1880,8 @@ static int rip_interface_delete_hook(struct interface *ifp)
 void rip_if_init(void)
 {
 	/* Default initial size of interface vector. */
-	if_add_hook(IF_NEW_HOOK, rip_interface_new_hook);
-	if_add_hook(IF_DELETE_HOOK, rip_interface_delete_hook);
+	hook_register_prio(if_add, 0, rip_interface_new_hook);
+	hook_register_prio(if_del, 0, rip_interface_delete_hook);
 
 	/* RIP network init. */
 	rip_enable_interface = vector_init(1);
@@ -1960,12 +1905,10 @@ void rip_if_init(void)
 
 	install_element(INTERFACE_NODE, &ip_rip_send_version_cmd);
 	install_element(INTERFACE_NODE, &ip_rip_send_version_1_cmd);
-	install_element(INTERFACE_NODE, &ip_rip_send_version_2_cmd);
 	install_element(INTERFACE_NODE, &no_ip_rip_send_version_cmd);
 
 	install_element(INTERFACE_NODE, &ip_rip_receive_version_cmd);
 	install_element(INTERFACE_NODE, &ip_rip_receive_version_1_cmd);
-	install_element(INTERFACE_NODE, &ip_rip_receive_version_2_cmd);
 	install_element(INTERFACE_NODE, &no_ip_rip_receive_version_cmd);
 
 	install_element(INTERFACE_NODE, &ip_rip_v2_broadcast_cmd);
