@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with Quagga; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -28,8 +27,9 @@
 #include "memory.h"
 #include "queue.h"
 #include "filter.h"
+#include "frr_pthread.h"
 
-#include "bgpd/bgpd.h"
+#include "bgpd/bgpd.c"
 #include "bgpd/bgp_open.h"
 #include "bgpd/bgp_debug.h"
 #include "bgpd/bgp_packet.h"
@@ -54,7 +54,7 @@ static int tty = 0;
 static struct test_segment {
 	const char *name;
 	const char *desc;
-	const u_char data[1024];
+	const uint8_t data[1024];
 	int len;
 #define SHOULD_PARSE	0
 #define SHOULD_ERR	-1
@@ -63,8 +63,8 @@ static struct test_segment {
 
 	/* AFI/SAFI validation */
 	int validate_afi;
-	afi_t afi;
-	safi_t safi;
+	iana_afi_t afi;
+	iana_safi_t safi;
 #define VALID_AFI 1
 #define INVALID_AFI 0
 	int afi_valid;
@@ -113,8 +113,8 @@ static struct test_segment mp_segments[] = {
 		SHOULD_PARSE,
 		0,
 		1,
-		AFI_IP,
-		SAFI_UNICAST,
+		IANA_AFI_IPV4,
+		IANA_SAFI_UNICAST,
 		VALID_AFI,
 	},
 	{
@@ -125,8 +125,8 @@ static struct test_segment mp_segments[] = {
 		SHOULD_PARSE,
 		0,
 		1,
-		AFI_IP6,
-		SAFI_UNICAST,
+		IANA_AFI_IPV6,
+		IANA_SAFI_UNICAST,
 		VALID_AFI,
 	},
 	/* 5 */
@@ -138,8 +138,8 @@ static struct test_segment mp_segments[] = {
 		SHOULD_PARSE,
 		0,
 		1,
-		AFI_IP,
-		SAFI_MULTICAST,
+		IANA_AFI_IPV4,
+		IANA_SAFI_MULTICAST,
 		VALID_AFI,
 	},
 	/* 6 */
@@ -151,7 +151,7 @@ static struct test_segment mp_segments[] = {
 		SHOULD_PARSE,
 		0,
 		1,
-		AFI_IP6,
+		IANA_AFI_IPV6,
 		IANA_SAFI_MPLS_VPN,
 		VALID_AFI,
 	},
@@ -164,20 +164,20 @@ static struct test_segment mp_segments[] = {
 		SHOULD_PARSE,
 		0,
 		1,
-		AFI_IP6,
+		IANA_AFI_IPV6,
 		IANA_SAFI_MPLS_VPN,
 		VALID_AFI,
 	},
 	/* 8 */
 	{
 		"MP6",
-		"MP IP4/MPLS-laveled VPN",
+		"MP IP4/MPLS-labeled VPN",
 		{CAPABILITY_CODE_MP, 0x4, 0x0, 0x1, 0x0, 0x80},
 		6,
 		SHOULD_PARSE,
 		0,
 		1,
-		AFI_IP,
+		IANA_AFI_IPV4,
 		IANA_SAFI_MPLS_VPN,
 		VALID_AFI,
 	},
@@ -211,8 +211,8 @@ static struct test_segment mp_segments[] = {
 		SHOULD_ERR,
 		0,
 		1,
-		AFI_IP,
-		SAFI_UNICAST,
+		IANA_AFI_IPV4,
+		IANA_SAFI_UNICAST,
 		VALID_AFI,
 	},
 	{NULL, NULL, {0}, 0, 0}};
@@ -797,14 +797,15 @@ static void parse_test(struct peer *peer, struct test_segment *t, int type)
 	int oldfailed = failed;
 	int len = t->len;
 #define RANDOM_FUZZ 35
-	stream_reset(peer->ibuf);
-	stream_put(peer->ibuf, NULL, RANDOM_FUZZ);
-	stream_set_getp(peer->ibuf, RANDOM_FUZZ);
+
+	stream_reset(peer->curr);
+	stream_put(peer->curr, NULL, RANDOM_FUZZ);
+	stream_set_getp(peer->curr, RANDOM_FUZZ);
 
 	switch (type) {
 	case CAPABILITY:
-		stream_putc(peer->ibuf, BGP_OPEN_OPT_CAP);
-		stream_putc(peer->ibuf, t->len);
+		stream_putc(peer->curr, BGP_OPEN_OPT_CAP);
+		stream_putc(peer->curr, t->len);
 		break;
 	case DYNCAP:
 		/*        for (i = 0; i < BGP_MARKER_SIZE; i++)
@@ -813,7 +814,7 @@ static void parse_test(struct peer *peer, struct test_segment *t, int type)
 			stream_putc (s, BGP_MSG_CAPABILITY);*/
 		break;
 	}
-	stream_write(peer->ibuf, t->data, t->len);
+	stream_write(peer->curr, t->data, t->len);
 
 	printf("%s: %s\n", t->name, t->desc);
 
@@ -826,7 +827,7 @@ static void parse_test(struct peer *peer, struct test_segment *t, int type)
 		as4 = peek_for_as4_capability(peer, len);
 		printf("peek_for_as4: as4 is %u\n", as4);
 		/* and it should leave getp as it found it */
-		assert(stream_get_getp(peer->ibuf) == RANDOM_FUZZ);
+		assert(stream_get_getp(peer->curr) == RANDOM_FUZZ);
 
 		ret = bgp_open_option_parse(peer, len, &capability);
 		break;
@@ -838,13 +839,12 @@ static void parse_test(struct peer *peer, struct test_segment *t, int type)
 		exit(1);
 	}
 
-	if (!ret && t->validate_afi) {
+	if (ret != BGP_Stop && t->validate_afi) {
 		afi_t afi;
 		safi_t safi;
 
 		/* Convert AFI, SAFI to internal values, check. */
-		if (bgp_map_afi_safi_iana2int(afi_int2iana(t->afi), t->safi,
-					      &afi, &safi)) {
+		if (bgp_map_afi_safi_iana2int(t->afi, t->safi, &afi, &safi)) {
 			if (t->afi_valid == VALID_AFI)
 				failed++;
 		}
@@ -866,10 +866,20 @@ static void parse_test(struct peer *peer, struct test_segment *t, int type)
 		failed++;
 	}
 
+	/*
+	 * Some of the functions used return BGP_Stop on error and some return
+	 * -1. If we have -1, keep it; if we have BGP_Stop, transform it to the
+	 * correct pass/fail code
+	 */
+	if (ret != -1)
+		ret = (ret == BGP_Stop) ? -1 : 0;
+
 	printf("parsed?: %s\n", ret ? "no" : "yes");
 
-	if (ret != t->parses)
+	if (ret != t->parses) {
+		printf("t->parses: %d\nret: %d\n", t->parses, ret);
 		failed++;
+	}
 
 	if (tty)
 		printf("%s",
@@ -900,11 +910,14 @@ int main(void)
 	term_bgp_debug_as4 = -1UL;
 
 	qobj_init();
-	master = thread_master_create();
+	master = thread_master_create(NULL);
 	bgp_master_init(master);
-	vrf_init();
+	vrf_init(NULL, NULL, NULL, NULL);
 	bgp_option_set(BGP_OPT_NO_LISTEN);
 
+	bgp_pthreads_init();
+	frr_pthread_get(PTHREAD_KEEPALIVES)->running = true;
+
 	if (fileno(stdout) >= 0)
 		tty = isatty(fileno(stdout));
 
@@ -920,6 +933,8 @@ int main(void)
 			peer->afc_adv[i][j] = 1;
 		}
 
+	peer->curr = stream_new(BGP_MAX_PACKET_SIZE);
+
 	i = 0;
 	while (mp_segments[i].name)
 		parse_test(peer, &mp_segments[i++], CAPABILITY);
