@@ -118,6 +118,10 @@ class Config(object):
 
         for line in file_output.split('\n'):
             line = line.strip()
+
+            # Compress duplicate whitespaces
+            line = ' '.join(line.split())
+
             if ":" in line:
                 qv6_line = get_normalized_ipv6_line(line)
                 self.lines.append(qv6_line)
@@ -319,6 +323,17 @@ end
  neighbor 2001:10::3 peer-group IBGPv6
  exit-address-family
 !
+end
+ address-family evpn
+  neighbor LEAF activate
+  advertise-all-vni
+  vni 10100
+   rd 65000:10100
+   route-target import 10.1.1.1:10100
+   route-target export 10.1.1.1:10100
+  exit-vni
+ exit-address-family
+!
 end
 router ospf
  ospf router-id 10.0.0.1
@@ -398,7 +413,7 @@ end
                 ctx_keys = []
                 current_context_lines = []
 
-            elif line == "exit-address-family" or line == "exit":
+            elif line in ["exit-address-family", "exit", "exit-vnc", "exit-vni"]:
                 # if this exit is for address-family ipv4 unicast, ignore the pop
                 if main_ctx_key:
                     self.save_contexts(ctx_keys, current_context_lines)
@@ -418,8 +433,14 @@ end
                 current_context_lines = []
                 new_ctx = False
                 log.debug('LINE %-50s: entering new context, %-50s', line, ctx_keys)
-
-            elif "address-family " in line:
+            elif (line.startswith("address-family ") or
+                  line.startswith("vnc defaults") or
+                  line.startswith("vnc l2-group") or
+                  line.startswith("vnc nve-group") or
+                  (line.startswith("vni ") and
+                   len(ctx_keys) == 2 and
+                   ctx_keys[0].startswith('router bgp') and
+                   ctx_keys[1] == 'address-family l2vpn evpn')):
                 main_ctx_key = []
 
                 # Save old context first
@@ -432,6 +453,8 @@ end
                     ctx_keys.append("address-family ipv6 unicast")
                 elif line == "address-family ipv4":
                     ctx_keys.append("address-family ipv4 unicast")
+                elif line == "address-family evpn":
+                    ctx_keys.append("address-family l2vpn evpn")
                 else:
                     ctx_keys.append(line)
 
@@ -533,7 +556,15 @@ def line_for_vtysh_file(ctx_keys, line, delete):
             for ctx_key in ctx_keys:
                 cmd.append(ctx_key)
 
-    return '\n' + '\n'.join(cmd)
+    cmd = '\n' + '\n'.join(cmd)
+
+    # There are some commands that are on by default so their "no" form will be
+    # displayed in the config.  "no bgp default ipv4-unicast" is one of these.
+    # If we need to remove this line we do so by adding "bgp default ipv4-unicast",
+    # not by doing a "no no bgp default ipv4-unicast"
+    cmd = cmd.replace('no no ', '')
+
+    return cmd
 
 
 def get_normalized_ipv6_line(line):
@@ -566,10 +597,15 @@ def get_normalized_ipv6_line(line):
     return norm_line.strip()
 
 
-def line_exist(lines, target_ctx_keys, target_line):
+def line_exist(lines, target_ctx_keys, target_line, exact_match=True):
     for (ctx_keys, line) in lines:
-        if ctx_keys == target_ctx_keys and line == target_line:
-            return True
+        if ctx_keys == target_ctx_keys:
+            if exact_match:
+                if line == target_line:
+                    return True
+            else:
+                if line.startswith(target_line):
+                    return True
     return False
 
 
@@ -582,140 +618,154 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
     for (ctx_keys, line) in lines_to_del:
         deleted = False
 
-        if ctx_keys[0].startswith('router bgp') and line and line.startswith('neighbor '):
-            """
-            BGP changed how it displays swpX peers that are part of peer-group. Older
-            versions of frr would display these on separate lines:
-                neighbor swp1 interface
-                neighbor swp1 peer-group FOO
+        if ctx_keys[0].startswith('router bgp') and line:
 
-            but today we display via a single line
-                neighbor swp1 interface peer-group FOO
+            if line.startswith('neighbor '):
+                '''
+                BGP changed how it displays swpX peers that are part of peer-group. Older
+                versions of frr would display these on separate lines:
+                    neighbor swp1 interface
+                    neighbor swp1 peer-group FOO
 
-            This change confuses frr-reload.py so check to see if we are deleting
-                neighbor swp1 interface peer-group FOO
+                but today we display via a single line
+                    neighbor swp1 interface peer-group FOO
 
-            and adding
-                neighbor swp1 interface
-                neighbor swp1 peer-group FOO
+                This change confuses frr-reload.py so check to see if we are deleting
+                    neighbor swp1 interface peer-group FOO
 
-            If so then chop the del line and the corresponding add lines
-            """
+                and adding
+                    neighbor swp1 interface
+                    neighbor swp1 peer-group FOO
 
-            re_swpx_int_peergroup = re.search('neighbor (\S+) interface peer-group (\S+)', line)
-            re_swpx_int_v6only_peergroup = re.search('neighbor (\S+) interface v6only peer-group (\S+)', line)
+                If so then chop the del line and the corresponding add lines
+                '''
 
-            if re_swpx_int_peergroup or re_swpx_int_v6only_peergroup:
-                swpx_interface = None
-                swpx_peergroup = None
+                re_swpx_int_peergroup = re.search('neighbor (\S+) interface peer-group (\S+)', line)
+                re_swpx_int_v6only_peergroup = re.search('neighbor (\S+) interface v6only peer-group (\S+)', line)
 
-                if re_swpx_int_peergroup:
-                    swpx = re_swpx_int_peergroup.group(1)
-                    peergroup = re_swpx_int_peergroup.group(2)
-                    swpx_interface = "neighbor %s interface" % swpx
-                elif re_swpx_int_v6only_peergroup:
-                    swpx = re_swpx_int_v6only_peergroup.group(1)
-                    peergroup = re_swpx_int_v6only_peergroup.group(2)
-                    swpx_interface = "neighbor %s interface v6only" % swpx
+                if re_swpx_int_peergroup or re_swpx_int_v6only_peergroup:
+                    swpx_interface = None
+                    swpx_peergroup = None
 
-                swpx_peergroup = "neighbor %s peer-group %s" % (swpx, peergroup)
-                found_add_swpx_interface = line_exist(lines_to_add, ctx_keys, swpx_interface)
-                found_add_swpx_peergroup = line_exist(lines_to_add, ctx_keys, swpx_peergroup)
-                tmp_ctx_keys = tuple(list(ctx_keys))
+                    if re_swpx_int_peergroup:
+                        swpx = re_swpx_int_peergroup.group(1)
+                        peergroup = re_swpx_int_peergroup.group(2)
+                        swpx_interface = "neighbor %s interface" % swpx
+                    elif re_swpx_int_v6only_peergroup:
+                        swpx = re_swpx_int_v6only_peergroup.group(1)
+                        peergroup = re_swpx_int_v6only_peergroup.group(2)
+                        swpx_interface = "neighbor %s interface v6only" % swpx
 
-                if not found_add_swpx_peergroup:
-                    tmp_ctx_keys = list(ctx_keys)
-                    tmp_ctx_keys.append('address-family ipv4 unicast')
-                    tmp_ctx_keys = tuple(tmp_ctx_keys)
-                    found_add_swpx_peergroup = line_exist(lines_to_add, tmp_ctx_keys, swpx_peergroup)
+                    swpx_peergroup = "neighbor %s peer-group %s" % (swpx, peergroup)
+                    found_add_swpx_interface = line_exist(lines_to_add, ctx_keys, swpx_interface)
+                    found_add_swpx_peergroup = line_exist(lines_to_add, ctx_keys, swpx_peergroup)
+                    tmp_ctx_keys = tuple(list(ctx_keys))
 
                     if not found_add_swpx_peergroup:
                         tmp_ctx_keys = list(ctx_keys)
-                        tmp_ctx_keys.append('address-family ipv6 unicast')
+                        tmp_ctx_keys.append('address-family ipv4 unicast')
                         tmp_ctx_keys = tuple(tmp_ctx_keys)
                         found_add_swpx_peergroup = line_exist(lines_to_add, tmp_ctx_keys, swpx_peergroup)
 
-                if found_add_swpx_interface and found_add_swpx_peergroup:
+                        if not found_add_swpx_peergroup:
+                            tmp_ctx_keys = list(ctx_keys)
+                            tmp_ctx_keys.append('address-family ipv6 unicast')
+                            tmp_ctx_keys = tuple(tmp_ctx_keys)
+                            found_add_swpx_peergroup = line_exist(lines_to_add, tmp_ctx_keys, swpx_peergroup)
+
+                    if found_add_swpx_interface and found_add_swpx_peergroup:
+                        deleted = True
+                        lines_to_del_to_del.append((ctx_keys, line))
+                        lines_to_add_to_del.append((ctx_keys, swpx_interface))
+                        lines_to_add_to_del.append((tmp_ctx_keys, swpx_peergroup))
+
+                '''
+                We changed how we display the neighbor interface command. Older
+                versions of frr would display the following:
+                    neighbor swp1 interface
+                    neighbor swp1 remote-as external
+                    neighbor swp1 capability extended-nexthop
+
+                but today we display via a single line
+                    neighbor swp1 interface remote-as external
+
+                and capability extended-nexthop is no longer needed because we
+                automatically enable it when the neighbor is of type interface.
+
+                This change confuses frr-reload.py so check to see if we are deleting
+                    neighbor swp1 interface remote-as (external|internal|ASNUM)
+
+                and adding
+                    neighbor swp1 interface
+                    neighbor swp1 remote-as (external|internal|ASNUM)
+                    neighbor swp1 capability extended-nexthop
+
+                If so then chop the del line and the corresponding add lines
+                '''
+                re_swpx_int_remoteas = re.search('neighbor (\S+) interface remote-as (\S+)', line)
+                re_swpx_int_v6only_remoteas = re.search('neighbor (\S+) interface v6only remote-as (\S+)', line)
+
+                if re_swpx_int_remoteas or re_swpx_int_v6only_remoteas:
+                    swpx_interface = None
+                    swpx_remoteas = None
+
+                    if re_swpx_int_remoteas:
+                        swpx = re_swpx_int_remoteas.group(1)
+                        remoteas = re_swpx_int_remoteas.group(2)
+                        swpx_interface = "neighbor %s interface" % swpx
+                    elif re_swpx_int_v6only_remoteas:
+                        swpx = re_swpx_int_v6only_remoteas.group(1)
+                        remoteas = re_swpx_int_v6only_remoteas.group(2)
+                        swpx_interface = "neighbor %s interface v6only" % swpx
+
+                    swpx_remoteas = "neighbor %s remote-as %s" % (swpx, remoteas)
+                    found_add_swpx_interface = line_exist(lines_to_add, ctx_keys, swpx_interface)
+                    found_add_swpx_remoteas = line_exist(lines_to_add, ctx_keys, swpx_remoteas)
+                    tmp_ctx_keys = tuple(list(ctx_keys))
+
+                    if found_add_swpx_interface and found_add_swpx_remoteas:
+                        deleted = True
+                        lines_to_del_to_del.append((ctx_keys, line))
+                        lines_to_add_to_del.append((ctx_keys, swpx_interface))
+                        lines_to_add_to_del.append((tmp_ctx_keys, swpx_remoteas))
+
+            '''
+            We made the 'bgp bestpath as-path multipath-relax' command
+            automatically assume 'no-as-set' since the lack of this option caused
+            weird routing problems. When the running config is shown in
+            releases with this change, the no-as-set keyword is not shown as it
+            is the default. This causes frr-reload to unnecessarily unapply
+            this option only to apply it back again, causing unnecessary session
+            resets.
+            '''
+            if 'multipath-relax' in line:
+                re_asrelax_new = re.search('^bgp\s+bestpath\s+as-path\s+multipath-relax$', line)
+                old_asrelax_cmd = 'bgp bestpath as-path multipath-relax no-as-set'
+                found_asrelax_old = line_exist(lines_to_add, ctx_keys, old_asrelax_cmd)
+
+                if re_asrelax_new and found_asrelax_old:
                     deleted = True
                     lines_to_del_to_del.append((ctx_keys, line))
-                    lines_to_add_to_del.append((ctx_keys, swpx_interface))
-                    lines_to_add_to_del.append((tmp_ctx_keys, swpx_peergroup))
+                    lines_to_add_to_del.append((ctx_keys, old_asrelax_cmd))
 
-            """
-            In 3.0.1 we changed how we display neighbor interface command. Older
-            versions of frr would display the following:
-                neighbor swp1 interface
-                neighbor swp1 remote-as external
-                neighbor swp1 capability extended-nexthop
+            '''
+            If we are modifying the BGP table-map we need to avoid a del/add and
+            instead modify the table-map in place via an add.  This is needed to
+            avoid installing all routes in the RIB the second the 'no table-map'
+            is issued.
+            '''
+            if line.startswith('table-map'):
+                found_table_map = line_exist(lines_to_add, ctx_keys, 'table-map', False)
 
-            but today we display via a single line
-                neighbor swp1 interface remote-as external
-
-            and capability extended-nexthop is no longer needed because we
-            automatically enable it when the neighbor is of type interface.
-
-            This change confuses frr-reload.py so check to see if we are deleting
-                neighbor swp1 interface remote-as (external|internal|ASNUM)
-
-            and adding
-                neighbor swp1 interface
-                neighbor swp1 remote-as (external|internal|ASNUM)
-                neighbor swp1 capability extended-nexthop
-
-            If so then chop the del line and the corresponding add lines
-            """
-            re_swpx_int_remoteas = re.search('neighbor (\S+) interface remote-as (\S+)', line)
-            re_swpx_int_v6only_remoteas = re.search('neighbor (\S+) interface v6only remote-as (\S+)', line)
-
-            if re_swpx_int_remoteas or re_swpx_int_v6only_remoteas:
-                swpx_interface = None
-                swpx_remoteas = None
-
-                if re_swpx_int_remoteas:
-                    swpx = re_swpx_int_remoteas.group(1)
-                    remoteas = re_swpx_int_remoteas.group(2)
-                    swpx_interface = "neighbor %s interface" % swpx
-                elif re_swpx_int_v6only_remoteas:
-                    swpx = re_swpx_int_v6only_remoteas.group(1)
-                    remoteas = re_swpx_int_v6only_remoteas.group(2)
-                    swpx_interface = "neighbor %s interface v6only" % swpx
-
-                swpx_remoteas = "neighbor %s remote-as %s" % (swpx, remoteas)
-                found_add_swpx_interface = line_exist(lines_to_add, ctx_keys, swpx_interface)
-                found_add_swpx_remoteas = line_exist(lines_to_add, ctx_keys, swpx_remoteas)
-                tmp_ctx_keys = tuple(list(ctx_keys))
-
-                if found_add_swpx_interface and found_add_swpx_remoteas:
-                    deleted = True
+                if found_table_map:
                     lines_to_del_to_del.append((ctx_keys, line))
-                    lines_to_add_to_del.append((ctx_keys, swpx_interface))
-                    lines_to_add_to_del.append((tmp_ctx_keys, swpx_remoteas))
 
         '''
-           In 3.0, we made bgp bestpath multipath as-relax command
-           automatically assume no-as-set since the lack of this option caused
-           weird routing problems and this problem was peculiar to this
-           implementation. When the running config is shown in relases after
-           3.0, the no-as-set is not shown as its the default. This causes
-           reload to unnecessarily unapply this option to only apply it back
-           again, causing unnecessary session resets. Handle this.
-        '''
-        if ctx_keys[0].startswith('router bgp') and line and 'multipath-relax' in line:
-            re_asrelax_new = re.search('^bgp\s+bestpath\s+as-path\s+multipath-relax$', line)
-            old_asrelax_cmd = 'bgp bestpath as-path multipath-relax no-as-set'
-            found_asrelax_old = line_exist(lines_to_add, ctx_keys, old_asrelax_cmd)
-
-            if re_asrelax_new and found_asrelax_old:
-                deleted = True
-                lines_to_del_to_del.append((ctx_keys, line))
-                lines_to_add_to_del.append((ctx_keys, old_asrelax_cmd))
-
-        '''
-           More old-to-new config handling. ip import-table no longer accepts
-           distance, but we honor the old syntax. But 'show running' shows only
-           the new syntax. This causes an unnecessary 'no import-table' followed
-           by the same old 'ip import-table' which causes perturbations in
-           announced routes leading to traffic blackholes. Fix this issue.
+        More old-to-new config handling. ip import-table no longer accepts
+        distance, but we honor the old syntax. But 'show running' shows only
+        the new syntax. This causes an unnecessary 'no import-table' followed
+        by the same old 'ip import-table' which causes perturbations in
+        announced routes leading to traffic blackholes. Fix this issue.
         '''
         re_importtbl = re.search('^ip\s+import-table\s+(\d+)$', ctx_keys[0])
         if re_importtbl:
@@ -745,6 +795,37 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
                     lines_to_del_to_del.append((ctx_keys, None))
                     lines_to_add_to_del.append(((tmpline,), None))
 
+        if (len(ctx_keys) == 3 and
+            ctx_keys[0].startswith('router bgp') and
+            ctx_keys[1] == 'address-family l2vpn evpn' and
+            ctx_keys[2].startswith('vni')):
+
+            re_route_target = re.search('^route-target import (.*)$', line) if line is not None else False
+
+            if re_route_target:
+                rt = re_route_target.group(1).strip()
+                route_target_import_line = line
+                route_target_export_line = "route-target export %s" % rt
+                route_target_both_line = "route-target both %s" % rt
+
+                found_route_target_export_line = line_exist(lines_to_del, ctx_keys, route_target_export_line)
+                found_route_target_both_line = line_exist(lines_to_add, ctx_keys, route_target_both_line)
+
+                '''
+                If the running configs has
+                    route-target import 1:1
+                    route-target export 1:1
+
+                and the config we are reloading against has
+                    route-target both 1:1
+
+                then we can ignore deleting the import/export and ignore adding the 'both'
+                '''
+                if found_route_target_export_line and found_route_target_both_line:
+                    lines_to_del_to_del.append((ctx_keys, route_target_import_line))
+                    lines_to_del_to_del.append((ctx_keys, route_target_export_line))
+                    lines_to_add_to_del.append((ctx_keys, route_target_both_line))
+
         if not deleted:
             found_add_line = line_exist(lines_to_add, ctx_keys, line)
 
@@ -788,6 +869,34 @@ def ignore_delete_re_add_lines(lines_to_add, lines_to_del):
     return (lines_to_add, lines_to_del)
 
 
+def ignore_unconfigurable_lines(lines_to_add, lines_to_del):
+    """
+    There are certain commands that cannot be removed.  Remove
+    those commands from lines_to_del.
+    """
+    lines_to_del_to_del = []
+
+    for (ctx_keys, line) in lines_to_del:
+
+        if (ctx_keys[0].startswith('frr version') or
+            ctx_keys[0].startswith('frr defaults') or
+            ctx_keys[0].startswith('password') or
+            ctx_keys[0].startswith('line vty') or
+
+            # This is technically "no"able but if we did so frr-reload would
+            # stop working so do not let the user shoot themselves in the foot
+            # by removing this.
+            ctx_keys[0].startswith('service integrated-vtysh-config')):
+
+            log.info("(%s, %s) cannot be removed" % (pformat(ctx_keys), line))
+            lines_to_del_to_del.append((ctx_keys, line))
+
+    for (ctx_keys, line) in lines_to_del_to_del:
+        lines_to_del.remove((ctx_keys, line))
+
+    return (lines_to_add, lines_to_del)
+
+
 def compare_context_objects(newconf, running):
     """
     Create a context diff for the two specified contexts
@@ -812,7 +921,7 @@ def compare_context_objects(newconf, running):
                 delete_bgpd = True
                 lines_to_del.append((running_ctx_keys, None))
 
-            # We cannot do 'no interface' in quagga, and so deal with it
+            # We cannot do 'no interface' in FRR, and so deal with it
             elif running_ctx_keys[0].startswith('interface'):
                 for line in running_ctx.lines:
                     lines_to_del.append((running_ctx_keys, line))
@@ -822,6 +931,13 @@ def compare_context_objects(newconf, running):
             elif "router bgp" in running_ctx_keys[0] and len(running_ctx_keys) > 1 and delete_bgpd:
                 continue
 
+            # Delete an entire vni sub-context under "address-family l2vpn evpn"
+            elif ("router bgp" in running_ctx_keys[0] and
+                  len(running_ctx_keys) > 2 and
+                  running_ctx_keys[1].startswith('address-family l2vpn evpn') and
+                  running_ctx_keys[2].startswith('vni ')):
+                lines_to_del.append((running_ctx_keys, None))
+
             elif ("router bgp" in running_ctx_keys[0] and
                   len(running_ctx_keys) > 1 and
                   running_ctx_keys[1].startswith('address-family')):
@@ -834,6 +950,9 @@ def compare_context_objects(newconf, running):
             elif running_ctx_keys and not any("address-family" in key for key in running_ctx_keys):
                 lines_to_del.append((running_ctx_keys, None))
 
+            elif running_ctx_keys and not any("vni" in key for key in running_ctx_keys):
+                lines_to_del.append((running_ctx_keys, None))
+
             # Global context
             else:
                 for line in running_ctx.lines:
@@ -863,9 +982,37 @@ def compare_context_objects(newconf, running):
                 lines_to_add.append((newconf_ctx_keys, line))
 
     (lines_to_add, lines_to_del) = ignore_delete_re_add_lines(lines_to_add, lines_to_del)
+    (lines_to_add, lines_to_del) = ignore_unconfigurable_lines(lines_to_add, lines_to_del)
 
     return (lines_to_add, lines_to_del)
 
+
+
+def vtysh_config_available():
+    """
+    Return False if no frr daemon is running or some other vtysh session is
+    in 'configuration terminal' mode which will prevent us from making any
+    configuration changes.
+    """
+
+    try:
+        cmd = ['/usr/bin/vtysh', '-c', 'conf t']
+        output = subprocess.check_output(cmd, stderr=subprocess.STDOUT).strip()
+
+        if 'VTY configuration is locked by other VTY' in output:
+            print output
+            log.error("'%s' returned\n%s\n" % (' '.join(cmd), output))
+            return False
+
+    except subprocess.CalledProcessError as e:
+        msg = "vtysh could not connect with any frr daemons"
+        print msg
+        log.error(msg)
+        return False
+
+    return True
+
+
 if __name__ == '__main__':
     # Command line options
     parser = argparse.ArgumentParser(description='Dynamically apply diff in frr configs')
@@ -986,6 +1133,10 @@ if __name__ == '__main__':
 
     elif args.reload:
 
+        # We will not be able to do anything, go ahead and exit(1)
+        if not vtysh_config_available():
+            sys.exit(1)
+
         log.debug('New Frr Config\n%s', newconf.get_lines())
 
         # This looks a little odd but we have to do this twice...here is why
@@ -1010,7 +1161,7 @@ if __name__ == '__main__':
         # the "neighbor 1.1.1.1 route-map FOO out" line...so we compare the
         # configs again to put this line back.
 
-        # There are many keywords in quagga that can only appear one time under
+        # There are many keywords in FRR that can only appear one time under
         # a context, take "bgp router-id" for example. If the config that we are
         # reloading against has the following:
         #
@@ -1038,7 +1189,7 @@ if __name__ == '__main__':
                 lines_to_add.extend(lines_to_add_first_pass)
 
             # Only do deletes on the first pass. The reason being if we
-            # configure a bgp neighbor via "neighbor swp1 interface" quagga
+            # configure a bgp neighbor via "neighbor swp1 interface" FRR
             # will automatically add:
             #
             # interface swp1
@@ -1078,7 +1229,7 @@ if __name__ == '__main__':
 
                     while True:
                         try:
-                            _ = subprocess.check_output(cmd)
+                            _ = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
 
                         except subprocess.CalledProcessError:
 
@@ -1122,19 +1273,11 @@ if __name__ == '__main__':
                         for line in lines_to_configure:
                             fh.write(line + '\n')
 
-                    output = subprocess.check_output(['/usr/bin/vtysh', '-f', filename])
-
-                    # exit non-zero if we see these errors
-                    for x in ('BGP instance name and AS number mismatch',
-                              'BGP instance is already running',
-                              '% not a local address'):
-                        for line in output.splitlines():
-                            if x in line:
-                                msg = "ERROR: %s" % x
-                                log.error(msg)
-                                print msg
-                                reload_ok = False
-
+                    try:
+                        subprocess.check_output(['/usr/bin/vtysh', '-f', filename], stderr=subprocess.STDOUT)
+                    except subprocess.CalledProcessError as e:
+                        log.warning("frr-reload.py failed due to\n%s" % e.output)
+                        reload_ok = False
                     os.unlink(filename)
 
         # Make these changes persistent
