@@ -14,13 +14,25 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
+
+#ifdef GNU_LINUX
+
+/* The following definition is to workaround an issue in the Linux kernel
+ * header files with redefinition of 'struct in6_addr' in both
+ * netinet/in.h and linux/in6.h.
+ * Reference - https://sourceware.org/ml/libc-alpha/2013-01/msg00599.html
+ */
+#define _LINUX_IN6_H
+
+#include <netinet/if_ether.h>
+#include <linux/if_bridge.h>
+#include <linux/if_link.h>
 #include <net/if_arp.h>
 #include <linux/sockios.h>
 #include <linux/ethtool.h>
@@ -38,6 +50,7 @@
 #include "privs.h"
 #include "nexthop.h"
 #include "vrf.h"
+#include "vrf_int.h"
 #include "mpls.h"
 
 #include "vty.h"
@@ -54,6 +67,7 @@
 #include "zebra/kernel_netlink.h"
 #include "zebra/if_netlink.h"
 
+extern struct zebra_privs_t zserv_privs;
 
 /* Note: on netlink systems, there should be a 1-to-1 mapping between interface
    names and ifindex values. */
@@ -82,7 +96,7 @@ static void set_ifindex(struct interface *ifp, ifindex_t ifi_index,
 			if_delete_update(oifp);
 		}
 	}
-	ifp->ifindex = ifi_index;
+	if_set_index(ifp, ifi_index);
 }
 
 /* Utility function to parse hardware link-layer address and update ifp */
@@ -209,8 +223,10 @@ static enum zebra_link_type netlink_to_zebra_link_type(unsigned int hwt)
 		return ZEBRA_LLT_IEEE802_TR;
 	case ARPHRD_IEEE80211:
 		return ZEBRA_LLT_IEEE80211;
+#ifdef ARPHRD_IEEE802154
 	case ARPHRD_IEEE802154:
 		return ZEBRA_LLT_IEEE802154;
+#endif
 #ifdef ARPHRD_IP6GRE
 	case ARPHRD_IP6GRE:
 		return ZEBRA_LLT_IP6GRE;
@@ -225,47 +241,24 @@ static enum zebra_link_type netlink_to_zebra_link_type(unsigned int hwt)
 	}
 }
 
+static void netlink_determine_zebra_iftype(char *kind, zebra_iftype_t *zif_type)
+{
+	*zif_type = ZEBRA_IF_OTHER;
 
-// Temporary Assignments to compile on older platforms.
-#ifndef IFLA_BR_MAX
-#define IFLA_BR_MAX   39
-#endif
+	if (!kind)
+		return;
 
-#ifndef IFLA_VXLAN_ID
-#define IFLA_VXLAN_ID 1
-#endif
-
-#ifndef IFLA_VXLAN_LOCAL
-#define IFLA_VXLAN_LOCAL  4
-#endif
-
-#ifndef IFLA_VXLAN_MAX
-#define IFLA_VXLAN_MAX 26
-#endif
-
-#ifndef IFLA_BRIDGE_MAX
-#define IFLA_BRIDGE_MAX   2
-#endif
-
-#ifndef IFLA_BRIDGE_VLAN_INFO
-#define IFLA_BRIDGE_VLAN_INFO 2
-#endif
-
-#ifndef BRIDGE_VLAN_INFO_PVID
-#define BRIDGE_VLAN_INFO_PVID  (1<<1)
-#endif
-
-#ifndef RTEXT_FILTER_BRVLAN
-#define RTEXT_FILTER_BRVLAN    (1<<1)
-#endif
-
-#ifndef NTF_SELF
-#define NTF_SELF   0x02
-#endif
-
-#ifndef IFLA_BR_VLAN_FILTERING
-#define IFLA_BR_VLAN_FILTERING  7
-#endif
+	if (strcmp(kind, "vrf") == 0)
+		*zif_type = ZEBRA_IF_VRF;
+	else if (strcmp(kind, "bridge") == 0)
+		*zif_type = ZEBRA_IF_BRIDGE;
+	else if (strcmp(kind, "vlan") == 0)
+		*zif_type = ZEBRA_IF_VLAN;
+	else if (strcmp(kind, "vxlan") == 0)
+		*zif_type = ZEBRA_IF_VXLAN;
+	else if (strcmp(kind, "macvlan") == 0)
+		*zif_type = ZEBRA_IF_MACVLAN;
+}
 
 #define parse_rtattr_nested(tb, max, rta)                                      \
 	netlink_parse_rtattr((tb), (max), RTA_DATA(rta), RTA_PAYLOAD(rta))
@@ -278,7 +271,7 @@ static void netlink_vrf_change(struct nlmsghdr *h, struct rtattr *tb,
 	struct rtattr *attr[IFLA_VRF_MAX + 1];
 	struct vrf *vrf;
 	struct zebra_vrf *zvrf;
-	u_int32_t nl_table_id;
+	uint32_t nl_table_id;
 
 	ifi = NLMSG_DATA(h);
 
@@ -303,7 +296,7 @@ static void netlink_vrf_change(struct nlmsghdr *h, struct rtattr *tb,
 		return;
 	}
 
-	nl_table_id = *(u_int32_t *)RTA_DATA(attr[IFLA_VRF_TABLE]);
+	nl_table_id = *(uint32_t *)RTA_DATA(attr[IFLA_VRF_TABLE]);
 
 	if (h->nlmsg_type == RTM_NEWLINK) {
 		if (IS_ZEBRA_DEBUG_KERNEL)
@@ -321,6 +314,21 @@ static void netlink_vrf_change(struct nlmsghdr *h, struct rtattr *tb,
 			return;
 		}
 
+		/*
+		 * This is the only place that we get the actual kernel table_id
+		 * being used.  We need it to set the table_id of the routes
+		 * we are passing to the kernel.... And to throw some totally
+		 * awesome parties. that too.
+		 *
+		 * At this point we *must* have a zvrf because the vrf_create
+		 * callback creates one.  We *must* set the table id
+		 * before the vrf_enable because of( at the very least )
+		 * static routes being delayed for installation until
+		 * during the vrf_enable callbacks.
+		 */
+		zvrf = (struct zebra_vrf *)vrf->info;
+		zvrf->table_id = nl_table_id;
+
 		/* Enable the created VRF. */
 		if (!vrf_enable(vrf)) {
 			zlog_err("Failed to enable VRF %s id %u", name,
@@ -328,14 +336,6 @@ static void netlink_vrf_change(struct nlmsghdr *h, struct rtattr *tb,
 			return;
 		}
 
-		/*
-		 * This is the only place that we get the actual kernel table_id
-		 * being used.  We need it to set the table_id of the routes
-		 * we are passing to the kernel.... And to throw some totally
-		 * awesome parties. that too.
-		 */
-		zvrf = (struct zebra_vrf *)vrf->info;
-		zvrf->table_id = nl_table_id;
 	} else // h->nlmsg_type == RTM_DELLINK
 	{
 		if (IS_ZEBRA_DEBUG_KERNEL)
@@ -353,37 +353,45 @@ static void netlink_vrf_change(struct nlmsghdr *h, struct rtattr *tb,
 	}
 }
 
-static int get_iflink_speed(const char *ifname)
+static int get_iflink_speed(struct interface *interface)
 {
 	struct ifreq ifdata;
 	struct ethtool_cmd ecmd;
 	int sd;
 	int rc;
+	const char *ifname = interface->name;
 
 	/* initialize struct */
 	memset(&ifdata, 0, sizeof(ifdata));
 
 	/* set interface name */
-	strcpy(ifdata.ifr_name, ifname);
+	strlcpy(ifdata.ifr_name, ifname, sizeof(ifdata.ifr_name));
 
 	/* initialize ethtool interface */
 	memset(&ecmd, 0, sizeof(ecmd));
 	ecmd.cmd = ETHTOOL_GSET; /* ETHTOOL_GLINK */
-	ifdata.ifr_data = (__caddr_t)&ecmd;
+	ifdata.ifr_data = (caddr_t)&ecmd;
 
 	/* use ioctl to get IP address of an interface */
-	sd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
+	if (zserv_privs.change(ZPRIVS_RAISE))
+		zlog_err("Can't raise privileges");
+	sd = vrf_socket(PF_INET, SOCK_DGRAM, IPPROTO_IP, interface->vrf_id,
+			NULL);
 	if (sd < 0) {
-		zlog_debug("Failure to read interface %s speed: %d %s", ifname,
-			   errno, safe_strerror(errno));
+		if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("Failure to read interface %s speed: %d %s",
+				   ifname, errno, safe_strerror(errno));
 		return 0;
 	}
-
 	/* Get the current link state for the interface */
-	rc = ioctl(sd, SIOCETHTOOL, (char *)&ifdata);
+	rc = vrf_ioctl(interface->vrf_id, sd, SIOCETHTOOL, (char *)&ifdata);
+	if (zserv_privs.change(ZPRIVS_LOWER))
+		zlog_err("Can't lower privileges");
 	if (rc < 0) {
-		zlog_debug("IOCTL failure to read interface %s speed: %d %s",
-			   ifname, errno, safe_strerror(errno));
+		if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug(
+				"IOCTL failure to read interface %s speed: %d %s",
+				ifname, errno, safe_strerror(errno));
 		ecmd.speed_hi = 0;
 		ecmd.speed = 0;
 	}
@@ -393,6 +401,162 @@ static int get_iflink_speed(const char *ifname)
 	return (ecmd.speed_hi << 16) | ecmd.speed;
 }
 
+uint32_t kernel_get_speed(struct interface *ifp)
+{
+	return get_iflink_speed(ifp);
+}
+
+static int netlink_extract_bridge_info(struct rtattr *link_data,
+				       struct zebra_l2info_bridge *bridge_info)
+{
+	struct rtattr *attr[IFLA_BR_MAX + 1];
+
+	memset(bridge_info, 0, sizeof(*bridge_info));
+	memset(attr, 0, sizeof attr);
+	parse_rtattr_nested(attr, IFLA_BR_MAX, link_data);
+	if (attr[IFLA_BR_VLAN_FILTERING])
+		bridge_info->vlan_aware =
+			*(uint8_t *)RTA_DATA(attr[IFLA_BR_VLAN_FILTERING]);
+	return 0;
+}
+
+static int netlink_extract_vlan_info(struct rtattr *link_data,
+				     struct zebra_l2info_vlan *vlan_info)
+{
+	struct rtattr *attr[IFLA_VLAN_MAX + 1];
+	vlanid_t vid_in_msg;
+
+	memset(vlan_info, 0, sizeof(*vlan_info));
+	memset(attr, 0, sizeof attr);
+	parse_rtattr_nested(attr, IFLA_VLAN_MAX, link_data);
+	if (!attr[IFLA_VLAN_ID]) {
+		if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("IFLA_VLAN_ID missing from VLAN IF message");
+		return -1;
+	}
+
+	vid_in_msg = *(vlanid_t *)RTA_DATA(attr[IFLA_VLAN_ID]);
+	vlan_info->vid = vid_in_msg;
+	return 0;
+}
+
+static int netlink_extract_vxlan_info(struct rtattr *link_data,
+				      struct zebra_l2info_vxlan *vxl_info)
+{
+	struct rtattr *attr[IFLA_VXLAN_MAX + 1];
+	vni_t vni_in_msg;
+	struct in_addr vtep_ip_in_msg;
+
+	memset(vxl_info, 0, sizeof(*vxl_info));
+	memset(attr, 0, sizeof attr);
+	parse_rtattr_nested(attr, IFLA_VXLAN_MAX, link_data);
+	if (!attr[IFLA_VXLAN_ID]) {
+		if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug(
+				"IFLA_VXLAN_ID missing from VXLAN IF message");
+		return -1;
+	}
+
+	vni_in_msg = *(vni_t *)RTA_DATA(attr[IFLA_VXLAN_ID]);
+	vxl_info->vni = vni_in_msg;
+	if (!attr[IFLA_VXLAN_LOCAL]) {
+		if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug(
+				"IFLA_VXLAN_LOCAL missing from VXLAN IF message");
+	} else {
+		vtep_ip_in_msg =
+			*(struct in_addr *)RTA_DATA(attr[IFLA_VXLAN_LOCAL]);
+		vxl_info->vtep_ip = vtep_ip_in_msg;
+	}
+
+	return 0;
+}
+
+/*
+ * Extract and save L2 params (of interest) for an interface. When a
+ * bridge interface is added or updated, take further actions to map
+ * its members. Likewise, for VxLAN interface.
+ */
+static void netlink_interface_update_l2info(struct interface *ifp,
+					    struct rtattr *link_data, int add)
+{
+	if (!link_data)
+		return;
+
+	if (IS_ZEBRA_IF_BRIDGE(ifp)) {
+		struct zebra_l2info_bridge bridge_info;
+
+		netlink_extract_bridge_info(link_data, &bridge_info);
+		zebra_l2_bridge_add_update(ifp, &bridge_info, add);
+	} else if (IS_ZEBRA_IF_VLAN(ifp)) {
+		struct zebra_l2info_vlan vlan_info;
+
+		netlink_extract_vlan_info(link_data, &vlan_info);
+		zebra_l2_vlanif_update(ifp, &vlan_info);
+	} else if (IS_ZEBRA_IF_VXLAN(ifp)) {
+		struct zebra_l2info_vxlan vxlan_info;
+
+		netlink_extract_vxlan_info(link_data, &vxlan_info);
+		zebra_l2_vxlanif_add_update(ifp, &vxlan_info, add);
+	}
+}
+
+static int netlink_bridge_interface(struct nlmsghdr *h, int len, ns_id_t ns_id,
+				    int startup)
+{
+	char *name = NULL;
+	struct ifinfomsg *ifi;
+	struct rtattr *tb[IFLA_MAX + 1];
+	struct interface *ifp;
+	struct rtattr *aftb[IFLA_BRIDGE_MAX + 1];
+	struct {
+		uint16_t flags;
+		uint16_t vid;
+	} * vinfo;
+	vlanid_t access_vlan;
+
+	/* Fetch name and ifindex */
+	ifi = NLMSG_DATA(h);
+	memset(tb, 0, sizeof tb);
+	netlink_parse_rtattr(tb, IFLA_MAX, IFLA_RTA(ifi), len);
+
+	if (tb[IFLA_IFNAME] == NULL)
+		return -1;
+	name = (char *)RTA_DATA(tb[IFLA_IFNAME]);
+
+	/* The interface should already be known, if not discard. */
+	ifp = if_lookup_by_index_per_ns(zebra_ns_lookup(ns_id), ifi->ifi_index);
+	if (!ifp) {
+		zlog_warn("Cannot find bridge IF %s(%u)", name, ifi->ifi_index);
+		return 0;
+	}
+	if (!IS_ZEBRA_IF_VXLAN(ifp))
+		return 0;
+
+	/* We are only interested in the access VLAN i.e., AF_SPEC */
+	if (!tb[IFLA_AF_SPEC])
+		return 0;
+
+	/* There is a 1-to-1 mapping of VLAN to VxLAN - hence
+	 * only 1 access VLAN is accepted.
+	 */
+	memset(aftb, 0, sizeof aftb);
+	parse_rtattr_nested(aftb, IFLA_BRIDGE_MAX, tb[IFLA_AF_SPEC]);
+	if (!aftb[IFLA_BRIDGE_VLAN_INFO])
+		return 0;
+
+	vinfo = RTA_DATA(aftb[IFLA_BRIDGE_VLAN_INFO]);
+	if (!(vinfo->flags & BRIDGE_VLAN_INFO_PVID))
+		return 0;
+
+	access_vlan = (vlanid_t)vinfo->vid;
+	if (IS_ZEBRA_DEBUG_KERNEL)
+		zlog_debug("Access VLAN %u for VxLAN IF %s(%u)", access_vlan,
+			   name, ifi->ifi_index);
+	zebra_l2_vxlanif_update_access_vlan(ifp, access_vlan);
+	return 0;
+}
+
 /* Called from interface_lookup_netlink().  This function is only used
    during bootstrap. */
 static int netlink_interface(struct sockaddr_nl *snl, struct nlmsghdr *h,
@@ -405,10 +569,14 @@ static int netlink_interface(struct sockaddr_nl *snl, struct nlmsghdr *h,
 	struct interface *ifp;
 	char *name = NULL;
 	char *kind = NULL;
+	char *desc = NULL;
 	char *slave_kind = NULL;
-	int vrf_device = 0;
 	struct zebra_ns *zns;
 	vrf_id_t vrf_id = VRF_DEFAULT;
+	zebra_iftype_t zif_type = ZEBRA_IF_OTHER;
+	zebra_slave_iftype_t zif_slave_type = ZEBRA_IF_SLAVE_NONE;
+	ifindex_t bridge_ifindex = IFINDEX_INTERNAL;
+	ifindex_t link_ifindex = IFINDEX_INTERNAL;
 
 	zns = zebra_ns_lookup(ns_id);
 	ifi = NLMSG_DATA(h);
@@ -420,11 +588,13 @@ static int netlink_interface(struct sockaddr_nl *snl, struct nlmsghdr *h,
 	if (len < 0)
 		return -1;
 
+	/* We are interested in some AF_BRIDGE notifications. */
 	if (ifi->ifi_family == AF_BRIDGE)
-		return 0;
+		return netlink_bridge_interface(h, len, ns_id, startup);
 
 	/* Looking up interface name. */
 	memset(tb, 0, sizeof tb);
+	memset(linkinfo, 0, sizeof linkinfo);
 	netlink_parse_rtattr(tb, IFLA_MAX, IFLA_RTA(ifi), len);
 
 #ifdef IFLA_WIRELESS
@@ -441,8 +611,10 @@ static int netlink_interface(struct sockaddr_nl *snl, struct nlmsghdr *h,
 		return -1;
 	name = (char *)RTA_DATA(tb[IFLA_IFNAME]);
 
+	if (tb[IFLA_IFALIAS])
+		desc = (char *)RTA_DATA(tb[IFLA_IFALIAS]);
+
 	if (tb[IFLA_LINKINFO]) {
-		memset(linkinfo, 0, sizeof linkinfo);
 		parse_rtattr_nested(linkinfo, IFLA_INFO_MAX, tb[IFLA_LINKINFO]);
 
 		if (linkinfo[IFLA_INFO_KIND])
@@ -453,45 +625,99 @@ static int netlink_interface(struct sockaddr_nl *snl, struct nlmsghdr *h,
 			slave_kind = RTA_DATA(linkinfo[IFLA_INFO_SLAVE_KIND]);
 #endif
 
-		if (kind && strcmp(kind, "vrf") == 0) {
-			vrf_device = 1;
-			netlink_vrf_change(h, tb[IFLA_LINKINFO], name);
-			vrf_id = (vrf_id_t)ifi->ifi_index;
-		}
+		netlink_determine_zebra_iftype(kind, &zif_type);
+	}
+
+	/* If VRF, create the VRF structure itself. */
+	if (zif_type == ZEBRA_IF_VRF && !vrf_is_backend_netns()) {
+		netlink_vrf_change(h, tb[IFLA_LINKINFO], name);
+		vrf_id = (vrf_id_t)ifi->ifi_index;
 	}
 
 	if (tb[IFLA_MASTER]) {
-		if (slave_kind && (strcmp(slave_kind, "vrf") == 0))
-			vrf_id = *(u_int32_t *)RTA_DATA(tb[IFLA_MASTER]);
+		if (slave_kind && (strcmp(slave_kind, "vrf") == 0)
+		    && !vrf_is_backend_netns()) {
+			zif_slave_type = ZEBRA_IF_SLAVE_VRF;
+			vrf_id = *(uint32_t *)RTA_DATA(tb[IFLA_MASTER]);
+		} else if (slave_kind && (strcmp(slave_kind, "bridge") == 0)) {
+			zif_slave_type = ZEBRA_IF_SLAVE_BRIDGE;
+			bridge_ifindex =
+				*(ifindex_t *)RTA_DATA(tb[IFLA_MASTER]);
+		} else
+			zif_slave_type = ZEBRA_IF_SLAVE_OTHER;
 	}
+	if (vrf_is_backend_netns())
+		vrf_id = (vrf_id_t)ns_id;
+
+	/* If linking to another interface, note it. */
+	if (tb[IFLA_LINK])
+		link_ifindex = *(ifindex_t *)RTA_DATA(tb[IFLA_LINK]);
 
 	/* Add interface. */
-	ifp = if_get_by_name(name, vrf_id);
+	ifp = if_get_by_name(name, vrf_id, 0);
 	set_ifindex(ifp, ifi->ifi_index, zns);
 	ifp->flags = ifi->ifi_flags & 0x0000fffff;
-	if (vrf_device)
-		SET_FLAG(ifp->status, ZEBRA_INTERFACE_VRF_LOOPBACK);
 	ifp->mtu6 = ifp->mtu = *(uint32_t *)RTA_DATA(tb[IFLA_MTU]);
 	ifp->metric = 0;
-	ifp->speed = get_iflink_speed(name);
+	ifp->speed = get_iflink_speed(ifp);
 	ifp->ptm_status = ZEBRA_PTM_STATUS_UNKNOWN;
 
+	if (desc)
+		ifp->desc = XSTRDUP(MTYPE_TMP, desc);
+
+	/* Set zebra interface type */
+	zebra_if_set_ziftype(ifp, zif_type, zif_slave_type);
+	if (IS_ZEBRA_IF_VRF(ifp))
+		SET_FLAG(ifp->status, ZEBRA_INTERFACE_VRF_LOOPBACK);
+
+	/* Update link. */
+	zebra_if_update_link(ifp, link_ifindex);
+
 	/* Hardware type and address. */
 	ifp->ll_type = netlink_to_zebra_link_type(ifi->ifi_type);
 	netlink_interface_update_hw_addr(tb, ifp);
 
 	if_add_update(ifp);
 
+	/* Extract and save L2 interface information, take additional actions.
+	 */
+	netlink_interface_update_l2info(ifp, linkinfo[IFLA_INFO_DATA], 1);
+	if (IS_ZEBRA_IF_BRIDGE_SLAVE(ifp))
+		zebra_l2if_update_bridge_slave(ifp, bridge_ifindex);
+
 	return 0;
 }
 
+/* Request for specific interface or address information from the kernel */
+static int netlink_request_intf_addr(struct zebra_ns *zns, int family, int type,
+				     uint32_t filter_mask)
+{
+	struct {
+		struct nlmsghdr n;
+		struct ifinfomsg ifm;
+		char buf[256];
+	} req;
+
+	/* Form the request, specifying filter (rtattr) if needed. */
+	memset(&req, 0, sizeof(req));
+	req.n.nlmsg_type = type;
+	req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct ifinfomsg));
+	req.ifm.ifi_family = family;
+
+	/* Include filter, if specified. */
+	if (filter_mask)
+		addattr32(&req.n, sizeof(req), IFLA_EXT_MASK, filter_mask);
+
+	return netlink_request(&zns->netlink_cmd, &req.n);
+}
+
 /* Interface lookup by netlink socket. */
 int interface_lookup_netlink(struct zebra_ns *zns)
 {
 	int ret;
 
 	/* Get interface information. */
-	ret = netlink_request(AF_PACKET, RTM_GETLINK, &zns->netlink_cmd);
+	ret = netlink_request_intf_addr(zns, AF_PACKET, RTM_GETLINK, 0);
 	if (ret < 0)
 		return ret;
 	ret = netlink_parse_info(netlink_interface, &zns->netlink_cmd, zns, 0,
@@ -499,8 +725,28 @@ int interface_lookup_netlink(struct zebra_ns *zns)
 	if (ret < 0)
 		return ret;
 
+	/* Get interface information - for bridge interfaces. */
+	ret = netlink_request_intf_addr(zns, AF_BRIDGE, RTM_GETLINK,
+					RTEXT_FILTER_BRVLAN);
+	if (ret < 0)
+		return ret;
+	ret = netlink_parse_info(netlink_interface, &zns->netlink_cmd, zns, 0,
+				 0);
+	if (ret < 0)
+		return ret;
+
+	/* Get interface information - for bridge interfaces. */
+	ret = netlink_request_intf_addr(zns, AF_BRIDGE, RTM_GETLINK,
+					RTEXT_FILTER_BRVLAN);
+	if (ret < 0)
+		return ret;
+	ret = netlink_parse_info(netlink_interface, &zns->netlink_cmd, zns, 0,
+				 0);
+	if (ret < 0)
+		return ret;
+
 	/* Get IPv4 address of the interfaces. */
-	ret = netlink_request(AF_INET, RTM_GETADDR, &zns->netlink_cmd);
+	ret = netlink_request_intf_addr(zns, AF_INET, RTM_GETADDR, 0);
 	if (ret < 0)
 		return ret;
 	ret = netlink_parse_info(netlink_interface_addr, &zns->netlink_cmd, zns,
@@ -509,7 +755,7 @@ int interface_lookup_netlink(struct zebra_ns *zns)
 		return ret;
 
 	/* Get IPv6 address of the interfaces. */
-	ret = netlink_request(AF_INET6, RTM_GETADDR, &zns->netlink_cmd);
+	ret = netlink_request_intf_addr(zns, AF_INET6, RTM_GETADDR, 0);
 	if (ret < 0)
 		return ret;
 	ret = netlink_parse_info(netlink_interface_addr, &zns->netlink_cmd, zns,
@@ -520,6 +766,33 @@ int interface_lookup_netlink(struct zebra_ns *zns)
 	return 0;
 }
 
+int kernel_interface_set_master(struct interface *master,
+				struct interface *slave)
+{
+	struct zebra_ns *zns = zebra_ns_lookup(NS_DEFAULT);
+
+	struct {
+		struct nlmsghdr n;
+		struct ifinfomsg ifa;
+		char buf[NL_PKT_BUF_SIZE];
+	} req;
+
+	memset(&req, 0, sizeof req);
+
+	req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct ifinfomsg));
+	req.n.nlmsg_flags = NLM_F_REQUEST;
+	req.n.nlmsg_type = RTM_SETLINK;
+	req.n.nlmsg_pid = zns->netlink_cmd.snl.nl_pid;
+
+	req.ifa.ifi_index = slave->ifindex;
+
+	addattr_l(&req.n, sizeof req, IFLA_MASTER, &master->ifindex, 4);
+	addattr_l(&req.n, sizeof req, IFLA_LINK, &slave->ifindex, 4);
+
+	return netlink_talk(netlink_talk_filter, &req.n, &zns->netlink_cmd, zns,
+			    0);
+}
+
 /* Interface address modification. */
 static int netlink_address(int cmd, int family, struct interface *ifp,
 			   struct connected *ifc)
@@ -533,8 +806,12 @@ static int netlink_address(int cmd, int family, struct interface *ifp,
 		char buf[NL_PKT_BUF_SIZE];
 	} req;
 
-	struct zebra_ns *zns = zebra_ns_lookup(NS_DEFAULT);
+	struct zebra_ns *zns;
 
+	if (vrf_is_backend_netns())
+		zns = zebra_ns_lookup((ns_id_t)ifp->vrf_id);
+	else
+		zns = zebra_ns_lookup(NS_DEFAULT);
 	p = ifc->address;
 	memset(&req, 0, sizeof req - NL_PKT_BUF_SIZE);
 
@@ -543,21 +820,29 @@ static int netlink_address(int cmd, int family, struct interface *ifp,
 	req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct ifaddrmsg));
 	req.n.nlmsg_flags = NLM_F_REQUEST;
 	req.n.nlmsg_type = cmd;
+	req.n.nlmsg_pid = zns->netlink_cmd.snl.nl_pid;
+
 	req.ifa.ifa_family = family;
 
 	req.ifa.ifa_index = ifp->ifindex;
-	req.ifa.ifa_prefixlen = p->prefixlen;
 
 	addattr_l(&req.n, sizeof req, IFA_LOCAL, &p->u.prefix, bytelen);
 
-	if (family == AF_INET && cmd == RTM_NEWADDR) {
-		if (!CONNECTED_PEER(ifc) && ifc->destination) {
+	if (family == AF_INET) {
+		if (CONNECTED_PEER(ifc)) {
+			p = ifc->destination;
+			addattr_l(&req.n, sizeof req, IFA_ADDRESS, &p->u.prefix,
+				  bytelen);
+		} else if (cmd == RTM_NEWADDR && ifc->destination) {
 			p = ifc->destination;
 			addattr_l(&req.n, sizeof req, IFA_BROADCAST,
 				  &p->u.prefix, bytelen);
 		}
 	}
 
+	/* p is now either ifc->address or ifc->destination */
+	req.ifa.ifa_prefixlen = p->prefixlen;
+
 	if (CHECK_FLAG(ifc->flags, ZEBRA_IFA_SECONDARY))
 		SET_FLAG(req.ifa.ifa_flags, IFA_F_SECONDARY);
 
@@ -579,6 +864,16 @@ int kernel_address_delete_ipv4(struct interface *ifp, struct connected *ifc)
 	return netlink_address(RTM_DELADDR, AF_INET, ifp, ifc);
 }
 
+int kernel_address_add_ipv6(struct interface *ifp, struct connected *ifc)
+{
+	return netlink_address(RTM_NEWADDR, AF_INET6, ifp, ifc);
+}
+
+int kernel_address_delete_ipv6(struct interface *ifp, struct connected *ifc)
+{
+	return netlink_address(RTM_DELADDR, AF_INET6, ifp, ifc);
+}
+
 int netlink_interface_addr(struct sockaddr_nl *snl, struct nlmsghdr *h,
 			   ns_id_t ns_id, int startup)
 {
@@ -588,7 +883,7 @@ int netlink_interface_addr(struct sockaddr_nl *snl, struct nlmsghdr *h,
 	struct interface *ifp;
 	void *addr;
 	void *broad;
-	u_char flags = 0;
+	uint8_t flags = 0;
 	char *label = NULL;
 	struct zebra_ns *zns;
 
@@ -709,19 +1004,80 @@ int netlink_interface_addr(struct sockaddr_nl *snl, struct nlmsghdr *h,
 			 */
 			if (!(ifa->ifa_flags
 			      & (IFA_F_DADFAILED | IFA_F_TENTATIVE)))
-				connected_add_ipv6(
-					ifp, flags, (struct in6_addr *)addr,
-					ifa->ifa_prefixlen,
-					(struct in6_addr *)broad, label);
+				connected_add_ipv6(ifp, flags,
+						   (struct in6_addr *)addr,
+						   (struct in6_addr *)broad,
+						   ifa->ifa_prefixlen, label);
 		} else
 			connected_delete_ipv6(ifp, (struct in6_addr *)addr,
-					      ifa->ifa_prefixlen,
-					      (struct in6_addr *)broad);
+					      (struct in6_addr *)broad,
+					      ifa->ifa_prefixlen);
 	}
 
 	return 0;
 }
 
+/* helper function called by if_netlink_change
+ * to delete interfaces in case the interface moved
+ * to an other netns
+ */
+static void if_netlink_check_ifp_instance_consistency(uint16_t cmd,
+						     struct interface *ifp,
+						     ns_id_t ns_id)
+{
+	struct interface *other_ifp;
+
+	/*
+	 * look if interface name is also found on other netns
+	 * - only if vrf backend is netns
+	 * - do not concern lo interface
+	 * - then remove previous one
+	 * - for new link case, check found interface is not active
+	 */
+	if (!vrf_is_backend_netns() ||
+	    !strcmp(ifp->name, "lo"))
+		return;
+	other_ifp = if_lookup_by_name_not_ns(ns_id, ifp->name);
+	if (!other_ifp)
+		return;
+	/* because previous interface may be inactive,
+	 * interface is moved back to default vrf
+	 * then one may find the same pointer; ignore
+	 */
+	if (other_ifp == ifp)
+		return;
+	if ((cmd == RTM_NEWLINK)
+	    && (CHECK_FLAG(other_ifp->status, ZEBRA_INTERFACE_ACTIVE)))
+		return;
+	if (IS_ZEBRA_DEBUG_KERNEL && cmd == RTM_NEWLINK) {
+		zlog_debug("RTM_NEWLINK %s(%u, VRF %u) replaces %s(%u, VRF %u)\n",
+			   ifp->name,
+			   ifp->ifindex,
+			   ifp->vrf_id,
+			   other_ifp->name,
+			   other_ifp->ifindex,
+			   other_ifp->vrf_id);
+	} else	if (IS_ZEBRA_DEBUG_KERNEL && cmd == RTM_DELLINK) {
+		zlog_debug("RTM_DELLINK %s(%u, VRF %u) is replaced by %s(%u, VRF %u)\n",
+			   ifp->name,
+			   ifp->ifindex,
+			   ifp->vrf_id,
+			   other_ifp->name,
+			   other_ifp->ifindex,
+			   other_ifp->vrf_id);
+	}
+	/* the found interface replaces the current one
+	 * remove it
+	 */
+	if (cmd == RTM_DELLINK)
+		if_delete(ifp);
+	else
+		if_delete(other_ifp);
+	/* the found interface is replaced by the current one
+	 * suppress it
+	 */
+}
+
 int netlink_link_change(struct sockaddr_nl *snl, struct nlmsghdr *h,
 			ns_id_t ns_id, int startup)
 {
@@ -732,14 +1088,20 @@ int netlink_link_change(struct sockaddr_nl *snl, struct nlmsghdr *h,
 	struct interface *ifp;
 	char *name = NULL;
 	char *kind = NULL;
+	char *desc = NULL;
 	char *slave_kind = NULL;
-	int vrf_device = 0;
 	struct zebra_ns *zns;
 	vrf_id_t vrf_id = VRF_DEFAULT;
+	zebra_iftype_t zif_type = ZEBRA_IF_OTHER;
+	zebra_slave_iftype_t zif_slave_type = ZEBRA_IF_SLAVE_NONE;
+	ifindex_t bridge_ifindex = IFINDEX_INTERNAL;
+	ifindex_t link_ifindex = IFINDEX_INTERNAL;
+
 
 	zns = zebra_ns_lookup(ns_id);
 	ifi = NLMSG_DATA(h);
 
+	/* assume if not default zns, then new VRF */
 	if (!(h->nlmsg_type == RTM_NEWLINK || h->nlmsg_type == RTM_DELLINK)) {
 		/* If this is not link add/delete message so print warning. */
 		zlog_warn("netlink_link_change: wrong kernel message %d",
@@ -751,11 +1113,13 @@ int netlink_link_change(struct sockaddr_nl *snl, struct nlmsghdr *h,
 	if (len < 0)
 		return -1;
 
+	/* We are interested in some AF_BRIDGE notifications. */
 	if (ifi->ifi_family == AF_BRIDGE)
-		return 0;
+		return netlink_bridge_interface(h, len, ns_id, startup);
 
 	/* Looking up interface name. */
 	memset(tb, 0, sizeof tb);
+	memset(linkinfo, 0, sizeof linkinfo);
 	netlink_parse_rtattr(tb, IFLA_MAX, IFLA_RTA(ifi), len);
 
 #ifdef IFLA_WIRELESS
@@ -773,7 +1137,6 @@ int netlink_link_change(struct sockaddr_nl *snl, struct nlmsghdr *h,
 	name = (char *)RTA_DATA(tb[IFLA_IFNAME]);
 
 	if (tb[IFLA_LINKINFO]) {
-		memset(linkinfo, 0, sizeof linkinfo);
 		parse_rtattr_nested(linkinfo, IFLA_INFO_MAX, tb[IFLA_LINKINFO]);
 
 		if (linkinfo[IFLA_INFO_KIND])
@@ -784,56 +1147,99 @@ int netlink_link_change(struct sockaddr_nl *snl, struct nlmsghdr *h,
 			slave_kind = RTA_DATA(linkinfo[IFLA_INFO_SLAVE_KIND]);
 #endif
 
-		if (kind && strcmp(kind, "vrf") == 0) {
-			vrf_device = 1;
-			netlink_vrf_change(h, tb[IFLA_LINKINFO], name);
-			vrf_id = (vrf_id_t)ifi->ifi_index;
-		}
+		netlink_determine_zebra_iftype(kind, &zif_type);
+	}
+
+	/* If linking to another interface, note it. */
+	if (tb[IFLA_LINK])
+		link_ifindex = *(ifindex_t *)RTA_DATA(tb[IFLA_LINK]);
+
+	if (tb[IFLA_IFALIAS]) {
+		desc = (char *)RTA_DATA(tb[IFLA_IFALIAS]);
+	}
+
+	/* If VRF, create or update the VRF structure itself. */
+	if (zif_type == ZEBRA_IF_VRF && !vrf_is_backend_netns()) {
+		netlink_vrf_change(h, tb[IFLA_LINKINFO], name);
+		vrf_id = (vrf_id_t)ifi->ifi_index;
 	}
 
 	/* See if interface is present. */
 	ifp = if_lookup_by_name_per_ns(zns, name);
 
+	if (ifp) {
+		if (ifp->desc)
+			XFREE(MTYPE_TMP, ifp->desc);
+		if (desc)
+			ifp->desc = XSTRDUP(MTYPE_TMP, desc);
+	}
+
 	if (h->nlmsg_type == RTM_NEWLINK) {
 		if (tb[IFLA_MASTER]) {
-			if (slave_kind && (strcmp(slave_kind, "vrf") == 0))
-				vrf_id =
-					*(u_int32_t *)RTA_DATA(tb[IFLA_MASTER]);
+			if (slave_kind && (strcmp(slave_kind, "vrf") == 0)
+			    && !vrf_is_backend_netns()) {
+				zif_slave_type = ZEBRA_IF_SLAVE_VRF;
+				vrf_id = *(uint32_t *)RTA_DATA(tb[IFLA_MASTER]);
+			} else if (slave_kind
+				   && (strcmp(slave_kind, "bridge") == 0)) {
+				zif_slave_type = ZEBRA_IF_SLAVE_BRIDGE;
+				bridge_ifindex =
+					*(ifindex_t *)RTA_DATA(tb[IFLA_MASTER]);
+			} else
+				zif_slave_type = ZEBRA_IF_SLAVE_OTHER;
 		}
-
+		if (vrf_is_backend_netns())
+			vrf_id = (vrf_id_t)ns_id;
 		if (ifp == NULL
 		    || !CHECK_FLAG(ifp->status, ZEBRA_INTERFACE_ACTIVE)) {
 			/* Add interface notification from kernel */
 			if (IS_ZEBRA_DEBUG_KERNEL)
 				zlog_debug(
-					"RTM_NEWLINK for %s(%u) (ifp %p) vrf_id %u flags 0x%x",
-					name, ifi->ifi_index, ifp, vrf_id,
+					"RTM_NEWLINK ADD for %s(%u) vrf_id %u type %d "
+					"sl_type %d master %u flags 0x%x",
+					name, ifi->ifi_index, vrf_id, zif_type,
+					zif_slave_type, bridge_ifindex,
 					ifi->ifi_flags);
 
 			if (ifp == NULL) {
 				/* unknown interface */
-				ifp = if_get_by_name(name, vrf_id);
+				ifp = if_get_by_name(name, vrf_id, 0);
 			} else {
 				/* pre-configured interface, learnt now */
 				if (ifp->vrf_id != vrf_id)
-					if_update(ifp, name, strlen(name),
-						  vrf_id);
+					if_update_to_new_vrf(ifp, vrf_id);
 			}
 
 			/* Update interface information. */
 			set_ifindex(ifp, ifi->ifi_index, zns);
 			ifp->flags = ifi->ifi_flags & 0x0000fffff;
-			if (vrf_device)
-				SET_FLAG(ifp->status,
-					 ZEBRA_INTERFACE_VRF_LOOPBACK);
 			ifp->mtu6 = ifp->mtu = *(int *)RTA_DATA(tb[IFLA_MTU]);
 			ifp->metric = 0;
 			ifp->ptm_status = ZEBRA_PTM_STATUS_UNKNOWN;
 
+			/* Set interface type */
+			zebra_if_set_ziftype(ifp, zif_type, zif_slave_type);
+			if (IS_ZEBRA_IF_VRF(ifp))
+				SET_FLAG(ifp->status,
+					 ZEBRA_INTERFACE_VRF_LOOPBACK);
+
+			/* Update link. */
+			zebra_if_update_link(ifp, link_ifindex);
+
 			netlink_interface_update_hw_addr(tb, ifp);
 
 			/* Inform clients, install any configured addresses. */
 			if_add_update(ifp);
+
+			/* Extract and save L2 interface information, take
+			 * additional actions. */
+			netlink_interface_update_l2info(
+				ifp, linkinfo[IFLA_INFO_DATA], 1);
+			if (IS_ZEBRA_IF_BRIDGE_SLAVE(ifp))
+				zebra_l2if_update_bridge_slave(ifp,
+							       bridge_ifindex);
+			if_netlink_check_ifp_instance_consistency(RTM_NEWLINK,
+								  ifp, ns_id);
 		} else if (ifp->vrf_id != vrf_id) {
 			/* VRF change for an interface. */
 			if (IS_ZEBRA_DEBUG_KERNEL)
@@ -845,31 +1251,64 @@ int netlink_link_change(struct sockaddr_nl *snl, struct nlmsghdr *h,
 
 			if_handle_vrf_change(ifp, vrf_id);
 		} else {
-			/* Interface status change. */
+			int was_bridge_slave;
+
+			/* Interface update. */
 			if (IS_ZEBRA_DEBUG_KERNEL)
 				zlog_debug(
-					"RTM_NEWLINK status for %s(%u) flags 0x%x",
-					name, ifp->ifindex, ifi->ifi_flags);
+					"RTM_NEWLINK update for %s(%u) "
+					"sl_type %d master %u flags 0x%x",
+					name, ifp->ifindex, zif_slave_type,
+					bridge_ifindex, ifi->ifi_flags);
 
 			set_ifindex(ifp, ifi->ifi_index, zns);
 			ifp->mtu6 = ifp->mtu = *(int *)RTA_DATA(tb[IFLA_MTU]);
 			ifp->metric = 0;
 
+			/* Update interface type - NOTE: Only slave_type can
+			 * change. */
+			was_bridge_slave = IS_ZEBRA_IF_BRIDGE_SLAVE(ifp);
+			zebra_if_set_ziftype(ifp, zif_type, zif_slave_type);
+
 			netlink_interface_update_hw_addr(tb, ifp);
 
 			if (if_is_no_ptm_operative(ifp)) {
 				ifp->flags = ifi->ifi_flags & 0x0000fffff;
-				if (!if_is_no_ptm_operative(ifp))
+				if (!if_is_no_ptm_operative(ifp)) {
+					if (IS_ZEBRA_DEBUG_KERNEL)
+						zlog_debug(
+							"Intf %s(%u) has gone DOWN",
+							name, ifp->ifindex);
 					if_down(ifp);
-				else if (if_is_operative(ifp))
+				} else if (if_is_operative(ifp)) {
 					/* Must notify client daemons of new
 					 * interface status. */
+					if (IS_ZEBRA_DEBUG_KERNEL)
+						zlog_debug(
+							"Intf %s(%u) PTM up, notifying clients",
+							name, ifp->ifindex);
 					zebra_interface_up_update(ifp);
+				}
 			} else {
 				ifp->flags = ifi->ifi_flags & 0x0000fffff;
-				if (if_is_operative(ifp))
+				if (if_is_operative(ifp)) {
+					if (IS_ZEBRA_DEBUG_KERNEL)
+						zlog_debug(
+							"Intf %s(%u) has come UP",
+							name, ifp->ifindex);
 					if_up(ifp);
+				}
 			}
+
+			/* Extract and save L2 interface information, take
+			 * additional actions. */
+			netlink_interface_update_l2info(
+				ifp, linkinfo[IFLA_INFO_DATA], 0);
+			if (IS_ZEBRA_IF_BRIDGE_SLAVE(ifp) || was_bridge_slave)
+				zebra_l2if_update_bridge_slave(ifp,
+							       bridge_ifindex);
+			if_netlink_check_ifp_instance_consistency(RTM_NEWLINK,
+								  ifp, ns_id);
 		}
 	} else {
 		/* Delete interface notification from kernel */
@@ -885,8 +1324,16 @@ int netlink_link_change(struct sockaddr_nl *snl, struct nlmsghdr *h,
 
 		UNSET_FLAG(ifp->status, ZEBRA_INTERFACE_VRF_LOOPBACK);
 
-		if (!vrf_device)
+		/* Special handling for bridge or VxLAN interfaces. */
+		if (IS_ZEBRA_IF_BRIDGE(ifp))
+			zebra_l2_bridge_del(ifp);
+		else if (IS_ZEBRA_IF_VXLAN(ifp))
+			zebra_l2_vxlanif_del(ifp);
+
+		if (!IS_ZEBRA_IF_VRF(ifp))
 			if_delete_update(ifp);
+		if_netlink_check_ifp_instance_consistency(RTM_DELLINK,
+							  ifp, ns_id);
 	}
 
 	return 0;
@@ -897,3 +1344,5 @@ void interface_list(struct zebra_ns *zns)
 {
 	interface_lookup_netlink(zns);
 }
+
+#endif /* GNU_LINUX */
