@@ -13,14 +13,15 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
+#ifdef HAVE_NETLINK
+
 #include "linklist.h"
 #include "if.h"
 #include "log.h"
@@ -39,10 +40,12 @@
 #include "zebra/zserv.h"
 #include "zebra/zebra_ns.h"
 #include "zebra/zebra_vrf.h"
+#include "zebra/rt.h"
 #include "zebra/debug.h"
 #include "zebra/kernel_netlink.h"
 #include "zebra/rt_netlink.h"
 #include "zebra/if_netlink.h"
+#include "zebra/rule_netlink.h"
 
 #ifndef SO_RCVBUFFORCE
 #define SO_RCVBUFFORCE  (33)
@@ -55,12 +58,13 @@
 
 #ifndef NLMSG_TAIL
 #define NLMSG_TAIL(nmsg)                                                       \
-	((struct rtattr *)(((u_char *)(nmsg)) + NLMSG_ALIGN((nmsg)->nlmsg_len)))
+	((struct rtattr *)(((uint8_t *)(nmsg))                                 \
+			   + NLMSG_ALIGN((nmsg)->nlmsg_len)))
 #endif
 
 #ifndef RTA_TAIL
 #define RTA_TAIL(rta)                                                          \
-	((struct rtattr *)(((u_char *)(rta)) + RTA_ALIGN((rta)->rta_len)))
+	((struct rtattr *)(((uint8_t *)(rta)) + RTA_ALIGN((rta)->rta_len)))
 #endif
 
 #ifndef RTNL_FAMILY_IP6MR
@@ -83,17 +87,31 @@ static const struct message nlmsg_str[] = {{RTM_NEWROUTE, "RTM_NEWROUTE"},
 					   {RTM_NEWNEIGH, "RTM_NEWNEIGH"},
 					   {RTM_DELNEIGH, "RTM_DELNEIGH"},
 					   {RTM_GETNEIGH, "RTM_GETNEIGH"},
+					   {RTM_NEWRULE, "RTM_NEWRULE"},
+					   {RTM_DELRULE, "RTM_DELRULE"},
+					   {RTM_GETRULE, "RTM_GETRULE"},
 					   {0}};
 
 static const struct message rtproto_str[] = {
-	{RTPROT_REDIRECT, "redirect"}, {RTPROT_KERNEL, "kernel"},
-	{RTPROT_BOOT, "boot"},	 {RTPROT_STATIC, "static"},
-	{RTPROT_GATED, "GateD"},       {RTPROT_RA, "router advertisement"},
-	{RTPROT_MRT, "MRT"},	   {RTPROT_ZEBRA, "Zebra"},
+	{RTPROT_REDIRECT, "redirect"},
+	{RTPROT_KERNEL, "kernel"},
+	{RTPROT_BOOT, "boot"},
+	{RTPROT_STATIC, "static"},
+	{RTPROT_GATED, "GateD"},
+	{RTPROT_RA, "router advertisement"},
+	{RTPROT_MRT, "MRT"},
+	{RTPROT_ZEBRA, "Zebra"},
 #ifdef RTPROT_BIRD
 	{RTPROT_BIRD, "BIRD"},
 #endif /* RTPROT_BIRD */
-	{RTPROT_MROUTED, "mroute"},    {0}};
+	{RTPROT_MROUTED, "mroute"},
+	{RTPROT_BGP, "BGP"},
+	{RTPROT_OSPF, "OSPF"},
+	{RTPROT_ISIS, "IS-IS"},
+	{RTPROT_RIP, "RIP"},
+	{RTPROT_RIPNG, "RIPNG"},
+	{RTPROT_ZSTATIC, "static"},
+	{0}};
 
 static const struct message family_str[] = {{AF_INET, "ipv4"},
 					    {AF_INET6, "ipv6"},
@@ -107,7 +125,7 @@ static const struct message rttype_str[] = {{RTN_UNICAST, "unicast"},
 					    {0}};
 
 extern struct thread_master *master;
-extern u_int32_t nl_rcvbufsize;
+extern uint32_t nl_rcvbufsize;
 
 extern struct zebra_privs_t zserv_privs;
 
@@ -121,7 +139,7 @@ int netlink_talk_filter(struct sockaddr_nl *snl, struct nlmsghdr *h,
 
 static int netlink_recvbuf(struct nlsock *nl, uint32_t newsize)
 {
-	u_int32_t oldsize;
+	uint32_t oldsize;
 	socklen_t newlen = sizeof(newsize);
 	socklen_t oldlen = sizeof(oldsize);
 	int ret;
@@ -176,7 +194,7 @@ static int netlink_socket(struct nlsock *nl, unsigned long groups,
 		return -1;
 	}
 
-	sock = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
+	sock = ns_socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE, ns_id);
 	if (sock < 0) {
 		zlog_err("Can't open %s socket: %s", nl->name,
 			 safe_strerror(errno));
@@ -228,25 +246,28 @@ static int netlink_information_fetch(struct sockaddr_nl *snl,
 	switch (h->nlmsg_type) {
 	case RTM_NEWROUTE:
 		return netlink_route_change(snl, h, ns_id, startup);
-		break;
 	case RTM_DELROUTE:
 		return netlink_route_change(snl, h, ns_id, startup);
-		break;
 	case RTM_NEWLINK:
 		return netlink_link_change(snl, h, ns_id, startup);
-		break;
 	case RTM_DELLINK:
 		return netlink_link_change(snl, h, ns_id, startup);
-		break;
 	case RTM_NEWADDR:
 		return netlink_interface_addr(snl, h, ns_id, startup);
-		break;
 	case RTM_DELADDR:
 		return netlink_interface_addr(snl, h, ns_id, startup);
-		break;
+	case RTM_NEWNEIGH:
+		return netlink_neigh_change(snl, h, ns_id);
+	case RTM_DELNEIGH:
+		return netlink_neigh_change(snl, h, ns_id);
+	case RTM_NEWRULE:
+		return netlink_rule_change(snl, h, ns_id, startup);
+	case RTM_DELRULE:
+		return netlink_rule_change(snl, h, ns_id, startup);
 	default:
-		zlog_warn("Unknown netlink nlmsg_type %d vrf %u\n",
-			  h->nlmsg_type, ns_id);
+		if (IS_ZEBRA_DEBUG_KERNEL)
+			zlog_debug("Unknown netlink nlmsg_type %d vrf %u\n",
+				   h->nlmsg_type, ns_id);
 		break;
 	}
 	return 0;
@@ -256,8 +277,9 @@ static int kernel_read(struct thread *thread)
 {
 	struct zebra_ns *zns = (struct zebra_ns *)THREAD_ARG(thread);
 	netlink_parse_info(netlink_information_fetch, &zns->netlink, zns, 5, 0);
-	zns->t_netlink = thread_add_read(zebrad.master, kernel_read, zns,
-					 zns->netlink.sock);
+	zns->t_netlink = NULL;
+	thread_add_read(zebrad.master, kernel_read, zns, zns->netlink.sock,
+			&zns->t_netlink);
 
 	return 0;
 }
@@ -271,18 +293,22 @@ static void netlink_install_filter(int sock, __u32 pid)
 		/* 0: ldh [4]	          */
 		BPF_STMT(BPF_LD | BPF_ABS | BPF_H,
 			 offsetof(struct nlmsghdr, nlmsg_type)),
-		/* 1: jeq 0x18 jt 3 jf 6  */
-		BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, htons(RTM_NEWROUTE), 1, 0),
-		/* 2: jeq 0x19 jt 3 jf 6  */
-		BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, htons(RTM_DELROUTE), 0, 3),
-		/* 3: ldw [12]		  */
+		/* 1: jeq 0x18 jt 5 jf next  */
+		BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, htons(RTM_NEWROUTE), 3, 0),
+		/* 2: jeq 0x19 jt 5 jf next  */
+		BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, htons(RTM_DELROUTE), 2, 0),
+		/* 3: jeq 0x19 jt 5 jf next  */
+		BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, htons(RTM_NEWNEIGH), 1, 0),
+		/* 4: jeq 0x19 jt 5 jf 8  */
+		BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, htons(RTM_DELNEIGH), 0, 3),
+		/* 5: ldw [12]		  */
 		BPF_STMT(BPF_LD | BPF_ABS | BPF_W,
 			 offsetof(struct nlmsghdr, nlmsg_pid)),
-		/* 4: jeq XX  jt 5 jf 6   */
+		/* 6: jeq XX  jt 7 jf 8   */
 		BPF_JUMP(BPF_JMP | BPF_JEQ | BPF_K, htonl(pid), 0, 1),
-		/* 5: ret 0    (skip)     */
+		/* 7: ret 0    (skip)     */
 		BPF_STMT(BPF_RET | BPF_K, 0),
-		/* 6: ret 0xffff (keep)   */
+		/* 8: ret 0xffff (keep)   */
 		BPF_STMT(BPF_RET | BPF_K, 0xffff),
 	};
 
@@ -356,9 +382,14 @@ int rta_addattr_l(struct rtattr *rta, unsigned int maxlen, int type, void *data,
 	return 0;
 }
 
+int addattr16(struct nlmsghdr *n, unsigned int maxlen, int type, uint16_t data)
+{
+	return addattr_l(n, maxlen, type, &data, sizeof(uint16_t));
+}
+
 int addattr32(struct nlmsghdr *n, unsigned int maxlen, int type, int data)
 {
-	return addattr_l(n, maxlen, type, &data, sizeof(u_int32_t));
+	return addattr_l(n, maxlen, type, &data, sizeof(uint32_t));
 }
 
 struct rtattr *addattr_nest(struct nlmsghdr *n, int maxlen, int type)
@@ -371,7 +402,7 @@ struct rtattr *addattr_nest(struct nlmsghdr *n, int maxlen, int type)
 
 int addattr_nest_end(struct nlmsghdr *n, struct rtattr *nest)
 {
-	nest->rta_len = (u_char *)NLMSG_TAIL(n) - (u_char *)nest;
+	nest->rta_len = (uint8_t *)NLMSG_TAIL(n) - (uint8_t *)nest;
 	return n->nlmsg_len;
 }
 
@@ -385,7 +416,7 @@ struct rtattr *rta_nest(struct rtattr *rta, int maxlen, int type)
 
 int rta_nest_end(struct rtattr *rta, struct rtattr *nest)
 {
-	nest->rta_len = (u_char *)RTA_TAIL(rta) - (u_char *)nest;
+	nest->rta_len = (uint8_t *)RTA_TAIL(rta) - (uint8_t *)nest;
 	return rta->rta_len;
 }
 
@@ -394,17 +425,17 @@ const char *nl_msg_type_to_str(uint16_t msg_type)
 	return lookup_msg(nlmsg_str, msg_type, "");
 }
 
-const char *nl_rtproto_to_str(u_char rtproto)
+const char *nl_rtproto_to_str(uint8_t rtproto)
 {
 	return lookup_msg(rtproto_str, rtproto, "");
 }
 
-const char *nl_family_to_str(u_char family)
+const char *nl_family_to_str(uint8_t family)
 {
 	return lookup_msg(family_str, family, "");
 }
 
-const char *nl_rttype_to_str(u_char rttype)
+const char *nl_rttype_to_str(uint8_t rttype)
 {
 	return lookup_msg(rttype_str, rttype, "");
 }
@@ -476,7 +507,7 @@ int netlink_parse_info(int (*filter)(struct sockaddr_nl *, struct nlmsghdr *,
 		if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_RECV) {
 			zlog_debug("%s: << netlink message dump [recv]",
 				   __func__);
-			zlog_hexdump(&msg, sizeof(msg));
+			zlog_hexdump(buf, status);
 		}
 
 		read_in++;
@@ -648,19 +679,25 @@ int netlink_talk(int (*filter)(struct sockaddr_nl *, struct nlmsghdr *, ns_id_t,
 {
 	int status;
 	struct sockaddr_nl snl;
-	struct iovec iov = {.iov_base = (void *)n, .iov_len = n->nlmsg_len};
-	struct msghdr msg = {
-		.msg_name = (void *)&snl,
-		.msg_namelen = sizeof snl,
-		.msg_iov = &iov,
-		.msg_iovlen = 1,
-	};
+	struct iovec iov;
+	struct msghdr msg;
 	int save_errno;
 
 	memset(&snl, 0, sizeof snl);
+	memset(&iov, 0, sizeof iov);
+	memset(&msg, 0, sizeof msg);
+
+	iov.iov_base = n;
+	iov.iov_len = n->nlmsg_len;
+	msg.msg_name = (void *)&snl;
+	msg.msg_namelen = sizeof snl;
+	msg.msg_iov = &iov;
+	msg.msg_iovlen = 1;
+
 	snl.nl_family = AF_NETLINK;
 
 	n->nlmsg_seq = ++nl->seq;
+	n->nlmsg_pid = nl->snl.nl_pid;
 
 	/* Request an acknowledgement by setting NLM_F_ACK */
 	n->nlmsg_flags |= NLM_F_ACK;
@@ -682,7 +719,7 @@ int netlink_talk(int (*filter)(struct sockaddr_nl *, struct nlmsghdr *, ns_id_t,
 
 	if (IS_ZEBRA_DEBUG_KERNEL_MSGDUMP_SEND) {
 		zlog_debug("%s: >> netlink message dump [sent]", __func__);
-		zlog_hexdump(&msg, sizeof(msg));
+		zlog_hexdump(n, n->nlmsg_len);
 	}
 
 	if (status < 0) {
@@ -699,44 +736,36 @@ int netlink_talk(int (*filter)(struct sockaddr_nl *, struct nlmsghdr *, ns_id_t,
 	return netlink_parse_info(filter, nl, zns, 0, startup);
 }
 
-/* Get type specified information from netlink. */
-int netlink_request(int family, int type, struct nlsock *nl)
+/* Issue request message to kernel via netlink socket. GET messages
+ * are issued through this interface.
+ */
+int netlink_request(struct nlsock *nl, struct nlmsghdr *n)
 {
 	int ret;
 	struct sockaddr_nl snl;
 	int save_errno;
 
-	struct {
-		struct nlmsghdr nlh;
-		struct rtgenmsg g;
-	} req;
-
 	/* Check netlink socket. */
 	if (nl->sock < 0) {
 		zlog_err("%s socket isn't active.", nl->name);
 		return -1;
 	}
 
+	/* Fill common fields for all requests. */
+	n->nlmsg_flags = NLM_F_ROOT | NLM_F_MATCH | NLM_F_REQUEST;
+	n->nlmsg_pid = nl->snl.nl_pid;
+	n->nlmsg_seq = ++nl->seq;
+
 	memset(&snl, 0, sizeof snl);
 	snl.nl_family = AF_NETLINK;
 
-	memset(&req, 0, sizeof req);
-	req.nlh.nlmsg_len = sizeof req;
-	req.nlh.nlmsg_type = type;
-	req.nlh.nlmsg_flags = NLM_F_ROOT | NLM_F_MATCH | NLM_F_REQUEST;
-	req.nlh.nlmsg_pid = nl->snl.nl_pid;
-	req.nlh.nlmsg_seq = ++nl->seq;
-	req.g.rtgen_family = family;
-
-	/* linux appears to check capabilities on every message
-	 * have to raise caps for every message sent
-	 */
+	/* Raise capabilities and send message, then lower capabilities. */
 	if (zserv_privs.change(ZPRIVS_RAISE)) {
 		zlog_err("Can't raise privileges");
 		return -1;
 	}
 
-	ret = sendto(nl->sock, (void *)&req, sizeof req, 0,
+	ret = sendto(nl->sock, (void *)n, n->nlmsg_len, 0,
 		     (struct sockaddr *)&snl, sizeof snl);
 	save_errno = errno;
 
@@ -760,7 +789,9 @@ void kernel_init(struct zebra_ns *zns)
 
 	/* Initialize netlink sockets */
 	groups = RTMGRP_LINK | RTMGRP_IPV4_ROUTE | RTMGRP_IPV4_IFADDR
-		 | RTMGRP_IPV6_ROUTE | RTMGRP_IPV6_IFADDR | RTMGRP_IPV4_MROUTE;
+		 | RTMGRP_IPV6_ROUTE | RTMGRP_IPV6_IFADDR | RTMGRP_IPV4_MROUTE
+		 | RTMGRP_NEIGH
+		 | RTNLGRP_IPV4_RULE | RTNLGRP_IPV6_RULE;
 
 	snprintf(zns->netlink.name, sizeof(zns->netlink.name),
 		 "netlink-listen (NS %u)", zns->ns_id);
@@ -785,9 +816,12 @@ void kernel_init(struct zebra_ns *zns)
 
 		netlink_install_filter(zns->netlink.sock,
 				       zns->netlink_cmd.snl.nl_pid);
-		zns->t_netlink = thread_add_read(zebrad.master, kernel_read,
-						 zns, zns->netlink.sock);
+		zns->t_netlink = NULL;
+		thread_add_read(zebrad.master, kernel_read, zns,
+				zns->netlink.sock, &zns->t_netlink);
 	}
+
+	rt_netlink_init();
 }
 
 void kernel_terminate(struct zebra_ns *zns)
@@ -804,3 +838,5 @@ void kernel_terminate(struct zebra_ns *zns)
 		zns->netlink_cmd.sock = -1;
 	}
 }
+
+#endif /* HAVE_NETLINK */
