@@ -13,13 +13,15 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
+
+#ifndef HAVE_NETLINK
+
 #include <net/if_types.h>
 #ifdef __OpenBSD__
 #include <netmpls/mpls.h>
@@ -38,6 +40,7 @@
 #include "privs.h"
 #include "vrf.h"
 
+#include "zebra/rt.h"
 #include "zebra/interface.h"
 #include "zebra/zserv.h"
 #include "zebra/debug.h"
@@ -86,13 +89,10 @@ extern struct zebra_privs_t zserv_privs;
 #if !defined(ROUNDUP)
 
 /*
- * It's a bug for a platform not to define rounding/alignment for
- * sockaddrs on the routing socket.  This warning really is
- * intentional, to provoke filing bug reports with operating systems
- * that don't define RT_ROUNDUP or equivalent.
+ * If you're porting to a platform that changed RT_ROUNDUP but doesn't
+ * have it in its headers, this will break rather obviously and you'll
+ * have to fix it here.
  */
-#warning                                                                       \
-	"net/route.h does not define RT_ROUNDUP; making unwarranted assumptions!"
 
 /* OS X (Xcode as of 2014-12) is known not to define RT_ROUNDUP */
 #ifdef __APPLE__
@@ -168,7 +168,7 @@ static inline void rta_copy(union sockunion *dest, caddr_t src)
 
 #define RTA_NAME_GET(DEST, RTA, RTMADDRS, PNT, LEN)                            \
 	if ((RTMADDRS) & (RTA)) {                                              \
-		u_char *pdest = (u_char *)(DEST);                              \
+		uint8_t *pdest = (uint8_t *)(DEST);                            \
 		int len = SAROUNDUP((PNT));                                    \
 		struct sockaddr_dl *sdl = (struct sockaddr_dl *)(PNT);         \
 		if (IS_ZEBRA_DEBUG_KERNEL)                                     \
@@ -324,11 +324,8 @@ static int ifan_read(struct if_announcemsghdr *ifan)
 				__func__, ifan->ifan_index, ifan->ifan_name);
 
 		/* Create Interface */
-		ifp = if_get_by_name_len(
-			ifan->ifan_name,
-			strnlen(ifan->ifan_name, sizeof(ifan->ifan_name)),
-			VRF_DEFAULT, 0);
-		ifp->ifindex = ifan->ifan_index;
+		ifp = if_get_by_name(ifan->ifan_name, VRF_DEFAULT, 0);
+		if_set_index(ifp, ifan->ifan_index);
 
 		if_get_metric(ifp);
 		if_add_update(ifp);
@@ -442,6 +439,12 @@ int ifm_read(struct if_msghdr *ifm)
 	RTA_ADDR_GET(NULL, RTA_IFA, ifm->ifm_addrs, cp);
 	RTA_ADDR_GET(NULL, RTA_AUTHOR, ifm->ifm_addrs, cp);
 	RTA_ADDR_GET(NULL, RTA_BRD, ifm->ifm_addrs, cp);
+#ifdef RTA_LABEL
+	RTA_ATTR_GET(NULL, RTA_LABEL, ifm->ifm_addrs, cp);
+#endif
+#ifdef RTA_SRC
+	RTA_ADDR_GET(NULL, RTA_SRC, ifm->ifm_addrs, cp);
+#endif
 
 	if (IS_ZEBRA_DEBUG_KERNEL)
 		zlog_debug("%s: sdl ifname %s", __func__,
@@ -512,7 +515,7 @@ int ifm_read(struct if_msghdr *ifm)
 		if (ifp == NULL) {
 			/* Interface that zebra was not previously aware of, so
 			 * create. */
-			ifp = if_create(ifname, ifnlen, VRF_DEFAULT);
+			ifp = if_create(ifname, VRF_DEFAULT);
 			if (IS_ZEBRA_DEBUG_KERNEL)
 				zlog_debug("%s: creating ifp for ifindex %d",
 					   __func__, ifm->ifm_index);
@@ -526,7 +529,7 @@ int ifm_read(struct if_msghdr *ifm)
 		 * Fill in newly created interface structure, or larval
 		 * structure with ifindex IFINDEX_INTERNAL.
 		 */
-		ifp->ifindex = ifm->ifm_index;
+		if_set_index(ifp, ifm->ifm_index);
 
 #ifdef HAVE_BSD_IFI_LINK_STATE /* translate BSD kernel msg for link-state */
 		bsd_linkdetect_translate(ifm);
@@ -662,6 +665,12 @@ static void ifam_read_mesg(struct ifa_msghdr *ifm, union sockunion *addr,
 	RTA_ADDR_GET(addr, RTA_IFA, ifm->ifam_addrs, pnt);
 	RTA_ADDR_GET(NULL, RTA_AUTHOR, ifm->ifam_addrs, pnt);
 	RTA_ADDR_GET(brd, RTA_BRD, ifm->ifam_addrs, pnt);
+#ifdef RTA_LABEL
+	RTA_ATTR_GET(NULL, RTA_LABEL, ifm->ifam_addrs, pnt);
+#endif
+#ifdef RTA_SRC
+	RTA_ADDR_GET(NULL, RTA_SRC, ifm->ifam_addrs, pnt);
+#endif
 
 	if (IS_ZEBRA_DEBUG_KERNEL) {
 		int family = sockunion_family(addr);
@@ -762,13 +771,12 @@ int ifam_read(struct ifa_msghdr *ifam)
 
 		if (ifam->ifam_type == RTM_NEWADDR)
 			connected_add_ipv6(ifp, flags, &addr.sin6.sin6_addr,
+					   NULL,
 					   ip6_masklen(mask.sin6.sin6_addr),
-					   &brd.sin6.sin6_addr,
 					   (isalias ? ifname : NULL));
 		else
-			connected_delete_ipv6(ifp, &addr.sin6.sin6_addr,
-					      ip6_masklen(mask.sin6.sin6_addr),
-					      &brd.sin6.sin6_addr);
+			connected_delete_ipv6(ifp, &addr.sin6.sin6_addr, NULL,
+					      ip6_masklen(mask.sin6.sin6_addr));
 		break;
 	default:
 		/* Unsupported family silently ignore... */
@@ -830,6 +838,17 @@ static int rtm_read_mesg(struct rt_msghdr *rtm, union sockunion *dest,
 	RTA_ADDR_GET(NULL, RTA_IFA, rtm->rtm_addrs, pnt);
 	RTA_ADDR_GET(NULL, RTA_AUTHOR, rtm->rtm_addrs, pnt);
 	RTA_ADDR_GET(NULL, RTA_BRD, rtm->rtm_addrs, pnt);
+#ifdef RTA_LABEL
+#if 0
+	union sockunion label;
+	memset(&label, 0, sizeof(label));
+	RTA_ATTR_GET(&label, RTA_LABEL, rtm->rtm_addrs, pnt);
+#endif
+	RTA_ATTR_GET(NULL, RTA_LABEL, rtm->rtm_addrs, pnt);
+#endif
+#ifdef RTA_SRC
+	RTA_ADDR_GET(NULL, RTA_SRC, rtm->rtm_addrs, pnt);
+#endif
 
 	/* If there is netmask information set it's family same as
 	   destination family*/
@@ -846,10 +865,11 @@ static int rtm_read_mesg(struct rt_msghdr *rtm, union sockunion *dest,
 void rtm_read(struct rt_msghdr *rtm)
 {
 	int flags;
-	u_char zebra_flags;
+	uint8_t zebra_flags;
 	union sockunion dest, mask, gate;
 	char ifname[INTERFACE_NAMSIZ + 1];
 	short ifnlen = 0;
+	struct nexthop nh;
 
 	zebra_flags = 0;
 
@@ -887,11 +907,17 @@ void rtm_read(struct rt_msghdr *rtm)
 	if (flags & RTF_STATIC)
 		SET_FLAG(zebra_flags, ZEBRA_FLAG_STATIC);
 
+	memset(&nh, 0, sizeof(nh));
+
+	nh.vrf_id = VRF_DEFAULT;
 	/* This is a reject or blackhole route */
-	if (flags & RTF_REJECT)
-		SET_FLAG(zebra_flags, ZEBRA_FLAG_REJECT);
-	if (flags & RTF_BLACKHOLE)
-		SET_FLAG(zebra_flags, ZEBRA_FLAG_BLACKHOLE);
+	if (flags & RTF_REJECT) {
+		nh.type = NEXTHOP_TYPE_BLACKHOLE;
+		nh.bh_type = BLACKHOLE_REJECT;
+	} else if (flags & RTF_BLACKHOLE) {
+		nh.type = NEXTHOP_TYPE_BLACKHOLE;
+		nh.bh_type = BLACKHOLE_NULL;
+	}
 
 	if (dest.sa.sa_family == AF_INET) {
 		struct prefix p;
@@ -1017,18 +1043,22 @@ void rtm_read(struct rt_msghdr *rtm)
 		if (rtm->rtm_type == RTM_CHANGE)
 			rib_delete(AFI_IP, SAFI_UNICAST, VRF_DEFAULT,
 				   ZEBRA_ROUTE_KERNEL, 0, zebra_flags, &p, NULL,
-				   NULL, 0, 0);
+				   NULL, 0, 0, true, NULL);
+
+		if (!nh.type) {
+			nh.type = NEXTHOP_TYPE_IPV4;
+			nh.gate.ipv4 = gate.sin.sin_addr;
+		}
 
-		union g_addr ggate = {.ipv4 = gate.sin.sin_addr};
 		if (rtm->rtm_type == RTM_GET || rtm->rtm_type == RTM_ADD
 		    || rtm->rtm_type == RTM_CHANGE)
 			rib_add(AFI_IP, SAFI_UNICAST, VRF_DEFAULT,
 				ZEBRA_ROUTE_KERNEL, 0, zebra_flags, &p, NULL,
-				&ggate, NULL, 0, 0, 0, 0, 0);
+				&nh, 0, 0, 0, 0, 0);
 		else
 			rib_delete(AFI_IP, SAFI_UNICAST, VRF_DEFAULT,
 				   ZEBRA_ROUTE_KERNEL, 0, zebra_flags, &p, NULL,
-				   &ggate, 0, 0);
+				   &nh, 0, 0, true, NULL);
 	}
 	if (dest.sa.sa_family == AF_INET6) {
 		/* One day we might have a debug section here like one in the
@@ -1059,18 +1089,24 @@ void rtm_read(struct rt_msghdr *rtm)
 		if (rtm->rtm_type == RTM_CHANGE)
 			rib_delete(AFI_IP6, SAFI_UNICAST, VRF_DEFAULT,
 				   ZEBRA_ROUTE_KERNEL, 0, zebra_flags, &p, NULL,
-				   NULL, 0, 0);
+				   NULL, 0, 0, true, NULL);
+
+		if (!nh.type) {
+			nh.type = ifindex ? NEXTHOP_TYPE_IPV6_IFINDEX
+					  : NEXTHOP_TYPE_IPV6;
+			nh.gate.ipv6 = gate.sin6.sin6_addr;
+			nh.ifindex = ifindex;
+		}
 
-		union g_addr ggate = {.ipv6 = gate.sin6.sin6_addr};
 		if (rtm->rtm_type == RTM_GET || rtm->rtm_type == RTM_ADD
 		    || rtm->rtm_type == RTM_CHANGE)
 			rib_add(AFI_IP6, SAFI_UNICAST, VRF_DEFAULT,
 				ZEBRA_ROUTE_KERNEL, 0, zebra_flags, &p, NULL,
-				&ggate, NULL, ifindex, 0, 0, 0, 0);
+				&nh, 0, 0, 0, 0, 0);
 		else
 			rib_delete(AFI_IP6, SAFI_UNICAST, VRF_DEFAULT,
 				   ZEBRA_ROUTE_KERNEL, 0, zebra_flags, &p, NULL,
-				   &ggate, ifindex, 0);
+				   &nh, 0, 0, true, NULL);
 	}
 }
 
@@ -1080,7 +1116,7 @@ void rtm_read(struct rt_msghdr *rtm)
  */
 int rtm_write(int message, union sockunion *dest, union sockunion *mask,
 	      union sockunion *gate, union sockunion *mpls, unsigned int index,
-	      int zebra_flags, int metric)
+	      enum blackhole_type bh_type, int metric)
 {
 	int ret;
 	caddr_t pnt;
@@ -1162,7 +1198,7 @@ int rtm_write(int message, union sockunion *dest, union sockunion *mask,
 		msg.rtm.rtm_flags |= RTF_MPLS;
 
 		if (mpls->smpls.smpls_label
-		    != htonl(MPLS_IMP_NULL_LABEL << MPLS_LABEL_OFFSET))
+		    != htonl(MPLS_LABEL_IMPLICIT_NULL << MPLS_LABEL_OFFSET))
 			msg.rtm.rtm_mpls = MPLS_OP_PUSH;
 	}
 #endif
@@ -1170,11 +1206,16 @@ int rtm_write(int message, union sockunion *dest, union sockunion *mask,
 	/* Tagging route with flags */
 	msg.rtm.rtm_flags |= (RTF_PROTO1);
 
-	/* Additional flags. */
-	if (zebra_flags & ZEBRA_FLAG_BLACKHOLE)
-		msg.rtm.rtm_flags |= RTF_BLACKHOLE;
-	if (zebra_flags & ZEBRA_FLAG_REJECT)
+	switch (bh_type) {
+	case BLACKHOLE_UNSPEC:
+		break;
+	case BLACKHOLE_REJECT:
 		msg.rtm.rtm_flags |= RTF_REJECT;
+		break;
+	default:
+		msg.rtm.rtm_flags |= RTF_BLACKHOLE;
+		break;
+	}
 
 
 #define SOCKADDRSET(X, R)                                                      \
@@ -1294,7 +1335,7 @@ static int kernel_read(struct thread *thread)
 		return 0;
 	}
 
-	thread_add_read(zebrad.master, kernel_read, NULL, sock);
+	thread_add_read(zebrad.master, kernel_read, NULL, sock, NULL);
 
 	if (IS_ZEBRA_DEBUG_KERNEL)
 		rtmsg_debug(&buf.r.rtm);
@@ -1344,7 +1385,8 @@ static void routing_socket(struct zebra_ns *zns)
 	if (zserv_privs.change(ZPRIVS_RAISE))
 		zlog_err("routing_socket: Can't raise privileges");
 
-	routing_sock = socket(AF_ROUTE, SOCK_RAW, 0);
+	routing_sock =
+		ns_socket(AF_ROUTE, SOCK_RAW, 0, zns->ns_id);
 
 	if (routing_sock < 0) {
 		if (zserv_privs.change(ZPRIVS_LOWER))
@@ -1364,7 +1406,7 @@ static void routing_socket(struct zebra_ns *zns)
 		zlog_err("routing_socket: Can't lower privileges");
 
 	/* kernel_read needs rewrite. */
-	thread_add_read(zebrad.master, kernel_read, NULL, routing_sock);
+	thread_add_read(zebrad.master, kernel_read, NULL, routing_sock, NULL);
 }
 
 /* Exported interface function.  This function simply calls
@@ -1378,3 +1420,5 @@ void kernel_terminate(struct zebra_ns *zns)
 {
 	return;
 }
+
+#endif /* !HAVE_NETLINK */
