@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -41,43 +40,38 @@
 #include "zebra/redistribute.h"
 #include "zebra/debug.h"
 #include "zebra/router-id.h"
+#include "zebra/zapi_msg.h"
 #include "zebra/zebra_memory.h"
+#include "zebra/zebra_vxlan.h"
 
 #define ZEBRA_PTM_SUPPORT
 
 /* array holding redistribute info about table redistribution */
 /* bit AFI is set if that AFI is redistributing routes from this table */
 static int zebra_import_table_used[AFI_MAX][ZEBRA_KERNEL_TABLE_MAX];
-static u_int32_t zebra_import_table_distance[AFI_MAX][ZEBRA_KERNEL_TABLE_MAX];
+static uint32_t zebra_import_table_distance[AFI_MAX][ZEBRA_KERNEL_TABLE_MAX];
 
-int is_zebra_import_table_enabled(afi_t afi, u_int32_t table_id)
+int is_zebra_import_table_enabled(afi_t afi, uint32_t table_id)
 {
-	if (is_zebra_valid_kernel_table(table_id))
+	/*
+	 * Make sure that what we are called with actualy makes sense
+	 */
+	if (afi == AFI_MAX)
+		return 0;
+
+	if (is_zebra_valid_kernel_table(table_id) &&
+	    table_id < ZEBRA_KERNEL_TABLE_MAX)
 		return zebra_import_table_used[afi][table_id];
 	return 0;
 }
 
-int is_default(struct prefix *p)
-{
-	if (p->family == AF_INET)
-		if (p->u.prefix4.s_addr == 0 && p->prefixlen == 0)
-			return 1;
-#if 0  /* IPv6 default separation is now pending until protocol daemon         \
-	  can handle that. */
-  if (p->family == AF_INET6)
-    if (IN6_IS_ADDR_UNSPECIFIED (&p->u.prefix6) && p->prefixlen == 0)
-      return 1;
-#endif /* 0 */
-	return 0;
-}
-
 static void zebra_redistribute_default(struct zserv *client, vrf_id_t vrf_id)
 {
 	int afi;
 	struct prefix p;
 	struct route_table *table;
 	struct route_node *rn;
-	struct rib *newrib;
+	struct route_entry *newre;
 
 	for (afi = AFI_IP; afi <= AFI_IP6; afi++) {
 		/* Lookup table.  */
@@ -92,21 +86,24 @@ static void zebra_redistribute_default(struct zserv *client, vrf_id_t vrf_id)
 		if (!rn)
 			continue;
 
-		RNODE_FOREACH_RIB(rn, newrib)
-		if (CHECK_FLAG(newrib->flags, ZEBRA_FLAG_SELECTED)
-		    && newrib->distance != DISTANCE_INFINITY)
-			zsend_redistribute_route(1, client, &rn->p, NULL,
-						 newrib);
+		RNODE_FOREACH_RE (rn, newre) {
+			if (CHECK_FLAG(newre->flags, ZEBRA_FLAG_SELECTED)
+			    && newre->distance != DISTANCE_INFINITY)
+				zsend_redistribute_route(
+					ZEBRA_REDISTRIBUTE_ROUTE_ADD, client,
+					&rn->p, NULL, newre);
+		}
 
 		route_unlock_node(rn);
 	}
 }
 
 /* Redistribute routes. */
-static void zebra_redistribute(struct zserv *client, int type, u_short instance,
-			       vrf_id_t vrf_id, int afi)
+static void zebra_redistribute(struct zserv *client, int type,
+			       unsigned short instance, vrf_id_t vrf_id,
+			       int afi)
 {
-	struct rib *newrib;
+	struct route_entry *newre;
 	struct route_table *table;
 	struct route_node *rn;
 
@@ -114,55 +111,57 @@ static void zebra_redistribute(struct zserv *client, int type, u_short instance,
 	if (!table)
 		return;
 
-	for (rn = route_top(table); rn; rn = route_next(rn))
-		RNODE_FOREACH_RIB(rn, newrib)
-		{
+	for (rn = route_top(table); rn; rn = srcdest_route_next(rn))
+		RNODE_FOREACH_RE (rn, newre) {
 			struct prefix *dst_p, *src_p;
+			char buf[PREFIX_STRLEN];
+
 			srcdest_rnode_prefixes(rn, &dst_p, &src_p);
 
 			if (IS_ZEBRA_DEBUG_EVENT)
 				zlog_debug(
-					"%s: checking: selected=%d, type=%d, distance=%d, "
-					"zebra_check_addr=%d",
+					"%s: client %s %s(%u) checking: selected=%d, type=%d, distance=%d, metric=%d zebra_check_addr=%d",
 					__func__,
-					CHECK_FLAG(newrib->flags,
-						   ZEBRA_FLAG_SELECTED),
-					newrib->type, newrib->distance,
-					zebra_check_addr(dst_p));
+					zebra_route_string(client->proto),
+					prefix2str(dst_p, buf, sizeof(buf)),
+					vrf_id, CHECK_FLAG(newre->flags,
+							   ZEBRA_FLAG_SELECTED),
+					newre->type, newre->distance,
+					newre->metric, zebra_check_addr(dst_p));
 
-			if (!CHECK_FLAG(newrib->flags, ZEBRA_FLAG_SELECTED))
+			if (!CHECK_FLAG(newre->flags, ZEBRA_FLAG_SELECTED))
 				continue;
 			if ((type != ZEBRA_ROUTE_ALL
-			     && (newrib->type != type
-				 || newrib->instance != instance)))
+			     && (newre->type != type
+				 || newre->instance != instance)))
 				continue;
-			if (newrib->distance == DISTANCE_INFINITY)
+			if (newre->distance == DISTANCE_INFINITY)
 				continue;
 			if (!zebra_check_addr(dst_p))
 				continue;
 
-			zsend_redistribute_route(1, client, dst_p, src_p,
-						 newrib);
+			zsend_redistribute_route(ZEBRA_REDISTRIBUTE_ROUTE_ADD,
+						 client, dst_p, src_p, newre);
 		}
 }
 
 /* Either advertise a route for redistribution to registered clients or */
 /* withdraw redistribution if add cannot be done for client */
 void redistribute_update(struct prefix *p, struct prefix *src_p,
-			 struct rib *rib, struct rib *prev_rib)
+			 struct route_entry *re, struct route_entry *prev_re)
 {
 	struct listnode *node, *nnode;
 	struct zserv *client;
 	int send_redistribute;
 	int afi;
-	char buf[INET6_ADDRSTRLEN];
+	char buf[PREFIX_STRLEN];
 
 	if (IS_ZEBRA_DEBUG_RIB) {
-		inet_ntop(p->family, &p->u.prefix, buf, INET6_ADDRSTRLEN);
 		zlog_debug(
-			"%u:%s/%d: Redist update rib %p (type %d), old %p (type %d)",
-			rib->vrf_id, buf, p->prefixlen, rib, rib->type,
-			prev_rib, prev_rib ? prev_rib->type : -1);
+			"%u:%s: Redist update re %p (type %d), old %p (type %d)",
+			re->vrf_id, prefix2str(p, buf, sizeof(buf)),
+			re, re->type, prev_re,
+			prev_re ? prev_re->type : -1);
 	}
 
 	afi = family2afi(p->family);
@@ -175,39 +174,50 @@ void redistribute_update(struct prefix *p, struct prefix *src_p,
 	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client)) {
 		send_redistribute = 0;
 
-		if (is_default(p)
-		    && vrf_bitmap_check(client->redist_default, rib->vrf_id))
+		if (is_default_prefix(p)
+		    && vrf_bitmap_check(client->redist_default, re->vrf_id))
 			send_redistribute = 1;
 		else if (vrf_bitmap_check(client->redist[afi][ZEBRA_ROUTE_ALL],
-					  rib->vrf_id))
+					  re->vrf_id))
 			send_redistribute = 1;
-		else if (rib->instance
+		else if (re->instance
 			 && redist_check_instance(
-				    &client->mi_redist[afi][rib->type],
-				    rib->instance))
+				    &client->mi_redist[afi][re->type],
+				    re->instance))
 			send_redistribute = 1;
-		else if (vrf_bitmap_check(client->redist[afi][rib->type],
-					  rib->vrf_id))
+		else if (vrf_bitmap_check(client->redist[afi][re->type],
+					  re->vrf_id))
 			send_redistribute = 1;
 
 		if (send_redistribute) {
-			zsend_redistribute_route(1, client, p, src_p, rib);
-		} else if (prev_rib
-			   && ((rib->instance
+			if (IS_ZEBRA_DEBUG_EVENT) {
+				zlog_debug(
+					   "%s: client %s %s(%u), type=%d, distance=%d, metric=%d",
+					   __func__,
+					   zebra_route_string(client->proto),
+					   prefix2str(p, buf, sizeof(buf)),
+					   re->vrf_id, re->type,
+					   re->distance, re->metric);
+			}
+			zsend_redistribute_route(ZEBRA_REDISTRIBUTE_ROUTE_ADD,
+						 client, p, src_p, re);
+		} else if (prev_re
+			   && ((re->instance
 				&& redist_check_instance(
 					   &client->mi_redist[afi]
-							     [prev_rib->type],
-					   rib->instance))
+							     [prev_re->type],
+					   re->instance))
 			       || vrf_bitmap_check(
-					  client->redist[afi][prev_rib->type],
-					  rib->vrf_id))) {
-			zsend_redistribute_route(0, client, p, src_p, prev_rib);
+					  client->redist[afi][prev_re->type],
+					  re->vrf_id))) {
+			zsend_redistribute_route(ZEBRA_REDISTRIBUTE_ROUTE_DEL,
+						 client, p, src_p, prev_re);
 		}
 	}
 }
 
 void redistribute_delete(struct prefix *p, struct prefix *src_p,
-			 struct rib *rib)
+			 struct route_entry *re)
 {
 	struct listnode *node, *nnode;
 	struct zserv *client;
@@ -216,12 +226,12 @@ void redistribute_delete(struct prefix *p, struct prefix *src_p,
 
 	if (IS_ZEBRA_DEBUG_RIB) {
 		inet_ntop(p->family, &p->u.prefix, buf, INET6_ADDRSTRLEN);
-		zlog_debug("%u:%s/%d: Redist delete rib %p (type %d)",
-			   rib->vrf_id, buf, p->prefixlen, rib, rib->type);
+		zlog_debug("%u:%s/%d: Redist delete re %p (type %d)",
+			   re->vrf_id, buf, p->prefixlen, re, re->type);
 	}
 
 	/* Add DISTANCE_INFINITY check. */
-	if (rib->distance == DISTANCE_INFINITY)
+	if (re->distance == DISTANCE_INFINITY)
 		return;
 
 	afi = family2afi(p->family);
@@ -232,34 +242,49 @@ void redistribute_delete(struct prefix *p, struct prefix *src_p,
 	}
 
 	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client)) {
-		if ((is_default(p)
-		     && vrf_bitmap_check(client->redist_default, rib->vrf_id))
+		if ((is_default_prefix(p)
+		     && vrf_bitmap_check(client->redist_default, re->vrf_id))
 		    || vrf_bitmap_check(client->redist[afi][ZEBRA_ROUTE_ALL],
-					rib->vrf_id)
-		    || (rib->instance
+					re->vrf_id)
+		    || (re->instance
 			&& redist_check_instance(
-				   &client->mi_redist[afi][rib->type],
-				   rib->instance))
-		    || vrf_bitmap_check(client->redist[afi][rib->type],
-					rib->vrf_id)) {
-			zsend_redistribute_route(0, client, p, src_p, rib);
+				   &client->mi_redist[afi][re->type],
+				   re->instance))
+		    || vrf_bitmap_check(client->redist[afi][re->type],
+					re->vrf_id)) {
+			zsend_redistribute_route(ZEBRA_REDISTRIBUTE_ROUTE_DEL,
+						 client, p, src_p, re);
 		}
 	}
 }
 
-void zebra_redistribute_add(int command, struct zserv *client, int length,
-			    struct zebra_vrf *zvrf)
+void zebra_redistribute_add(ZAPI_HANDLER_ARGS)
 {
-	afi_t afi;
-	int type;
-	u_short instance;
+	afi_t afi = 0;
+	int type = 0;
+	unsigned short instance;
 
-	afi = stream_getc(client->ibuf);
-	type = stream_getc(client->ibuf);
-	instance = stream_getw(client->ibuf);
+	STREAM_GETC(msg, afi);
+	STREAM_GETC(msg, type);
+	STREAM_GETW(msg, instance);
 
-	if (type == 0 || type >= ZEBRA_ROUTE_MAX)
+	if (IS_ZEBRA_DEBUG_EVENT)
+		zlog_debug(
+			"%s: client proto %s afi=%d, wants %s, vrf %u, instance=%d",
+			__func__, zebra_route_string(client->proto), afi,
+			zebra_route_string(type), zvrf_id(zvrf), instance);
+
+	if (afi == 0 || afi > AFI_MAX) {
+		zlog_warn("%s: Specified afi %d does not exist",
+			  __PRETTY_FUNCTION__, afi);
 		return;
+	}
+
+	if (type == 0 || type >= ZEBRA_ROUTE_MAX) {
+		zlog_warn("%s: Specified Route Type %d does not exist",
+			  __PRETTY_FUNCTION__, type);
+		return;
+	}
 
 	if (instance) {
 		if (!redist_check_instance(&client->mi_redist[afi][type],
@@ -272,26 +297,40 @@ void zebra_redistribute_add(int command, struct zserv *client, int length,
 	} else {
 		if (!vrf_bitmap_check(client->redist[afi][type],
 				      zvrf_id(zvrf))) {
+			if (IS_ZEBRA_DEBUG_EVENT)
+				zlog_debug("%s: setting vrf %u redist bitmap",
+					   __func__, zvrf_id(zvrf));
 			vrf_bitmap_set(client->redist[afi][type],
 				       zvrf_id(zvrf));
 			zebra_redistribute(client, type, 0, zvrf_id(zvrf), afi);
 		}
 	}
+
+stream_failure:
+	return;
 }
 
-void zebra_redistribute_delete(int command, struct zserv *client, int length,
-			       struct zebra_vrf *zvrf)
+void zebra_redistribute_delete(ZAPI_HANDLER_ARGS)
 {
-	afi_t afi;
-	int type;
-	u_short instance;
+	afi_t afi = 0;
+	int type = 0;
+	unsigned short instance;
 
-	afi = stream_getc(client->ibuf);
-	type = stream_getc(client->ibuf);
-	instance = stream_getw(client->ibuf);
+	STREAM_GETC(msg, afi);
+	STREAM_GETC(msg, type);
+	STREAM_GETW(msg, instance);
 
-	if (type == 0 || type >= ZEBRA_ROUTE_MAX)
+	if (afi == 0 || afi > AFI_MAX) {
+		zlog_warn("%s: Specified afi %d does not exist",
+			  __PRETTY_FUNCTION__, afi);
 		return;
+	}
+
+	if (type == 0 || type >= ZEBRA_ROUTE_MAX) {
+		zlog_warn("%s: Specified Route Type %d does not exist",
+			  __PRETTY_FUNCTION__, type);
+		return;
+	}
 
 	/*
 	 * NOTE: no need to withdraw the previously advertised routes. The
@@ -303,17 +342,18 @@ void zebra_redistribute_delete(int command, struct zserv *client, int length,
 		redist_del_instance(&client->mi_redist[afi][type], instance);
 	else
 		vrf_bitmap_unset(client->redist[afi][type], zvrf_id(zvrf));
+
+stream_failure:
+	return;
 }
 
-void zebra_redistribute_default_add(int command, struct zserv *client,
-				    int length, struct zebra_vrf *zvrf)
+void zebra_redistribute_default_add(ZAPI_HANDLER_ARGS)
 {
 	vrf_bitmap_set(client->redist_default, zvrf_id(zvrf));
 	zebra_redistribute_default(client, zvrf_id(zvrf));
 }
 
-void zebra_redistribute_default_delete(int command, struct zserv *client,
-				       int length, struct zebra_vrf *zvrf)
+void zebra_redistribute_default_delete(ZAPI_HANDLER_ARGS)
 {
 	vrf_bitmap_unset(client->redist_default, zvrf_id(zvrf));
 }
@@ -325,7 +365,8 @@ void zebra_interface_up_update(struct interface *ifp)
 	struct zserv *client;
 
 	if (IS_ZEBRA_DEBUG_EVENT)
-		zlog_debug("MESSAGE: ZEBRA_INTERFACE_UP %s", ifp->name);
+		zlog_debug("MESSAGE: ZEBRA_INTERFACE_UP %s(%u)",
+			   ifp->name, ifp->vrf_id);
 
 	if (ifp->ptm_status || !ifp->ptm_enable) {
 		for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client))
@@ -344,7 +385,8 @@ void zebra_interface_down_update(struct interface *ifp)
 	struct zserv *client;
 
 	if (IS_ZEBRA_DEBUG_EVENT)
-		zlog_debug("MESSAGE: ZEBRA_INTERFACE_DOWN %s", ifp->name);
+		zlog_debug("MESSAGE: ZEBRA_INTERFACE_DOWN %s(%u)",
+			   ifp->name, ifp->vrf_id);
 
 	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client)) {
 		zsend_interface_update(ZEBRA_INTERFACE_DOWN, client, ifp);
@@ -358,7 +400,7 @@ void zebra_interface_add_update(struct interface *ifp)
 	struct zserv *client;
 
 	if (IS_ZEBRA_DEBUG_EVENT)
-		zlog_debug("MESSAGE: ZEBRA_INTERFACE_ADD %s[%d]", ifp->name,
+		zlog_debug("MESSAGE: ZEBRA_INTERFACE_ADD %s(%u)", ifp->name,
 			   ifp->vrf_id);
 
 	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client))
@@ -375,7 +417,8 @@ void zebra_interface_delete_update(struct interface *ifp)
 	struct zserv *client;
 
 	if (IS_ZEBRA_DEBUG_EVENT)
-		zlog_debug("MESSAGE: ZEBRA_INTERFACE_DELETE %s", ifp->name);
+		zlog_debug("MESSAGE: ZEBRA_INTERFACE_DELETE %s(%u)",
+			   ifp->name, ifp->vrf_id);
 
 	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client)) {
 		client->ifdel_cnt++;
@@ -395,14 +438,17 @@ void zebra_interface_address_add_update(struct interface *ifp,
 		char buf[PREFIX_STRLEN];
 
 		p = ifc->address;
-		zlog_debug("MESSAGE: ZEBRA_INTERFACE_ADDRESS_ADD %s on %s",
-			   prefix2str(p, buf, sizeof(buf)), ifc->ifp->name);
+		zlog_debug("MESSAGE: ZEBRA_INTERFACE_ADDRESS_ADD %s on %s(%u)",
+			   prefix2str(p, buf, sizeof(buf)), ifp->name,
+			   ifp->vrf_id);
 	}
 
 	if (!CHECK_FLAG(ifc->conf, ZEBRA_IFC_REAL))
 		zlog_warn(
 			"WARNING: advertising address to clients that is not yet usable.");
 
+	zebra_vxlan_add_del_gw_macip(ifp, ifc->address, 1);
+
 	router_id_add_address(ifc);
 
 	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client))
@@ -425,10 +471,13 @@ void zebra_interface_address_delete_update(struct interface *ifp,
 		char buf[PREFIX_STRLEN];
 
 		p = ifc->address;
-		zlog_debug("MESSAGE: ZEBRA_INTERFACE_ADDRESS_DELETE %s on %s",
-			   prefix2str(p, buf, sizeof(buf)), ifc->ifp->name);
+		zlog_debug("MESSAGE: ZEBRA_INTERFACE_ADDRESS_DELETE %s on %s(%u)",
+			   prefix2str(p, buf, sizeof(buf)),
+			   ifp->name, ifp->vrf_id);
 	}
 
+	zebra_vxlan_add_del_gw_macip(ifp, ifc->address, 0);
+
 	router_id_del_address(ifc);
 
 	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client))
@@ -483,95 +532,82 @@ void zebra_interface_vrf_update_add(struct interface *ifp, vrf_id_t old_vrf_id)
 	}
 }
 
-int zebra_add_import_table_entry(struct route_node *rn, struct rib *rib,
+int zebra_add_import_table_entry(struct route_node *rn, struct route_entry *re,
 				 const char *rmap_name)
 {
-	struct rib *newrib;
+	struct route_entry *newre;
+	struct route_entry *same;
 	struct prefix p;
-	struct nexthop *nhop;
-	union g_addr *gate;
 	route_map_result_t ret = RMAP_MATCH;
+	afi_t afi;
 
+	afi = family2afi(rn->p.family);
 	if (rmap_name)
 		ret = zebra_import_table_route_map_check(
-			AFI_IP, rib->type, &rn->p, rib->nexthop, rib->vrf_id,
-			rib->tag, rmap_name);
+			afi, re->type, re->instance, &rn->p, re->ng.nexthop,
+			re->vrf_id, re->tag, rmap_name);
 
-	if (ret == RMAP_MATCH) {
-		if (rn->p.family == AF_INET) {
-			p.family = AF_INET;
-			p.prefixlen = rn->p.prefixlen;
-			p.u.prefix4 = rn->p.u.prefix4;
-
-			if (rib->nexthop_num == 1) {
-				nhop = rib->nexthop;
-				if (nhop->type == NEXTHOP_TYPE_IFINDEX)
-					gate = NULL;
-				else
-					gate = (union g_addr *)&nhop->gate.ipv4;
-
-				rib_add(AFI_IP, SAFI_UNICAST, rib->vrf_id,
-					ZEBRA_ROUTE_TABLE, rib->table, 0, &p,
-					NULL, gate,
-					(union g_addr *)&nhop->src.ipv4,
-					nhop->ifindex, zebrad.rtm_table_default,
-					rib->metric, rib->mtu,
-					zebra_import_table_distance
-						[AFI_IP][rib->table]);
-			} else if (rib->nexthop_num > 1) {
-				newrib = XCALLOC(MTYPE_RIB, sizeof(struct rib));
-				newrib->type = ZEBRA_ROUTE_TABLE;
-				newrib->distance =
-					zebra_import_table_distance[AFI_IP]
-								   [rib->table];
-				newrib->flags = rib->flags;
-				newrib->metric = rib->metric;
-				newrib->mtu = rib->mtu;
-				newrib->table = zebrad.rtm_table_default;
-				newrib->nexthop_num = 0;
-				newrib->uptime = time(NULL);
-				newrib->instance = rib->table;
-
-				/* Assuming these routes are never recursive */
-				for (nhop = rib->nexthop; nhop;
-				     nhop = nhop->next)
-					rib_copy_nexthops(newrib, nhop);
-
-				rib_add_multipath(AFI_IP, SAFI_UNICAST, &p,
-						  NULL, newrib);
-			}
-		}
-	} else {
-		zebra_del_import_table_entry(rn, rib);
+	if (ret != RMAP_MATCH) {
+		UNSET_FLAG(re->flags, ZEBRA_FLAG_SELECTED);
+		zebra_del_import_table_entry(rn, re);
+		return 0;
 	}
-	/* DD: Add IPv6 code */
+
+	prefix_copy(&p, &rn->p);
+
+	RNODE_FOREACH_RE (rn, same) {
+		if (CHECK_FLAG(same->status, ROUTE_ENTRY_REMOVED))
+			continue;
+
+		if (same->type == re->type && same->instance == re->instance
+		    && same->table == re->table
+		    && same->type != ZEBRA_ROUTE_CONNECT)
+			break;
+	}
+
+	if (same) {
+		UNSET_FLAG(same->flags, ZEBRA_FLAG_SELECTED);
+		zebra_del_import_table_entry(rn, same);
+	}
+
+	newre = XCALLOC(MTYPE_RE, sizeof(struct route_entry));
+	newre->type = ZEBRA_ROUTE_TABLE;
+	newre->distance = zebra_import_table_distance[afi][re->table];
+	newre->flags = re->flags;
+	newre->metric = re->metric;
+	newre->mtu = re->mtu;
+	newre->table = zebrad.rtm_table_default;
+	newre->nexthop_num = 0;
+	newre->uptime = time(NULL);
+	newre->instance = re->table;
+	route_entry_copy_nexthops(newre, re->ng.nexthop);
+
+	rib_add_multipath(afi, SAFI_UNICAST, &p, NULL, newre);
+
 	return 0;
 }
 
-int zebra_del_import_table_entry(struct route_node *rn, struct rib *rib)
+int zebra_del_import_table_entry(struct route_node *rn, struct route_entry *re)
 {
 	struct prefix p;
+	afi_t afi;
 
-	if (rn->p.family == AF_INET) {
-		p.family = AF_INET;
-		p.prefixlen = rn->p.prefixlen;
-		p.u.prefix4 = rn->p.u.prefix4;
+	afi = family2afi(rn->p.family);
+	prefix_copy(&p, &rn->p);
 
-		rib_delete(AFI_IP, SAFI_UNICAST, rib->vrf_id, ZEBRA_ROUTE_TABLE,
-			   rib->table, rib->flags, &p, NULL, NULL, 0,
-			   zebrad.rtm_table_default);
-	}
-	/* DD: Add IPv6 code */
+	rib_delete(afi, SAFI_UNICAST, re->vrf_id, ZEBRA_ROUTE_TABLE, re->table,
+		   re->flags, &p, NULL, re->ng.nexthop,
+		   zebrad.rtm_table_default, re->metric, false, NULL);
 
 	return 0;
 }
 
 /* Assuming no one calls this with the main routing table */
-int zebra_import_table(afi_t afi, u_int32_t table_id, u_int32_t distance,
+int zebra_import_table(afi_t afi, uint32_t table_id, uint32_t distance,
 		       const char *rmap_name, int add)
 {
 	struct route_table *table;
-	struct rib *rib;
+	struct route_entry *re;
 	struct route_node *rn;
 
 	if (!is_zebra_valid_kernel_table(table_id)
@@ -597,8 +633,10 @@ int zebra_import_table(afi_t afi, u_int32_t table_id, u_int32_t distance,
 		else {
 			rmap_name =
 				zebra_get_import_table_route_map(afi, table_id);
-			if (rmap_name)
+			if (rmap_name) {
 				zebra_del_import_table_route_map(afi, table_id);
+				rmap_name = NULL;
+			}
 		}
 
 		zebra_import_table_used[afi][table_id] = 1;
@@ -609,8 +647,10 @@ int zebra_import_table(afi_t afi, u_int32_t table_id, u_int32_t distance,
 			ZEBRA_TABLE_DISTANCE_DEFAULT;
 
 		rmap_name = zebra_get_import_table_route_map(afi, table_id);
-		if (rmap_name)
+		if (rmap_name) {
 			zebra_del_import_table_route_map(afi, table_id);
+			rmap_name = NULL;
+		}
 	}
 
 	for (rn = route_top(table); rn; rn = route_next(rn)) {
@@ -620,23 +660,21 @@ int zebra_import_table(afi_t afi, u_int32_t table_id, u_int32_t distance,
 		if (!rn->info)
 			continue;
 
-		RNODE_FOREACH_RIB(rn, rib)
-		{
-			if (CHECK_FLAG(rib->status, RIB_ENTRY_REMOVED))
+		RNODE_FOREACH_RE (rn, re) {
+			if (CHECK_FLAG(re->status, ROUTE_ENTRY_REMOVED))
 				continue;
 			break;
 		}
 
-		if (!rib)
+		if (!re)
 			continue;
 
 		if (((afi == AFI_IP) && (rn->p.family == AF_INET))
 		    || ((afi == AFI_IP6) && (rn->p.family == AF_INET6))) {
 			if (add)
-				zebra_add_import_table_entry(rn, rib,
-							     rmap_name);
+				zebra_add_import_table_entry(rn, re, rmap_name);
 			else
-				zebra_del_import_table_entry(rn, rib);
+				zebra_del_import_table_entry(rn, re);
 		}
 	}
 	return 0;
@@ -652,28 +690,25 @@ int zebra_import_table_config(struct vty *vty)
 
 	for (afi = AFI_IP; afi < AFI_MAX; afi++) {
 		for (i = 1; i < ZEBRA_KERNEL_TABLE_MAX; i++) {
-			if (is_zebra_import_table_enabled(afi, i)) {
-				if (zebra_import_table_distance[afi][i]
-				    != ZEBRA_TABLE_DISTANCE_DEFAULT) {
-					vty_out(vty,
-						"%s import-table %d distance %d",
-						afi_str[afi], i,
-						zebra_import_table_distance[afi]
-									   [i]);
-				} else {
-					vty_out(vty, "%s import-table %d",
-						afi_str[afi], i);
-				}
+			if (!is_zebra_import_table_enabled(afi, i))
+				continue;
 
-				rmap_name = zebra_get_import_table_route_map(
-					afi, i);
-				if (rmap_name)
-					vty_out(vty, " route-map %s",
-						rmap_name);
-
-				vty_out(vty, "%s", VTY_NEWLINE);
-				write = 1;
+			if (zebra_import_table_distance[afi][i]
+			    != ZEBRA_TABLE_DISTANCE_DEFAULT) {
+				vty_out(vty, "%s import-table %d distance %d",
+					afi_str[afi], i,
+					zebra_import_table_distance[afi][i]);
+			} else {
+				vty_out(vty, "%s import-table %d", afi_str[afi],
+					i);
 			}
+
+			rmap_name = zebra_get_import_table_route_map(afi, i);
+			if (rmap_name)
+				vty_out(vty, " route-map %s", rmap_name);
+
+			vty_out(vty, "\n");
+			write = 1;
 		}
 	}
 
@@ -685,48 +720,45 @@ void zebra_import_table_rm_update()
 	afi_t afi;
 	int i;
 	struct route_table *table;
-	struct rib *rib;
+	struct route_entry *re;
 	struct route_node *rn;
 	const char *rmap_name;
 
 	for (afi = AFI_IP; afi < AFI_MAX; afi++) {
 		for (i = 1; i < ZEBRA_KERNEL_TABLE_MAX; i++) {
-			if (is_zebra_import_table_enabled(afi, i)) {
-				rmap_name = zebra_get_import_table_route_map(
-					afi, i);
-				if (!rmap_name)
-					return;
+			if (!is_zebra_import_table_enabled(afi, i))
+				continue;
 
-				table = zebra_vrf_other_route_table(
-					afi, i, VRF_DEFAULT);
-				for (rn = route_top(table); rn;
-				     rn = route_next(rn)) {
-					/* For each entry in the non-default
-					 * routing table,
-					 * add the entry in the main table
-					 */
-					if (!rn->info)
+			rmap_name = zebra_get_import_table_route_map(afi, i);
+			if (!rmap_name)
+				return;
+
+			table = zebra_vrf_other_route_table(afi, i,
+							    VRF_DEFAULT);
+			for (rn = route_top(table); rn; rn = route_next(rn)) {
+				/* For each entry in the non-default
+				 * routing table,
+				 * add the entry in the main table
+				 */
+				if (!rn->info)
+					continue;
+
+				RNODE_FOREACH_RE (rn, re) {
+					if (CHECK_FLAG(re->status,
+						       ROUTE_ENTRY_REMOVED))
 						continue;
-
-					RNODE_FOREACH_RIB(rn, rib)
-					{
-						if (CHECK_FLAG(
-							    rib->status,
-							    RIB_ENTRY_REMOVED))
-							continue;
-						break;
-					}
-
-					if (!rib)
-						continue;
-
-					if (((afi == AFI_IP)
-					     && (rn->p.family == AF_INET))
-					    || ((afi == AFI_IP6)
-						&& (rn->p.family == AF_INET6)))
-						zebra_add_import_table_entry(
-							rn, rib, rmap_name);
+					break;
 				}
+
+				if (!re)
+					continue;
+
+				if (((afi == AFI_IP)
+				     && (rn->p.family == AF_INET))
+				    || ((afi == AFI_IP6)
+					&& (rn->p.family == AF_INET6)))
+					zebra_add_import_table_entry(rn, re,
+								     rmap_name);
 			}
 		}
 	}
@@ -741,8 +773,8 @@ void zebra_interface_parameters_update(struct interface *ifp)
 	struct zserv *client;
 
 	if (IS_ZEBRA_DEBUG_EVENT)
-		zlog_debug("MESSAGE: ZEBRA_INTERFACE_LINK_PARAMS %s",
-			   ifp->name);
+		zlog_debug("MESSAGE: ZEBRA_INTERFACE_LINK_PARAMS %s(%u)",
+			   ifp->name, ifp->vrf_id);
 
 	for (ALL_LIST_ELEMENTS(zebrad.client_list, node, nnode, client))
 		if (client->ifinfo)
