@@ -15,10 +15,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -35,12 +34,13 @@
 #include "command.h"
 #include "privs.h"
 #include "vrf.h"
+#include "ns.h"
 
 #include "zebra/interface.h"
 #include "zebra/rtadv.h"
 #include "zebra/debug.h"
 #include "zebra/rib.h"
-#include "zebra/zserv.h"
+#include "zebra/zapi_msg.h"
 #include "zebra/zebra_ns.h"
 #include "zebra/zebra_vrf.h"
 
@@ -63,6 +63,10 @@ extern struct zebra_privs_t zserv_privs;
 #define ALLNODE   "ff02::1"
 #define ALLROUTER "ff02::2"
 
+/* Order is intentional.  Matches RFC4191.  This array is also used for
+   command matching, so only modify with care. */
+const char *rtadv_pref_strs[] = {"medium", "high", "INVALID", "low", 0};
+
 enum rtadv_event {
 	RTADV_START,
 	RTADV_STOP,
@@ -91,7 +95,7 @@ static int rtadv_increment_received(struct zebra_ns *zns, ifindex_t *ifindex)
 	return ret;
 }
 
-static int rtadv_recv_packet(struct zebra_ns *zns, int sock, u_char *buf,
+static int rtadv_recv_packet(struct zebra_ns *zns, int sock, uint8_t *buf,
 			     int buflen, struct sockaddr_in6 *from,
 			     ifindex_t *ifindex, int *hoplimit)
 {
@@ -104,6 +108,7 @@ static int rtadv_recv_packet(struct zebra_ns *zns, int sock, u_char *buf,
 	char adata[1024];
 
 	/* Fill in message and iovec. */
+	memset(&msg, 0, sizeof(msg));
 	msg.msg_name = (void *)from;
 	msg.msg_namelen = sizeof(struct sockaddr_in6);
 	msg.msg_iov = &iov;
@@ -159,10 +164,10 @@ static void rtadv_send_packet(int sock, struct interface *ifp)
 	int len = 0;
 	struct zebra_if *zif;
 	struct rtadv_prefix *rprefix;
-	u_char all_nodes_addr[] = {0xff, 0x02, 0, 0, 0, 0, 0, 0,
-				   0,    0,    0, 0, 0, 0, 0, 1};
+	uint8_t all_nodes_addr[] = {0xff, 0x02, 0, 0, 0, 0, 0, 0,
+				    0,    0,    0, 0, 0, 0, 0, 1};
 	struct listnode *node;
-	u_int16_t pkt_RouterLifetime;
+	uint16_t pkt_RouterLifetime;
 
 	/*
 	 * Allocate control message bufffer.  This is dynamic because
@@ -172,11 +177,13 @@ static void rtadv_send_packet(int sock, struct interface *ifp)
 	 */
 	if (adata == NULL) {
 		/* XXX Free on shutdown. */
-		adata = malloc(CMSG_SPACE(sizeof(struct in6_pktinfo)));
+		adata = calloc(1, CMSG_SPACE(sizeof(struct in6_pktinfo)));
 
-		if (adata == NULL)
+		if (adata == NULL) {
 			zlog_err(
 				"rtadv_send_packet: can't malloc control data");
+			exit(-1);
+		}
 	}
 
 	/* Logging of packet. */
@@ -308,7 +315,7 @@ static void rtadv_send_packet(int sock, struct interface *ifp)
 
 #ifdef DEBUG
 		{
-			u_char buf[INET6_ADDRSTRLEN];
+			uint8_t buf[INET6_ADDRSTRLEN];
 
 			zlog_debug("DEBUG %s",
 				   inet_ntop(AF_INET6, &pinfo->nd_opt_pi_prefix,
@@ -377,7 +384,6 @@ static int rtadv_timer(struct thread *thread)
 {
 	struct zebra_ns *zns = THREAD_ARG(thread);
 	struct vrf *vrf;
-	struct listnode *node, *nnode;
 	struct interface *ifp;
 	struct zebra_if *zif;
 	int period;
@@ -391,41 +397,48 @@ static int rtadv_timer(struct thread *thread)
 		rtadv_event(zns, RTADV_TIMER_MSEC, 10 /* 10 ms */);
 	}
 
-	RB_FOREACH(vrf, vrf_id_head, &vrfs_by_id)
-	for (ALL_LIST_ELEMENTS(vrf->iflist, node, nnode, ifp)) {
-		if (if_is_loopback(ifp)
-		    || CHECK_FLAG(ifp->status, ZEBRA_INTERFACE_VRF_LOOPBACK)
-		    || !if_is_operative(ifp))
-			continue;
+	RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id)
+		FOR_ALL_INTERFACES (vrf, ifp) {
+			if (if_is_loopback(ifp)
+			    || CHECK_FLAG(ifp->status,
+					  ZEBRA_INTERFACE_VRF_LOOPBACK)
+			    || !if_is_operative(ifp))
+				continue;
 
-		zif = ifp->info;
+			zif = ifp->info;
 
-		if (zif->rtadv.AdvSendAdvertisements) {
-			if (zif->rtadv.inFastRexmit) {
-				/* We assume we fast rexmit every sec so no
-				 * additional vars */
-				if (--zif->rtadv.NumFastReXmitsRemain <= 0)
-					zif->rtadv.inFastRexmit = 0;
+			if (zif->rtadv.AdvSendAdvertisements) {
+				if (zif->rtadv.inFastRexmit) {
+					/* We assume we fast rexmit every sec so
+					 * no
+					 * additional vars */
+					if (--zif->rtadv.NumFastReXmitsRemain
+					    <= 0)
+						zif->rtadv.inFastRexmit = 0;
 
-				if (IS_ZEBRA_DEBUG_SEND)
-					zlog_debug(
-						"Fast RA Rexmit on interface %s",
-						ifp->name);
+					if (IS_ZEBRA_DEBUG_SEND)
+						zlog_debug(
+							"Fast RA Rexmit on interface %s",
+							ifp->name);
 
-				rtadv_send_packet(zns->rtadv.sock, ifp);
-			} else {
-				zif->rtadv.AdvIntervalTimer -= period;
-				if (zif->rtadv.AdvIntervalTimer <= 0) {
-					/* FIXME: using MaxRtrAdvInterval each
-					   time isn't what section
-					   6.2.4 of RFC4861 tells to do. */
-					zif->rtadv.AdvIntervalTimer =
-						zif->rtadv.MaxRtrAdvInterval;
 					rtadv_send_packet(zns->rtadv.sock, ifp);
+				} else {
+					zif->rtadv.AdvIntervalTimer -= period;
+					if (zif->rtadv.AdvIntervalTimer <= 0) {
+						/* FIXME: using
+						   MaxRtrAdvInterval each
+						   time isn't what section
+						   6.2.4 of RFC4861 tells to do.
+						   */
+						zif->rtadv.AdvIntervalTimer =
+							zif->rtadv
+								.MaxRtrAdvInterval;
+						rtadv_send_packet(
+							zns->rtadv.sock, ifp);
+					}
 				}
 			}
 		}
-	}
 
 	return 0;
 }
@@ -439,7 +452,7 @@ static void rtadv_process_solicit(struct interface *ifp)
 	rtadv_send_packet(zns->rtadv.sock, ifp);
 }
 
-static void rtadv_process_advert(u_char *msg, unsigned int len,
+static void rtadv_process_advert(uint8_t *msg, unsigned int len,
 				 struct interface *ifp,
 				 struct sockaddr_in6 *addr)
 {
@@ -513,7 +526,7 @@ static void rtadv_process_advert(u_char *msg, unsigned int len,
 }
 
 
-static void rtadv_process_packet(u_char *buf, unsigned int len,
+static void rtadv_process_packet(uint8_t *buf, unsigned int len,
 				 ifindex_t ifindex, int hoplimit,
 				 struct sockaddr_in6 *from,
 				 struct zebra_ns *zns)
@@ -583,7 +596,7 @@ static int rtadv_read(struct thread *thread)
 {
 	int sock;
 	int len;
-	u_char buf[RTADV_MSG_SIZE];
+	uint8_t buf[RTADV_MSG_SIZE];
 	struct sockaddr_in6 from;
 	ifindex_t ifindex = 0;
 	int hoplimit = -1;
@@ -609,7 +622,7 @@ static int rtadv_read(struct thread *thread)
 	return 0;
 }
 
-static int rtadv_make_socket(void)
+static int rtadv_make_socket(ns_id_t ns_id)
 {
 	int sock;
 	int ret = 0;
@@ -619,14 +632,13 @@ static int rtadv_make_socket(void)
 		zlog_err("rtadv_make_socket: could not raise privs, %s",
 			 safe_strerror(errno));
 
-	sock = socket(AF_INET6, SOCK_RAW, IPPROTO_ICMPV6);
+	sock = ns_socket(AF_INET6, SOCK_RAW, IPPROTO_ICMPV6, ns_id);
 
 	if (zserv_privs.change(ZPRIVS_LOWER))
 		zlog_err("rtadv_make_socket: could not lower privs, %s",
 			 safe_strerror(errno));
 
 	if (sock < 0) {
-		close(sock);
 		return -1;
 	}
 
@@ -664,6 +676,7 @@ static int rtadv_make_socket(void)
 			 sizeof(struct icmp6_filter));
 	if (ret < 0) {
 		zlog_info("ICMP6_FILTER set fail: %s", safe_strerror(errno));
+		close(sock);
 		return ret;
 	}
 
@@ -788,20 +801,19 @@ static void ipv6_nd_suppress_ra_set(struct interface *ifp,
  * if the operator has explicitly enabled RA. The enable request can also
  * specify a RA interval (in seconds).
  */
-void zebra_interface_radv_set(struct zserv *client, int sock, u_short length,
-			      struct zebra_vrf *zvrf, int enable)
+static void zebra_interface_radv_set(ZAPI_HANDLER_ARGS, int enable)
 {
 	struct stream *s;
-	unsigned int ifindex;
+	ifindex_t ifindex;
 	struct interface *ifp;
 	struct zebra_if *zif;
 	int ra_interval;
 
-	s = client->ibuf;
+	s = msg;
 
 	/* Get interface index and RA interval. */
-	ifindex = stream_getl(s);
-	ra_interval = stream_getl(s);
+	STREAM_GETL(s, ifindex);
+	STREAM_GETL(s, ra_interval);
 
 	if (IS_ZEBRA_DEBUG_EVENT)
 		zlog_debug("%u: IF %u RA %s from client %s, interval %ds",
@@ -826,17 +838,33 @@ void zebra_interface_radv_set(struct zserv *client, int sock, u_short length,
 
 	zif = ifp->info;
 	if (enable) {
+		SET_FLAG(zif->rtadv.ra_configured, BGP_RA_CONFIGURED);
 		ipv6_nd_suppress_ra_set(ifp, RA_ENABLE);
 		if (ra_interval
-		    && (ra_interval * 1000) < zif->rtadv.MaxRtrAdvInterval)
+		    && (ra_interval * 1000) < zif->rtadv.MaxRtrAdvInterval
+		    && !CHECK_FLAG(zif->rtadv.ra_configured,
+				   VTY_RA_INTERVAL_CONFIGURED))
 			zif->rtadv.MaxRtrAdvInterval = ra_interval * 1000;
 	} else {
-		if (!zif->rtadv.configured) {
+		UNSET_FLAG(zif->rtadv.ra_configured, BGP_RA_CONFIGURED);
+		if (!CHECK_FLAG(zif->rtadv.ra_configured,
+				VTY_RA_INTERVAL_CONFIGURED))
 			zif->rtadv.MaxRtrAdvInterval =
 				RTADV_MAX_RTR_ADV_INTERVAL;
+		if (!CHECK_FLAG(zif->rtadv.ra_configured, VTY_RA_CONFIGURED))
 			ipv6_nd_suppress_ra_set(ifp, RA_SUPPRESS);
-		}
 	}
+stream_failure:
+	return;
+}
+
+void zebra_interface_radv_disable(ZAPI_HANDLER_ARGS)
+{
+	zebra_interface_radv_set(client, hdr, msg, zvrf, 0);
+}
+void zebra_interface_radv_enable(ZAPI_HANDLER_ARGS)
+{
+	zebra_interface_radv_set(client, hdr, msg, zvrf, 1);
 }
 
 DEFUN (ipv6_nd_suppress_ra,
@@ -852,13 +880,14 @@ DEFUN (ipv6_nd_suppress_ra,
 	if (if_is_loopback(ifp)
 	    || CHECK_FLAG(ifp->status, ZEBRA_INTERFACE_VRF_LOOPBACK)) {
 		vty_out(vty,
-			"Cannot configure IPv6 Router Advertisements on this  interface%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+			"Cannot configure IPv6 Router Advertisements on this  interface\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
-	ipv6_nd_suppress_ra_set(ifp, RA_SUPPRESS);
-	zif->rtadv.configured = 0;
+	if (!CHECK_FLAG(zif->rtadv.ra_configured, BGP_RA_CONFIGURED))
+		ipv6_nd_suppress_ra_set(ifp, RA_SUPPRESS);
+
+	UNSET_FLAG(zif->rtadv.ra_configured, VTY_RA_CONFIGURED);
 	return CMD_SUCCESS;
 }
 
@@ -876,13 +905,12 @@ DEFUN (no_ipv6_nd_suppress_ra,
 	if (if_is_loopback(ifp)
 	    || CHECK_FLAG(ifp->status, ZEBRA_INTERFACE_VRF_LOOPBACK)) {
 		vty_out(vty,
-			"Cannot configure IPv6 Router Advertisements on this interface%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+			"Cannot configure IPv6 Router Advertisements on this interface\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	ipv6_nd_suppress_ra_set(ifp, RA_ENABLE);
-	zif->rtadv.configured = 1;
+	SET_FLAG(zif->rtadv.ra_configured, VTY_RA_CONFIGURED);
 	return CMD_SUCCESS;
 }
 
@@ -903,14 +931,12 @@ DEFUN (ipv6_nd_ra_interval_msec,
 	struct zebra_ns *zns;
 
 	zns = zvrf->zns;
-	VTY_GET_INTEGER_RANGE("router advertisement interval", interval,
-			      argv[idx_number]->arg, 70, 1800000);
+	interval = strtoul(argv[idx_number]->arg, NULL, 10);
 	if ((zif->rtadv.AdvDefaultLifetime != -1
 	     && interval > (unsigned)zif->rtadv.AdvDefaultLifetime * 1000)) {
 		vty_out(vty,
-			"This ra-interval would conflict with configured ra-lifetime!%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+			"This ra-interval would conflict with configured ra-lifetime!\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (zif->rtadv.MaxRtrAdvInterval % 1000)
@@ -919,6 +945,7 @@ DEFUN (ipv6_nd_ra_interval_msec,
 	if (interval % 1000)
 		zns->rtadv.adv_msec_if_count++;
 
+	SET_FLAG(zif->rtadv.ra_configured, VTY_RA_INTERVAL_CONFIGURED);
 	zif->rtadv.MaxRtrAdvInterval = interval;
 	zif->rtadv.MinRtrAdvInterval = 0.33 * interval;
 	zif->rtadv.AdvIntervalTimer = 0;
@@ -942,14 +969,12 @@ DEFUN (ipv6_nd_ra_interval,
 	struct zebra_ns *zns;
 
 	zns = zvrf->zns;
-	VTY_GET_INTEGER_RANGE("router advertisement interval", interval,
-			      argv[idx_number]->arg, 1, 1800);
+	interval = strtoul(argv[idx_number]->arg, NULL, 10);
 	if ((zif->rtadv.AdvDefaultLifetime != -1
 	     && interval > (unsigned)zif->rtadv.AdvDefaultLifetime)) {
 		vty_out(vty,
-			"This ra-interval would conflict with configured ra-lifetime!%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+			"This ra-interval would conflict with configured ra-lifetime!\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (zif->rtadv.MaxRtrAdvInterval % 1000)
@@ -958,6 +983,7 @@ DEFUN (ipv6_nd_ra_interval,
 	/* convert to milliseconds */
 	interval = interval * 1000;
 
+	SET_FLAG(zif->rtadv.ra_configured, VTY_RA_INTERVAL_CONFIGURED);
 	zif->rtadv.MaxRtrAdvInterval = interval;
 	zif->rtadv.MinRtrAdvInterval = 0.33 * interval;
 	zif->rtadv.AdvIntervalTimer = 0;
@@ -987,9 +1013,15 @@ DEFUN (no_ipv6_nd_ra_interval,
 	if (zif->rtadv.MaxRtrAdvInterval % 1000)
 		zns->rtadv.adv_msec_if_count--;
 
-	zif->rtadv.MaxRtrAdvInterval = RTADV_MAX_RTR_ADV_INTERVAL;
-	zif->rtadv.MinRtrAdvInterval = RTADV_MIN_RTR_ADV_INTERVAL;
+	UNSET_FLAG(zif->rtadv.ra_configured, VTY_RA_INTERVAL_CONFIGURED);
+
+	if (CHECK_FLAG(zif->rtadv.ra_configured, BGP_RA_CONFIGURED))
+		zif->rtadv.MaxRtrAdvInterval = 10000;
+	else
+		zif->rtadv.MaxRtrAdvInterval = RTADV_MAX_RTR_ADV_INTERVAL;
+
 	zif->rtadv.AdvIntervalTimer = zif->rtadv.MaxRtrAdvInterval;
+	zif->rtadv.MinRtrAdvInterval = RTADV_MIN_RTR_ADV_INTERVAL;
 
 	return CMD_SUCCESS;
 }
@@ -1007,8 +1039,7 @@ DEFUN (ipv6_nd_ra_lifetime,
 	struct zebra_if *zif = ifp->info;
 	int lifetime;
 
-	VTY_GET_INTEGER_RANGE("router lifetime", lifetime,
-			      argv[idx_number]->arg, 0, 9000);
+	lifetime = strtoul(argv[idx_number]->arg, NULL, 10);
 
 	/* The value to be placed in the Router Lifetime field
 	 * of Router Advertisements sent from the interface,
@@ -1016,9 +1047,8 @@ DEFUN (ipv6_nd_ra_lifetime,
 	 * MaxRtrAdvInterval and 9000 seconds. -- RFC4861, 6.2.1 */
 	if ((lifetime != 0 && lifetime * 1000 < zif->rtadv.MaxRtrAdvInterval)) {
 		vty_out(vty,
-			"This ra-lifetime would conflict with configured ra-interval%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+			"This ra-lifetime would conflict with configured ra-interval\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	zif->rtadv.AdvDefaultLifetime = lifetime;
@@ -1054,9 +1084,7 @@ DEFUN (ipv6_nd_reachable_time,
 	int idx_number = 3;
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct zebra_if *zif = ifp->info;
-	VTY_GET_INTEGER_RANGE("reachable time", zif->rtadv.AdvReachableTime,
-			      argv[idx_number]->arg, 1,
-			      RTADV_MAX_REACHABLE_TIME);
+	zif->rtadv.AdvReachableTime = strtoul(argv[idx_number]->arg, NULL, 10);
 	return CMD_SUCCESS;
 }
 
@@ -1088,9 +1116,8 @@ DEFUN (ipv6_nd_homeagent_preference,
 	int idx_number = 3;
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct zebra_if *zif = ifp->info;
-	VTY_GET_INTEGER_RANGE("home agent preference",
-			      zif->rtadv.HomeAgentPreference,
-			      argv[idx_number]->arg, 0, 65535);
+	zif->rtadv.HomeAgentPreference =
+		strtoul(argv[idx_number]->arg, NULL, 10);
 	return CMD_SUCCESS;
 }
 
@@ -1122,9 +1149,7 @@ DEFUN (ipv6_nd_homeagent_lifetime,
 	int idx_number = 3;
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct zebra_if *zif = ifp->info;
-	VTY_GET_INTEGER_RANGE("home agent lifetime",
-			      zif->rtadv.HomeAgentLifetime,
-			      argv[idx_number]->arg, 0, RTADV_MAX_HALIFETIME);
+	zif->rtadv.HomeAgentLifetime = strtoul(argv[idx_number]->arg, NULL, 10);
 	return CMD_SUCCESS;
 }
 
@@ -1323,8 +1348,8 @@ DEFUN (ipv6_nd_prefix,
 
 	ret = str2prefix_ipv6(prefix, &rp.prefix);
 	if (!ret) {
-		vty_out(vty, "Malformed IPv6 prefix%s", VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Malformed IPv6 prefix\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	apply_mask_ipv6(&rp.prefix); /* RFC4861 4.6.2 */
 	rp.AdvOnLinkFlag = !offlink;
@@ -1342,9 +1367,8 @@ DEFUN (ipv6_nd_prefix,
 				? UINT32_MAX
 				: strtoll(preflifetime, NULL, 10);
 		if (rp.AdvPreferredLifetime > rp.AdvValidLifetime) {
-			vty_out(vty, "Invalid preferred lifetime%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
+			vty_out(vty, "Invalid preferred lifetime\n");
+			return CMD_WARNING_CONFIG_FAILED;
 		}
 	}
 
@@ -1379,15 +1403,15 @@ DEFUN (no_ipv6_nd_prefix,
 
 	ret = str2prefix_ipv6(prefix, &rp.prefix);
 	if (!ret) {
-		vty_out(vty, "Malformed IPv6 prefix%s", VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Malformed IPv6 prefix\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	apply_mask_ipv6(&rp.prefix); /* RFC4861 4.6.2 */
 
 	ret = rtadv_prefix_reset(zebra_if, &rp);
 	if (!ret) {
-		vty_out(vty, "Non-existant IPv6 prefix%s", VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Non-existant IPv6 prefix\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	return CMD_SUCCESS;
@@ -1400,8 +1424,8 @@ DEFUN (ipv6_nd_router_preference,
        "Neighbor discovery\n"
        "Default router preference\n"
        "High default router preference\n"
-       "Low default router preference\n"
-       "Medium default router preference (default)\n")
+       "Medium default router preference (default)\n"
+       "Low default router preference\n")
 {
 	int idx_high_medium_low = 3;
 	VTY_DECLVAR_CONTEXT(interface, ifp);
@@ -1452,8 +1476,7 @@ DEFUN (ipv6_nd_mtu,
 	int idx_number = 3;
 	VTY_DECLVAR_CONTEXT(interface, ifp);
 	struct zebra_if *zif = ifp->info;
-	VTY_GET_INTEGER_RANGE("MTU", zif->rtadv.AdvLinkMTU,
-			      argv[idx_number]->arg, 1, 65535);
+	zif->rtadv.AdvLinkMTU = strtoul(argv[idx_number]->arg, NULL, 10);
 	return CMD_SUCCESS;
 }
 
@@ -1472,9 +1495,76 @@ DEFUN (no_ipv6_nd_mtu,
 	return CMD_SUCCESS;
 }
 
+/* Dump interface ND information to vty. */
+static int nd_dump_vty(struct vty *vty, struct interface *ifp)
+{
+	struct zebra_if *zif;
+	struct rtadvconf *rtadv;
+	int interval;
+
+	zif = (struct zebra_if *)ifp->info;
+	rtadv = &zif->rtadv;
+
+	if (rtadv->AdvSendAdvertisements) {
+		vty_out(vty,
+			"  ND advertised reachable time is %d milliseconds\n",
+			rtadv->AdvReachableTime);
+		vty_out(vty,
+			"  ND advertised retransmit interval is %d milliseconds\n",
+			rtadv->AdvRetransTimer);
+		vty_out(vty, "  ND router advertisements sent: %d rcvd: %d\n",
+			zif->ra_sent, zif->ra_rcvd);
+		interval = rtadv->MaxRtrAdvInterval;
+		if (interval % 1000)
+			vty_out(vty,
+				"  ND router advertisements are sent every "
+				"%d milliseconds\n",
+				interval);
+		else
+			vty_out(vty,
+				"  ND router advertisements are sent every "
+				"%d seconds\n",
+				interval / 1000);
+		if (rtadv->AdvDefaultLifetime != -1)
+			vty_out(vty,
+				"  ND router advertisements live for %d seconds\n",
+				rtadv->AdvDefaultLifetime);
+		else
+			vty_out(vty,
+				"  ND router advertisements lifetime tracks ra-interval\n");
+		vty_out(vty,
+			"  ND router advertisement default router preference is "
+			"%s\n",
+			rtadv_pref_strs[rtadv->DefaultPreference]);
+		if (rtadv->AdvManagedFlag)
+			vty_out(vty,
+				"  Hosts use DHCP to obtain routable addresses.\n");
+		else
+			vty_out(vty,
+				"  Hosts use stateless autoconfig for addresses.\n");
+		if (rtadv->AdvHomeAgentFlag) {
+			vty_out(vty,
+				"  ND router advertisements with Home Agent flag bit set.\n");
+			if (rtadv->HomeAgentLifetime != -1)
+				vty_out(vty,
+					"  Home Agent lifetime is %u seconds\n",
+					rtadv->HomeAgentLifetime);
+			else
+				vty_out(vty,
+					"  Home Agent lifetime tracks ra-lifetime\n");
+			vty_out(vty, "  Home Agent preference is %u\n",
+				rtadv->HomeAgentPreference);
+		}
+		if (rtadv->AdvIntervalOption)
+			vty_out(vty,
+				"  ND router advertisements with Adv. Interval option.\n");
+	}
+	return 0;
+}
+
 
 /* Write configuration about router advertisement. */
-void rtadv_config_write(struct vty *vty, struct interface *ifp)
+static int rtadv_config_write(struct vty *vty, struct interface *ifp)
 {
 	struct zebra_if *zif;
 	struct listnode *node;
@@ -1486,54 +1576,55 @@ void rtadv_config_write(struct vty *vty, struct interface *ifp)
 
 	if (!(if_is_loopback(ifp)
 	      || CHECK_FLAG(ifp->status, ZEBRA_INTERFACE_VRF_LOOPBACK))) {
-		if (zif->rtadv.AdvSendAdvertisements)
-			vty_out(vty, " no ipv6 nd suppress-ra%s", VTY_NEWLINE);
+		if (zif->rtadv.AdvSendAdvertisements
+		    && CHECK_FLAG(zif->rtadv.ra_configured, VTY_RA_CONFIGURED))
+			vty_out(vty, " no ipv6 nd suppress-ra\n");
 	}
 
 	interval = zif->rtadv.MaxRtrAdvInterval;
-	if (interval % 1000)
-		vty_out(vty, " ipv6 nd ra-interval msec %d%s", interval,
-			VTY_NEWLINE);
-	else if (interval != RTADV_MAX_RTR_ADV_INTERVAL)
-		vty_out(vty, " ipv6 nd ra-interval %d%s", interval / 1000,
-			VTY_NEWLINE);
+	if (CHECK_FLAG(zif->rtadv.ra_configured, VTY_RA_INTERVAL_CONFIGURED)) {
+		if (interval % 1000)
+			vty_out(vty, " ipv6 nd ra-interval msec %d\n",
+				interval);
+		else if (interval != RTADV_MAX_RTR_ADV_INTERVAL)
+			vty_out(vty, " ipv6 nd ra-interval %d\n",
+				interval / 1000);
+	}
 
 	if (zif->rtadv.AdvIntervalOption)
-		vty_out(vty, " ipv6 nd adv-interval-option%s", VTY_NEWLINE);
+		vty_out(vty, " ipv6 nd adv-interval-option\n");
 
 	if (zif->rtadv.AdvDefaultLifetime != -1)
-		vty_out(vty, " ipv6 nd ra-lifetime %d%s",
-			zif->rtadv.AdvDefaultLifetime, VTY_NEWLINE);
+		vty_out(vty, " ipv6 nd ra-lifetime %d\n",
+			zif->rtadv.AdvDefaultLifetime);
 
 	if (zif->rtadv.HomeAgentPreference)
-		vty_out(vty, " ipv6 nd home-agent-preference %u%s",
-			zif->rtadv.HomeAgentPreference, VTY_NEWLINE);
+		vty_out(vty, " ipv6 nd home-agent-preference %u\n",
+			zif->rtadv.HomeAgentPreference);
 
 	if (zif->rtadv.HomeAgentLifetime != -1)
-		vty_out(vty, " ipv6 nd home-agent-lifetime %u%s",
-			zif->rtadv.HomeAgentLifetime, VTY_NEWLINE);
+		vty_out(vty, " ipv6 nd home-agent-lifetime %u\n",
+			zif->rtadv.HomeAgentLifetime);
 
 	if (zif->rtadv.AdvHomeAgentFlag)
-		vty_out(vty, " ipv6 nd home-agent-config-flag%s", VTY_NEWLINE);
+		vty_out(vty, " ipv6 nd home-agent-config-flag\n");
 
 	if (zif->rtadv.AdvReachableTime)
-		vty_out(vty, " ipv6 nd reachable-time %d%s",
-			zif->rtadv.AdvReachableTime, VTY_NEWLINE);
+		vty_out(vty, " ipv6 nd reachable-time %d\n",
+			zif->rtadv.AdvReachableTime);
 
 	if (zif->rtadv.AdvManagedFlag)
-		vty_out(vty, " ipv6 nd managed-config-flag%s", VTY_NEWLINE);
+		vty_out(vty, " ipv6 nd managed-config-flag\n");
 
 	if (zif->rtadv.AdvOtherConfigFlag)
-		vty_out(vty, " ipv6 nd other-config-flag%s", VTY_NEWLINE);
+		vty_out(vty, " ipv6 nd other-config-flag\n");
 
 	if (zif->rtadv.DefaultPreference != RTADV_PREF_MEDIUM)
-		vty_out(vty, " ipv6 nd router-preference %s%s",
-			rtadv_pref_strs[zif->rtadv.DefaultPreference],
-			VTY_NEWLINE);
+		vty_out(vty, " ipv6 nd router-preference %s\n",
+			rtadv_pref_strs[zif->rtadv.DefaultPreference]);
 
 	if (zif->rtadv.AdvLinkMTU)
-		vty_out(vty, " ipv6 nd mtu %d%s", zif->rtadv.AdvLinkMTU,
-			VTY_NEWLINE);
+		vty_out(vty, " ipv6 nd mtu %d\n", zif->rtadv.AdvLinkMTU);
 
 	for (ALL_LIST_ELEMENTS_RO(zif->rtadv.AdvPrefixList, node, rprefix)) {
 		vty_out(vty, " ipv6 nd prefix %s",
@@ -1557,8 +1648,9 @@ void rtadv_config_write(struct vty *vty, struct interface *ifp)
 			vty_out(vty, " no-autoconfig");
 		if (rprefix->AdvRouterAddressFlag)
 			vty_out(vty, " router-address");
-		vty_out(vty, "%s", VTY_NEWLINE);
+		vty_out(vty, "\n");
 	}
+	return 0;
 }
 
 
@@ -1568,12 +1660,10 @@ static void rtadv_event(struct zebra_ns *zns, enum rtadv_event event, int val)
 
 	switch (event) {
 	case RTADV_START:
-		if (!rtadv->ra_read)
-			rtadv->ra_read = thread_add_read(zebrad.master,
-							 rtadv_read, zns, val);
-		if (!rtadv->ra_timer)
-			rtadv->ra_timer = thread_add_event(zebrad.master,
-							   rtadv_timer, zns, 0);
+		thread_add_read(zebrad.master, rtadv_read, zns, val,
+				&rtadv->ra_read);
+		thread_add_event(zebrad.master, rtadv_timer, zns, 0,
+				 &rtadv->ra_timer);
 		break;
 	case RTADV_STOP:
 		if (rtadv->ra_timer) {
@@ -1586,19 +1676,16 @@ static void rtadv_event(struct zebra_ns *zns, enum rtadv_event event, int val)
 		}
 		break;
 	case RTADV_TIMER:
-		if (!rtadv->ra_timer)
-			rtadv->ra_timer = thread_add_timer(
-				zebrad.master, rtadv_timer, zns, val);
+		thread_add_timer(zebrad.master, rtadv_timer, zns, val,
+				 &rtadv->ra_timer);
 		break;
 	case RTADV_TIMER_MSEC:
-		if (!rtadv->ra_timer)
-			rtadv->ra_timer = thread_add_timer_msec(
-				zebrad.master, rtadv_timer, zns, val);
+		thread_add_timer_msec(zebrad.master, rtadv_timer, zns, val,
+				      &rtadv->ra_timer);
 		break;
 	case RTADV_READ:
-		if (!rtadv->ra_read)
-			rtadv->ra_read = thread_add_read(zebrad.master,
-							 rtadv_read, zns, val);
+		thread_add_read(zebrad.master, rtadv_read, zns, val,
+				&rtadv->ra_read);
 		break;
 	default:
 		break;
@@ -1608,7 +1695,7 @@ static void rtadv_event(struct zebra_ns *zns, enum rtadv_event event, int val)
 
 void rtadv_init(struct zebra_ns *zns)
 {
-	zns->rtadv.sock = rtadv_make_socket();
+	zns->rtadv.sock = rtadv_make_socket(zns->ns_id);
 }
 
 void rtadv_terminate(struct zebra_ns *zns)
@@ -1625,6 +1712,9 @@ void rtadv_terminate(struct zebra_ns *zns)
 
 void rtadv_cmd_init(void)
 {
+	hook_register(zebra_if_extra_info, nd_dump_vty);
+	hook_register(zebra_if_config_wr, rtadv_config_write);
+
 	install_element(INTERFACE_NODE, &ipv6_nd_suppress_ra_cmd);
 	install_element(INTERFACE_NODE, &no_ipv6_nd_suppress_ra_cmd);
 	install_element(INTERFACE_NODE, &ipv6_nd_ra_interval_cmd);
