@@ -16,10 +16,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -93,7 +92,6 @@ typedef struct zfpm_stats_t_ {
 
 	unsigned long updates_triggered;
 	unsigned long redundant_triggers;
-	unsigned long non_fpm_table_triggers;
 
 	unsigned long dests_del_after_update;
 
@@ -312,31 +310,6 @@ static time_t zfpm_get_elapsed_time(time_t reference)
 	return now - reference;
 }
 
-/*
- * zfpm_is_table_for_fpm
- *
- * Returns TRUE if the the given table is to be communicated to the
- * FPM.
- */
-static inline int zfpm_is_table_for_fpm(struct route_table *table)
-{
-	rib_table_info_t *info;
-
-	info = rib_table_info(table);
-
-	/*
-	 * We only send the unicast tables in the main instance to the FPM
-	 * at this point.
-	 */
-	if (zvrf_id(info->zvrf) != 0)
-		return 0;
-
-	if (info->safi != SAFI_UNICAST)
-		return 0;
-
-	return 1;
-}
-
 /*
  * zfpm_rnodes_iter_init
  */
@@ -372,10 +345,7 @@ static inline struct route_node *zfpm_rnodes_iter_next(zfpm_rnodes_iter_t *iter)
 		 */
 		route_table_iter_cleanup(&iter->iter);
 
-		while ((table = rib_tables_iter_next(&iter->tables_iter))) {
-			if (zfpm_is_table_for_fpm(table))
-				break;
-		}
+		table = rib_tables_iter_next(&iter->tables_iter);
 
 		if (!table)
 			return NULL;
@@ -466,8 +436,8 @@ static inline void zfpm_read_on(void)
 	assert(!zfpm_g->t_read);
 	assert(zfpm_g->sock >= 0);
 
-	THREAD_READ_ON(zfpm_g->master, zfpm_g->t_read, zfpm_read_cb, 0,
-		       zfpm_g->sock);
+	thread_add_read(zfpm_g->master, zfpm_read_cb, 0, zfpm_g->sock,
+			&zfpm_g->t_read);
 }
 
 /*
@@ -478,8 +448,8 @@ static inline void zfpm_write_on(void)
 	assert(!zfpm_g->t_write);
 	assert(zfpm_g->sock >= 0);
 
-	THREAD_WRITE_ON(zfpm_g->master, zfpm_g->t_write, zfpm_write_cb, 0,
-			zfpm_g->sock);
+	thread_add_write(zfpm_g->master, zfpm_write_cb, 0, zfpm_g->sock,
+			 &zfpm_g->t_write);
 }
 
 /*
@@ -510,7 +480,6 @@ static int zfpm_conn_up_thread_cb(struct thread *thread)
 	zfpm_rnodes_iter_t *iter;
 	rib_dest_t *dest;
 
-	assert(zfpm_g->t_conn_up);
 	zfpm_g->t_conn_up = NULL;
 
 	iter = &zfpm_g->t_conn_up_state.iter;
@@ -538,8 +507,9 @@ static int zfpm_conn_up_thread_cb(struct thread *thread)
 
 		zfpm_g->stats.t_conn_up_yields++;
 		zfpm_rnodes_iter_pause(iter);
-		zfpm_g->t_conn_up = thread_add_background(
-			zfpm_g->master, zfpm_conn_up_thread_cb, 0, 0);
+		zfpm_g->t_conn_up = NULL;
+		thread_add_timer_msec(zfpm_g->master, zfpm_conn_up_thread_cb,
+				      NULL, 0, &zfpm_g->t_conn_up);
 		return 0;
 	}
 
@@ -570,8 +540,9 @@ static void zfpm_connection_up(const char *detail)
 	zfpm_rnodes_iter_init(&zfpm_g->t_conn_up_state.iter);
 
 	zfpm_debug("Starting conn_up thread");
-	zfpm_g->t_conn_up = thread_add_background(zfpm_g->master,
-						  zfpm_conn_up_thread_cb, 0, 0);
+	zfpm_g->t_conn_up = NULL;
+	thread_add_timer_msec(zfpm_g->master, zfpm_conn_up_thread_cb, NULL, 0,
+			      &zfpm_g->t_conn_up);
 	zfpm_g->stats.t_conn_up_starts++;
 }
 
@@ -622,7 +593,6 @@ static int zfpm_conn_down_thread_cb(struct thread *thread)
 
 	assert(zfpm_g->state == ZFPM_STATE_IDLE);
 
-	assert(zfpm_g->t_conn_down);
 	zfpm_g->t_conn_down = NULL;
 
 	iter = &zfpm_g->t_conn_down_state.iter;
@@ -655,8 +625,9 @@ static int zfpm_conn_down_thread_cb(struct thread *thread)
 
 		zfpm_g->stats.t_conn_down_yields++;
 		zfpm_rnodes_iter_pause(iter);
-		zfpm_g->t_conn_down = thread_add_background(
-			zfpm_g->master, zfpm_conn_down_thread_cb, 0, 0);
+		zfpm_g->t_conn_down = NULL;
+		thread_add_timer_msec(zfpm_g->master, zfpm_conn_down_thread_cb,
+				      NULL, 0, &zfpm_g->t_conn_down);
 		return 0;
 	}
 
@@ -701,8 +672,9 @@ static void zfpm_connection_down(const char *detail)
 	assert(!zfpm_g->t_conn_down);
 	zfpm_debug("Starting conn_down thread");
 	zfpm_rnodes_iter_init(&zfpm_g->t_conn_down_state.iter);
-	zfpm_g->t_conn_down = thread_add_background(
-		zfpm_g->master, zfpm_conn_down_thread_cb, 0, 0);
+	zfpm_g->t_conn_down = NULL;
+	thread_add_timer_msec(zfpm_g->master, zfpm_conn_down_thread_cb, NULL, 0,
+			      &zfpm_g->t_conn_down);
 	zfpm_g->stats.t_conn_down_starts++;
 
 	zfpm_set_state(ZFPM_STATE_IDLE, detail);
@@ -719,7 +691,6 @@ static int zfpm_read_cb(struct thread *thread)
 	fpm_msg_hdr_t *hdr;
 
 	zfpm_g->stats.read_cb_calls++;
-	assert(zfpm_g->t_read);
 	zfpm_g->t_read = NULL;
 
 	/*
@@ -742,7 +713,14 @@ static int zfpm_read_cb(struct thread *thread)
 		nbyte = stream_read_try(ibuf, zfpm_g->sock,
 					FPM_MSG_HDR_LEN - already);
 		if (nbyte == 0 || nbyte == -1) {
-			zfpm_connection_down("closed socket in read");
+			if (nbyte == -1) {
+				char buffer[1024];
+
+				sprintf(buffer, "closed socket in read(%d): %s",
+					errno, safe_strerror(errno));
+				zfpm_connection_down(buffer);
+			} else
+				zfpm_connection_down("closed socket in read");
 			return 0;
 		}
 
@@ -772,7 +750,14 @@ static int zfpm_read_cb(struct thread *thread)
 		nbyte = stream_read_try(ibuf, zfpm_g->sock, msg_len - already);
 
 		if (nbyte == 0 || nbyte == -1) {
-			zfpm_connection_down("failed to read message");
+			if (nbyte == -1) {
+				char buffer[1024];
+
+				sprintf(buffer, "failed to read message(%d) %s",
+					errno, safe_strerror(errno));
+				zfpm_connection_down(buffer);
+			} else
+				zfpm_connection_down("failed to read message");
 			return 0;
 		}
 
@@ -824,7 +809,7 @@ static int zfpm_writes_pending(void)
  * Returns the number of bytes written to the buffer. 0 or a negative
  * value indicates an error.
  */
-static inline int zfpm_encode_route(rib_dest_t *dest, struct rib *rib,
+static inline int zfpm_encode_route(rib_dest_t *dest, struct route_entry *re,
 				    char *in_buf, size_t in_buf_len,
 				    fpm_msg_type_e *msg_type)
 {
@@ -840,7 +825,7 @@ static inline int zfpm_encode_route(rib_dest_t *dest, struct rib *rib,
 
 	case ZFPM_MSG_FORMAT_PROTOBUF:
 #ifdef HAVE_PROTOBUF
-		len = zfpm_protobuf_encode_route(dest, rib, (uint8_t *)in_buf,
+		len = zfpm_protobuf_encode_route(dest, re, (uint8_t *)in_buf,
 						 in_buf_len);
 		*msg_type = FPM_MSG_TYPE_PROTOBUF;
 #endif
@@ -849,8 +834,8 @@ static inline int zfpm_encode_route(rib_dest_t *dest, struct rib *rib,
 	case ZFPM_MSG_FORMAT_NETLINK:
 #ifdef HAVE_NETLINK
 		*msg_type = FPM_MSG_TYPE_NETLINK;
-		cmd = rib ? RTM_NEWROUTE : RTM_DELROUTE;
-		len = zfpm_netlink_encode_route(cmd, dest, rib, in_buf,
+		cmd = re ? RTM_NEWROUTE : RTM_DELROUTE;
+		len = zfpm_netlink_encode_route(cmd, dest, re, in_buf,
 						in_buf_len);
 		assert(fpm_msg_align(len) == len);
 		*msg_type = FPM_MSG_TYPE_NETLINK;
@@ -867,24 +852,11 @@ static inline int zfpm_encode_route(rib_dest_t *dest, struct rib *rib,
 /*
  * zfpm_route_for_update
  *
- * Returns the rib that is to be sent to the FPM for a given dest.
+ * Returns the re that is to be sent to the FPM for a given dest.
  */
-struct rib *zfpm_route_for_update(rib_dest_t *dest)
+struct route_entry *zfpm_route_for_update(rib_dest_t *dest)
 {
-	struct rib *rib;
-
-	RIB_DEST_FOREACH_ROUTE(dest, rib)
-	{
-		if (!CHECK_FLAG(rib->status, RIB_ENTRY_SELECTED_FIB))
-			continue;
-
-		return rib;
-	}
-
-	/*
-	 * We have no route for this destination.
-	 */
-	return NULL;
+	return dest->selected_fib;
 }
 
 /*
@@ -901,7 +873,7 @@ static void zfpm_build_updates(void)
 	size_t msg_len;
 	size_t data_len;
 	fpm_msg_hdr_t *hdr;
-	struct rib *rib;
+	struct route_entry *re;
 	int is_add, write_msg;
 	fpm_msg_type_e msg_type;
 
@@ -931,8 +903,8 @@ static void zfpm_build_updates(void)
 
 		data = fpm_msg_data(hdr);
 
-		rib = zfpm_route_for_update(dest);
-		is_add = rib ? 1 : 0;
+		re = zfpm_route_for_update(dest);
+		is_add = re ? 1 : 0;
 
 		write_msg = 1;
 
@@ -947,7 +919,7 @@ static void zfpm_build_updates(void)
 		}
 
 		if (write_msg) {
-			data_len = zfpm_encode_route(dest, rib, (char *)data,
+			data_len = zfpm_encode_route(dest, re, (char *)data,
 						     buf_end - data, &msg_type);
 
 			assert(data_len);
@@ -994,7 +966,6 @@ static int zfpm_write_cb(struct thread *thread)
 	int num_writes;
 
 	zfpm_g->stats.write_cb_calls++;
-	assert(zfpm_g->t_write);
 	zfpm_g->t_write = NULL;
 
 	/*
@@ -1027,7 +998,7 @@ static int zfpm_write_cb(struct thread *thread)
 			break;
 
 		bytes_written =
-			write(zfpm_g->sock, STREAM_PNT(s), bytes_to_write);
+			write(zfpm_g->sock, stream_pnt(s), bytes_to_write);
 		zfpm_g->stats.write_calls++;
 		num_writes++;
 
@@ -1079,7 +1050,6 @@ static int zfpm_connect_cb(struct thread *t)
 	int sock, ret;
 	struct sockaddr_in serv;
 
-	assert(zfpm_g->t_connect);
 	zfpm_g->t_connect = NULL;
 	assert(zfpm_g->state == ZFPM_STATE_ACTIVE);
 
@@ -1231,8 +1201,8 @@ static void zfpm_start_connect_timer(const char *reason)
 	delay_secs = zfpm_calc_connect_delay();
 	zfpm_debug("scheduling connect in %ld seconds", delay_secs);
 
-	THREAD_TIMER_ON(zfpm_g->master, zfpm_g->t_connect, zfpm_connect_cb, 0,
-			delay_secs);
+	thread_add_timer(zfpm_g->master, zfpm_connect_cb, 0, delay_secs,
+			 &zfpm_g->t_connect);
 	zfpm_set_state(ZFPM_STATE_ACTIVE, reason);
 }
 
@@ -1281,15 +1251,6 @@ static int zfpm_trigger_update(struct route_node *rn, const char *reason)
 
 	dest = rib_dest_from_rnode(rn);
 
-	/*
-	 * Ignore the trigger if the dest is not in a table that we would
-	 * send to the FPM.
-	 */
-	if (!zfpm_is_table_for_fpm(rib_dest_table(dest))) {
-		zfpm_g->stats.non_fpm_table_triggers++;
-		return 0;
-	}
-
 	if (CHECK_FLAG(dest->flags, RIB_DEST_UPDATE_FPM)) {
 		zfpm_g->stats.redundant_triggers++;
 		return 0;
@@ -1319,7 +1280,6 @@ static int zfpm_trigger_update(struct route_node *rn, const char *reason)
  */
 static int zfpm_stats_timer_cb(struct thread *t)
 {
-	assert(zfpm_g->t_stats);
 	zfpm_g->t_stats = NULL;
 
 	/*
@@ -1363,8 +1323,8 @@ void zfpm_start_stats_timer(void)
 {
 	assert(!zfpm_g->t_stats);
 
-	THREAD_TIMER_ON(zfpm_g->master, zfpm_g->t_stats, zfpm_stats_timer_cb, 0,
-			ZFPM_STATS_IVL_SECS);
+	thread_add_timer(zfpm_g->master, zfpm_stats_timer_cb, 0,
+			 ZFPM_STATS_IVL_SECS, &zfpm_g->t_stats);
 }
 
 /*
@@ -1372,9 +1332,8 @@ void zfpm_start_stats_timer(void)
  */
 #define ZFPM_SHOW_STAT(counter)                                                \
 	do {                                                                   \
-		vty_out(vty, "%-40s %10lu %16lu%s", #counter,                  \
-			total_stats.counter, zfpm_g->last_ivl_stats.counter,   \
-			VTY_NEWLINE);                                          \
+		vty_out(vty, "%-40s %10lu %16lu\n", #counter,                  \
+			total_stats.counter, zfpm_g->last_ivl_stats.counter);  \
 	} while (0)
 
 /*
@@ -1385,9 +1344,8 @@ static void zfpm_show_stats(struct vty *vty)
 	zfpm_stats_t total_stats;
 	time_t elapsed;
 
-	vty_out(vty, "%s%-40s %10s     Last %2d secs%s%s", VTY_NEWLINE,
-		"Counter", "Total", ZFPM_STATS_IVL_SECS, VTY_NEWLINE,
-		VTY_NEWLINE);
+	vty_out(vty, "\n%-40s %10s     Last %2d secs\n\n", "Counter", "Total",
+		ZFPM_STATS_IVL_SECS);
 
 	/*
 	 * Compute the total stats up to this instant.
@@ -1407,7 +1365,6 @@ static void zfpm_show_stats(struct vty *vty)
 	ZFPM_SHOW_STAT(route_adds);
 	ZFPM_SHOW_STAT(route_dels);
 	ZFPM_SHOW_STAT(updates_triggered);
-	ZFPM_SHOW_STAT(non_fpm_table_triggers);
 	ZFPM_SHOW_STAT(redundant_triggers);
 	ZFPM_SHOW_STAT(dests_del_after_update);
 	ZFPM_SHOW_STAT(t_conn_down_starts);
@@ -1425,8 +1382,8 @@ static void zfpm_show_stats(struct vty *vty)
 
 	elapsed = zfpm_get_elapsed_time(zfpm_g->last_stats_clear_time);
 
-	vty_out(vty, "%sStats were cleared %lu seconds ago%s", VTY_NEWLINE,
-		(unsigned long)elapsed, VTY_NEWLINE);
+	vty_out(vty, "\nStats were cleared %lu seconds ago\n",
+		(unsigned long)elapsed);
 }
 
 /*
@@ -1435,7 +1392,7 @@ static void zfpm_show_stats(struct vty *vty)
 static void zfpm_clear_stats(struct vty *vty)
 {
 	if (!zfpm_is_enabled()) {
-		vty_out(vty, "The FPM module is not enabled...%s", VTY_NEWLINE);
+		vty_out(vty, "The FPM module is not enabled...\n");
 		return;
 	}
 
@@ -1448,7 +1405,7 @@ static void zfpm_clear_stats(struct vty *vty)
 
 	zfpm_g->last_stats_clear_time = monotime(NULL);
 
-	vty_out(vty, "Cleared FPM stats%s", VTY_NEWLINE);
+	vty_out(vty, "Cleared FPM stats\n");
 }
 
 /*
@@ -1458,7 +1415,7 @@ DEFUN (show_zebra_fpm_stats,
        show_zebra_fpm_stats_cmd,
        "show zebra fpm stats",
        SHOW_STR
-       "Zebra information\n"
+       ZEBRA_STR
        "Forwarding Path Manager information\n"
        "Statistics\n")
 {
@@ -1473,7 +1430,7 @@ DEFUN (clear_zebra_fpm_stats,
        clear_zebra_fpm_stats_cmd,
        "clear zebra fpm stats",
        CLEAR_STR
-       "Zebra information\n"
+       ZEBRA_STR
        "Clear Forwarding Path Manager information\n"
        "Statistics\n")
 {
@@ -1594,10 +1551,11 @@ static int fpm_remote_srv_write(struct vty *vty)
 
 	in.s_addr = zfpm_g->fpm_server;
 
-	if (zfpm_g->fpm_server != FPM_DEFAULT_IP
-	    || zfpm_g->fpm_port != FPM_DEFAULT_PORT)
-		vty_out(vty, "fpm connection ip %s port %d%s", inet_ntoa(in),
-			zfpm_g->fpm_port, VTY_NEWLINE);
+	if ((zfpm_g->fpm_server != FPM_DEFAULT_IP
+	     && zfpm_g->fpm_server != INADDR_ANY)
+	    || (zfpm_g->fpm_port != FPM_DEFAULT_PORT && zfpm_g->fpm_port != 0))
+		vty_out(vty, "fpm connection ip %s port %d\n", inet_ntoa(in),
+			zfpm_g->fpm_port);
 
 	return 0;
 }
