@@ -13,29 +13,32 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 #include "zebra/rt.h"
 #include "zebra/zebra_mpls.h"
 
-int kernel_add_lsp(zebra_lsp_t *lsp)
+#if !defined(HAVE_NETLINK) && !defined(OPEN_BSD)
+
+void kernel_add_lsp(zebra_lsp_t *lsp)
 {
-	return 0;
+	return;
 }
-int kernel_upd_lsp(zebra_lsp_t *lsp)
+void kernel_upd_lsp(zebra_lsp_t *lsp)
 {
-	return 0;
+	return;
 }
-int kernel_del_lsp(zebra_lsp_t *lsp)
+void kernel_del_lsp(zebra_lsp_t *lsp)
 {
-	return 0;
+	return;
 }
 int mpls_kernel_init(void)
 {
 	return -1;
 };
+
+#endif /* !defined(HAVE_NETLINK) && !defined(OPEN_BSD) */
