@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -92,6 +91,7 @@ static int zebra_ptm_handle_msg_cb(void *arg, void *in_ctxt);
 void zebra_bfd_peer_replay_req(void);
 void zebra_ptm_send_status_req(void);
 void zebra_ptm_reset_status(int ptm_disable);
+static int zebra_ptm_bfd_client_deregister(struct zserv *client);
 
 const char ZEBRA_PTM_SOCK_NAME[] = "\0/var/run/ptmd.socket";
 
@@ -125,17 +125,12 @@ void zebra_ptm_init(void)
 	ptm_cb.reconnect_time = ZEBRA_PTM_RECONNECT_TIME_INITIAL;
 
 	ptm_cb.ptm_sock = -1;
+
+	hook_register(zapi_client_close, zebra_ptm_bfd_client_deregister);
 }
 
 void zebra_ptm_finish(void)
 {
-	int proto;
-
-	for (proto = 0; proto < ZEBRA_ROUTE_MAX; proto++)
-		if (CHECK_FLAG(ptm_cb.client_flags[proto],
-			       ZEBRA_PTM_BFD_CLIENT_FLAG_REG))
-			zebra_ptm_bfd_client_deregister(proto);
-
 	buffer_flush_all(ptm_cb.wb, ptm_cb.ptm_sock);
 
 	free(ptm_hdl);
@@ -157,7 +152,7 @@ void zebra_ptm_finish(void)
 	if (ptm_cb.wb)
 		buffer_free(ptm_cb.wb);
 
-	if (ptm_cb.ptm_sock != -1)
+	if (ptm_cb.ptm_sock >= 0)
 		close(ptm_cb.ptm_sock);
 }
 
@@ -177,14 +172,14 @@ static int zebra_ptm_flush_messages(struct thread *thread)
 		close(ptm_cb.ptm_sock);
 		ptm_cb.ptm_sock = -1;
 		zebra_ptm_reset_status(0);
-		ptm_cb.t_timer =
-			thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
-					 ptm_cb.reconnect_time);
+		ptm_cb.t_timer = NULL;
+		thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
+				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
 		return (-1);
 	case BUFFER_PENDING:
-		ptm_cb.t_write = thread_add_write(zebrad.master,
-						  zebra_ptm_flush_messages,
-						  NULL, ptm_cb.ptm_sock);
+		ptm_cb.t_write = NULL;
+		thread_add_write(zebrad.master, zebra_ptm_flush_messages, NULL,
+				 ptm_cb.ptm_sock, &ptm_cb.t_write);
 		break;
 	case BUFFER_EMPTY:
 		break;
@@ -203,17 +198,16 @@ static int zebra_ptm_send_message(char *data, int size)
 		close(ptm_cb.ptm_sock);
 		ptm_cb.ptm_sock = -1;
 		zebra_ptm_reset_status(0);
-		ptm_cb.t_timer =
-			thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
-					 ptm_cb.reconnect_time);
+		ptm_cb.t_timer = NULL;
+		thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
+				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
 		return -1;
 	case BUFFER_EMPTY:
 		THREAD_OFF(ptm_cb.t_write);
 		break;
 	case BUFFER_PENDING:
-		THREAD_WRITE_ON(zebrad.master, ptm_cb.t_write,
-				zebra_ptm_flush_messages, NULL,
-				ptm_cb.ptm_sock);
+		thread_add_write(zebrad.master, zebra_ptm_flush_messages, NULL,
+				 ptm_cb.ptm_sock, &ptm_cb.t_write);
 		break;
 	}
 
@@ -231,9 +225,9 @@ int zebra_ptm_connect(struct thread *t)
 
 	if (ptm_cb.ptm_sock != -1) {
 		if (init) {
-			ptm_cb.t_read = thread_add_read(zebrad.master,
-							zebra_ptm_sock_read,
-							NULL, ptm_cb.ptm_sock);
+			ptm_cb.t_read = NULL;
+			thread_add_read(zebrad.master, zebra_ptm_sock_read,
+					NULL, ptm_cb.ptm_sock, &ptm_cb.t_read);
 			zebra_bfd_peer_replay_req();
 		}
 		zebra_ptm_send_status_req();
@@ -243,9 +237,9 @@ int zebra_ptm_connect(struct thread *t)
 		if (ptm_cb.reconnect_time > ZEBRA_PTM_RECONNECT_TIME_MAX)
 			ptm_cb.reconnect_time = ZEBRA_PTM_RECONNECT_TIME_MAX;
 
-		ptm_cb.t_timer =
-			thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
-					 ptm_cb.reconnect_time);
+		ptm_cb.t_timer = NULL;
+		thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
+				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
 	} else if (ptm_cb.reconnect_time >= ZEBRA_PTM_RECONNECT_TIME_MAX) {
 		ptm_cb.reconnect_time = ZEBRA_PTM_RECONNECT_TIME_INITIAL;
 	}
@@ -259,23 +253,24 @@ DEFUN (zebra_ptm_enable,
        "Enable neighbor check with specified topology\n")
 {
 	struct vrf *vrf;
-	struct listnode *i;
 	struct interface *ifp;
 	struct zebra_if *if_data;
 
 	ptm_cb.ptm_enable = ZEBRA_IF_PTM_ENABLE_ON;
 
-	RB_FOREACH(vrf, vrf_name_head, &vrfs_by_name)
-	for (ALL_LIST_ELEMENTS_RO(vrf->iflist, i, ifp))
-		if (!ifp->ptm_enable) {
-			if_data = (struct zebra_if *)ifp->info;
-			if (if_data && (if_data->ptm_enable
+	RB_FOREACH (vrf, vrf_name_head, &vrfs_by_name)
+		FOR_ALL_INTERFACES (vrf, ifp)
+			if (!ifp->ptm_enable) {
+				if_data = (struct zebra_if *)ifp->info;
+				if (if_data
+				    && (if_data->ptm_enable
 					== ZEBRA_IF_PTM_ENABLE_UNSPEC)) {
-				ifp->ptm_enable = ZEBRA_IF_PTM_ENABLE_ON;
+					ifp->ptm_enable =
+						ZEBRA_IF_PTM_ENABLE_ON;
+				}
+				/* Assign a default unknown status */
+				ifp->ptm_status = ZEBRA_PTM_STATUS_UNKNOWN;
 			}
-			/* Assign a default unknown status */
-			ifp->ptm_status = ZEBRA_PTM_STATUS_UNKNOWN;
-		}
 
 	zebra_ptm_connect(NULL);
 
@@ -361,7 +356,7 @@ DEFUN (no_zebra_ptm_enable_if,
 void zebra_ptm_write(struct vty *vty)
 {
 	if (ptm_cb.ptm_enable)
-		vty_out(vty, "ptm-enable%s", VTY_NEWLINE);
+		vty_out(vty, "ptm-enable\n");
 
 	return;
 }
@@ -433,7 +428,7 @@ static void if_bfd_session_update(struct interface *ifp, struct prefix *dp,
 		} else {
 			zlog_debug(
 				"MESSAGE: ZEBRA_INTERFACE_BFD_DEST_UPDATE %s/%d "
-				"with src %s/%d and vrf %d %s event",
+				"with src %s/%d and vrf %u %s event",
 				inet_ntop(dp->family, &dp->u.prefix, buf[0],
 					  INET6_ADDRSTRLEN),
 				dp->prefixlen,
@@ -646,53 +641,54 @@ int zebra_ptm_sock_read(struct thread *thread)
 			close(ptm_cb.ptm_sock);
 			ptm_cb.ptm_sock = -1;
 			zebra_ptm_reset_status(0);
-			ptm_cb.t_timer = thread_add_timer(
-				zebrad.master, zebra_ptm_connect, NULL,
-				ptm_cb.reconnect_time);
+			ptm_cb.t_timer = NULL;
+			thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
+					 ptm_cb.reconnect_time,
+					 &ptm_cb.t_timer);
 			return (-1);
 		}
 	}
 
-	ptm_cb.t_read = thread_add_read(zebrad.master, zebra_ptm_sock_read,
-					NULL, ptm_cb.ptm_sock);
+	ptm_cb.t_read = NULL;
+	thread_add_read(zebrad.master, zebra_ptm_sock_read, NULL,
+			ptm_cb.ptm_sock, &ptm_cb.t_read);
 
 	return 0;
 }
 
 /* BFD peer/dst register/update */
-int zebra_ptm_bfd_dst_register(struct zserv *client, int sock, u_short length,
-			       int command, struct zebra_vrf *zvrf)
+void zebra_ptm_bfd_dst_register(ZAPI_HANDLER_ARGS)
 {
 	struct stream *s;
 	struct prefix src_p;
 	struct prefix dst_p;
-	u_char multi_hop;
-	u_char multi_hop_cnt;
-	u_char detect_mul;
+	uint8_t multi_hop;
+	uint8_t multi_hop_cnt;
+	uint8_t detect_mul;
 	unsigned int min_rx_timer;
 	unsigned int min_tx_timer;
 	char if_name[INTERFACE_NAMSIZ];
-	u_char len;
+	uint8_t len;
 	void *out_ctxt;
 	char buf[INET6_ADDRSTRLEN];
 	char tmp_buf[64];
 	int data_len = ZEBRA_PTM_SEND_MAX_SOCKBUF;
 	unsigned int pid;
 
-	if (command == ZEBRA_BFD_DEST_UPDATE)
+	if (hdr->command == ZEBRA_BFD_DEST_UPDATE)
 		client->bfd_peer_upd8_cnt++;
 	else
 		client->bfd_peer_add_cnt++;
 
 	if (IS_ZEBRA_DEBUG_EVENT)
 		zlog_debug("bfd_dst_register msg from client %s: length=%d",
-			   zebra_route_string(client->proto), length);
+			   zebra_route_string(client->proto), hdr->length);
 
 	if (ptm_cb.ptm_sock == -1) {
-		ptm_cb.t_timer =
-			thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
-					 ptm_cb.reconnect_time);
-		return -1;
+		ptm_cb.t_timer = NULL;
+		thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
+				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		return;
 	}
 
 	ptm_lib_init_msg(ptm_hdl, 0, PTMLIB_MSG_TYPE_CMD, NULL, &out_ctxt);
@@ -702,21 +698,21 @@ int zebra_ptm_bfd_dst_register(struct zserv *client, int sock, u_short length,
 	ptm_lib_append_msg(ptm_hdl, out_ctxt, ZEBRA_PTM_BFD_CLIENT_FIELD,
 			   tmp_buf);
 
-	s = client->ibuf;
+	s = msg;
 
-	pid = stream_getl(s);
+	STREAM_GETL(s, pid);
 	sprintf(tmp_buf, "%d", pid);
 	ptm_lib_append_msg(ptm_hdl, out_ctxt, ZEBRA_PTM_BFD_SEQID_FIELD,
 			   tmp_buf);
 
-	dst_p.family = stream_getw(s);
+	STREAM_GETW(s, dst_p.family);
 
 	if (dst_p.family == AF_INET)
 		dst_p.prefixlen = IPV4_MAX_BYTELEN;
 	else
 		dst_p.prefixlen = IPV6_MAX_BYTELEN;
 
-	stream_get(&dst_p.u.prefix, s, dst_p.prefixlen);
+	STREAM_GET(&dst_p.u.prefix, s, dst_p.prefixlen);
 	if (dst_p.family == AF_INET) {
 		inet_ntop(AF_INET, &dst_p.u.prefix4, buf, sizeof(buf));
 		ptm_lib_append_msg(ptm_hdl, out_ctxt,
@@ -727,32 +723,32 @@ int zebra_ptm_bfd_dst_register(struct zserv *client, int sock, u_short length,
 				   ZEBRA_PTM_BFD_DST_IP_FIELD, buf);
 	}
 
-	min_rx_timer = stream_getl(s);
+	STREAM_GETL(s, min_rx_timer);
 	sprintf(tmp_buf, "%d", min_rx_timer);
 	ptm_lib_append_msg(ptm_hdl, out_ctxt, ZEBRA_PTM_BFD_MIN_RX_FIELD,
 			   tmp_buf);
-	min_tx_timer = stream_getl(s);
+	STREAM_GETL(s, min_tx_timer);
 	sprintf(tmp_buf, "%d", min_tx_timer);
 	ptm_lib_append_msg(ptm_hdl, out_ctxt, ZEBRA_PTM_BFD_MIN_TX_FIELD,
 			   tmp_buf);
-	detect_mul = stream_getc(s);
+	STREAM_GETC(s, detect_mul);
 	sprintf(tmp_buf, "%d", detect_mul);
 	ptm_lib_append_msg(ptm_hdl, out_ctxt, ZEBRA_PTM_BFD_DETECT_MULT_FIELD,
 			   tmp_buf);
 
-	multi_hop = stream_getc(s);
+	STREAM_GETC(s, multi_hop);
 	if (multi_hop) {
 		sprintf(tmp_buf, "%d", 1);
 		ptm_lib_append_msg(ptm_hdl, out_ctxt,
 				   ZEBRA_PTM_BFD_MULTI_HOP_FIELD, tmp_buf);
-		src_p.family = stream_getw(s);
+		STREAM_GETW(s, src_p.family);
 
 		if (src_p.family == AF_INET)
 			src_p.prefixlen = IPV4_MAX_BYTELEN;
 		else
 			src_p.prefixlen = IPV6_MAX_BYTELEN;
 
-		stream_get(&src_p.u.prefix, s, src_p.prefixlen);
+		STREAM_GET(&src_p.u.prefix, s, src_p.prefixlen);
 		if (src_p.family == AF_INET) {
 			inet_ntop(AF_INET, &src_p.u.prefix4, buf, sizeof(buf));
 			ptm_lib_append_msg(ptm_hdl, out_ctxt,
@@ -763,7 +759,7 @@ int zebra_ptm_bfd_dst_register(struct zserv *client, int sock, u_short length,
 					   ZEBRA_PTM_BFD_SRC_IP_FIELD, buf);
 		}
 
-		multi_hop_cnt = stream_getc(s);
+		STREAM_GETC(s, multi_hop_cnt);
 		sprintf(tmp_buf, "%d", multi_hop_cnt);
 		ptm_lib_append_msg(ptm_hdl, out_ctxt,
 				   ZEBRA_PTM_BFD_MAX_HOP_CNT_FIELD, tmp_buf);
@@ -774,14 +770,14 @@ int zebra_ptm_bfd_dst_register(struct zserv *client, int sock, u_short length,
 					   zvrf_name(zvrf));
 	} else {
 		if (dst_p.family == AF_INET6) {
-			src_p.family = stream_getw(s);
+			STREAM_GETW(s, src_p.family);
 
 			if (src_p.family == AF_INET)
 				src_p.prefixlen = IPV4_MAX_BYTELEN;
 			else
 				src_p.prefixlen = IPV6_MAX_BYTELEN;
 
-			stream_get(&src_p.u.prefix, s, src_p.prefixlen);
+			STREAM_GET(&src_p.u.prefix, s, src_p.prefixlen);
 			if (src_p.family == AF_INET) {
 				inet_ntop(AF_INET, &src_p.u.prefix4, buf,
 					  sizeof(buf));
@@ -796,8 +792,8 @@ int zebra_ptm_bfd_dst_register(struct zserv *client, int sock, u_short length,
 						   buf);
 			}
 		}
-		len = stream_getc(s);
-		stream_get(if_name, s, len);
+		STREAM_GETC(s, len);
+		STREAM_GET(if_name, s, len);
 		if_name[len] = '\0';
 
 		ptm_lib_append_msg(ptm_hdl, out_ctxt,
@@ -814,19 +810,22 @@ int zebra_ptm_bfd_dst_register(struct zserv *client, int sock, u_short length,
 		zlog_debug("%s: Sent message (%d) %s", __func__, data_len,
 			   ptm_cb.out_data);
 	zebra_ptm_send_message(ptm_cb.out_data, data_len);
-	return 0;
+
+	return;
+
+stream_failure:
+	ptm_lib_cleanup_msg(ptm_hdl, out_ctxt);
 }
 
 /* BFD peer/dst deregister */
-int zebra_ptm_bfd_dst_deregister(struct zserv *client, int sock, u_short length,
-				 struct zebra_vrf *zvrf)
+void zebra_ptm_bfd_dst_deregister(ZAPI_HANDLER_ARGS)
 {
 	struct stream *s;
 	struct prefix src_p;
 	struct prefix dst_p;
-	u_char multi_hop;
+	uint8_t multi_hop;
 	char if_name[INTERFACE_NAMSIZ];
-	u_char len;
+	uint8_t len;
 	char buf[INET6_ADDRSTRLEN];
 	char tmp_buf[64];
 	int data_len = ZEBRA_PTM_SEND_MAX_SOCKBUF;
@@ -837,13 +836,13 @@ int zebra_ptm_bfd_dst_deregister(struct zserv *client, int sock, u_short length,
 
 	if (IS_ZEBRA_DEBUG_EVENT)
 		zlog_debug("bfd_dst_deregister msg from client %s: length=%d",
-			   zebra_route_string(client->proto), length);
+			   zebra_route_string(client->proto), hdr->length);
 
 	if (ptm_cb.ptm_sock == -1) {
-		ptm_cb.t_timer =
-			thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
-					 ptm_cb.reconnect_time);
-		return -1;
+		ptm_cb.t_timer = NULL;
+		thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
+				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		return;
 	}
 
 	ptm_lib_init_msg(ptm_hdl, 0, PTMLIB_MSG_TYPE_CMD, NULL, &out_ctxt);
@@ -855,21 +854,21 @@ int zebra_ptm_bfd_dst_deregister(struct zserv *client, int sock, u_short length,
 	ptm_lib_append_msg(ptm_hdl, out_ctxt, ZEBRA_PTM_BFD_CLIENT_FIELD,
 			   tmp_buf);
 
-	s = client->ibuf;
+	s = msg;
 
-	pid = stream_getl(s);
+	STREAM_GETL(s, pid);
 	sprintf(tmp_buf, "%d", pid);
 	ptm_lib_append_msg(ptm_hdl, out_ctxt, ZEBRA_PTM_BFD_SEQID_FIELD,
 			   tmp_buf);
 
-	dst_p.family = stream_getw(s);
+	STREAM_GETW(s, dst_p.family);
 
 	if (dst_p.family == AF_INET)
 		dst_p.prefixlen = IPV4_MAX_BYTELEN;
 	else
 		dst_p.prefixlen = IPV6_MAX_BYTELEN;
 
-	stream_get(&dst_p.u.prefix, s, dst_p.prefixlen);
+	STREAM_GET(&dst_p.u.prefix, s, dst_p.prefixlen);
 	if (dst_p.family == AF_INET)
 		inet_ntop(AF_INET, &dst_p.u.prefix4, buf, sizeof(buf));
 	else
@@ -877,20 +876,20 @@ int zebra_ptm_bfd_dst_deregister(struct zserv *client, int sock, u_short length,
 	ptm_lib_append_msg(ptm_hdl, out_ctxt, ZEBRA_PTM_BFD_DST_IP_FIELD, buf);
 
 
-	multi_hop = stream_getc(s);
+	STREAM_GETC(s, multi_hop);
 	if (multi_hop) {
 		sprintf(tmp_buf, "%d", 1);
 		ptm_lib_append_msg(ptm_hdl, out_ctxt,
 				   ZEBRA_PTM_BFD_MULTI_HOP_FIELD, tmp_buf);
 
-		src_p.family = stream_getw(s);
+		STREAM_GETW(s, src_p.family);
 
 		if (src_p.family == AF_INET)
 			src_p.prefixlen = IPV4_MAX_BYTELEN;
 		else
 			src_p.prefixlen = IPV6_MAX_BYTELEN;
 
-		stream_get(&src_p.u.prefix, s, src_p.prefixlen);
+		STREAM_GET(&src_p.u.prefix, s, src_p.prefixlen);
 		if (src_p.family == AF_INET)
 			inet_ntop(AF_INET, &src_p.u.prefix4, buf, sizeof(buf));
 		else
@@ -904,14 +903,14 @@ int zebra_ptm_bfd_dst_deregister(struct zserv *client, int sock, u_short length,
 					   zvrf_name(zvrf));
 	} else {
 		if (dst_p.family == AF_INET6) {
-			src_p.family = stream_getw(s);
+			STREAM_GETW(s, src_p.family);
 
 			if (src_p.family == AF_INET)
 				src_p.prefixlen = IPV4_MAX_BYTELEN;
 			else
 				src_p.prefixlen = IPV6_MAX_BYTELEN;
 
-			stream_get(&src_p.u.prefix, s, src_p.prefixlen);
+			STREAM_GET(&src_p.u.prefix, s, src_p.prefixlen);
 			if (src_p.family == AF_INET) {
 				inet_ntop(AF_INET, &src_p.u.prefix4, buf,
 					  sizeof(buf));
@@ -927,8 +926,8 @@ int zebra_ptm_bfd_dst_deregister(struct zserv *client, int sock, u_short length,
 			}
 		}
 
-		len = stream_getc(s);
-		stream_get(if_name, s, len);
+		STREAM_GETC(s, len);
+		STREAM_GET(if_name, s, len);
 		if_name[len] = '\0';
 
 		ptm_lib_append_msg(ptm_hdl, out_ctxt,
@@ -941,16 +940,19 @@ int zebra_ptm_bfd_dst_deregister(struct zserv *client, int sock, u_short length,
 			   ptm_cb.out_data);
 
 	zebra_ptm_send_message(ptm_cb.out_data, data_len);
-	return 0;
+
+	return;
+
+stream_failure:
+	ptm_lib_cleanup_msg(ptm_hdl, out_ctxt);
 }
 
 /* BFD client register */
-int zebra_ptm_bfd_client_register(struct zserv *client, int sock,
-				  u_short length)
+void zebra_ptm_bfd_client_register(ZAPI_HANDLER_ARGS)
 {
 	struct stream *s;
 	unsigned int pid;
-	void *out_ctxt;
+	void *out_ctxt = NULL;
 	char tmp_buf[64];
 	int data_len = ZEBRA_PTM_SEND_MAX_SOCKBUF;
 
@@ -958,13 +960,16 @@ int zebra_ptm_bfd_client_register(struct zserv *client, int sock,
 
 	if (IS_ZEBRA_DEBUG_EVENT)
 		zlog_debug("bfd_client_register msg from client %s: length=%d",
-			   zebra_route_string(client->proto), length);
+			   zebra_route_string(client->proto), hdr->length);
+
+	s = msg;
+	STREAM_GETL(s, pid);
 
 	if (ptm_cb.ptm_sock == -1) {
-		ptm_cb.t_timer =
-			thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
-					 ptm_cb.reconnect_time);
-		return -1;
+		ptm_cb.t_timer = NULL;
+		thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
+				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		return;
 	}
 
 	ptm_lib_init_msg(ptm_hdl, 0, PTMLIB_MSG_TYPE_CMD, NULL, &out_ctxt);
@@ -976,9 +981,6 @@ int zebra_ptm_bfd_client_register(struct zserv *client, int sock,
 	ptm_lib_append_msg(ptm_hdl, out_ctxt, ZEBRA_PTM_BFD_CLIENT_FIELD,
 			   tmp_buf);
 
-	s = client->ibuf;
-
-	pid = stream_getl(s);
 	sprintf(tmp_buf, "%d", pid);
 	ptm_lib_append_msg(ptm_hdl, out_ctxt, ZEBRA_PTM_BFD_SEQID_FIELD,
 			   tmp_buf);
@@ -992,29 +994,41 @@ int zebra_ptm_bfd_client_register(struct zserv *client, int sock,
 
 	SET_FLAG(ptm_cb.client_flags[client->proto],
 		 ZEBRA_PTM_BFD_CLIENT_FLAG_REG);
-	return 0;
+
+	return;
+
+stream_failure:
+	/*
+	 * IF we ever add more STREAM_GETXXX functions after the out_ctxt
+	 * is allocated then we need to add this code back in
+	 *
+	 * if (out_ctxt)
+	 *	ptm_lib_cleanup_msg(ptm_hdl, out_ctxt);
+	 */
+	return;
 }
 
 /* BFD client deregister */
-void zebra_ptm_bfd_client_deregister(int proto)
+int zebra_ptm_bfd_client_deregister(struct zserv *client)
 {
+	uint8_t proto = client->proto;
 	void *out_ctxt;
 	char tmp_buf[64];
 	int data_len = ZEBRA_PTM_SEND_MAX_SOCKBUF;
 
 	if (proto != ZEBRA_ROUTE_OSPF && proto != ZEBRA_ROUTE_BGP
-	    && proto != ZEBRA_ROUTE_OSPF6)
-		return;
+	    && proto != ZEBRA_ROUTE_OSPF6 && proto != ZEBRA_ROUTE_PIM)
+		return 0;
 
 	if (IS_ZEBRA_DEBUG_EVENT)
 		zlog_err("bfd_client_deregister msg for client %s",
 			 zebra_route_string(proto));
 
 	if (ptm_cb.ptm_sock == -1) {
-		ptm_cb.t_timer =
-			thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
-					 ptm_cb.reconnect_time);
-		return;
+		ptm_cb.t_timer = NULL;
+		thread_add_timer(zebrad.master, zebra_ptm_connect, NULL,
+				 ptm_cb.reconnect_time, &ptm_cb.t_timer);
+		return 0;
 	}
 
 	ptm_lib_init_msg(ptm_hdl, 0, PTMLIB_MSG_TYPE_CMD, NULL, &out_ctxt);
@@ -1034,6 +1048,8 @@ void zebra_ptm_bfd_client_deregister(int proto)
 
 	zebra_ptm_send_message(ptm_cb.out_data, data_len);
 	UNSET_FLAG(ptm_cb.client_flags[proto], ZEBRA_PTM_BFD_CLIENT_FLAG_REG);
+
+	return 0;
 }
 
 int zebra_ptm_get_enable_state(void)
@@ -1061,10 +1077,9 @@ void zebra_ptm_show_status(struct vty *vty, struct interface *ifp)
 {
 	vty_out(vty, "  PTM status: ");
 	if (ifp->ptm_enable) {
-		vty_out(vty, "%s%s", zebra_ptm_get_status_str(ifp->ptm_status),
-			VTY_NEWLINE);
+		vty_out(vty, "%s\n", zebra_ptm_get_status_str(ifp->ptm_status));
 	} else {
-		vty_out(vty, "disabled%s", VTY_NEWLINE);
+		vty_out(vty, "disabled\n");
 	}
 }
 
@@ -1087,30 +1102,30 @@ void zebra_ptm_send_status_req(void)
 void zebra_ptm_reset_status(int ptm_disable)
 {
 	struct vrf *vrf;
-	struct listnode *i;
 	struct interface *ifp;
 	int send_linkup;
 
-	RB_FOREACH(vrf, vrf_id_head, &vrfs_by_id)
-	for (ALL_LIST_ELEMENTS_RO(vrf->iflist, i, ifp)) {
-		send_linkup = 0;
-		if (ifp->ptm_enable) {
-			if (!if_is_operative(ifp))
-				send_linkup = 1;
+	RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id)
+		FOR_ALL_INTERFACES (vrf, ifp) {
+			send_linkup = 0;
+			if (ifp->ptm_enable) {
+				if (!if_is_operative(ifp))
+					send_linkup = 1;
 
-			if (ptm_disable)
-				ifp->ptm_enable = ZEBRA_IF_PTM_ENABLE_OFF;
-			ifp->ptm_status = ZEBRA_PTM_STATUS_UNKNOWN;
+				if (ptm_disable)
+					ifp->ptm_enable =
+						ZEBRA_IF_PTM_ENABLE_OFF;
+				ifp->ptm_status = ZEBRA_PTM_STATUS_UNKNOWN;
 
-			if (if_is_operative(ifp) && send_linkup) {
-				if (IS_ZEBRA_DEBUG_EVENT)
-					zlog_debug(
-						"%s: Bringing up interface %s",
-						__func__, ifp->name);
-				if_up(ifp);
+				if (if_is_operative(ifp) && send_linkup) {
+					if (IS_ZEBRA_DEBUG_EVENT)
+						zlog_debug(
+							"%s: Bringing up interface %s",
+							__func__, ifp->name);
+					if_up(ifp);
+				}
 			}
 		}
-	}
 }
 
 void zebra_ptm_if_init(struct zebra_if *zebra_ifp)
@@ -1128,5 +1143,5 @@ void zebra_ptm_if_set_ptm_state(struct interface *ifp,
 void zebra_ptm_if_write(struct vty *vty, struct zebra_if *zebra_ifp)
 {
 	if (zebra_ifp->ptm_enable == ZEBRA_IF_PTM_ENABLE_OFF)
-		vty_out(vty, " no ptm-enable%s", VTY_NEWLINE);
+		vty_out(vty, " no ptm-enable\n");
 }
