@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -49,8 +48,10 @@
 #include "zebra/interface.h"
 #include "zebra/zebra_memory.h"
 
-static void free_state(vrf_id_t vrf_id, struct rib *rib, struct route_node *rn);
-static void copy_state(struct rnh *rnh, struct rib *rib, struct route_node *rn);
+static void free_state(vrf_id_t vrf_id, struct route_entry *re,
+		       struct route_node *rn);
+static void copy_state(struct rnh *rnh, struct route_entry *re,
+		       struct route_node *rn);
 #define lookup_rnh_table(v, f)                                                 \
 	({                                                                     \
 		struct zebra_vrf *zvrf;                                        \
@@ -61,14 +62,20 @@ static void copy_state(struct rnh *rnh, struct rib *rib, struct route_node *rn);
 		t;                                                             \
 	})
 
-static int compare_state(struct rib *r1, struct rib *r2);
+static int compare_state(struct route_entry *r1, struct route_entry *r2);
 static int send_client(struct rnh *rnh, struct zserv *client, rnh_type_t type,
 		       vrf_id_t vrf_id);
 static void print_rnh(struct route_node *rn, struct vty *vty);
+static int zebra_client_cleanup_rnh(struct zserv *client);
 
 int zebra_rnh_ip_default_route = 0;
 int zebra_rnh_ipv6_default_route = 0;
 
+void zebra_rnh_init(void)
+{
+	hook_register(zapi_client_close, zebra_client_cleanup_rnh);
+}
+
 static inline struct route_table *get_rnh_table(vrf_id_t vrfid, int family,
 						rnh_type_t type)
 {
@@ -159,9 +166,9 @@ struct rnh *zebra_lookup_rnh(struct prefix *p, vrf_id_t vrfid, rnh_type_t type)
 void zebra_free_rnh(struct rnh *rnh)
 {
 	rnh->flags |= ZEBRA_NHT_DELETED;
-	list_free(rnh->client_list);
-	list_free(rnh->zebra_static_route_list);
-	list_free(rnh->zebra_pseudowire_list);
+	list_delete_and_null(&rnh->client_list);
+	list_delete_and_null(&rnh->zebra_static_route_list);
+	list_delete_and_null(&rnh->zebra_pseudowire_list);
 	free_state(rnh->vrf_id, rnh->state, rnh->node);
 	XFREE(MTYPE_RNH, rnh);
 }
@@ -303,6 +310,7 @@ static void addr2hostprefix(int af, const union g_addr *addr,
 		prefix->u.prefix6 = addr->ipv6;
 		break;
 	default:
+		memset(prefix, 0, sizeof(*prefix));
 		zlog_warn("%s: unknown address family %d", __func__, af);
 		break;
 	}
@@ -343,7 +351,7 @@ void zebra_deregister_rnh_pseudowire(vrf_id_t vrf_id, struct zebra_pw *pw)
  * resolving a NH.
  */
 static int zebra_rnh_apply_nht_rmap(int family, struct route_node *prn,
-				    struct rib *rib, int proto)
+				    struct route_entry *re, int proto)
 {
 	int at_least_one = 0;
 	int rmap_family; /* Route map has diff AF family enum */
@@ -352,10 +360,11 @@ static int zebra_rnh_apply_nht_rmap(int family, struct route_node *prn,
 
 	rmap_family = (family == AF_INET) ? AFI_IP : AFI_IP6;
 
-	if (prn && rib) {
-		for (nexthop = rib->nexthop; nexthop; nexthop = nexthop->next) {
+	if (prn && re) {
+		for (nexthop = re->ng.nexthop; nexthop;
+		     nexthop = nexthop->next) {
 			ret = zebra_nht_route_map_check(rmap_family, proto,
-							&prn->p, rib, nexthop);
+							&prn->p, re, nexthop);
 			if (ret != RMAP_DENYMATCH) {
 				SET_FLAG(nexthop->flags, NEXTHOP_FLAG_ACTIVE);
 				at_least_one++; /* at least one valid NH */
@@ -368,18 +377,17 @@ static int zebra_rnh_apply_nht_rmap(int family, struct route_node *prn,
 }
 
 /*
- * Determine appropriate route (RIB entry) resolving a tracked entry
- * (nexthop or BGP route for import).
+ * Determine appropriate route (RE entry) resolving a tracked BGP route
+ * for BGP route for import.
  */
-static struct rib *zebra_rnh_resolve_entry(vrf_id_t vrfid, int family,
-					   rnh_type_t type,
-					   struct route_node *nrn,
-					   struct rnh *rnh,
-					   struct route_node **prn)
+static struct route_entry *
+zebra_rnh_resolve_import_entry(vrf_id_t vrfid, int family,
+			       struct route_node *nrn, struct rnh *rnh,
+			       struct route_node **prn)
 {
 	struct route_table *route_table;
 	struct route_node *rn;
-	struct rib *rib;
+	struct route_entry *re;
 
 	*prn = NULL;
 
@@ -391,52 +399,24 @@ static struct rib *zebra_rnh_resolve_entry(vrf_id_t vrfid, int family,
 	if (!rn)
 		return NULL;
 
-	/* When resolving nexthops, do not resolve via the default route unless
-	 * 'ip nht resolve-via-default' is configured.
-	 */
-	if ((type == RNH_NEXTHOP_TYPE)
-	    && (is_default_prefix(&rn->p)
-		&& !nh_resolve_via_default(rn->p.family)))
-		rib = NULL;
-	else if ((type == RNH_IMPORT_CHECK_TYPE)
-		 && CHECK_FLAG(rnh->flags, ZEBRA_NHT_EXACT_MATCH)
-		 && !prefix_same(&nrn->p, &rn->p))
-		rib = NULL;
-	else {
-		/* Identify appropriate route entry. */
-		RNODE_FOREACH_RIB(rn, rib)
-		{
-			if (CHECK_FLAG(rib->status, RIB_ENTRY_REMOVED))
-				continue;
-			if (!CHECK_FLAG(rib->status, RIB_ENTRY_SELECTED_FIB))
-				continue;
+	/* Unlock route node - we don't need to lock when walking the tree. */
+	route_unlock_node(rn);
 
-			if (CHECK_FLAG(rnh->flags, ZEBRA_NHT_CONNECTED)) {
-				if (rib->type == ZEBRA_ROUTE_CONNECT)
-					break;
-				if (rib->type == ZEBRA_ROUTE_NHRP) {
-					struct nexthop *nexthop;
-					for (nexthop = rib->nexthop; nexthop;
-					     nexthop = nexthop->next)
-						if (nexthop->type
-						    == NEXTHOP_TYPE_IFINDEX)
-							break;
-					if (nexthop)
-						break;
-				}
-			} else if ((type == RNH_IMPORT_CHECK_TYPE)
-				   && (rib->type == ZEBRA_ROUTE_BGP))
-				continue;
-			else
-				break;
-		}
+	if (CHECK_FLAG(rnh->flags, ZEBRA_NHT_EXACT_MATCH)
+	    && !prefix_same(&nrn->p, &rn->p))
+		return NULL;
+
+	/* Identify appropriate route entry. */
+	RNODE_FOREACH_RE (rn, re) {
+		if (!CHECK_FLAG(re->status, ROUTE_ENTRY_REMOVED)
+		    && CHECK_FLAG(re->flags, ZEBRA_FLAG_SELECTED)
+		    && (re->type != ZEBRA_ROUTE_BGP))
+			break;
 	}
 
-	/* Need to unlock route node */
-	route_unlock_node(rn);
-	if (rib)
+	if (re)
 		*prn = rn;
-	return rib;
+	return re;
 }
 
 /*
@@ -445,27 +425,26 @@ static struct rib *zebra_rnh_resolve_entry(vrf_id_t vrfid, int family,
  */
 static void zebra_rnh_eval_import_check_entry(vrf_id_t vrfid, int family,
 					      int force, struct route_node *nrn,
-					      struct rnh *rnh, struct rib *rib)
+					      struct rnh *rnh,
+					      struct route_entry *re)
 {
 	int state_changed = 0;
 	struct zserv *client;
 	char bufn[INET6_ADDRSTRLEN];
 	struct listnode *node;
-	struct nexthop *nexthop, *tnexthop;
-	int recursing;
+	struct nexthop *nexthop;
 
-	if (rib && (rnh->state == NULL)) {
-		for (ALL_NEXTHOPS_RO(rib->nexthop, nexthop, tnexthop,
-				     recursing))
+	if (re && (rnh->state == NULL)) {
+		for (ALL_NEXTHOPS(re->ng, nexthop))
 			if (CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_FIB)) {
 				state_changed = 1;
 				break;
 			}
-	} else if (!rib && (rnh->state != NULL))
+	} else if (!re && (rnh->state != NULL))
 		state_changed = 1;
 
-	if (compare_state(rib, rnh->state))
-		copy_state(rnh, rib, nrn);
+	if (compare_state(re, rnh->state))
+		copy_state(rnh, re, nrn);
 
 	if (state_changed || force) {
 		if (IS_ZEBRA_DEBUG_NHT) {
@@ -488,7 +467,7 @@ static void zebra_rnh_notify_protocol_clients(vrf_id_t vrfid, int family,
 					      struct route_node *nrn,
 					      struct rnh *rnh,
 					      struct route_node *prn,
-					      struct rib *rib)
+					      struct route_entry *re)
 {
 	struct listnode *node;
 	struct zserv *client;
@@ -498,7 +477,7 @@ static void zebra_rnh_notify_protocol_clients(vrf_id_t vrfid, int family,
 
 	if (IS_ZEBRA_DEBUG_NHT) {
 		prefix2str(&nrn->p, bufn, INET6_ADDRSTRLEN);
-		if (prn && rib) {
+		if (prn && re) {
 			prefix2str(&prn->p, bufp, INET6_ADDRSTRLEN);
 			zlog_debug("%u:%s: NH resolved over route %s", vrfid,
 				   bufn, bufp);
@@ -508,13 +487,13 @@ static void zebra_rnh_notify_protocol_clients(vrf_id_t vrfid, int family,
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(rnh->client_list, node, client)) {
-		if (prn && rib) {
+		if (prn && re) {
 			/* Apply route-map for this client to route resolving
 			 * this
 			 * nexthop to see if it is filtered or not.
 			 */
 			num_resolving_nh = zebra_rnh_apply_nht_rmap(
-				family, prn, rib, client->proto);
+				family, prn, re, client->proto);
 			if (num_resolving_nh)
 				rnh->filtered[client->proto] = 0;
 			else
@@ -541,16 +520,71 @@ static void zebra_rnh_notify_protocol_clients(vrf_id_t vrfid, int family,
 	}
 }
 
+static void zebra_rnh_process_pbr_tables(int family,
+					 struct route_node *nrn,
+					 struct rnh *rnh,
+					 struct route_node *prn,
+					 struct route_entry *re)
+{
+	struct zebra_ns_table *znst;
+	struct route_entry *o_re;
+	struct route_node *o_rn;
+	struct listnode *node;
+	struct zserv *client;
+	struct zebra_ns *zns;
+	afi_t afi = AFI_IP;
+
+	if (family == AF_INET6)
+		afi = AFI_IP6;
+
+	/*
+	 * We are only concerned about nexthops that change for
+	 * anyone using PBR
+	 */
+	for (ALL_LIST_ELEMENTS_RO(rnh->client_list, node, client)) {
+		if (client->proto == ZEBRA_ROUTE_PBR)
+			break;
+	}
+
+	if (!client)
+		return;
+
+	zns = zebra_ns_lookup(NS_DEFAULT);
+	RB_FOREACH (znst, zebra_ns_table_head, &zns->ns_tables) {
+		if (afi != znst->afi)
+			continue;
+
+		for (o_rn = route_top(znst->table);
+		     o_rn; o_rn = srcdest_route_next(o_rn)) {
+			RNODE_FOREACH_RE (o_rn, o_re) {
+				if (o_re->type == ZEBRA_ROUTE_PBR)
+					break;
+
+			}
+
+			/*
+			 * If we have a PBR route and a nexthop changes
+			 * just rethink it.  Yes this is a hammer, but
+			 * a small one
+			 */
+			if (o_re) {
+				SET_FLAG(o_re->status, ROUTE_ENTRY_CHANGED);
+				rib_queue_add(o_rn);
+			}
+		}
+	}
+}
+
 static void zebra_rnh_process_static_routes(vrf_id_t vrfid, int family,
 					    struct route_node *nrn,
 					    struct rnh *rnh,
 					    struct route_node *prn,
-					    struct rib *rib)
+					    struct route_entry *re)
 {
 	struct listnode *node;
 	int num_resolving_nh = 0;
 	struct route_node *static_rn;
-	struct rib *srib;
+	struct route_entry *sre;
 	struct nexthop *nexthop;
 	char bufn[INET6_ADDRSTRLEN];
 	char bufp[INET6_ADDRSTRLEN];
@@ -562,11 +596,11 @@ static void zebra_rnh_process_static_routes(vrf_id_t vrfid, int family,
 			prefix2str(&prn->p, bufp, INET6_ADDRSTRLEN);
 	}
 
-	if (prn && rib) {
+	if (prn && re) {
 		/* Apply route-map for "static" to route resolving this
 		 * nexthop to see if it is filtered or not.
 		 */
-		num_resolving_nh = zebra_rnh_apply_nht_rmap(family, prn, rib,
+		num_resolving_nh = zebra_rnh_apply_nht_rmap(family, prn, re,
 							    ZEBRA_ROUTE_STATIC);
 		if (num_resolving_nh)
 			rnh->filtered[ZEBRA_ROUTE_STATIC] = 0;
@@ -578,9 +612,8 @@ static void zebra_rnh_process_static_routes(vrf_id_t vrfid, int family,
 	/* Evaluate each static route associated with this nexthop. */
 	for (ALL_LIST_ELEMENTS_RO(rnh->zebra_static_route_list, node,
 				  static_rn)) {
-		RNODE_FOREACH_RIB(static_rn, srib)
-		{
-			if (srib->type != ZEBRA_ROUTE_STATIC)
+		RNODE_FOREACH_RE (static_rn, sre) {
+			if (sre->type != ZEBRA_ROUTE_STATIC)
 				continue;
 
 			/* Set the filter flag for the correct nexthop - static
@@ -588,7 +621,7 @@ static void zebra_rnh_process_static_routes(vrf_id_t vrfid, int family,
 			 * be having multiple. We care here only about
 			 * registered nexthops.
 			 */
-			for (nexthop = srib->nexthop; nexthop;
+			for (nexthop = sre->ng.nexthop; nexthop;
 			     nexthop = nexthop->next) {
 				switch (nexthop->type) {
 				case NEXTHOP_TYPE_IPV4:
@@ -629,7 +662,7 @@ static void zebra_rnh_process_static_routes(vrf_id_t vrfid, int family,
 			if (IS_ZEBRA_DEBUG_NHT) {
 				prefix2str(&static_rn->p, bufs,
 					   INET6_ADDRSTRLEN);
-				if (prn && rib)
+				if (prn && re)
 					zlog_debug(
 						"%u:%s: NH change %s, scheduling static route %s",
 						vrfid, bufn,
@@ -643,14 +676,92 @@ static void zebra_rnh_process_static_routes(vrf_id_t vrfid, int family,
 						vrfid, bufn, bufs);
 			}
 
-			SET_FLAG(srib->status, RIB_ENTRY_CHANGED);
-			SET_FLAG(srib->status, RIB_ENTRY_NEXTHOPS_CHANGED);
+			SET_FLAG(sre->status, ROUTE_ENTRY_CHANGED);
+			SET_FLAG(sre->status, ROUTE_ENTRY_NEXTHOPS_CHANGED);
 		}
 
 		rib_queue_add(static_rn);
 	}
 }
 
+/*
+ * Determine appropriate route (route entry) resolving a tracked
+ * nexthop.
+ */
+static struct route_entry *
+zebra_rnh_resolve_nexthop_entry(vrf_id_t vrfid, int family,
+				struct route_node *nrn, struct rnh *rnh,
+				struct route_node **prn)
+{
+	struct route_table *route_table;
+	struct route_node *rn;
+	struct route_entry *re;
+
+	*prn = NULL;
+
+	route_table = zebra_vrf_table(family2afi(family), SAFI_UNICAST, vrfid);
+	if (!route_table)
+		return NULL;
+
+	rn = route_node_match(route_table, &nrn->p);
+	if (!rn)
+		return NULL;
+
+	/* Unlock route node - we don't need to lock when walking the tree. */
+	route_unlock_node(rn);
+
+	/* While resolving nexthops, we may need to walk up the tree from the
+	 * most-specific match. Do similar logic as in zebra_rib.c
+	 */
+	while (rn) {
+		/* Do not resolve over default route unless allowed &&
+		 * match route to be exact if so specified
+		 */
+		if (is_default_prefix(&rn->p)
+		    && !rnh_resolve_via_default(rn->p.family))
+			return NULL;
+
+		/* Identify appropriate route entry. */
+		RNODE_FOREACH_RE (rn, re) {
+			if (CHECK_FLAG(re->status, ROUTE_ENTRY_REMOVED))
+				continue;
+			if (!CHECK_FLAG(re->flags, ZEBRA_FLAG_SELECTED))
+				continue;
+
+			if (CHECK_FLAG(rnh->flags, ZEBRA_NHT_CONNECTED)) {
+				if ((re->type == ZEBRA_ROUTE_CONNECT)
+				    || (re->type == ZEBRA_ROUTE_STATIC))
+					break;
+				if (re->type == ZEBRA_ROUTE_NHRP) {
+					struct nexthop *nexthop;
+
+					for (nexthop = re->ng.nexthop; nexthop;
+					     nexthop = nexthop->next)
+						if (nexthop->type
+						    == NEXTHOP_TYPE_IFINDEX)
+							break;
+					if (nexthop)
+						break;
+				}
+			} else
+				break;
+		}
+
+		/* Route entry found, we're done; else, walk up the tree. */
+		if (re) {
+			*prn = rn;
+			return re;
+		}
+
+		if (CHECK_FLAG(rnh->flags, ZEBRA_NHT_CONNECTED))
+			rn = rn->parent;
+		else
+			return NULL;
+	}
+
+	return NULL;
+}
+
 static void zebra_rnh_process_pseudowires(vrf_id_t vrfid, struct rnh *rnh)
 {
 	struct zebra_pw *pw;
@@ -669,7 +780,7 @@ static void zebra_rnh_eval_nexthop_entry(vrf_id_t vrfid, int family, int force,
 					 struct route_node *nrn,
 					 struct rnh *rnh,
 					 struct route_node *prn,
-					 struct rib *rib)
+					 struct route_entry *re)
 {
 	int state_changed = 0;
 
@@ -683,10 +794,10 @@ static void zebra_rnh_eval_nexthop_entry(vrf_id_t vrfid, int family, int force,
 		else
 			memset(&rnh->resolved_route, 0, sizeof(struct prefix));
 
-		copy_state(rnh, rib, nrn);
+		copy_state(rnh, re, nrn);
 		state_changed = 1;
-	} else if (compare_state(rib, rnh->state)) {
-		copy_state(rnh, rib, nrn);
+	} else if (compare_state(re, rnh->state)) {
+		copy_state(rnh, re, nrn);
 		state_changed = 1;
 	}
 
@@ -702,6 +813,9 @@ static void zebra_rnh_eval_nexthop_entry(vrf_id_t vrfid, int family, int force,
 		zebra_rnh_process_static_routes(vrfid, family, nrn, rnh, prn,
 						rnh->state);
 
+		zebra_rnh_process_pbr_tables(family, nrn, rnh, prn,
+					     rnh->state);
+
 		/* Process pseudowires attached to this nexthop */
 		zebra_rnh_process_pseudowires(vrfid, rnh);
 	}
@@ -712,7 +826,7 @@ static void zebra_rnh_evaluate_entry(vrf_id_t vrfid, int family, int force,
 				     rnh_type_t type, struct route_node *nrn)
 {
 	struct rnh *rnh;
-	struct rib *rib;
+	struct route_entry *re;
 	struct route_node *prn;
 	char bufn[INET6_ADDRSTRLEN];
 
@@ -724,47 +838,58 @@ static void zebra_rnh_evaluate_entry(vrf_id_t vrfid, int family, int force,
 
 	rnh = nrn->info;
 
-	/* Identify route entry (RIB) resolving this tracked entry. */
-	rib = zebra_rnh_resolve_entry(vrfid, family, type, nrn, rnh, &prn);
+	/* Identify route entry (RE) resolving this tracked entry. */
+	if (type == RNH_IMPORT_CHECK_TYPE)
+		re = zebra_rnh_resolve_import_entry(vrfid, family, nrn, rnh,
+						    &prn);
+	else
+		re = zebra_rnh_resolve_nexthop_entry(vrfid, family, nrn, rnh,
+						     &prn);
 
 	/* If the entry cannot be resolved and that is also the existing state,
 	 * there is nothing further to do.
 	 */
-	if (!rib && rnh->state == NULL && !force)
+	if (!re && rnh->state == NULL && !force)
 		return;
 
 	/* Process based on type of entry. */
 	if (type == RNH_IMPORT_CHECK_TYPE)
 		zebra_rnh_eval_import_check_entry(vrfid, family, force, nrn,
-						  rnh, rib);
+						  rnh, re);
 	else
 		zebra_rnh_eval_nexthop_entry(vrfid, family, force, nrn, rnh,
-					     prn, rib);
+					     prn, re);
 }
 
 /*
- * Clear the RIB_ENTRY_NEXTHOPS_CHANGED flag
- * from the rib entries.
+ * Clear the ROUTE_ENTRY_NEXTHOPS_CHANGED flag
+ * from the re entries.
  *
  * Please note we are doing this *after* we have
  * notified the world about each nexthop as that
- * we can have a situation where one rib entry
+ * we can have a situation where one re entry
  * covers multiple nexthops we are interested in.
  */
 static void zebra_rnh_clear_nhc_flag(vrf_id_t vrfid, int family,
 				     rnh_type_t type, struct route_node *nrn)
 {
 	struct rnh *rnh;
-	struct rib *rib;
+	struct route_entry *re;
 	struct route_node *prn;
 
 	rnh = nrn->info;
 
-	rib = zebra_rnh_resolve_entry(vrfid, family, type, nrn, rnh, &prn);
+	/* Identify route entry (RIB) resolving this tracked entry. */
+	if (type == RNH_IMPORT_CHECK_TYPE)
+		re = zebra_rnh_resolve_import_entry(vrfid, family, nrn, rnh,
+						    &prn);
+	else
+		re = zebra_rnh_resolve_nexthop_entry(vrfid, family, nrn, rnh,
+						     &prn);
 
-	if (rib) {
-		UNSET_FLAG(rib->status, RIB_ENTRY_NEXTHOPS_CHANGED);
-		UNSET_FLAG(rib->status, RIB_ENTRY_LABELS_CHANGED);
+	if (re) {
+		UNSET_FLAG(re->status, ROUTE_ENTRY_NEXTHOPS_CHANGED);
+		UNSET_FLAG(re->status, ROUTE_ENTRY_LABELS_CHANGED);
 	}
 }
 
@@ -826,72 +951,46 @@ void zebra_print_rnh_table(vrf_id_t vrfid, int af, struct vty *vty,
 			print_rnh(rn, vty);
 }
 
-int zebra_cleanup_rnh_client(vrf_id_t vrf_id, int family, struct zserv *client,
-			     rnh_type_t type)
-{
-	struct route_table *ntable;
-	struct route_node *nrn;
-	struct rnh *rnh;
-
-	if (IS_ZEBRA_DEBUG_NHT)
-		zlog_debug("%u: Client %s RNH cleanup for family %d type %d",
-			   vrf_id, zebra_route_string(client->proto), family,
-			   type);
-
-	ntable = get_rnh_table(vrf_id, family, type);
-	if (!ntable) {
-		zlog_debug("cleanup_rnh_client: rnh table not found\n");
-		return -1;
-	}
-
-	for (nrn = route_top(ntable); nrn; nrn = route_next(nrn)) {
-		if (!nrn->info)
-			continue;
-
-		rnh = nrn->info;
-		zebra_remove_rnh_client(rnh, client, type);
-	}
-	return 1;
-}
-
 /**
- * free_state - free up the rib structure associated with the rnh.
+ * free_state - free up the re structure associated with the rnh.
  */
-static void free_state(vrf_id_t vrf_id, struct rib *rib, struct route_node *rn)
+static void free_state(vrf_id_t vrf_id, struct route_entry *re,
+		       struct route_node *rn)
 {
 
-	if (!rib)
+	if (!re)
 		return;
 
-	/* free RIB and nexthops */
-	zebra_deregister_rnh_static_nexthops(vrf_id, rib->nexthop, rn);
-	nexthops_free(rib->nexthop);
-	XFREE(MTYPE_RIB, rib);
+	/* free RE and nexthops */
+	zebra_deregister_rnh_static_nexthops(vrf_id, re->ng.nexthop, rn);
+	nexthops_free(re->ng.nexthop);
+	XFREE(MTYPE_RE, re);
 }
 
-static void copy_state(struct rnh *rnh, struct rib *rib, struct route_node *rn)
+static void copy_state(struct rnh *rnh, struct route_entry *re,
+		       struct route_node *rn)
 {
-	struct rib *state;
-	struct nexthop *nh;
+	struct route_entry *state;
 
 	if (rnh->state) {
 		free_state(rnh->vrf_id, rnh->state, rn);
 		rnh->state = NULL;
 	}
 
-	if (!rib)
+	if (!re)
 		return;
 
-	state = XCALLOC(MTYPE_RIB, sizeof(struct rib));
-	state->type = rib->type;
-	state->metric = rib->metric;
+	state = XCALLOC(MTYPE_RE, sizeof(struct route_entry));
+	state->type = re->type;
+	state->distance = re->distance;
+	state->metric = re->metric;
+	state->vrf_id = re->vrf_id;
 
-	for (nh = rib->nexthop; nh; nh = nh->next)
-		rib_copy_nexthops(state, nh);
+	route_entry_copy_nexthops(state, re->ng.nexthop);
 	rnh->state = state;
 }
 
-static int compare_state(struct rib *r1, struct rib *r2)
+static int compare_state(struct route_entry *r1, struct route_entry *r2)
 {
 
 	if (!r1 && !r2)
@@ -900,14 +999,17 @@ static int compare_state(struct rib *r1, struct rib *r2)
 	if ((!r1 && r2) || (r1 && !r2))
 		return 1;
 
+	if (r1->distance != r2->distance)
+		return 1;
+
 	if (r1->metric != r2->metric)
 		return 1;
 
 	if (r1->nexthop_num != r2->nexthop_num)
 		return 1;
 
-	if (CHECK_FLAG(r1->status, RIB_ENTRY_NEXTHOPS_CHANGED)
-	    || CHECK_FLAG(r1->status, RIB_ENTRY_LABELS_CHANGED))
+	if (CHECK_FLAG(r1->status, ROUTE_ENTRY_NEXTHOPS_CHANGED)
+	    || CHECK_FLAG(r1->status, ROUTE_ENTRY_LABELS_CHANGED))
 		return 1;
 
 	return 0;
@@ -917,22 +1019,21 @@ static int send_client(struct rnh *rnh, struct zserv *client, rnh_type_t type,
 		       vrf_id_t vrf_id)
 {
 	struct stream *s;
-	struct rib *rib;
+	struct route_entry *re;
 	unsigned long nump;
-	u_char num;
-	struct nexthop *nexthop;
+	uint8_t num;
+	struct nexthop *nh;
 	struct route_node *rn;
 	int cmd = (type == RNH_IMPORT_CHECK_TYPE) ? ZEBRA_IMPORT_CHECK_UPDATE
 						  : ZEBRA_NEXTHOP_UPDATE;
 
 	rn = rnh->node;
-	rib = rnh->state;
+	re = rnh->state;
 
 	/* Get output stream. */
-	s = client->obuf;
-	stream_reset(s);
+	s = stream_new(ZEBRA_MAX_PACKET_SIZ);
 
-	zserv_create_header(s, cmd, vrf_id);
+	zclient_create_header(s, cmd, vrf_id);
 
 	stream_putw(s, rn->p.family);
 	switch (rn->p.family) {
@@ -949,49 +1050,54 @@ static int send_client(struct rnh *rnh, struct zserv *client, rnh_type_t type,
 			 __FUNCTION__, rn->p.family);
 		break;
 	}
-	if (rib) {
-		stream_putc(s, rib->distance);
-		stream_putl(s, rib->metric);
+	if (re) {
+		stream_putc(s, re->type);
+		stream_putw(s, re->instance);
+		stream_putc(s, re->distance);
+		stream_putl(s, re->metric);
 		num = 0;
 		nump = stream_get_endp(s);
 		stream_putc(s, 0);
-		for (nexthop = rib->nexthop; nexthop; nexthop = nexthop->next)
-			if ((CHECK_FLAG(nexthop->flags, NEXTHOP_FLAG_FIB)
-			     || CHECK_FLAG(nexthop->flags,
-					   NEXTHOP_FLAG_RECURSIVE))
-			    && CHECK_FLAG(nexthop->flags,
-					  NEXTHOP_FLAG_ACTIVE)) {
-				stream_putc(s, nexthop->type);
-				switch (nexthop->type) {
+		for (nh = re->ng.nexthop; nh; nh = nh->next)
+			if ((CHECK_FLAG(nh->flags, NEXTHOP_FLAG_FIB)
+			     || CHECK_FLAG(nh->flags, NEXTHOP_FLAG_RECURSIVE))
+			    && CHECK_FLAG(nh->flags, NEXTHOP_FLAG_ACTIVE)) {
+				stream_putc(s, nh->type);
+				switch (nh->type) {
 				case NEXTHOP_TYPE_IPV4:
-					stream_put_in_addr(s,
-							   &nexthop->gate.ipv4);
-					stream_putl(s, nexthop->ifindex);
+				case NEXTHOP_TYPE_IPV4_IFINDEX:
+					stream_put_in_addr(s, &nh->gate.ipv4);
+					stream_putl(s, nh->ifindex);
 					break;
 				case NEXTHOP_TYPE_IFINDEX:
-					stream_putl(s, nexthop->ifindex);
-					break;
-				case NEXTHOP_TYPE_IPV4_IFINDEX:
-					stream_put_in_addr(s,
-							   &nexthop->gate.ipv4);
-					stream_putl(s, nexthop->ifindex);
+					stream_putl(s, nh->ifindex);
 					break;
 				case NEXTHOP_TYPE_IPV6:
-					stream_put(s, &nexthop->gate.ipv6, 16);
-					stream_putl(s, nexthop->ifindex);
-					break;
 				case NEXTHOP_TYPE_IPV6_IFINDEX:
-					stream_put(s, &nexthop->gate.ipv6, 16);
-					stream_putl(s, nexthop->ifindex);
+					stream_put(s, &nh->gate.ipv6, 16);
+					stream_putl(s, nh->ifindex);
 					break;
 				default:
 					/* do nothing */
 					break;
 				}
+				if (nh->nh_label) {
+					stream_putc(s,
+						    nh->nh_label->num_labels);
+					if (nh->nh_label->num_labels)
+						stream_put(
+							s,
+							&nh->nh_label->label[0],
+							nh->nh_label->num_labels
+								* sizeof(mpls_label_t));
+				} else
+					stream_putc(s, 0);
 				num++;
 			}
 		stream_putc_at(s, nump, num);
 	} else {
+		stream_putc(s, 0); // type
+		stream_putw(s, 0); // instance
 		stream_putc(s, 0); // distance
 		stream_putl(s, 0); // metric
 		stream_putc(s, 0); // nexthops
@@ -1000,7 +1106,7 @@ static int send_client(struct rnh *rnh, struct zserv *client, rnh_type_t type,
 
 	client->nh_last_upd_time = monotime(NULL);
 	client->last_write_cmd = cmd;
-	return zebra_server_send_message(client);
+	return zebra_server_send_message(client, s);
 }
 
 static void print_nh(struct nexthop *nexthop, struct vty *vty)
@@ -1034,7 +1140,7 @@ static void print_nh(struct nexthop *nexthop, struct vty *vty)
 	default:
 		break;
 	}
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
 }
 
 static void print_rnh(struct route_node *rn, struct vty *vty)
@@ -1046,23 +1152,21 @@ static void print_rnh(struct route_node *rn, struct vty *vty)
 	char buf[BUFSIZ];
 
 	rnh = rn->info;
-	vty_out(vty, "%s%s%s",
+	vty_out(vty, "%s%s\n",
 		inet_ntop(rn->p.family, &rn->p.u.prefix, buf, BUFSIZ),
 		CHECK_FLAG(rnh->flags, ZEBRA_NHT_CONNECTED) ? "(Connected)"
-							    : "",
-		VTY_NEWLINE);
+							    : "");
 	if (rnh->state) {
-		vty_out(vty, " resolved via %s%s",
-			zebra_route_string(rnh->state->type), VTY_NEWLINE);
-		for (nexthop = rnh->state->nexthop; nexthop;
+		vty_out(vty, " resolved via %s\n",
+			zebra_route_string(rnh->state->type));
+		for (nexthop = rnh->state->ng.nexthop; nexthop;
 		     nexthop = nexthop->next)
 			print_nh(nexthop, vty);
 	} else
-		vty_out(vty, " unresolved%s%s",
+		vty_out(vty, " unresolved%s\n",
 			CHECK_FLAG(rnh->flags, ZEBRA_NHT_CONNECTED)
 				? "(Connected)"
-				: "",
-			VTY_NEWLINE);
+				: "");
 
 	vty_out(vty, " Client list:");
 	for (ALL_LIST_ELEMENTS_RO(rnh->client_list, node, client))
@@ -1070,9 +1174,67 @@ static void print_rnh(struct route_node *rn, struct vty *vty)
 			client->sock,
 			rnh->filtered[client->proto] ? "(filtered)" : "");
 	if (!list_isempty(rnh->zebra_static_route_list))
-		vty_out(vty, " zebra[static routes]%s",
+		vty_out(vty, " zebra%s",
 			rnh->filtered[ZEBRA_ROUTE_STATIC] ? "(filtered)" : "");
 	if (!list_isempty(rnh->zebra_pseudowire_list))
 		vty_out(vty, " zebra[pseudowires]");
-	vty_out(vty, "%s", VTY_NEWLINE);
+	vty_out(vty, "\n");
+}
+
+static int zebra_cleanup_rnh_client(vrf_id_t vrf_id, int family,
+				    struct zserv *client, rnh_type_t type)
+{
+	struct route_table *ntable;
+	struct route_node *nrn;
+	struct rnh *rnh;
+
+	if (IS_ZEBRA_DEBUG_NHT)
+		zlog_debug("%u: Client %s RNH cleanup for family %d type %d",
+			   vrf_id, zebra_route_string(client->proto), family,
+			   type);
+
+	ntable = get_rnh_table(vrf_id, family, type);
+	if (!ntable) {
+		zlog_debug("cleanup_rnh_client: rnh table not found\n");
+		return -1;
+	}
+
+	for (nrn = route_top(ntable); nrn; nrn = route_next(nrn)) {
+		if (!nrn->info)
+			continue;
+
+		rnh = nrn->info;
+		zebra_remove_rnh_client(rnh, client, type);
+	}
+	return 1;
+}
+
+/* Cleanup registered nexthops (across VRFs) upon client disconnect. */
+static int zebra_client_cleanup_rnh(struct zserv *client)
+{
+	struct vrf *vrf;
+	struct zebra_vrf *zvrf;
+
+	RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id) {
+		zvrf = vrf->info;
+		if (zvrf) {
+			zebra_cleanup_rnh_client(zvrf_id(zvrf), AF_INET, client,
+						 RNH_NEXTHOP_TYPE);
+			zebra_cleanup_rnh_client(zvrf_id(zvrf), AF_INET6,
+						 client, RNH_NEXTHOP_TYPE);
+			zebra_cleanup_rnh_client(zvrf_id(zvrf), AF_INET, client,
+						 RNH_IMPORT_CHECK_TYPE);
+			zebra_cleanup_rnh_client(zvrf_id(zvrf), AF_INET6,
+						 client, RNH_IMPORT_CHECK_TYPE);
+			if (client->proto == ZEBRA_ROUTE_LDP) {
+				hash_iterate(zvrf->lsp_table,
+					     mpls_ldp_lsp_uninstall_all,
+					     zvrf->lsp_table);
+				mpls_ldp_ftn_uninstall_all(zvrf, AFI_IP);
+				mpls_ldp_ftn_uninstall_all(zvrf, AFI_IP6);
+			}
+		}
+	}
+
+	return 0;
 }
