@@ -13,10 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GNU Zebra; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License along
+ * with this program; see the file COPYING; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -39,7 +38,7 @@
 #include "zebra/zebra_rnh.h"
 #include "zebra/zebra_routemap.h"
 
-static u_int32_t zebra_rmap_update_timer = ZEBRA_RMAP_DEFAULT_UPDATE_TIMER;
+static uint32_t zebra_rmap_update_timer = ZEBRA_RMAP_DEFAULT_UPDATE_TIMER;
 static struct thread *zebra_t_rmap_update = NULL;
 char *proto_rm[AFI_MAX][ZEBRA_ROUTE_MAX + 1]; /* "any" == ZEBRA_ROUTE_MAX */
 /* NH Tracking route map */
@@ -49,12 +48,13 @@ char *zebra_import_table_routemap[AFI_MAX][ZEBRA_KERNEL_TABLE_MAX];
 struct nh_rmap_obj {
 	struct nexthop *nexthop;
 	vrf_id_t vrf_id;
-	u_int32_t source_protocol;
+	uint32_t source_protocol;
+	uint8_t instance;
 	int metric;
 	route_tag_t tag;
 };
 
-static void zebra_route_map_set_delay_timer(u_int32_t value);
+static void zebra_route_map_set_delay_timer(uint32_t value);
 
 
 /* Add zebra route map rule */
@@ -63,25 +63,26 @@ static int zebra_route_match_add(struct vty *vty, const char *command,
 {
 	VTY_DECLVAR_CONTEXT(route_map_index, index);
 	int ret;
+	int retval = CMD_SUCCESS;
 
 	ret = route_map_add_match(index, command, arg);
-	if (ret) {
-		switch (ret) {
-		case RMAP_RULE_MISSING:
-			vty_out(vty, "%% Zebra Can't find rule.%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
-		case RMAP_COMPILE_ERROR:
-			vty_out(vty, "%% Zebra Argument is malformed.%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
+	switch (ret) {
+	case RMAP_RULE_MISSING:
+		vty_out(vty, "%% Zebra Can't find rule.\n");
+		retval = CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_ERROR:
+		vty_out(vty, "%% Zebra Argument is malformed.\n");
+		retval = CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_SUCCESS:
+		if (type != RMAP_EVENT_MATCH_ADDED) {
+			route_map_upd8_dependency(type, arg, index->map->name);
 		}
+		break;
 	}
 
-	if (type != RMAP_EVENT_MATCH_ADDED) {
-		route_map_upd8_dependency(type, arg, index->map->name);
-	}
-	return CMD_SUCCESS;
+	return retval;
 }
 
 /* Delete zebra route map rule. */
@@ -90,6 +91,7 @@ static int zebra_route_match_delete(struct vty *vty, const char *command,
 {
 	VTY_DECLVAR_CONTEXT(route_map_index, index);
 	int ret;
+	int retval = CMD_SUCCESS;
 	char *dep_name = NULL;
 	const char *tmpstr;
 	char *rmap_name = NULL;
@@ -108,28 +110,27 @@ static int zebra_route_match_delete(struct vty *vty, const char *command,
 	}
 
 	ret = route_map_delete_match(index, command, arg);
-	if (ret) {
-		switch (ret) {
-		case RMAP_RULE_MISSING:
-			vty_out(vty, "%% Zebra Can't find rule.%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
-		case RMAP_COMPILE_ERROR:
-			vty_out(vty, "%% Zebra Argument is malformed.%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
-		}
+	switch (ret) {
+	case RMAP_RULE_MISSING:
+		vty_out(vty, "%% Zebra Can't find rule.\n");
+		retval = CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_ERROR:
+		vty_out(vty, "%% Zebra Argument is malformed.\n");
+		retval = CMD_WARNING_CONFIG_FAILED;
+		break;
+	case RMAP_COMPILE_SUCCESS:
+		if (type != RMAP_EVENT_MATCH_DELETED && dep_name)
+			route_map_upd8_dependency(type, dep_name, rmap_name);
+		break;
 	}
 
-	if (type != RMAP_EVENT_MATCH_DELETED && dep_name)
-		route_map_upd8_dependency(type, dep_name, rmap_name);
-
 	if (dep_name)
 		XFREE(MTYPE_ROUTE_MAP_RULE, dep_name);
 	if (rmap_name)
 		XFREE(MTYPE_ROUTE_MAP_NAME, rmap_name);
 
-	return CMD_SUCCESS;
+	return retval;
 }
 
 /* 'match tag TAG'
@@ -229,6 +230,33 @@ DEFUN (no_match_ip_address_prefix_len,
 					RMAP_EVENT_MATCH_DELETED);
 }
 
+DEFUN (match_ipv6_address_prefix_len,
+       match_ipv6_address_prefix_len_cmd,
+       "match ipv6 address prefix-len (0-128)",
+       MATCH_STR
+       IPV6_STR
+       "Match prefix length of ipv6 address\n"
+       "Match prefix length of ipv6 address\n"
+       "Prefix length\n")
+{
+	return zebra_route_match_add(vty, "ipv6 address prefix-len",
+				     argv[4]->arg, RMAP_EVENT_MATCH_ADDED);
+}
+
+DEFUN (no_match_ipv6_address_prefix_len,
+       no_match_ipv6_address_prefix_len_cmd,
+       "no match ipv6 address prefix-len [(0-128)]",
+       NO_STR
+       MATCH_STR
+       IPV6_STR
+       "Match prefix length of ip address\n"
+       "Match prefix length of ip address\n"
+       "Prefix length\n")
+{
+	char *plen = (argc == 6) ? argv[5]->arg : NULL;
+	return zebra_route_match_delete(vty, "ipv6 address prefix-len", plen,
+					RMAP_EVENT_MATCH_DELETED);
+}
 
 DEFUN (match_ip_nexthop_prefix_len,
        match_ip_nexthop_prefix_len_cmd,
@@ -258,10 +286,9 @@ DEFUN (no_match_ip_nexthop_prefix_len,
 					RMAP_EVENT_MATCH_DELETED);
 }
 
-
 DEFUN (match_source_protocol,
        match_source_protocol_cmd,
-       "match source-protocol <bgp|ospf|rip|ripng|isis|ospf6|connected|system|kernel|static>",
+       "match source-protocol <bgp|ospf|rip|ripng|isis|ospf6|pim|nhrp|eigrp|babel|connected|system|kernel|static|sharp>",
        MATCH_STR
        "Match protocol via which the route was learnt\n"
        "BGP protocol\n"
@@ -270,19 +297,23 @@ DEFUN (match_source_protocol,
        "RIPNG protocol\n"
        "ISIS protocol\n"
        "OSPF6 protocol\n"
+       "PIM protocol\n"
+       "NHRP protocol\n"
+       "EIGRP protocol\n"
+       "BABEL protocol\n"
        "Routes from directly connected peer\n"
        "Routes from system configuration\n"
        "Routes from kernel\n"
-       "Statically configured routes\n")
+       "Statically configured routes\n"
+       "SHARP process\n")
 {
 	char *proto = argv[2]->text;
 	int i;
 
 	i = proto_name2num(proto);
 	if (i < 0) {
-		vty_out(vty, "invalid protocol name \"%s\"%s", proto,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "invalid protocol name \"%s\"\n", proto);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	return zebra_route_match_add(vty, "source-protocol", proto,
 				     RMAP_EVENT_MATCH_ADDED);
@@ -290,7 +321,7 @@ DEFUN (match_source_protocol,
 
 DEFUN (no_match_source_protocol,
        no_match_source_protocol_cmd,
-       "no match source-protocol [<bgp|ospf|rip|ripng|isis|ospf6|connected|system|kernel|static>]",
+       "no match source-protocol [<bgp|ospf|rip|ripng|isis|ospf6|pim|nhrp|eigrp|babel|connected|system|kernel|static|sharp>]",
        NO_STR
        MATCH_STR
        "No match protocol via which the route was learnt\n"
@@ -300,16 +331,47 @@ DEFUN (no_match_source_protocol,
        "RIPNG protocol\n"
        "ISIS protocol\n"
        "OSPF6 protocol\n"
+       "PIM protocol\n"
+       "NHRP protocol\n"
+       "EIGRP protocol\n"
+       "BABEL protocol\n"
        "Routes from directly connected peer\n"
        "Routes from system configuration\n"
        "Routes from kernel\n"
-       "Statically configured routes\n")
+       "Statically configured routes\n"
+       "SHARP process\n")
 {
 	char *proto = (argc == 4) ? argv[3]->text : NULL;
 	return zebra_route_match_delete(vty, "source-protocol", proto,
 					RMAP_EVENT_MATCH_DELETED);
 }
 
+DEFUN (match_source_instance,
+       match_source_instance_cmd,
+       "match source-instance (0-255)",
+       MATCH_STR
+       "Match the protocol's instance number\n"
+       "The instance number\n")
+{
+	char *instance = argv[2]->arg;
+
+	return zebra_route_match_add(vty, "source-instance", instance,
+				     RMAP_EVENT_MATCH_ADDED);
+}
+
+DEFUN (no_match_source_instance,
+       no_match_source_instance_cmd,
+       "no match source-instance [(0-255)]",
+       NO_STR MATCH_STR
+       "Match the protocol's instance number\n"
+       "The instance number\n")
+{
+	char *instance = (argc == 4) ? argv[3]->arg : NULL;
+
+	return zebra_route_match_delete(vty, "source-instance", instance,
+					RMAP_EVENT_MATCH_ADDED);
+}
+
 /* set functions */
 
 DEFUN (set_src,
@@ -329,9 +391,8 @@ DEFUN (set_src,
 
 	if (inet_pton(AF_INET, argv[idx_ip]->arg, &src.ipv4) != 1) {
 		if (inet_pton(AF_INET6, argv[idx_ip]->arg, &src.ipv6) != 1) {
-			vty_out(vty, "%% not a valid IPv4/v6 address%s",
-				VTY_NEWLINE);
-			return CMD_WARNING;
+			vty_out(vty, "%% not a valid IPv4/v6 address\n");
+			return CMD_WARNING_CONFIG_FAILED;
 		}
 
 		p.family = family = AF_INET6;
@@ -344,13 +405,11 @@ DEFUN (set_src,
 	}
 
 	if (!zebra_check_addr(&p)) {
-		vty_out(vty, "%% not a valid source IPv4/v6 address%s",
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% not a valid source IPv4/v6 address\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
-	RB_FOREACH(vrf, vrf_id_head, &vrfs_by_id)
-	{
+	RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id) {
 		if (family == AF_INET)
 			pif = if_lookup_exact_address((void *)&src.ipv4,
 						      AF_INET, vrf->vrf_id);
@@ -363,8 +422,8 @@ DEFUN (set_src,
 	}
 
 	if (!pif) {
-		vty_out(vty, "%% not a local address%s", VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "%% not a local address\n");
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	VTY_DECLVAR_CONTEXT(route_map_index, index);
@@ -388,16 +447,15 @@ DEFUN (no_set_src,
 DEFUN (zebra_route_map_timer,
        zebra_route_map_timer_cmd,
        "zebra route-map delay-timer (0-600)",
-       "Zebra information\n"
+       ZEBRA_STR
        "Set route-map parameters\n"
        "Time to wait before route-map updates are processed\n"
        "0 means event-driven updates are disabled\n")
 {
 	int idx_number = 3;
-	u_int32_t rmap_delay_timer;
+	uint32_t rmap_delay_timer;
 
-	VTY_GET_INTEGER_RANGE("delay-timer", rmap_delay_timer,
-			      argv[idx_number]->arg, 0, 600);
+	rmap_delay_timer = strtoul(argv[idx_number]->arg, NULL, 10);
 	zebra_route_map_set_delay_timer(rmap_delay_timer);
 
 	return (CMD_SUCCESS);
@@ -407,7 +465,7 @@ DEFUN (no_zebra_route_map_timer,
        no_zebra_route_map_timer_cmd,
        "no zebra route-map delay-timer [(0-600)]",
        NO_STR
-       "Zebra information\n"
+       ZEBRA_STR
        "Set route-map parameters\n"
        "Reset delay-timer to default value, 30 secs\n"
        "0 means event-driven updates are disabled\n")
@@ -436,9 +494,8 @@ DEFUN (ip_protocol,
 	else
 		i = proto_name2num(proto);
 	if (i < 0) {
-		vty_out(vty, "invalid protocol name \"%s\"%s", proto,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "invalid protocol name \"%s\"\n", proto);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	if (proto_rm[AFI_IP][i]) {
 		if (strcmp(proto_rm[AFI_IP][i], rmap) == 0)
@@ -477,9 +534,8 @@ DEFUN (no_ip_protocol,
 		i = proto_name2num(proto);
 
 	if (i < 0) {
-		vty_out(vty, "invalid protocol name \"%s\"%s", proto,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "invalid protocol name \"%s\"\n", proto);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (!proto_rm[AFI_IP][i])
@@ -507,21 +563,19 @@ DEFUN (show_ip_protocol,
 {
 	int i;
 
-	vty_out(vty, "Protocol    : route-map %s", VTY_NEWLINE);
-	vty_out(vty, "------------------------%s", VTY_NEWLINE);
+	vty_out(vty, "Protocol    : route-map \n");
+	vty_out(vty, "------------------------\n");
 	for (i = 0; i < ZEBRA_ROUTE_MAX; i++) {
 		if (proto_rm[AFI_IP][i])
-			vty_out(vty, "%-10s  : %-10s%s", zebra_route_string(i),
-				proto_rm[AFI_IP][i], VTY_NEWLINE);
+			vty_out(vty, "%-10s  : %-10s\n", zebra_route_string(i),
+				proto_rm[AFI_IP][i]);
 		else
-			vty_out(vty, "%-10s  : none%s", zebra_route_string(i),
-				VTY_NEWLINE);
+			vty_out(vty, "%-10s  : none\n", zebra_route_string(i));
 	}
 	if (proto_rm[AFI_IP][i])
-		vty_out(vty, "%-10s  : %-10s%s", "any", proto_rm[AFI_IP][i],
-			VTY_NEWLINE);
+		vty_out(vty, "%-10s  : %-10s\n", "any", proto_rm[AFI_IP][i]);
 	else
-		vty_out(vty, "%-10s  : none%s", "any", VTY_NEWLINE);
+		vty_out(vty, "%-10s  : none\n", "any");
 
 	return CMD_SUCCESS;
 }
@@ -544,9 +598,8 @@ DEFUN (ipv6_protocol,
 	else
 		i = proto_name2num(proto);
 	if (i < 0) {
-		vty_out(vty, "invalid protocol name \"%s\"%s", proto,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "invalid protocol name \"%s\"\n", proto);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	if (proto_rm[AFI_IP6][i]) {
 		if (strcmp(proto_rm[AFI_IP6][i], rmap) == 0)
@@ -584,9 +637,8 @@ DEFUN (no_ipv6_protocol,
 	else
 		i = proto_name2num(proto);
 	if (i < 0) {
-		vty_out(vty, "invalid protocol name \"%s\"%s", proto,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "invalid protocol name \"%s\"\n", proto);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	if (!proto_rm[AFI_IP6][i])
 		return CMD_SUCCESS;
@@ -614,21 +666,19 @@ DEFUN (show_ipv6_protocol,
 {
 	int i;
 
-	vty_out(vty, "Protocol    : route-map %s", VTY_NEWLINE);
-	vty_out(vty, "------------------------%s", VTY_NEWLINE);
+	vty_out(vty, "Protocol    : route-map \n");
+	vty_out(vty, "------------------------\n");
 	for (i = 0; i < ZEBRA_ROUTE_MAX; i++) {
 		if (proto_rm[AFI_IP6][i])
-			vty_out(vty, "%-10s  : %-10s%s", zebra_route_string(i),
-				proto_rm[AFI_IP6][i], VTY_NEWLINE);
+			vty_out(vty, "%-10s  : %-10s\n", zebra_route_string(i),
+				proto_rm[AFI_IP6][i]);
 		else
-			vty_out(vty, "%-10s  : none%s", zebra_route_string(i),
-				VTY_NEWLINE);
+			vty_out(vty, "%-10s  : none\n", zebra_route_string(i));
 	}
 	if (proto_rm[AFI_IP6][i])
-		vty_out(vty, "%-10s  : %-10s%s", "any", proto_rm[AFI_IP6][i],
-			VTY_NEWLINE);
+		vty_out(vty, "%-10s  : %-10s\n", "any", proto_rm[AFI_IP6][i]);
 	else
-		vty_out(vty, "%-10s  : none%s", "any", VTY_NEWLINE);
+		vty_out(vty, "%-10s  : none\n", "any");
 
 	return CMD_SUCCESS;
 }
@@ -651,9 +701,8 @@ DEFUN (ip_protocol_nht_rmap,
 	else
 		i = proto_name2num(proto);
 	if (i < 0) {
-		vty_out(vty, "invalid protocol name \"%s\"%s", proto,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "invalid protocol name \"%s\"\n", proto);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	if (nht_rm[AFI_IP][i]) {
 		if (strcmp(nht_rm[AFI_IP][i], rmap) == 0)
@@ -687,9 +736,8 @@ DEFUN (no_ip_protocol_nht_rmap,
 				       : proto_name2num(proto);
 
 	if (i < 0) {
-		vty_out(vty, "invalid protocol name \"%s\"%s", proto,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "invalid protocol name \"%s\"\n", proto);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (!nht_rm[AFI_IP][i])
@@ -713,21 +761,19 @@ DEFUN (show_ip_protocol_nht,
 {
 	int i;
 
-	vty_out(vty, "Protocol    : route-map %s", VTY_NEWLINE);
-	vty_out(vty, "------------------------%s", VTY_NEWLINE);
+	vty_out(vty, "Protocol    : route-map \n");
+	vty_out(vty, "------------------------\n");
 	for (i = 0; i < ZEBRA_ROUTE_MAX; i++) {
 		if (nht_rm[AFI_IP][i])
-			vty_out(vty, "%-10s  : %-10s%s", zebra_route_string(i),
-				nht_rm[AFI_IP][i], VTY_NEWLINE);
+			vty_out(vty, "%-10s  : %-10s\n", zebra_route_string(i),
+				nht_rm[AFI_IP][i]);
 		else
-			vty_out(vty, "%-10s  : none%s", zebra_route_string(i),
-				VTY_NEWLINE);
+			vty_out(vty, "%-10s  : none\n", zebra_route_string(i));
 	}
 	if (nht_rm[AFI_IP][i])
-		vty_out(vty, "%-10s  : %-10s%s", "any", nht_rm[AFI_IP][i],
-			VTY_NEWLINE);
+		vty_out(vty, "%-10s  : %-10s\n", "any", nht_rm[AFI_IP][i]);
 	else
-		vty_out(vty, "%-10s  : none%s", "any", VTY_NEWLINE);
+		vty_out(vty, "%-10s  : none\n", "any");
 
 	return CMD_SUCCESS;
 }
@@ -750,9 +796,8 @@ DEFUN (ipv6_protocol_nht_rmap,
 	else
 		i = proto_name2num(proto);
 	if (i < 0) {
-		vty_out(vty, "invalid protocol name \"%s\"%s", proto,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "invalid protocol name \"%s\"\n", proto);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 	if (nht_rm[AFI_IP6][i])
 		XFREE(MTYPE_ROUTE_MAP_NAME, nht_rm[AFI_IP6][i]);
@@ -781,14 +826,13 @@ DEFUN (no_ipv6_protocol_nht_rmap,
 	else
 		i = proto_name2num(proto);
 	if (i < 0) {
-		vty_out(vty, "invalid protocol name \"%s\"%s", proto,
-			VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "invalid protocol name \"%s\"\n", proto);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (nht_rm[AFI_IP6][i] && rmap && strcmp(rmap, nht_rm[AFI_IP6][i])) {
-		vty_out(vty, "invalid route-map \"%s\"%s", rmap, VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "invalid route-map \"%s\"\n", rmap);
+		return CMD_WARNING_CONFIG_FAILED;
 	}
 
 	if (nht_rm[AFI_IP6][i]) {
@@ -811,21 +855,19 @@ DEFUN (show_ipv6_protocol_nht,
 {
 	int i;
 
-	vty_out(vty, "Protocol    : route-map %s", VTY_NEWLINE);
-	vty_out(vty, "------------------------%s", VTY_NEWLINE);
+	vty_out(vty, "Protocol    : route-map \n");
+	vty_out(vty, "------------------------\n");
 	for (i = 0; i < ZEBRA_ROUTE_MAX; i++) {
 		if (nht_rm[AFI_IP6][i])
-			vty_out(vty, "%-10s  : %-10s%s", zebra_route_string(i),
-				nht_rm[AFI_IP6][i], VTY_NEWLINE);
+			vty_out(vty, "%-10s  : %-10s\n", zebra_route_string(i),
+				nht_rm[AFI_IP6][i]);
 		else
-			vty_out(vty, "%-10s  : none%s", zebra_route_string(i),
-				VTY_NEWLINE);
+			vty_out(vty, "%-10s  : none\n", zebra_route_string(i));
 	}
 	if (nht_rm[AFI_IP][i])
-		vty_out(vty, "%-10s  : %-10s%s", "any", nht_rm[AFI_IP6][i],
-			VTY_NEWLINE);
+		vty_out(vty, "%-10s  : %-10s\n", "any", nht_rm[AFI_IP6][i]);
 	else
-		vty_out(vty, "%-10s  : none%s", "any", VTY_NEWLINE);
+		vty_out(vty, "%-10s  : none\n", "any");
 
 	return CMD_SUCCESS;
 }
@@ -1025,10 +1067,10 @@ static struct route_map_rule_cmd route_match_ip_address_prefix_list_cmd = {
 /* `match ip address prefix-len PREFIXLEN' */
 
 static route_map_result_t
-route_match_ip_address_prefix_len(void *rule, struct prefix *prefix,
-				  route_map_object_t type, void *object)
+route_match_address_prefix_len(void *rule, struct prefix *prefix,
+			       route_map_object_t type, void *object)
 {
-	u_int32_t *prefixlen = (u_int32_t *)rule;
+	uint32_t *prefixlen = (uint32_t *)rule;
 
 	if (type == RMAP_ZEBRA) {
 		return ((prefix->prefixlen == *prefixlen) ? RMAP_MATCH
@@ -1037,9 +1079,9 @@ route_match_ip_address_prefix_len(void *rule, struct prefix *prefix,
 	return RMAP_NOMATCH;
 }
 
-static void *route_match_ip_address_prefix_len_compile(const char *arg)
+static void *route_match_address_prefix_len_compile(const char *arg)
 {
-	u_int32_t *prefix_len;
+	uint32_t *prefix_len;
 	char *endptr = NULL;
 	unsigned long tmpval;
 
@@ -1052,7 +1094,7 @@ static void *route_match_ip_address_prefix_len_compile(const char *arg)
 	if (*endptr != '\0' || errno || tmpval > UINT32_MAX)
 		return NULL;
 
-	prefix_len = XMALLOC(MTYPE_ROUTE_MAP_COMPILED, sizeof(u_int32_t));
+	prefix_len = XMALLOC(MTYPE_ROUTE_MAP_COMPILED, sizeof(uint32_t));
 
 	if (!prefix_len)
 		return prefix_len;
@@ -1061,16 +1103,20 @@ static void *route_match_ip_address_prefix_len_compile(const char *arg)
 	return prefix_len;
 }
 
-static void route_match_ip_address_prefix_len_free(void *rule)
+static void route_match_address_prefix_len_free(void *rule)
 {
 	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
 }
 
 static struct route_map_rule_cmd route_match_ip_address_prefix_len_cmd = {
-	"ip address prefix-len", route_match_ip_address_prefix_len,
-	route_match_ip_address_prefix_len_compile,
-	route_match_ip_address_prefix_len_free};
+	"ip address prefix-len", route_match_address_prefix_len,
+	route_match_address_prefix_len_compile,
+	route_match_address_prefix_len_free};
 
+static struct route_map_rule_cmd route_match_ipv6_address_prefix_len_cmd = {
+	"ipv6 address prefix-len", route_match_address_prefix_len,
+	route_match_address_prefix_len_compile,
+	route_match_address_prefix_len_free};
 
 /* `match ip nexthop prefix-len PREFIXLEN' */
 
@@ -1078,7 +1124,7 @@ static route_map_result_t
 route_match_ip_nexthop_prefix_len(void *rule, struct prefix *prefix,
 				  route_map_object_t type, void *object)
 {
-	u_int32_t *prefixlen = (u_int32_t *)rule;
+	uint32_t *prefixlen = (uint32_t *)rule;
 	struct nh_rmap_obj *nh_data;
 	struct prefix_ipv4 p;
 
@@ -1108,8 +1154,8 @@ route_match_ip_nexthop_prefix_len(void *rule, struct prefix *prefix,
 
 static struct route_map_rule_cmd route_match_ip_nexthop_prefix_len_cmd = {
 	"ip next-hop prefix-len", route_match_ip_nexthop_prefix_len,
-	route_match_ip_address_prefix_len_compile, /* reuse */
-	route_match_ip_address_prefix_len_free     /* reuse */
+	route_match_address_prefix_len_compile, /* reuse */
+	route_match_address_prefix_len_free     /* reuse */
 };
 
 /* `match source-protocol PROTOCOL' */
@@ -1119,7 +1165,7 @@ static route_map_result_t route_match_source_protocol(void *rule,
 						      route_map_object_t type,
 						      void *object)
 {
-	u_int32_t *rib_type = (u_int32_t *)rule;
+	uint32_t *rib_type = (uint32_t *)rule;
 	struct nh_rmap_obj *nh_data;
 
 	if (type == RMAP_ZEBRA) {
@@ -1135,11 +1181,11 @@ static route_map_result_t route_match_source_protocol(void *rule,
 
 static void *route_match_source_protocol_compile(const char *arg)
 {
-	u_int32_t *rib_type;
+	uint32_t *rib_type;
 	int i;
 
 	i = proto_name2num(arg);
-	rib_type = XMALLOC(MTYPE_ROUTE_MAP_COMPILED, sizeof(u_int32_t));
+	rib_type = XMALLOC(MTYPE_ROUTE_MAP_COMPILED, sizeof(uint32_t));
 
 	*rib_type = i;
 
@@ -1155,6 +1201,47 @@ static struct route_map_rule_cmd route_match_source_protocol_cmd = {
 	"source-protocol", route_match_source_protocol,
 	route_match_source_protocol_compile, route_match_source_protocol_free};
 
+/* `source-instance` */
+static route_map_result_t route_match_source_instance(void *rule,
+						      struct prefix *prefix,
+						      route_map_object_t type,
+						      void *object)
+{
+	uint8_t *instance = (uint8_t *)rule;
+	struct nh_rmap_obj *nh_data;
+
+	if (type != RMAP_ZEBRA)
+		return RMAP_NOMATCH;
+
+	nh_data = (struct nh_rmap_obj *)object;
+	if (!nh_data)
+		return RMAP_DENYMATCH;
+
+	return (nh_data->instance == *instance) ? RMAP_MATCH : RMAP_NOMATCH;
+}
+
+static void *route_match_source_instance_compile(const char *arg)
+{
+	uint8_t *instance;
+	int i;
+
+	i = atoi(arg);
+	instance = XMALLOC(MTYPE_ROUTE_MAP_COMPILED, sizeof(uint8_t));
+
+	*instance = i;
+
+	return instance;
+}
+
+static void route_match_source_instance_free(void *rule)
+{
+	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
+}
+
+static struct route_map_rule_cmd route_match_source_instance_cmd = {
+	"source-instance", route_match_source_instance,
+	route_match_source_instance_compile, route_match_source_instance_free};
+
 /* `set src A.B.C.D' */
 
 /* Set src. */
@@ -1176,7 +1263,7 @@ static void *route_set_src_compile(const char *arg)
 	union g_addr src, *psrc;
 
 	if ((inet_pton(AF_INET6, arg, &src.ipv6) == 1)
-	    || (src.ipv4.s_addr && (inet_pton(AF_INET, arg, &src.ipv4) == 1))) {
+	    || (inet_pton(AF_INET, arg, &src.ipv4) == 1)) {
 		psrc = XMALLOC(MTYPE_ROUTE_MAP_COMPILED, sizeof(union g_addr));
 		*psrc = src;
 		return psrc;
@@ -1215,7 +1302,7 @@ static int zebra_route_map_update_timer(struct thread *thread)
 	return (0);
 }
 
-static void zebra_route_map_set_delay_timer(u_int32_t value)
+static void zebra_route_map_set_delay_timer(uint32_t value)
 {
 	zebra_rmap_update_timer = value;
 	if (!value && zebra_t_rmap_update) {
@@ -1229,13 +1316,13 @@ static void zebra_route_map_set_delay_timer(u_int32_t value)
 void zebra_route_map_write_delay_timer(struct vty *vty)
 {
 	if (vty && (zebra_rmap_update_timer != ZEBRA_RMAP_DEFAULT_UPDATE_TIMER))
-		vty_out(vty, "zebra route-map delay-timer %d%s",
-			zebra_rmap_update_timer, VTY_NEWLINE);
+		vty_out(vty, "zebra route-map delay-timer %d\n",
+			zebra_rmap_update_timer);
 	return;
 }
 
 route_map_result_t zebra_route_map_check(int family, int rib_type,
-					 struct prefix *p,
+					 uint8_t instance, struct prefix *p,
 					 struct nexthop *nexthop,
 					 vrf_id_t vrf_id, route_tag_t tag)
 {
@@ -1246,6 +1333,7 @@ route_map_result_t zebra_route_map_check(int family, int rib_type,
 	nh_obj.nexthop = nexthop;
 	nh_obj.vrf_id = vrf_id;
 	nh_obj.source_protocol = rib_type;
+	nh_obj.instance = instance;
 	nh_obj.metric = 0;
 	nh_obj.tag = tag;
 
@@ -1279,9 +1367,10 @@ void zebra_del_import_table_route_map(afi_t afi, uint32_t table)
 }
 
 route_map_result_t
-zebra_import_table_route_map_check(int family, int rib_type, struct prefix *p,
-				   struct nexthop *nexthop, vrf_id_t vrf_id,
-				   route_tag_t tag, const char *rmap_name)
+zebra_import_table_route_map_check(int family, int re_type, uint8_t instance,
+				   struct prefix *p, struct nexthop *nexthop,
+				   vrf_id_t vrf_id, route_tag_t tag,
+				   const char *rmap_name)
 {
 	struct route_map *rmap = NULL;
 	route_map_result_t ret = RMAP_DENYMATCH;
@@ -1289,11 +1378,12 @@ zebra_import_table_route_map_check(int family, int rib_type, struct prefix *p,
 
 	nh_obj.nexthop = nexthop;
 	nh_obj.vrf_id = vrf_id;
-	nh_obj.source_protocol = rib_type;
+	nh_obj.source_protocol = re_type;
+	nh_obj.instance = instance;
 	nh_obj.metric = 0;
 	nh_obj.tag = tag;
 
-	if (rib_type >= 0 && rib_type < ZEBRA_ROUTE_MAX)
+	if (re_type >= 0 && re_type < ZEBRA_ROUTE_MAX)
 		rmap = route_map_lookup_by_name(rmap_name);
 	if (rmap) {
 		ret = route_map_apply(rmap, p, RMAP_ZEBRA, &nh_obj);
@@ -1303,7 +1393,8 @@ zebra_import_table_route_map_check(int family, int rib_type, struct prefix *p,
 }
 
 route_map_result_t zebra_nht_route_map_check(int family, int client_proto,
-					     struct prefix *p, struct rib *rib,
+					     struct prefix *p,
+					     struct route_entry *re,
 					     struct nexthop *nexthop)
 {
 	struct route_map *rmap = NULL;
@@ -1311,10 +1402,11 @@ route_map_result_t zebra_nht_route_map_check(int family, int client_proto,
 	struct nh_rmap_obj nh_obj;
 
 	nh_obj.nexthop = nexthop;
-	nh_obj.vrf_id = rib->vrf_id;
-	nh_obj.source_protocol = rib->type;
-	nh_obj.metric = rib->metric;
-	nh_obj.tag = rib->tag;
+	nh_obj.vrf_id = nexthop->vrf_id;
+	nh_obj.source_protocol = re->type;
+	nh_obj.instance = re->instance;
+	nh_obj.metric = re->metric;
+	nh_obj.tag = re->tag;
 
 	if (client_proto >= 0 && client_proto < ZEBRA_ROUTE_MAX)
 		rmap = route_map_lookup_by_name(nht_rm[family][client_proto]);
@@ -1331,10 +1423,12 @@ route_map_result_t zebra_nht_route_map_check(int family, int client_proto,
 static void zebra_route_map_mark_update(const char *rmap_name)
 {
 	/* rmap_update_timer of 0 means don't do route updates */
-	if (zebra_rmap_update_timer && !zebra_t_rmap_update)
-		zebra_t_rmap_update = thread_add_timer(
-			zebrad.master, zebra_route_map_update_timer, NULL,
-			zebra_rmap_update_timer);
+	if (zebra_rmap_update_timer && !zebra_t_rmap_update) {
+		zebra_t_rmap_update = NULL;
+		thread_add_timer(zebrad.master, zebra_route_map_update_timer,
+				 NULL, zebra_rmap_update_timer,
+				 &zebra_t_rmap_update);
+	}
 }
 
 static void zebra_route_map_add(const char *rmap_name)
@@ -1363,45 +1457,41 @@ void zebra_routemap_config_write_protocol(struct vty *vty)
 
 	for (i = 0; i < ZEBRA_ROUTE_MAX; i++) {
 		if (proto_rm[AFI_IP][i])
-			vty_out(vty, "ip protocol %s route-map %s%s",
-				zebra_route_string(i), proto_rm[AFI_IP][i],
-				VTY_NEWLINE);
+			vty_out(vty, "ip protocol %s route-map %s\n",
+				zebra_route_string(i), proto_rm[AFI_IP][i]);
 
 		if (proto_rm[AFI_IP6][i])
-			vty_out(vty, "ipv6 protocol %s route-map %s%s",
-				zebra_route_string(i), proto_rm[AFI_IP6][i],
-				VTY_NEWLINE);
+			vty_out(vty, "ipv6 protocol %s route-map %s\n",
+				zebra_route_string(i), proto_rm[AFI_IP6][i]);
 
 		if (nht_rm[AFI_IP][i])
-			vty_out(vty, "ip nht %s route-map %s%s",
-				zebra_route_string(i), nht_rm[AFI_IP][i],
-				VTY_NEWLINE);
+			vty_out(vty, "ip nht %s route-map %s\n",
+				zebra_route_string(i), nht_rm[AFI_IP][i]);
 
 		if (nht_rm[AFI_IP6][i])
-			vty_out(vty, "ipv6 nht %s route-map %s%s",
-				zebra_route_string(i), nht_rm[AFI_IP6][i],
-				VTY_NEWLINE);
+			vty_out(vty, "ipv6 nht %s route-map %s\n",
+				zebra_route_string(i), nht_rm[AFI_IP6][i]);
 	}
 
 	if (proto_rm[AFI_IP][ZEBRA_ROUTE_MAX])
-		vty_out(vty, "ip protocol %s route-map %s%s", "any",
-			proto_rm[AFI_IP][ZEBRA_ROUTE_MAX], VTY_NEWLINE);
+		vty_out(vty, "ip protocol %s route-map %s\n", "any",
+			proto_rm[AFI_IP][ZEBRA_ROUTE_MAX]);
 
 	if (proto_rm[AFI_IP6][ZEBRA_ROUTE_MAX])
-		vty_out(vty, "ipv6 protocol %s route-map %s%s", "any",
-			proto_rm[AFI_IP6][ZEBRA_ROUTE_MAX], VTY_NEWLINE);
+		vty_out(vty, "ipv6 protocol %s route-map %s\n", "any",
+			proto_rm[AFI_IP6][ZEBRA_ROUTE_MAX]);
 
 	if (nht_rm[AFI_IP][ZEBRA_ROUTE_MAX])
-		vty_out(vty, "ip nht %s route-map %s%s", "any",
-			nht_rm[AFI_IP][ZEBRA_ROUTE_MAX], VTY_NEWLINE);
+		vty_out(vty, "ip nht %s route-map %s\n", "any",
+			nht_rm[AFI_IP][ZEBRA_ROUTE_MAX]);
 
 	if (nht_rm[AFI_IP6][ZEBRA_ROUTE_MAX])
-		vty_out(vty, "ipv6 nht %s route-map %s%s", "any",
-			nht_rm[AFI_IP6][ZEBRA_ROUTE_MAX], VTY_NEWLINE);
+		vty_out(vty, "ipv6 nht %s route-map %s\n", "any",
+			nht_rm[AFI_IP6][ZEBRA_ROUTE_MAX]);
 
 	if (zebra_rmap_update_timer != ZEBRA_RMAP_DEFAULT_UPDATE_TIMER)
-		vty_out(vty, "zebra route-map delay-timer %d%s",
-			zebra_rmap_update_timer, VTY_NEWLINE);
+		vty_out(vty, "zebra route-map delay-timer %d\n",
+			zebra_rmap_update_timer);
 }
 
 void zebra_route_map_init()
@@ -1452,17 +1542,25 @@ void zebra_route_map_init()
 	route_map_install_match(&route_match_ip_address_cmd);
 	route_map_install_match(&route_match_ip_address_prefix_list_cmd);
 	route_map_install_match(&route_match_ip_address_prefix_len_cmd);
+	route_map_install_match(&route_match_ipv6_address_prefix_len_cmd);
 	route_map_install_match(&route_match_ip_nexthop_prefix_len_cmd);
 	route_map_install_match(&route_match_source_protocol_cmd);
+	route_map_install_match(&route_match_source_instance_cmd);
+
 	/* */
 	route_map_install_set(&route_set_src_cmd);
 	/* */
 	install_element(RMAP_NODE, &match_ip_nexthop_prefix_len_cmd);
 	install_element(RMAP_NODE, &no_match_ip_nexthop_prefix_len_cmd);
 	install_element(RMAP_NODE, &match_ip_address_prefix_len_cmd);
+	install_element(RMAP_NODE, &match_ipv6_address_prefix_len_cmd);
+	install_element(RMAP_NODE, &no_match_ipv6_address_prefix_len_cmd);
 	install_element(RMAP_NODE, &no_match_ip_address_prefix_len_cmd);
 	install_element(RMAP_NODE, &match_source_protocol_cmd);
 	install_element(RMAP_NODE, &no_match_source_protocol_cmd);
+	install_element(RMAP_NODE, &match_source_instance_cmd);
+	install_element(RMAP_NODE, &no_match_source_instance_cmd);
+
 	/* */
 	install_element(RMAP_NODE, &set_src_cmd);
 	install_element(RMAP_NODE, &no_set_src_cmd);
